﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * DDLBasic
 *
 * Description:
 *
 *  DuplicatedObject (以下 DO) による P2P 通信を行うサンプルです。
 *  マッチメイクは AutoMmDirectStream サンプルと同等の処理を行っています。
 *
 *  DO の詳細な定義は AutoMmDupObj.ddl に、それらを使用するクラスは
 *  Avatar.cpp 及び World.cpp を主とするサブクラスで記述されています。
 *  合わせてご参照ください。
 *
 *  本サンプルはステーション毎に作成される DO として Avatar を、
 *  全ステーションで唯一存在する既知オブジェクトとして World を定義します。
 *  World は 2次元で構成されており、Avatar がその中をランダムに移動する、という
 *  イメージです。
 *
 *  セッションを作成するステーションが World (既知オブジェクト)を作成し、
 *  パブリッシュします。これにより、各ステーションに既知オブジェクトが確実に認識されます。
 *  各ステーション(セッションマスタを含む)はセッション参加後に Avatar オブジェクトを
 *  作成し、同じくパブリッシュします。これにより、自分で作成された Avatar
 *  の複製(デュプリカ)が他の各ステーションに作成されます。
 *
 *  マスタオブジェクトの変更をデュプリカに伝える場合、Update 関数を呼びます。
 *  Avatar::DoPhysic 関数内にてこの関数を呼び出しています。
 *
 *  ただし、ddl で推定フィルタによる更新 (extrapolation_filter) による型定義を
 *  行っている場合は、反映させるために Refresh を呼び出す必要があります。
 *  こちらも本サンプルでは Avatar::DoPhysic 内で行っています。
 *  他にループバックと推定フィルタの組み合わせでの更新も行っています。
 *
 *  型定義により、更新の方法が異なる場合がありますので、詳細はドキュメントを
 *  参照してください。
 *
 * アプリケーションへの組み込み方の例:
 * 
 *  AutoMatchmakeGameLoop() は実際のゲームにおけるゲームループを模しています。この処理をゲームループに実装してください。
 *  nn::nex::Scheduler::DispatchAll() や各種計算処理はここに記載されている順に呼び出すことを推奨します。
 *  ゲームループから AutoMatchmakeSample クラスの関数を呼び出し、シーケンス処理を進めます。
 *  シーケンス処理はP2P送受信処理とは分離されています。Communicate() を編集することにより任意のP2P送受信処理を行えます。
 *  
 *  P2P通信の初期化と終了処理の詳細については P2PApiChain.h を参照してください。
 *  AutoMatchmakeSample::AutoMatchmake() を修正してゲームにおいて必要なマッチメイクパラメータを MatchmakeExtension::AutoMatchmake() に渡すようにしてください。
 *  
 *  シーケンス処理やP2P通信処理中に発生するエラーは、本サンプルで使用しているAPIや実装しているシーケンスの範囲において全て適切に補足されるように実装されています。
 *  ただし、シーケンスを増やしたり使用するAPIを変えた場合はそれによりNEX内部で発生するエラーが変わったり追加のユーザー定義結果が必要になる場合があります。
 *  アプリケーションでは AutoMatchmakeSample::GetLastResult() で取得した最終的なエラーを画面に表示するなどしてプレイヤーに通知するか、
 *  アプリケーションで適切に処理してください。
 */

#include <nex.h>

#include <NexDemoUtil/NexUtil.h>
#include <NexDemoUtil/NexUtilNetZ.h>
#include <NexDemoUtil/P2PApiChain.h>

#include "Avatar.h"
#include "World.h"

namespace
{
// マッチメイクで使用するゲームモード
const nn::nex::qUnsignedInt32 MATCHMAKE_GAME_MODE = 1002;

// 参加人数
const nn::nex::qUnsignedInt16 MATCHMAKE_MAX_PARTICIPANTS = 3;

// P2P通信で送受信を行う時間(フレーム)
const nn::nex::qUnsignedInt32 COMMUNICATION_DURATION = 60 * 60;  // 60秒

// ユーザー定義の結果コード
enum UserDefinedResultCode
{
    USER_DEFINED_RESULT_CODE_SUCCESS                         =  0,  // 処理に成功し、エラーがない状態
    USER_DEFINED_RESULT_CODE_GAME_OVER                       = -1,  // ゲームオーバーが通知された (このケースでは正常終了します)
    USER_DEFINED_RESULT_CODE_NETWORK_DISCONNECTED            = -2,  // ACによるネットワーク切断を検出
    USER_DEFINED_RESULT_CODE_P2P_SESSION_ORPHAN              = -3,  // P2Pセッションにおいて孤立状態になった
    USER_DEFINED_RESULT_CODE_MATCHMAKESESSION_UNREGISTERED   = -4,  // 意図せずマッチメイクセッションが削除された
    USER_DEFINED_RESULT_CODE_UNEXPECTED_HOST_CHANGE          = -5,  // 意図せずホストの変更が通知された
    USER_DEFINED_RESULT_CODE_UNEXPECTED_RMC_FAILURE          = -6,  // 意図せずRMCが失敗した
    USER_DEFINED_RESULT_CODE_DISCONNECTED_FROM_P2P           = -7   // 意図せずP2Pセッションから切断された
};

/*!
    @brief セッション参加要求受付時のコールバック。

    @param[in] pOperation 参加受付に対する操作。
 */
void CheckJoiningProcesses(nn::nex::JoinSessionOperation* pOperation)
{
    // ここでP2Pセッションへの参加の拒否ができますが、この段階での拒否はマッチメイク成功率が下がることに繋がります。
    // アプリケーション都合による参加拒否は極力控えてください。
    // このサンプルでは最大人数とマッチメイクセッションの一致を確認します。
    // 基本的にはマッチメイクセッションに設定した最大人数より多くP2Pセッションに参加されることはありませんが、
    // 処理タイミングのずれなどにより多く参加される場合があります。

    // 既にセッションへの参加が承認されているステーションの数を取得します。(数には、参加要求をしてきたステーションは含まれない)
    qUnsignedInt32 uiApprovedPlayerCount = nn::nex::Station::GetApprovedStationNum();

    // マッチメイクセッションの一致を確認するため、事前に Station::SetLocalIdentificationToken() で設定された内容を取得し、
    // 自分と参加者で一致しているか比較します。
    // Station::SetLocalIdentificationToken() は P2PStartApiChain::Call() で行っています。
    nn::nex::String password = NEX_T("");
    if(nn::nex::Station::GetLocalInstance() != NULL)
    {
        password = nn::nex::Station::GetLocalInstance()->GetIdentificationToken();
    }

    NEX_DEMO_LOG("CheckJoiningProcesses Master-Password: " STRING_FORMAT " JoiningStation-Password: " STRING_FORMAT "\n",
        password.CStr(), pOperation->GetJoiningStation()->GetIdentificationToken().CStr());

    if (uiApprovedPlayerCount < MATCHMAKE_MAX_PARTICIPANTS && pOperation->GetJoiningStation()->GetIdentificationToken() == password)
    {
        pOperation->Approve();
    }
    else
    {
        // Joinしようとしているセッションは Session::JoinSession() の非同期処理が QERROR(DOCore, JoinDenied) で失敗する
        pOperation->Deny();
    }
}

/*!
    @brief P2P通信で使用する、オペレーションコールバック (ステーション参加時、退出時などのコールバックに利用されます)。

    @param[in] pOperation ステーションに対する操作。
 */
void StationOperationBeginCallback(nn::nex::StationOperation* pOperation)
{
    (void)pOperation;
    NEX_DEMO_LOG("Begin operation: " STRING_FORMAT ", StationID: %d\n", pOperation->GetClassNameString(), pOperation->GetTargetStation()->GetStationID());
}

/*!
    @brief P2P通信で使用する、オペレーションコールバック (ステーション参加時、退出時などのコールバックに利用されます)。

    @param[in] pOperation ステーションに対する操作。
 */
void StationOperationEndCallback(nn::nex::StationOperation* pOperation)
{
    (void)pOperation;
    NEX_DEMO_LOG("End operation: " STRING_FORMAT ", sid: %d, State: %d\n",
        pOperation->GetClassNameString(), pOperation->GetTargetStation()->GetStationID(), pOperation->GetTargetStation()->GetState());
}

/*!
    @brief オートマッチメイクやP2P通信のセットアップなどを行うステートマシン。
 */
class AutoMatchmakeSample
{
public:
    AutoMatchmakeSample(nn::nex::NgsFacade* pNgsFacade)
        : m_state(STATE_AUTOMATCHMAKE)
        , m_finished(false)
        , m_errorOccurred(false)
        , m_unexpectedHostChangeOccurred(false)
        , m_unexpectedRmcFailureOccurred(false)
        , m_lastResult()
        , m_myPid(INVALID_PRINCIPALID)
        , m_ownerPid(INVALID_PRINCIPALID)
        , m_gid(INVALID_GATHERINGID)
        , m_pNgsFacade(pNgsFacade)
        , m_pNetZ(NULL)
        , m_pHostMigrationCallback(NULL)
        , m_matchMakingClient()
        , m_natTraversalClient()
        , m_notificationHandler(this)
        , m_joinedMatchmakeSession()
        , m_pP2pInitializeApiChain(NULL)
        , m_pP2pStartApiChain(NULL)
        , m_pP2pShutdownApiChain(NULL)
    {
        nn::nex::Credentials* pCred = m_pNgsFacade->GetCredentials();
        NEX_DEMO_ASSERT(pCred != NULL);
        m_myPid = pCred->GetPrincipalID();

        // 各Bind()処理はpCredがNULLでない場合は必ず成功します
        NEX_DEMO_VERIFY(m_matchMakingClient.Bind(pCred));

        // NATトラバーサルを行う場合に事前にNATTraversalClient::Bind()の呼び出しが必要になります。
        NEX_DEMO_VERIFY(m_natTraversalClient.Bind(pCred));

        // NGSからの通知をハンドリングするイベントハンドラを設定します。
        m_pNgsFacade->RegisterNotificationEventHandler(&m_notificationHandler);
    };
    virtual ~AutoMatchmakeSample()
    {
        // 処理が完了していない非同期処理を全てキャンセルする
        m_callContextStore.CancelAll();

        m_pNgsFacade->UnregisterNotificationEventHandler(&m_notificationHandler);
    };
private:
    enum State
    {
        STATE_AUTOMATCHMAKE,
        STATE_P2PINITIALIZE,
        STATE_P2PINITIALIZE_WAITING,
        STATE_P2PSTART,
        STATE_P2PSTART_WAITING,
        STATE_COMMUNICATE,
        STATE_P2PSHUTDOWN,
        STATE_P2PSHUTDOWN_WAITING,
        STATE_ENDPARTICIPATION,
        STATE_RMC_RESULT_WAITING,
        STATE_FINISH
    };

    /*!
        @brief NGSから通知を処理するイベントハンドラ。
     */
    class MyNotificationEventHandler : public nn::nex::NotificationEventHandler
    {
    public:
        MyNotificationEventHandler(AutoMatchmakeSample* sample):m_pSample(sample){};
        virtual ~MyNotificationEventHandler(){};

        void ProcessNotificationEvent(const nn::nex::NotificationEvent &oEvent)
        {
            nn::nex::StringStream ss;
            ss << oEvent;
            NEX_DEMO_LOG("[Notified] " STRING_FORMAT "\n", ss.CStr());

            // 通知毎の処理
            // 必要があれば通知をいったんキューに入れ、都合の良いタイミングで取り出して処理する等行ってください
            switch (oEvent.GetType())
            {
            case nn::nex::NotificationEvents::ParticipationEvent:
                switch (oEvent.GetSubType())
                {
                case nn::nex::ParticipationEvents::Participate:
                    NEX_DEMO_LOG("  ParticipationEvents::Participate: [PID " NEX_PID_FORMAT "] join [gathering ID %u]\n",
                        static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                    break;
                case nn::nex::ParticipationEvents::EndParticipation:
                    NEX_DEMO_LOG("  ParticipationEvents::EndParticipation: [PID " NEX_PID_FORMAT "] end [gathering ID %u]\n",
                        static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                    break;
                case nn::nex::ParticipationEvents::Disconnect:
                    NEX_DEMO_LOG("  ParticipationEvents::Disconnect: [PID " NEX_PID_FORMAT "] disconnect [gathering ID %u]\n",
                        static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                    break;
                default:
                    NEX_DEMO_LOG("  ParticipationEvents: unsupported participation event\n");
                    break;
                }
                break;
            case nn::nex::NotificationEvents::OwnershipChangeEvent:
                NEX_DEMO_LOG("  NotificationEvents::OwnershipChangeEvent: [gathering ID %u] changed owner to [PID " NEX_PID_FORMAT "]\n",
                    static_cast<nn::nex::GatheringID>(oEvent.GetParam1()), static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()));
                m_pSample->SetOwnerPid(static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()));
                break;
            case nn::nex::NotificationEvents::GatheringUnregistered:
                NEX_DEMO_LOG("  NotificationEvents::GatheringUnregistered: [gathering ID %u] unregistered\n", static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                // 本サンプルでは自分が参加している限りマッチメイクセッションは削除されない。
                break;
            case nn::nex::NotificationEvents::HostChangeEvent:
                NEX_DEMO_LOG("  NotificationEvents::HostChangeEvent: [gathering ID %u] host information has been changed. New host is " NEX_PID_FORMAT "\n",
                    static_cast<nn::nex::GatheringID>(oEvent.GetParam1()), static_cast<nn::nex::PrincipalID>(oEvent.GetSource()));
                // NotificationEvents::HostChangeEvent は以前のホストに対して通知される
                // 本サンプルではホスト情報の変更はホストマイグレーション拡張機能でのみ行っている。
                // ホストが通信可能な状態でこの通知がなされた場合は自身の通信環境が悪くてP2Pセッションが分離したと判断しエラーとする
                if (oEvent.GetParam1() == m_pSample->GetGatheringID())
                {
                    m_pSample->UnexpectedHostChangeOccurred();
                }
                break;
            default:
                NEX_DEMO_LOG("  NotificationEvents: unsupported notification event\n");
                break;
            }
        }
    private:
        AutoMatchmakeSample* m_pSample;
    };

public:
    qBool ProcessState();
    qBool CanCommunicate() const { return m_state == STATE_COMMUNICATE; }
    qBool InitializeCommunication();
    qBool StartCommunication();
    void FinishCommunication();
    nex_demo_util::ResultContainer GetLastResult() const { return m_lastResult; }
    void CloseParticipation();
private:
    void NotifyGameOverToAllParticipants();
    void UnexpectedHostChangeOccurred() { m_unexpectedHostChangeOccurred = true; }
    void UnexpectedRmcFailureOccurred() { m_unexpectedRmcFailureOccurred = true; }
    nex_demo_util::ResultContainer CheckCondition();
    void ErrorOccurred(const nex_demo_util::ResultContainer& result);
    void SetState(State state){m_state = state;};
    State GetState() const {return m_state;};
    nn::nex::PrincipalID GetMyPid() const {return m_myPid;};
    void SetOwnerPid(nn::nex::PrincipalID pid){m_ownerPid = pid;};
    nn::nex::PrincipalID GetOwnerPid() const {return m_ownerPid;};
    qBool IsOwner() const {return GetOwnerPid() == GetMyPid();};
    void SetGatheringID(nn::nex::GatheringID gid){m_gid = gid;};
    nn::nex::GatheringID GetGatheringID() const {return m_gid;};
    void AutoMatchmake();
    void OnCompletionAutoMatchmake(nn::nex::ProtocolCallContext* pCallContext);
    void EndParticipation();
    void OnCompletionEndParticipation(nn::nex::ProtocolCallContext* pCallContext);
    void OnCompletionCloseParticipation(nn::nex::ProtocolCallContext* pCallContext);

    State m_state;                                      // シーケンス処理の状態
    qBool m_finished;                                   // シーケンス処理が完了したか
    qBool m_errorOccurred;                              // エラーが発生した状態か
    qBool m_unexpectedHostChangeOccurred;               // 予期しないホスト情報の変更があったか
    qBool m_unexpectedRmcFailureOccurred;               // 予期しないRMC呼び出しの失敗があったか
    nex_demo_util::ResultContainer m_lastResult;        // シーケンス処理の最終結果
    nex_demo_util::CallContextStore m_callContextStore; // 非同期処理実行中のCallContextが保存されるコンテナです。キャンセル処理が必要なもののみ格納されます。
    nn::nex::PrincipalID m_myPid;                       // 自分のプリンシパルID
    nn::nex::PrincipalID m_ownerPid;                    // マッチメイクセッションのオーナーのプリンシパルID
    nn::nex::GatheringID m_gid;                         // 参加したギャザリングID
    nn::nex::NgsFacade* m_pNgsFacade;                   // ログイン済みのNgsFacadeオブジェクトへのポインタ
    nn::nex::NetZ* m_pNetZ;                             // NetZオブジェクトが格納されるポインタ
    nn::nex::HostMigrationCallback* m_pHostMigrationCallback;   // ホストマイグレーション拡張機能で使用されるオブジェクトへのポインタ
    nn::nex::MatchmakeExtensionClient m_matchMakingClient;      // マッチメイク関連のAPI呼び出しに使用するオブジェクト
    nn::nex::NATTraversalClient m_natTraversalClient;           // NATトラバーサル機能を使用するために必要なオブジェクト
    MyNotificationEventHandler m_notificationHandler;           // NGSからの通知をハンドリングするオブジェクト
    nn::nex::MatchmakeSession m_joinedMatchmakeSession;         // 参加したマッチメイクセッション

    nex_demo_util::P2PInitializeApiChain* m_pP2pInitializeApiChain;    // P2P通信の初期化処理を行うオブジェクトへのポインタ
    nex_demo_util::P2PStartApiChain* m_pP2pStartApiChain;              // P2P通信の開始処理を行うオブジェクトへのポインタ
    nex_demo_util::P2PShutdownApiChain* m_pP2pShutdownApiChain;        // P2P通信の終了処理を行うオブジェクトへのポインタ
};

/*!
    @brief P2P通信でセッションマスターが全参加者にセッションの終了を通知する。
 */
void AutoMatchmakeSample::NotifyGameOverToAllParticipants()
{
    if (nn::nex::Session::GetRole() == nn::nex::SESSION_ROLE_MASTER)
    {
        // GameOver() は非同期に全参加者に通知を送信します。
        NEX_DEMO_LOG("Notify GameOver\n");
        nn::nex::Session::GetInstance()->GameOver();
    }
}

/*!
    @brief P2P通信のための初期化処理を行う。
 */
qBool AutoMatchmakeSample::InitializeCommunication()
{
    if (IsOwner())
    {
        World* pWorld = World::CreateWellKnown(nn::nex::g_hWorld);
        pWorld->SetBoundary(0, 0, 100, 100);
        pWorld->Publish();
    }
    return true;
}

/*!
    @brief P2P通信開始時の処理を行う。
 */
qBool AutoMatchmakeSample::StartCommunication()
{
    // DO を作成し、パブリッシュする
    // これで他のステーションにデュプリカが作成される
    // 初期位置はランダム
    qReal x = static_cast<qReal>(nn::nex::Platform::GetRandomNumber(100));
    qReal y = static_cast<qReal>(nn::nex::Platform::GetRandomNumber(100));

    Avatar* pAvatar = Avatar::Create();
    if (!pAvatar)
    {
        /*
           セッション参加後の複製オブジェクトの初期化処理が完了するよりも前に、
           親と通信できなくなったレアなケースで、DuplicatedObject::Create でNULLが返ります。
           この場合、NetZインスタンスを破棄してセッションから抜ける必要があります。
        */
        NEX_DEMO_LOG("Avatar create is faild.\n");
        return false;
    }

    // Publishの前にデータをセットすることで、他ステーションに複製オブジェクトが発行された時点でこのデータセットを保持しています
    pAvatar->SetPosition(x, y);
    if (!pAvatar->Publish())
    {
        NEX_DEMO_LOG("Avatar publish is faild.\n");
        /*
           Publishが失敗した場合、以下のようにして複製オブジェクトの主参照を削除する必要があります
           この場合、NetZインスタンスを破棄してセッションから抜ける必要があります。
         */
        pAvatar->DeleteMainRef();
        return false;
    }
    return true;
}

/*!
    @brief P2P通信を終了させる。
 */
void AutoMatchmakeSample::FinishCommunication()
{
    NEX_DEMO_ASSERT(CanCommunicate());
    if (!CanCommunicate())
    {
        return;
    }

    // 全員に解散を通知
    NotifyGameOverToAllParticipants();

    SetState(STATE_P2PSHUTDOWN);
}

/*!
    @brief 接続状態をチェックする。STATE_COMMUNICATEのときのみ使用される。ユーザー定義エラーもしくはqResultを返す。

    @retval nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_SUCCESS) 正常。
    @retval それ以外 切断された場合など、対戦を続行できない状態。
 */
nex_demo_util::ResultContainer AutoMatchmakeSample::CheckCondition()
{
    // ゲームサーバから切断されていないか
    nn::nex::qResult ngsResult = m_pNgsFacade->IsConnected();
    if(!ngsResult)
    {
        NEX_DEMO_LOG("Disconnected from NGS.\n");
        return nex_demo_util::ResultContainer(ngsResult);
    }

    // マッチメイクセッションが削除されていないか
    // 本サンプルでは能動的なマッチメイクセッションの削除はしておらず、削除は想定されないのでエラーとする
    if (GetState() > STATE_P2PINITIALIZE && 
        GetState() < STATE_P2PSHUTDOWN_WAITING && 
        GetGatheringID() == INVALID_GATHERINGID)
    {
        NEX_DEMO_LOG("MatchmakeSession is unregistered.\n");
        return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_MATCHMAKESESSION_UNREGISTERED);
    }


    // P2Pで送受信中のチェック
    if (CanCommunicate())
    {
        // NetZのエラーチェック
        if (m_pNetZ)
        {
            nn::nex::qResult netZError = m_pNetZ->GetFatalError();
            if (!netZError)
            {
                // 復帰不能なエラーが発生
                NEX_DEMO_LOG("Fatal error occurred in NetZ.\n");
                nex_demo_util::ReportQResultReturnCode(netZError);
                return nex_demo_util::ResultContainer(netZError);
            }
        }

        // P2Pセッションが孤立状態になってないかチェック
        // 典型的には、自身がセッションマスターではないときに自分自身の接続状況が悪くてP2Pセッションから切断された場合に孤立状態になります
        // P2P通信中(P2Pセッション確立後、NetZ::Terminate()まで)のみチェックします
        if (m_pHostMigrationCallback && m_pHostMigrationCallback->SessionHostIsOrphan())
        {
            // 孤立状態ならエラーとして扱いマッチメイクを終了する
            NEX_DEMO_LOG("I'm an orphan.\n");
            return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_P2P_SESSION_ORPHAN);
        }

        // 意図せずホスト情報の変更があった場合はP2Pセッションの分離があったと判断し、エラーとする
        // 典型的には、自身がセッションマスターのときに自分自身の接続状況が悪くてP2Pセッションから切断された場合に発生します
        if (m_unexpectedHostChangeOccurred)
        {
            // セッションマスター一人が孤立したとは限らないのでセッション内全員を解散させる必要がある。
            NEX_DEMO_LOG("Detected unexpected host change.\n");
            NotifyGameOverToAllParticipants();
            return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_UNEXPECTED_HOST_CHANGE);
        }

        // セッションが終了したかどうかをチェック
        // Session::GameOver() を利用している場合のみ必要なチェックです
        if (nn::nex::Session::GetInstance()->CheckGameIsOver())
        {
            NEX_DEMO_LOG("Notified GameOver.\n");
            return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_GAME_OVER);
        }

        // 意図しない非同期処理の失敗があったかチェック
        if (m_unexpectedRmcFailureOccurred)
        {
            NEX_DEMO_LOG("Detected unexpected failure.\n");
            return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_UNEXPECTED_RMC_FAILURE);
        }
    }

    // ネットワークの接続性確認
    if(!nex_demo_util::IsNetworkConnected())
    {
        NEX_DEMO_LOG("Network connection unavailable.\n");
        return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_NETWORK_DISCONNECTED);
    }
    return nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_SUCCESS);
}

/*!
    @brief オートマッチメイクを行う。 マッチメイクセッションに参加または作成する。
 */
void AutoMatchmakeSample::AutoMatchmake()
{
    NEX_DEMO_LOG("\nCall AutoMatchmake().\n");

    // 作成するマッチメイクセッションのパラメータをセットアップ
    nn::nex::MatchmakeSession matchmakeSession;

    // 締切状態の設定
    matchmakeSession.SetOpenParticipation(true);

    // 最小・最大参加人数
    matchmakeSession.SetMinParticipants(1);
    matchmakeSession.SetMaxParticipants(MATCHMAKE_MAX_PARTICIPANTS);

    // ゲームモード
    matchmakeSession.SetGameMode(MATCHMAKE_GAME_MODE);

    // マッチメイクシステムタイプ
    matchmakeSession.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // オーナーがマッチメイクセッションから退出したときにオーナーの委譲を行うか
    matchmakeSession.SetFlag(nn::nex::GatheringFlags::MigrateOwner);

    // 他人がオーナー権限を奪えるか。MatchmakingClient::StaticUpdateSessionHostWithOwner() を使うため必要。
    matchmakeSession.SetFlag(nn::nex::GatheringFlags::ChangeOwnerByOtherHost);

    // 参加などの通知をマッチメイクセッションの参加者全員に行いたい場合はこのフラグをセットする
    // すでに参加しているユーザーの nn::nex::ParticipationEvents::Participate 通知を受け取りたい場合は
    // nn::nex::GatheringFlags::NotifyParticipationEventsToAllParticipantsReproducibly を利用することもできます。
    // これは P2P 通信を行わずにマッチメイクセッションで参加者の管理を行う場合に有用です。
    //matchmakeSession->SetFlag(nn::nex::GatheringFlags::NotifyParticipationEventsToAllParticipants);

    // オートマッチメイクのパラメータをセットアップ
    // AutoMatchmakeParam::SetSearchCriteria() や AutoMatchmakeParam::SetTargetGatheringId() を使用していないため、
    // AutoMatchmakeParam::SetSourceMatchmakeSession() で指定したパラメータから自動的に検索条件が作成されます。
    nn::nex::AutoMatchmakeParam autoMatchmakeParam;
    autoMatchmakeParam.SetSourceMatchmakeSession(matchmakeSession);
    autoMatchmakeParam.SetJoinMessage(NEX_T("auto matchmake session"));

    // RMC呼び出し時に完了コールバックが呼ばれる場合があるので先にステートを変更する
    SetState(STATE_RMC_RESULT_WAITING);

    // オートマッチメイクを行う
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&AutoMatchmakeSample::OnCompletionAutoMatchmake), this));
    m_matchMakingClient.AutoMatchmake(pCallContext, autoMatchmakeParam, &m_joinedMatchmakeSession);
}

/*!
    @brief オートマッチメイク完了コールバック。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void AutoMatchmakeSample::OnCompletionAutoMatchmake(nn::nex::ProtocolCallContext* pCallContext)
{
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("     AutoMatchmake() failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        ErrorOccurred(nex_demo_util::ResultContainer(result));
        return;
    }

    NEX_DEMO_LOG("     AutoMatchmake() succeeded.\n");
    SetGatheringID(m_joinedMatchmakeSession.GetID());
    SetOwnerPid(m_joinedMatchmakeSession.GetOwnerPID());
    NEX_DEMO_LOG("[PID Info >> JoinedGID:%u OwnerPID:" NEX_PID_FORMAT " MyPID:" NEX_PID_FORMAT "]\n",
                GetGatheringID(), 
                GetOwnerPid(),
                GetMyPid());

    SetState(STATE_P2PINITIALIZE);
}

/*!
    @brief マッチメイクセッション退出。
 */
void AutoMatchmakeSample::EndParticipation()
{
    if (GetGatheringID() == INVALID_GATHERINGID)
    {
        // 参加していないのでEndParticipation()は呼び出さない。
        SetState(STATE_FINISH);
        return;
    }
    NEX_DEMO_LOG("\nCall EndParticipation().\n");

    // RMC呼び出し時に完了コールバックが呼ばれる場合があるので先にステートを変更する
    SetState(STATE_RMC_RESULT_WAITING);
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&AutoMatchmakeSample::OnCompletionEndParticipation), this));
    m_matchMakingClient.EndParticipation(pCallContext, GetGatheringID());
    SetGatheringID(INVALID_GATHERINGID);
}

/*!
    @brief マッチメイクセッション退出完了時に呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void AutoMatchmakeSample::OnCompletionEndParticipation(nn::nex::ProtocolCallContext* pCallContext)
{
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result && !(result == QERROR(RendezVous, SessionVoid)))
    {
        // すでにサーバーから切断されているときなどは失敗する
        NEX_DEMO_LOG("     EndParticipation() failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
    }
    else
    {
        NEX_DEMO_LOG("     EndParticipation() succeeded.\n");
    }
    SetState(STATE_FINISH);
}

/*!
    @brief マッチメイクセッションの締め切り処理を行う。

    ステートの変化には依存しない形で呼び出しを行います。AutoMatchmakeSampleオブジェクトを破棄するときに処理中だった場合は自動的にキャンセルされます。
 */
void AutoMatchmakeSample::CloseParticipation()
{
    NEX_DEMO_LOG("\nCall CloseParticipation().\n");

    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&AutoMatchmakeSample::OnCompletionCloseParticipation), this));

    // あとで非同期処理をキャンセルできるようにするため、生成したCallContextを非同期処理実行中のみm_callContextStoreに保存するようにします。
    nex_demo_util::CallContextStoreAssociator::Register(&m_callContextStore, pCallContext);
    m_matchMakingClient.CloseParticipation(pCallContext, GetGatheringID());
}

/*!
    @brief マッチメイクセッション締め切り完了時に呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void AutoMatchmakeSample::OnCompletionCloseParticipation(nn::nex::ProtocolCallContext* pCallContext)
{
    switch (pCallContext->GetState())
    {
    case nn::nex::CallContext::CallSuccess:
        NEX_DEMO_LOG("     CloseParticipation() succeeded.\n");
        break;
    case nn::nex::CallContext::CallCancelled:
        // キャンセルされるのは ~AutoMatchmakeSample() からのみなのでなにもしない
        NEX_DEMO_LOG("     CloseParticipation() cancelled.\n");
        break;
    case nn::nex::CallContext::CallFailure:
        NEX_DEMO_LOG("     CloseParticipation() failed.\n");
        nex_demo_util::ReportQResultReturnCode(pCallContext->GetOutcome());
        UnexpectedRmcFailureOccurred();
        break;
    default:
        NEX_DEMO_ASSERT(false);
        UnexpectedRmcFailureOccurred();
        break;
    }
}

/*!
    @brief 初回エラー発生時にエラーステートに遷移する。

    @param[in] result 発生したエラーが格納された結果。
 */
void AutoMatchmakeSample::ErrorOccurred(const nex_demo_util::ResultContainer& result)
{
    // すでにエラー状態ならなにもしない
    if (m_errorOccurred)
    {
        return;
    }

    m_errorOccurred = true;
    m_lastResult = result;

    // P2P初期化後
    if (m_pNetZ)
    {
        SetState(STATE_P2PSHUTDOWN);
    }
    // マッチメイクセッション参加後
    else if (GetGatheringID() != INVALID_GATHERINGID)
    {
        SetState(STATE_ENDPARTICIPATION);
    }
    // マッチメイクセッション参加前
    else
    {
        SetState(STATE_FINISH);
    }
}

/*!
    @brief ステートを遷移させる。
 */
qBool AutoMatchmakeSample::ProcessState()
{
    switch(GetState())
    {
    // オートマッチメイク
    case STATE_AUTOMATCHMAKE:
        AutoMatchmake();
        break;
    // P2P通信の初期化
    case STATE_P2PINITIALIZE:
        m_pP2pInitializeApiChain = new nex_demo_util::P2PInitializeApiChain(
            &m_pNetZ,
            &m_pHostMigrationCallback,
            &m_matchMakingClient,
            m_joinedMatchmakeSession.GetSessionKey());
        m_pP2pInitializeApiChain->Call();
        SetState(STATE_P2PINITIALIZE_WAITING);
        break;
    // P2P通信の初期化完了待ち
    case STATE_P2PINITIALIZE_WAITING:
        if (m_pP2pInitializeApiChain->IsCompleted())
        {
            nn::nex::qResult result = m_pP2pInitializeApiChain->GetResult();
            if (result)
            {
                SetState(STATE_P2PSTART);
            }
            else
            {
                ErrorOccurred(nex_demo_util::ResultContainer(result));
            }
            delete m_pP2pInitializeApiChain;
            m_pP2pInitializeApiChain = NULL;
        }
        break;
    // P2P通信の開始
    case STATE_P2PSTART:
        if (InitializeCommunication())
        {
            m_pP2pStartApiChain = new nex_demo_util::P2PStartApiChain(
                &m_matchMakingClient,
                m_pHostMigrationCallback,
                IsOwner(),
                GetGatheringID());
            m_pP2pStartApiChain->Call();
            SetState(STATE_P2PSTART_WAITING);
        }
        else
        {
            SetState(STATE_ENDPARTICIPATION);
        }
        break;
    // P2P通信の開始完了待ち
    case STATE_P2PSTART_WAITING:
        if (m_pP2pStartApiChain->IsCompleted())
        {
            nn::nex::qResult result = m_pP2pStartApiChain->GetResult();
            if (result)
            {
                if (StartCommunication())
                {
                    SetState(STATE_COMMUNICATE);
                }
                else
                {
                    ErrorOccurred(nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_DISCONNECTED_FROM_P2P));
                }
            }
            else
            {
                ErrorOccurred(nex_demo_util::ResultContainer(result));
            }
            delete m_pP2pStartApiChain;
            m_pP2pStartApiChain = NULL;
        }
        break;
    // P2P通信が可能なステート
    case STATE_COMMUNICATE:
        {
            nex_demo_util::ResultContainer result = CheckCondition();
            if (!result)
            {
                if (result == nex_demo_util::ResultContainer(USER_DEFINED_RESULT_CODE_GAME_OVER))
                {
                    // GAME_OVERなら終了処理へ移行
                    SetState(STATE_P2PSHUTDOWN);
                }
                else
                {
                    // GAME_OVER以外のエラーならエラー状態にする
                    ErrorOccurred(result);
                }
            }
        }
        break;
    // P2P通信の終了処理
    case STATE_P2PSHUTDOWN:
        m_pP2pShutdownApiChain = new nex_demo_util::P2PShutdownApiChain(
            &m_pNetZ,
            &m_pHostMigrationCallback,
            GetGatheringID());
        m_pP2pShutdownApiChain->Call();
        SetState(STATE_P2PSHUTDOWN_WAITING);
        break;
    // P2P通信の終了処理完了待ち
    case STATE_P2PSHUTDOWN_WAITING:
        if (m_pP2pShutdownApiChain->IsCompleted())
        {
            // P2PShutdownApiChainは失敗しない
            NEX_DEMO_ASSERT(m_pP2pShutdownApiChain->GetResult());
            SetState(STATE_ENDPARTICIPATION);
            delete m_pP2pShutdownApiChain;
            m_pP2pShutdownApiChain = NULL;
        }
        break;
    // マッチメイクセッション退出
    case STATE_ENDPARTICIPATION:
        EndParticipation();
        break;
    // RMC (AutoMatchmake(), EndParticipation()) 処理結果待ち
    case STATE_RMC_RESULT_WAITING:
        break;
    // 全ての処理が完了
    case STATE_FINISH:
        m_finished = true;
        break;
    }

    return m_finished;
}

/*!
    @brief 接続情報をデバッグログに出力します。
 */
void OutputConnectionInfo()
{
    nn::nex::ScopedCS oCS(nn::nex::Scheduler::GetInstance()->SystemLock());
    nn::nex::Station::SelectionIterator itStations;
    itStations.GotoStart();
    while (!itStations.EndReached())
    {
        NEX_DEMO_LOG("Station %8x: URL = " STRING_FORMAT "\n",
                     itStations->GetHandle().GetValue(),
                     itStations->GetStationURL(0));

        ++itStations;
    }
    NEX_DEMO_LOG("\n");
}

/*!
    @brief P2P通信で送受信する。

    本サンプルではDOを使用しています。

    @param[in] communicationFrame ゲームループを回るごとにインクリメントされる値。
 */
void Communicate(nn::nex::qUnsignedInt32 communicationFrame)
{
    if (communicationFrame % 60 == 0)
    {
        // 全アバタを更新します
        nn::nex::ScopedCS oCS(nn::nex::Scheduler::GetInstance()->SystemLock());
        Avatar::SelectionIterator itAvatar;
        itAvatar.GotoStart();
        while (!itAvatar.EndReached())
        {
            itAvatar->DoPhysic();
            ++itAvatar;
        }
    }

    if (communicationFrame % 60 == 0)
    {
        // 接続されているステーションを表示
        NEX_DEMO_LOG("\n\nStations currently in Session\n");
        NEX_DEMO_LOG("-----------------------------\n");
        OutputConnectionInfo();

        // DO の状態も表示する
        {
            nn::nex::ScopedCS oCS(nn::nex::Scheduler::GetInstance()->SystemLock());
            Avatar::SelectionIterator itAvatar;
            itAvatar.GotoStart();
            while (!itAvatar.EndReached())
            {
                itAvatar->Render();
                ++itAvatar;
            }

            // ワールドの描画
            World::Ref refWorld(nn::nex::g_hWorld);
            refWorld->Render();
        }
        NEX_DEMO_LOG("-----------------------------\n");
    }
}

/*!
    @brief オートマッチメイクゲームループ。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
qBool AutoMatchmakeGameLoop(nn::nex::NgsFacade* pNgsFacade)
{
    AutoMatchmakeSample sample(pNgsFacade);
    nn::nex::qUnsignedInt32 communicationFrame = 0;
    qBool isMatchmakeSessionOpen = true;

    for(;;)
    {
        // ゲームフレームのスタート

        // このDispatchAllでは主にパケットの受信が行われる
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // ここでゲームのキー入力を行う
        // ここでゲームの計算処理を行う

        // ステートの遷移
        qBool finished = sample.ProcessState();

        // 終了確認
        if(finished)
        {
            break;
        }

        // P2P通信が可能な状態での処理
        if (sample.CanCommunicate())
        {
            // 最大人数分集まったら参加を締め切る。締め切らなくても最大人数より多くは参加されませんが、人数が減った際にも参加されなくなります。
            if (isMatchmakeSessionOpen && nn::nex::Session::GetRole() == nn::nex::SESSION_ROLE_MASTER)
            {
                if (nn::nex::Station::GetApprovedStationNum() >= MATCHMAKE_MAX_PARTICIPANTS)
                {
                    // 本サンプルではP2Pセッションのホストがマッチメイクセッションのオーナーと一致するようにしているため CloseParticipation() を呼び出せます。
                    // P2Pセッションへの参加処理は若干時間がかかるため、最大人数未満の状態でマッチメイクセッションを締め切ると
                    // その後若干の間P2Pセッションへ参加してくる可能性があることに注意してください。
                    sample.CloseParticipation();
                    isMatchmakeSessionOpen = false;
                }
            }

            // P2Pの送受信処理を行う
            Communicate(communicationFrame);
            if (++communicationFrame > COMMUNICATION_DURATION)
            {
                // 一定時間経ったらP2P通信を終了する
                sample.FinishCommunication();
            }
        }

        // パケット処理の遅延を減らすため、1ゲームフレームでゲームの計算処理の前と後の二回 Scheduler::DispatchAll() を呼び出すのが望ましい
        // このDispatchAllでは主にパケットの送信が行われる
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // ここでゲームの描画処理を行う

        // ゲームでは適宜フレームの同期を行ってください
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
    }

    // エラー終了した場合はエラー内容を表示
    nex_demo_util::ResultContainer result = sample.GetLastResult();
    if (!result)
    {
        // 必要に応じてエラー・EULAアプレット/エラービューアでエラーメッセージを表示してください。
        result.ReportError("An error occurred in AutoMatchmakeSample.\n");
    }

    return true;
}

/*!
    @brief NEXの初期化後にNGSへログインし、マッチメイク用のゲームループに遷移します。
 */
qInt NEXSampleMain()
{

    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログのレベルを設定します
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // P2P通信がうまくいかない場合には、以下のコメントアウトを外して
    // NATトラバーサルログを出力してください。
    //nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS|TRACE_NAT);

    // セッション参加要求受付時のコールバックを設定
    nn::nex::Session::RegisterJoinApprovalCallback(CheckJoiningProcesses);

    // P2P通信で使用する、オペレーションコールバックの設定 (ステーション参加時、退出時などのコールバックに利用されます)
    nn::nex::Station::RegisterOperationBeginCallback(StationOperationBeginCallback);
    nn::nex::Station::RegisterOperationEndCallback(StationOperationEndCallback);

    // 必要に応じてP2P通信で使用するReliable通信用の最大SubStreamID MAX_SUBSTREAM_IDを指定します
    //nn::nex::Stream::GetSettings().SetMaxUserReliableSubStreams(1);

    // ネットワークトポロジ設定
    nn::nex::NetZ::SetNetworkTopology(nn::nex::NetworkTopology::Peer2Peer);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);
    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            // オートマッチメイクゲームループ
            if (!AutoMatchmakeGameLoop(&ngsFacade))
            {
                NEX_DEMO_LOG("AutoMatchmakeGameLoop finished with error.\n");
            }

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }
    }

    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    NEX_DEMO_LOG("Leaving application...\n");

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("**************************************\n");
    NEX_DEMO_LOG(" AutoMmDupObj sample\n");
    NEX_DEMO_LOG("**************************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
