﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
#include "Avatar.h"
#include "World.h"
#include <NexDemoUtil/NexUtil.h>


/*!
    @brief コンストラクタ。

    原点でオブジェクトを生成します。
 */
Avatar::Avatar()
{
    SetPosition(0.0f, 0.0f);
    UpdateMoveDir();
}

/*!
    @brief コンストラクタ。

    @param[in] x 出現させるX座標。
    @param[in] y 出現させるY座標。
 */
Avatar::Avatar(qReal x, qReal y)
{
    SetPosition(x, y);
    UpdateMoveDir();
}

/*!
    @brief デストラクタ。
 */
Avatar::~Avatar()
{
    NEX_DEMO_LOG("Avatar Destruct!!\n");
}

/*!
    @brief 複製オブジェクトが初期化されたときにライブラリから呼び出されます。
 */
void Avatar::InitDO()
{
    if (!IsADuplicationMaster())
    {
        return;
    }

    // 自身のステーションID を取得
    m_dsInfo.stationID = nn::nex::Station::GetLocalInstance()->GetStationID();

    // 推定遅延時間を設定
    m_dsExtrapolatedLoopbackPosition.GetPHBDRParameters()->SetExtrapolationDelay(Avatar::EXTRAPOLATION_DELAY_TIME);
    m_dsExtrapolatedLoopbackPosition.SetLoopbackDelay(Avatar::LOOPBACK_DELAY_TIME);

    // m_dsPositionをSUBSTREAM_ID経由で送信
    // 必要に応じてnn::nex::SubStreamIDDefine::SYSTEM以外を指定することでシステムとは別チャネルで送受信できます。
    // その際、使用するストリームIDの最大値を
    // nn::nex::Stream::GetSettings().SetMaxUserReliableSubStreams(1);
    // というように設定しておく必要があります。
    m_dsPosition.SetSubStreamID(SUBSTREAM_ID);

    NEX_DEMO_LOG("*** Avatar::InitDO!!\n");
}

/*!
    @brief 位置を設定する。
 */
void Avatar::SetPosition(qReal x, qReal y)
{
    m_dsPosition.Set(x, y);
    m_dsExtrapolatedPosition.Set(m_dsPosition.x, m_dsPosition.y);
    m_dsExtrapolatedLoopbackPosition.Set(m_dsPosition.x, m_dsPosition.y);
}

/*!
    @brief 位置を更新する。
 */
void Avatar::MovePosition(qReal x, qReal y)
{
    m_dsPosition.Move(x, y);
    m_dsExtrapolatedPosition.Set(m_dsPosition.x, m_dsPosition.y);
    m_dsExtrapolatedLoopbackPosition.Set(m_dsPosition.x, m_dsPosition.y);
}

/*!
    @brief メンバの更新及び DO のアップデートを行う。
 */
void Avatar::DoPhysic()
{
    // 自身が複製マスタの場合のみ更新する
    if (IsADuplicationMaster())
    {
        // 移動更新
        (void)MovePosition(m_velocity[0], m_velocity[1]);

        // 時間チェックして経過していたら移動パラメータを更新
        if (nn::nex::SystemClock::GetTime() - m_movedTime > Avatar::MOVE_TIME)
        {
            UpdateMoveDir();
        }

        // ワールドとの当たり判定
        UpdateCollisionWithWorld();

        // 更新して自身の情報をデュプリカに伝播させます
        Update();

        // ループバック更新
        m_dsExtrapolatedLoopbackPosition.UpdateLoopback();
        m_dsExtrapolatedLoopbackPosition.RefreshLoopback();
    }
    else
    {
        // 推定フィルタの補間を更新します
        Refresh();
    }
}

/*!
    @brief 描画処理。
 */
void Avatar::Render()
{
    // 位置情報を出力
    NEX_DEMO_LOG("Avatar station ID: %d\n", m_dsInfo.stationID);
    NEX_DEMO_LOG("Avatar Info (                     Pos) >> x:%f y:%f\n", m_dsPosition.x, m_dsPosition.y);
    NEX_DEMO_LOG("Avatar Info (Extrapolated         Pos) >> x:%f y:%f\n", m_dsExtrapolatedPosition.x,
                 m_dsExtrapolatedPosition.y);
    NEX_DEMO_LOG("Avatar Info (ExtrapolatedLoopback Pos) >> x:%f y:%f\n", m_dsExtrapolatedLoopbackPosition.x,
                 m_dsExtrapolatedLoopbackPosition.y);
}

/*!
    @brief 移動量の更新。
 */
void Avatar::UpdateMoveDir()
{
    for (int i=0; i<2; ++i)
    {
        int dir = nn::nex::Platform::GetRandomNumber(2);
        m_velocity[i] = dir == 0 ? 1.0f : -1.0f * MOVE_POWER;
    }

    // 経過時間もリセット
    m_movedTime = nn::nex::SystemClock::GetTime();
}

/*!
    @brief 世界とのコリジョン更新。
 */
void Avatar::UpdateCollisionWithWorld()
{
    UpdateCollisionWithWorldSub(m_dsPosition.x, m_dsPosition.y);
    UpdateCollisionWithWorldSub(m_dsExtrapolatedPosition.x, m_dsExtrapolatedPosition.y);
    UpdateCollisionWithWorldSub(m_dsExtrapolatedLoopbackPosition.x, m_dsExtrapolatedLoopbackPosition.y);
}

/*!
    @brief 世界とのコリジョンを更新するためのヘルパー関数。
 */
void Avatar::UpdateCollisionWithWorldSub(qReal& x, qReal& y)
{
    qReal minx, miny, maxx, maxy;

    World::Ref refWorld(nn::nex::g_hWorld);

    // 境界取得
    refWorld->GetBoundary(minx, miny, maxx, maxy);

    if (x < minx || x > maxx)
    {
        if (x < minx)
        {
            x = minx;
        }
        else
        {
            x = maxx;
        }
        m_velocity[0] *= -1.0f;
        m_movedTime = nn::nex::SystemClock::GetTime();
        NEX_DEMO_LOG("Collision X!!\n");
    }
    if (y < miny || y > maxy)
    {
        if (y < miny)
        {
            y = miny;
        }
        else
        {
            y = maxy;
        }
        m_velocity[1] *= -1.0f;
        m_movedTime = nn::nex::SystemClock::GetTime();
        NEX_DEMO_LOG("Collision Y!!\n");
    }
}
