﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * Messaging
 *
 * Description:
 *
 *  メッセージング処理を行うサンプルです。
 *
 *  メッセージ送受信は各送受信者が同じゲームサーバにログインしている状態のみ使用可能です。
 *  本サンプルではログインした後、同じマッチメイクセッションに参加しているクライアントに対して
 *  メッセージを送ります。
 *
 *  ログイン後、A ボタン押下でマッチメイクセッションの参加者にメッセージを送信します。
 *  送信メッセージは、テキストとバイナリの2種類を使用します。
 *  本サンプルを実行中の他のフレンドがいる場合、ログに受信メッセージが表示されます。
 *  B ボタン押下でサンプルを終了します。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>
#include <NexDemoUtil/DemoPad.h>

namespace
{

// 最大参加人数
const qUnsignedInt32 MESSAGING_MAX_PARTICIPANTS = 5;

// 使用するゲームモード
const qUnsignedInt32 MESSAGING_GAME_MODE = 1002;

/*!
    @brief 送信メッセージを処理するイベントハンドラ。
 */
class MyMessagingNotificationHandler: public nn::nex::MessagingNotificationHandler
{
public:
    void DeliverInstantMessage(const nn::nex::DataHolder& hMsg)
    {
        // TextMessage,BinaryMessageを判別します。

        // TextMessage::DynamicCast()が成功する場合、受信メッセージはTextMessageです。
        nn::nex::TextMessage* pTextMsg = nn::nex::TextMessage::DynamicCast(hMsg.Get());
        if (pTextMsg)
        {
            NEX_DEMO_LOG("*** Recieved message from:[" NEX_PID_FORMAT "] subject[" STRING_FORMAT "] text body:[" STRING_FORMAT "]\n",
                         pTextMsg->GetSenderID(),
                         pTextMsg->GetSubject().CStr(),
                         pTextMsg->GetTextBody().CStr());
            return;
        }

        // BinaryMessage::DynamicCast()が成功する場合、受信メッセージはBinaryMessageです。
        nn::nex::BinaryMessage* pBinaryMsg = nn::nex::BinaryMessage::DynamicCast(hMsg.Get());
        if (pBinaryMsg)
        {
            NEX_DEMO_LOG("*** Recieved message from:[" NEX_PID_FORMAT "] subject[" STRING_FORMAT "] binary size:[" NEX_SIZE_T_FORMAT "]\n",
                         pBinaryMsg->GetSenderID(),
                         pBinaryMsg->GetSubject().CStr(),
                         pBinaryMsg->GetBufferSize());
            return;
        }
    }
};

/*!
    @brief 既存のマッチメイクセッションに参加または新規作成する。

    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[out] pGatheringId 参加したマッチメイクセッションのギャザリングID

    @return 処理が成功したかどうか。
 */
qBool DoAutoMatchmake(nn::nex::MatchmakeExtensionClient *pMatchMakingClient,
                      nn::nex::GatheringID* pGatheringId)
{
    *pGatheringId = INVALID_GATHERINGID;

    // 検索条件の作成
    // 検索に失敗した時に作成するマッチメイクの設定を行い、
    // 同様に検索条件も設定する。
    nn::nex::MatchmakeSession matchmakeSession;

    // 常に待ちうけ状態
    matchmakeSession.SetOpenParticipation(true);

    // 最小・最大参加人数
    matchmakeSession.SetMaxParticipants(MESSAGING_MAX_PARTICIPANTS);

    // ゲームモード
    matchmakeSession.SetGameMode(MESSAGING_GAME_MODE);

    // システムタイプ
    matchmakeSession.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // 検索条件の設定
    nn::nex::MatchmakeSessionSearchCriteria criteria;
    criteria.SetMaxParticipants(matchmakeSession.GetMaxParticipants());
    criteria.SetGameMode(matchmakeSession.GetGameMode());
    criteria.SetMatchmakeSystemType(matchmakeSession.GetMatchmakeSystemType());

    nn::nex::qList<nn::nex::MatchmakeSessionSearchCriteria> criteriaList;
    criteriaList.push_back(criteria);

    nn::nex::AutoMatchmakeParam autoMatchmakeParam;
    autoMatchmakeParam.SetSourceMatchmakeSession(matchmakeSession);
    autoMatchmakeParam.SetSearchCriteria(criteriaList);

    // マッチメイクセッションを検索または作成する
    nn::nex::ProtocolCallContext oContext;
    pMatchMakingClient->AutoMatchmake(&oContext, autoMatchmakeParam, &matchmakeSession);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to AutoMatchmake\n");
        return false;
    }

    *pGatheringId = matchmakeSession.GetID();

    return true;
}

/*!
    @brief マッチメイクセッションの終了処理を行う。

    自分がオーナーである場合はマッチメイクセッションを削除し、参加者である場合は退出する。

    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[in] gatheringId 退出するマッチメイクセッションのギャザリングID

    @return 処理が成功したかどうか。
 */
qBool DoEndParticipation(nn::nex::MatchmakeExtensionClient *pMatchMakingClient,
                         nn::nex::GatheringID gatheringId)
{
    nn::nex::ProtocolCallContext     oContext;

    pMatchMakingClient->EndParticipation(&oContext, gatheringId);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        if(oContext.GetOutcome() == QERROR(RendezVous, SessionVoid))
        {
            NEX_DEMO_LOG("Already unregisterd. Don't care.");
            return true;
        }

        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to EndParticipation\n");
        return false;
    }

    return true;
}

/*!
    @brief 指定したギャザリングIDの参加者宛（自分も含む）にメッセージを送信する。

    @param[in] pMessagingClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[in] target 送信先のギャザリングID。
    @param[in] message 送信するテキストメッセージ。
    @param[in] binary 送信するバイナリメッセージ。
 */
void SendInstantMessage(nn::nex::MessagingClient *pMessagingClient,
                        nn::nex::GatheringID target, const nn::nex::String& message, const nn::nex::qVector<qByte>& binary)
{
    // テキストメッセージを送信する。
    {
        nn::nex::TextMessage *pTextMessage = new nn::nex::TextMessage();
        nn::nex::UserMessageDataHolder oDataHolder = pTextMessage;

        // テキストメッセージをセットする
        pTextMessage->SetSubject(NEX_T("This is a TextMessage"));
        pTextMessage->SetTextBody(message);

        // メッセージ送信
        nn::nex::ProtocolCallContext oContext;
        NEX_DEMO_LOG("Send message to session %d participants ... \n", target);

        // SendInstant/SendChat が送信対象のクライアントやマッチメイクセッションが存在しない場合でも
        // 非同期処理が成功を返します。
        pMessagingClient->SendChat(&oContext, target, oDataHolder);
        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "MessagingClient::SendChat TextMessage failed!!\n");
        }

        NEX_DEMO_LOG("MessagingClient::SendChat TextMessage successfly.\n");
    }

    // バイナリメッセージを送信する。
    {
        nn::nex::BinaryMessage *pBinaryMessage = new nn::nex::BinaryMessage();
        nn::nex::UserMessageDataHolder oDataHolder = pBinaryMessage;

        // バイナリメッセージをセットする
        pBinaryMessage->SetSubject(NEX_T("This is a BinaryMessage"));
        pBinaryMessage->SetBuffer(static_cast<const void*>(&binary[0]), binary.size());

        // メッセージ送信
        nn::nex::ProtocolCallContext oContext;
        NEX_DEMO_LOG("Send message to session %d participants ... \n", target);

        // SendInstant/SendChat が送信対象のクライアントやマッチメイクセッションが存在しない場合でも
        // 非同期処理が成功を返します。
        pMessagingClient->SendChat(&oContext, target, oDataHolder);
        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "MessagingClient::SendChat BinaryMessage failed!!\n");
        }

        NEX_DEMO_LOG("MessagingClient::SendChat BinaryMessage successfly.\n");
    }
}

MyMessagingNotificationHandler s_MyMessagingNotificationHandler;

/*!
    @brief メッセージングで通信します。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
    @param[in] gatheringId 送信先のギャザリングID。
 */
void DoMessaging(nn::nex::NgsFacade* pNgsFacade, nn::nex::GatheringID gatheringId)
{
    nn::nex::MessagingClient messageClient;
    messageClient.Bind(pNgsFacade->GetCredentials());

    // メッセージ通知を処理するイベントハンドラを登録
    messageClient.RegisterMessagingNotificationHandler(&s_MyMessagingNotificationHandler);

    // HID初期化
    nex_demo_util::DemoPad::Initialize();
    nex_demo_util::PadStatus oPadStatus;

    // ボタン押下でにメッセージを送信します。
    {
        NEX_DEMO_LOG("\nPress A to Send Message.\n");
        NEX_DEMO_LOG("Press B to Quit Sample.\n");

        while (1)
        {
            nex_demo_util::DemoPad::Update(&oPadStatus);

            if (oPadStatus.trigger & DEMO_PAD_BUTTON_A)
            {
                nn::nex::StringStream ss;
                ss << NEX_T("Hello! I am ") << pNgsFacade->GetCredentials()->GetPrincipalID();
                nn::nex::String message(ss.CStr());

                nn::nex::qVector<qByte> binary(123);

                SendInstantMessage(&messageClient, gatheringId, message, binary);
            }
            else if (oPadStatus.trigger & DEMO_PAD_BUTTON_B)
            {
                break;
            }

            nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
            nn::nex::Scheduler::GetInstance()->DispatchAll();
        }
    }
    // HID終了
    nex_demo_util::DemoPad::Finalize();
}

/*!
    @brief マッチメイクセッションに入りメッセージングによる通信を行います。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void DoMatchmakingAndMessaging(nn::nex::NgsFacade* pNgsFacade)
{
    nn::nex::MatchmakeExtensionClient matchMakingClient;
    matchMakingClient.Bind(pNgsFacade->GetCredentials());

    // オートマッチメイクでマッチメイクセッションに参加する。
    nn::nex::GatheringID gatheringId;

    if(!DoAutoMatchmake(&matchMakingClient, &gatheringId))
    {
        NEX_DEMO_LOG("Fail to DoAutoMatchmake\n");
        return;
    }

    // メッセージングで通信する。
    DoMessaging(pNgsFacade, gatheringId);

    // マッチメイクセッションから退出する。
    if(!DoEndParticipation(&matchMakingClient, gatheringId))
    {
        NEX_DEMO_LOG("Fail to DoEndParticipation\n");
        return;
    }
}

/*!
    @brief NEXの初期化後にNGSへログインし、マッチメイクとメッセージングによる通信処理に遷移します。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            DoMatchmakingAndMessaging(&ngsFacade);

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }

    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*************************************\n");
    NEX_DEMO_LOG(" AutoMmMessagingEasy sample\n");
    NEX_DEMO_LOG("*************************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
