﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * CreateAndBrowse
 *
 * Description:
 *
 *  マッチメイクセッションの作成/検索/参加を行うサンプルです。
 *
 *  本サンプルではキー入力によりマッチメイクセッションの作成、検索/参加を選択します。
 *  NGSへのログインが完了後、Aボタンを押すとマッチメイクセッションの作成を行います。
 *  (マッチメイクセッションの作成は自身の参加を伴います)
 *
 *  NGSへのログインが完了後、Bボタンを押すとマッチメイクセッションの検索を行います。
 *  他の端末でマッチメイクセッションの作成をしていればここでマッチメイクセッションが見つかります。
 *  その後見つかったマッチメイクセッションを選択して参加する事ができます。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>
#include <NexDemoUtil/DemoPad.h>

namespace
{
/// マッチメイクセッションの最大参加人数
const qUnsignedInt16 MATCHMAKE_MAX_PARTICIPANTS = 2;

/// マッチメイクセッションに指定するゲームモード
const qUnsignedInt32 MATCHMAKE_GAME_MODE = 1001;

/*!
    @brief NGSから通知を処理するイベントハンドラ。
 */
class MyNotificationEventHandler : public nn::nex::NotificationEventHandler
{
public:
    void ProcessNotificationEvent(const nn::nex::NotificationEvent &oEvent)
    {
        nn::nex::StringStream ss;
        ss << oEvent;
        NEX_DEMO_LOG("Notification " STRING_FORMAT "\n\n",ss.CStr());

        // メッセージ毎の処理
        switch (oEvent.GetType())
        {
        case nn::nex::NotificationEvents::ParticipationEvent:
            switch (oEvent.GetSubType())
            {
            case nn::nex::ParticipationEvents::Participate:
                NEX_DEMO_LOG("PARTICIPATION_EVENT PARTICIPATE: [PID " NEX_PID_FORMAT "] join [MatchmakeSession %u]\n",
                    static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;

            case nn::nex::ParticipationEvents::EndParticipation:
                NEX_DEMO_LOG("PARTICIPATION_EVENT END: [PID " NEX_PID_FORMAT "] end [MatchmakeSession %u]\n",
                    static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;

            case nn::nex::ParticipationEvents::Disconnect:
                NEX_DEMO_LOG("PARTICIPATION_EVENT DISCONNECT: [PID " NEX_PID_FORMAT "] disconnect [MatchmakeSession %u]\n",
                    static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;

            default:
                NEX_DEMO_LOG("PARTICIPATION_EVENT: unsupported participation event\n");
                break;
            }
            break;

        case nn::nex::NotificationEvents::OwnershipChangeEvent:
            NEX_DEMO_LOG("NOTIFICATION_EVENT OWNERSHIPCHANGE: [MatchmakeSession %u] changed owner to [PID " NEX_PID_FORMAT "]\n",
                static_cast<nn::nex::GatheringID>(oEvent.GetParam1()), static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()));
            break;

        case nn::nex::NotificationEvents::GatheringUnregistered:
            NEX_DEMO_LOG("NOTIFICATION_EVENT GATHERINGUNREGISTERED: [MatchmakeSession %u] unregistered\n",
                static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
            break;

        default:
            NEX_DEMO_LOG("NOTIFICATION_EVENT: unsupported notification event\n");
            break;
        }
    }
};


/// マッチメイクで使用するギャザリングID
nn::nex::GatheringID s_JoinedGatheringID = INVALID_GATHERINGID;

/// 通知を処理するイベントハンドラ
MyNotificationEventHandler s_MyNotificationEventHandler;


/*!
    @brief ゲームのメインループを想定した関数です。Bボタンを押すと抜けます。
 */
void RunGameMainLoop()
{
    NEX_DEMO_LOG("In RunGameMainLoop\n");
    NEX_DEMO_LOG("Push B to leave.\n");

    nex_demo_util::PadStatus oPadStatus;

    // 押しっぱなしでトリガーと認識されないように一回Update()しておく。
    nex_demo_util::DemoPad::Update(&oPadStatus);
    while(true)
    {
        nex_demo_util::DemoPad::Update(&oPadStatus);
        if(oPadStatus.trigger == DEMO_PAD_BUTTON_B)
        {
            break;
        }
        nn::nex::Scheduler::GetInstance()->DispatchAll();
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
    }
}

/*!
    @brief マッチメイクセッション一覧の表示を行う。

    @param[in] lstMatchmakeSession 表示するマッチメイクセッションのリスト。
    @param[in] index カーソルを表示するマッチメイクセッションのインデックス。
 */
void PrintMatchmakeSessions(const nn::nex::qList<nn::nex::MatchmakeSession>& lstMatchmakeSession, int index)
{
    (void)index;
    nn::nex::qList<nn::nex::MatchmakeSession>::const_iterator it(lstMatchmakeSession.begin());
    while(it != lstMatchmakeSession.end())
    {
        const nn::nex::MatchmakeSession& matchmakeSession = *it;
        (void)matchmakeSession;
        NEX_DEMO_LOG(STRING_FORMAT" gid:%u, desc:" STRING_FORMAT " owner:" NEX_PID_FORMAT " host:" NEX_PID_FORMAT " %d/%d\n",
                     // 現在選択しているセッション
                     std::distance(lstMatchmakeSession.begin(), it) == index ? NEX_T("*") : NEX_T(" "),
                     // GathringID
                     matchmakeSession.GetID(),
                     // Description
                     matchmakeSession.GetDescription().CStr(),
                     // マッチメイクセッションオーナーのPID
                     matchmakeSession.GetOwnerPID(),
                     // セッションホストのPID
                     matchmakeSession.GetHostPID(),
                     // 現在の参加人数
                     matchmakeSession.GetParticipantCount(),
                     // 最大参加人数
                     matchmakeSession.GetMaxParticipants()
                    );
        ++it;
    }
}

/*!
    @brief マッチメイクセッションの検索を同期的に行う。

    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[out] pLstMatchmakeSession 見つかったマッチメイクセッションを格納するリスト。

    @retval true 検索処理に成功。pLstMatchmakeSessionに見つかったマッチメイクセッションが格納されます。
    @retval false 検索処理に失敗。
 */
bool CallBrowseMatchmakeSession(nn::nex::MatchmakeExtensionClient *pMatchMakingClient,
                                nn::nex::qList<nn::nex::MatchmakeSession> *pLstMatchmakeSession)
{
    NEX_DEMO_LOG("In CallBrowseMatchmakeSession\n");

    nn::nex::ProtocolCallContext     oContext;

    // 検索条件の設定
    nn::nex::MatchmakeSessionSearchCriteria criteria;

    // アプリケーション固有のゲームモード
    criteria.SetGameMode(MATCHMAKE_GAME_MODE);

    // アプリケーションで自由に定義できる検索属性値
    criteria.SetAttribute(0, 3333);

    // マッチメイクシステムタイプ
    criteria.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // 参加可能なマッチメイクセッションをのみ検索
    criteria.SetExcludeLocked(true);

    // 満員のものを除外する
    criteria.SetVacantOnly(true);

    // 検索範囲 デフォルトでオフセット0, 最大20件
    nn::nex::ResultRange range;
    criteria.SetResultRange(range);

    // マッチメイクセッションを検索
    pMatchMakingClient->BrowseMatchmakeSession(&oContext, criteria, pLstMatchmakeSession);
    oContext.Wait();
    if(!oContext.GetOutcome())
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to BrowseMatchmakeSession.");
        return false;
    }

    return true;
}

/*!
    @brief マッチメイクセッションへの参加を同期的に行う。

    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。
    @param[in] gid 参加するマッチメイクセッションのギャザリング ID。

    @retval true 参加処理に成功。
    @retval false 参加処理に失敗。
 */
bool JoinMatchmakeSession(nn::nex::MatchmakeExtensionClient *pMatchMakingClient,
                          nn::nex::GatheringID gid)
{
    nn::nex::ProtocolCallContext oContext;

    nn::nex::JoinMatchmakeSessionParam joinMachmakeSessionParam;
    joinMachmakeSessionParam.SetTargetGatheringId(gid);
    nn::nex::MatchmakeSession joinedMatchmakingSession;

    pMatchMakingClient->JoinMatchmakeSession(&oContext, joinMachmakeSessionParam, &joinedMatchmakingSession);
    oContext.Wait();
    if(!oContext.GetOutcome())
    {
        if(oContext.GetOutcome() == QERROR(RendezVous, SessionFull))
        {
            NEX_DEMO_LOG("The MatchmakeSession is already full.\n");
        }
        if(oContext.GetOutcome() == QERROR(RendezVous, SessionVoid))
        {
            NEX_DEMO_LOG("The MatchmakeSession is already unregistered.\n");
        }
        if(oContext.GetOutcome() == QERROR(RendezVous, DeniedByParticipants))
        {
            NEX_DEMO_LOG("Denid by Participants. You are in participant's blocklist.\n");
        }
        if(oContext.GetOutcome() == QERROR(RendezVous, ParticipantInBlockList))
        {
            NEX_DEMO_LOG("Participant is in your blocklist.\n");
        }
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to JoinMatchmakeSession.");
        return false;
    }
    s_JoinedGatheringID = gid;
    return true;
}

/*!
    @brief キー入力で参加するマッチメイクセッションを選択します。

    @param[in] lstMatchmakeSession 候補としてリストアップするマッチメイクセッションのリスト。
    @param[out] pGid 選択されたマッチメイクセッションのギャザリング ID。

    @retval true 選択された。pGidにギャザリングIDが格納されます。
    @retval false 選択が中断された。
 */
bool SelectMatchmakeSession(const nn::nex::qList<nn::nex::MatchmakeSession>& lstMatchmakeSession,
                            nn::nex::GatheringID* pGid)
{
    int index = 0;
    int gatheringsSize = lstMatchmakeSession.size();

    enum {None = 0, Redraw = 1, Quit = 2} state = None;

    while(state != Quit)
    {
        PrintMatchmakeSessions(lstMatchmakeSession, index);
        NEX_DEMO_LOG("[UP/DOWN]:Select MatchmakeSession [A]:Join MatchmakeSession [B]:Cancel\n");
        nex_demo_util::PadStatus oPadStatus;

        // 押しっぱなしでトリガーと認識されないように一回Update()しておく。
        nex_demo_util::DemoPad::Update(&oPadStatus);

        //入力待ち
        do
        {
            state = None;
            nex_demo_util::DemoPad::Update(&oPadStatus);
            switch(oPadStatus.trigger)
            {
            case DEMO_PAD_BUTTON_DOWN:
                index = (index + 1) % gatheringsSize;
                state = Redraw;
                break;
            case DEMO_PAD_BUTTON_UP:
                index = (index + gatheringsSize - 1) % gatheringsSize;
                state = Redraw;
                break;
            case DEMO_PAD_BUTTON_A:
            {
                nn::nex::qList<nn::nex::MatchmakeSession>::const_iterator it(lstMatchmakeSession.begin());
                std::advance(it, index);
                *pGid = it->GetID();
                return true;
            }
            case DEMO_PAD_BUTTON_B:
                state = Quit;
                break;
            }
            nn::nex::Scheduler::GetInstance()->DispatchAll();
            nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
        }
        while(state == None);
    }
    return false;
}

/*!
    @brief マッチメイクセッションの検索と参加を行う。

    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。

    @retval true 検索と参加が完了した。
    @retval false 検索もしくは参加が失敗した、もしくはマッチメイクセッションが一つも見つからなかった、
                  もしくはマッチメイクセッションの選択が中断された。
 */
bool BrowseAndJoinMatchmakeSession(nn::nex::MatchmakeExtensionClient *pMatchMakingClient)
{
    while(true)
    {
        nn::nex::qList<nn::nex::MatchmakeSession> lstMatchmakeSession;
        nn::nex::GatheringID gid(INVALID_GATHERINGID);
        if(!CallBrowseMatchmakeSession(pMatchMakingClient, &lstMatchmakeSession))
        {
            return false;
        }


        NEX_DEMO_LOG(NEX_SIZE_T_FORMAT " sessions found.\n", lstMatchmakeSession.size());

        if(lstMatchmakeSession.empty())
        {
            break;
        }
        if(!SelectMatchmakeSession(lstMatchmakeSession, &gid))
        {
            break;
        }
        if(gid != INVALID_GATHERINGID)
        {
            return JoinMatchmakeSession(pMatchMakingClient, gid);
        }
    }
    return false;
}

/*!
    @brief マッチメイクセッションの作成を同期的に行う。

    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。

    @retval true 作成が成功した。
    @retval false 作成が失敗した。
 */
bool CallCreateMatchmakeSession(nn::nex::MatchmakeExtensionClient *pMatchMakingClient)
{
    NEX_DEMO_LOG("In CallCreateMatchmakeSession\n");
    nn::nex::MatchmakeSession matchmakeSession;

    // 説明文字列
    matchmakeSession.SetDescription(NEX_T("description"));

    // 最小参加人数
    matchmakeSession.SetMinParticipants(1);

    // 最大参加人数
    matchmakeSession.SetMaxParticipants(MATCHMAKE_MAX_PARTICIPANTS);

    // アプリケーション固有のゲームモード
    matchmakeSession.SetGameMode(MATCHMAKE_GAME_MODE);

    // アプリケーションで自由に定義できる検索属性値
    matchmakeSession.SetAttribute(0, 3333);
    matchmakeSession.SetAttribute(1, 4444);

    // マッチメイクシステムタイプ
    matchmakeSession.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // アプリケーション定義のデータ
    nn::nex::qVector<nn::nex::qUnsignedInt8> applicationBuffer;
    applicationBuffer.push_back(1);
    applicationBuffer.push_back(2);
    applicationBuffer.push_back(3);
    matchmakeSession.SetApplicationBuffer(applicationBuffer);

    // マッチメイクセッションをを作成した瞬間から参加可能に設定
    // P2P通信と組み合わせて使用する場合はこの値をFalseに設定し、
    // P2P通信の準備が済んでからOpenParticipationを呼び出してください。
    matchmakeSession.SetOpenParticipation(true);

    // マッチメイクシステムタイプ
    matchmakeSession.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // 参加などの通知対象をマッチメイクセッションの参加者全員に行いたい場合はこのフラグをセットする
    // すでに参加しているユーザーの nn::nex::ParticipationEvents::Participate 通知を受け取りたい場合は
    // nn::nex::GatheringFlags::NotifyParticipationEventsToAllParticipantsReproducibly を利用することもできます。
    //matchmakeSession.SetFlag(nn::nex::GatheringFlags::NotifyParticipationEventsToAllParticipants);

    nn::nex::CreateMatchmakeSessionParam createMatchmakeSessionParam;
    createMatchmakeSessionParam.SetSourceMatchmakeSession(matchmakeSession);

    nn::nex::ProtocolCallContext oContext;
    pMatchMakingClient->CreateMatchmakeSession(&oContext, createMatchmakeSessionParam, &matchmakeSession);
    oContext.Wait();
    if(!oContext.GetOutcome())
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to CreateMatchmakeSession.");
        return false;
    }

    s_JoinedGatheringID = matchmakeSession.GetID();

    return true;
}

/*!
    @brief キー入力に応じてマッチメイクセッションを作成するか、検索/参加するかを選択します。

    @param[out] bCreateSessionRequired 作成するならtrue、検索/参加するならfalseが格納されます。

    @retval true 選択された。bCreateSessionRequiredに結果が格納されます。
    @retval false 選択されなかった。
 */
bool SelectMatchmakeMode(qBool *bCreateSessionRequired)
{
    // モードを選択
    NEX_DEMO_LOG("[A]:Create MatchmakeSession [X]:Browse MatchmakeSession [B]:Quit\n");
    nex_demo_util::PadStatus oPadStatus;

    // 押しっぱなしでトリガーと認識されないように一回Update()しておく。
    nex_demo_util::DemoPad::Update(&oPadStatus);

    while(true)
    {
        nex_demo_util::DemoPad::Update(&oPadStatus);
        switch(oPadStatus.trigger)
        {
        case DEMO_PAD_BUTTON_A:
            *bCreateSessionRequired = true;
            return true;
        case DEMO_PAD_BUTTON_X:
            *bCreateSessionRequired = false;
            return true;
        case DEMO_PAD_BUTTON_B:
            return false;
        }
        nn::nex::Scheduler::GetInstance()->DispatchAll();
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
    }
}

/*!
    @brief マッチメイクセッションの作成、検索/参加、退出の各操作を遷移します。
   
    キー入力に応じて作成もしくは検索/参加を行います。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void CreateAndBrowseMatchmakeSession(nn::nex::NgsFacade *pNgsFacade)
{
    nn::nex::MatchmakeExtensionClient matchMakingClient;
    matchMakingClient.Bind(pNgsFacade->GetCredentials());

    // HID初期化
    nex_demo_util::DemoPad::Initialize();

    while(true)
    {
        // モードの選択
        qBool bCreateSessionRequired;
        if(!SelectMatchmakeMode(&bCreateSessionRequired))
        {
            // QUIT が選択された。
            break;
        }

        // 選択されたモードに応じてマッチメイクセッションを作成/検索
        if(bCreateSessionRequired)
        {
            // マッチメイクセッションを作成
            if(!CallCreateMatchmakeSession(&matchMakingClient))
            {
                // 失敗したのでモードの選択からやり直す。
                continue;
            }
        }
        else
        {
            // マッチメイクセッションの検索、参加
            if(!BrowseAndJoinMatchmakeSession(&matchMakingClient))
            {
                // 失敗したのでモードの選択からやり直す。
                continue;
            }
        }

        // マッチメイクセッション参加中
        RunGameMainLoop();

        // マッチメイクセッションから退出
        // ・EndParticipationを呼んだ結果参加者が0になった場合、そのマッチメイクセッションは自動的に削除されます
        // ・自身がマッチメイクセッションのオーナーの場合は EndParticipation の代わりに UnregisterGatheringを呼ぶこともできます。
        //   その場合、マッチメイクセッションは即座に削除されますが、残った参加者には
        //   NotificationEvents::GatheringUnregistered イベントが通知されます。
        //   ゲームの種類によってはこのイベントをハンドリングしてマッチメイクを終了させることができます。
        // ・P2P通信と組み合わせて使用する場合、セッションホストの退出時にホストの同期が必要になる事があります。
        NEX_DEMO_LOG("Leave from MatchmakeSession gid: %d\n", s_JoinedGatheringID);
        nn::nex::ProtocolCallContext oContext;
        matchMakingClient.EndParticipation(&oContext, s_JoinedGatheringID);
        oContext.Wait();
        if(!oContext.GetOutcome())
        {
            if (oContext.GetOutcome() != QERROR(RendezVous, SessionVoid))
            {
                nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to EndParticipation.");
            }
        }
    }

    // HID終了
    nex_demo_util::DemoPad::Finalize();
}

/*!
    @brief NEXの初期化後にNGSへログインし、マッチメイクセッションの作成、検索/参加処理に進みます。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;

        // NGSからの通知を処理するイベントハンドラを登録
        ngsFacade.RegisterNotificationEventHandler(&s_MyNotificationEventHandler);

        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            CreateAndBrowseMatchmakeSession(&ngsFacade);

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }

        // イベントハンドラを削除
        ngsFacade.UnregisterNotificationEventHandler(&s_MyNotificationEventHandler);
    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("****************************************\n");
    NEX_DEMO_LOG(" CreateAndBrowse sample\n");
    NEX_DEMO_LOG("****************************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
