﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include "Console.h"

using namespace nexdemo;

// コンストラクタです。
Console::Console()
{
    m_ppBuffer      = NULL;
    m_pRenderBuffer = NULL;

    m_isInitialized = false;

    // 変数を初期化する
    Finalize();
}

// デストラクタです。
Console::~Console()
{
    Finalize();
}

// 初期化処理を行います。
void Console::Initialize(s16 x, s16 y, u16 windowCharsPerLine, u16 windowLines, u32 bufferCharsPerLine, u32 bufferLines, u8 lineSpace)
{
    if (windowCharsPerLine == 0 || windowLines == 0 || bufferCharsPerLine == 0 || bufferLines == 0)
    {
        return;
    }
    if (m_isInitialized)
    {
        Finalize();
    }

    m_x                      = x;
    m_y                      = y;
    m_windowCharsPerLine     = windowCharsPerLine;
    m_windowLines            = windowLines;
    m_bufferCharsPerLine     = bufferCharsPerLine;
    m_bufferLines            = bufferLines;
    m_usedBufferCharsPerLine = 0;
    m_usedBufferLines        = 0;
    m_lineSpace              = lineSpace;
    m_cursorX                = 0;
    m_cursorY                = 0;
    m_scrollX                = 0;
    m_scrollY                = 0;
    m_isOver                 = false;
    m_isFull                 = false;

    m_ppBuffer = new char*[m_bufferLines];

    for (u32 i = 0; i < m_bufferLines; i++)
    {
        m_ppBuffer[i] = new char[m_bufferCharsPerLine + 1];
        m_ppBuffer[i][m_bufferCharsPerLine] = '\0';
    }

    m_pRenderBuffer = new char[m_windowCharsPerLine + 1];
    m_pRenderBuffer[m_windowCharsPerLine] = '\0';

    // コンソールの内容をクリアする
    Clear();

    m_isInitialized = true;
}

// 終了処理を行います。
void Console::Finalize()
{
    m_isInitialized = false;

    if (m_ppBuffer)
    {
        for (u32 i = 0; i < m_bufferLines; i++)
        {
            delete[] m_ppBuffer[i];
        }

        delete[] m_ppBuffer;
    }

    if (m_pRenderBuffer)
    {
        delete[] m_pRenderBuffer;
    }

    m_ppBuffer           = NULL;
    m_pRenderBuffer      = NULL;
    m_x                  = 0;
    m_y                  = 0;
    m_windowCharsPerLine = 0;
    m_windowLines        = 0;
    m_bufferCharsPerLine = 0;
    m_bufferLines        = 0;
    m_lineSpace          = 0;
    m_cursorX            = 0;
    m_cursorY            = 0;
    m_scrollX            = 0;
    m_scrollY            = 0;
    m_isOver             = false;
    m_isFull             = false;
}

// コンソールの内容をクリアします。
void Console::Clear()
{
    if (!m_isInitialized)
    {
        return;
    }

    if (m_ppBuffer)
    {
        for (u32 i = 0; i < m_bufferLines; i++)
        {
            if (m_ppBuffer[i])
            {
                memset(m_ppBuffer[i], 0, m_bufferCharsPerLine);
            }
        }
    }
}

// 文字列を出力します。
void Console::Printf(const char* pFormat, ...)
{
    if (!m_isInitialized || !pFormat)
    {
        return;
    }

    // バッファ
    char buffer[NN_DBG_PRINTF_BUFFER_LENGTH + 1] = {0};
    // 引数リスト
    va_list pArgs;

    va_start(pArgs, pFormat);
    {
        vsnprintf(buffer, NN_DBG_PRINTF_BUFFER_LENGTH, pFormat, pArgs);
    }
    va_end(pArgs);

    PutString(buffer);
}

// 文字列を出力します。
void Console::PutString(const char* pString)
{
    if (!m_isInitialized || !pString)
    {
        return;
    }

    u32 i = 0;

    while (pString[i] != '\0')
    {
        switch (pString[i])
        {
        case '\n':
        {
            PutChar('\0');

            if (m_cursorX > m_usedBufferCharsPerLine)
            {
                m_usedBufferCharsPerLine = m_cursorX;
            }

            m_cursorX = 0;
            m_cursorY++;

            if (m_cursorY >= m_windowLines)
            {
                m_isOver = true;
            }
            if (m_cursorY == m_bufferLines)
            {
                m_isFull  = true;
                m_cursorY = 0;
            }

            if (!m_isFull)
            {
                m_usedBufferLines++;

                if (m_isOver)
                {
                    Scroll(0, 1);
                }
            }

            PutChar('\0');
        }
        break;
        case '\t':
        {
            // 空白の数(タブ幅4で計算)
            u32 space = 4 - (m_cursorX % 4);

            for (u32 t = 0; t < space; t++)
            {
                PutChar(' ');
            }
        }
        break;
        default:
        {
            // 表示可能な文字のみ表示する
            if (isprint(pString[i]))
            {
                PutChar(pString[i]);
            }
        }
        }

        i++;
    }
}

// コンソールの内容を描画します。
void Console::Render(demo::RenderSystemDrawing* pRenderSystem)
{
    if (!m_isInitialized || !pRenderSystem)
    {
        return;
    }

    // 余白
    u8 margin = GetMargin();
    // 1行の高さ
    u16 lineHeight = GetLineHeight();

    for (s32 i = 0; i < m_windowLines; i++)
    {
        s32 current = i;

        // ウィンドウに表示できる行数以上のデータが溜まった場合
        if (m_isOver)
        {
            // バッファを使いきった場合
            if (m_isFull)
            {
                // スクロールを有効にし、カーソル位置も考慮する
                current = (i + m_scrollY + m_cursorY + 1 + m_bufferLines) % m_bufferLines;
            }
            else
            {
                // スクロールを有効にする
                current = i + m_scrollY;
            }
        }

        // ウィンドウの文字数より画面バッファ文字数のほうが大きかったら、画面に表示する分だけ切り取る
        if (m_windowCharsPerLine < m_bufferCharsPerLine)
        {
            // 画面に表示する文字列をコピーする
            strncpy(m_pRenderBuffer, &m_ppBuffer[current][m_scrollX], m_windowCharsPerLine);
            // テキストを描画する
            pRenderSystem->DrawText(m_x, m_y + i * lineHeight + margin, m_pRenderBuffer);
        }
        else
        {
            // テキストを描画する
            pRenderSystem->DrawText(m_x, m_y + i * lineHeight + margin, &m_ppBuffer[current][m_scrollX]);
        }
    }
}

// コンソールの内容をスクロールします。
void Console::Scroll(s16 scrollX, s16 scrollY)
{
    if (!m_isInitialized)
    {
        return;
    }

    s32 newScrollX = m_scrollX + scrollX;
    s32 newScrollY = m_scrollY + scrollY;

    // 右端
    s32 rightEnd = m_usedBufferCharsPerLine - m_windowCharsPerLine;
    // 下端
    s32 bottomEnd = m_usedBufferLines - m_windowLines;

    if (newScrollX < 0)
    {
        newScrollX = 0;
    }
    if (newScrollY < 0)
    {
        newScrollY = 0;
    }
    if (newScrollX > rightEnd && rightEnd > 0)
    {
        newScrollX = rightEnd;
    }
    if (newScrollY > bottomEnd && bottomEnd > 0)
    {
        newScrollY = bottomEnd;
    }

    m_scrollX = newScrollX;
    m_scrollY = newScrollY;
}

// X座標のスクロール位置を設定します。
void Console::SetScrollX(s32 scrollX)
{
    if (!m_isInitialized)
    {
        return;
    }

    // 右端
    s32 rightEnd = m_usedBufferCharsPerLine - m_windowCharsPerLine;

    if (scrollX < 0)
    {
        scrollX = 0;
    }
    if (scrollX > rightEnd && rightEnd > 0)
    {
        scrollX = rightEnd;
    }

    m_scrollX = scrollX;
}

// Y座標のスクロール位置を設定します。
void Console::SetScrollY(s32 scrollY)
{
    if (!m_isInitialized)
    {
        return;
    }

    // 下端
    s32 bottomEnd = m_usedBufferLines - m_windowLines;

    if (scrollY < 0)
    {
        scrollY = 0;
    }
    if (scrollY > bottomEnd && bottomEnd > 0)
    {
        scrollY = bottomEnd;
    }

    m_scrollY = scrollY;
}

// 文字をバッファに追加する。
void Console::PutChar(char c)
{
    m_ppBuffer[m_cursorY][m_cursorX] = c;

    if (c != '\0')
    {
        bool isNewLine = false;

        m_cursorX++;

        if (m_cursorX == m_bufferCharsPerLine)
        {
            m_cursorX                = 0;
            m_usedBufferCharsPerLine = m_bufferCharsPerLine;
            m_cursorY++;

            isNewLine = true;
        }

        if (m_cursorY >= m_windowLines)
        {
            m_isOver = true;
        }
        if (m_cursorY == m_bufferLines)
        {
            m_isFull  = true;
            m_cursorY = 0;
        }

        if (isNewLine && !m_isFull)
        {
            m_usedBufferLines++;

            if (m_isOver)
            {
                Scroll(0, 1);
            }
        }
    }
}
