﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * JoinFromFriendList
 *
 * Description:
 *
 *  フレンドリストからフレンドが参加しているマッチメイクセッションへ合流するサンプルです。
 *
 *  通常起動時はマッチメイクセッションを登録して、フレンドにマッチメイクセッションの参加情報を
 *  プレゼンスし、規定人数まで集まるまで待ち受けます。
 *
 *  フレンドリストから起動したアプリケーションの場合は、合流先のユーザーの
 *  プレゼンス情報を取得し、マッチメイクセッションに参加したのちにP2Pセッションに参加します。
 *  参加後に規定人数に達していない場合はフレンドにマッチメイクセッションの参加情報を
 *  プレゼンスし、規定人数まで集まるまで待ち受けます。
 *
 *  オーナー/参加者のどちらであっても規定人数に達した段階で、そのマッチメイクセッションには
 *  参加できないことをプレゼンスします。
 *  
 *  実行例として、一台の本体（以下、本体 A）から JoinFromFriendList を実行し、
 *  もう一台の本体（以下、本体 B）から合流する方法を以下に示します。
 *  
 *  1. 以下の設定がなされた本体 A,B を用意する。
 *      ・インターネット接続可能である
 *      ・フレンド関係構築済みである
 *      ・HOME メニューモードである
 *  2. 本体 A,B 両方に JoinFromFriendList をロードし、サンプルのアイコンが HOME メニューに表示されていることを確認する。
 *  3. 本体 A の HOME メニューから JoinFromFriendList を実行する。
 *  4. 本体 B の HOME メニューからフレンドリストを開く。
 *  5. 本体 B のフレンドリストから本体 A のユーザーのフレンドカードを選択し、「プレイ中のソフトに参加する」ボタンをタッチ。
 *  6. 画面上のログで合流に成功したことを確認する。
 *
 * Note:
 *  nn::frinends::UpdateGameMode() でプレゼンスを更新しても、前回の更新から10秒程度経過するまでは
 *  フレンドに通知されません。またネットワークの状態によってはプレゼンス変更通知が遅れることもあるため、
 *  フレンドプレゼンスライブラリで合流可能と判定されても必ず合流可能であるとは限らないことにご注意ください。
 *
 *  このサンプルをビルドするには CTR-SDK 3.1 以降が必要です。
 */

#include <nn.h>
#include <nn/applet.h>
#include <nn/friends.h>
#include <nex.h>

#include <NexDemoUtil/NexUtil.h>
#include <NexDemoUtil/NexUtilNetZ.h>

#include "Graphics.h"
#include "Console.h"

namespace
{

// マッチメイクで使用するゲームモード
const qUnsignedInt32 MATCHMAKE_GAME_MODE = 0;

// 合流可能なゲームモードを制限するマスクです。
const qUnsignedInt64 MATCHMAKE_GAME_MODE_MASK = 0x0000000000000001;

// 接続人数
const qUnsignedInt16 MATCHMAKE_MAX_PARTICIPANTS = 2;

// セッションを作成した状態で一人ぼっちのまま待機する時間(ミリ秒)
const nn::nex::TimeInterval ORPHAN_WAIT_TIME_MS = 60 * 1000;

// スリープする時間(ミリ秒)
const qUnsignedInt32 SLEEP_PERIOD_MS = 100;

// マッチメイク後、送受信を行う時間(ミリ秒)
const nn::nex::TimeInterval MATCHMAKING_TIME_MS = 60* 1000;

// コンソール
nexdemo::Console s_console;

// 排他制御用クリティカルセクション
nn::os::CriticalSection s_criticalSection;
}   // namespace {

/*!
    @brief ログを描画します。
 */
void RenderLog()
{
    // レンダラ
    demo::RenderSystemDrawing* pRenderSystem = nexdemo::Graphics::GetRenderSystem();

    // 画面をクリアする
    pRenderSystem->SetRenderTarget(NN_GX_DISPLAY0);
    pRenderSystem->Clear();

    // 背景を描画する
    pRenderSystem->SetColor(NEXDEMO_RGB(0, 0, 0));
    pRenderSystem->FillRectangle(0, 0, NN_GX_DISPLAY0_HEIGHT, NN_GX_DISPLAY0_WIDTH);

    // 描画する
    pRenderSystem->SetColor(NEXDEMO_RGB(255, 255, 255));
    s_console.Render(pRenderSystem);

    pRenderSystem->SwapBuffers();
    pRenderSystem->WaitVsync(NN_GX_DISPLAY_BOTH);
}

namespace nn
{
namespace dbg
{
namespace detail
{
/*!
    @brief デバッガの出力をリダイレクトします。
 */
void PutString(const char* text, s32 length)
{
    // 描画中にPutStringされないように排他制御しておく
    nn::os::CriticalSection::ScopedLock s_scopedLock(s_criticalSection);

    (void)length;

    if (s_console.IsInitialized())
    {
        // コンソールに出力
        s_console.PutString(text);
        // 再描画
        RenderLog();
    }
}
}
}
}

namespace
{

/*!
    @brief マッチメイクセッションとP2Pセッションを作成します。

    @param[out] pCreatedMatchmakeSession 作成されたマッチメイクセッションを格納するポインタ。
    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。

    @return 処理が成功したかどうか。
 */
qBool CreateMatchmakeSessionAndP2pSession(nn::nex::MatchmakeSession* pCreatedMatchmakeSession,
                                          nn::nex::MatchmakeExtensionClient *pMatchMakingClient)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::MatchmakeSession sourceMatchmakeSession;

    /////////////////////////////////////////////////////////////////
    // マッチメイクセッションの条件設定
    {
        // ゲームモード
        sourceMatchmakeSession.SetGameMode(MATCHMAKE_GAME_MODE);

        // 最小・最大参加人数
        sourceMatchmakeSession.SetMinParticipants(1);
        sourceMatchmakeSession.SetMaxParticipants(MATCHMAKE_MAX_PARTICIPANTS);

        // マッチメイクシステムタイプ
        sourceMatchmakeSession.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);
    }

    /////////////////////////////////////////////////////////////////
    // マッチメイクセッションの登録
    {
        nn::nex::CreateMatchmakeSessionParam createMatchmakeSessionParam;
        createMatchmakeSessionParam.SetSourceMatchmakeSession(sourceMatchmakeSession);

        pMatchMakingClient->CreateMatchmakeSession(&oContext, createMatchmakeSessionParam, pCreatedMatchmakeSession);
        oContext.Wait();
        if(oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to create matchmakesession\n");
            return false;
        }

        NEX_DEMO_LOG("create matchmakesession ID:%u\n", pCreatedMatchmakeSession->GetID());
        nn::nex::Network::GetInstance()->SetP2PDataPacketSessionSignatureKey(pCreatedMatchmakeSession->GetSessionKey());
    }

    /////////////////////////////////////////////////////////////////
    // P2Pセッションをオープンする
    if (!nn::nex::Session::CreateSession(NULL))
    {
        NEX_DEMO_LOG("Failed to create a Net-Z session\n");
        return false;
    }
    NEX_DEMO_LOG("Net-Z session created successfly\n");

    /////////////////////////////////////////////////////////////////
    // マッチメイクセッションを募集状態にする
    pMatchMakingClient->OpenParticipation(&oContext, pCreatedMatchmakeSession->GetID());
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to open participation.\n");
        return false;
    }

    return true;
}

/*!
    @brief マッチメイクセッションとP2Pセッションに参加します。

    @param[out] pJoinedMatchmakeSession 参加したマッチメイクセッションを格納するポインタ。
    @param[in] pMatchMakingClient Credentials をバインド済みのオブジェクトを指定してください。

    @return 処理が成功したかどうか。
 */
qBool ParticipateMatchmakeSessionAndP2pSession(nn::nex::MatchmakeSession* pJoinedMatchmakeSession,
                                               nn::nex::MatchmakeExtensionClient *pMatchMakingClient)
{
    nn::nex::ProtocolCallContext oContext;

    /////////////////////////////////////////////////////////////////
    // フレンドプレゼンスからマッチメイクセッション情報を取得する(存在するはず)
    nn::friends::FriendKey friendKey;
    nn::friends::IsFromFriendList(&friendKey);

    // プレゼンス情報を取得
    nn::friends::FriendPresence presence;
    nn::friends::GetFriendPresence(&presence, &friendKey);

    // 念のためにもう一度確認
    if(!presence.IsJoinable(MATCHMAKE_GAME_MODE_MASK))
    {
        NEX_DEMO_LOG("may be not joinable session\n");
        return false;
    }

    /////////////////////////////////////////////////////////////////
    // NGSからマッチメイクセッションを取得する(存在するはず)
    pMatchMakingClient->FindMatchmakeSessionByGatheringId(&oContext, presence.gameMode.joinGroupId, pJoinedMatchmakeSession);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to obtain a matchmakesession\n");
        return false;
    }

    /////////////////////////////////////////////////////////////////
    // マッチメイクセッションへの参加
    nn::nex::qVector<nn::nex::qByte> sessionKey;
    pMatchMakingClient->JoinMatchmakeSession(&oContext, pJoinedMatchmakeSession->GetID(), &sessionKey);
    oContext.Wait();
    if(oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to join matchmakesession\n");
        return false;
    }

    nn::nex::Network::GetInstance()->SetP2PDataPacketSessionSignatureKey(sessionKey);

    /////////////////////////////////////////////////////////////////
    // P2PセッションホストのURLを取得
    nn::nex::qList<nn::nex::StationURL> lstURLs;
    pMatchMakingClient->GetSessionURLs(&oContext, pJoinedMatchmakeSession->GetID(), &lstURLs);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        // Join したマッチメイクセッションが削除されていることもありえます。
        if(oContext.GetOutcome() == QERROR(RendezVous, SessionVoid))
        {
            NEX_DEMO_LOG("matchmake session is already unregistered.\n");
        }
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to obtain a list of URLs to connect to the game host\n");
        return false;
    }

    /////////////////////////////////////////////////////////////////
    // P2Pセッションに参加を試みる
    if (!nn::nex::Session::JoinSession(&oContext, lstURLs))
    {
        NEX_DEMO_LOG("Failed to JoinSession\n");
        return false;
    }

    oContext.Wait();
    if(oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to  the Net-Z session\n");
        return false;
    }

    return true;
}

/*!
    @brief P2P通信の接続情報を表示します。
 */
void OutputConnectionInfo()
{
    nn::nex::ScopedCS oCS(nn::nex::Scheduler::GetInstance()->SystemLock());
    nn::nex::Station::SelectionIterator itStations;
    itStations.GotoStart();
    while (!itStations.EndReached())
    {
        NEX_DEMO_LOG("Station %8x: URL = " STRING_FORMAT ", RTT = %d\n",
                     itStations->GetHandle().GetValue(),
                     itStations->GetStationURL(0),
                     itStations->GetRTTAverage());

        ++itStations;
    }
    NEX_DEMO_LOG("\n");
}

/*!
    @brief 人数が揃うまで待ちます。
   
   @param[in] NetZ 初期化済みのオブジェクト。
 */
qBool WaitForParticipantsLoop(const nn::nex::NetZ* pNetZ)
{
    qBool bReadyToRun = false;

    NEX_DEMO_LOG("Starting NetZ session...\n");

    // 規定人数に到達するまで待つ
    nn::nex::Time tiReadyStart = nn::nex::SystemClock::GetTime();
    nn::nex::Time tiWaitStart = tiReadyStart;

    int oldNumStations = 0;
    while (1)
    {
        nn::nex::Time currentTime = nn::nex::SystemClock::GetTime();

        int numStations = 0;
        {
            nn::nex::ScopedCS oCS(nn::nex::Scheduler::GetInstance()->SystemLock());
            nn::nex::Station::SelectionIterator itStations;
            numStations = itStations.Count();
        }

        if (oldNumStations != numStations)
        {
            // 接続人数に変化があった場合、待ち時間リセット
            tiWaitStart = currentTime;
        }

        // 以前の人数を記憶
        oldNumStations = numStations;

        // 一人ぼっちの状態が一定時間続いた場合終了する
        if (numStations == 1 && currentTime - tiWaitStart >= ORPHAN_WAIT_TIME_MS)
        {
            NEX_DEMO_LOG("*** time passed as. abort.\n");
            break;
        }

        if (currentTime - tiReadyStart >= 3000)
        {
            // 現在接続人数表示
            NEX_DEMO_LOG("waiting ... NumStations: %d/%d\n", numStations, MATCHMAKE_MAX_PARTICIPANTS);

            // 接続情報も表示
            OutputConnectionInfo();

            tiReadyStart = currentTime;
        }

        // メッセージのディスパッチ
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // NetZのフェイタルエラーチェック
        nn::nex::qResult netZError = pNetZ->GetFatalError();
        if ( QFAILED(netZError.GetReturnCode()) )
        {
            // 復帰不能なエラーが発生
            nex_demo_util::ReportQResultReturnCode(netZError, "NetZ fatal error happend!\n");
            break;
        }

        // 接続人数チェック
        if (numStations == MATCHMAKE_MAX_PARTICIPANTS)
        {
            // 規定人数に達し、メインループに移行する準備が整った
            bReadyToRun = true;
            break;
        }

        nn::nex::Platform::Sleep(SLEEP_PERIOD_MS);
    }

    return bReadyToRun;
}

/*!
    @brief マッチメイク後のメインループを行います。

    @param[in] NetZ 初期化済みのオブジェクト。
 */
void RunGameLoop(const nn::nex::NetZ* pNetZ)
{
    NEX_DEMO_LOG("Start NetZ session.\n");

    nn::nex::Time tiSessionStart = nn::nex::SystemClock::GetTime();
    nn::nex::Time tiLogStart     = tiSessionStart;

    // 送受信を行う
    while (1)
    {
        nn::nex::Time currentTime = nn::nex::SystemClock::GetTime();

        if (currentTime - tiSessionStart >= MATCHMAKING_TIME_MS)
        {
            // 指定時間送受信を行なったので終了する
            break;
        }

        // メッセージのディスパッチ
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        // NetZのフェイタルエラーチェック
        nn::nex::qResult netZError = pNetZ->GetFatalError();
        if ( QFAILED(netZError.GetReturnCode()) )
        {
            // 復帰不能なエラーが発生
            nex_demo_util::ReportQResultReturnCode(netZError, "NetZ fatal error happend!\n");
            break;
        }

        if (currentTime - tiLogStart >= 3 * 1000)
        {
            // 接続されているステーションを表示
            tiLogStart = nn::nex::SystemClock::GetTime();

            NEX_DEMO_LOG("Stations currently in Session\n");
            NEX_DEMO_LOG("-----------------------------\n");
            OutputConnectionInfo();
        }

        nn::nex::Platform::Sleep(SLEEP_PERIOD_MS);
    }

    NEX_DEMO_LOG("Stop NetZ session.\n");
}

/*!
    @brief マッチメイクセッションの情報に基づいてフレンドプレゼンスを更新します。
   
   @param[in] matchmakeSession 基にするマッチメイクセッション情報。
   @param[in] isJoinable 合流可能状態かどうか。
   @param[in] 自身がマッチメイクセッションのオーナーかどうか。
 */
void UpdateGameMode(const nn::nex::MatchmakeSession& matchmakeSession, bool isJoinable, qBool isOwner)
{
    nn::friends::GameMode gameMode;
    char16* gameDescription = L"Join From FriendList";

    gameMode.joinAvailabilityFlag = isJoinable ? nn::friends::JOIN_AVAILABILITY_JOINABLE : nn::friends::JOIN_AVAILABILITY_NOT_JOINABLE;
    gameMode.joinGameMode = matchmakeSession.GetGameMode();
    gameMode.joinGroupId = matchmakeSession.GetID();
    // 自分がマッチメイクセッションを作成した場合には GetOwnerPID() で PrincipalId を取得できないので、自分の PrincipalId をセット
    if(isOwner)
    {
        gameMode.ownerPrincipalId = nn::friends::GetMyPrincipalId();
    }
    else
    {
        gameMode.ownerPrincipalId = matchmakeSession.GetOwnerPID();
    }
    switch (matchmakeSession.GetMatchmakeSystemType())
    {
    case nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY:
        gameMode.matchmakeSystemType = nn::friends::MATCHMAKE_SYSTEM_TYPE_ANYBODY;
        break;
    case nn::nex::MATCHMAKE_SYSTEM_TYPE_FRIEND:
        gameMode.matchmakeSystemType = nn::friends::MATCHMAKE_SYSTEM_TYPE_FRIEND;
        break;
    default:
        // コミュニティマッチはANYBODY扱い
        gameMode.matchmakeSystemType = nn::friends::MATCHMAKE_SYSTEM_TYPE_ANYBODY;
        break;
    }

    nn::friends::UpdateGameMode(gameMode, gameDescription);
}

/*!
    @brief 自身のアプリケーションがフレンドリストから起動されたものかどうかを調べます。

    @retval true フレンドリストから起動された。
    @retval false フレンドリストから起動されていない。
 */
bool IsInvokedFromFriendList()
{
    // フレンドリストからの起動であるかどうかを判定
    // nn::friends::GetFriendPresence() は事前にフレンドサーバーへのログインが必要となるため、 nex_demo_util::GameServerLogin() より後に実行する
    nn::friends::FriendKey friendKey;
    if (!nn::friends::IsFromFriendList(&friendKey))
    {
        NEX_DEMO_LOG("this program run from Home (or Test) Menu\n", friendKey.principalId);
        return false;
    }

    NEX_DEMO_LOG("this program run from friendlist (pid:%u)\n", friendKey.principalId);

    // プレゼンス情報を取得して合流可能であるかを確認
    nn::friends::FriendPresence presence;
    nn::friends::GetFriendPresence(&presence, &friendKey);
    NEX_DEMO_LOG("  GameId                : %08x\n", presence.gameMode.joinGameId);
    NEX_DEMO_LOG("  GameMode              : %u\n", presence.gameMode.joinGameMode);
    NEX_DEMO_LOG("  GroupId(GatheringId)  : %u\n", presence.gameMode.joinGroupId);
    NEX_DEMO_LOG("  MatchmakeSystemType   : %u\n", presence.gameMode.matchmakeSystemType);
    NEX_DEMO_LOG("  Owner                 : %u\n", presence.gameMode.ownerPrincipalId);
    NEX_DEMO_LOG("  JoinAvailabilityFlag  : %08x\n", presence.gameMode.joinAvailabilityFlag);

    if(presence.IsJoinable(MATCHMAKE_GAME_MODE_MASK))
    {
        return true;
    }

    // フレンドリストからの起動であってもタイミングによっては合流不可になる
    NEX_DEMO_LOG("this program run from friendlist but is not joinable.\n");

    return false;
}

/*!
    @brief フレンドリストからの合流かどうかを判断し、マッチメイク処理を行います。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void DoMatchmake(nn::nex::NgsFacade* pNgsFacade)
{
    nn::nex::NATTraversalClient natTraversalClient;
    natTraversalClient.Bind(pNgsFacade->GetCredentials());
    nn::nex::MatchmakeExtensionClient matchMakingClient;
    matchMakingClient.Bind(pNgsFacade->GetCredentials());

    qBool isOwner = false;
    // NAT Traversal を有効にする
    nn::nex::ConnectivityManager::GetInstance()->StartNATSession();

    // NetZ オブジェクトの作成
    nn::nex::NetZ *pNetZ = new nn::nex::NetZ;

    if (!pNetZ->IsValid())
    {
        NEX_DEMO_LOG("NetZ is invalid!!\n");
        delete pNetZ;
        nn::nex::ConnectivityManager::GetInstance()->StopNATSession();
        return;
    }

    // Stationに表示される名前のセットアップ
    nex_demo_util::SetupLocalPlayerName();

    // マッチメイク開始
    bool bReadyToRun = false;
    nn::nex::MatchmakeSession matchmakeSession;

    // フレンドリストからの起動であればマッチメイクセッションに参加を試みる、そうでなければマッチメイクセッションを作成する。
    if(IsInvokedFromFriendList())
    {
        isOwner = false;
        bReadyToRun = ParticipateMatchmakeSessionAndP2pSession(&matchmakeSession, &matchMakingClient);
    }
    else
    {
        isOwner = true;
        bReadyToRun = CreateMatchmakeSessionAndP2pSession(&matchmakeSession, &matchMakingClient);
    }

    if(!bReadyToRun)
    {
        delete pNetZ;
        nn::nex::ConnectivityManager::GetInstance()->StopNATSession();
        return;
    }

    // プレゼンスを行う
    UpdateGameMode(matchmakeSession, true, isOwner);

    // 待ち受け
    if (WaitForParticipantsLoop(pNetZ))
    {
        // オーナーである場合は締め切る
        if(isOwner)
        {
            nn::nex::ProtocolCallContext oContext;
            matchMakingClient.CloseParticipation(&oContext, matchmakeSession.GetID());
            oContext.Wait();
        }

        // 合流不可状態にしてプレゼンスする
        UpdateGameMode(matchmakeSession, false, isOwner);
        RunGameLoop(pNetZ);
    }

    // NetZ オブジェクトの削除
    {
        nn::nex::CallContext oContext;
        pNetZ->Terminate(&oContext);
        oContext.Wait();
        delete pNetZ;
    }

    // NAT Traversal を終了
    nn::nex::ConnectivityManager::GetInstance()->StopNATSession();

    // マッチメイクセッションから抜ける
    {
        nn::nex::ProtocolCallContext oContext;
        matchMakingClient.EndParticipation(&oContext, matchmakeSession.GetID());
        oContext.Wait();
    }

    // GameModeなどを空にしてプレゼンス
    matchmakeSession.Reset();
    UpdateGameMode(matchmakeSession, false, false);
}

/*!
    @brief NEXの初期化後にNGSへログインし、合流処理を行います。
 */
qInt NEXSampleMain()
{
    nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMicroSeconds(10));

    // 起動時に終了ボタンが押されていた場合の処理
    if (nn::applet::IsExpectedToCloseApplication())
    {
        nn::applet::PrepareToCloseApplication();
        nn::applet::CloseApplication();
    }

    // HID初期化
    nn::hid::Initialize();
    nn::hid::PadReader oPadReader;

    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // ステーション不一致検出回数 デフォルトは5秒間隔で検出チェックなので、
    // NAT トラバーサルのタイムアウトより10秒長くなるように設定
    nn::nex::Session::SetDetectStationInconsitencyThresholdCount(MATCHMAKE_MAX_PARTICIPANTS + 2);

    // ネットワークトポロジ設定
    nn::nex::NetZ::SetNetworkTopology(nn::nex::NetworkTopology::Peer2Peer);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            DoMatchmake(&ngsFacade);

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }

    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    s_console.Finalize();
    nexdemo::Graphics::Finalize();

    nn::os::SetDeviceMemorySize(0);
    nn::os::SetHeapSize(0);

    nn::applet::PrepareToCloseApplication();
    nn::applet::CloseApplication();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    // ログコンソールの初期化
    {
        nexdemo::Graphics::Initialize();
        s_criticalSection.Initialize();

        // 行間
        u8 lineSpace = 2;
        // 1行あたりの文字数
        u16 charsPerLine = NN_GX_DISPLAY0_HEIGHT / nexdemo::Console::FONT_SIZE;
        // 行数
        u16 lines = NN_GX_DISPLAY0_WIDTH / (nexdemo::Console::FONT_SIZE + lineSpace);

        s_console.Initialize(0, 0, charsPerLine, lines, lineSpace);
    }

    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("   JoinFromFriendList sample\n");
    NEX_DEMO_LOG("*******************************\n\n");

    qInt returnCode = NEXSampleMain();

    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
