﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <nn/gx.h>
#include <nn/math.h>

#include "demo/Utility/demo_Utility.h"
#include "demo/RenderData/demo_TrianglesRenderData.h"

namespace demo
{

/*!
@internal

@brief    ProgramObjectクラスが保持できる最大シェーダ数
*/
const u32 PROGRAM_OBJECT_MAX_SHADERS = 1;

/*!
@internal

@brief シェーダプログラムの基底クラスです。
*/

class ProgramObject : private nn::util::NonCopyable<ProgramObject>
{
public:
    /*!
    @internal

    @brief    コンストラクタです。
    */
    ProgramObject(void);

    /*!
    @internal

    @brief    デストラクタです。
    */
    virtual ~ProgramObject(void);

public:
    /*!
    @internal

    @brief    シェーダプログラムオブジェクトの初期処理を行います。
    */
    virtual bool Initialize(const GLuint shaderId) = 0;

    /*!
    @internal

    @brief    シェーダプログラムオブジェクトの終了処理を行います。
    */
    virtual bool Finalize(void) = 0;

public:
    /*!
    @internal

    @brief    シェーダプログラムオブジェクト使用前のレンダリングステートの設定を行います。
    */
    virtual bool Begin(void);

    /*!
    @internal

    @brief    シェーダプログラムオブジェクトを使用します。
    */
    virtual bool Use(void) = 0;

    /*!
    @internal

    @brief    シェーダプログラムオブジェクト使用後のレンダリングステートの設定を行います。
    */
    virtual bool End(void);

public:
    /*!
    @internal

    @brief    シェーダプログラムオブジェクトに設定された頂点属性を取得します。

    @return   頂点属性
    */
    virtual u32 GetVertexAttributes(void);

    /*!
    @internal

    @brief    3次元の頂点の位置座標を、射影行列とモデルビュー行列によってクリップ座標に変換するかどうかを設定します。

    @param[in]    use3d    クリップ座標系に変換するかどうかについて(trueかfalse)
    */
    virtual void SetUse3d(const bool use3d);

    /*!
    @internal

    @brief    シェーダプログラムオブジェクトの射影行列を設定します。

    @param[in]    projectionMatrix    射影行列
    */
    virtual void SetProjectionMatrix(const nn::math::MTX44& projectionMatrix);

    /*!
    @internal

    @brief    シェーダプログラムオブジェクトのモデルビュー行列を設定します。

    @param[in]    modelViewMatrix    モデルビュー行列
    */
    virtual void SetModelViewMatrix(const nn::math::MTX44& modelViewMatrix);

    /*!
    @internal

    @brief    テクスチャオブジェクトのハンドルを設定します。

    @param[in]    textureId    テクスチャオブジェクトのハンドル
    */
    virtual void SetTextureId(const GLuint textureId);

    /*!
    @internal

    @brief    シェーダプログラムオブジェクトの内部状態を更新します。
    */
    virtual void Update(void);

protected:
    virtual void InitializeVertexAttributes(void);
    virtual void InitializeUniforms(void);
    virtual void UpdateModelViewProjectionMatrix(void);

protected:
    bool m_Initialized;
    bool m_Padding0[3];

    u32 m_VertexAttributes;
    bool m_Use3d;
    u8 m_Padding[3];

    GLuint m_ProgramId;
    GLuint m_ShaderIds[PROGRAM_OBJECT_MAX_SHADERS];
    GLint m_UniformLocation[MAX_UNIFORM_LOCATIONS_NUM];

    nn::math::MTX44 m_ProjectionMatrix;
    nn::math::MTX44 m_ModelViewMatrix;
};

}

