﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * MatchmakeReferee
 *
 * Description:
 *
 *  MatchmakeReferee のサンプルです。
 *  MATCHMAKE_MAX_PARTICIPANTS で指定された人数でマッチメイクした後、
 *  ラウンド開始から終了処理まで行います。
 *  また、P2Pの実装は省略しています。
 *  
 *  サンプルに実装されているシーケンスは以下の通りです。
 * 
 * 1. サーバーログイン
 * 2. NEX ユニーク ID 関連の処理 (必要に応じて省略可) 
 *    - 取得済みかの確認、無ければ新規取得 : AcquireNexUniqueIdWithPassword()
 *    - 関連付け : AssociateNexUniqueIdWithMyPrincipalId()
 * 3. 個人統計の取得 (又は作成) : GetOrCreateStats()
 * 4. 未集計のラウンドの取得 : GetNotSummarizedRound()   
 * 5. マッチメイク
 * 6. ラウンド開始 : StartRound()
 * 7. ラウンド開始パラメータ取得 : GetStartRoundParam()
 * 8. 参加者の個人統計取得 : GetStats()
 * 9. ラウンド終了 : EndRound() 又は, EndRoundWithoutReport()
 * 10. 結果取得
 *     - ラウンド結果取得 : GetRound()
 *     - 自分の個人統計を取得 : GetStats()
 * 11. マッチメイクセッション退出、ログアウト
 *
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

namespace
{

// マッチメイクで使用するゲームモード
const nn::nex::qUnsignedInt32 MATCHMAKE_GAME_MODE = 1001;

// 参加人数
const nn::nex::qUnsignedInt16 MATCHMAKE_MAX_PARTICIPANTS = 2;

// MatchmakeReferee で使うカテゴリ
const nn::nex::qUnsignedInt32 STATS_CATEGORY = 97;

// レート初期値
const nn::nex::qUnsignedInt32 INITIAL_RATING = 1500;

/*!
    @brief MatchmakeRefereeの一連の処理を行うステートマシン。
 */
class MatchmakeRefereeSample
{
public:
    /*!
        @brief コンストラクタ。

        @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
     */
    MatchmakeRefereeSample(nn::nex::NgsFacade* pNgsFacade)
        : m_pCurrentCallContext(NULL)
        , m_state(STATE_MATCHMAKEREFEREE_PREPARE)
        , m_finished(false)
        , m_pNgsFacade(pNgsFacade)
        , m_notificationHandler(this)
    {
        nn::nex::Credentials* pCred = m_pNgsFacade->GetCredentials();

        // ログインに成功していればCredentialsは有効な値となります。
        NEX_DEMO_ASSERT(pCred != NULL); 

        m_myPid = pCred->GetPrincipalID();
        NEX_DEMO_VERIFY(m_utilityClient.Bind(pCred));
        NEX_DEMO_VERIFY(m_matchMakingClient.Bind(pCred));
        NEX_DEMO_VERIFY(m_refereeClient.Bind(pCred));

        m_pNgsFacade->RegisterNotificationEventHandler(&m_notificationHandler);
    }

    /*!
        @brief デストラクタ。
     */
    virtual ~MatchmakeRefereeSample()
    {
        m_pNgsFacade->UnregisterNotificationEventHandler(&m_notificationHandler);
        if(NULL != m_pCurrentCallContext)
        {
            m_pCurrentCallContext->Cancel();
        }
    };

    enum State
    {
        STATE_MATCHMAKEREFEREE_PREPARE,
        STATE_MATCHMAKEREFEREE_PREPARE_WAITING,
        STATE_AUTOMATCH,
        STATE_AUTOMATCH_WAITING,
        STATE_START_ROUND,
        STATE_START_ROUND_WAITING,
        STATE_GET_START_ROUND_PARAM,
        STATE_GET_OTHER_STATS,
        STATE_END_ROUND,
        STATE_ROUND_SUMMARIZE_WAITING,
        STATE_GET_RESULTS,
        STATE_GET_RESULTS_WAITING,
        STATE_ENDPARTICIPATION,
        STATE_RMC_RESULT_WAITING,
        STATE_ERROR_BEFORE_MATCHMAKE,
        STATE_ERROR_AFTER_MATCHMAKE
    };

    /*!
        @brief NGSから通知を処理するイベントハンドラ。
     */
    class MyNotificationEventHandler : public nn::nex::NotificationEventHandler
    {
    public:
        MyNotificationEventHandler(MatchmakeRefereeSample* sample):m_pSample(sample){};
        virtual ~MyNotificationEventHandler(){};

        void ProcessNotificationEvent(const nn::nex::NotificationEvent &oEvent)
        {
            nn::nex::StringStream ss;
            ss << oEvent;
            NEX_DEMO_LOG("Notification " STRING_FORMAT "\n\n", ss.CStr());

            // メッセージ毎の処理
            switch (oEvent.GetType())
            {
            case nn::nex::NotificationEvents::ParticipationEvent:
                switch (oEvent.GetSubType())
                {
                case nn::nex::ParticipationEvents::Participate:
                    NEX_DEMO_LOG("PARTICIPATION_EVENT PARTICIPATE: [PID " NEX_PID_FORMAT "] join [gathering ID %u]\n",
                        static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                    // オーナーは通知で参加者を管理。
                    m_pSample->m_participants.push_back(static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()));
                    // 全員集まったら締め切ってラウンド開始処理へ移行する
                    // 通知で自分以外の参加者をカウントするので、最大参加人数 - 1 分通知がきたら、
                    // 全員集まったということ
                    if(m_pSample->m_participants.size() == (MATCHMAKE_MAX_PARTICIPANTS - 1) &&
                       m_pSample->GetState() == STATE_AUTOMATCH_WAITING)
                    {
                        m_pSample->m_participants.push_back(m_pSample->GetMyPid());
                        m_pSample->SetState(STATE_START_ROUND);
                    }
                    break;
                case nn::nex::ParticipationEvents::EndParticipation:
                    NEX_DEMO_LOG("PARTICIPATION_EVENT END: [PID " NEX_PID_FORMAT "] end [gathering ID %u]\n",
                        static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                    break;
                case nn::nex::ParticipationEvents::Disconnect:
                    NEX_DEMO_LOG("PARTICIPATION_EVENT DISCONNECT: [PID " NEX_PID_FORMAT "] disconnect [gathering ID %u]\n",
                        static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()), static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                    break;
                default:
                    NEX_DEMO_LOG("PARTICIPATION_EVENT: unsupported participation event\n");
                    break;
                }
                break;
            case nn::nex::NotificationEvents::OwnershipChangeEvent:
                NEX_DEMO_LOG("NOTIFICATION_EVENT OWNERSHIPCHANGE: [gathering ID %u] changed owner to [PID " NEX_PID_FORMAT "]\n",
                    static_cast<nn::nex::GatheringID>(oEvent.GetParam1()), static_cast<nn::nex::PrincipalID>(oEvent.GetParam2()));
                break;
            case nn::nex::NotificationEvents::GatheringUnregistered:
                NEX_DEMO_LOG("NOTIFICATION_EVENT GATHERINGUNREGISTERED: [gathering ID %u] unregistered\n", static_cast<nn::nex::GatheringID>(oEvent.GetParam1()));
                break;
            case nn::nex::NotificationEvents::RoundStarted:
                NEX_DEMO_LOG("NOTIFICATION_EVENT ROUND STARTED: [round ID %llu]\n", oEvent.GetParam1());
                if(!m_pSample->IsOwner())
                {
                    // オーナー以外は StartRound() のパラメータを取得する。
                    m_pSample->m_roundId = oEvent.GetParam1();
                    m_pSample->SetState(STATE_GET_START_ROUND_PARAM);
                }
                break;
            case nn::nex::NotificationEvents::FirstRoundReportReceived:
                NEX_DEMO_LOG("NOTIFICATION_EVENT FIRST ROUNDREPORT RECEIVED: [round ID %llu]\n", oEvent.GetParam1());
                break;
            case nn::nex::NotificationEvents::RoundSummarized:
                NEX_DEMO_LOG("NOTIFICATION_EVENT ROUND SUMMARIZED: [round ID %llu]\n", oEvent.GetParam1());
                m_pSample->SetState(STATE_GET_RESULTS);
                break;
            default:
                NEX_DEMO_LOG("NOTIFICATION_EVENT: unsupported notification event\n");
                break;
            }
        }
    private:
        MatchmakeRefereeSample* m_pSample;
    };

    /*!
        @brief NEXユニークIDの取得や、統計情報の作成などのMatchmakeReferee機能を利用するための一連の準備処理を行うクラスです。
     */
    class MatchmakeRefereePrepareApiChain {
    public:
        MatchmakeRefereePrepareApiChain(MatchmakeRefereeSample* sample)
            : m_pSample(sample)
            , m_Result()
            , m_IsCompleted(false) {};
        ~MatchmakeRefereePrepareApiChain() 
        {
            if(NULL != m_pCurrentCallContext)
            {
                m_pCurrentCallContext->Cancel();
            }
        };

        void Call() { LoadUniueId(); };
        const nn::nex::qResult& GetResult() const { return m_Result; }
        nn::nex::qBool IsCompleted() const { return m_IsCompleted; }

    private:
        /*!
            @brief セーブデータから関連NEXユニークIDを取得。
         */
        void LoadUniueId()
        {
            // セーブデータ読み込みは未実装
            m_savedNexUniqueIds.clear();
            OnCompletionLoadUniueId(); 
        };
        /*!
            @brief セーブデータから関連NEXユニークIDを取得する処理が完了した時に呼び出されるコールバック関数。
         */
        void OnCompletionLoadUniueId()
        {
            AcquireUniqueId();
        };
        /*!
            @brief NEXユニークIDの新規取得。
         */
        void AcquireUniqueId()
        {
            NEX_DEMO_LOG("\nCall AcquireNexUniqueIdWithPassword().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereePrepareApiChain::OnCompletionAcquireUniqueId), this));
            m_pSample->m_utilityClient.AcquireNexUniqueIdWithPassword(pCallContext, &m_uniqueIdInfo);
        };
        /*!
            @brief NEXユニークIDの新規取得が完了したときに呼び出されるコールバック関数。

            @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
         */
        void OnCompletionAcquireUniqueId(nn::nex::ProtocolCallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     AcquireNexUniqueIdWithPassword() succeeded.\n");
                m_uniqueIdInfo.Trace(TRACE_ALL);
                AssociateUniqueId();
            }
            else
            {
                NEX_DEMO_LOG("     AcquireNexUniqueIdWithPassword() failed.\n");
                m_IsCompleted = true;
            }
        };
        /*!
            @brief NEXユニークIDの関連付け。
         */
        void AssociateUniqueId()
        {
            NEX_DEMO_LOG("\nCall AssociateNexUniqueIdWithMyPrincipalId().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            m_savedNexUniqueIds.insert(m_savedNexUniqueIds.begin(), m_uniqueIdInfo);
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereePrepareApiChain::OnCompletionAssociateUniqueId), this));
            m_pSample->m_utilityClient.AssociateNexUniqueIdWithMyPrincipalId(pCallContext, m_savedNexUniqueIds);
        };
        /*!
            @brief NEXユニークIDの関連付けが完了したときに呼び出されるコールバック関数。

            @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
         */
        void OnCompletionAssociateUniqueId(nn::nex::ProtocolCallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     AssociateNexUniqueIdWithMyPrincipalId() succeeded.\n");
                GetOrCreateStats();
            }
            else
            {
                NEX_DEMO_LOG("     AssociateNexUniqueIdWithMyPrincipalId() failed.\n");
                m_IsCompleted = true;
            }        
        };
        /*!
            @brief 統計情報の取得または作成。
         */
        void GetOrCreateStats()
        {
            NEX_DEMO_LOG("\nCall GetOrCreateStats().\n");
            nn::nex::MatchmakeRefereeStatsInitParam param;
            param.SetCategory(STATS_CATEGORY);
            param.SetInitialRatingValue(INITIAL_RATING);
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereePrepareApiChain::OnCompletionGetOrCreateStats), this));
            m_pSample->m_refereeClient.GetOrCreateStats(pCallContext, param, &m_tempStats);
        };
        /*!
            @brief 統計情報の取得または作成が完了したときに呼び出されるコールバック関数。

            @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
         */
        void OnCompletionGetOrCreateStats(nn::nex::ProtocolCallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     GetOrCreateStats() succeeded.\n");
                m_tempStats.Trace();
                GetNotSummarizedRound();
            }
            else
            {
                NEX_DEMO_LOG("     GetOrCreateStats() failed.\n");
                m_IsCompleted = true;
            }        
        };
        /*!
            @brief 未集計のラウンド情報の取得。
         */
        void GetNotSummarizedRound()
        {
            NEX_DEMO_LOG("\nCall GetNotSummarizedRound().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereePrepareApiChain::OnCompletionGetNotSummarizedRound), this));
            m_pSample->m_refereeClient.GetNotSummarizedRound(pCallContext, &m_tempRounds);
        };
        /*!
            @brief 未集計のラウンド情報の取得が完了したときに呼び出されるコールバック関数。

            @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
         */
        void OnCompletionGetNotSummarizedRound(nn::nex::ProtocolCallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     GetNotSummarizedRound() succeeded.\n");
                for (nn::nex::qVector<nn::nex::MatchmakeRefereeRound>::const_iterator it = m_tempRounds.begin();
                        it != m_tempRounds.end(); ++it)
                {
                    it->Trace(TRACE_ALL);
                }
            }
            else
            {
                NEX_DEMO_LOG("     GetNotSummarizedRound() failed.\n");
            }
            m_IsCompleted = true;
        };

        MatchmakeRefereeSample* m_pSample;
        nn::nex::ProtocolCallContext* m_pCurrentCallContext;

        nn::nex::UniqueIdInfo m_uniqueIdInfo;
        nn::nex::MatchmakeRefereeStats m_tempStats;
        nn::nex::qVector<nn::nex::MatchmakeRefereeRound> m_tempRounds;
        nn::nex::qVector<nn::nex::UniqueIdInfo> m_savedNexUniqueIds;

        nn::nex::qResult m_Result;
        nn::nex::qBool m_IsCompleted;
    };

    /*!
        @brief 集計結果を取得する一連の処理を行うクラスです。
     */
    class MatchmakeRefereeGetResultApiChain {
    public:
        MatchmakeRefereeGetResultApiChain(MatchmakeRefereeSample* sample)
            : m_pSample(sample)
            , m_Result()
            , m_IsCompleted(false) {};
        ~MatchmakeRefereeGetResultApiChain() 
        {
            if(NULL != m_pCurrentCallContext)
            {
                m_pCurrentCallContext->Cancel();
            }
        };

        void Call() { GetRound(); };
        const nn::nex::qResult& GetResult() const { return m_Result; }
        nn::nex::qBool IsCompleted() const { return m_IsCompleted; }

    private:
        /*!
            @brief 集計結果取得(ラウンド)。
         */
        void GetRound()
        {
            NEX_DEMO_LOG("\nCall GetRound().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereeGetResultApiChain::OnCompletionGetRound), this));
            m_pSample->m_refereeClient.GetRound(pCallContext, m_pSample->m_roundId, &m_tempRound);
        };
        /*!
            @brief 集計結果の取得(ラウンド)が完了したときに呼び出されるコールバック関数。

            @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
         */
        void OnCompletionGetRound(nn::nex::ProtocolCallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     GetRound() succeeded.\n");
                m_tempRound.Trace();
                GetMyStats();
            }
            else
            {
                NEX_DEMO_LOG("     GetRound() failed.\n");
                m_IsCompleted = true;
            }        
        };
        /*!
            @brief 集計結果取得(個人統計)。
         */
        void GetMyStats()
        {
            NEX_DEMO_LOG("\nCall GetStats().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            nn::nex::MatchmakeRefereeStatsTarget target;
            target.SetPrincipalId(m_pSample->GetMyPid());
            target.SetCategory(STATS_CATEGORY);
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereeGetResultApiChain::OnCompletionGetMyStats), this));
            m_pSample->m_refereeClient.GetStats(pCallContext, target, &m_tempStats);
        };
        /*!
            @brief 集計結果の取得(個人統計)が完了したときに呼び出されるコールバック関数。

            @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
         */
        void OnCompletionGetMyStats(nn::nex::ProtocolCallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     GetStats() succeeded.\n");
                m_tempStats.Trace(TRACE_ALL);
            }
            else
            {
                NEX_DEMO_LOG("     GetStats() failed.\n");
            }
            m_IsCompleted = true;
        };

        MatchmakeRefereeSample* m_pSample;
        nn::nex::ProtocolCallContext* m_pCurrentCallContext;

        nn::nex::MatchmakeRefereeStats m_tempStats;
        nn::nex::MatchmakeRefereeRound m_tempRound;

        nn::nex::qResult m_Result;
        nn::nex::qBool m_IsCompleted;
    };

    /*!
        @brief ラウンドを開始する一連の処理を行うクラスです。
     */
    class MatchmakeRefereeStartRoundApiChain {
    public:
        MatchmakeRefereeStartRoundApiChain(MatchmakeRefereeSample* sample)
            : m_pSample(sample)
            , m_Result()
            , m_IsCompleted(false) {};
        ~MatchmakeRefereeStartRoundApiChain() 
        {
            if(NULL != m_pCurrentCallContext)
            {
                m_pCurrentCallContext->Cancel();
            }
        };

        void Call() { CloseParticipation(); };
        const nn::nex::qResult& GetResult() const { return m_Result; }
        nn::nex::qBool IsCompleted() const { return m_IsCompleted; }

    private:
        /*!
            @brief マッチメイク締切処理。
         */
        void CloseParticipation()
        {
            NEX_DEMO_LOG("\nCall CloseParticipation().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereeStartRoundApiChain::OnCompletionCloseParticipation), this));
            m_pSample->m_matchMakingClient.CloseParticipation(pCallContext, m_pSample->GetGatheringID());
        };
        /*!
            @brief マッチメイク締切処理が完了したときに呼び出されるコールバック関数。

            @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
         */
        void OnCompletionCloseParticipation(nn::nex::ProtocolCallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     CloseParticipation() succeeded.\n");
                StartRound();
            }
            else
            {
                NEX_DEMO_LOG("     CloseParticipation() failed.\n");
                m_IsCompleted = true;
            }        
        };
        /*!
            @brief ラウンド開始処理。
         */
        void StartRound()
        {
            nn::nex::MatchmakeRefereeStartRoundParam startParam;
            startParam.SetGatheringId(m_pSample->GetGatheringID());
            startParam.SetPersonalDataCategory(STATS_CATEGORY);
            startParam.SetPrincipalIds(m_pSample->m_participants);

            NEX_DEMO_LOG("\nCall StartRound().\n");
            nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
            m_pCurrentCallContext = pCallContext;
            nn::nex::MatchmakeRefereeStatsTarget target;
            target.SetPrincipalId(m_pSample->GetMyPid());
            target.SetCategory(STATS_CATEGORY);
            pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
            pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereeStartRoundApiChain::OnCompletionStartRound), this));
            m_pSample->m_refereeClient.StartRound(pCallContext, startParam, &m_pSample->m_roundId);
        };
        /*!
            @brief ラウンド開始処理が完了したときに呼び出されるコールバック関数。

            @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
         */
        void OnCompletionStartRound(nn::nex::ProtocolCallContext* pCallContext)
        {
            m_pCurrentCallContext = NULL;
            m_Result = pCallContext->GetOutcome();
            if (m_Result)
            {
                NEX_DEMO_LOG("     StartRound() succeeded.\n");
                NEX_DEMO_LOG("roundId = %llu\n", m_pSample->m_roundId);
            }
            else
            {
                NEX_DEMO_LOG("     StartRound() failed.\n");
            }
            m_IsCompleted = true;
        };

        MatchmakeRefereeSample* m_pSample;
        nn::nex::ProtocolCallContext* m_pCurrentCallContext;

        nn::nex::qResult m_Result;
        nn::nex::qBool m_IsCompleted;
    };

    void SetState(State state){m_state = state;};
    State GetState(){return m_state;};
    nn::nex::PrincipalID GetMyPid(){return m_myPid;};
    void SetOwnerPid(nn::nex::PrincipalID pid){m_ownerPid = pid;};
    nn::nex::PrincipalID GetOwnerPid(){return m_ownerPid;};
    qBool IsOwner(){return GetOwnerPid() == GetMyPid();};
    void SetGatheringID(nn::nex::GatheringID gid){m_gid = gid;};
    nn::nex::GatheringID GetGatheringID(){return m_gid;};
    qBool IsConnected();

    qBool GameLoop();
    void AutoMatchmake();
    void OnCompletionAutoMatchmake(nn::nex::ProtocolCallContext* pCallContext);
    void GetStartRoundParam();
    void OnCompletionGetStartRoundParam(nn::nex::ProtocolCallContext* pCallContext);
    void GetOtherStats();
    void OnCompletionGetOtherStats(nn::nex::ProtocolCallContext* pCallContext);
    void EndRound();
    void OnCompletionEndRound(nn::nex::ProtocolCallContext* pCallContext);
    void EndParticipation();
    void OnCompletionEndParticipation(nn::nex::ProtocolCallContext* pCallContext);

    nn::nex::qVector<nn::nex::PrincipalID> m_participants;
private:
    nn::nex::ProtocolCallContext* m_pCurrentCallContext;
    State m_state;
    qBool m_finished;
    nn::nex::PrincipalID m_myPid;
    nn::nex::PrincipalID m_ownerPid;
    nn::nex::GatheringID m_gid;
    nn::nex::NgsFacade* m_pNgsFacade;
    nn::nex::UtilityClient m_utilityClient;
    nn::nex::MatchmakeExtensionClient m_matchMakingClient;
    nn::nex::MatchmakeRefereeClient m_refereeClient;
    MyNotificationEventHandler m_notificationHandler;
    qUnsignedInt64 m_roundId;

    nn::nex::MatchmakeRefereeStats m_tempStats;
    nn::nex::MatchmakeSession m_matchmakeSession;
    nn::nex::MatchmakeRefereeStartRoundParam m_tempStartParam;
    nn::nex::qVector<nn::nex::MatchmakeRefereeStats> m_tempStatsVec;
    nn::nex::qVector<nn::nex::qResult> m_tempResults;
};

/*!
    @brief ゲームサーバーとの接続が維持されているかを確認します。

    @retval true 維持されている。
    @retval false 切断されている。
 */
qBool MatchmakeRefereeSample::IsConnected()
{
    //ゲームサーバから切断されているか
    if(!m_pNgsFacade->IsConnected())
    {
        NEX_DEMO_LOG("disconnected from NGS\n");
        return false;
    }

    // ネットワークの接続性確認
    if(!nex_demo_util::IsNetworkConnected())
    {
        NEX_DEMO_LOG("network connection unavailable\n");
        return false;
    }

    return true;
}

/*!
    @brief オートマッチメイクを行います マッチメイクセッションに参加もしくは作成します。
 */
void MatchmakeRefereeSample::AutoMatchmake()
{
    NEX_DEMO_LOG("\nCall AutoMatchmake().\n");
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereeSample::OnCompletionAutoMatchmake), this));

    // オートマッチのパラメータ
    nn::nex::MatchmakeSession matchmakeSession;

    // 締切状態の設定
    matchmakeSession.SetOpenParticipation(true);

    // 最小・最大参加人数
    matchmakeSession.SetMinParticipants(1);
    matchmakeSession.SetMaxParticipants(MATCHMAKE_MAX_PARTICIPANTS);

    // ゲームモード
    matchmakeSession.SetGameMode(MATCHMAKE_GAME_MODE);

    // マッチメイクシステムタイプ
    matchmakeSession.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // オーナーがログアウトしたときにOwnerの委譲を行うか
    matchmakeSession.SetFlag(nn::nex::GatheringFlags::MigrateOwner);

    // 同様に検索条件を設定
    nn::nex::MatchmakeSessionSearchCriteria criteria;
    criteria.SetMinParticipants(matchmakeSession.GetMinParticipants());
    criteria.SetMaxParticipants(matchmakeSession.GetMaxParticipants());
    criteria.SetGameMode(matchmakeSession.GetGameMode());
    criteria.SetMatchmakeSystemType(matchmakeSession.GetMatchmakeSystemType());

    nn::nex::qList<nn::nex::MatchmakeSessionSearchCriteria> criteriaList;
    criteriaList.push_back(criteria);

    nn::nex::AutoMatchmakeParam autoMatchmakeParam;
    autoMatchmakeParam.SetSourceMatchmakeSession(matchmakeSession);
    autoMatchmakeParam.SetSearchCriteria(criteriaList);
    autoMatchmakeParam.SetJoinMessage(NEX_T("auto matchmake session"));
    // オートマッチを行う
    m_matchMakingClient.AutoMatchmake(pCallContext, autoMatchmakeParam, &m_matchmakeSession);
}

/*!
    @brief オートマッチメイクが完了したときに呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void MatchmakeRefereeSample::OnCompletionAutoMatchmake(nn::nex::ProtocolCallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (result)
    {
        NEX_DEMO_LOG("     AutoMatchmake() succeeded.\n");
        SetGatheringID(m_matchmakeSession.GetID());
        SetOwnerPid(m_matchmakeSession.GetOwnerPID());
        NEX_DEMO_LOG("[PID Info >> JoinedGID:%u OwnerPID:" NEX_PID_FORMAT " MyPID:" NEX_PID_FORMAT "]\n", 
                    GetGatheringID(), 
                    GetOwnerPid(), 
                    GetMyPid());
    }
    else
    {
        NEX_DEMO_LOG("     AutoMatchmake() failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        SetState(STATE_ERROR_BEFORE_MATCHMAKE);
    }
}

/*!
    @brief ラウンド開始パラメータ取得処理。
 */
void MatchmakeRefereeSample::GetStartRoundParam()
{
    NEX_DEMO_LOG("\nCall GetStartRoundParam().\n");
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereeSample::OnCompletionGetStartRoundParam), this));
    m_refereeClient.GetStartRoundParam(pCallContext, m_roundId, &m_tempStartParam);
}

/*!
    @brief ラウンド開始パラメータ取得が完了したときに呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void MatchmakeRefereeSample::OnCompletionGetStartRoundParam(nn::nex::ProtocolCallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (result)
    {
        NEX_DEMO_LOG("     GetStartRoundParam() succeeded.\n");
        m_tempStartParam.Trace();
        m_participants = m_tempStartParam.GetPrincipalIds();
        SetState(STATE_GET_OTHER_STATS);
    }
    else
    {
        NEX_DEMO_LOG("     GetStartRoundParam() failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        SetState(STATE_ERROR_AFTER_MATCHMAKE);
    }
}

/*!
    @brief ラウンド参加者の情報取得処理。
 */
void MatchmakeRefereeSample::GetOtherStats()
{
    NEX_DEMO_LOG("\nCall GetStats().\n");
    nn::nex::qVector<nn::nex::MatchmakeRefereeStatsTarget> targets;
    for(nn::nex::qVector<nn::nex::PrincipalID>::const_iterator it = m_participants.begin();
        it != m_participants.end(); ++it)
    {
        nn::nex::MatchmakeRefereeStatsTarget target;
        target.SetCategory(STATS_CATEGORY);
        target.SetPrincipalId(*it);
        targets.push_back(target);
    }
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereeSample::OnCompletionGetOtherStats), this));
    m_refereeClient.GetStats(pCallContext, targets, &m_tempStatsVec, &m_tempResults);
}

/*!
    @brief ラウンド参加者の情報取得処理が完了したときに呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void MatchmakeRefereeSample::OnCompletionGetOtherStats(nn::nex::ProtocolCallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (result)
    {
        NEX_DEMO_LOG("     GetStats() succeeded.\n");
        nn::nex::qVector<nn::nex::MatchmakeRefereeStats>::const_iterator its = m_tempStatsVec.begin();
        nn::nex::qVector<nn::nex::qResult>::const_iterator itr = m_tempResults.begin();
        for (; its != m_tempStatsVec.end(); ++its, ++itr)
        {
            if(*itr)
            {
                its->Trace(TRACE_ALL);
            }
            else
            {
                itr->Trace(TRACE_ALL);
            }
        }
        SetState(STATE_END_ROUND);
    }
    else
    {
        NEX_DEMO_LOG("     GetStats() failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        SetState(STATE_ERROR_AFTER_MATCHMAKE);
    }
}

/*!
    @brief ラウンド終了処理。
 */
void MatchmakeRefereeSample::EndRound()
{
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereeSample::OnCompletionEndRound), this));

    // 20%の確率で EndRoundWithoutReport() する。
    if(nn::nex::Platform::GetRandomNumber(100) < 20)
    {
        NEX_DEMO_LOG("\nCall EndRoundWithoutReport().\n");
        m_refereeClient.EndRoundWithoutReport(pCallContext, m_roundId);
        return;
    }

    nn::nex::qVector<nn::nex::MatchmakeRefereePersonalRoundResult> personalRoundResults;
    
    // 参加者全員で同じレポートを送信するため、順序を揃える。
    std::sort(m_participants.begin(), m_participants.end());
    
    for(nn::nex::qVector<nn::nex::PrincipalID>::const_iterator it = m_participants.begin();
        it != m_participants.end(); ++it)
    {
        // P2P実装省略の為、ラウンドIDをキーにして各ユーザの個人統計を決める。
        nn::nex::qUnsignedInt64 key = m_roundId + *it;
        nn::nex::MatchmakeRefereePersonalRoundResult pRes;
        const nn::nex::qUnsignedInt32 ENUM_ROUND_RESULT_NUM = 3;
        const nn::nex::qUnsignedInt32 PERSONAL_ROUND_RESULT_FLAG_MAX = 3;
        pRes.SetPrincipalId(*it);
        pRes.SetPersonalRoundResultFlag(key % PERSONAL_ROUND_RESULT_FLAG_MAX);
        pRes.SetRoundWinLoss(key % ENUM_ROUND_RESULT_NUM);
        switch(key % ENUM_ROUND_RESULT_NUM)
        {
        case nn::nex::MatchmakeRefereeConstants::ROUND_RESULT_LOSS:
            pRes.SetRatingValueChange(-100);
            break;
        case nn::nex::MatchmakeRefereeConstants::ROUND_RESULT_WIN:
            pRes.SetRatingValueChange(100);
            break;
        case nn::nex::MatchmakeRefereeConstants::ROUND_RESULT_DRAW:
            break;
        }
        const qChar8 binary[] = {0x1, 0x2, 0x3, 0x4};
        pRes.SetBuffer(binary, sizeof(binary));
        personalRoundResults.push_back(pRes);

    }
    nn::nex::MatchmakeRefereeEndRoundParam endRoundParam;
    endRoundParam.SetRoundId(m_roundId);
    endRoundParam.SetPersonalRoundResults(personalRoundResults);
    NEX_DEMO_LOG("\nCall EndRound().\n");
    m_refereeClient.EndRound(pCallContext, endRoundParam);
}

/*!
    @brief ラウンド終了処理が完了したときに呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void MatchmakeRefereeSample::OnCompletionEndRound(nn::nex::ProtocolCallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (result)
    {
        NEX_DEMO_LOG("     EndRound()/WithoutReport() succeeded.\n");
    }
    else
    {
        NEX_DEMO_LOG("     EndRound()/WithoutReport() failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        SetState(STATE_ERROR_AFTER_MATCHMAKE);        
    }
}

/*!
    @brief マッチメイクセッションから退出します。
 */
void MatchmakeRefereeSample::EndParticipation()
{
    if (GetGatheringID() == INVALID_GATHERINGID)
    {
        // 参加していないのでEndParticipation()は呼び出さない。
        m_finished = true;
        return;
    }
    NEX_DEMO_LOG("\nCall EndParticipation().\n");
    nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
    m_pCurrentCallContext = pCallContext;
    pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
    pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MatchmakeRefereeSample::OnCompletionEndParticipation), this));
    m_matchMakingClient.EndParticipation(pCallContext, GetGatheringID());
}

/*!
    @brief マッチメイクセッションから退出する処理が完了した時に呼び出されるコールバック関数。

    @param[in] pCallContext 非同期処理に紐づけられている呼び出しコンテキスト。
 */
void MatchmakeRefereeSample::OnCompletionEndParticipation(nn::nex::ProtocolCallContext* pCallContext)
{
    m_pCurrentCallContext = NULL;
    nn::nex::qResult result = pCallContext->GetOutcome();
    if (!result && !(result == QERROR(RendezVous, SessionVoid)))
    {
        NEX_DEMO_LOG("     EndParticipation() failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
    }
    else
    {
        NEX_DEMO_LOG("     EndParticipation() succeeded.\n");
    }
    m_finished = true;
}

/*!
    @brief ゲームメインループ。
 */
qBool MatchmakeRefereeSample::GameLoop()
{
    MatchmakeRefereePrepareApiChain* prepareApiChain = NULL;
    MatchmakeRefereeGetResultApiChain* getResultApiChain = NULL;
    MatchmakeRefereeStartRoundApiChain* startRoundApiChain = NULL;

    for(;;)
    {
        switch(GetState())
        {
        // MatchmakeReferee ログイン後処理
        case STATE_MATCHMAKEREFEREE_PREPARE:
            prepareApiChain = new MatchmakeRefereePrepareApiChain(this);
            prepareApiChain->Call();
            SetState(STATE_MATCHMAKEREFEREE_PREPARE_WAITING);
            break;
        // MatchmakeReferee ログイン後処理 完了待ち
        case STATE_MATCHMAKEREFEREE_PREPARE_WAITING:
            if(prepareApiChain->IsCompleted())
            {
                if(prepareApiChain->GetResult())
                {
                    SetState(STATE_AUTOMATCH);
                }
                else
                {   
                    nex_demo_util::ReportQResultReturnCode(prepareApiChain->GetResult());
                    SetState(STATE_ERROR_BEFORE_MATCHMAKE);
                }
                delete prepareApiChain;
                prepareApiChain = NULL;
            }
            break;            
            // マッチメイク
        case STATE_AUTOMATCH:
            AutoMatchmake();
            SetState(STATE_AUTOMATCH_WAITING);
            break;
        // マッチメイク完了待ち
        case STATE_AUTOMATCH_WAITING:
            // タイムアウト処理省略
            break;
        // ラウンド開始処理
        case STATE_START_ROUND:
            startRoundApiChain = new MatchmakeRefereeStartRoundApiChain(this);
            startRoundApiChain->Call();
            SetState(STATE_START_ROUND_WAITING);
            break;
        // ラウンド開始処理待ち
        case STATE_START_ROUND_WAITING:
            if(startRoundApiChain->IsCompleted())
            {
                if(startRoundApiChain->GetResult())
                {
                    SetState(STATE_GET_OTHER_STATS);
                }
                else
                {   
                    nex_demo_util::ReportQResultReturnCode(startRoundApiChain->GetResult());
                    SetState(STATE_ERROR_AFTER_MATCHMAKE);
                }
                delete startRoundApiChain;
                startRoundApiChain = NULL;
            }
            break;
        // ラウンド開始パラメータ取得処理
        case STATE_GET_START_ROUND_PARAM:
            GetStartRoundParam();
            SetState(STATE_RMC_RESULT_WAITING);
            break;
        // ラウンド参加者の情報取得
        case STATE_GET_OTHER_STATS:
            GetOtherStats();
            SetState(STATE_RMC_RESULT_WAITING);
            break;
        // ラウンド終了処理
        case STATE_END_ROUND:
            EndRound();
            SetState(STATE_ROUND_SUMMARIZE_WAITING);
            break;
        // 集計待ち
        case STATE_ROUND_SUMMARIZE_WAITING:
            // タイムアウト処理省略
            break;
        // 集計結果取得
        case STATE_GET_RESULTS:
            getResultApiChain = new MatchmakeRefereeGetResultApiChain(this);
            getResultApiChain->Call();
            SetState(STATE_GET_RESULTS_WAITING);
            break;
        case STATE_GET_RESULTS_WAITING:
            if(getResultApiChain->IsCompleted())
            {
                if(getResultApiChain->GetResult())
                {
                    SetState(STATE_ENDPARTICIPATION);
                }
                else
                {   
                    nex_demo_util::ReportQResultReturnCode(getResultApiChain->GetResult());
                    SetState(STATE_ERROR_AFTER_MATCHMAKE);
                }
                delete getResultApiChain;
                getResultApiChain = NULL;
            }
            break;
        // マッチメイクセッション退出
        case STATE_ENDPARTICIPATION:
            EndParticipation();
            SetState(STATE_RMC_RESULT_WAITING);
            break;
        // RMC処理結果待ち
        case STATE_RMC_RESULT_WAITING:
            break;
        // マッチメイク前のエラー
        case STATE_ERROR_BEFORE_MATCHMAKE:
            m_finished = true;
            break;
        // マッチメイク後のエラー
        case STATE_ERROR_AFTER_MATCHMAKE:
            EndParticipation();
            SetState(STATE_RMC_RESULT_WAITING);
            break;
        }

        // 接続確認
        if (!IsConnected())
        {
            m_finished = true;
        }

        // 終了確認
        if(m_finished)
        {
            break;
        }

        // メッセージのディスパッチ
        nn::nex::Scheduler::GetInstance()->DispatchAll();
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
    }

    return true;
}

/*!
    @brief NEXの初期化後にNGSへログインし、MatchmakeReferee用のメインループに遷移します。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            {
                MatchmakeRefereeSample sample(&ngsFacade);

                // ゲームループへ遷移
                if(!sample.GameLoop())
                {
                    NEX_DEMO_LOG("GameLoop finished with error.\n");
                }
            }

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }

    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("    MatchmakeReferee sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
