﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

/*
 *  RankingSimple
 *
 *  本サンプルは、ランキングの基本的な使用法を示します。
 *  処理の内容は以下のようになります。
 *
 *  1. ゲームサーバにログインし、ランキングを利用可能な状態にします。
 *  2. ランキング共通データを登録します。
 *  3. スコアを登録します。
 *  4. ランキングリスト(TOP10)を取得します。
 *  5. スコアから順位を推定します。
 *  6. ランキングの統計を取得します。
 *  7. スコアの属性値を変更します。
 *  8. スコアを削除します。
 *  9. スコアを再登録します。
 * 10. ランキングの利用を終了し、ゲームサーバからログアウトします。
 */

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

/*
   NEXユニークID を利用してプリンシパルID よりも細かい単位でスコアを登録する場合は、
   このマクロを有効にして NEXユニークID を取得してください。
*/
// #define RANKING_DEMO_USE_NEX_UNIQUE_ID

// バナー表示用マクロ
#define BANNER(...) do                                                               \
                    {                                                                \
                        NEX_DEMO_LOG("\n***************************************\n"); \
                        NEX_DEMO_LOG(__VA_ARGS__);                                   \
                        NEX_DEMO_LOG("\n");                                          \
                    }                                                                \
                    while (0)

namespace
{
// NEXユニークID（プリンシパルID単位でスコアを登録する場合は INVALID_UNIQUEID を指定します）
qUnsignedInt64 s_nexUniqueId = nn::nex::INVALID_UNIQUEID;

// 使用するランキングのカテゴリ。
const qUnsignedInt32 RANKING_CATEGORY = 0;

// グループ
const qByte RANKING_GROUP0 = 0;
const qByte RANKING_GROUP1 = 0;

#ifdef RANKING_DEMO_USE_NEX_UNIQUE_ID
/*!
    @brief NEXユニークIDを取得する（NEXユニークIDを利用する場合のみ）。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
qBool AcquireNexUniqueId(nn::nex::NgsFacade* pNgsFacade)
{
    BANNER("AcquireNexUniqueId()");
    /*
     *  Utility サービスを使用し、NEXユニークIDを取得します。
     *  NEXユニークIDは UtilityClient::AcquireNexUniqueId() を呼び出す度に新しく発行されるゲームサーバ内でユニークなIDです。
     *  プリンシパルIDより細かい単位でスコアを分けて登録したい場合に利用します。
     *
     *  NEXユニークIDを取得した場合は、セーブ領域に取得したIDを保存して下さい。
     *  NEXユニークIDの保存に成功した場合は、次回以降、セーブ領域から取得して利用してください。
     */

    // セーブ領域からNEXユニークIDの取得を試みる
    {
        /*
        if (LoadNexUniqueId(&s_nexUniqueId))
        {
            // 有効なNEXユニークIDかどうかを確認
            if(nn::nex::UtilityClient::IsValidNexUniqueId(s_nexUniqueId)) return;
        }
        */
    }

    // 取得できなかった場合、サーバから取得する

    nn::nex::UtilityClient utilityClient;
    NEX_DEMO_ASSERT(utilityClient.Bind(pNgsFacade->GetCredentials()));

    // NEXユニークIDを取得
    nn::nex::ProtocolCallContext oContext;
    utilityClient.AcquireNexUniqueId(&oContext, &s_nexUniqueId);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to AcquireNexUniqueId()\n");
        return false;
    }

    NEX_DEMO_LOG("AcquireNexUniqueId() Success. (NexUniqueId=%llu)  \n", s_nexUniqueId);

    return true;
}
#endif /* RANKING_DEMO_USE_NEX_UNIQUE_ID */

/*!
    @brief ランキング共通データを登録する。

    @param[in] pRankingClient Credentials をバインド済みのオブジェクトを指定してください。
 */
qBool UploadCommonData(nn::nex::RankingClient *pRankingClient)
{
    BANNER("UploadCommonData()");
    /*
     *  ランキング共通データを登録します。
     *
     *  ランキング共通データとは、カテゴリ共通のデータ(例：プレイヤ名)のことです。
     *  このデータは Ranking::GetRanking() で取得され、すべての人に公開されます(＝誰でも取得できます)。
     */

    // ランキング共通データ
    const char* pCommonData = "PLAYER";
    // ランキング共通データのサイズ(プレイヤ名+終端文字)
    qByte commonDataSize = static_cast<qByte>(strlen(pCommonData) + 1);

    // ランキング共通データを登録する
    nn::nex::ProtocolCallContext oContext;
    pRankingClient->UploadCommonData(&oContext, pCommonData, commonDataSize, s_nexUniqueId);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to UploadCommonData()\n");
        return false;
    }
    NEX_DEMO_LOG("UploadCommonData() Success.\n");
    return true;
}

/*!
    @brief スコアを登録する。

    @param[in] pRankingClient Credentials をバインド済みのオブジェクトを指定してください。
 */
qBool UploadScore(nn::nex::RankingClient *pRankingClient)
{
    BANNER("UploadScore()");
    /*
     *  スコアを登録します。
     *
     *  カテゴリとグループを指定して、どこにスコアを登録するのかを決定します。
     *  パラメータには任意の64ビットデータ(例：キャラクタ番号)を指定できます。
     */

    // スコアデータを作成する
    nn::nex::RankingScoreData scoreData;
    {
        // カテゴリを指定します。
        scoreData.SetCategory(RANKING_CATEGORY);
        // スコアをセットします。
        scoreData.SetScore(nn::nex::Platform::GetRandomNumber(10000));
        // スコアの並び順を指定します。
        scoreData.SetOrderBy(nn::nex::RankingConstants::ORDER_BY_DESC); // 降順
        // スコアの更新モードを指定します。
        scoreData.SetUpdateMode(nn::nex::RankingConstants::UPDATE_MODE_NORMAL);
        // グループ0の値をセットします。
        scoreData.SetGroup0(RANKING_GROUP0);
        // グループ1の値をセットします。
        scoreData.SetGroup1(RANKING_GROUP1);
        // パラメータの値をセットします。
        scoreData.SetParam((qUnsignedInt64)nn::nex::Platform::GetRandomNumber(16));
    }

    // スコアを登録する
    nn::nex::ProtocolCallContext oContext;
    pRankingClient->UploadScore(&oContext, scoreData, s_nexUniqueId);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to UploadScore()\n");
        return false;
    }
    NEX_DEMO_LOG("UploadScore() Success. (score=%u)\n", scoreData.GetScore());
    return true;
}

/*!
    @brief ランキングのリストを表示する。

    @param[in] rankDataList 表示したいランキングデータ。
 */
void PrintRankDataList(const nn::nex::qVector<nn::nex::RankingRankData>& rankDataList)
{
    if(rankDataList.empty())
    {
        NEX_DEMO_LOG("RankDataList is empty.\n");
    }

    nn::nex::qVector<nn::nex::RankingRankData>::const_iterator it = rankDataList.begin();
    while(it != rankDataList.end())
    {
        NEX_DEMO_LOG("** [%2d] -------------------------------\n", static_cast<int>(it - rankDataList.begin()));
        NEX_DEMO_LOG("** principalId    = " NEX_PID_FORMAT "\n", it->GetPrincipalId());
        NEX_DEMO_LOG("** nexUniqueId    = %llu\n", it->GetUniqueId());
        NEX_DEMO_LOG("** order          = %d\n", it->GetOrder());
        NEX_DEMO_LOG("** category       = %d\n", it->GetCategory());
        const nn::nex::DateTime& updateTime = it->GetUpdateTime();
        (void)updateTime;
        NEX_DEMO_LOG("** updateTime     = %04d-%02d-%02d %02d:%02d:%02d\n",
                updateTime.GetYear(), updateTime.GetMonth(), updateTime.GetDay(),
                updateTime.GetHour(), updateTime.GetMinute(), updateTime.GetSecond());
        NEX_DEMO_LOG("** score          = %d\n", it->GetScore());
        NEX_DEMO_LOG("** group0         = %d\n", it->GetGroup0());
        NEX_DEMO_LOG("** group1         = %d\n", it->GetGroup1());
        NEX_DEMO_LOG("** param          = %llu\n", it->GetParam());
        NEX_DEMO_LOG("** commonDataSize = " NEX_SIZE_T_FORMAT "\n", it->GetCommonData().size());

        ++it;
    }
}

/*!
    @brief ランキングリスト(TOP10)を取得する。

    @param[in] pRankingClient Credentials をバインド済みのオブジェクトを指定してください。
 */
qBool GetRanking(nn::nex::RankingClient *pRankingClient)
{
    BANNER("GetRanking()");
    /*
     *  ランキングリストを取得します。
     *
     *  ・範囲ランキング
     *  ・周辺ランキング
     *  ・フレンド範囲ランキング
     *  ・フレンド周辺ランキング
     *  を取得することができます。
     *
     *  他人のプリンシパルID・ユニークIDを指定して、他人の周辺ランキングを取得することもできます。
     */

    // 順位計算パラメータを設定する
    nn::nex::RankingOrderParam orderParam;
    {
        // スコアが同じ場合の順位の計算方法を指定します
        //  - 上位2名が同点の場合、3番目が3位となります。
        orderParam.SetOrderCalculation(nn::nex::RankingConstants::ORDER_CALCULATION_113);
        //  - 上位2名が同点の場合、更新時間によって順位が決まります。
        //s_orderParam.SetOrderCalculation(nn::nex::RankingConstants::ORDER_CALCULATION_123);

        // フィルタリングするグループの種類を指定します。
        //  - グループによるフィルタリングを行いません。
        orderParam.SetFilterGroupIndex(nn::nex::RankingConstants::FILTER_GROUP_INDEX_NONE);
        //  - グループ0でフィルタリングします。
        //orderParam.SetFilterGroupIndex(nn::nex::RankingConstants::FILTER_GROUP_INDEX_0);
        //  - グループ1でフィルタリングします。
        //orderParam.SetFilterGroupIndex(nn::nex::RankingConstants::FILTER_GROUP_INDEX_1);

        // フィルタリングするグループの値を指定します。
        orderParam.SetFilterGroupNum(0);

        // ランキングの集計期間(タイムスコープ)を指定します
        //  - 全ての期間でのランキングを取得します
        orderParam.SetTimeScope(nn::nex::RankingConstants::TIME_SCOPE_ALL);
        //  - RMAS上で設定した期間(1) デフォルトは"直近の火曜日0時から現在まで"になります。
        //orderParam.SetTimeScope(nn::nex::RankingConstants::TIME_SCOPE_CUSTOM_0);
        //  - RMAS上で設定した期間(2) デフォルトは"直近の毎月1日0時から現在まで"になります。
        //orderParam.SetTimeScope(nn::nex::RankingConstants::TIME_SCOPE_CUSTOM_1);

        // 取得するリストの最大の長さ、オフセットを指定します
        orderParam.SetLength(10);
        orderParam.SetOffset(0);
    }

    // ランキング取得モードを指定します。
    // - 範囲ランキング(offset=0でトップランキングとなる)
    nn::nex::RankingConstants::RankingMode rankingMode = nn::nex::RankingConstants::RANKING_MODE_RANGE;
    // - 周辺ランキング(指定したユーザの周辺の順位のランキングリストを取得します)
    //nn::nex::RankingConstants::RankingMode rankingMode = nn::nex::RankingConstants::RANKING_MODE_NEAR;
    // - フレンド範囲ランキング(自分のフレンド内でのランキングを取得します)
    //nn::nex::RankingConstants::RankingMode rankingMode = nn::nex::RankingConstants::RANKING_MODE_FRIEND_RANGE;
    // - フレンド周辺ランキング(フレンドランキングでの自分の周辺のランキングリストを取得します)
    //nn::nex::RankingConstants::RankingMode rankingMode = nn::nex::RankingConstants::RANKING_MODE_FRIEND_NEAR;
    // - 指定したユーザのランキングデータを取得
    //nn::nex::RankingConstants::RankingMode rankingMode = nn::nex::RankingConstants::RANKING_MODE_USER;

    // ランキングの取得結果
    nn::nex::RankingResult rankingResult;

    // ランキングリストを取得する
    nn::nex::ProtocolCallContext oContext;
    pRankingClient->GetRanking(&oContext, rankingMode, RANKING_CATEGORY, orderParam, &rankingResult, s_nexUniqueId);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to GetRanking()\n");
        return false;
    }
    // 結果を出力します
    NEX_DEMO_LOG("GetRanking(RANKING_MODE_RANGE) Result:\n");
    PrintRankDataList(rankingResult.GetRankDataList());
    return true;
}

/*!
    @brief スコアから順位を推定する。

    @param[in] pRankingClient Credentials をバインド済みのオブジェクトを指定してください。
 */
qBool GetApproxOrder(nn::nex::RankingClient *pRankingClient)
{
    BANNER("GetApproxOrder()");
    /*
     *  スコアがアップロードされた場合の順位の推定結果を取得します。
     *
     *  この操作により実際にスコアが登録されることはありません。
     */

    // スコア
    qUnsignedInt32 score = nn::nex::Platform::GetRandomNumber(10000);
    NEX_DEMO_LOG("Score: %d\n", score);

    // 順位計算パラメータの設定
    nn::nex::RankingOrderParam orderParam;
    {
        orderParam.SetOrderCalculation(nn::nex::RankingConstants::ORDER_CALCULATION_113);
        orderParam.SetFilterGroupIndex(nn::nex::RankingConstants::FILTER_GROUP_INDEX_NONE);
        orderParam.SetTimeScope(nn::nex::RankingConstants::TIME_SCOPE_ALL);
    }

    // 推定結果を取得する
    qUnsignedInt32 order = 0;
    nn::nex::ProtocolCallContext oContext;
    pRankingClient->GetApproxOrder(&oContext, RANKING_CATEGORY, orderParam, score, &order, s_nexUniqueId);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to GetApproxOrder()\n");
        return false;
    }
    // 結果を出力します
    NEX_DEMO_LOG("GetApproxOrder() Success. (order=%d)\n", order);
    return true;
}

/*!
    @brief ランキングの統計を取得する。

    @param[in] pRankingClient Credentials をバインド済みのオブジェクトを指定してください。
 */
qBool GetStats(nn::nex::RankingClient *pRankingClient)
{
    BANNER("GetStats()");
    /*
     *  ランキングの統計値(スコアの総数、合計、平均、最小値、最大値)を取得します。
     */

    // 取得する統計値を指定するフラグを用意する
    qUnsignedInt32 flags = (
                               nn::nex::RankingConstants::STATS_FLAG_TOTAL   |
                               nn::nex::RankingConstants::STATS_FLAG_SUM     |
                               nn::nex::RankingConstants::STATS_FLAG_MIN     |
                               nn::nex::RankingConstants::STATS_FLAG_MAX     |
                               nn::nex::RankingConstants::STATS_FLAG_AVERAGE
                           );

    // 順位計算パラメータの設定
    nn::nex::RankingOrderParam orderParam;
    {
        orderParam.SetFilterGroupIndex(nn::nex::RankingConstants::FILTER_GROUP_INDEX_NONE);
        orderParam.SetTimeScope(nn::nex::RankingConstants::TIME_SCOPE_ALL);
    }

    // 統計結果を取得する
    nn::nex::RankingStats stats;
    nn::nex::ProtocolCallContext oContext;
    pRankingClient->GetStats(&oContext, RANKING_CATEGORY, orderParam, flags, &stats);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to GetStats()\n");
        return false;
    }
    // 結果を出力します
    NEX_DEMO_LOG("GetStats Result:\n");
    NEX_DEMO_LOG("** TOTAL: %f\n", stats.GetValue(nn::nex::RankingConstants::STATS_FLAG_TOTAL));
    NEX_DEMO_LOG("** SUM: %f\n", stats.GetValue(nn::nex::RankingConstants::STATS_FLAG_SUM));
    NEX_DEMO_LOG("** MIN: %f\n", stats.GetValue(nn::nex::RankingConstants::STATS_FLAG_MIN));
    NEX_DEMO_LOG("** MAX: %f\n", stats.GetValue(nn::nex::RankingConstants::STATS_FLAG_MAX));
    NEX_DEMO_LOG("** AVERAGE: %f\n", stats.GetValue(nn::nex::RankingConstants::STATS_FLAG_AVERAGE));
    return true;
}

/*!
    @brief スコアの属性値を変更する。

    @param[in] pRankingClient Credentials をバインド済みのオブジェクトを指定してください。
 */
qBool ChangeAttributes(nn::nex::RankingClient *pRankingClient)
{
    BANNER("ChangeAttributes()");
    /*
     *  スコアの属性値(グループ0、グループ1、パラメータ)を変更します。
     */

    // 変更パラメータを作成する
    nn::nex::RankingChangeAttributesParam changeParam;
    {
        // 変更する属性値を指定します。
        // RankingConstants::ModificationFlag の論理和を用いて複数指定できます。
        changeParam.SetModificationFlag(nn::nex::RankingConstants::MODIFICATION_FLAG_GROUP0 |
                                        nn::nex::RankingConstants::MODIFICATION_FLAG_GROUP1 |
                                        nn::nex::RankingConstants::MODIFICATION_FLAG_PARAM);
        // グループ0の値をセットします。
        changeParam.SetGroup0(static_cast<qByte>(nn::nex::Platform::GetRandomNumber(100)));
        // グループ1の値をセットします。
        changeParam.SetGroup1(static_cast<qByte>(nn::nex::Platform::GetRandomNumber(100)));
        // パラメータの値をセットします。
        changeParam.SetParam(static_cast<qUnsignedInt64>(nn::nex::Platform::GetRandomNumber(16)));

        NEX_DEMO_LOG("Change group0=%d, group1=%d, param=%llu\n",
                     changeParam.GetGroup0(), changeParam.GetGroup1(), changeParam.GetParam());
    }

    // 属性値を変更する
    nn::nex::ProtocolCallContext oContext;
    pRankingClient->ChangeAttributes(&oContext, RANKING_CATEGORY, changeParam, s_nexUniqueId);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to ChangeAttributes()\n");
        return false;
    }
    NEX_DEMO_LOG("ChangeAttributes() Success.\n");

    // 変更後の自分のスコアを取得する
    {
        // 順位計算パラメータの設定
        nn::nex::RankingOrderParam orderParam;
        orderParam.SetOrderCalculation(nn::nex::RankingConstants::ORDER_CALCULATION_113);
        orderParam.SetFilterGroupIndex(nn::nex::RankingConstants::FILTER_GROUP_INDEX_NONE);
        orderParam.SetTimeScope(nn::nex::RankingConstants::TIME_SCOPE_ALL);

        // ランキングの取得結果
        nn::nex::RankingResult rankingResult;
        // ランキングモードを自分のランキングデータに指定
        nn::nex::RankingConstants::RankingMode rankingMode = nn::nex::RankingConstants::RANKING_MODE_USER;

        // ランキングリストを取得する
        pRankingClient->GetRanking(&oContext, rankingMode, RANKING_CATEGORY, orderParam, &rankingResult, s_nexUniqueId);
        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to GetRanking()\n");
            return false;
        }
        // 結果を出力します
        NEX_DEMO_LOG("GetRanking(RANKING_MODE_USER) Result:\n");
        PrintRankDataList(rankingResult.GetRankDataList());
    }
    return true;
}

/*!
    @brief スコアを削除する。

    @param[in] pRankingClient Credentials をバインド済みのオブジェクトを指定してください。
 */
qBool DeleteScore(nn::nex::RankingClient *pRankingClient)
{
    BANNER("DeleteScore()");
    /*
     *  スコアを削除します。
     *  カテゴリを指定した場合、指定したカテゴリのスコアだけを削除します。
     *  カテゴリを指定しない場合、全カテゴリの削除を一括削除します。
     */

    // スコアを削除する
    nn::nex::ProtocolCallContext oContext;
    pRankingClient->DeleteScore(&oContext, RANKING_CATEGORY, s_nexUniqueId);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to DeleteScore()\n");
        return false;
    }
    NEX_DEMO_LOG("DeleteScore() Success\n");
    return true;
}

/*!
    @brief スコアを削除する。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void CallRankingApis(nn::nex::NgsFacade* pNgsFacade)
{
    nn::nex::RankingClient rankingClient;
    NEX_DEMO_VERIFY(rankingClient.Bind(pNgsFacade->GetCredentials()));

#ifdef RANKING_DEMO_USE_NEX_UNIQUE_ID
    // NEXユニークIDを取得する
    if (!AcquireNexUniqueId())
    {
        return;
    }
#endif /* RANKING_DEMO_USE_NEX_UNIQUE_ID */

    // ランキング共通データを登録する
    if (!UploadCommonData(&rankingClient))
    {
        return;
    }
    // スコアを登録する
    if (!UploadScore(&rankingClient))
    {
        return;
    }
    // ランキングリスト(TOP10)を取得する
    if (!GetRanking(&rankingClient))
    {
        return;
    }
    // スコアから順位を推定する
    if (!GetApproxOrder(&rankingClient))
    {
        return;
    }
    // ランキングの統計を取得する
    if (!GetStats(&rankingClient))
    {
        return;
    }
    // スコアの属性値を変更する
    if (!ChangeAttributes(&rankingClient))
    {
        return;
    }
    // スコアを削除する
    if (!DeleteScore(&rankingClient))
    {
        return;
    }
    // スコアを再登録する
    if (!UploadScore(&rankingClient))
    {
        return;
    }
}

/*!
    @brief NEXの初期化後にNGSへログインし、ランキングの各種APIを実行します。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            CallRankingApis(&ngsFacade);

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }

    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     Ranking sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
