﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

namespace
{

/*!
    @brief 複数のDataStoreMetaInfoの内容をデバッグログに表示する。

    @param[in] metaInfos 表示するオブジェクトのベクター。
 */
void PrintGetMetaResult(const nn::nex::qVector<nn::nex::DataStoreMetaInfo>& metaInfos)
{
    NEX_DEMO_LOG("+---------+--------------------+------------------------+----------+-----------+\n");
    NEX_DEMO_LOG("| data_id |      owner_id      |         name           |   size   | data_type |\n");
    NEX_DEMO_LOG("+---------+--------------------+------------------------+----------+-----------+\n");

    nn::nex::qVector<nn::nex::DataStoreMetaInfo>::const_iterator it;
    for (it = metaInfos.begin(); it != metaInfos.end(); ++it)
    {
        if(!it->IsValid())
        {
            // 不正なものは表示しない
            continue;
        }
        NEX_DEMO_LOG("|%9llu|%20" NEX_PID_FORMAT_SPECIFIER "|%-24" STRING_FORMAT_SPECIFIER "|%10u|%11u|\n",
                     it->GetDataId(),
                     it->GetOwnerId(),
                     it->GetName().GetBufferPtr(),
                     it->GetSize(),
                     it->GetDataType()
                    );
    }
    NEX_DEMO_LOG("+---------+--------------------+------------------------+----------+-----------+\n");
}

/*!
    @brief DataStoreMetaInfoの内容をデバッグログに表示する。

    @param[in] metaInfo 表示するオブジェクト。
 */
void PrintGetMetaResult(const nn::nex::DataStoreMetaInfo& metaInfo)
{
    nn::nex::qVector<nn::nex::DataStoreMetaInfo> metaInfos;
    metaInfos.push_back(metaInfo);
    PrintGetMetaResult(metaInfos);
}

/*!
    @brief DataStoreSearchResultの内容をデバッグログに表示する。

    @param[in] searchResult 表示するオブジェクト。
 */
void PrintSearchResult(const nn::nex::DataStoreSearchResult& searchResult)
{
    NEX_DEMO_LOG("SearchResult::totalCount = %u\n", searchResult.totalCount);
    PrintGetMetaResult(searchResult.GetResult());
}

/*!
    @brief PostObjectで使用するイベントリスナー。
 */
class MyDataStorePostObjectEventListener : public nn::nex::DataStorePostObjectEventListener
{
public:
    /*!
        @brief コンストラクタ。

        @param[in] buffer 送信するバッファ。バッファは内部にコピーされます。
        @param[in] size 送信するバッファのサイズ。
     */
    MyDataStorePostObjectEventListener(const void* buffer, qUnsignedInt32 size)
        : m_ReadBuffer()
    {
        const qByte* bytePtr = static_cast<const nn::nex::qByte*>(buffer);
        m_ReadBuffer.assign(bytePtr, bytePtr + size);
    }

    /*!
        @brief データを送信する際にライブラリ内部から呼び出されるコールバック関数。

        @param[out] buffer 送信するデータを書き込むバッファ。
        @param[in] bufferSize bufferに書き込めるサイズ。
        @param[out] completed 送信が完了した際にtrueを代入するフラグ。

        @return bufferに書き込んだサイズ。
     */
    virtual size_t PostChunkedBuffer(void* buffer, size_t bufferSize, qBool* completed)
    {
        size_t copySize = (std::min)(m_ReadBuffer.size(), bufferSize);
        std::copy(m_ReadBuffer.begin(), m_ReadBuffer.begin() + copySize, static_cast<nn::nex::qByte*>(buffer));
        m_ReadBuffer.erase(m_ReadBuffer.begin(), m_ReadBuffer.begin() + copySize);
        if(m_ReadBuffer.empty())
        {
            *completed = true;
        }
        NEX_DEMO_LOG("Sending: " NEX_SIZE_T_FORMAT " bytes.\n", copySize);
        return copySize;
    };
private:
    nn::nex::qVector<nn::nex::qByte> m_ReadBuffer;
};

/*!
    @brief GetObjectで使用するイベントリスナー。
 */
class MyDataStoreGetObjectEventListener : public nn::nex::DataStoreGetObjectEventListener
{
public:
    /*!
        @brief データの受信が開始される際にライブラリ内部から呼び出されるコールバック関数。

        @param[in] dataId データに割り当てられたデータID。
        @param[in] size 受信するデータのサイズ。
     */
    virtual void Begin(qUnsignedInt64 dataId, size_t size)
    {
        (void)dataId;
        (void)size;
    }

    /*!
        @brief データを受信した際にライブラリ内部から呼び出されるコールバック関数。

        @param[in] buffer 受信したバッファ。
        @param[in] size bufferのサイズ。
     */
    virtual qBool ProcessResponse(const void* buffer, size_t bufferSize)
    {
        const nn::nex::qByte* byteArray = static_cast<const nn::nex::qByte*>(buffer);
        m_ReceivedBuffer.insert(m_ReceivedBuffer.end(), byteArray, byteArray + bufferSize);
        return true;
    }
private:
    nn::nex::qVector<nn::nex::qByte> m_ReceivedBuffer;
};

/*!
    @brief DataStoreの各種APIを呼び出すクラスです。
 */
class DataStoreSample
{
public:
    /*!
        @brief コンストラクタです。

        @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
     */
    DataStoreSample(nn::nex::NgsFacade* pNgsFacade)
        : m_dataStoreClient()
    {
        m_dataStoreClient.Bind(pNgsFacade->GetCredentials());

        // HTTP通信スレッドの優先度を上げる。通信スレッドの優先度はメインスレッドより高くすることを推奨します。
        m_dataStoreClient.SetHttpThreadPriority(10);
    }
    void CallVariousApi();

private:
    qBool PostObjectBuffered(qUnsignedInt64 *pDataId);
    qBool PostObjectEventListener(qUnsignedInt64 *pDataId);
    qBool PostObjectComplex(qUnsignedInt64 *pDataId);
    qBool GetMetaInfo(qUnsignedInt64 dataId);
    qBool GetMetaInfos(const nn::nex::qVector<nn::nex::qUnsignedInt64>& dataIds);
    qBool GetObjectBuffered(qUnsignedInt64 dataId);
    qBool GetObjectEventListener(qUnsignedInt64 dataId);
    qBool ChangeMetaInfo(qUnsignedInt64 dataId);
    qBool UpdateObject(qUnsignedInt64 dataId);
    qBool SearchObject();
    qBool RateObject(qUnsignedInt64 dataId);
    qBool GetRating(qUnsignedInt64 dataId);
    qBool ResetRating(qUnsignedInt64 dataId);
    qBool DeleteObject(qUnsignedInt64 dataId);
    qBool PerpetuateObject(qUnsignedInt64 dataId);
    qBool GetPersistenceInfo();
    qBool UnperpetuateObject();
    qBool GetPasswordInfo(qUnsignedInt64 dataId);
    qBool AtomicPost();
    qBool ParallelPostAndGet();

private:
    nn::nex::DataStoreClient m_dataStoreClient;
    static const qUnsignedInt32 POST_DATA_SIZE = 200 * 1024;
    static const qUnsignedInt32 METABINARY_SIZE = 1024;
};

/*!
    @brief データをアップロードします。

    送信データは一度に指定します。大きいデータをアップロードするとその分のメモリを消費します。
    メモリを消費しない方法は DataStoreSample::PostObjectEventListener() を参照してください。

    @param[out] pDataId アップロードされたデータのデータIDを書き込むポインタ。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::PostObjectBuffered(qUnsignedInt64* pDataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStorePostParam postParam;
    void* data = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;

    // falseにするとdataがNEXの内部バッファにコピーされないため非同期処理が終わるまでアプリケーション側でバッファを保持する必要があります。
    nn::nex::qBool useInternalBuffer = false;

    postParam.SetSize(POST_DATA_SIZE);
    memset(data, 0xaa, POST_DATA_SIZE);

    m_dataStoreClient.PostObject(&oContext, postParam, pDataId, data, useInternalBuffer);
    oContext.Wait();
    std::free(data);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("PostObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("PostObjectBuffered succeeded. DataID: %llu\n", *pDataId);

    return true;
}

/*!
    @brief データをアップロードします。

    送信データは少しずつイベントリスナーから指定します。大きいデータをアップロードするのに適しています。

    @param[out] pDataId アップロードされたデータのデータIDを書き込むポインタ。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::PostObjectEventListener(qUnsignedInt64* pDataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStorePostParam postParam;
    void* data = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;

    postParam.SetSize(POST_DATA_SIZE);
    memset(data, 0xaa, POST_DATA_SIZE);
    MyDataStorePostObjectEventListener myDataStorePostObjectEventListener(data, POST_DATA_SIZE);

    m_dataStoreClient.PostObject(&oContext, postParam, pDataId, &myDataStorePostObjectEventListener);
    oContext.Wait();
    std::free(data);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("PostObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("PostObjectEventListener succeeded. DataID: %llu\n", *pDataId);

    return true;
}

/*!
    @brief データをアップロードします。

    送信データに様々なメタデータを付加する例です。

    @param[out] pDataId アップロードされたデータのデータIDを書き込むポインタ。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::PostObjectComplex(qUnsignedInt64 *pDataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStorePostParam postParam;
    nn::nex::DataStoreRatingInitParam ratingInitParam;
    void* data = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;
    nn::nex::qVector<nn::nex::String> tags;
    nn::nex::qMap<nn::nex::qInt8, nn::nex::DataStoreRatingInitParam> ratings;
    nn::nex::qByte metaBinary[METABINARY_SIZE];

    // falseにするとdataがNEXの内部バッファにコピーされないため非同期処理が終わるまでアプリケーション側でバッファを保持する必要があります。
    nn::nex::qBool useInternalBuffer = false;

    tags.push_back("tagvalue1");
    tags.push_back("tagvalue2");
    tags.push_back("tagvalue3");
    for (nn::nex::qInt8 i = 0; i < 3; ++i)
    {
        ratings.insert(std::make_pair(i, ratingInitParam));
    }
    memset(metaBinary, 0xbb, METABINARY_SIZE);

    postParam.SetSize(POST_DATA_SIZE);
    postParam.SetName("sample name");
    postParam.SetDataType(9999);
    postParam.SetTags(tags);
    postParam.SetRatingSetting(ratings);
    postParam.SetMetaBinary(metaBinary, METABINARY_SIZE);

    memset(data, 0xaa, POST_DATA_SIZE);

    m_dataStoreClient.PostObject(&oContext, postParam, pDataId, data, useInternalBuffer);
    oContext.Wait();
    std::free(data);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("PostObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("PostObjectComplex succeeded. DataID: %llu\n", *pDataId);

    return true;
}

/*!
    @brief データに付加されたメタ情報を取得します。

    @param[in] dataId メタデータを取得するデータのデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::GetMetaInfo(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreGetMetaParam getmetaParam;
    nn::nex::DataStoreMetaInfo metaInfo;
    nn::nex::qResult result;

    getmetaParam.SetDataId(dataId);

    m_dataStoreClient.GetMeta(&oContext, getmetaParam, &metaInfo);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetMetaInfo failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("GetMetaInfo succeeded.\n");

    PrintGetMetaResult(metaInfo);

    return true;
}

/*!
    @brief データに付加されたメタ情報を一度に複数個取得します。

    @param[in] dataId メタデータを取得するデータのデータIDのベクター。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::GetMetaInfos(const nn::nex::qVector<nn::nex::qUnsignedInt64>& dataIds)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreGetMetaParam getmetaParam;
    nn::nex::qVector<nn::nex::DataStoreMetaInfo> metaInfos;
    nn::nex::qVector<nn::nex::qResult> results;
    nn::nex::qResult result;

    // データIDを複数指定するオーバーロード関数の場合、DataStoreGetMetaParam::SetDataId()で指定したデータIDは無視されます。
    m_dataStoreClient.GetMeta(&oContext, dataIds, getmetaParam, &metaInfos, &results);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetMetaInfo failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("GetMetaInfos succeeded.\n");

    // 個々のデータの取得に成功したか確認
    for(qUnsignedInt32 i=0; i<dataIds.size(); i++)
    {
        if(!results.at(i))
        {
            NEX_DEMO_LOG("GetMetaInfos partially failed. DataId: %llu\n", dataIds.at(i));
            nex_demo_util::ReportQResultReturnCode(results.at(i));
        }
    }

    PrintGetMetaResult(metaInfos);

    return true;
}

/*!
    @brief データをダウンロードします。

    取得するデータを保存するメモリは事前に確保しておく必要があります。
    事前に確保しない方法については DataStoreSample::GetObjectEventListener() を参照してください。
   
    @param[in] dataId 取得するデータのデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::GetObjectBuffered(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreGetParam getParam;
    void* buf = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;
    size_t writtenSize = 0;

    getParam.SetDataId(dataId);
    m_dataStoreClient.GetObject(&oContext, getParam, buf, POST_DATA_SIZE, &writtenSize);
    oContext.Wait();
    std::free(buf);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("GetObjectBuffered succeeded.\n");

    return true;
}

/*!
    @brief データをダウンロードします。

    取得するデータはイベントリスナーから分割して受け取ることができます。
   
    @param[in] dataId 取得するデータのデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::GetObjectEventListener(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreGetParam getParam;
    nn::nex::qResult result;
    MyDataStoreGetObjectEventListener myDataStoreGetObjectEventListener;

    getParam.SetDataId(dataId);
    m_dataStoreClient.GetObject(&oContext, getParam, &myDataStoreGetObjectEventListener);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("GetObjectEventListener succeeded.\n");

    return true;
}

/*!
    @brief データのメタ情報を変更します。

    @param[in] dataId メタ情報を変更するデータのデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::ChangeMetaInfo(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreChangeMetaParam changemetaParam;
    nn::nex::qResult result;
    nn::nex::qVector<nn::nex::String> tags;
    nn::nex::qByte metaBinary[METABINARY_SIZE];

    tags.push_back("tagvalue4");
    tags.push_back("tagvalue5");
    tags.push_back("tagvalue6");
    memset(metaBinary, 0xdd, METABINARY_SIZE);

    changemetaParam.SetDataId(dataId);
    changemetaParam.SetModificationFlag(
        nn::nex::DataStoreConstants::MODIFICATION_FLAG_NAME
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_ACCESS_PERMISSION
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_UPDATE_PERMISSION
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_PERIOD
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_METABINARY
        | nn::nex::DataStoreConstants::MODIFICATION_FLAG_TAGS);
    changemetaParam.SetName("modified name");
    changemetaParam.SetAccessPermission(nn::nex::DataStorePermission(nn::nex::DataStoreConstants::PERMISSION_PUBLIC));
    changemetaParam.SetUpdatePermission(nn::nex::DataStorePermission(nn::nex::DataStoreConstants::PERMISSION_PRIVATE));
    changemetaParam.SetPeriod(100);
    changemetaParam.SetMetaBinary(metaBinary, METABINARY_SIZE);
    changemetaParam.SetTags(tags);

    m_dataStoreClient.ChangeMeta(&oContext, changemetaParam);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("ChangeMetaInfo failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("ChangeMetaInfo succeeded.\n");

    return true;
}

/*!
    @brief データを更新します。

    @param[in] dataId 変更するデータのデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::UpdateObject(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreUpdateParam updateParam;
    void* data = std::malloc(POST_DATA_SIZE);
    nn::nex::qResult result;

    updateParam.SetDataId(dataId);
    updateParam.SetSize(POST_DATA_SIZE);

    memset(data, 0xcc, POST_DATA_SIZE);

    m_dataStoreClient.UpdateObject(&oContext, updateParam, data, false);
    oContext.Wait();
    std::free(data);
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("UpdateObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("UpdateObject succeeded.\n");

    return true;
}

/*!
    @brief データを検索します。

    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::SearchObject()
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreSearchParam searchParam;
    nn::nex::DataStoreSearchResult searchResult;
    nn::nex::qResult result;

    m_dataStoreClient.SearchObject(&oContext, searchParam, &searchResult);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("SearchObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("SearchObject succeeded.\n");

    PrintSearchResult(searchResult);

    return true;
}

/*!
    @brief データを評価します。

    @param[in] dataId 評価対象のデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::RateObject(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreRatingTarget target;
    nn::nex::DataStoreRateObjectParam rateParam;
    nn::nex::qResult result;

    target.SetDataId(dataId);
    target.SetSlot(0);

    m_dataStoreClient.RateObject(&oContext, target, rateParam, NULL);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("RateObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("RateObject succeeded.\n");

    return true;
}

/*!
    @brief データの評価を取得します。

    @param[in] dataId 評価を取得する対象のデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::GetRating(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreRatingTarget target;
    nn::nex::DataStoreRatingInfo ratingInfo;
    nn::nex::qResult result;

    target.SetDataId(dataId);
    target.SetSlot(0);

    m_dataStoreClient.GetRating(&oContext, target, &ratingInfo);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetRating failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("GetRating succeeded.\n");

    NEX_DEMO_LOG("ratingInfo::GetTotalValue() = %lld\n", ratingInfo.GetTotalValue());
    NEX_DEMO_LOG("ratingInfo::GetCount() = %u\n", ratingInfo.GetCount());

    return true;
}

/*!
    @brief データの評価を初期化します。

    @param[in] dataId 評価を初期化する対象のデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::ResetRating(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreRatingTarget target;
    nn::nex::qResult result;

    target.SetDataId(dataId);
    target.SetSlot(0);

    m_dataStoreClient.ResetRating(&oContext, target);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("ResetRating failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("ResetRating succeeded.\n");

    return true;
}

/*!
    @brief データを削除します。

    @param[in] dataId 削除する対象のデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::DeleteObject(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DataStoreDeleteParam deleteParam;
    nn::nex::qResult result;

    deleteParam.SetDataId(dataId);

    m_dataStoreClient.DeleteObject(&oContext, deleteParam);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("DeleteObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("DeleteObject succeeded.\n");

    return true;
}

/*!
    @brief データを永続化します。

    @param[in] dataId 永続化する対象のデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::PerpetuateObject(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qResult result;
    nn::nex::qUnsignedInt16 persistenceSlotId = 0;
    nn::nex::qBool deleteLastObject = false;

    m_dataStoreClient.PerpetuateObject(&oContext, persistenceSlotId, dataId, deleteLastObject);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("PerpetuateObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("PerpetuateObject succeeded.\n");

    return true;
}

/*!
    @brief 永続化情報を取得します。

    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::GetPersistenceInfo()
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qResult result;
    nn::nex::qUnsignedInt16 persistenceSlotId = 0;
    nn::nex::DataStorePersistenceInfo persistenceInfo;

    m_dataStoreClient.GetPersistenceInfo(&oContext, m_dataStoreClient.GetCredentials()->GetPrincipalID(), persistenceSlotId, &persistenceInfo);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetPersistenceInfo failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("GetPersistenceInfo succeeded.\n");

    NEX_DEMO_LOG("persistenceInfo.GetPrincipalId() = " NEX_PID_FORMAT "\n", persistenceInfo.GetPrincipalId());
    NEX_DEMO_LOG("persistenceInfo.GetPersistenceSlotId() = %u\n", persistenceInfo.GetPersistenceSlotId());
    NEX_DEMO_LOG("persistenceInfo.GetDataId() = %llu\n", persistenceInfo.GetDataId());

    return true;
}

/*!
    @brief データを非永続化します。

    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::UnperpetuateObject()
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qResult result;
    nn::nex::qUnsignedInt16 persistenceSlotId = 0;
    nn::nex::qBool deleteLastObject = false;

    m_dataStoreClient.UnperpetuateObject(&oContext, persistenceSlotId, deleteLastObject);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("UnperpetuateObject failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("UnperpetuateObject succeeded.\n");

    return true;
}

/*!
    @brief データのパスワードを取得します。

    @param[in] dataId パスワードを取得する対象のデータID。
    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::GetPasswordInfo(qUnsignedInt64 dataId)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qResult result;
    nn::nex::DataStorePasswordInfo passwordInfo;

    m_dataStoreClient.GetPasswordInfo(&oContext, dataId, &passwordInfo);
    oContext.Wait();
    result = oContext.GetOutcome();
    if (!result)
    {
        NEX_DEMO_LOG("GetPasswordInfo failed.\n");
        nex_demo_util::ReportQResultReturnCode(result);
        return false;
    }
    NEX_DEMO_LOG("GetPasswordInfo succeeded.\n");

    NEX_DEMO_LOG("passwordInfo.GetDataId() = %llu\n", passwordInfo.GetDataId());
    NEX_DEMO_LOG("passwordInfo.GetAccessPassword() = %llu\n", passwordInfo.GetAccessPassword());
    NEX_DEMO_LOG("passwordInfo.GetUpdatePassword() = %llu\n", passwordInfo.GetUpdatePassword());

    return true;
}

/*!
    @brief 複数のデータを一度にアップロードします。

    アップロードした複数のデータはアトミックに有効化されます。

    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::AtomicPost()
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::qVector<qUnsignedInt64> dataIds;

    // 三つアップロードする
    // ここでは一つずつアップロードしていますが、並列にアップロードすることも可能です。
    for(qUnsignedInt32 i=0; i<3; i++)
    {
        nn::nex::DataStorePostParam postParam;
        qUnsignedInt64 dataId = nn::nex::DataStoreConstants::INVALID_DATAID;
        void* data = std::malloc(POST_DATA_SIZE);
        nn::nex::qResult result;

        // falseにするとdataがNEXの内部バッファにコピーされないため非同期処理が終わるまでアプリケーション側でバッファを保持する必要があります。
        nn::nex::qBool useInternalBuffer = false;

        // DataStoreConstants::NEED_COMPLETION を指定するとデータが無効な状態でサーバに登録され、
        // DataStoreClient::CompleteSuspendedPostObject() 呼び出し時にアトミックに有効化されます。
        postParam.SetDataFlag(nn::nex::DataStoreConstants::DATA_FLAG_NEED_COMPLETION);

        postParam.SetSize(POST_DATA_SIZE);
        memset(data, 0xaa, POST_DATA_SIZE);

        m_dataStoreClient.PostObject(&oContext, postParam, &dataId, data, useInternalBuffer);
        oContext.Wait();
        std::free(data);
        result = oContext.GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("PostObject failed.\n");
            nex_demo_util::ReportQResultReturnCode(result);
            return false;
        }
        dataIds.push_back(dataId);
    }

    oContext.Reset();

    // アップロードしたデータの有効化
    {
        nn::nex::qResult result;
        m_dataStoreClient.CompleteSuspendedPostObject(&oContext, dataIds);
        oContext.Wait();
        result = oContext.GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("CompleteSuspendedPostObject failed.\n");
            nex_demo_util::ReportQResultReturnCode(result);
            return false;
        }
    }
    NEX_DEMO_LOG("AtomicPost succeeded.");

    return true;
}

/*!
    @brief 複数のデータを並列にアップロードしたりとダウンロードしたりする例です。

    @return 処理が成功したかどうか。
 */
qBool DataStoreSample::ParallelPostAndGet()
{
    const qUnsignedInt64 numParallel = 3;
    nn::nex::ProtocolCallContext oContexts[numParallel];
    qUnsignedInt64 dataIds[numParallel];

    // 並列にアップロードする
    // 並列に処理することで通信のオーバーヘッドを隠蔽でき、全体の処理時間を短縮できます。
    // DataStoreの全ての非同期処理は並列に実行可能ですが、
    // HTTP通信を伴う処理 (DataStoreClient::GetObject(), DataStoreClient::PostObject(), DataStoreClient::UpdateObject) は最大3並列までにしてください。
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        nn::nex::DataStorePostParam postParam;
        void* data = std::malloc(POST_DATA_SIZE);
        nn::nex::qBool useInternalBuffer = true;

        postParam.SetSize(POST_DATA_SIZE);
        memset(data, 0xaa, POST_DATA_SIZE);

        m_dataStoreClient.PostObject(&oContexts[i], postParam, &dataIds[i], data, useInternalBuffer);

        // ここで CallContext::Wait() を呼び出さず、連続で DataStoreClient::PostObject() を呼び出します。

        // useInternalBufferがtrueなのですぐ解放可能です。
        std::free(data);
    }

    // 全ての非同期処理が完了するまで待つ
    // 通常のケースと同様に CallContext::Wait() を呼び出さずに非同期的に処理することも可能です。
    // (例えば毎フレームScheduler::Dispatch() を呼び出し CallContext::GetStatus() で完了を確認する等)
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        oContexts[i].Wait();
    }

    // 非同期処理の結果を確認
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        nn::nex::qResult result = oContexts[i].GetOutcome();
        if (!result)
        {
            NEX_DEMO_LOG("PostObject failed.\n");
            nex_demo_util::ReportQResultReturnCode(result);
            return false;
        }

        // CallContext を使い回す場合は CallContext::Reset() を呼び出してください。
        oContexts[i].Reset();
    }

    void* buffers[numParallel];
    size_t writtenSizes[numParallel];

    // 並列にダウンロードする
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        nn::nex::DataStoreGetParam getParam;
        nn::nex::qResult result;
        buffers[i] = std::malloc(POST_DATA_SIZE);
        writtenSizes[i] = 0;

        getParam.SetDataId(dataIds[i]);
        m_dataStoreClient.GetObject(&oContexts[i], getParam, buffers[i], POST_DATA_SIZE, &writtenSizes[i]);

        // ここで CallContext::Wait() を呼び出さず、連続で DataStoreClient::PostObject() を呼び出します。
    }

    // 全ての非同期処理が完了するまで待つ
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        oContexts[i].Wait();
    }

    // 非同期処理の結果を確認
    for(qUnsignedInt32 i=0; i<numParallel; i++)
    {
        nn::nex::qResult result = oContexts[i].GetOutcome();
        std::free(buffers[i]);
        if (!result)
        {
            NEX_DEMO_LOG("GetObject failed.\n");
            nex_demo_util::ReportQResultReturnCode(result);
            return false;
        }
    }
    NEX_DEMO_LOG("ParallelPostAndGet succeeded.");

    return true;
}

/*!
    @brief データストアの様々なAPIを呼び出します。
 */
void DataStoreSample::CallVariousApi()
{
    qUnsignedInt64 dataId = nn::nex::DataStoreConstants::INVALID_DATAID;
    nn::nex::qVector<nn::nex::qUnsignedInt64> dataIds;

    // データをアップロードする。(バッファ使用)
    if (!PostObjectBuffered(&dataId))
    {
        return;
    }
    dataIds.push_back(dataId);

    // データをアップロードする。(イベントリスナー使用)
    if (!PostObjectEventListener(&dataId))
    {
        return;
    }
    dataIds.push_back(dataId);

    // 複雑なデータをアップロードする。
    if (!PostObjectComplex(&dataId))
    {
        return;
    }
    dataIds.push_back(dataId);

    // データのメタ情報を取得する。
    if (!GetMetaInfo(dataId))
    {
        return;
    }

    // データを取得する。(バッファ使用)
    if (!GetObjectBuffered(dataId))
    {
        return;
    }

    // データを取得する。(イベントリスナー使用)
    if (!GetObjectEventListener(dataId))
    {
        return;
    }

    // データのメタ情報を変更する。
    if (!ChangeMetaInfo(dataId))
    {
        return;
    }

    // データのメタ情報を取得する。
    if (!GetMetaInfo(dataId))
    {
        return;
    }

    // 複数のデータのメタ情報を取得する。
    if (!GetMetaInfos(dataIds))
    {
        return;
    }

    // データを更新する。
    if (!UpdateObject(dataId))
    {
        return;
    }

    // データを取得する。(バッファ使用)
    if (!GetObjectBuffered(dataId))
    {
        return;
    }

    // データを検索する
    if (!SearchObject())
    {
        return;
    }

    // データを評価する
    if (!RateObject(dataId))
    {
        return;
    }

    // データの評価を取得する
    if (!GetRating(dataId))
    {
        return;
    }

    // データの評価をリセットする
    if (!ResetRating(dataId))
    {
        return;
    }

    // データを永続化する
    if (!PerpetuateObject(dataId))
    {
        return;
    }

    // 永続化情報を取得する
    if (!GetPersistenceInfo())
    {
        return;
    }

    // データを非永続化する
    if (!UnperpetuateObject())
    {
        return;
    }

    // パスワード情報を取得する
    if (!GetPasswordInfo(dataId))
    {
        return;
    }

    // データを削除する
    if (!DeleteObject(dataId))
    {
        return;
    }

    // 一括でアップロードする
    if (!AtomicPost())
    {
        return;
    }

    // 並列にアップロード、ダウンロードする
    if (!ParallelPostAndGet())
    {
        return;
    }
    NEX_DEMO_LOG("CallVariousApi succeeded.\n");
}


/*!
    @brief NEXの初期化後にNGSへログインし、DataStoreの各種APIを実行します。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // HTTPライブラリの初期化
    // DataStoreでファイルサーバーを使用する際はHTTPライブラリの初期化が必要です。
    if (!nex_demo_util::InitializeHttpLibrary())
    {
        NEX_DEMO_LOG("nex_demo_util::InitializeHttpLibrary failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::EventLog::GetInstance()->GetOutputFormat()->ShowDateTime(true); // EventLogに日付を追加
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);
    nn::nex::TraceLog::GetInstance()->GetOutputFormat()->ShowDateTime(true); // TraceLogに日付を追加

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            // DataStoreの各種APIを呼び出す。
            DataStoreSample sample(&ngsFacade);
            sample.CallVariousApi();

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }

    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // HTTPライブラリの終了処理。
    nex_demo_util::FinalizeHttpLibrary();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     DataStore sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
