﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

/*
 * NEX Sample
 *
 * Subscriber
 *
 * Description:
 *
 *  Subscriber のサンプルです。
 *  サンプルに実装されているシーケンスは以下の通りです。
 *
 * 1. NGSにログイン
 * 2. Subscriberの一連の処理を実行 (詳細はSubscriberSampleクラスを参照)
 * 3. NGSからログアウト
 *
 */

#include <algorithm>
#include <functional>

#include <nex.h>
#include <NexDemoUtil/NexUtil.h>

namespace
{

/*!
    @brief SubscriberContentの内容をデバッグログに表示する。

    @param[in] contents 表示するオブジェクトのベクター。
 */
void PrintContents(const nn::nex::qVector<nn::nex::SubscriberContent>& contents)
{
    nn::nex::StringStream ss;
    for (nn::nex::qVector<nn::nex::SubscriberContent>::const_iterator it = contents.begin(); it != contents.end(); ++it)
    {
        // operator<<(StringStream, SubscriberContent) は内部情報を文字列としてStringStreamに渡す関数です。
        // SubscriberContent::Trace()とは異なりリリースビルドでも利用できます。
        ss << *it << nn::nex::String("\n");
    }
    NEX_DEMO_LOG(STRING_FORMAT, ss.CStr());
}

/*!
    @brief Subscriber の各種APIを実行するクラス
 */
class SubscriberSample
{
public:
    SubscriberSample(nn::nex::NgsFacade* pNgsFacade)
        : m_pNgsFacade(pNgsFacade)
        , m_subscriberClient()
    {
        nn::nex::Credentials* pCred = m_pNgsFacade->GetCredentials();
        NEX_DEMO_ASSERT(pCred != NULL);

        // Bind()処理はpCredがNULLでない場合は必ず成功する
        NEX_DEMO_VERIFY(m_subscriberClient.Bind(pCred));

        // 本サンプルでは予約済みトピック番号0,1のトピックを使用する
        m_topics.push_back(nn::nex::SubscriberUtil::ReservedTopicNumToTopicString(0));
        m_topics.push_back(nn::nex::SubscriberUtil::ReservedTopicNumToTopicString(1));
    }
    void CallSubscriberApis();

private:

    qBool PostContent();
    qBool GetContent();

    nn::nex::qVector<nn::nex::String> m_topics;               // トピックのリスト
    nn::nex::NgsFacade* m_pNgsFacade;                         // ログイン済みのNgsFacadeオブジェクトへのポインタ
    nn::nex::SubscriberClient m_subscriberClient;             // サブスクライバー関連のAPI呼び出しに使用するオブジェクトへのポインタ
};

/*!
    @brief トピックへのコンテンツの投稿を行う。

    @return 処理が成功したかどうか。
 */
qBool SubscriberSample::PostContent()
{
    nn::nex::SubscriberPostContentParam param;
    param.SetTopic(m_topics);
    param.SetMessage("message");
    param.SetBinary("binary", sizeof("binary"));

    nn::nex::ProtocolCallContext oContext;
    nn::nex::qUnsignedInt64 contentId;
    m_subscriberClient.PostContent(&oContext, param, &contentId);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to PutScore()\n");
        return false;
    }
    NEX_DEMO_LOG("PostContent() succeeded. ContentId: %llu\n", contentId);
    return true;
}

/*!
    @brief トピックからのコンテンツの取得を行う。

    @return 処理が成功したかどうか。
 */
qBool SubscriberSample::GetContent()
{
    nn::nex::SubscriberGetContentParam param;
    param.SetTopic(m_topics[0]);
    nn::nex::qVector<nn::nex::SubscriberContent> contents;

    nn::nex::ProtocolCallContext oContext;
    m_subscriberClient.GetContent(&oContext, param, &contents);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to PutScore()\n");
        return false;
    }
    PrintContents(contents);
    return true;
}

/*!
    サブスクライバーの各種APIを呼び出します。
 */
void SubscriberSample::CallSubscriberApis()
{
    if (!PostContent())
    {
        return;
    }
    if (!GetContent())
    {
        return;
    }
}

/*!
    @brief NEXの初期化後にNGSへログインし、サブスクライバーのAPIを呼び出します。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            {
                // サブスクライバーのAPI呼び出し
                SubscriberSample sample(&ngsFacade);
                sample.CallSubscriberApis();
            }

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }
    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     Subscriber sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
