﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

/*
 * NEX Sample
 *
 * Debug
 *
 * Description:
 *
 * デバッグ補助機能のサンプルです
 *
 * DebugClient を使用して、ゲームサーバ側のAPI呼び出し履歴の記録機能を有効化し、定期的に履歴の取得・出力を行います。
 * 本サンプルでは、DebugClient::GetApiCalls() に履歴を取得する範囲を時刻で指定して、同じ履歴を重複して出力しないようにしています。
 *
 * API呼び出し履歴の記録機能の有効・無効の状態はゲームサーバ全体の設定値となります。
 * DebugClient::EnableApiRecorder(), DebugClient::DisableApiRecorder() の呼び出し時には注意してください。
 * 特に、サンプルデモサーバを利用している場合は、DebugClient::DisableApiRecorder()を呼び出さないようにしてください。
 *
 */

#include <nex.h>
#include <RendezVous/Services/Jugem/Debug/src/Client/DebugClient.h> // nex.h からはincludeされない
#include <NexDemoUtil/NexUtil.h>

namespace
{
// ログを表示し続ける時間(ミリ秒)
const nn::nex::TimeInterval API_LOG_TIME = 10 * 60 * 1000;

// ログを取得する間隔(ミリ秒)
const nn::nex::TimeInterval API_LOG_INTERVAL = 30 * 1000;

/*!
    @brief NGSのAPI呼び出し記録の収集をしばらく行います。

    LOG_TIME の間、API_LOG_INTERVAL の間隔でAPI呼び出しの記録を取得・表示します。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
    @param[in] pClient Credentials をバインド済みのオブジェクトを指定してください。
 */
void PrintApiRecordLoop(nn::nex::NgsFacade *pNgsFacade, nn::nex::DebugClient* pClient)
{
    nn::nex::ProtocolCallContext oContext;

    NEX_DEMO_LOG("--------API CALL LOG START----------\n");

    // 現在時刻から取得する。全ての履歴を取得したい場合は DateTime::Never を指定
    nn::nex::DateTime since;
    pNgsFacade->GetServerTime(&since);

    nn::nex::Time startTime = nn::nex::SystemClock::GetTime();
    nn::nex::Time prevCallTime = nn::nex::SystemClock::GetTime();
    nn::nex::qSet<nn::nex::PrincipalID> callerSet; // 一度でもAPIを呼んだクライアントの集合

    while (true)
    {
        nn::nex::Platform::Sleep(nex_demo_util::SLEEP_DURATION);
        nn::nex::Scheduler::GetInstance()->DispatchAll();

        nn::nex::Time currentTime = nn::nex::SystemClock::GetTime();

        if (currentTime - startTime >= API_LOG_TIME)
        {
            break;
        }

        if (currentTime - prevCallTime < API_LOG_INTERVAL)
        {
            continue;
        }

        nn::nex::qList<nn::nex::ApiCallData> apiCallDataList;
        nn::nex::qList<nn::nex::PrincipalID> principalIdList; // 指定しない場合、全員の呼び出し履歴を取得

        // 取得する
        pClient->GetApiCalls(&oContext, &apiCallDataList, principalIdList, since);
        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to GetApiCalls.\n");
            return;
        }

        // リストは呼び出し順にソートされていることが保証される
        for(nn::nex::qList<nn::nex::ApiCallData>::iterator it = apiCallDataList.begin(); it != apiCallDataList.end(); ++it)
        {
            NEX_DEMO_LOG("%02d:%02d:%02d " NEX_PID_FORMAT " " STRING_FORMAT "\n", it->GetCalledTime().GetHour(),
                         it->GetCalledTime().GetMinute(),
                         it->GetCalledTime().GetSecond(),
                         it->GetPrincipalId(), it->GetMethodName().CStr());

            // APIを呼んだクライアントのPIDを覚えておく
            callerSet.insert(it->GetPrincipalId());
        }

        // 取得した履歴の最後の時刻を次の呼び出しのsinceに指定することで、重複なく履歴を取得することができる。
        if(apiCallDataList.size() > 0)
        {
            since = apiCallDataList.back().GetCalledTime();
        }

        prevCallTime = currentTime;
    }

    NEX_DEMO_LOG("--------API CALL LOG END----------\n");

    NEX_DEMO_LOG("--------PRINT API CALL SUMMARY START----------\n");

    // 一度でも履歴に残っているクライアントの呼び出し履歴を集計して表示する
    for(nn::nex::qSet<nn::nex::PrincipalID>::iterator itPid = callerSet.begin(); itPid != callerSet.end(); ++itPid)
    {
        nn::nex::qList<nn::nex::ApiCallSummary> summary;

        // ガイドラインの呼び出し頻度を超えるもののみ取得(全期間)
        pClient->GetApiCallSummary(&oContext, &summary, *itPid, nn::nex::DateTime::Never, nn::nex::DateTime::Never, true);
        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to GetApiCalls.\n");
            return;
        }

        NEX_DEMO_LOG("-- PrincipalID " NEX_PID_FORMAT " Summary -- \n", *itPid);
        for(nn::nex::qList<nn::nex::ApiCallSummary>::iterator it = summary.begin(); it != summary.end(); ++it)
        {
            NEX_DEMO_LOG(STRING_FORMAT", %u, %u, %u, %u, %u\n", it->GetMethodName().CStr(),
                         it->GetPeriod(),
                         it->GetLimit(),
                         it->GetResult(),
                         it->GetCount(),
                         it->GetTotalCount());
        }
        NEX_DEMO_LOG("\n");
    }

    NEX_DEMO_LOG("--------PRINT API CALL SUMMARY END----------\n");
}

/*!
    @brief NGSのAPI呼び出し記録機能の有効化を行い、記録内容の収集をしばらく行います。

    @param[in] pNgsFacade ログイン済みのオブジェクトを指定してください。
 */
void DebugMain(nn::nex::NgsFacade *pNgsFacade)
{
    nn::nex::ProtocolCallContext oContext;
    nn::nex::DebugClient debugClient;
    debugClient.Bind(pNgsFacade->GetCredentials());

    // APIの記録の有効化
    debugClient.EnableApiRecorder(&oContext);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to EnableApiRecorder.\n");
        return;
    }

    // 状態の確認 他のクライアントが無効化してるかもしれないので
    nn::nex::qBool isApiRecorderEnabled;
    debugClient.IsApiRecorderEnabled(&oContext, &isApiRecorderEnabled);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to IsApiRecorderEnabled.\n");
        return;
    }

    NEX_DEMO_LOG("isApiRecorderEnabled:%d\n", isApiRecorderEnabled);
    if(!isApiRecorderEnabled)
    {
        return;
    }

    // 取得・表示
    PrintApiRecordLoop(pNgsFacade, &debugClient);

    // サンプルデモサーバを使用している場合は、他の開発者が使用している可能性があるのでコメントアウト
    /*
    debugClient.DisableApiRecorder(&oContext);
    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        nex_demo_util::ReportQResultReturnCode(oContext.GetOutcome(), "Failed to DisableApiRecorder.\n");
        return;
    }
    */
}

/*!
    @brief NEXの初期化後にNGSへログインし、API呼び出し履歴の収集を行います。
 */
qInt NEXSampleMain()
{
    // インターネットに接続します
    if (!nex_demo_util::NetInit())
    {
        NEX_DEMO_LOG("nex_demo_util::NetInit failed.\n");
        return 1;
    }

    // アプリケーション定義のメモリアロケータを NEX に設定します。
    nex_demo_util::InitializeMemoryAllocator();

    // ログレベルの設定(設定を行うかどうかは任意です)
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);
    nn::nex::TraceLog::GetInstance()->SetFlag(TRACE_ALWAYS);

    // スレッドモードの設定と初期化
    nn::nex::Core::SetThreadMode(nn::nex::Core::ThreadModeSafeTransportBuffer);
    nn::nex::Initialize(true);

    {
        // ゲームサーバーへログインするためのクライアントを用意します。
        nn::nex::NgsFacade ngsFacade;
        nex_demo_util::LoginUtilityForSampleDemo loginUtilityForSampleDemo;

        // ゲームサーバーにログインします
        if (loginUtilityForSampleDemo.PerformTotalLogin(&ngsFacade))
        {
            DebugMain(&ngsFacade);

            // ゲームサーバーからログアウトします
            nn::nex::ProtocolCallContext logoutCallContext;
            ngsFacade.Logout(&logoutCallContext);
            logoutCallContext.Wait();
        }

    }
    nn::nex::Finalize(true);

    // InitializeMemoryAllocator() で設定したメモリアロケータを登録解除します。
    nex_demo_util::FinalizeMemoryAllocator();

    // インターネット接続を終了します。
    nex_demo_util::NetTerm();

    return 0;
}

/*!
    @brief 本サンプルのエントリポイントです。
 */
qInt GenericMain()
{
    NEX_DEMO_LOG("*******************************\n");
    NEX_DEMO_LOG("     Debug sample\n");
    NEX_DEMO_LOG("*******************************\n\n");


    qInt returnCode = NEXSampleMain();


    NEX_DEMO_LOG("Leaving application...\n");

    return returnCode;
}

}   // namespace {

NEX_MAIN_WRAPPER(GenericMain)
