﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <nn/fnd.h>

/*!
  @brief UnitHeapを複数用いて、高速なメモリ確保・開放を実現するクラスです。

  本サンプルのMultiUnitHeapを改変してアプリケーションに組み込んでいただいても構いません。

  本サンプルで紹介するMultiUnitHeapはUnitHeapを複数使い、
  要求されたアロケーションサイズを確保可能な最適なUnitHeapを検索しています。
  すべてのUnitHeapでのメモリ確保が失敗した場合には、緊急用のExpHeapからメモリ確保を行います。

  UnitHeapはCPU負荷低減を最優先としており、メモリ使用量の点においては他のヒープより劣ります。
  UnitHeapについての詳細はSDKのマニュアル等をご覧下さい。

  MultiUnitHeapTemplateにおけるメモリ配分パラメータの設定は、アプリケーション側に任されています。
  最初は多めにメモリを供給し、ピーク値を観察しながら適正なメモリサイズに近づけていく方法をお勧めします。
  GetDebugCount を使って、関数コール時の各UnitHeapの確保数、その時点での最大確保数などを取得できます。


  以下に、本サンプルで用意したMultiUnitHeapの使い方を記述します。

  1. 初期化の準備
    MultiUnitHeapに必要なメモリを準備します。
    GetRequiredMemorySizeにて、各UnitHeapのUnitサイズ・Unit数を指定して必要なメモリサイズを知ることができます。
    GetRequiredMemorySizeで得られたメモリブロックを用意します。
    このメモリブロックには4バイトアライメントが必要です。

    初期化にはMultiUnitHeapでのメモリ確保が失敗したときの緊急用のヒープも必要です。
    Initializeに初期化済のヒープを渡してください。
    MultiUnitHeapが絶対にメモリ確保に失敗しないのであれば緊急用ヒープは不要ですが、
    NEXはセットされたアロケータがメモリ確保に失敗したときの動作を考慮していませんのでご注意ください。

  2. 初期化処理
    Initializeに、1.で準備したMultiUnitHeap用バッファ、緊急用ヒープ、各UnitHeapのUnitサイズ・Unit数を渡します。
    本サンプルでは、Initialize失敗時にASSERTを仕込んでいますが、本来のアプリで失敗する可能性がある場合は
    bool値を返すなどしてきちんとハンドリングしてください。

  3. NEXライブラリへのアロケータ登録処理
    MultiUnitHeapを使ったメモリ確保関数、開放感数を用意し、
    nn::nex::MemoryManager::SetBasicMemoryFunctions を使ってNEXにセットしてください。
    nn::nex::GlobalVariables::AcquireInstance を呼び出すよりも前にセットする必要があります。

  4. メモリ枯渇時の対処
    MultiUnitHeapが枯渇した場合、緊急用のヒープからメモリが確保されます。
    その時、IsOutOfMemory がtrueを返すようになります。
    シングルスレッドモードであれば、nn::nex::Scheduler::Dispatch 後に
    IsOutOfMemory にてメモリ枯渇が発生していないかをチェックしてください。
    メモリ枯渇が発生した場合にはNEXライブラリをシャットダウンする必要があります。
    詳細なハンドリングについてはプログラミングマニュアルを参照してください。

  5. NEXライブラリ終了後の処理
    Finalize を呼び出してMultiUnitHeapを破棄してください。
    nn::nex::NetZ::Terminate が失敗した場合、メモリが開放されないまま残っていることがあります。
    その場合、Finalize の前に Invalidate を呼び出してヒープを破壊する必要があります。

 */

#define NEX_SAMPLE_MUH_PRINT NN_LOG
#define NEX_SAMPLE_MUH_ASSERT NN_ASSERT

template<size_t UnitHeapNum, class TUnitHeap, class TReservedHeap>
class MultiUnitHeapTemplate
{
public:

    /*!
     * @brief   デバッグ用の情報を扱う構造体です。
     */
    struct DebugCount
    {
        size_t peak;        //!< 確保数の最大値
        size_t use;         //!< 現在の確保数
        size_t unitSize;    //!< ユニットヒープのユニットサイズ
    };

    MultiUnitHeapTemplate() :
        m_pReservedHeap(NULL),
        m_bInitialized(false),
        m_bOutOfMemory(false)
    {}

    /*!
      @brief MultiUnitHeapTemplateの構築に必要となるメモリの量を得ます。
      @param[in]  unitSize 希望するユニットサイズを格納した配列。
      @param[in]  unitObjs 希望するユニットの個数を格納した配列。
      @return 指定したセッティングでMultiUnitHeapTemplateを構築するために必要なメモリサイズ。
      */
    size_t GetRequiredMemorySize(const size_t unitSize[UnitHeapNum], const size_t unitObjs[UnitHeapNum])
    {
        size_t total = 0;
        for(size_t i=0; i<UnitHeapNum; ++i)
        {
            size_t required = TUnitHeap::GetRequiredHeapSize(unitSize[i] + HEADER_SIZE, unitObjs[i], ALIGN);
            total += required;
        }
        return total;
    }

    /*!
    @brief 初期化された状態であるかどうかを返します。
    @see Initialize, Finalize
    @return MultiUnitHeapTemplateが初期化されていればtrue、そうでなければfalse。
    */
    bool IsInitialized(void) const
    {
        return m_bInitialized;
    }


    /*!
    @brief 初期化します。
           本関数を実行しても、内部のDebugCountはリセットされません。
    @param[in] pMem     MultiUnitHeapTemplateに渡すメモリ領域の先頭アドレス。4バイトアライメントが必要です。
    @param[in] size     pMemで渡したメモリ領域のサイズ。
    @param[in] unitSize ユニットヒープのサイズを収めた配列。
    @param[in] unitObjs ユニットヒープの個数を収めた配列。
    @param[in] pReservedHeap UnitHeap枯渇時に利用するヒープへのポインタ。MultiUnitHeap内では初期化・破棄は行いません。
    @see Finalize, IsInitialized, ResetDebugCount
    */
    void Initialize(void *pMem, size_t size, const size_t unitSize[UnitHeapNum], const size_t unitObjs[UnitHeapNum], TReservedHeap* pReservedHeap)
    {
        (void)size;
        NEX_SAMPLE_MUH_ASSERT(pMem);
        NEX_SAMPLE_MUH_ASSERT(pReservedHeap);
        NEX_SAMPLE_MUH_ASSERT(IsInitialized()==false);
        NEX_SAMPLE_MUH_ASSERT(size%ALIGN==0);
        NEX_SAMPLE_MUH_ASSERT(GetRequiredMemorySize(unitSize, unitObjs) <= size);

        // 各ユニットヒープを初期化。
        {
            u8 *p = static_cast<u8*>(pMem);
            for(size_t i=0; i<UnitHeapNum; ++i)
            {
                const size_t realSize = unitSize[i] + HEADER_SIZE;
                const size_t required = TUnitHeap::GetRequiredHeapSize(realSize, unitObjs[i], ALIGN);
                m_UnitHeapInstance[i].Initialize(realSize, reinterpret_cast<uptr>(p), required);
                p += required;
            }
        }

        m_pReservedHeap = pReservedHeap;
        m_bOutOfMemory = false;
        m_bInitialized = true;
    }

    /*!
    @brief 解放処理を実行します。
           この関数を呼び出す際は、MultiUnitHeapTemplateが確保したメモリが
           全て解放されていなければなりません。この条件が満たされないと、アサートで停止することがあります。
           このアサートを意図して回避するために、@ref Invalidate を用意しています。
           本関数を実行しても、内部のデバッグカウントはリセットされません。
    @see Initialize, IsInitialized, Invalidate, ResetDebugCount
     */
    void Finalize(void)
    {
        NEX_SAMPLE_MUH_ASSERT(m_bInitialized);
        m_bInitialized = false;
        for(size_t i=0; i<UnitHeapNum; ++i)
        {
            m_UnitHeapInstance[i].Finalize();
        }
        m_pReservedHeap = NULL;
    }

    /*!
    @brief MultiUnitHeapTemplateを無効化します。
           メモリリークが発生している状況でMultiUnitHeapTemplateの @ref Finalize を呼び出すと、
           アサート停止することがあります。これを意図して回避したいときに、本関数を
           呼び出してください。
    @see Finalize
     */
    void Invalidate(void)
    {
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            m_UnitHeapInstance[i].Invalidate();
        }
    }

    /*!
    @brief メモリ消費量をコンソールに出力します。
    */
    void Dump(void)
    {
        DebugCount debugCount[UnitHeapNum];
        GetDebugCount(debugCount);
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            NEX_SAMPLE_MUH_PRINT("unit[%d], use:%d, peak:%d\n",
                                 i, debugCount[i].use, debugCount[i].peak
                                );
        }
    }


    /*!
    @brief MultiUnitHeapTemplateが管理する全てのユニットヒープが解放された状態かどうかを返します。
    @return 全てのユニットヒープが解放されていればtrue、そうでなければfalseが返ります。
    */
    bool IsClean(void)
    {
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            if (m_UnitHeapInstance[i].IsClean() == false)
                return false;
        }
        return true;
    }

    /*!
    @brief DebugCount を取得します。関数コール時のUnitHeapの使用率などが得られます。
    */
    void GetDebugCount(DebugCount dst[UnitHeapNum])
    {
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            dst[i] = m_UnitHeapInstance[i].GetDebugCount();
        }
    }

    /*!
    @brief 内部に持っている DebugCount をリセットします。
    */
    void ResetDebugCount()
    {
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            m_UnitHeapInstance[i].ResetDebugCount();
        }
    }

    /*!
    @brief 要求サイズに適切なUnitHeapを検索してメモリを確保します。
           UnitHeapでメモリ確保できなかった場合は、緊急用のヒープからメモリ確保します。
    */
    void* Allocate(size_t size)
    {
        void* ret = NULL;
        size_t realSize = size + HEADER_SIZE;
        nn::fnd::HeapBase* pAllocatedHeap = NULL;
        for (size_t i=0 ; i<UnitHeapNum ; ++i)
        {
            ret = m_UnitHeapInstance[i].Allocate(realSize);
            if (ret)
            {
                pAllocatedHeap = m_UnitHeapInstance[i].GetHeapPtr();
                break;
            }
        }
        if (ret == NULL)
        {
            // MultiUnitHeapが枯渇、または要求サイズが大きすぎる
            // リザーブヒープからメモリ確保を行い、IsOutOfMemoryがtrueを返すよう設定します
            NEX_SAMPLE_MUH_PRINT("MultiUnitHeap is failed to Allocate %u byte. Use the reserved heap.\n", realSize);
            NEX_SAMPLE_MUH_ASSERT(m_pReservedHeap);
            ret = m_pReservedHeap->Allocate(realSize);
            NEX_SAMPLE_MUH_ASSERT(ret);
            pAllocatedHeap = m_pReservedHeap;

            // 要求サイズが大きすぎてMultiUnitHeapでメモリ確保できていない場合は、
            // 厳密にはMultiUnitHeapが枯渇しているわけではありません。
            // このような場合にIsOutOfMemoryをtrueにしたくない場合は、以下if文のコメントアウトを外してください。
            // UnitHeapを扱いつつ、要求サイズが大きな場合はExpHeapも活用するといったことが可能です。
            // ただし、MultiUnitHeapが本当に枯渇したときには
            // リザーブヒープからのメモリ確保が必ず成功するように十分なサイズのリザーブヒープをご使用ください。
            //
            // if ( m_UnitHeapInstance[UnitHeapNum].GetUnitSize() <= realSize ) // 本当にMultiUnitHeapが枯渇したときだけIsOutOfMemoryをtrueにするif文
            {
                m_bOutOfMemory = true;
            }
        }

        if (ret)
        {
            // ヘッダ準備
            NEX_SAMPLE_MUH_ASSERT(pAllocatedHeap);
            static_cast<MultiUnitHeapHeader*>(ret)->SetHeapBasePtr(pAllocatedHeap);
            ret = static_cast<u8*>(ret) + HEADER_SIZE;  // ヘッダの分、ずらす。
        }

        NEX_SAMPLE_MUH_ASSERT(ret);
        return ret;
    }

    /*!
    @brief 指定したメモリを確保したヒープに対してメモリ解放を行います。
    */
    void  Free(void *pMem)
    {
        MultiUnitHeapHeader *pHead = reinterpret_cast<MultiUnitHeapHeader*>(static_cast<u8*>(pMem) - HEADER_SIZE);
        pHead->GetHeapBasePtr()->FreeV(pHead);
    }

    /*!
    @brief 緊急用ヒープが使われたかどうかを返します。
    @return 緊急用ヒープが使われたことがあればtrue、そうでなければfalse。
    */
    bool IsOutOfMemory() const
    {
        return m_bOutOfMemory;
    }

private:
    static const size_t ALIGN = 4;

    class MultiUnitHeapHeader
    {
    public:
        MultiUnitHeapHeader(void) : m_pHeap(NULL) {}
        void SetHeapBasePtr(nn::fnd::HeapBase *p)
        {
            m_pHeap = p;
        }
        nn::fnd::HeapBase* GetHeapBasePtr(void)
        {
            return m_pHeap;
        }
    private:
        nn::fnd::HeapBase *m_pHeap;
    };

    NN_STATIC_ASSERT(sizeof(MultiUnitHeapHeader)%ALIGN==0);
    static const size_t HEADER_SIZE = sizeof(MultiUnitHeapHeader);

    class UnitHeapInstance
    {
    public:
        UnitHeapInstance() :
            m_unitSize(0)
        {
            ResetDebugCount();
        }
        TUnitHeap *GetHeapPtr()
        {
            return &m_unitHeap;
        }
        void Initialize(size_t unitSize, uptr addr, size_t memSize)
        {
            m_unitSize = unitSize;
            m_unitHeap.Initialize(unitSize, addr, memSize, ALIGN);
        }
        void Invalidate()
        {
            m_unitHeap.Invalidate();
        }
        void Finalize()
        {
            m_unitHeap.Finalize();
        }
        void* Allocate(size_t size)
        {
            if (size > m_unitSize) return NULL;
            void* ret = m_unitHeap.Allocate();
            if (ret)
            {
                size_t use = m_unitHeap.GetAllocatedCount();
                if ( use > m_dbgCount.peak )
                    m_dbgCount.peak = use;
            }
            return ret;
        }
        DebugCount& GetDebugCount()
        {
            m_dbgCount.unitSize = m_unitSize;
            m_dbgCount.use = m_unitHeap.GetAllocatedCount();
            return m_dbgCount;
        }
        void ResetDebugCount()
        {
            memset(&m_dbgCount, 0, sizeof(m_dbgCount));
        }
        bool IsClean() const
        {
            return m_unitHeap.GetAllocatedCount() == 0;
        }
        size_t GetUnitSize() const
        {
            return m_unitSize;
        }
    private:
        DebugCount m_dbgCount;
        size_t m_unitSize;
        TUnitHeap m_unitHeap;
    };

    UnitHeapInstance m_UnitHeapInstance[UnitHeapNum];
    TReservedHeap *m_pReservedHeap;

    bool m_bInitialized;
    bool m_bOutOfMemory;
};

