﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <nex.h>
#include "NexUtil.h"

namespace nex_demo_util
{

/*!
    @brief   CallContext の使い方を練習するためのクラスです。

    本クラスはサンプルデモにおいて CallContext を使った非同期処理の実行方法を示すためのものです。
    実用性はありません。
 */
class CallContextTrainer
{
public:
    /*!
        @brief   コンストラクタです。
     */
    CallContextTrainer()
        : m_delayMs(0)
        , m_qresult(true)
    {
    }
    /*!
        @brief   デストラクタです。
     */
    virtual ~CallContextTrainer()
    {
    }

    /*!
        @brief   非同期処理が完了するまでの時間(ミリ秒)を設定します。

        @param[in] delayMs 非同期処理が完了するまでの時間(ミリ秒)
     */
    void SetDelay(nn::nex::TimeInterval delayMs)
    {
        m_delayMs = delayMs;
    }

    /*!
        @brief   非同期処理が完了したときの結果を設定します。

        @param[in] qresult 非同期処理がしたときの結果
     */
    void SetResult(nn::nex::qResult qresult)
    {
        m_qresult = qresult;
    }

    /*!
        @brief   足し算を非同期に実行します。

        非同期処理が完了すると pSum に operand1 と operand2 を足した数が格納されます。
        SetDelay() をで完了するまでの時間を設定している場合はその時間が経過した後に処理が完了します。
        SetResult() で結果を設定している場合は非同期処理の結果がその設定したものになります。
        失敗する結果を設定した場合は pSum に結果は格納されません。
        また、成功した場合は結果をログに出力します。

        @param[in,out] pCallContext 非同期処理を管理するオブジェクト
        @param[in]  operand1 足される数
        @param[in]  operand2 足す数
        @param[out] pSum 和
     */
    qBool Add(nn::nex::CallContext* pCallContext, qInt32 operand1, qInt32 operand2, qInt32* pSum);

    /*!
        @brief   コンソールへのログ出力を非同期に実行します。

        SetDelay() をで完了するまでの時間を設定している場合はその時間が経過した後に処理が完了します。
        SetResult() で結果を設定している場合は非同期処理の結果がその設定したものになります。
        失敗する結果を設定した場合はログ出力は行われません。

        @param[in,out] pCallContext 非同期処理を管理するオブジェクト
        @param[in]  text ログ出力する文字列
     */
    qBool Log(nn::nex::CallContext* pCallContext, const nn::nex::String& text);

private:
    nn::nex::TimeInterval m_delayMs;
    nn::nex::qResult m_qresult;
};

}   // end of namespace nex_demo_util
