﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#if defined(NEX_NNSDK_FAMILY)
#include <atomic>
#endif

#include <nex.h>
#include "NexUtilDefs.h"

namespace nex_demo_util
{

/*!
    @brief メモリアロケータとして使用するユーティリティクラスのインタフェース。
 */
class DemoMemBase
{
public:
    static const int ALIGNMENT = 32;

    DemoMemBase();
    virtual ~DemoMemBase() {};

    /*!
        @brief 初期化処理を行う。

        @param[in] memBlock このユーティリティで使用するメモリブロック
        @param[in] memSize memBlockのサイズ。
     */
    void Initialize(void* memBlock, size_t memSize);

    /*!
        @brief 終了処理を行う。
     */
    void Finalize();

    /*!
        @brief 現時点でのメモリ確保回数が0かどうかを返す。
     */
    qBool IsClean() const;

    /*!
        @brief 指定したサイズのメモリを返す。

        @param[in] size 確保するメモリのサイズ。
        @return 確保したメモリ。
     */
    void* AllocFunction(size_t size);

    /*!
        @brief AllocFunction() で確保したメモリを解放する。
        @param[in] p 解放するメモリのアドレス。
     */
    void FreeFunction(void* p);

    /*!
        @brief 現時点でのメモリ使用量を取得する。
        @return 現時点でのメモリ使用量。
     */
    size_t GetUsage() const;

    /*!
        @brief 現時点でのメモリ使用量のピークを取得する。
        @return 現時点でのメモリ使用量のピーク。
     */
    size_t GetMaxUsage() const;

    /*!
        @brief 現時点でのメモリ確保回数を取得する。
        @return 現時点でのメモリ確保回数。
     */
    size_t GetCurrentAllocCount() const;

    /*!
        @brief 現時点での累計メモリ確保回数を取得する。
        @return 現時点での累計メモリ確保回数。
     */
    size_t GetTotalAllocCount() const;

    /*!
        @brief メモリアロケータを破壊する。
     */
    void Invalidate();

    /*!
        @brief メモリの使用状況をダンプする。
     */
    virtual void Dump() = 0;

#if defined(NEX_NNSDK_FAMILY)
    /*!
        @brief メモリの割り当てを変更する。
     */
    void * ReallocFunction(void *ptr, size_t size);
#endif

    static qBool isThreadAllocateCheck;

private:
    /*!
        @brief 割り当てたメモリのサイズを取得する。

        @param[in] p 割り当てたメモリのアドレス。

        @return 割り当てたサイズ。
     */
    virtual size_t GetBlockSize(const void* p) const = 0;

    /*!
        @brief 初期化処理を行う。

        @param[in] memBlock このユーティリティで使用するメモリブロック
        @param[in] memSize memBlockのサイズ。
     */
    virtual void InnerInitialize(void* memBlock, size_t memSize) = 0;

    /*!
        @brief 終了処理を行う。
     */
    virtual void InnerFinalize() = 0;

    /*!
        @brief 指定したサイズのメモリを返す。

        @param[in] size 確保するメモリのサイズ。
        @return 確保したメモリ。
     */
    virtual void* InnerAlloc(size_t size) = 0;

    /*!
        @brief AllocFunction() で確保したメモリを解放する。
        @param[in] p 解放するメモリのアドレス。
     */
    virtual void InnerFree(void* p) = 0;

    /*!
        @brief メモリアロケータを破壊する。
     */
    virtual void InnerInvalidate() = 0;

#if defined(NEX_NNSDK_FAMILY)
    /*!
        @brief メモリの割り当てを変更する。
     */
    virtual void* InnerRealloc(void *ptr, size_t size) = 0;
#endif

#if defined(NEX_NNSDK_FAMILY)
    std::atomic_size_t m_memUsage;
    std::atomic_size_t m_currentAllocCount;
    std::atomic_size_t m_totalAllocCount;
#else
    size_t m_memUsage;
    size_t m_currentAllocCount;
    size_t m_totalAllocCount;
#endif
    size_t m_memMaxUsage;
    mutable nn::nex::ForcedCriticalSection m_cs;

};

/*!
    @brief メモリアロケータとして使用するユーティリティクラス。
 */
class DemoMem : public DemoMemBase
{
public:
    DemoMem() {};
    virtual ~DemoMem() {};
    void Dump();
private:

    /*!
        @brief 割り当てたメモリのサイズを取得する。

        @param[in] p 割り当てたメモリのアドレス。

        @return 割り当てたサイズ。
     */
    virtual size_t GetBlockSize(const void* p) const ;

    /*!
        @brief 初期化処理を行う。

        @param[in] memBlock このユーティリティで使用するメモリブロック
        @param[in] memSize memBlockのサイズ。
     */
    virtual void InnerInitialize(void* memBlock, size_t memSize);

    /*!
        @brief 終了処理を行う。
     */
    virtual void InnerFinalize();

    /*!
        @brief 指定したサイズのメモリを返す。

        @param[in] size 確保するメモリのサイズ。
        @return 確保したメモリ。
     */
    virtual void* InnerAlloc(size_t size);

    /*!
        @brief AllocFunction() で確保したメモリを解放する。
        @param[in] p 解放するメモリのアドレス。
     */
    virtual void InnerFree(void* p);

    /*!
        @brief メモリアロケータを破壊する。
     */
    virtual void InnerInvalidate();
#if defined(NEX_NNSDK_FAMILY)

    /*!
        @brief メモリの割り当てを変更する。
     */
    virtual void* InnerRealloc(void *ptr, size_t size);
#endif
};

}   // end of namespace nex_demo_util
