﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
/*
 * NEX Sample
 *
 * Description:
 *
 * メッセージングによる送受信を行うクラスを定義しています。
 * 本機能では送受信するオブジェクトに TextMessage を使用しています。バイナリを送受信したい場合は BinaryMessage を利用してください。
 * 
 * 本機能はそのままアプリケーションで利用したり、必要に応じて修正して組み込んだりすることが可能です。
 */

#include <nex.h>
#include "NexUtil.h"

namespace nex_demo_util
{

/*
 @brief    メッセージングを行うクラスです。
 */
class MessageSender
{
public:
    /*
     @brief    メッセージングを行うオブジェクトを生成します。

     @param[in]     pNgsFacade ログイン済みのオブジェクトを指定してください。
     */
    MessageSender(nn::nex::NgsFacade* pNgsFacade)
        : m_pNgsFacade(pNgsFacade)
        , m_pMessagingClient(NULL)
        , m_pMessagingNotificationHandler(NULL)
        , m_callContextStore()
        , m_firstError(true)
    {
        Bind();
    };
    virtual ~MessageSender()
    {
        // 処理が完了していない非同期処理を全てキャンセルする
        m_callContextStore.CancelAll();
        Unbind();
    };

    /*
     @brief    指定したマッチメイクセッションの参加者全員へメッセージを送信する

     @param[in] target 送信先のギャザリングID
     @param[in] message 送信するメッセージ
     */
    void SendChat(nn::nex::GatheringID target, const nn::nex::String& message)
    {
        NEX_DEMO_LOG("\nCall SendChat().\n");

        nn::nex::TextMessage* pTextMessage = new nn::nex::TextMessage();
        nn::nex::UserMessageDataHolder oDataHolder = pTextMessage;

        // メッセージをセットする
        pTextMessage->SetSubject(NEX_T("This is a TextMessage"));
        pTextMessage->SetTextBody(message);
        NEX_DEMO_LOG("*** Sending message to gid:[%u] subject[" STRING_FORMAT "] text body:[" STRING_FORMAT "]\n",
                     target,
                     pTextMessage->GetSubject().CStr(),
                     pTextMessage->GetTextBody().CStr());

        nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MessageSender::OnCompletionSend), this));

        // あとで非同期処理をキャンセルできるようにするため、生成したCallContextを非同期処理実行中のみm_callContextStoreに保存するようにします。
        nex_demo_util::CallContextStoreAssociator::Register(&m_callContextStore, pCallContext);
        m_pMessagingClient->SendChat(pCallContext, target, oDataHolder);
    }

    /*
     @brief    指定したプレイヤーへメッセージを送信する

     @param[in] target 送信先のプリンシパルID
     @param[in] message 送信するメッセージ
     */
    void SendInstant(nn::nex::PrincipalID target, const nn::nex::String& message)
    {
        NEX_DEMO_LOG("\nCall SendInstant().\n");

        nn::nex::TextMessage* pTextMessage = new nn::nex::TextMessage();
        nn::nex::UserMessageDataHolder oDataHolder = pTextMessage;

        // メッセージをセットする
        pTextMessage->SetSubject(NEX_T("This is a TextMessage"));
        pTextMessage->SetTextBody(message);
        NEX_DEMO_LOG("*** Sending message to pid:[" NEX_PID_FORMAT "] subject[" STRING_FORMAT "] text body:[" STRING_FORMAT "]\n",
                     target,
                     pTextMessage->GetSubject().CStr(),
                     pTextMessage->GetTextBody().CStr());

        nn::nex::ProtocolCallContext* pCallContext = new nn::nex::ProtocolCallContext();
        pCallContext->SetFlag(nn::nex::CallContext::DeleteOnCompletion);
        pCallContext->RegisterCompletionCallbackGeneric(std::bind1st(std::mem_fun(&MessageSender::OnCompletionSend), this));

        // あとで非同期処理をキャンセルできるようにするため、生成したCallContextを非同期処理実行中のみm_callContextStoreに保存するようにします。
        nex_demo_util::CallContextStoreAssociator::Register(&m_callContextStore, pCallContext);
        m_pMessagingClient->SendInstant(pCallContext, target, oDataHolder);
    }

    /*
     @brief    最初に発生した送信エラーを取得する

     @return 最初に発生した送信エラー
     */
    const nn::nex::qResult& GetFirstError() const { return m_firstError; }

    /*
     @brief    未処理の受信メッセージがあるかどうかを調べる

     @return 未処理の受信メッセージがあるかどうか
     */
    qBool HasReceivedMessage() const { return m_pMessagingNotificationHandler->HasReceivedMessage(); }

    /*
     @brief    未処理の受信メッセージを一つ取り出す

     @return 受信メッセージ
     */
    nn::nex::TextMessage DequeueReceivedMessage() { return m_pMessagingNotificationHandler->DequeueReceivedMessage(); }

private:
    // 受信メッセージを処理するイベントハンドラ
    class MyMessagingNotificationHandler: public nn::nex::MessagingNotificationHandler
    {
    public:
        MyMessagingNotificationHandler(){}
        void DeliverInstantMessage(const nn::nex::DataHolder& hMsg)
        {
            // TextMessage,BinaryMessageを判別します。
            // TextMessage::DynamicCast()が成功する場合、受信メッセージはTextMessageです。
            nn::nex::TextMessage* pTextMsg = nn::nex::TextMessage::DynamicCast(hMsg.Get());
            NEX_DEMO_LOG("*** Recieved message from:[" NEX_PID_FORMAT "] subject[" STRING_FORMAT "] text body:[" STRING_FORMAT "]\n",
                         pTextMsg->GetSenderID(),
                         pTextMsg->GetSubject().CStr(),
                         pTextMsg->GetTextBody().CStr());
            if (pTextMsg)
            {
                EnqueueReceivedMessage(*pTextMsg);
                return;
            }
        }
        qBool HasReceivedMessage() const { return !m_notificationQueue.empty(); }
        nn::nex::TextMessage DequeueReceivedMessage() { nn::nex::TextMessage message = m_notificationQueue.front(); m_notificationQueue.pop_front(); return message; }
    private:
        void EnqueueReceivedMessage(const nn::nex::TextMessage& message) { m_notificationQueue.push_back(message); }
        nn::nex::qList<nn::nex::TextMessage> m_notificationQueue; // 通知キュー
    };

    void Bind()
    {
        nn::nex::Credentials* pCred = m_pNgsFacade->GetCredentials();
        NEX_DEMO_ASSERT(pCred != NULL);
        m_pMessagingClient = new nn::nex::MessagingClient();
        NEX_DEMO_VERIFY(m_pMessagingClient->Bind(pCred));
        m_pMessagingNotificationHandler = new MyMessagingNotificationHandler();
        m_pMessagingClient->RegisterMessagingNotificationHandler(m_pMessagingNotificationHandler);
    }

    void Unbind()
    {
        m_pMessagingClient->UnregisterMessagingNotificationHandler();
        delete m_pMessagingNotificationHandler;
        m_pMessagingNotificationHandler = NULL;
        m_pMessagingClient->Unbind();
        delete m_pMessagingClient;
        m_pMessagingClient = NULL;
    }

    /*
     @brief    メッセージ送信完了コールバック

     @param[in] pCallContext 送信時に指定したCallContext
     */
    void OnCompletionSend(nn::nex::ProtocolCallContext* pCallContext)
    {
        switch (pCallContext->GetState())
        {
        case nn::nex::CallContext::CallSuccess:
            NEX_DEMO_LOG("     Send*() succeeded.\n");
            break;
        case nn::nex::CallContext::CallCancelled:
            // キャンセルされるのは ~MessageSender() からのみなのでなにもしない
            NEX_DEMO_LOG("     Send*() cancelled.\n");
            break;
        case nn::nex::CallContext::CallFailure:
            NEX_DEMO_LOG("     Send*() failed.\n");
            nex_demo_util::ReportQResultReturnCode(pCallContext->GetOutcome());
            ErrorOccurred(pCallContext->GetOutcome());
            break;
        default:
            NEX_DEMO_ASSERT(false);
            break;
        }
    }

    void ErrorOccurred(const nn::nex::qResult& result) { if (m_firstError) { m_firstError = result; } }

    nn::nex::NgsFacade* m_pNgsFacade;                           // ログイン済みのNgsFacadeオブジェクトへのポインタ
    nn::nex::MessagingClient* m_pMessagingClient;               // メッセージング処理のAPI呼び出しに使用するオブジェクトへのポインタ
    MyMessagingNotificationHandler* m_pMessagingNotificationHandler;    // メッセージングに関する通知をハンドリングするオブジェクトへのポインタ
    nex_demo_util::CallContextStore m_callContextStore;         // 非同期処理実行中のCallContextが保存されるコンテナです。キャンセル処理が必要なもののみ格納されます。
    nn::nex::qResult m_firstError;                              // 最初に発生したエラー
};

}   // end of namespace nex_demo_util
