﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/
#pragma once

#include <algorithm>
#include <nex.h>
#include "NexUtilDefs.h"


namespace nex_demo_util
{

/*
    @brief      ユーザー定義の結果コードとqResultのいずれかを保持するクラス

    qResultもしくはアプリケーション独自の結果コードのいずれかを保持したい場面で利用します。
*/
class ResultContainer
{
public:
    /*
        @brief      ResultContainerが保持している結果コードのタイプ
    */
    enum Type
    {
        TYPE_UNDEFINED,     // デフォルトコンストラクタで生成した場合のタイプ。常に成功の結果となります。
        TYPE_USER_DEFINED,  // ユーザー定義の結果コード。ここでは0以上の場合成功の結果としています。
        TYPE_QRESULT        // qResult
    };

    /*
        @brief      ResultContainerを生成する。常に成功の結果を保持する。
    */
    ResultContainer()
        : m_type(TYPE_UNDEFINED)
        , m_userresult(0)
        , m_qresult()
    {
    }
    /*
        @brief      ユーザー定義の結果コードを持つResultContainerを生成する。
    */
    explicit ResultContainer(nn::nex::qInt32 userresult)
        : m_type(TYPE_USER_DEFINED)
        , m_userresult(userresult)
        , m_qresult()
    {
    }
    /*
        @brief      qResultを持つResultContainerを生成する。
    */
    explicit ResultContainer(const nn::nex::qResult& qresult)
        : m_type(TYPE_QRESULT)
        , m_userresult(0)
        , m_qresult(qresult)
    {
    }

    /*
        @brief      成功したかどうかを返す。
    */
    qBool IsSuccess() const
    {
        switch (m_type)
        {
        case TYPE_UNDEFINED:
            return true;
        case TYPE_USER_DEFINED:
            return m_userresult >= 0;
        case TYPE_QRESULT:
            return QSUCCEEDED(m_qresult.GetReturnCode());
        default:
            SYSTEMCHECK(false);
            return false;
        }
    }

    /*
        @brief      比較演算子。型と結果コードが一致しているときのみ一致する。
    */
    qBool operator== (const ResultContainer& rhs) const
    {
        return m_type == rhs.m_type && m_userresult == rhs.m_userresult && m_qresult == rhs.m_qresult;
    }
    qBool operator!= (const ResultContainer& rhs) const { return !(*this == rhs); }
    /*
        @brief      結果を出力する
    */
    void ReportError(const char* pHeaderText = NULL)
    {
        if (pHeaderText)
        {
            NEX_DEMO_LOG("%s", pHeaderText);
        }

        switch (GetType())
        {
        case TYPE_USER_DEFINED:
            NEX_DEMO_LOG("UserDefined Error code: %d\n", GetUserResult());
            break;
        case TYPE_QRESULT:
            NEX_DEMO_LOG("qResult Return Code: 0x%x " STRING_FORMAT "\n", GetQResult().GetReturnCode(), GetQResult().GetReturnCodeString());
            break;
        default:
            NEX_DEMO_LOG("Undefined Error type: %d\n", GetType());
            break;
        }
    }

    /*
        @brief      bool値へのキャスト演算子
    */
    operator qBool() const { return IsSuccess(); }
    /*
        @brief      保持しているTypeを取得する。
    */
    Type GetType() const { return m_type; }
    /*
        @brief      qResultを取得する。保持しているタイプが TYPE_QRESULT 以外の場合は呼び出せません。
    */
    nn::nex::qResult GetQResult() const { NEX_DEMO_ASSERT(m_type == TYPE_QRESULT); return m_qresult; }
    /*
        @brief      ユーザー定義の結果コードを取得する。保持しているタイプが TYPE_USER_DEFINED 以外の場合は呼び出せません。
    */
    nn::nex::qInt32 GetUserResult() const { NEX_DEMO_ASSERT(m_type == TYPE_USER_DEFINED); return m_userresult; }
private:
    Type m_type;
    nn::nex::qInt32 m_userresult;
    nn::nex::qResult m_qresult;
};

/*
    @brief      CallContextのポインタを保持するクラス。

        実行中の非同期処理を後で全てキャンセルしたい場合に利用できます。
*/
class CallContextStore
{
public:
    /*
        @brief      指定した CallContext を記憶します
    */
    void Store(nn::nex::CallContext* cc) { m_callContexts.push_back(cc); }
    /*
        @brief      指定した CallContext を削除します
    */
    void Remove(nn::nex::CallContext* cc) { m_callContexts.erase(std::remove(m_callContexts.begin(), m_callContexts.end(), cc), m_callContexts.end()); }
    /*
        @brief      記憶してある CallContext を全てキャンセルします
    */
    void CancelAll()
    {
        for (nn::nex::qVector<nn::nex::CallContext*>::iterator it=m_callContexts.begin(); it!=m_callContexts.end(); ++it)
        {
            (*it)->Cancel();
        }
    }
private:
    nn::nex::qVector<nn::nex::CallContext*> m_callContexts;
};

/*
    @brief      CallContextStore への挿入・削除を自動化する機能を持つクラス。

        CallContextStore に対する。Store(), Remove() を自動化します。
*/
class CallContextStoreAssociator
{
public:
    /*
        @brief      指定したCallContextをCallContextStoreに記憶し、非同期処理終了時に削除されるようにします。

        @param callContextStore 記憶先の CallContextStore
        @param pCallContext 記憶する CallContext
    */
    static void Register(CallContextStore* callContextStore, nn::nex::CallContext* pCallContext)
    {
        // pCallContextの非同期処理完了時に自動的にdeleteされます
        new CallContextStoreAssociator(callContextStore, pCallContext);
    }
private:
    CallContextStoreAssociator(CallContextStore* callContextStore, nn::nex::CallContext* pCallContext)
        : m_pCallContextStore(callContextStore)
    {
        // CallContextStoreに記憶する
        m_pCallContextStore->Store(pCallContext);
        // コールバックオブジェクトを生成
        nn::nex::Callback<CallContextStoreAssociator, nn::nex::CallContext*>* pCallback
            = new nn::nex::Callback<CallContextStoreAssociator, nn::nex::CallContext*>(this, &CallContextStoreAssociator::OnComplete, pCallContext);
        // 成功しても失敗してもコールバックする
        pCallContext->RegisterCompletionCallback(pCallback, false);
    }
    ~CallContextStoreAssociator(){}

    // 非同期処理終了時に呼び出されるコールバック
    void OnComplete(nn::nex::CallContext* cc)
    {
        // CallContextStoreから削除する
        m_pCallContextStore->Remove(cc);
        // 非同期処理完了時に自身のオブジェクトも削除します。
        delete this;
    }
    CallContextStore* m_pCallContextStore;
};

/*
    @brief      一連の処理を非同期で行う機能の基底クラス
*/
class ApiChain
{
public:
    ApiChain()
        : m_IsCompleted(false)
        , m_Result()
    {}
    virtual ~ApiChain(){};
    virtual void Call()=0;
    virtual const nn::nex::qResult& GetResult() const { return m_Result; }
    nn::nex::qBool virtual IsCompleted() const { return m_IsCompleted; }
protected:
    virtual void Complete(const nn::nex::qResult& result) { m_IsCompleted = true; m_Result = result; }
    nn::nex::qBool m_IsCompleted;
private:
    nn::nex::qResult m_Result;
};

}   // end of namespace nex_demo_util
