﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Platform/Core/qSet.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <OnlineCore/src/Core/PeriodicJob.h>

BEGIN_NAMESPACE_NEX

//{secret}
class Message;
class Station;

// Group=Miscellaneous Classes

/*
Description: Controls how messages are bundled.
Remarks: Message bundling can be used to reduce bandwidth usage. This class is used to control how messages
are bundled before they are sent across the network. On the local station, a message bundle will be created
for each remote station that the local station knows. All messages to be sent to a given remote station
will then be stored in the message bundle relating to that particular remote station.

Message bundles may be sent either automatically or manually. If bundles are sent automatically the frequency
at which they are sent is set by either specifying a time delay, via SetMaximumFlushDelay, the maximum
size that the bundle may attain, via SetMaximumBundleSize, or both. When either of these two conditions is met
the message bundle will be sent. Alternatively, message bundles may be sent manually via the methods Station::FlushBundle
or Station::FlushAllBundles. These two methods will send message bundles regardless of the time delay and
maximum bundle size settings.

Note that if there is one reliable message in a bundle, then all the messages in that bundle will be sent reliably.
User-defined reliable messages will include reliable dataset updates, continuity break updates, and RMCs for which the
/SendUnreliableMessage/ CallContext::Flags is not set.
See Also: Station::FlushBundle, Station::FlushAllBundles
*/

/*!
 * @brief   メッセージバンドル機能を制御します。
 *
 * メッセージバンドルを利用することによってネットワーク帯域幅を節約することが出来ます。
 * このクラスは、メッセージが送信される前にどのようにバンドルされていくかを制御します。
 * ローカルのステーションでは、メッセージバンドルはすでにローカルステーションによって存在が確認された
 * ネットワーク越しの各ステーションごとに作成されます。
 * すべてのメッセージは相手先のステーションに送信される前に、それぞれのメッセージバンドルバッファにためられます。
 * ただし、DirectTransport/DirectStream経由のメッセージはためられません。
 *
 * メッセージバンドルは自動的にか、あるいは手動で送信されます。メッセージバンドルが自動的に
 * 送信される場合、送信頻度は@ref SetMaximumFlushDelay() にて設定された最大時間を経過するか
*  最大のメッセージサイズに達するかで決定されます。
 * これらのいずれか、あるいは両方の条件を満たした場合、メッセージバンドルは相手先のステーションに向けて送信されます。
 * 一方、手動でメッセージバンドルを送信する場合は、@ref Station::FlushBundle() か、
 * @ref Station::FlushAllBundles() を利用することで送信することが出来ます。
 * これら二つの関数はメッセージの最大サイズや、前回送信してからの経過時刻などの設定に関係なく送信されます。
 *
 * 規定値は、有効で、最大遅延時間は0で、Dispatchごとにバンドリングされて送信されます。
 *
 * メッセージが、全員に送られるUnreliable(到着性を保証しない)メッセージと、Reliable(到着性を保証する)メッセージの場合で、
 * 異なるキューに溜められてバンドリングされます。個別に送られるUnreliableメッセージは、バンドリングされません。
 *
 * システム定義のReliableメッセージは、特にDOのDuplica同期時に多く生成されます。
 * ユーザー定義のReliableメッセージは、Reliableで更新することを指定したデータセット、
 * 推定航法の連続性の切断(@ref DataSet::IndicateContinuityBreak())、
 * @ref CallContext::Flags の中にあるSendUnreliableMessageフラグのついていないRMC(リモートメソッドコール)が挙げられます。
 *
 * メッセージバンドルについての詳細はNEXプログラミングマニュアルもご覧ください。
 */
class BundlingPolicy: public RootObject
{
    //Group =Methods
public:

    // {secret}
    typedef enum _Flag
    {
        // {secret}
        // See bug 2416 for discussion
        MixReliability=1<<0
    } Flag;

    /*
    Remarks: A BundlingPolicy object should never be directly instantiated. Instead, it is created by $PRODUCTNAME$
    each time the local station discovers a remote station. To access the BundlingPolicy object from within your
    game use the GetInstance method.
    */

    /*!
     * @brief   このコンストラクタはアプリケーション側から呼び出さないでください。
     *
     * このインスタンスは、ローカルのステーションが相手のステーションを発見されるたびに
     * システムによって自動的に作成されます。BunldingPolicyを利用するためには、
     * @ref GetInstance() メソッドによってインスタンスを取得してください。
     */
    BundlingPolicy();

    /*
    Remarks: A BundlingPolicy object should never be directly destroyed. It is destroyed by $PRODUCTNAME$ when
    the local station loses connection with a remote station or the session terminates.
    */

    /*!
     * @brief   このデストラクタはアプリケーション側から呼び出さないでください。
     *
     * このオブジェクトは、相手先のステーションとの接続が切れたとき、
     * あるいはセッションが終了したときに自動的に破棄されます。
     */
    virtual ~BundlingPolicy();

    /*
    Description: Returns a pointer to the BundlingPolicy instance.
    Returns: A reference to the BundlingPolicy object.
    Remarks: Use the reference returned by this method to control message bundling.
    For example, set the maximum bundling delay as follows:

            BundlingPolicy::GetInstance()->SetMaximumFlushDelay((Time)1000);

    */

    /*!
     * @brief   BundlingPolicyのインスタンスを取得します。
     *
     * この関数を利用して取得したインスタンスをメッセージバンドルを制御するために利用してください。
     *
     * @return  BundlingPolicyオブジェクトへのポインタが返ります。
     */
    static BundlingPolicy *GetInstance();

    /*
    Description: Enables message bundling.
    Remarks: Message bundling is disabled by default. Use this method to enable it.
    See Also: Disable, IsEnabled
    */

    /*!
     * @brief   メッセージバンドル機能を有効にします。
     *
     * メッセージバンドル機能はデフォルトで有効になっています。
     * この関数を利用することでメッセージバンドル機能を有効にすることが出来ます。
     * この関数は実行中に呼び出して動的に有効 / 無効を変更することが出来ます。
     *
     * @see     Disable, IsEnabled
     */
    void         Enable();

    /*
    Description: Disables message bundling.
    Remarks: Message bundling is disabled by default.
    See Also: Enable, IsEnabled
    */

    /*!
     * @brief   メッセージバンドル機能を無効にします。
     *
     * メッセージバンドル機能はデフォルトで有効になっています。
     * この関数を利用することでメッセージバンドル機能を有効にすることが出来ます。
     * この関数は実行中に呼び出して動的に有効 / 無効を変更することが出来ます。
     *
     * @see     Enable, IsEnabled
     */
    void         Disable();

    /*
    Description: Returns whether or not message bundling is enabled.
    Returns: true if message bundling is enabled, false if not or an error occurred.
    Remarks: Message bundling is disabled by default but may be enabled via the method Enable.
    See Also: Enable, Disable
    */

    /*!
     * @brief   メッセージバンドル機能が有効になっているかどうかを取得します。
     * @retval  true  メッセージバンドルが有効になっています(デフォルト状態ではメッセージバンドルは有効です)。
     * @retval  false メッセージバンドルが無効になっています。
     * @see     Enable, Disable
     */
    qBool IsEnabled() const;

    //n1589
    void SetFlushCheckPeriod(TimeInterval period);

    /*
    Description: Sets the maximum time to wait before sending a queued message to a given station.
    Parameters:
    tiDelay: The maximum time in milliseconds that will expire before a queued message is sent to a given station.
    The default value is 75 ms.
    Remarks: During the delay time, all messages to be sent to a given station are stored in a bundle.
    When the time delay expires, all messages in the bundle are sent.
    See Also: GetMaximumFlushDelay
    */

    /*!
     * @brief   ためられたメッセージを送信するための遅延時間(送信まで どのぐらい待つか)を指定します。
     *
     * 遅延時間が過ぎるまで、送信されようとしたメッセージはすべて内部的にためられ、送信されません。
     * この遅延時間が経過したあと、溜められたメッセージはバンドリングされ、次回Dispatch時に実際に送信されます。
     * その際には、StreamBundlingによるパケットのバンドリングがフラッシュされます。
     *
     * 0の場合には、Dispatchの終わりに行われるパケット送信処理直前までためられてバンドリングされます。
     * また、StreamBundlingによるパケットのバンドリングをフラッシュしないため、パケットのバンドリングとの併用も可能です。
     *
     * @param   tiDelay [in] 最大遅延時間をミリ秒単位で指定します。0の場合は、Dispatchの終わりに行われるパケット送信処理直前までためられます。
     *                       規定値は、0です。
     * @see     GetMaximumFlushDelay
     */
    void SetMaximumFlushDelay(TimeInterval tiDelay);

    /*
    Description: Returns the maximum flush delay in milliseconds.
    Returns: The maximum flush delay in milliseconds as set via SetMaximumFlushDelay.
    Remarks: The maximum flush delay is the maximum time that will expire before a message queued in a bundle
    is sent to a given station. The default value is 75 ms.
    See Also: SetMaximumFlushDelay
    */

    /*!
     * @brief   最大遅延時間をミリ秒単位で取得します。
     *
     * 遅延時間が過ぎるまで、送信されようとしたメッセージはすべて内部的にためられ、送信されません。
     * この遅延時間が経過したあと、それらのバンドルされたメッセージは一度に送信されます。
     *
     * この関数は、@ref SetMaximumFlushDelay() で指定された値を返します。
     *
     * @return  最大遅延時間がミリ秒単位で返ります。
     * @see     SetMaximumFlushDelay
     */
    TimeInterval GetMaximumFlushDelay() const
    {
        return m_tiDelay;
    }

    /*
    Description: Sets the maximum size of a bundle.
    Parameters:
    uiSize: The maximum size that a bundle may attain before it is sent.
    Remarks: When the maximum bundle size is reached, all messages in the bundle are sent. The maximum size
    is limited by the Maximum Transmission Unit (MTU) of the transport layer used to send messages to the
    given remote station. The actual maximum bundle size will be the smaller value between the MTU and that
    set via SetMaximumBundleSize.
    */

    /*! @internal
     * @brief   最大のバンドルサイズを設定します。
     *
     * 最大のバンドルサイズに達すると、バンドルされているすべてのメッセージは送信されます。
     * 最大サイズは相手先のステーションにメッセージを送信する伝送レイヤーの
     * MTU(Maximum Transmission Unit : ネットワークにおいて1回の転送に送信できる最大サイズ)に制限されます。
     * そのため、使用される最大のバンドルサイズは、この関数によって設定された値と、
     * MTUのどちらか小さいほうになります。
     *
     * @param   uiSize [in] 最大のバンドルサイズを指定します。
     * @attention サポートされません
     */
    void SetMaximumBundleSize(qUnsignedInt32 uiSize)
    {
        m_uiMaxSize = uiSize;
    }

    // {secret}
    // TODO - This should be exposed.
    qUnsignedInt32 GetMaximumBundleSize() const
    {
        return m_uiMaxSize;
    }

    /*!
     * @brief Unreliableメッセージをバンドリングするか指定します。
     *
     * @param [in] b バンドリングの設定フラグ
     */
    void SetEnableUnreliableBundling(qBool b)
    {
        m_bEnableUnreliableBundling = b;
    }

    /*!
     * @brief Unreliableメッセージをバンドリングするかを取得します。
     *
     * デフォルトではTrueです。
     *
     * @return バンドリングの設定フラグ
     */
    qBool IsEnableUnreliableBundling()
    {
        return m_bEnableUnreliableBundling;
    }

    // {secret}
    void SetFlag(qUnsignedInt32 uiFlag);
    // {secret}
    void ClearFlag(qUnsignedInt32 uiFlag);
    // {secret}
    qBool FlagIsSet(qUnsignedInt32 uiFlag) const;

    qSet<Station *>& GetNeedFlushStations()
    {
        return needFlushStations;
    }

private:

    class FlushJob: public PeriodicJob
    {
    public:
        FlushJob(): PeriodicJob(Job::NetZ, NEX_T("FlushBundleJob"))
        {
            m_tLastFlush=0;
        }
        virtual void Execute();
    private:
        Time m_tLastFlush;
    };

    qUnsignedInt32 m_uiFlags;

    TimeInterval    m_tiDelay;
    qUnsignedInt32  m_uiMaxSize;
    FlushJob *m_pFlushJob;
    qBool           m_bEnableUnreliableBundling;

    qSet<Station *> needFlushStations;

};

END_NAMESPACE_NEX

