﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/MethodID.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <NetZ/src/Core/DOCore/RMCContext.h>
#include <NetZ/src/Core/DOCore/DOOperation.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>
#include <OnlineCore/src/Platform/Core/qMap.h>

BEGIN_NAMESPACE_NEX

class DuplicatedObject;

//group=System Operations

/*
Description: This DOOperation is performed whenever an RMC is called on a duplicated object.
Remarks: A CallMethodOperation will occur as a result of an RMC being called.

When a CallMethodOperation is called the following sequence of events occurs:
    o   An RMC is called by a given Station,
    o   The SafetyExecutive is called to determine if the call may proceed. If so, we continue, if not we exit.
    o   The method DuplicatedObject::OperationBegin is called,
    o   The RMC is called,
    o   The method DuplicatedObject::OperationEnd is called. If an RMC is postponed,
    then when it is recalled it will once again pass via the SafetyExecutive before the operation begins.

The major purpose of this operation class is to ensure that embeded RMC calls do not deadlock while
waiting for a return value. This is achieved by postponing the RMC call when it is not yet ready
to return the result and then recalling the RMC a short time later. In the meantime any queued
messages are processed so that if the original RMC relies on a queued message, that message will
be processed and the RMC call can be completed.

The following example illustrates one manner in which the CallMethodOperation class can be used,
in this case to compute factorial numbers.

$import CallMethodOperation TOP\Tests\Integration\RMC\TestObject.cpp$

In addition, this class may be used to control how, when, and by whom RMCs are called. All RMCs
are uniquely identified by $PRODUCTNAME$ enabling given RMCs to be called or denied according to
the station that calls them or the particular circumstances surrounding the call. For example,
you may want to restrict certain calls so that they can only be called by a server station or
only called during the game setup or at a certain point in the game. For example, implementing
restrictions so that certain RMCs can only be called on server stations could be implemented
as follows:

            qBool RestrictSpecialRMCs(const Operation & oOperation) {
                static MethodID  idSpecialRMC =MethodIDGenerator::GetID (NEX_T("SpecialRMC"));

                // First we check that the operation being called is
                // a CallMethodOperation and that the RMC is a SpecialRMC.
                if (oOperation.GetType()==Operation::CallMethod &&
                      CallMethodOperation::DynamicCast(&oOperation)->GetTargetMethodID()
                                                        ==idSpecialRMC) {
                    // As only server processes can call this special RMC we check
                    // if the station that called the RMC is a server.
                    Station::Ref refCallOrigin(oOperation.GetOrigin());
                    if (refCallOrigin.IsValid() && refCallOrigin->GetProcessType()
                                                ==Station::ServerProcess) {
                        return true;
                    } else {
                        return false;
                    }
                }
                // All other operations are allowed.
                return true;
            }

See Also: Remote Method Calls Overview
*/

/*!
 * @brief   このDOOperationは、RMCが複製オブジェクトに対して呼び出されたときに実行されます。
 *
 * CallMethodOperationはRMC(リモートメソッドコール)が呼び出されようとしたときの結果として発生します。
 * CallMethodOperationは以下のシーケンスにしたがって呼び出されます。
 *
 * @li RMCが与えられたステーションから呼び出されます。
 * @li @ref SafetyExecutive が呼び出され、この操作を実行してもよいのか判定します。問題なければ続行し、問題があればここで終了します。
 * @li @ref DuplicatedObject::OperationBegin が呼び出されます。
 * @li RMCが実行されます。
 * @li @ref DuplicatedObject::OperationEnd が呼び出されます。もしもRMCが後回しにされた場合、
 *     そのときは もう一度呼び出されるときに、操作が始まる前に@ref SafetyExecutive を通過します。
 *
 * CallMethodOperationの主な目的は、埋め込みRMC呼び出しが戻り値を返すまでデッドロックしないことを確かめることです。
 * これは、まだ呼び出しの結果を返すための準備が整っていないときにRMCが後回しにされ、
 * 短い時間のあともう一度実行されることでこの目的を達成します。
 * もしも元のRMCがためられたメッセージを中継したら、そのメッセージが処理されてRMC呼び出しを完了するために、
 * それまではどんなメッセージも処理されません。
 *
 * 加えてこのクラスは、いつ、どのように、そして誰によってRMCが呼び出したかを制御するためにも利用できます。
 * すべてのRMCはシステムによって一意に識別されています。
 * それによって、RMCを呼び出したステーション、あるいは他にRMCが呼び出された状況によってなどから、
 * RMCが実行したり拒否したりすることができます。
 */
class CallMethodOperation: public DOOperation
{
public:

    //group=Methods

    // {secret}
    CallMethodOperation(RMCContext::ID idRMC, DOHandle hSourceStation, qUnsignedInt32 uiFlags,
                        DOHandle hTargetObject, MethodID idTargetMethod, Message* pCallMessage);
    //{secret}
    virtual ~CallMethodOperation();

    //{secret}
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
    //{secret}
    void DispatchCall();
    //{secret}
    void PerformLocalCall();

    NEX_IMPL_OPERATION_COMMON_METHODS(CallMethod);

    // Access to members.

    //{secret}
    RMCContext::ID GetRMCID() const
    {
        return m_idRMC;
    }
    //{secret}
    qResult GetOutcome() const
    {
        return m_rOutcome;
    }
    //{secret}
    void  SetOutcome(qResult rOutcome) const
    {
        m_rOutcome = rOutcome;
    }

    /*!
     * @brief   RMC呼び出しを行ったステーションへの複製オブジェクトハンドルを取得します。
     *
     * @return  RMC呼び出しを行ったステーションへの複製オブジェクトハンドルが返ります。
     */
    DOHandle GetSourceStation() const
    {
        return GetOrigin();
    }

    /*
    Description: Returns the MethodID of the RMC that this operation calls.
    Returns: The MethodID of the RMC that this operation calls.
    Remarks: Each method (Actions, RMC) defined in the DDL is assigned a unique MethodID by $PRODUCTNAME$.
    See Also: MethodID
    */

    /*!
     * @brief   RMCのMethodIDを取得します。
     *
     * DDL(データ定義言語)に定義されているそれぞれのRMC(リモートメソッドコール)は、
     * システムによって一意な@ref MethodID が割り当てられます。
     *
     * @return  このコンテキストのRMCに対する@ref MethodID が返ります。
     */
    MethodID GetTargetMethodID() const
    {
        return m_idTargetMethod;
    }

    // {secret}
    Message* GetCallMessage() const;

    // {secret}
    void CreateResponseHeader();
    // {secret}
    void AddFailureDetails();
    // {secret}
    Message* PrepareSuccessMessage() const;
    // {secret}
    void SendResponse();
    // {secret}
    void CancelResponse();

    /*
    Description: Postpones a CallMethodOperation.
    Parameters:
    tiPostponeDelay: The amount of time in milliseconds that this operation will be
    postponed. The default value is zero.

    Remarks: To avoid deadlocks in embeded RMCs, the original RMC call can be postponed if it
    is not yet ready to return the result and then recalled a short time later. During call
    postponement, any queued messages are processed so that if the original RMC relies on a
    queued message, that message will be processed and the RMC call can complete.

    Note that when either of the DOCallContext flags TargetObjectMustBeMaster or TargetObjectMustHaveAuthority
    is set, these flags are only guaranteed to be true the initial time that the method is called. Therefore, if
    the call is posponed these flags are not rechecked and may no longer be true.
    See Also: OperationIsPostponed, GetDelay, DOCallContext::Flags
    */

    /*!
     * @brief   操作を後回しにします。
     *
     * 埋め込まれたRMC(リモートメソッドコール)のデッドロックを回避するために、
     * 実行する準備が整っておらず、短い時間待ってからもう一度呼び出したい場合には、
     * 元のRMC呼び出しを保留することが出来ます。呼び出しが保留されている間、
     * もしも元のRMCがためられたメッセージを中継したら、そのメッセージが処理されてRMC呼び出しを完了するために、
     * それまではどんなメッセージも処理されません。
     *
     * @ref DOCallContext::Flags のTargetObjectMustBeMasterまたは
     * @ref DOCallContext::Flags のTargetObjectMustHaveAuthorityがセットされている場合でも、
     * これらのフラグは最初の呼び出しのときのみ、それが正しい状態であると保証します。
     * したがって、この関数によって実行が後回しにされた場合、
     * TargetObjectMustBeMasterフラグがセットされていたとしても
     * デュプリカ(@ref DuplicatedObject::IsADuplica)になるなど、
     * フラグと複製オブジェクトの状態が一致しない可能性もあります(これらは再チェックされません)。
     *
     * @param[in]   tiPostponeDelay 実行を遅延する時間を指定します。
     */
    void PostponeOperation(TimeInterval tiPostponeDelay);

    /*
    Description: Returns whether or not this CallMethodOperation has been postponed.
    Returns: true if this operation has been postponed, false if not or an error occurred.
    Remarks: A CallMethodOperation can be postponed and then recalled via PostponeOperation.
    See Also: PostponeOperation, GetDelay
    */

    /*!
     * @brief   この操作は後回しにされたかどうかを返します。
     *
     * CallMethodOperationは、@ref PostponeOperation() によって実行を後回しにさせることが出来ます。
     *
     * @retval  true  実行が後回しにされたことを示します。
     * @retval  false 実行が後回しにされていないことを示します。
     * @see     PostponeOperation
     */
    qBool OperationIsPostponed() const;

    /*
    Description: Returns the amount of time for which this CallMethodOperation will be postponed.
    Returns: The /PostponeDelay/ in milliseconds for this operation.
    Remarks: The delay is set via PostponeOperation. The default value is zero.
    See Also: PostponeOperation, OperationIsPostponed
    */

    /*!
     * @brief   この操作が後回しにされたときの遅延時間を取得します。
     *
     * 遅延時間は@ref PostponeOperation() が実行されたときに設定されます。
     * デフォルトは0です。
     *
     * @return  この操作の遅延時間がミリ秒単位で返ります。
     * @see     PostponeOperation, OperationIsPostponed
     */
    TimeInterval GetDelay() const
    {
        return m_tiPostponeDelay;
    }

    /*
    Description: Returns the number of times that this CallMethodOperation has been called.
    Returns: The number of times that this CallMethodOperation has been called.
    Remarks: The attempt count commences at zero and is incremented by one each time the
    CallMethodOperation is postponed and then recalled.
    See Also: PostponeOperation
    */

    /*!
     * @brief   この操作が何回呼び出されたかを取得します。
     *
     * カウントは0から始まり、CallMethodOperationの実行が後回しにされて、
     * 再実行されるたびに1ずつインクリメントされます。
     *
     * @return  この操作が呼び出された回数が返ります(カウントは0から始まります)。
     * @see     PostponeOperation
     */
    size_t GetAttemptCount();

    /*
    Description: Attaches a UserContext to this CallMethodOperation object.
    Parameters:
    oDelayContext: The UserContext to be attached to this CallMethodOperation object.
    See Also: GetUserContext
    */

    /*!
     * @brief   このCallMethodOperationにUserContextをアタッチさせます。
     * @param[in]   oDelayContext このオブジェクトにアタッチさせる@ref UserContext への参照を指定します。
     * @see     GetUserContext
     */
    void AttachUserContext(const UserContext& oDelayContext);

    /*
    Description: Returns the UserContext that refers to this CallMethodOperation.
    Returns: The UserContext that relates to this CallMethodOperation.
    Remarks: A UserContext is attached to a CallMethodOperation object via AttachUserContext.
    See Also: AttachUserContext
    */

    /*!
     * @brief   このCallMethodOperationにアタッチされているUserContextを取得します。
     *
     * @ref AttachUserContext() を呼び出すことによって、@ref UserContext をアタッチできます。
     *
     * @return  このCallMethodOperationにアタッチされているUserContextへの参照が返ります。
     */
    UserContext& GetUserContext();


    /*!
     * @brief Reliable通信の場合に、受信時に使われたSubStreamIDを取得します。
     *
     * @return 受信時に利用されたSubStreamID
     *
     * @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()
     *
     */
    SubStreamID GetSubStreamID() const
    {
        return m_subStreamID;
    }

private:

#ifdef NEX_TRACE_ENABLED    // n2370
    // {secret}
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // NEX_TRACE_ENABLED

    //{secret}
    virtual qBool CallsBackOnDataSet()
    {
        return false;
    }
    //{secret}
    virtual qBool CallsBackOnDataSet(qByte /*byIndex*/)
    {
        return false;
    }

private:

    size_t m_uiAttemptNumber;
    qBool m_bPostponed;
    UserContext m_oDelayContext;
    TimeInterval m_tiPostponeDelay;
    // Source info.
    // {secret}
    RMCContext::ID m_idRMC;

    // Method info.
    // {secret}
    MethodID m_idTargetMethod;
    Message* m_pCallMessage;

    // Caller flags.
    // {secret}
    qUnsignedInt32 m_uiFlags;

    // These variables are mutable: when we signal an operation (always
    // const), we use these variables to set the call outcome.
    mutable Message* m_pResponseMessage;
    mutable qResult m_rOutcome;

    SubStreamID m_subStreamID;
};

END_NAMESPACE_NEX

