﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/EventHandler.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>
#include <OnlineCore/src/Transport/Interface/TransportError.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <NetZ/src/Core/DOCore/DOCoreErrors.h>

BEGIN_NAMESPACE_NEX
class Event;

// Group=Results Classes

/*
Summary: This class defines the context of a call to a duplicated object.
Remarks: This class is used to define the call context of calls made to fetch and migrate
duplicated objects and of RMCs.
*/

/*!
 * @brief   複製オブジェクトに対する非同期実行を管理するためのクラスです。
 *
 * このクラスは複製オブジェクトのフェッチや複製オブジェクトのマイグレーション、
 * 複製オブジェクトに対するRMC(リモートメソッドコール)に利用されます。
 */
class DOCallContext: public CallContext
{
    //Group=Type Definitions
public:
    friend class CallRegister;

    // {secret}
    typedef qUnsignedInt16 ID;

    //Description: This enumerates the different Types of call contexts that can be called by $PRODUCTNAME$.
    //See Also: Type
    /*!
     * @brief   呼び出し種類を示すための列挙体です。
     */
    enum Type
    {
        //A call to perform a Remote Method Call.
        /*!
         * @brief   この呼び出しはRMC(リモートメソッドコール)に対するものです。
         */
        RMC,
        //A call to fetch a Duplica.
        /*!
         * @brief   この呼び出しはデュプリカのフェッチに対するものです。
         */
        FetchCall,
        //A call to migrate a Duplication master.
        /*!
         * @brief   この呼び出しは複製マスターのマイグレーションに対するものです。
         */
        MigrationCall
    };

    // Description: This enumerates the various Flags that may be used with a DOCallContext.
    // See Also: Flags, DuplicatedObject::HasAuthority, CallMethodOperation::PostponeOperation
    /*!
     * @brief   DOCallContext に使用可能な各種フラグを列挙します。
     */
    enum Flags
    {
        // The call will be synchronous, /i.e./ the system will wait for the call to return
        // a value or an error before performing the next task.
        /*!
         * @brief   この呼び出しは同期呼び出しとなります(関数は作業をすべて終えるまで返ってきません)。
         */
        SynchronousCall                 = CallContext::FirstFlagAvailable,
        // The call will only be called on an object that is a Duplication master. This flag can only be set
        // for a RMCContext, is only valid if the flag CallOnMaster is set, and may be use in conjunction with the
        // RetryOnTargetValidationFailure flag.
        /*!
         * @brief   この呼び出しはオブジェクトの複製マスターのみで呼び出されます。
         *          このフラグは@ref RMCContext のためのみに存在しています。CallOnMasterフラグが同時にセットされているときのみ有効です。
         *          また、RetryOnTargetValidationFailureフラグと一緒に利用されることがあります。
         */
        TargetObjectMustBeMaster        = (CallContext::FirstFlagAvailable<<1),
        // The call will only be made on an object that has authority. Refer to
        // DuplicatedObject::HasAuthority for more details about object authority. This flag can only be set
        // for a RMCContext, is only valid if the flag CallOnMaster is set, and may be use in conjunction with the
        // RetryOnTargetValidationFailure flag.
        /*!
         * @brief   この呼び出しを行うとき、対象となっているオブジェクトに権限がある必要があります(@ref DuplicatedObject::HasAuthority を参照)。
         *          このフラグは@ref RMCContext のためのみに存在しています。CallOnMasterフラグが同時にセットされているときのみ有効です。
         *          また、RetryOnTargetValidationFailureフラグと一緒に利用されることがあります。
         */
        TargetObjectMustHaveAuthority   = (CallContext::FirstFlagAvailable<<2),
        // If the outcome of a call is ErrorInvalidRole or ErrorNoAuthority, the call will try to
        // resend the message to the Duplication master until the call outcome is either Success
        // or an error other than ErrorInvalidRole or ErrorNoAuthority is returned. This flag can only
        // be set for a RMCContext and is only valid when the CallOnMaster flag and either of the flags
        // TargetObjectMustHaveAuthority or TargetObjectMustBeMaster is set.
        /*!
         * @brief   この呼び出しの結果(@ref Outcome)が、ErrorInvalidRoleまたはErrorNoAuthorityのときに、
         *          複製マスターに対してもう一度同一のメッセージを、Successが返ってくるか、
         *          あるいはErrorInvalidRoleかErrorNoAuthority以外のエラーが返るまで送信し続けます。
         *          このフラグは@ref RMCContext のためのみに存在しています。CallOnMasterフラグと
         *          TargetObjectMustHaveAuthorityまたはTargetObjectMustBeMasterのうちどちらかがセットされているときのみ有効です。
         */
        RetryOnTargetValidationFailure  = (CallContext::FirstFlagAvailable<<3),
        // The method will be called on all Duplicas. This flag can only be set for a RMCContext, is only
        // valid when the OneWayCall flag is set, and can only be called by a Duplication master.
        /*!
         * @brief   この呼び出しは、すべてのデュプリカに呼び出されます。
         *          このフラグは@ref RMCContext のためのみに存在しています。OneWayCallフラグも同時にセットされているときのみ有効です。
         *          複製マスターからのみ呼び出すことができます。
         */
        CallOnDuplicas                  = (CallContext::FirstFlagAvailable<<4),
        // The method will be called on the Duplication master. This flag can only be set for an RMCContext.
        // This flag can only correctly pass a FlagsAreValid test if it can verify that the call's target
        // object is a master. To do this, it must have a valid reference to the target object.
        /*!
         * @brief   この呼び出しは、複製マスターに呼び出されます。
         *          このフラグは@ref RMCContext のためのみに存在しています。呼び出し対象オブジェクトが
         *          複製マスターであると確認できた場合のみに@ref FlagsAreValid() テストを正しく通過することが出来ます。
         *          これを通過するためには、正しい対象の複製オブジェクトへの参照を持っている必要があります。
         */
        CallOnMaster                    = (CallContext::FirstFlagAvailable<<5),
        // This flag should not be set directly. It is automatically set when SetTargetStation is called.
        /*!
         * @brief   このフラグは直接使用しないでください。@ref SetTargetStation 関数が呼び出されたときに自動的にセットされます。
         */
        CallOnTargetStation             = (CallContext::FirstFlagAvailable<<6),
        /* The method will be called on all neighbouring stations of the local Station, /i.e./
        all the Stations in the Session that the local Station knows. The set of Stations depends on
        which Stations the local Station has discovered, by means of DuplicationSpaces or another
        discovery mechanism. This flag can only be set for a RMCContext and is only valid if the
        OneWayCall flag is also set.
        */

        /*!
         * @brief   メソッドは、全ての近傍ステーションに対して呼び出されます。
                                    つまり、ローカルステーションが知りうるセッション中の全ステーションに対して呼び出されます。
                                    近傍ステーションとしてどのステーションが選択されるかは、ローカルステーションが使用する
                                    複製スペース、あるいはその他の発見メカニズムに依存します。このフラグは @ref RMCContext
                                    に対してのみセットすることができ、OneWayCall フラグが同時にセットされているときのみ有効です。
                                    現在の実装は、CallOnDuplicas相当ですので、CallOnDuplicasの利用を推奨します。
         */
        CallOnNeighbouringStations      = (CallContext::FirstFlagAvailable<<7),
        // The method will be called on the local Station. This flag can only be set for a RMCContext.
        /*!
         * @brief   このメソッドはローカルステーションに対して呼び出されます。@ref RMCContext のためだけに存在しています。
         */
        CallOnLocalStation              = (CallContext::FirstFlagAvailable<<8),
#ifndef FOR_DOC_PURPOSE
        // {secret}
        /*!
          @internal
         */
        VoiceData                       = (CallContext::FirstFlagAvailable<<9)
#endif
    };


    // Description: This enumerates the various Outcomes that a call may return.
    // See Also: Outcome, DuplicatedObject::HasAuthority
    /*!
     * @brief   この列挙体は、呼び出しの結果を表します。
     */
    enum Outcome
    {
        // The call has not yet terminated and therefore, the
        // outcome is not known.
        /*!
         * @brief   この呼び出しはまだ終了していないので、結果は不明です。
         */
        UnknownOutcome                      = QERROR(Core, Unknown),
        // The call terminated successfully.
        /*!
         * @brief   この呼び出しは正常に終了しました。
         */
        Success                             = QSUCCESS(DOCore, Success),
        // The target station of the call could not be reached.
        /*!
         * @brief   この呼び出しが対象のステーションにまで到達しませんでした。
         */
        ErrorStationNotReached              = QERROR(DOCore, StationNotReached),
        // The call was invoked but the target station was disconnected before a response was received.
        /*!
         * @brief   この呼び出しは実行されましたが、返答を受け取る前に対象のステーションが切断しました。
         */
        ErrorTargetStationDisconnect        = QERROR(DOCore, TargetStationDisconnect),
        // The call was invoked but the local station is leaving the session and a response
        // will never be received.
        /*!
         * @brief   この呼び出しは実行されましたが、ローカルステーションがセッションから離脱中なので、
         *          返答メッセージは受け取りません。
         */
        ErrorLocalStationLeaving            = QERROR(DOCore, LocalStationLeaving),
        // The object on which the call was called could not be found on the target station of the call.
        /*!
         * @brief   この呼び出しが呼び出されたオブジェクトが、呼び出しの対象のステーションに見つかりませんでした。
         */
        ErrorObjectNotFound                 = QERROR(DOCore, ObjectNotFound),
        // The object was found, but the role of the object was not
        // the one expected.  For example, this occurs if the flag
        // TargetObjectMustBeMaster was set but a RMC was called
        // on a Duplica instead of a Duplication master.
        /*!
         * @brief   オブジェクトは見つかりましたが、期待された役割ではありませんでした。
         *          たとえば、@ref TargetObjectMustBeMaster フラグがセットされていたのに、
         *          RMC(リモートメソッドコール)の対象が複製マスターではない場合にこの結果となります。
         */
        ErrorInvalidRole                    = QERROR(DOCore, InvalidRole),
        // The call failed because it did not complete before the time out set for the call expired.
        /*!
         * @brief   呼び出しはタイムアウトを経過したため完了しませんでした。
         */
        ErrorCallTimeout                    = QERROR(DOCore, CallTimeout),
        // The method invocation of a RMC failed.  This can be caused by
        // conflicting versions of the DDL between the caller and the
        // callee. This outcome is only valid if the context is a RMCContext.
        /*!
         * @brief   RMCのメソッド呼び出しが失敗しました。DDL(データ定義言語)のバージョンが、
         *          呼び出し元と呼び出し先で競合していることが原因である可能性があります。
         *          もしくは、呼び出し先で該当するRMCのメソッドがない可能性があります。
         *          この結果は、@ref RMCContext でのみ返ります。
         */
        ErrorRMCDispatchFailed              = QERROR(DOCore, RMCDispatchFailed),
        // The migration of an object is in progress. This outcome is only valid if the context
        // is a MigrationContext.
        /*!
         * @brief   オブジェクトはマイグレーション(複製オブジェクトコントロール権のステーション間移動)中です。
         *          この結果は、@ref MigrationContext でのみ有効です。
         */
        ErrorMigrationInProgress            = QERROR(DOCore, MigrationInProgress),
        // The call was made with invalid parameters. This can be a cstr
        // string larger than the size defined in the DDL or an
        // incorrect combination of flags, for example.
        /*!
         * @brief   呼び出しは、無効なパラメータで行われました。
         *          文字列がDDL(データ定義言語)で指定されたサイズを超えた大きさで呼び出されたか、
         *          フラグの組み合わせに問題がある場合などにこの結果となります。
         */
        ErrorInvalidParameters              = QERROR(Core, InvalidArgument),
        // Access to the given call was denied by the target.
        /*!
         * @brief   この呼び出しは対象で拒否されました。
         */
        ErrorAccessDenied                   = QERROR(Core, AccessDenied),
        // The object was found, but the TargetObjectMustHaveAuthority flag was set and the
        // object did not have authority. Refer to DuplicatedObject::HasAuthority for
        // more details about object authority.
        /*!
         * @brief   オブジェクトは見つかりましたが、呼び出しにTargetObjectMustHaveAuthorityフラグが
         *          ついているのに、対象のオブジェクトに権限がついていない場合に返ります。
         *          権限についての詳細は@ref DuplicatedObject::HasAuthority() を参照してください。
         */
        ErrorNoAuthority                    = QERROR(DOCore, NoAuthority),
        // No target station was specified for this call.
        /*!
        * @brief    この関数呼び出しの対象となるステーションは指定されませんでした。
                 */
        NoTargetStationSpecified            = QERROR(DOCore, NoTargetStationSpecified),
        // The call has been postponed. A call may be postponed via CallMethodOperation::PostponeOperation.
        /*!
        * @brief    この関数呼び出しは延期されました。
                             *              CallMethodOperation::PostponeOperation を経由すると、関数呼び出しは延期されます。
                 */
        CallPostponed                       = QSUCCESS(DOCore, CallPostponed),

        /*!
         * @brief 送信バッファが一時的にいっぱいになり、呼び出しができませんでした。
         */

        ErrorReliableSendBufferFull = QERROR(Transport, ReliableSendBufferFull),

        /*!
         * @brief 指定されたSubStreamIDが不適切で、相手と通信できません。
         */
        ErrorInvalidSubStreamID = QERROR(Transport, InvalidSubStreamID),

        /*!
         * @brief パケットバッファが不足しているため、呼び出しができませんでした。パケットバッファの設定は@ref PacketBufferManager を参照してください。
         */
        ErrorPacketBufferFull = QERROR(Transport, PacketBufferFull)

    };
    //ToDO ここにフィールドを追加した場合には、DOCallContext::GetOutcomeString()に変換処理を追加すること
    
    
    /*
    Parameters:
    hTargetStation: The target station of the call.
    bSynchronousCall: true if the call is made synchronously, which is the default,
    false if asynchronously.

    Remarks: If no arguments are passed to the constructor, the DOCallContext instance will
    initialize the value of the hTargetStation to INVALID_DOHANDLE. By default the call
    will be made synchronously, /i.e./ the system will wait for the call to return a value or
    an error before performing the next task. The method Wait can be used if the call is
    to be made asynchronously.
    */

    /*!
     * @brief   オブジェクトを初期化します。
     * @param[in]   hTargetStation       この呼び出しの対象となるステーションへの複製オブジェクトハンドル(@ref DOHandle)を指定します。
     *                                   デフォルト引数は、INVALID_DOHANDLEとなります。
     * @param[in]   bSynchronousCall     この呼び出しを同期実行(関数は作業を終了するまで返らない)とするかどうかを指定します。
     *                                   デフォルト引数は、true(同期実行とする)です。
     */
    DOCallContext(DOHandle hTargetStation=INVALID_DOHANDLE,
                  qBool bSynchronousCall=true);

    virtual ~DOCallContext();

#ifdef NEX_MOVE_SEMANTICS_SUPPORT

    /*!
    @brief    ムーブコンストラクタです。

    @param[in]    rhs    ProtocolCallContext オブジェクト。
    */
    DOCallContext(DOCallContext&& rhs) NEX_NOEXCEPT;

    /*!
    @brief    ムーブ代入演算子です。

    @param[in]    rhs    ProtocolCallContext オブジェクト。
    */
    DOCallContext& operator=(DOCallContext&& rhs) NEX_NOEXCEPT
    {
        if (this == &rhs)
        {
            return *this;
        }
        this->Swap(rhs);
        return *this;
    }
#endif

    //Group=Methods

    /*
    Description: Sets the target station of the call.
    Parameters:
    hTargetStation: A DOHandle to the target station of the call.

    See Also: GetTargetStation, Flags

    Remarks: When this method is called, the CallOnTargetStation flag is automatically set.
    By default, the DOHandle for the target station is INVALID_DOHANDLE. The significance of the
    target station depends on the DOCallContext. For a RMCContext, the target station is the station
    that the RMC is called on, for a FetchContext it is the station from which we wish to fetch a
    Duplica, and for a MigrationContext it is the station to which we wish to migrate an object.

    Refer to DuplicatedObject::FetchDuplica for an example of this method in use.
    */

    /*!
     * @brief   呼び出し対象のステーションを設定します。
     *
     * この関数が呼び出されたときに、CallOnTargetStationフラグが自動的に設定されます。
     * コンストラクタになにも引数を設定しなかった場合、対象のステーションはINVALID_DOHANDLEとなります。
     * 「対象となるステーション」の意味は、オブジェクトの型(派生先のクラス)によって異なります。
     * たとえば、@ref RMCContext では、対象のステーションとは、RMC(リモートメソッドコール)の呼び出し先となります。
     * @ref FetchContext では、デュプリカを取ってくる対象のステーションになります。
     * @ref MigrationContext では、オブジェクトの移転先のステーションとなります。
     * @param[in]   hTargetStation   呼び出し対象となるステーションのハンドル。
     * @see GetTargetStation, Flags
     */
    void SetTargetStation(DOHandle hTargetStation);

    /*
    Description: Returns the target station of the call.
    Returns: The DOHandle of the target station of the call.
    Remarks: In general, this method returns the Station set via SetTargetStation. However, for a RMC where the
    OneWayCall flag is *not* set this method will return the target Station set via either SetTargetStation or the
    CallOnMaster or CallOnLocalStation flags. If the OneWayCall flag is set, this method will only return the
    Station set via SetTargetStation regardless of whether other target stations were set via the various flags.
    See Also: SetTargetStation, Flags
    */

    /*!
     * @brief   呼び出し対象のステーションを取得します。
     *
     * 一般的には、この関数は@ref SetTargetStation() にて設定されたステーションへの
     * 複製オブジェクトハンドル(@ref DOHandle)を返します。
     * しかし、OneWayCall(@ref Flags を参照)が指定されていないRMC(リモートメソッドコール)については、
     * この関数は@ref SetTargetStation() で指定したステーションか、CallOnMasterかCallOnLocalStationフラグが返ります。
     * OneWayCallフラグがセットされていたら、他のフラグによってセットされたほかの対象のステーションにかかわらず
     * @ref SetTargetStation() でセットされたステーションを返します。
     *
     * @return  呼び出し対象のステーションが返ります。
     * @see     SetTargetStation
     */
    DOHandle GetTargetStation()
    {
        return m_refTargetStation.GetHandleValue();
    }

    /*
    Description: Returns whether or not the combination of flags currently in use is valid.
    Returns: true if the combination of flags currently in use is valid, false if not or an error occurs.
    Note: This method can only correctly verify the CallOnDuplicas flag if it can verify that the call's target
    object is a master. To do this, it must have a valid reference to the target object. This method can, therefore,
    sometimes return true if all other flags are true but the CallOnDuplicas flag cannot be correctly verified.
    See Also: Flags
    */

    /*!
     * @brief   フラグ(@ref Flags)の組み合わせが正しいかどうかを取得します。
     *
     * @retval  true    フラグの組み合わせが正しいときに返ります。
     * @retval  false   フラグの組み合わせが正しくないかエラーが発生したときに返ります。
     */
    virtual qBool FlagsAreValid() const;

    /*
    Description: Clears the target Station(s) that have previously been set for this DOCallContext.
    Remarks: This method clears any "target" flags that have been set and sets the target Station for
    the call to INVALID_DOHANDLE, which is the default value for the target station. This method can
    be called before setting the target station to ensure that the method is called on the desired stations.
    See Also: SetTargetStation, Flags
    */

    /*!
     * @brief   このDOCallContextに前に設定されていた対象のステーション情報を消去します。
     *
     * この関数は、このDOCallContextオブジェクトに設定されているすべての対象に関するフラグを降ろします。
     * その後、対象のステーションへの複製オブジェクトハンドルをINVALID_DOHANDLEに設定します(これがデフォルト状態)。
     * 確実に正しい対象に対してこの呼び出しが実行されるために、対象のステーションを設定する前にこの関数を実行します。
     */
    void ClearTargets()
    {
        ClearFlag(CallOnDuplicas | CallOnMaster | CallOnTargetStation | CallOnNeighbouringStations | CallOnLocalStation);
        SetTargetStation(INVALID_DOHANDLE);
    }

    /*!
     * @brief   非同期処理が完了するまたはタイムアウトを越えるまで、システムを待機させます。 
     *
     * @ref DOCallContext::Flags の SynchronousCall が オフの時に呼び出すことができます。
     */
    qBool Wait(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    /*
    Description: A call is automatically cancelled if the DOCallContext is deleted.

    Remarks: If the DOCallContext does not allow cancellation of the call, which is the case for a
    Migration call, the call will lock until it has terminated.
    */

    /*!
     * @brief   この呼び出しは、DOCallContextが削除された場合に自動的にキャンセルされます。
     *
     * このDOCallContextが呼び出しに対してキャンセルを許可しない場合(マイグレーションの呼び出しの場合)、
     * 呼び出しは終了するまでロックアップします。
     */
    qBool Cancel(State eNewState=CallCancelled);


    /*!
     * @brief Reliable通信の場合に、送信時に使うSubStreamIDを指定します。
     *
     * デフォルトは SubStreamIDDefine::SYSTEMで、システムと共用です。
     *
     * @param[in] subStreamID 送信時に利用するSubStreamID
     *
     * @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()
     *
     */
    void SetSubStreamID(SubStreamID subStreamID)
    {
        m_subStreamID = subStreamID;
    }

    /*!
     * @brief Reliable通信の場合に、送信時に使うSubStreamIDを取得します。
     *
     * デフォルトは SubStreamIDDefine::SYSTEMで、システムと共用です。
     *
     * @return 送信時に利用するSubStreamID
     *
     * @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()
     *
     */

    SubStreamID GetSubStreamID() const
    {
        return m_subStreamID;
    }

    /*! @refmanoverload{name2}
     * @brief    非同期処理の結果を @ref DOCallContext::Outcome として取得します。
     * 
     * 特定の非同期処理に対する結果を取得します。
     * 結果の内容については @ref CallContext をパラメータとしてとる関数を参照してください。
     * 
     * @return     @ref DOCallContext::Outcome としての非同期処理の結果。
     */
    DOCallContext::Outcome GetOutcome() const;

    /*!
     * @brief @ref DOCallContext::Outcome の値を該当する名前へ変換します。
     * 
     * @param[in] uOutcome Outcomeの値
     * @return uOutcomeに該当するOutcome名
     */
    static const qChar* GetOutcomeString(Outcome uOutcome);

    // Access to members.
    // {secret}
    ID GetID()
    {
        return m_idRMCCall;
    }

    void Swap(DOCallContext& rhs) NEX_NOEXCEPT
    {
        CallContext::Swap(rhs);
        std::swap(m_idRMCCall, rhs.m_idRMCCall);
        std::swap(m_hSourceStation, rhs.m_hSourceStation);
        std::swap(m_refTargetStation, rhs.m_refTargetStation);
        std::swap(m_refTargetObject, rhs.m_refTargetObject);
        std::swap(m_pEventHandler, rhs.m_pEventHandler);
        std::swap(m_pResponseEvent, rhs.m_pResponseEvent);
        std::swap(m_subStreamID, rhs.m_subStreamID);
        std::swap(m_responseSubStreamID, rhs.m_responseSubStreamID);
    }

    /*!
    @brief    この @ref DOCallContext の完了時に実行するコールバックを設定します。

    設定したコールバックはこの @ref DOCallContext が完了状態に遷移したときに呼び出されます。
    このコールバックを使用することで、 @ref DOCallContext が完了したかどうかを確認するためにポーリングする手間を省くことができます。

    callback には @ref DOCallContext* を引数に取る関数ポインタ、関数オブジェクト、ラムダ式を指定できます。
    AddCallContextArgumentAdapter() を併用すると引数を一つも取らない関数を渡すこともできます。

    関数オブジェクトを渡す場合は値もしくは参照型で渡す方法とポインタ型で渡す方法の二種類があり、それぞれ挙動が異なります。
    値もしくは参照型で渡した場合は内部で関数オブジェクトがコピーもしくはムーブされ、コールバックはそのコピーもしくはムーブされたオブジェクトに対して行われます。
    そのため、本関数呼び出し後は指定したオブジェクトを即座に破棄する事ができます。
    ポインタ型で渡した場合は、コールバックはそのポインタが指すオブジェクトに対して行われます。
    そのため、コールバックが呼び出されるまでその関数オブジェクトを呼び出し側で保持しておく必要があります。

    本関数を複数回呼び出すことで複数のコールバックを登録することが出来ます。
    複数のコールバックを登録した場合はデフォルトでは登録順にコールバックの呼び出しが行われ、最初に登録されたコールバックは最初に呼び出されます。
    この順序を変更したい場合は、コールバックを登録する際に bAddToEnd パラメータを設定する必要があります。

    callback に関数オブジェクトをポインタで渡した場合は deleteCallbackAfterCall を true にする事で
    callback の呼び出し後に自動的に callback を delete することができます。
    もし callback を delete するのではなくアプリケーション独自の解放処理を行いたい場合は、
    再度本関数を呼び出しそのための関数オブジェクトを追加する事で実装が可能です。

    コールバック関数内では NEX のブロック関数を呼び出さないようにしてください。

    @param[in]  callback コールバックする関数ポインタもしくは関数オブジェクトもしくはラムダ式。
    @param[in]  deleteCallbackAfterCall callback の実行が完了した後自動的に callback を delete するかどうか。
    @param[in]  bAddToEnd true（デフォルト）であれば、コールバックはリストの最後に追加されます。falseであればリストの先頭に追加されます。
    */
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(TCallback&& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<DOCallContext>(std::forward<TCallback>(callback), deleteCallbackAfterCall, bAddToEnd);
    }
#else
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(const TCallback& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<DOCallContext>(callback, deleteCallbackAfterCall, bAddToEnd);
    }
#endif

private:

    // Called by the CallRegister, on registration.
    // {secret}
    void SetID(DOCallContext::ID id);
public:

    // {secret}
    qUnsignedInt GetTargetCount() const;

    // {secret}
    void SignalResponse(UserContext byResponseContext=UserContext());

    // {secret}
    virtual void ProcessResponse(UserContext byResponseContext, State* pResultingState, Outcome* pResultingOutcome);

    // {secret}
    void SignalOutcome(DOHandle hSourceStation, Outcome uOutcome);
    // {secret}
    virtual void ProcessOutcome(DOHandle /*hSourceStation*/, Outcome /*uOutcome*/) {};

    // {secret}
    DOHandle GetSourceStation()
    {
        return m_hSourceStation;
    }

#ifdef NEX_TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif  // NEX_TRACE_ENABLED

    // {secret}
    virtual Type GetType() const=0;
    // {secret}
    void PrepareForDestruction();

    SubStreamID GetResponseSubStreamID()
    {
        return m_responseSubStreamID;
    }


protected:
    // {secret}
    void SignalFailure(Outcome eOutcome);
    // {secret}
    void SetCancelledState();
    // {secret}
    virtual void ProcessCallCompletion();

    void SetResponseSubStreamID(SubStreamID subStreamID)
    {
        m_responseSubStreamID = subStreamID;
    }

private:

    // {secret}
    qBool InternalSetState(State eNewState, Outcome eNewOutcome);
    // {secret}
    qBool InternalCancel(State eNewState, Outcome eNewOutcome);
    // {secret}
    virtual qBool ProcessFault();

    // Deactivate copy-ctor + op=
    // {secret}
    DOCallContext(const DOCallContext&);
    // {secret}
    DOCallContext& operator=(const DOCallContext&);

private:

    // Portion of info filled by the RMCRegister upon
    // registration.
    // {secret}
    ID m_idRMCCall;

    DOHandle m_hSourceStation;


protected:
    // User controlled info.
    // {secret}
    DORef m_refTargetStation;
    DORef m_refTargetObject;

    // Events.
    // {secret}
    EventHandler* m_pEventHandler;
    Event* m_pResponseEvent;

    SubStreamID m_subStreamID;
    SubStreamID m_responseSubStreamID;
};


END_NAMESPACE_NEX

