﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/ParseTree/DOClassID.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/qMap.h>

#include <NetZ/src/Core/DOCore/MethodID.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DupSpace.h>

BEGIN_NAMESPACE_NEX

class RMCContext;
class CallMethodOperation;
class DuplicatedObject;
class Message;
class DOOperation;
class DataSet;
class Adapter; // This is an opaque type, used to support the adapter extension.

class Variable;
class DatasetDeclaration;
class UpdatePolicy;
class StationUpdateFilter;
class GlobalUpdateFilter;
class Station;

// {Secret}
class DOClass: public RootObject
{
public:
    qUnsignedInt32* m_puiGenerateObjectID;

    DOClass(DOClassID uiClassID);
    virtual ~DOClass();

    enum Category
    {
        Static, /* The class is compiled with the application */
        Dynamic, /* The class isn't compiled with the application, but was found in the ParseTree */
        Unknown /* The class isn't compiled nor found in the ParseTree */
    };

    // {secret}
    static Category GetCategory(DOClassID id);
    virtual Category GetCategory()
    {
        return Static;
    }


    // The following methods are called after construction and
    // before destruction, but when the virtual table is set up
    // correctly.
    void CompleteInitialisation();
    void PrepareToLeave();

    virtual void DataSetsOperation(qUnsignedInt32 uiOp);

    virtual DuplicatedObject* Create()=0;
    virtual void Delete(DuplicatedObject* pDO)=0;

    DOClassID GetID() const
    {
        return m_uiClassID;
    }
    // Callbacks...
    virtual qBool ApproveFaultRecovery(DuplicatedObject* pDO)=0;
    virtual qBool ApproveEmigration(DuplicatedObject* pDO, qUnsignedInt32 uiReason)=0;
    virtual void Trace(DuplicatedObject* pDO, qUnsignedInt64 uiTraceFlag)=0;

    /* These two following methods are there to support the adapter
     * extension.  Unless the adapter extension is used, they will
     * never be called.
     */
    virtual Adapter* CreateAdapter(DuplicatedObject* pDO, qUnsignedInt uiIndex);
    virtual void DeleteAdapter(DuplicatedObject* pDO, qUnsignedInt uiIndex, Adapter* pAdapterToDelete);

    virtual void SpecificAddDSToDiscoveryMessage(DuplicatedObject* pDO, Station* pNewStation, Message* pMsg)=0;
    virtual void SpecificExtractDSFromDiscoveryMessage(DuplicatedObject* pDO, Message* pMsg)=0;
    virtual qBool SpecificExtractADataset(DuplicatedObject* pDO, Message* pMsg, qByte byDatasetIndex)=0;
    virtual qResult SpecificUpdate(DuplicatedObject* pDO, DataSet *pDataset, const Time& tUpdateTime)=0;
    virtual qBool SpecificRefresh(DuplicatedObject* pDO, DataSet *pDataset, const Time& tRefreshTime)=0;

    virtual qBool IsAKindOf(DOClassID idDOCID)
    {
        if (DuplicatedObject_ClassID == idDOCID)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    qBool MightRequestRemoteID() const;
    qBool GenerateObjectID( qUnsignedInt32 *uiRetID, qUnsignedInt32 uiTimeout = 3000 );
    qBool CreateIDGenerator();

    virtual const qChar* GetClassNameString() const=0;
    virtual const qChar* GetDatasetNameString(qByte /*byDatasetIndex*/) const
    {
        return NEX_T("Invalid dataset index");
    };

    DOHandle GetIDGenerator() const;

    // Access to variables via ParseTree.
    virtual qBool FormatVariableValue(const DuplicatedObject* pDO,
                                      Variable* pDataset,
                                      Variable* pVariable, String *pstrResult) const
    {
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(pDataset);
        DUMMY_PARAM(pVariable);
        DUMMY_PARAM(pstrResult);
        return false;
    }

    // Actions...
    virtual qBool DispatchAction(DuplicatedObject* pDO, MethodID idMethod, Message* pMsg)
    {
        DUMMY_PARAM(pDO);
        DUMMY_PARAM(idMethod);
        DUMMY_PARAM(pMsg);
        return false;
    }

    // RMCs...
    virtual void DispatchRMCCall(const CallMethodOperation& refOp);
    virtual qBool DispatchRMCResult(RMCContext* pContext);

    virtual qBool ValidCastTowards(qUnsignedInt32 uiClassID)
    {
        DUMMY_PARAM(uiClassID);
        // If we get here, it is not a valid cast.
        return false;
    }

    static DOClass* FindDOClass(qUnsignedInt32 uiClassID);

    qBool SetIDGeneratorStartupID( qUnsignedInt32 uiStartupID );

    virtual void FillDupSpacesInfo(DupSpace::Role eRole,
                                   DupSpace::ID* pArray, qUnsignedInt *puiIndex);

    UpdatePolicy* GetUpdatePolicy(qByte byIndex) const
    {
        qMap<qByte,UpdatePolicy*>::const_iterator it = m_mapUpdatePolicy.find(byIndex);
        SYSTEMCHECK(it!=m_mapUpdatePolicy.end());
        return (*it).second;
    }
    void CreateUpdatePolicy(qByte byIndex);

private:

    qBool GenerateObjectIDCouldBlock(qUnsignedInt32 uiTimeout) const;

private:
    qUnsignedInt32  m_uiClassID;
    qInt32          m_iBootstrapID;
    qMap<qByte,UpdatePolicy*> m_mapUpdatePolicy;
};

// Group = Miscellaneous Types and Macros

/*
Summary: Returns the DOClassID associated to a DOClass. For example,
DOCLASSID(Avatar) returns the DOClassID for the DOClass Avatar.
*/
#define DOCLASSID(DO)   _DO_##DO::GetStaticClassID()

END_NAMESPACE_NEX

