﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/ParseTree/DOClassID.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>

#if NEX_ENABLE_DOFILTER_MINMAX_CACHE_OPTIMIZATION
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#endif  // NEX_ENABLE_DOFILTER_MINMAX_CACHE_OPTIMIZATION

#include <cstddef>
#include <algorithm>

BEGIN_NAMESPACE_NEX
// Group=Object Duplication Subsystem

class DOClassDeclaration;
class DOClass;
class ParseTree;

class DOClassesTable: public RootObject
{
    // Group=
public:
    static const std::size_t DOCLASSTABLE_VALID_HANDLE_INIT_SIZE = 20;
    DOClassesTable();
    ~DOClassesTable();

    typedef qVector< qByte > DOClcassIDVector;

    static inline DOClass* GetDOClass(DOClassID id)
    {
        DOClass* pDOClass = GetDOClassesTable()->FindDOClass(id);
        //TODO bug 4792: make sure the "see refman page ..." in the comment below is pointing to the correct refman page.
        VERBOSESYSTEMCHECK(pDOClass!=NULL,
                           QTRACE(TRACE_SYSTEMCHECK, NEX_T("This SYSTEMCHECK is normally caused by compiling multiple DDL files and not having use or include in the main DDL file of all other DDL files. The main DDL files is the file that contains the title statement. See \"DDL Preprocessor\" in the reference manual for more information.")
                                  << NEX_T("Failed on GetDOClass(") << hex << id << NEX_T(")")) );
        return pDOClass;
    }

    static void CreateDOClassesTable()
    {
        if (!s_poTheTable.GetValue())
        {
            s_poTheTable=qNew DOClassesTable();
        }
    }

    static DOClassesTable* GetDOClassesTable()
    {
        return s_poTheTable.GetValue();
    }

    static void TearDownDOClassesTable();

    void RegisterDOClass(qUnsignedInt32 uiClassID, DOClass* pDOClass);

    inline DOClass* FindDOClass(qUnsignedInt32 uiClassID)
    {
        if (uiClassID>=m_uiCurrentSize)
        {
            return NULL;
        }
        return m_ppTable[uiClassID];
    }

    qUnsignedInt32 GetCurrentSize()
    {
        return m_uiCurrentSize;
    }

    DOClass* GetDOClassByIndex(qUnsignedInt32 uiIndex)
    {
        SYSTEMCHECK(uiIndex<m_uiCurrentSize);
        return m_ppTable[uiIndex];
    }

    qUnsignedInt32 GetNextIndex(qUnsignedInt32 uiIndex)
    {
        for(; uiIndex<m_uiCurrentSize; uiIndex++)
        {
            if (m_ppTable[uiIndex] == NULL)
            {
                break;
            }
        }

        SYSTEMCHECK(uiIndex <= MAX_DOCLASSID && "Too many doclasses");
        return uiIndex;
    }


    void CreateObjectIDGenerators();

    qUnsignedInt32 GetNbDOClasses();
    qUnsignedInt32 GetLastClassID();

#if NEX_ENABLE_DOFILTER_MINMAX_CACHE_OPTIMIZATION

    /**
     * @brief 指定されたDOクラスIDの種類であるものの中で、ハンドル値のとりうる最大値を取得します。
     * @return とりうる最大値が返ります。
     */
    DOHandle GetMaximumValidHandle( DOClassID idDOCID )
    {
        qMap< DOClassID, MinMaxHandleCache >::const_iterator it = m_mapHandleCache.find( idDOCID );
        if ( it == m_mapHandleCache.end() )
        {
            BuildHandleCache( idDOCID );
            it = m_mapHandleCache.find( idDOCID );
            SYSTEMCHECK( it != m_mapHandleCache.end() );
        }
        SYSTEMCHECK( it->first == idDOCID );
        return it->second.maximumHandle;
    }

    /**
     * @brief 指定されたDOクラスIDの種類であるものの中で、ハンドル値のとりうる最小値を取得します。
     * @return とりうる最小値が返ります。
     */
    DOHandle GetMinimumValidHandle( DOClassID idDOCID )
    {
        qMap< DOClassID, MinMaxHandleCache >::const_iterator it = m_mapHandleCache.find( idDOCID );
        if ( it == m_mapHandleCache.end() )
        {
            BuildHandleCache( idDOCID );
            it = m_mapHandleCache.find( idDOCID );
            SYSTEMCHECK( it != m_mapHandleCache.end() );
        }
        SYSTEMCHECK( it->first == idDOCID );
        return it->second.minimumHandle;
    }

#endif  // NEX_ENABLE_DOFILTER_MINMAX_CACHE_OPTIMIZATION

    const DOClcassIDVector& GetValidSortedDOClassIDs()
    {
        if (!isSortedValidDOClassID) //ソートが必要か
        {
            isSortedValidDOClassID=true;
            std::sort(m_ValidDOClassID.begin(), m_ValidDOClassID.end());
        }
        return m_ValidDOClassID;
    }

private:
    qUnsignedInt32 m_uiCurrentSize;
    DOClass** m_ppTable;

#if NEX_ENABLE_DOFILTER_MINMAX_CACHE_OPTIMIZATION
    // n2370:   ハンドルの最大値と最小値をキャッシュします。
    //          取得するタイミングで、キャッシュが存在している場合はそこから取得し、取得できなければ再計算されます。
    //          キャッシュ情報は、RegisterDOClassされるとクリアされます。
    //          最大値最小値をキャッシュしておくことで、負荷が大きいIsAKindOfDOFilterを高速化することができます。

    void BuildHandleCache( DOClassID idDOCID );

    /**
     * @brief ハンドルの最大最小をキャッシュするための構造体
     */
    struct MinMaxHandleCache
    {
        //! ハンドルの最大値
        DOHandle maximumHandle;

        //! ハンドルの最小値
        DOHandle minimumHandle;
    };

    //! ハンドルのマップ
    qMap< DOClassID, MinMaxHandleCache > m_mapHandleCache;


#endif  // NEX_ENABLE_DOFILTER_MINMAX_CACHE_OPTIMIZATION
    //! 有効なDOClassIDの一覧
    DOClcassIDVector m_ValidDOClassID;
    qBool isSortedValidDOClassID;

    // Variable used to implement the singleton...
    OSDK_DISABLE_EXPORT_WARNING(static PseudoGlobalVariable<DOClassesTable*> s_poTheTable);
};

END_NAMESPACE_NEX

