﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>

BEGIN_NAMESPACE_NEX

// The instances it manages.
class StationConnectionManager;
class DOSelections;
class ObjDupProtocol;
class StationManager;
class SessionDiscoveryTable;
class OperationManager;
class SystemComponent;
class ErrorDescriptionTable;
class BundlingPolicy;
class StationIdentification;
class SafetyExecutive;
class DOSubset;
class ConnectionOrientedStream;
class FirstErrorHolder;

//n2788
class DirectTransport;
class DirectStream;


#define _NEX_MANAGEINSTANCE(TYPE,NAME)      \
    public:                             \
        const TYPE* Get##NAME() const {return m_p##NAME;}       \
        TYPE* Get##NAME() {return m_p##NAME;}               \
    private:                                \
        TYPE *m_p##NAME

#define _NEX_MANAGEGLOBALINSTANCE(TYPE,NAME)    \
    public:                                 \
        static TYPE* Get##NAME() {return s_p##NAME;}                \
    private:                                \
        static TYPE *s_p##NAME


#define NEX_MANAGEINSTANCE(C)   _NEX_MANAGEINSTANCE(C,C)
#define NEX_MANAGEGLOBALINSTANCE(C) _NEX_MANAGEGLOBALINSTANCE(C,C)


/**
 * @brief   複製オブジェクトの関連の設定を行うためのクラスです。
 *
 * このクラスはシステムによって自動的にインスタンスが作成され、破棄されます。
 * @ref NetZ インスタンスが生成されているときに、
 * @ref DOCore::GetInstance() で本クラスオブジェクトへのポインタを取得できます。
 */
class DOCore: public PseudoSingleton
{
public:
    /*
    Description: Returns a pointer to the DOCore instance.
    Returns: A pointer to the DOCore object.
    */

    /*!
     * @brief   DOCoreインスタンスへのポインタを返します。
     *
     * DOCoreオブジェクトを操作するためには、この関数が返す参照を使用して下さい。
     *
     * @ref NetZ インスタンス生成後に @ref DOCore オブジェクトへのポインタを取得できます。
     * @ref NetZ インスタンス生成前、破棄後はNULLを返します。
     * @return  DOCoreインスタンスへのポインタ。
     */
    static DOCore* GetInstance()
    {
        return (DOCore*) InstanceControl::FindInstance(IC_TYPE_DOCORE, PseudoSingleton::GetCurrentContext());
    }

    DOCore(FirstErrorHolder *pFirstErrorHolder = NULL);
    virtual ~DOCore();

#ifdef NEX_TRACE_ENABLED    // n2370
    void TraceSystemState(qUnsignedInt64 uiTraceFlag);
#endif  // NEX_TRACE_ENABLED

    // GetSafetyExecutive
    NEX_MANAGEINSTANCE(SafetyExecutive);
    // this is connection pool. DOCore initiates this instance, but it's not used from in this class.
    // GetStationConnectionManager
    NEX_MANAGEINSTANCE(StationConnectionManager);
    // GetStationManager
    NEX_MANAGEINSTANCE(StationManager);
    // GetDOSelections
    NEX_MANAGEINSTANCE(DOSelections);
    // GetObjDupProtocol
    NEX_MANAGEINSTANCE(ObjDupProtocol);
    // GetSessionDiscoveryTable
    NEX_MANAGEINSTANCE(SessionDiscoveryTable);
    // GetBundlingPolicy
    NEX_MANAGEINSTANCE(BundlingPolicy);
    // GetStationIdentification
    NEX_MANAGEINSTANCE(StationIdentification);
    // GetStream()
    _NEX_MANAGEINSTANCE(ConnectionOrientedStream,Stream);
    // GetDOOperationManager
    _NEX_MANAGEINSTANCE(OperationManager,DOOperationManager);
    // GetJoinProcessing
    _NEX_MANAGEINSTANCE(SystemComponent,JoinProcessing);
    // GetDOCoreState
    _NEX_MANAGEINSTANCE(SystemComponent,DOCoreState);
    // GetUserDOs
    _NEX_MANAGEINSTANCE(DOSubset,UserDOs);

    // The 'fault processing' system state is in place and set while
    // we process faults, but never initialized and tested for, in
    // the core, yet.
    // GetFaultProcessing
    _NEX_MANAGEINSTANCE(SystemComponent,FaultProcessing);

    //n2788:GetDirectTransport
    NEX_MANAGEINSTANCE(DirectTransport);
    NEX_MANAGEINSTANCE(DirectStream);

public:
    // Job queueuing.  Should be moved elsewhere.
    void QueuePeriodicJobs();
    void CancelPeriodicJobs();

    qBool IsReadyToLeave();

    void SetToReadyState(); // After a successfull join/create session.
    void SetToTerminatingState(); // When JobClearTheStore begins.
    void SetToTerminatedState(); // When JobClearTheStore ends.
    void SetToCorruptedState();

    qBool IsInNormalOperation() const;
    qBool IsTerminating() const;
    qBool IsTerminated() const;
    qBool IsCorrupted() const;

    // Always stays true, once termination has been initiated.
    qBool HasStartedTermination() const;

    // fatal error
    void SetFatalError(qResult err);
    FirstErrorHolder *GetFirstFatalErrorHolder() const
    {
        return m_pFirstErrorHolder;
    }
    qBool IsFaultRecoveryJobProcessFailed()
    {
        return m_isFaultRecoveryJobProcessFailed;
    }
    void SetIsFaultRecoveryJobProcessFailed()
    {
        m_isFaultRecoveryJobProcessFailed=true;
    }

    /*!
    @brief ユーザーDOのReliable通信における最大送信バッファ数を取得します。

    送信処理で蓄積された送信バッファ数が本設定値にまで達していた場合、送信処理はReliableSendBufferFullのエラーを返します。<br>
    初期値は32です。

    @return ユーザーDOのReliable通信における最大送信バッファ数を返します。
    */
    size_t GetMaxUserDOReliableSendBufferNum() const
    {
        return m_maxUserDoReliableSendBufferNum;
    }
    /*!
    @brief ユーザーDOのReliable通信における最大送信バッファ数を設定します。

    送信処理で蓄積された送信バッファ数が本設定値にまで達していた場合、送信処理はReliableSendBufferFullのエラーを返します。<br>
    蓄積された送信バッファ数がライブラリの上限を超える場合は、@ref NetZ::GetFatalError でReliableSendBufferFullFatalのフェイタルエラーが発生しますので、
    設定の最大値は96程度を目安としてください。

    @param[in] bufferNum ユーザーDOのReliable通信における最大送信バッファ数を指定します。
    */
    void SetMaxUserDOReliableSendBufferNum(size_t bufferNum)
    {
        m_maxUserDoReliableSendBufferNum = bufferNum;
    }

    /*!
    @brief ユーザーRMCのReliable通信における最大送信バッファ数を取得します。

    送信処理で蓄積された送信バッファ数が本設定値にまで達していた場合、送信処理はReliableSendBufferFullのエラーを返します。<br>
    初期値は48です。

    @return ユーザーRMCのReliable通信における最大送信バッファ数を返します。
    */
    size_t GetMaxUserRMCReliableSendBufferNum() const
    {
        return m_maxUserRMCReliableSendBufferNum;
    }
    /*!
    @brief ユーザーRMCのReliable通信における最大送信バッファ数を設定します。

    送信処理で蓄積された送信バッファ数が本設定値にまで達していた場合、送信処理はReliableSendBufferFullのエラーを返します。<br>
    蓄積された送信バッファ数がライブラリの上限を超える場合は、@ref NetZ::GetFatalError でReliableSendBufferFullFatalのフェイタルエラーが発生しますので、
    設定の最大値は96程度を目安としてください。

    @param[in] bufferNum ユーザーRMCのReliable通信における最大送信バッファ数を指定します。
    */
    void SetMaxUserRMCReliableSendBufferNum(size_t bufferNum)
    {
        m_maxUserRMCReliableSendBufferNum = bufferNum;
    }

private:
    static qUnsignedInt32 s_uiDOCoreCount;
    static ErrorDescriptionTable s_oErrorTable;

    void CreateGlobals();
    void DeleteGlobals();

    // fatal error
    FirstErrorHolder *m_pFirstErrorHolder;


    qBool m_isFaultRecoveryJobProcessFailed;

    size_t m_maxUserDoReliableSendBufferNum; // reliable送信バッファ数
    size_t m_maxUserRMCReliableSendBufferNum; // reliable送信バッファ数

};

#undef _NEX_MANAGEINSTANCE
#undef _NEX_MANAGEGLOBALINSTANCE
#undef NEX_MANAGEINSTANCE
#undef NEX_MANAGEGLOBALINSTANCE

END_NAMESPACE_NEX

