﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/DDLTypes/OnlineCoreTypes.h>
#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/MemberList.h>
#include <NetZ/src/Core/DOCore/MemberQueue.h>
#include <NetZ/src/Core/DOCore/MemberVector.h>

#include <NetZ/src/Core/DOCore/DOOperation.h>

////////////////////////////////////////////////////////////////////////////////
// This file defines the built-in types of the DDL compiler. Note that those
// types must also be declared in the following files:
//
// - \src\ObjDup\DOCoreTypes.ddl
// - \src\ddl\CompilerDriver\CompilerDriver.cpp (DeclareBuiltInTypes method)
////////////////////////////////////////////////////////////////////////////////

BEGIN_NAMESPACE_NEX


//==============================================================================
// DECLARE_MEMBERCONTAINER_DDLTYPE macro defines a C++ DDLType class for the
// specified DDLTYPENAME DDL custom variable type (member containers). The class
// allows to parse a variable of that type to and from (Add/Extract) a NetZ
// message. Therefore, it is expected that the CPPTYPENAME C++ class name
// inherits from the MemberContainer class.
//
// [bug:4768] changes the Add and Extract method so that it calls the
// AddSourceTo and ExtractFrom methods of the MemberContainer class.
//
// [bug 4837] OperationEnd switches the duplication mode by content when
// detecting a change in master station (network fault).
//==============================================================================
#define DECLARE_MEMBERCONTAINER_DDLTYPE(DDLTYPENAME, CPPTYPENAME)               \
template< class TDDL_Type >                                                     \
class DDLTYPE(DDLTYPENAME) : public DDLType {                                   \
public:                                                                         \
    typedef CPPTYPENAME<TDDL_Type> CPPType;                                     \
    static void Add(Message *pMessageToSend, const CPPType &oValue, qBool bCompleteUpdate = true) { \
        const_cast<CPPType&>(oValue).AddSourceTo(pMessageToSend, bCompleteUpdate);                  \
    }                                                                           \
    static void Extract(Message *pReceivedMessage, CPPType *pValue) {           \
        pValue->ExtractFrom(pReceivedMessage);                                  \
    }                                                                           \
    static void OperationBegin(void* pVariable, void* pDOOperation) {           \
        DUMMY_PARAM(pVariable); DUMMY_PARAM(pDOOperation);                      \
    }                                                                           \
    static void OperationEnd(void* pVariable, void* pDOOperation) {             \
        if (static_cast<DOOperation*>(pDOOperation)->GetType() == Operation::ChangeMasterStation) {\
            static_cast<CPPType*>(pVariable)->ClearOperations();                \
            static_cast<CPPType*>(pVariable)->SetContentUpdate();               \
        }                                                                       \
    }                                                                           \
};

////////////////////////////////////////////////////////////////////////////////
// Important: Changing or adding a new membercontainer type names here implies
// updating Attribute::IsAContainer() implementation in /src/ddl/BackEnd/
////////////////////////////////////////////////////////////////////////////////
DECLARE_MEMBERCONTAINER_DDLTYPE(memberlist, MemberList)
DECLARE_MEMBERCONTAINER_DDLTYPE(memberqueue, MemberQueue)
DECLARE_MEMBERCONTAINER_DDLTYPE(membervector, MemberVector)

// Doxygenでのドキュメント生成時に
// DOCoreTypes.h:107: warning: Found ';' while parsing initializer list! (doxygen could be confused by a macro call without semicolon)
// という警告が出るので抑止する
#ifndef FOR_DOC_PURPOSE
//==============================================================================
// The DDLTYPE(dohandle) class allows to parse a variable of the "dohandle" type
// to and from (Add/Extract) a NetZ message. The DDL "dohandle" type is a special
// type because of the way the value is formatted in a text string.
//==============================================================================

class OSDK_API DDLTYPE(dohandle): public DDLType
{
public:
    typedef DOHandle CPPType;
    static void Add(Message* pMessageToSend, const DOHandle& oValue)
    {
        (*pMessageToSend) << oValue;
    }
    static void Extract(Message* pReceivedMessage, DOHandle* pValue)
    {
        (*pReceivedMessage) >> *pValue;
    }
    static qBool FormatVariableValue(const void *pVariable, String* pstrValue)
    {
        SYSTEMCHECK(pVariable != NULL);
        const DOHandle hDO = *(DOHandle*)pVariable;
        pstrValue->Format(NEX_T("%s %x"), hDO.GetClassNameString(), hDO.GetValue());
        return true;
    }
};
#endif


END_NAMESPACE_NEX

