﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/Type.h>

BEGIN_NAMESPACE_NEX

// Group = ID Classes

/*
Description: Defines a duplicated object ID.
Remarks: A DOID is a unique identification (22-bit value) within a duplicated object class. It is
assigned to an object upon creation and may be either generated automatically by
$PRODUCTNAME$ or set by the user.

If DOID s are set by the user, the user must ensure that within each duplicated object
class each DOID is unique.

The DOHandle of a duplicated object is made up of the object's DOClassID and its DOID.
See Also: DuplicatedObject::Create
*/

/*!
 * @brief   複製オブジェクトのID表すクラスです。
 *
 * 複製オブジェクトIDは、複製オブジェクトクラスの中で複製オブジェクトのインスタンスを一意に示す22bitの識別子です。
 * この、複製オブジェクトIDは、Net-Zによって生成順に自動的に割り当てられるか、ユーザーによってセットされます
 * (IDを割り当てるために@ref IDGenerator クラスを利用します)。
 *
 * もし複製オブジェクトIDがユーザーによってセットされる場合、
 * 同一の複製オブジェクトクラスの中では必ず一意になるように割り当てる必要があります。
 *
 * 複製オブジェクトハンドル(@ref DOHandle)はこのクラスと、複製オブジェクトクラスID(@ref DOClassID)に
 * よって構成されます。
 *
 * @see     DuplicatedObject::Create
 */
class DOID : public RootObject
{
public:
    enum
    {
        //! 無効なIDであることを示す数値です。
        InvalidID = 0
    };

    /*
    Parameters:
    nID: A duplicated object ID.
    */

    /*!
     * @refmanoverload{name1}
     *
     * @brief   複製オブジェクトIDを作成します。
     *
     * コンストラクタになにも指定しない場合、
     * この複製オブジェクトIDは無効なIDとして初期化されます。
     */
    DOID() : m_nID(static_cast<ID>(InvalidID)) {}

    /*!
     * @refmanoverload{name2}
     *
     * @brief   複製オブジェクトIDを作成します。
     *
     * 指定されたIDの値をコピーして初期化します。
     *
     * @param   doid [in] コピー元の複製オブジェクトIDを指定します。
     */
    DOID(const DOID& doid) : m_nID(doid.m_nID) {}

    /*!
     * @brief   複製オブジェクトIDを作成します。
     *
     * 整数値を指定してそれをID値として利用し、初期化します。
     *
     * @param   nID [in] この複製オブジェクトIDにセットする値を指定します。
     */
    explicit DOID(qUnsignedInt32 nID) : m_nID(static_cast<ID>(nID)) {}

    const DOID &operator=( const DOID &idOther )
    {
        m_nID = idOther.m_nID;
        return *this;
    }

    /*!
     * @brief   この複製オブジェクトIDを符号なし32bit値として取得します。
     * @return  この複製オブジェクトIDを符号なし32bit値としたものが返ります。
     */
    operator qUnsignedInt32 () const
    {
        return GetValue();
    }

    /*!
     * @brief   この複製オブジェクトIDを符号なし32bit値として取得します。
     * @return  この複製オブジェクトIDを符号なし32bit値としたものが返ります。
     */
    qUnsignedInt32 GetValue() const
    {
        return static_cast<qUnsignedInt32>(m_nID);
    }

private:
    ID m_nID;
};

END_NAMESPACE_NEX


