﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <OnlineCore/src/Core/Operation.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

/*
Description: Encapsulates duplicated object operations.
Remarks: The types of DuplicatedObject operations are AddToStore, ChangeMasterStation,
CallMethod, FaultRecovery, RemoveFromStore, ChangeDupSet, and UpdateDataSet. All
DOOperations pass via the SafetyExecutive to ensure that the given operation is permitted
before proceeding.

Refer to DuplicatedObject::OperationEnd for an example of how DOOperations can be used.
*/

/*!
 * @brief   複製オブジェクトに対する操作をカプセル化します。
 *
 * このクラスは、システム操作クラスから派生して、
 * かつそれぞれの複製オブジェクトに対する操作の基底クラスとなります。
 * このクラスから派生して、アプリケーションから利用可能なクラスは
 * @ref AddToStoreOperation , @ref ChangeMasterStationOperation , @ref CallMethodOperation ,
 * @ref FaultRecoveryOperation , @ref RemoveFromStoreOperation ,
 * @ref UpdateDataSetOperation となります。
 *
 * 操作が実行される前には、本当に実行してもよいかを確認するために
 * @ref SafetyExecutive が実行されます。
 *
 * @see     DuplicatedObject::OperationBegin, DuplicatedObject::OperationEnd
 */
class DOOperation: public Operation
{
    //Group=Methods
protected:
    // The operations are always derived from

    //{secret}
    DOOperation(DOHandle hOperationOrigin, DuplicatedObject* pDO);
    //{secret}
    DOOperation(DOHandle hOperationOrigin, DOHandle hTarget);

public:
    //{secret}
    virtual ~DOOperation();

    /*
    Description: Returns the DOHandle of the duplicated object that is associated with this operation.
    Returns: A DOHandle to the DuplicatedObject instance that is associated with the operation.
    See Also: GetAssociatedDO, DOHandle, DuplicatedObject
    */

    /*!
     * @brief   この操作に関連付けられている複製オブジェクトハンドルを取得します。
     * @see         GetAssociatedDO, DOHandle, DuplicatedObject
     * @return  この操作に関連付けられている複製オブジェクトハンドル（@ref DOHandle ） が返ります。
     */
    DOHandle GetAssociatedDOHandle() const
    {
        return m_refAssociatedDO.GetHandle();
    }

    /*
    Description: Returns a DOHandle to the Station that initiated the call that resulted
    in the Operation being called.
    Returns: A DOHandle to the Station that initiated the call that resulted in the
    Operation being called.
    Remarks: This method returns a DOHandle to the Station that directly triggered the
    Operation. For example, if a given Station calls DuplicatedObject::FetchDuplica which
    triggers a ChangeDupSetOperation, then this method will return a handle to the station
    where DuplicatedObject::FetchDuplica was called.

    This method is typically used by the SafetyExecutive in order to determine if the station that
    triggers an Operation has the right to do so.

    Refer to CallMethodOperation for an example of this method in use.
    See Also: SafetyExecutive
    */

    /*!
     * @brief   操作オブジェクトを結果として生じるような呼び出しを行ったステーションへの複製オブジェクトハンドルを取得します。
     *
     * たとえば、ステーションが@ref DuplicatedObject::FetchDuplica 関数を
     * 呼び出して@ref ChangeDupSetOperation 操作を誘発させた場合、
     * この関数は、@ref DuplicatedObject::FetchDuplica 関数を呼び出した
     * ステーションのDOHandleが返ります。
     *
     * この関数の典型的な利用例は、@ref SafetyExecutive にて、
     * そのステーションが操作を実際に行うための権利があるかどうかを判定するために
     * 使用することです。
     *
     * @return  操作オブジェクトを生じさせるような呼び出しを行ったステーションへの複製オブジェクトハンドルが返ります。
     * @see     SafetyExecutive
     */
    DOHandle GetOrigin() const
    {
        return Operation::GetOrigin();
    }


    /*
    Description: Returns a pointer to the DuplicatedObject instance that is associated with this operation.
    Returns: A DuplicatedObject pointer to the instance that is associated with the operation.
    See Also: GetAssociatedDOHandle, DuplicatedObject
    */

    /*!
     * @brief   操作に関連付けられている複製オブジェクトへのポインタを取得します。
     * @return  操作に関連付けられている複製オブジェクトへのポインタ(@ref DuplicatedObject)が返ります。
     * @see     GetAssociatedDOHandle, DuplicatedObject
     */
    DuplicatedObject* GetAssociatedDO() const
    {
        return m_refAssociatedDO.GetDOPtr();
    }

public:

    // {secret}
    virtual DOHandle GetImplicitStationConnection() const
    {
        return INVALID_DOHANDLE;
    }

    // {secret}
    virtual DOOperation* Clone() const
    {
        return NULL;
    }

public:

    // {secret}
    virtual qBool CallsBackOnDataSet()=0;
    // {secret}
    virtual qBool CallsBackOnDataSet(qByte byIndex)=0;

protected:

    DuplicatedObject::Ref m_refAssociatedDO;
};

//{secret}
extern qBool FilterOutCoreDOOperations(const Operation* pOperation);


END_NAMESPACE_NEX

