﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Core/RefBuffer.h>
#include <OnlineCore/src/Core/SystemComponent.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DOCore.h>

BEGIN_NAMESPACE_NEX

class Job;
/*!
@brief 複製オブジェクトを使用しない単純な送受信(ダイレクトストリーム)機能を扱うクラスです。
*/
class DirectStream: public SystemComponent
{
public:
    /*!
    @brief DirectStreamで扱う通信の種類です。
    */
    enum StreamType
    {
        Unreliable, //!< 非信頼性通信
        Reliable //!< 信頼性通信
    };
    /*!
    @brief DirectStreamでの受信情報です。
    */
    struct Data : public RootObject
    {
        StationID id; //!< 送信元のStationID
        StreamType type; //!< 通信タイプ
        SubStreamID subStreamID; //!<Reliable通信時のSubStreamID
        RefBuffer buffer; //!< 受信バッファ
    };
public:

    NEX_DECLARE_SYSCOMPONENT_METHODS(DirectStream,SystemComponent);

    /*!
    @brief このコンストラクタはアプリケーション側から直接呼び出さないでください。DirectStreamクラスのインスタンスは、システムによって自動的に生成されます。
     */
    DirectStream();
    /*!
    @brief このデストラクタはアプリケーション側から直接呼び出さないでください。DirectStreamクラスのインスタンスは、システムによって自動的に破棄されます。
     */
    virtual ~DirectStream();
    /*!
    @brief DirectStreamのインスタンスを取得します。
    
    NetZが生成されていない状態では、NULLとなります。
    NetZオブジェクトが正常に生成されていれば、本関数の返り値はNULLでないことを期待できます。

    @return DirectStreamオブジェクトへのポインタが返ります。
     */
    static DirectStream* GetInstance()
    {
        if (DOCore::GetInstance() == NULL)
        {
            return NULL;
        }
        return DOCore::GetInstance()->GetDirectStream();
    }

public:
    /*!
    @brief DirectStreamの機能を有効化します。

    ダイレクトストリームを使用して送受信を行う前に必ず呼び出してください。<br>
    有効化してない状態では送信処理はエラーを返します。<br>
    有効化するまでに受信したパケットはライブラリ内で破棄されます。
    */
    void Enable();
    /*!
    @brief DirectStreamの機能を無効化します。

    無効化しても、すでに送信用キューに蓄積された送信データがあれば引き続き送信され続けます。
    受信においてはReliable通信に対する到達確認の応答がライブラリ内部で行われます。
    */
    void Disable();

    /*!
    @refmanoverload{name1}

    @brief 特定のステーションへデータを送信します。

    StationIDは@ref Station::GetStationID で取得できます。
    Reliable通信を指定した送信では、送信バッファに空きがない場合にReliableSendBufferFullエラーを返します。
    これは一時的に送信バッファの上限に達したことを示すエラーであるため、
    ReliableSendBufferFullが発生した場合は、ディスパッチを行い時間をおいてから再度送信を行ってください。

    パケットバッファに空きがない場合PacketBufferFullエラーを返します。
    これは一時的にパケットバッファが不足していることを示すエラーであるため、
    ディスパッチを行い時間をおいてから再度送信を行ってください。
    高頻度でPacketBufferFullが発生する場合は、パケットバッファを増やす対策も行ってください。
    パケットバッファの設定については、@ref PacketBufferManager を参照してください。

    @param[in] id 宛先のステーションのStationIDを指定します。
    @param[in] type 通信の種類を指定します。
    @param[in] pBuffer 送信するデータを指定します。
    @param[in] size 送信するデータのサイズを指定します。Reliable通信では32768以下、それ以外の通信では1300(P2Pトポロジー以外の場合は1250)以下となるようにしてください。
    @retval "QSUCCESS(Core, Success)" 送信リクエストが成功した場合
    @retval "QERROR(Core, InvalidSequence)" ダイレクトストリームを有効化してない場合
    @retval "QERROR(Transport, Unknown)" PRUDPUserProtocol::GetInstance()がNULLの場合
    @retval "QERROR(Core, CallInitiationFailure)" 呼び出しコンテキストの初期化に失敗した場合
    @retval "QERROR(Core, InvalidArgument)" 引数の値が不正の場合 (pBufferがNULLであるか、sizeが超過しています。)
    @retval "QERROR(Transport, ConnectionFailure)" 宛先Stationとの接続が確立されていなかった場合
    @retval "QERROR(Transport, InvalidStation)" 宛先Stationが存在しなかった場合 (接続処理中であるか、すでに存在していません。)
    @retval "QERROR(Transport, ReliableSendBufferFull)" Reliable通信の送信バッファが一杯の場合。アプリケーションによるハンドリングが必要です。（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
    @retval "QERROR(Transport, PacketBufferFull)" パケットバッファ枯渇により送信できなかった場合。アプリケーションによるハンドリングが必要です。 （本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
    */
    qResult Send(StationID id, StreamType type, const qByte* pBuffer, size_t size)
    {
        return Send(id, type, 0, pBuffer, size);
    }
    /*!
    @refmanoverload{name2}

    @brief 特定のステーションへデータを送信します。

    呼び出しコンテキストを指定することで、Reliable通信の到達確認を非同期処理で検知することができます。
    Reliable通信の場合、到達確認ができた場合にSuccessとなります。到達確認ができるまでは完了しません。
    Reliable通信以外では、呼び出しコンテキストは呼び出し終了時点で完了状態となります。
    呼び出しコンテキストをキャンセルしても、送信リクエストが成功したReliable通信の送信は継続されます。<br>
    StationIDは@ref Station::GetStationID で取得できます。
    呼び出しコンテキストが終了した時の処理結果は、@ref CallContext::GetOutcome で取得できます。
    呼び出しコンテキストの処理結果については、返り値qResult型の@ref Send の返り値を参照してください。

    @param[in,out] pContext 呼び出しコンテキストを指定します。
    @param[in] id 宛先のステーションのStationIDを指定します。
    @param[in] type 通信の種類を指定します。
    @param[in] pBuffer 送信するデータを指定します。
    @param[in] size 送信するデータのサイズを指定します。Reliable通信では32768以下、それ以外の通信では1300(P2Pトポロジー以外の場合は1250)以下となるようにしてください。
    @retval true 送信リクエストが成功した場合
    @retval false 送信リクエストが失敗した場合
    */
    qBool Send(CallContext *pContext, StationID id, StreamType type, const qByte* pBuffer, size_t size)
    {
        return Send(pContext, id, type, 0, pBuffer, size);
    }

    /*!
    @brief 特定のステーションへUnreliableデータを送信します。

    StationIDは@ref Station::GetStationID で取得できます。

    パケットバッファに空きがない場合PacketBufferFullエラーを返します。
    これは一時的にパケットバッファが不足していることを示すエラーであるため、
    必要に応じてパケットバッファを増やしてください。パケットバッファの設定については、@ref PacketBufferManager を参照してください。

    @param[in] id 宛先のステーションのStationIDを指定します。
    @param[in] pBuffer 送信するデータを指定します。
    @param[in] size 送信するデータのサイズを指定します。1300(P2Pトポロジー以外の場合は1250)以下となるようにしてください。
    @retval "QSUCCESS(Core, Success)" 送信リクエストが成功した場合
    @retval "QERROR(Core, InvalidSequence)" ダイレクトストリームを有効化してない場合
    @retval "QERROR(Transport, Unknown)" PRUDPUserProtocol::GetInstance()がNULLの場合
    @retval "QERROR(Core, CallInitiationFailure)" 呼び出しコンテキストの初期化に失敗した場合
    @retval "QERROR(Core, InvalidArgument)" 引数の値が不正の場合 (pBufferがNULLであるか、sizeが超過しています。)
    @retval "QERROR(Transport, ConnectionFailure)" 宛先Stationとの接続が確立されていなかった場合
    @retval "QERROR(Transport, InvalidStation)" 宛先Stationが存在しなかった場合 (接続処理中であるか、すでに存在していません。)
    @retval "QERROR(Transport, PacketBufferFull)" パケットバッファ枯渇により送信できなかった場合。アプリケーションによるハンドリングが必要です。 （本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
    */
    qResult SendUnreliable(StationID id, const qByte* pBuffer, size_t size)
    {
        return Send(id, Unreliable, 0, pBuffer, size);
    }

    /*!
    @refmanoverload{name1}

    @brief 特定のステーションへSubStreamIDを指定したReliableデータを送信します。

    StationIDは@ref Station::GetStationID で取得できます。
    　　送信バッファに空きがない場合にReliableSendBufferFullエラーを返します。
    これは一時的に送信バッファの上限に達したことを示すエラーであるため、
    ReliableSendBufferFullが発生した場合は、ディスパッチを行い時間をおいてから再度送信を行ってください。

    パケットバッファに空きがない場合PacketBufferFullエラーを返します。
    これは一時的にパケットバッファが不足していることを示すエラーであるため、
    ディスパッチを行い時間をおいてから再度送信を行ってください。
    高頻度でPacketBufferFullが発生する場合は、パケットバッファを増やす対策も行ってください。
    パケットバッファの設定については、@ref PacketBufferManager を参照してください。

    @param[in] id 宛先のステーションのStationIDを指定します。
    @param[in] subStreamID 送信に利用するSubStreamIDを指定します。 SubStreamIDDefine::SYSTEMは、システムと共有で、 SendでReliable通信を行った場合と同等です。
    @param[in] pBuffer 送信するデータを指定します。
    @param[in] size 送信するデータのサイズを指定します。Reliable通信では32768以下、それ以外の通信では1300(P2Pトポロジー以外の場合は1250)以下となるようにしてください。
    @retval "QSUCCESS(Core, Success)" 送信リクエストが成功した場合
    @retval "QERROR(Core, InvalidSequence)" ダイレクトストリームを有効化してない場合
    @retval "QERROR(Transport, Unknown)" PRUDPUserProtocol::GetInstance()がNULLの場合
    @retval "QERROR(Core, CallInitiationFailure)" 呼び出しコンテキストの初期化に失敗した場合
    @retval "QERROR(Core, InvalidArgument)" 引数の値が不正の場合 (pBufferがNULLであるか、sizeが超過しています。)
    @retval "QERROR(Transport, ConnectionFailure)" 宛先Stationとの接続が確立されていなかった場合
    @retval "QERROR(Transport, InvalidStation)" 宛先Stationが存在しなかった場合 (接続処理中であるか、すでに存在していません。)
    @retval "QERROR(Transport, ReliableSendBufferFull)" Reliable通信の送信バッファが一杯の場合（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
    @retval "QERROR(Transport, InvalidSubStreamID)" 指定されたSubStreamIDが無効なIDの場合。相手のステーションが受け付けることができないSubStreamIDの場合も発生します。
    @retval "QERROR(Transport, PacketBufferFull)" パケットバッファ枯渇により送信できなかった場合。アプリケーションによるハンドリングが必要です。 （本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）

    @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()

    */
    qResult SendReliable(StationID id, SubStreamID subStreamID, const qByte* pBuffer, size_t size)
    {
        return Send(id, Reliable, subStreamID, pBuffer, size);
    }
    /*!
    @refmanoverload{name2}

    @brief 特定のステーションへSubStreamIDを指定したReliableデータを送信します。

    呼び出しコンテキストを指定することで、Reliable通信の到達確認を非同期処理で検知することができます。
    到達確認ができた場合にSuccessとなります。到達確認ができるまでは完了しません。
    呼び出しコンテキストをキャンセルしても、送信リクエストが成功したReliable通信の送信は継続されます。<br>
    StationIDは@ref Station::GetStationID で取得できます。
    呼び出しコンテキストが終了した時の処理結果は、@ref CallContext::GetOutcome で取得できます。
    呼び出しコンテキストの処理結果については、返り値qResult型の@ref SendReliable の返り値を参照してください。

    @param[in,out] pContext 呼び出しコンテキストを指定します。
    @param[in] id 宛先のステーションのStationIDを指定します。
    @param[in] subStreamID 送信に利用するSubStreamIDを指定します。 SubStreamIDDefine::SYSTEMは、システムと共有で、 SendでReliable通信を行った場合と同等です。
    @param[in] pBuffer 送信するデータを指定します。
    @param[in] size 送信するデータのサイズを指定します。Reliable通信では32768以下、それ以外の通信では1300(P2Pトポロジー以外の場合は1250)以下となるようにしてください。
    @retval true 送信リクエストが成功した場合
    @retval false 送信リクエストが失敗した場合

    @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()
    */
    qBool SendReliable(CallContext *pContext, StationID id, SubStreamID subStreamID, const qByte* pBuffer, size_t size)
    {
        return Send(pContext, id, Reliable, subStreamID, pBuffer, size);
    }

    /*!
    @brief セッション参加中の全ステーションへデータを送信します。

    パケットバッファに空きがない場合PacketBufferFullエラーを返します。
    これは一時的にパケットバッファが不足していることを示すエラーであるため、
    ディスパッチを行い時間をおいてから再度送信を行ってください。
    高頻度でPacketBufferFullが発生する場合は、パケットバッファを増やす対策も行ってください。
    パケットバッファの設定については、@ref PacketBufferManager を参照してください。

    @param[in] type 通信の種類を指定します。Reliable通信は指定できません。
    @param[in] pBuffer 送信するデータを指定します。
    @param[in] size 送信するデータのサイズを指定します。1300(P2Pトポロジー以外の場合は1250)以下となるようにしてください。
    @retval "QSUCCESS(Core, Success)" 送信リクエストが成功した場合
    @retval "QERROR(Core, InvalidSequence)" ダイレクトストリームを有効化してない場合
    @retval "QERROR(Transport, Unknown)" PRUDPUserProtocol::GetInstance()がNULLの場合
    @retval "QERROR(Core, InvalidArgument)" 引数の値が不正の場合 (pBufferがNULLであるか、sizeが超過しています。)
    @retval "QERROR(Transport, ConnectionFailure)" 送信成功したステーションが一台も存在しなかった場合
    @retval "QERROR(Transport, PacketBufferFull)" パケットバッファ枯渇により送信できなかった場合。アプリケーションによるハンドリングが必要です。 （本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
    */
    qResult SendAll(StreamType type, const qByte* pBuffer, size_t size);

    /*!
    @brief セッション参加中の全ステーションへUnreliableデータを送信します。

    パケットバッファに空きがない場合PacketBufferFullエラーを返します。
    これは一時的にパケットバッファが不足していることを示すエラーであるため、
    必要に応じてパケットバッファを増やしてください。パケットバッファの設定については、@ref PacketBufferManager を参照してください。

    @param[in] pBuffer 送信するデータを指定します。
    @param[in] size 送信するデータのサイズを指定します。1300(P2Pトポロジー以外の場合は1250)以下となるようにしてください。
    @retval "QSUCCESS(Core, Success)" 送信リクエストが成功した場合
    @retval "QERROR(Core, InvalidSequence)" ダイレクトストリームを有効化してない場合
    @retval "QERROR(Transport, Unknown)" PRUDPUserProtocol::GetInstance()がNULLの場合
    @retval "QERROR(Core, InvalidArgument)" 引数の値が不正の場合 (pBufferがNULLであるか、sizeが超過しています。)
    @retval "QERROR(Transport, ConnectionFailure)" 送信成功したステーションが一台も存在しなかった場合
    @retval "QERROR(Transport, PacketBufferFull)" パケットバッファ枯渇により送信できなかった場合。アプリケーションによるハンドリングが必要です。 （本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
    */
    qResult SendUnreliableAll(const qByte* pBuffer, size_t size)
    {
        return SendAll(Unreliable, pBuffer, size);
    }

    /*!
    @brief ディスパッチ処理によって受信バッファに蓄積された受信データを取得します。

    本関数を呼び出すまで、ディスパッチによって受信バッファにデータが蓄積されます。
    本関数呼び出しで受信データを取得することで、受信バッファはクリアされます。
    本関数を呼び出しせずに、受信バッファにデータが蓄積され続けると、受信バッファの上限に達した時に、
    受信あふれとして@ref NetZ::GetFatalError で、QERROR(Transport, NoBuffer)のフェイタルエラーが発生します。
    受信あふれが起こる場合は、他Stationからの送信数を減らすように設計するか、
    @ref Scheduler::Dispatch または@ref Scheduler::DispatchAll とともに本関数の呼び出し頻度を増やしてください。
    @param[out] recvData 受信済みデータの格納先を指定します。
    */
    void GetReceivedData(qVector<DirectStream::Data>& recvData);

    /*! @internal
    @brief データ取得をせずに、受信バッファにたまったデータ数を取得する。
    @return 受信バッファにたまったデータ数を返します。
    */
    size_t GetReceivedDataNum();

    /*!
    @brief Reliable通信における最大送信バッファ数を取得します。

    送信処理で蓄積された送信バッファ数が本設定値にまで達していた場合、送信処理はReliableSendBufferFullのエラーを返します。<br>
    初期値は48です。

    @return Reliable通信における最大送信バッファ数を返します。
    */
    size_t GetMaxReliableSendBufferNum() const
    {
        return m_maxReliableSendBufferNum;
    }
    /*!
    @brief Reliable通信における最大送信バッファ数を設定します。

    送信処理で蓄積された送信バッファ数が本設定値にまで達していた場合、送信処理はReliableSendBufferFullのエラーを返します。<br>
    蓄積された送信バッファ数がライブラリの上限を超える場合は、@ref NetZ::GetFatalError でReliableSendBufferFullFatalのフェイタルエラーが発生しますので、
    設定の最大値は96程度を目安としてください。

    @param[in] bufferNum Reliable通信における最大送信バッファ数を指定します。
    */
    void SetMaxReliableSendBufferNum(size_t bufferNum)
    {
        m_maxReliableSendBufferNum = bufferNum;
    }

    /*!
    @brief StationIDで指定されるStation宛のReliable通信の滞留数を取得します。
    取得した値は、ダイレクトストリームを使用したReliable通信以外に、NEX内部で使用されるReliable通信のパケットも含まれます。

    @param[in] id Reliable通信の滞留数を取得する対象ステーションのStationIDを指定します。
    @param[in] subStreamID 複数本Reliableを使う時のIDを指定します。省略時は、 SubStreamIDDefine::SYSTEMが指定されて、システムと共用になります。
    @return StationID で指定される Station 宛の Reliable 通信の滞留数が返ります。ローカルステーション、切断された Station、存在しない Station や存在しない SubStreamID を指定した場合は -1 が返ります。

    @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()

    */
    qInt32 GetPendingReliableBufferNum(StationID id, SubStreamID subStreamID =  SubStreamIDDefine::SYSTEM);

    /*! @internal
    @brief Reliableでの最大送信バッファサイズを取得します。
    @return Reliableでの最大送信バッファサイズを返します。
    */
    size_t GetMaxReliableSendBufferSize() const
    {
        return m_maxReliableSendBufferSize;
    }
    /*! @internal
    @brief Reliableでの最大送信バッファサイズを設定します。
    */
    void SetMaxReliableSendBufferSize(size_t bufferSize)
    {
        m_maxReliableSendBufferSize = bufferSize;
    }

    /*! @internal
    @brief Unreliableでの最大送信バッファサイズを取得します。
    @return Unreliableでの最大送信バッファサイズを返します。
    */
    size_t GetMaxUnreliableSendBufferSize() const
    {
        return m_maxUnreliableSendBufferSize;
    }
    /*! @internal
    @brief Unreliableでの最大送信バッファサイズを設定します。
    */
    void SetMaxUnreliableSendBufferSize(size_t bufferSize)
    {
        m_maxUnreliableSendBufferSize = bufferSize;
    }

    /*! @internal
    @brief 最大受信バッファ数を取得します。
    @return 最大受信バッファ数を返します。
    */
    size_t GetMaxRecvBufferNum() const
    {
        return m_maxRecvBufferNum;
    }
    /*! @internal
    @brief 最大受信バッファ数を設定します。
    @param[in] bufferNum Reliable通信における最大受信バッファ数を指定します。
    */
    void SetMaxRecvBufferNum(size_t bufferNum)
    {
        m_maxRecvBufferNum = bufferNum;
    }

private: // private method

    // 呼び出しコンテキスト有り無しのSendの共通関数
    qResult SendCommon(CallContext *pContext, StationID id, StreamType type, qByte subStreamID, const qByte* pBuffer, size_t size);

    // PRUDPPRotocolに渡す受信コールバック関数です。
    static void ReceiveCallback(nn::nex::StationID srcId, Buffer* pBuffer, qUnsignedInt32 packetTypeFlags, qByte subStreamID);

    void SetFatalError(qResult err);
    void SetRecvOverflow(qBool overflow)
    {
        m_recvOverflow = overflow;
    }
    qBool IsRecvOverflow()
    {
        return m_recvOverflow;
    }

    /*! @internal
    @brief 通信タイプに対して、送信可能なバッファサイズかを判定する
    @param[in] type 通信タイプ
    @param[in] bufferSize 送信するバッファのサイズ
    @retval true 送信可能なサイズ
    @retval false 送信不可能なサイズ
    */
    qBool CheckSendBufferSize(DirectStream::StreamType type, size_t bufferSize);

private: // private member
    qResult Send(StationID id, StreamType type, qByte subStreamID, const qByte* pBuffer, size_t size);
    qBool Send(CallContext *pContext, StationID id, StreamType type, qByte subStreamID, const qByte* pBuffer, size_t size);

    qVector<DirectStream::Data> m_recvData; // 受信済みデータ
    CriticalSection m_csRecvData;
    class AckCallBackJob;
    Job* m_pAckCallBackJob; // Ack受信完了した呼び出しコンテキストの状態を更新するJob

    // アプリケーション側でチューニングできるように可変パラメーターとする
    size_t m_maxReliableSendBufferNum; // reliable送信バッファ数
    size_t m_maxReliableSendBufferSize; // reliable送信サイズ最大
    size_t m_maxUnreliableSendBufferSize; // unreliable送信サイズ最大
    size_t m_maxRecvBufferNum; // 最大受信バッファ数
    qBool m_recvOverflow;
};

END_NAMESPACE_NEX

