﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/RefTemplate.h>
#include <NetZ/src/Core/DOCore/SelectionIteratorTemplate.h>
#include <NetZ/src/Core/DOCore/MethodID.h>
#include <NetZ/src/Core/DOCore/DupSpace.h>
#include <NetZ/src/Core/DOCore/MemberContainer.h>
#include <NetZ/src/Core/DOCore/DOSelections.h>
#include <NetZ/src/Core/DOCore/WKHandle.h>
#include <OnlineCore/src/Core/LogicalClock.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h> /* For MSG_RELIABLE */
#include <NetZ/src/Core/DOCore/DORole.h>
#include <NetZ/src/Core/DOCore/Selection.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Core/Operation.h>
#include <OnlineCore/src/Core/StateMachine.h>
#include <NetZ/src/Core/DOCore/MasterStationRef.h>
#include <OnlineCore/src/Transport/Interface/SubStreamID.h>


BEGIN_NAMESPACE_NEX

class DOClass;
class DataSet;
class DOOperation;
class FetchContext;
class MigrationContext;
class OperationManager;

class Variable;

class UpdateDataSetOperation;
class ChangeDupSetOperation;
class AddToStoreOperation;
class RemoveFromStoreOperation;
class ChangeMasterStationOperation;

//group=DDL Output

/*
$ifdef VOICE$
{secret}
$else$
Description: Represents the distance between two objects when the distance is unknown or undefined.
Remarks: This value will be used if, for Distance-based dead reckoning or for Distance-based Migration,
the observer or how to calculate the distance between the observer and an object is not defined.
See Also:  Dead Reckoning Overview, DuplicatedObject::ComputeDistance, Station::RegisterObserver,
Distance-based Migration
$endif$
*/
#define UNKNOWN_DISTANCE QREAL(-1.0)

/*
$ifdef VOICE$
{secret}
$else$
Description: Generates the name of the duplication object's base class.
Remarks:
For example, lets say a duplicated object is declared like this in a DDL file :

      doclass GameObject {
          // DDL declarations
      };

Then the user-defined /GameObject/ class is declared like this :

      class GameObject : public DOCLASS(GameObject)  {
          // User specific variables and methods
      };
$endif$
*/


/*!
 @defgroup  nex_define nex_define
 @{

*/

/*!
 * @brief ベースのデータセットクラスを参照するときに用いるマクロです。
 *
 * DDLが以下のように定義されているときに、<br>
      doclass GameObject {<br>
      };<br>
 * アプリケーションでは、GameObjectクラスを以下のように定義します。<br>
 *
      class GameObject : public DOCLASS(GameObject)  {<br>
      };<br>
 *
 * @param C クラス名
 */
#define DOCLASS(C) _DO_##C


/*!
 @}
 */
/*
Description: The root of all user-defined duplicated objects.

Remarks: If a DOClass is defined in the DDL, the corresponding DOClass generated by the
DDL compiler will ultimately inherit from the DuplicatedObject class.
The DuplicatedObject class contains regular methods, inherited by
every user-defined doclass.  It also defines system callbacks.  $PRODUCTNAME$ system
callbacks have the semantics of a virtual method meaning that if they are
overridden by the user doclass, the overriding method will be called whenever applicable.

$ifdef NETZ$
In NetZ, all DuplicatedObjects are global objects unless specified nonglobal in their DDL definitions.
When global objects are created and published, they will be instantiated on the local station and
copies, or Duplica s, of the object published on all the remote stations connected to the session.
When an object is deleted, via DeleteMainRef, all Duplica s of the object are also deleted.
$endif$

$ifdef VOICE$
In NetZ, all DuplicatedObjects are global objects unless specified nonglobal in their DDL definitions.
When global objects are created and published, they will be instantiated on the local station and
copies, or Duplica s, of the object published on all the remote stations connected to the session.
When an object is deleted, via DeleteMainRef, all Duplica s of the object are also deleted.
$endif$

See Also: Actions Overview, Fault Tolerance Overview
*/
/**
 * @brief   すべての複製オブジェクトの基底クラスです。
 *
 * DDL(データ定義言語)によって複製オブジェクトクラスが定義されると、それに対応するクラスはDDLコンパイラによって
 * 自動的に生成されますが、それらすべてのクラスは、このDuplicatedObjectクラスより派生したものになります。
 * このDuplicatedObjectクラスは、複製オブジェクトに対する一般的な関数を含んでいて、
 * それらの関数はユーザー定義の複製オブジェクトでも利用することが出来ます。
 *
 * また、システムからのコールバックも定義します。
 * いくつかのvirtual関数は、ユーザー定義の複製オブジェクトクラスからオーバーライドすることが
 * 出来ることを意味していて、オーバーライドされた関数は適切なタイミングで呼び出されます。
 */
// {tag:DuplicatedObject class}
class DuplicatedObject: public StateMachine
{
    //group=DO Methods
public:

    //Remarks: Duplicated objects should never be directly instantiated.  DuplicatedObject::Create should
    //be used instead.
    /**
     * @brief   このコンストラクタはアプリケーション側から呼び出さないでください。
     *
     * 代わりに、@ref DuplicatedObject::Create を呼び出します。
     */
    DuplicatedObject();

    /*
    Remarks: The correct way to destroy a duplicated object is through the method
    DuplicatedObject::DeleteMainRef. When a duplicated object is destroyed, the datasets and
    well-known handles associated with it are also destroyed. The duplicated object destructor
    has the semantics of a virtual method and, therefore, it can be rewritten in a user subclass.

    Note: The duplicated object destructor will not be called on an object until all references, both
    Hard reference s and Soft reference s, to the object have been deleted.

    See Also: DataSet, WKHandle, DeleteMainRef
    */
    /**
     * @brief   このデストラクタはアプリケーション側から呼び出さないでください。
     *
     * 複製オブジェクトを破棄する正しい方法は、@ref DeleteMainRef を呼び出すことです。
     * 複製オブジェクトが破棄されるとき、関連付けられているデータセットと、既知ハンドルも、破棄されます。
     * この複製オブジェクトのデストラクタはvirtual関数ですので、ユーザー定義の複製オブジェクトで
     * これをオーバーライドすることが可能です。
     *
     * このデストラクタは即座に呼び出されません。
     * このオブジェクトに対するすべての参照がなくなった段階で破棄されることになります。
     */
    virtual ~DuplicatedObject();

    /*
    Summary: Returns a DOHandle to the duplicated object that refers to the local instance
    of the duplicated object.
    Returns: The DOHandle to the duplicated object or INVALID_DOHANDLE if the call signalled
    an error. The error SYSTEMERROR_GEN_INVALID_OPERATION is possible.
    See also: GetHandleValue
    */

    /*!
     * @brief   この複製オブジェクトの複製オブジェクトハンドルを取得します。
     * @return  複製オブジェクトハンドルが返ります。
     * @retval  INVALID_DOHANDLE エラーが発生したときに返ります。このとき、SYSTEMERROR_GEN_INVALID_OPERATION_WARNINGエラーがシグナルされます。
     * @see     GetHandleValue
     */
    DOHandle GetHandle() const
    {
        if (m_dohMyself.GetDOID()==0)
        {
            SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_INVALID_OPERATION_WARNING,0);
            return INVALID_DOHANDLE;
        }
        else
        {
            return m_dohMyself;
        }
    }

    /*
    Summary: Returns a qUnsignedInt32 that represents the duplicated object instance.
    Returns: A qUnsignedInt32 that represents the duplicated object or INVALID_DOHANDLE if the
    call signalled an error. The error SYSTEMERROR_GEN_INVALID_OPERATION is possible.
    See also: GetHandle
    */

    /*!
     * @brief   この複製オブジェクトのインスタンスを表す@ref qUnsignedInt32 型の値を取得します。
     * @return  この複製オブジェクトのインスタンスを表す@ref qUnsignedInt32 型の値が返ります。
     * @retval  INVALID_DOHANDLE エラーが発生したときに返ります。このとき、SYSTEMERROR_GEN_INVALID_OPERATION_WARNING エラーがシグナルされます。
     * @see     GetHandle
     */
    qUnsignedInt32 GetHandleValue() const
    {
        return GetHandle();
    }

#ifdef NEX_DEBUG_VERSION
    // {secret}
    void CheckStateAccess() const;
#else
    // {secret}
    void CheckStateAccess() const {}
#endif

    // {Secret}
    void SetDOClassID(DOClassID uiClassID)
    {
        CheckStateAccess();
        m_dohMyself.SetDOClassID(uiClassID);
    }
    // {Secret}
    void SetDOID(DOID uiObjectID)
    {
        CheckStateAccess();
        m_dohMyself.SetDOID(uiObjectID);
    }
    // {Secret}
    void SetHandle(DOHandle hNewHandle)
    {
        CheckStateAccess();
        m_dohMyself=hNewHandle;
    }

    /*
    Description: Returns whether or not a DuplicatedObject instance is a Global duplicated object.
    Returns: true if this duplicated object is a Global duplicated object, false if not or an error occurred.
    Remarks: A Global duplicated object is either a Well-known duplicated object or an object that has the
    DDL property /globaldo/ (refer to DDL Properties for details). These objects are known globally across
    the Session and are guaranteed to be duplicated to all stations participating in the session.

    $ifdef NETZ$
    Note: All NetZ duplicated objects are automatically Global duplicated object s unless otherwise specified using
    the nonglobal property in the DDL declaration.
    $endif$

    $ifdef VOICE$
    Note: All NetZ duplicated objects are automatically Global duplicated object s. unless otherwise specified using
    the nonglobal property in the DDL declaration.
    $endif$

    See Also: IsAWellKnownDO, IsACoreDO, IsAUserDO, DDL Properties
    */

    /*!
     * @brief   このインスタンスがグローバル複製オブジェクトであるかどうかを取得します。
     *
     * グローバル複製オブジェクトとは、既知オブジェクトであるか、あるいは
     * DDL(データ定義言語)にて、globaldoプロパティが設定された複製オブジェクトです
     * (DDLの文法についての詳細はNEXプログラミングマニュアルを参照してください)。
     * これらのオブジェクトはセッション中で全体的にアクセスでき、
     * セッションに参加中のすべてのステーション(参加者)に対して複製されていることが保証されています。
     *
     * @retval  true  グローバル複製オブジェクトのインスタンスである場合に返ります。
     * @retval  false グローバル複製オブジェクトのインスタンスであに場合に返ります。
     * @see     IsAWellKnownDO, IsACoreDO, IsAUserDO
     */
    qBool IsGlobal() const;

    // {secret}
    virtual qBool HasGlobalDOProperty() const
    {
        return false;
    }

    // {secret}
    virtual qBool HasForcedNonGlobalProperty() const
    {
        return false;
    }

    /*
    Description: Returns the class name of the DuplicatedObject as a string.
    Returns: A string that represents the class name of the DuplicatedObject.
    Remarks: Gets the class name of the duplicated object instance as a string in your traces.
    Example:
        qReal DuplicatedObject::ComputeDistance(DuplicatedObject * pObservedDuplicatedObject){
            nex_tprintf(NEX_T("Computing distance for Duplicated Object of class %s"),
                    pObservedDuplicatedObject->GetClassNameString());
            [...]
        }
    */

    /*!
     * @brief   複製オブジェクトのクラス名を文字列として取得します。
     *
     * この関数をトレースに利用することでデバッグに役立てることが出来ます。
     *
     * @return  この複製オブジェクトクラス名を表す文字列へのポインタが返ります。
     */
    const qChar* GetClassNameString() const;

    /*
    Description: Returns the class name of the specified dataset associated to this DuplicatedObject.
    Parameters:
    byDatasetIndex: The index of the dataset.
    Returns: The class name of the dataset associated to this DuplicatedObject specified by the given index as a string.
    Remarks:
    This method is useful for obtaining the dataset class name in traces. $PRODUCTNAME$ sequentially assigns a
    dataset index to each dataset associated to a given duplicated object as declared in the DDL file. The index is a
    positive integer starting at one.
    */
    /**
     * @brief   この複製オブジェクトに関連付けられているデータセットの名称を取得します。
     *
     * この関数はトレースを行う際に便利に利用することが出来ます。
     * @a byDatasetIndexで指定しなければならないインデックス番号は、
     * DDLファイルに宣言された順番に割り当てられていきます。
     * この番号は1から始まる正の整数となります。
     *
     * @param[in]   byDatasetIndex  データセットのインデックス番号を指定します。
     *
     * @return  この複製オブジェクトに関連付けられているデータセットのうち、
     *          @a byDatasetIndexに対応するインデックス番号のデータセットの名称文字列へのポインタが返ります。
     */
    const qChar* GetDatasetNameString(qByte byDatasetIndex) const;

    // {secret}
    // TODO.  Document.  CR212.
    qBool FormatVariableValue(Variable* pDataset, Variable* pVariable, String *pstrResult) const;

    /*
    Description: Returns the DOClassID of the duplicated object.
    Returns: The DOClassID of the duplicated object. This ID is generated by the DDL compiler.
    See also: DOCLASSID
    */

    /*!
     * @brief   この複製オブジェクトの複製オブジェクトクラスIDを取得します。
     *
     * 複製オブジェクトクラスIDとは、@ref DOClassID として定義される、
     * 複製オブジェクトのクラスを指します。同一のクラスであれば同じIDとなります。
     * このIDはDDLコンパイラによって自動的に生成されます。
     * DOCLASSIDマクロを利用すると、指定された複製オブジェクトクラスのDOClassIDを取得することが出来ます。
     *
     * @return  この複製オブジェクトインスタンスの、複製オブジェクトクラスIDが返ります。
     */
    DOClassID GetDOClassID() const
    {
        return m_dohMyself.GetDOClassID();
    }

    /*
    Description: Returns whether or not a DuplicatedObject instance is an instance
    of a specified doclass.
    Parameters:
    idDOCID:  The DOClassID of the DuplicatedObject.
    Returns: true if the DuplicatedObject instance has this DOClassID, false if not or an error occurred.
    Remarks: Use the DOCLASSID macro to retrieve the DOClassID for a given DOClass.
    Example:
        NamedObject::SelectionIterator  itNamedObject;
        itNamedObject.GotoStart();
        while(! (itNamedObject.EndReached())){
            if (ItNamedObject->IsA(DOCLASSID(Avatar))){
                itNamedObject++;
            }
        }

    See Also: IsAKindOf
    */

    /*!
     * @brief   この複製オブジェクトインスタンスが指定された複製オブジェクトクラスのインスタンスであるかを調べます。
     *
     * DOCLASSIDマクロを利用すると、指定された複製オブジェクトクラスの@ref DOClassID を取得することが出来ます。
     *
     * @param[in]   idDOCID 複製オブジェクトクラスIDを指定します。
     *
     * @retval  true  この複製オブジェクトは指定された@a idDOCIDで示される複製オブジェクトクラスのインスタンスです。
     * @retval  false この複製オブジェクトは指定された@a idDOCIDで示される複製オブジェクトクラスのインスタンスではありません。
     * @see     IsAKindOf
     */
    qBool IsA(DOClassID idDOCID) const
    {
        return m_dohMyself.GetDOClassID() == idDOCID;
    }

    /*
    Description: Returns whether or not a DuplicatedObject instance is an instance or
    a descendant of the specified DOCLASS.
    Parameters:
    idDOCID: the DOClassID of the DuplicatedObject.
    Returns: true if the DuplicatedObject instance is an instance of or a descendant class
    of this DOClassID, false if not or an error occurred.
    Remarks: Use the DOCLASSID macro to get an instance of this class.
    Example:
        NamedObject::SelectionIterator  itNamedObject;
        itNamedObject.GotoStart();
        while(! (itNamedObject.EndReached())){
            if (ItNamedObject->IsAKindOf(DOCLASSID(Avatar))){
                itNamedObject++;
            }
        }

    See Also: IsA
    */

    /*!
     * @brief   この複製オブジェクトインスタンスが指定された複製オブジェクトクラス、またはその派生関係であるかどうかを調べます。
     *
     * DOCLASSIDマクロを利用すると、指定された複製オブジェクトクラスの@ref DOClassID を取得することが出来ます。
     *
     * @param[in]   idDOCID 複製オブジェクトクラスIDを指定します。
     *
     * @retval  true  この複製オブジェクトは指定された@a idDOCIDで示される複製オブジェクトクラスかその派生関係のインスタンスです。
     * @retval  false この複製オブジェクトは指定された@a idDOCIDで示される複製オブジェクトクラスでもその破線関係クラスのインスタンスでもありません。
     * @see     IsAKindOf
     */
    qBool IsAKindOf(DOClassID idDOCID) const;

    // {Secret}
    DOID GetDOID() const
    {
        return m_dohMyself.GetDOID();
    }

    /*
    Summary: Propagates the contents of all datasets to an object's duplicas or only the contents
    of the specified dataset.
    Parameters:
    refDataset:  An optional pointer to the specific dataset that is to be updated.
    tUpdateTime: (optional) Specifies the point in (absolute) time at which the
    Update is valid. Otherwise, the current time as given by the SessionClock is
    used. This parameter should be set when the time stamp is changed as done when Lookahead
    or Loopback techniques are applied on the Duplication master.
    Remarks:  This method must be called on a Duplication master.  It has the semantics of a virtual
    method and therefore, it can be called on a base class and all the derived class datasets will also
    be updated.  The call can be made
    with or without the parameter to specify which of the duplicated object's datasets should
    be updated. For example, suppose we have a Position dataset and the m_dsPosition dataset
    member of type Position is associated with the Sphere doclass in the DDL file, then it
    is possible to update just that Position dataset as follows:

          // Assuming that all we have is a pointer to our Sphere duplicated object
          poSphere->Update(poSphere->m_dsPosition);

    Alternatively, if we have an extrapolated dataset and a MainLoop method that is called periodically on every
    game object we could update and refresh each Sphere object's datasets via:

            void Sphere::MainLoop() {
                if (IsADuplicationMaster()) {
                    // compute the physics and then update the objects position.
                    Update();
                } else {
                    // Refresh the position.
                    Refresh(m_Pos);
                }
                // Displays the object at the correct position.
            }

    If a station cannot directly communicate with a station to which it wishes to send a message
    $PRODUCTNAME$ will automatically route the message via another station. This effectively deals with
    the problem of stations being unable to communicate with each other due to the fact that they
    have no common transport type.

    Note: Depending on the dataset update policy, Update does not necessary
    generate a network message. If some of the datasets are extrapolated, Update must be called
    regularly ( /e.g./ after DoPhysics)  even if the contents of the datasets did not change. This
    is necessary to maintain a coherent prediction model for the
    extrapolated datasets.

    Returns: true if the object was successfully updated, false if not or an error occurred.
    The error SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA will be signalled if this method is called on a Duplica,
    SYSTEMERROR_GEN_INVALID_PARAMETER if an invalid dataset is specified and
    SYSTEMERROR_MISSING_DATASET_CALLBACK if no callback is defined for a custom update policy.
    See Also: DataSet::SetLoopbackDelay, Refresh
    */

    /*!
     * @refmanoverload{name1}
     *
     * @brief   すべてのデータセットの内容が更新されたことをセッションに存在するデュプリカに通知します。
     *
     * この関数は、複製マスターのみが呼び出してください。
     *
     * この関数は内部的にCPUチックを取得します。
     * CTRで最大限パフォーマンスを得るために、別の位置で@ref SessionClock の値を取得して
     * その値を明示的に指定するオーバーロードを利用することを推奨します。
     *
     * Reliable通信を指定した送信では、送信対象となったステーションへの送信バッファのうち一つでも
     * 空きがない場合にReliableSendBufferFullエラーを返します。
     * また、Reliable通信に限らず送信に必要なパケットバッファの空きが十分に無い場合はPacketBufferFullエラーを返します。
     * これらは一時的に送信バッファの上限に達したことを示すエラーであるため、
     * ReliableSendBufferFullまたはPacketBufferFullが発生した場合は、ディスパッチを行い時間をおいてから再度送信を行ってください。
     * 高頻度でPacketBufferFullが発生する場合は、パケットバッファを増やす対策も行ってください。
     * パケットバッファの設定については、@ref PacketBufferManager を参照してください。
     *
     * エラーとなった場合は、エラーとなったあるデータセットが、すべてのステーションへ到達しないことを保証しますが、
     * エラー以前のデータセットは到達します。
     *
     * @retval "QSUCCESS(Core, Success)" 送信リクエストが成功した場合
     * @retval "QERROR(DOCore, InvalidRole)" 複製マスター以外から呼び出した場合
     * @retval "QERROR(Transport, ReliableSendBufferFull)" Reliable通信の送信バッファが一杯の場合（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
     * @retval "QERROR(Transport, InvalidSubStreamID)" DataSet::SetSubStreamID() で指定されたSubStreamIDが無効なIDの場合
     * @retval "QERROR(Transport, PacketBufferFull)" パケットバッファの空きが不足した場合（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
     *
     * @see SubStreamIDDefine, StreamSettings::SetMaxUserReliableSubStreams()
     *
     */
    qResult Update()
    {
        return UpdateImpl(NULL, Time::GetSessionTime());
    }

    /*!
     * @refmanoverload{name2}
     *
     * @brief   すべてのデータセットの内容が更新されたことをセッションに存在するデュプリカに通知します。
     *
     * この関数は、複製マスターのみが呼び出してください。
     *
     * このオーバーロードでは、明示的に現在の@ref SessionClock の値を指定して
     * 更新処理を行うことが出来ます。多く存在する複製オブジェクトそれぞれに対して
     * Updateを行う必要がある場合、ループの外側で@ref SessionClock を取得して、
     * その値をこのオーバーロードに渡すことで最適化することが出来ます。
     *
     * Reliable通信を指定した送信では、送信対象となったステーションへの送信バッファのうち一つでも
     * 空きがない場合にReliableSendBufferFullエラーを返します。
     * また、Reliable通信に限らず送信に必要なパケットバッファの空きが十分に無い場合はPacketBufferFullエラーを返します。
     * これらは一時的に送信バッファの上限に達したことを示すエラーであるため、
     * ReliableSendBufferFullまたはPacketBufferFullが発生した場合は、ディスパッチを行い時間をおいてから再度送信を行ってください。
     * 高頻度でPacketBufferFullが発生する場合は、パケットバッファを増やす対策も行ってください。
     * パケットバッファの設定については、@ref PacketBufferManager を参照してください。
     *
     * エラーとなった場合は、エラーとなったあるデータセットが、すべてのステーションへ到達しないことを保証しますが、
     * エラー以前のデータセットは到達します。
     *
     * @param[in]  tUpdateTime 現在のセッション時刻を指定します。
     *
     * @retval "QSUCCESS(Core, Success)" 送信リクエストが成功した場合
     * @retval "QERROR(DOCore, InvalidRole)" 複製マスター以外から呼び出した場合
     * @retval "QERROR(Transport, ReliableSendBufferFull)" Reliable通信の送信バッファが一杯の場合（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
     * @retval "QERROR(Transport, InvalidSubStreamID)" DataSet::SetSubStreamID() で指定されたSubStreamIDが無効なIDの場合
     * @retval "QERROR(Transport, PacketBufferFull)" パケットバッファの空きが不足した場合（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
     */
    qResult Update(const Time& tUpdateTime)
    {
        return UpdateImpl(NULL, tUpdateTime);
    }

    /*!
     * @refmanoverload{name3}
     *
     * @brief   指定されたデータセットの内容が更新されたことをセッションに存在するデュプリカに通知します。
     *
     * この関数は、複製マスターのみが呼び出してください。
     *
     * すべてのデータセットを更新する場合に比べて通信量を削減することが出来ます。
     * この関数は内部的にCPUチックを取得します。
     * CTRで最大限パフォーマンスを得るために、別の位置で@ref SessionClock の値を取得して
     * その値を明示的に指定するオーバーロードを利用することを推奨します。
     *
     * Reliable通信を指定した送信では、送信対象となったステーションへの送信バッファのうち一つでも
     * 空きがない場合にReliableSendBufferFullエラーを返します。
     * また、Reliable通信に限らず送信に必要なパケットバッファの空きが十分に無い場合はPacketBufferFullエラーを返します。
     * これらは一時的に送信バッファの上限に達したことを示すエラーであるため、
     * ReliableSendBufferFullまたはPacketBufferFullが発生した場合は、ディスパッチを行い時間をおいてから再度送信を行ってください。
     * 高頻度でPacketBufferFullが発生する場合は、パケットバッファを増やす対策も行ってください。
     * パケットバッファの設定については、@ref PacketBufferManager を参照してください。
     *
     * エラーとなった場合は、すべてのステーションへ到達しないことを保証します。
     *
     * @param[in]  refDataset  更新するデータセットを指定します。
     *
     * @retval "QSUCCESS(Core, Success)" 送信リクエストが成功した場合
     * @retval "QERROR(DOCore, InvalidRole)" 複製マスター以外から呼び出した場合
     * @retval "QERROR(Transport, ReliableSendBufferFull)" Reliable通信の送信バッファが一杯の場合（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
     * @retval "QERROR(Transport, InvalidSubStreamID)" DataSet::SetSubStreamID() で指定されたSubStreamIDが無効なIDの場合
     * @retval "QERROR(Transport, PacketBufferFull)" パケットバッファの空きが不足した場合（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
     */
    qResult Update(DataSet& refDataset)
    {
        return UpdateImpl(&refDataset, Time::GetSessionTime());
    }

    /*!
     * @refmanoverload{name4}
     *
     * @brief   指定されたデータセットの内容が更新されたことをセッションに存在するデュプリカに通知します。
     *
     * この関数は、複製マスターのみが呼び出してください。
     *
     * すべてのデータセットを更新する場合に比べて通信量を削減することが出来ます。
     * このオーバーロードでは、明示的に現在の@ref SessionClock の値を指定して
     * 更新処理を行うことが出来ます。多く存在する複製オブジェクトそれぞれに対して
     * Updateを行う必要がある場合、ループの外側で@ref SessionClock を取得して、
     * その値をこのオーバーロードに渡すことで最適化することが出来ます。
     *
     * Reliable通信を指定した送信では、送信対象となったステーションへの送信バッファのうち一つでも
     * 空きがない場合にReliableSendBufferFullエラーを返します。
     * また、Reliable通信に限らず送信に必要なパケットバッファの空きが十分に無い場合はPacketBufferFullエラーを返します。
     * これらは一時的に送信バッファの上限に達したことを示すエラーであるため、
     * ReliableSendBufferFullまたはPacketBufferFullが発生した場合は、ディスパッチを行い時間をおいてから再度送信を行ってください。
     * 高頻度でPacketBufferFullが発生する場合は、パケットバッファを増やす対策も行ってください。
     * パケットバッファの設定については、@ref PacketBufferManager を参照してください。
     *
     * エラーとなった場合は、すべてのステーションへ到達しないことを保証します。
     *
     * @param[in]  refDataset  更新するデータセットを指定します。
     * @param[in]  tUpdateTime 現在のセッション時刻を指定します。
     *
     * @retval "QSUCCESS(Core, Success)" 送信リクエストが成功した場合
     * @retval "QERROR(DOCore, InvalidRole)" 複製マスター以外から呼び出した場合
     * @retval "QERROR(Transport, ReliableSendBufferFull)" Reliable通信の送信バッファが一杯の場合（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
     * @retval "QERROR(Transport, InvalidSubStreamID)" DataSet::SetSubStreamID() で指定されたSubStreamIDが無効なIDの場合
     * @retval "QERROR(Transport, PacketBufferFull)" パケットバッファの空きが不足した場合（本関数でこのエラーが返っても @ref NetZ::GetFatalError で取得できるフェイタルエラーにはセットされません）
     */
    qResult Update(DataSet& refDataset, const Time& tUpdateTime)
    {
        return UpdateImpl(&refDataset, tUpdateTime);
    }



    /*
    Summary: Uses the content of the buffer to refresh the variables of all or a specified buffered dataset of a Duplica.
    Parameters:
    refDataset: An optional pointer to the dataset that is to be refreshed.
    tRefreshTime: (optional) Specifies the point in (absolute) time at which to refresh the value.
    Otherwise the current time as given by the SessionClock is used.
    Returns: true if the values of the object were successfully refreshed, false if not or an error occurred.
    The error SYSTEMERROR_GEN_INVALID_OPERATION will be signalled
    if the dataset is not buffered or extrapolated,
    SYSTEMERROR_OBJDUP_INVALID_ON_MASTER if this method is called on a Duplication master, and
    SYSTEMERROR_GEN_INVALID_PARAMETER if refDataset is invalid or does not exist.
    Remarks:  This method must be called on the buffered datasets of a Duplica to
    refresh the value of the dataset variables. If an extrapolation filter
    is used, this method will refresh the dataset variables with an
    extrapolation of their value. This method is only valid for the buffered datasets
    of a Duplica.

    If we have an extrapolated dataset and a MainLoop method that is called periodically on every game object we
    could update and refresh each Sphere object's datasets via:

        void Sphere::MainLoop() {
            if (IsADuplicationMaster()) {
                // compute the physics and then update the objects position.
                Update();
            } else {
                // Refresh the position.
                Refresh(m_Pos);
            }
            // Displays the object at the correct position.
        }

    See Also: Update
    */

    /*!
     * @refmanoverload{name1}
     *
     * @brief   デュプリカのデータセットの情報をバッファから転送します。
     *
     * DDL(データ定義言語)によって、データセットについてバッファを利用する設定にしている場合は、
     * メッセージ受信に対応して即座にデータセットは更新されず、それらのメッセージは一時的にバッファにプールされます。
     * DDLにて、bufferedプロパティ(バッファを利用する)あるいはextrapolation_filterプロパティ(推測航法を利用する)を
     * 指示していると、そのデータセットはバッファを利用することになります。
     *
     * この関数を利用すると、デュプリカが持っているすべてのデータセットについて、
     * バッファにためられた情報をデータセットに転送します。
     *
     * この関数は、複製オブジェクトのインスタンスがデュプリカであるときのみ呼び出すことが出来ます。
     *
     * CTRでは負荷軽減のために、明示的にセッションクロックを指定するオーバーロードを利用することを推奨します。
     *
     * @retval  true    正常に転送できた場合に返ります。
     * @retval  false   転送に失敗した場合に返ります。
     *                  複製マスターがこの関数を呼び出した場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     *
     * @see     IsADuplica
     */
    qBool Refresh()
    {
        return RefreshImpl(NULL, Time::GetSessionTime());
    }

    /*!
     * @refmanoverload{name2}
     *
     * @brief   デュプリカのデータセットの情報をバッファから転送します。
     *
     * DDL(データ定義言語)によって、データセットについてバッファを利用する設定にしている場合は、
     * メッセージ受信に対応して即座にデータセットは更新されず、それらのメッセージは一時的にバッファにプールされます。
     * DDLにて、bufferedプロパティ(バッファを利用する)あるいはextrapolation_filterプロパティ(推測航法を利用する)を
     * 指示していると、そのデータセットはバッファを利用することになります。
     *
     * この関数を利用すると、デュプリカが持っているすべてのデータセットについて、
     * バッファにためられた情報をデータセットに転送します。
     *
     * この関数は、複製オブジェクトのインスタンスがデュプリカであるときのみ呼び出すことが出来ます。
     *
     * @param[in]   tRefreshTime    値を更新する時刻を指定します。
     * @retval  true    正常に転送できた場合に返ります。
     * @retval  false   転送に失敗した場合に返ります。
     *                  複製マスターがこの関数を呼び出した場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     *
     * @see     IsADuplica
     */
    qBool Refresh(const Time& tRefreshTime)
    {
        return RefreshImpl(NULL, tRefreshTime);
    }

    /*!
     * @refmanoverload{name3}
     *
     * @brief   デュプリカのデータセットの情報をバッファから転送します。
     *
     * DDL(データ定義言語)によって、データセットについてバッファを利用する設定にしている場合は、
     * メッセージ受信に対応して即座にデータセットは更新されず、それらのメッセージは一時的にバッファにプールされます。
     * DDLにて、bufferedプロパティ(バッファを利用する)あるいはextrapolation_filterプロパティ(推測航法を利用する)を
     * 指示していると、そのデータセットはバッファを利用することになります。
     *
     * この関数を利用すると、指定されたデータセットの
     * バッファにためられた情報をデータセットに転送します。
     *
     * この関数は、複製オブジェクトのインスタンスがデュプリカであるときのみ呼び出すことが出来ます。
     *
     * CTRでは負荷軽減のために、明示的にセッションクロックを指定するオーバーロードを利用することを推奨します。
     *
     * @param[in]   refDataset      転送するデータセットを指定します。
     * @retval  true    正常に転送できた場合に返ります。
     * @retval  false   転送に失敗した場合に返ります。
     *                  複製マスターがこの関数を呼び出した場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     *
     * @see     IsADuplica
     */
    qBool Refresh(DataSet& refDataset)
    {
        return RefreshImpl(&refDataset, Time::GetSessionTime());
    }

    /*!
     * @refmanoverload{name4}
     *
     * @brief   デュプリカのデータセットの情報をバッファから転送します。
     *
     * DDL(データ定義言語)によって、データセットについてバッファを利用する設定にしている場合は、
     * メッセージ受信に対応して即座にデータセットは更新されず、それらのメッセージは一時的にバッファにプールされます。
     * DDLにて、bufferedプロパティ(バッファを利用する)あるいはextrapolation_filterプロパティ(推測航法を利用する)を
     * 指示していると、そのデータセットはバッファを利用することになります。
     *
     * この関数を利用すると、指定されたデータセットの
     * バッファにためられた情報をデータセットに転送します。
     *
     * この関数は、複製オブジェクトのインスタンスがデュプリカであるときのみ呼び出すことが出来ます。
     *
     * @param[in]   refDataset      転送するデータセットを指定します。
     * @param[in]   tRefreshTime    値を更新する時刻を指定します。
     * @retval  true    正常に転送できた場合に返ります。
     * @retval  false   転送に失敗した場合に返ります。
     *                  複製マスターがこの関数を呼び出した場合、SYSTEMERROR_OBJDUP_INVALID_ON_MASTERがシグナルされます。
     *
     * @see     IsADuplica
     */
    qBool Refresh(DataSet& refDataset, const Time& tRefreshTime)
    {
        return RefreshImpl(&refDataset, tRefreshTime);
    }


    /**********
     *
     * Callbacks.
     *
     **********/

    //{secret}
    qBool CallApproveFaultRecovery();
    //{secret}
    qBool CallApproveEmigration(qUnsignedInt32 uiReason);

    /**********
     *
     * Default callback implementation...
     *
     **********/

    // group=DO System Callbacks

    /*
    Summary: Called by $PRODUCTNAME$ to determine whether a duplicated object
    accepts to become a Duplication master after the station that controlled
    it has failed.
    Returns: true if a new host can and does become the new Duplication master and gains control of the duplicated object,
    false if the duplicated object is destroyed by $PRODUCTNAME$ or an error occurs.
    Remarks: When a fault occurs, this callback is called on all instances (Duplication master and Duplica s) of the
    given object, meaning all instances of the object must return the same result. The following code shows implementation
    of the ApproveFaultRecovery system callback, which dictates that Avatar objects are fault-tolerant:

            class Avatar : Public DOCLASS(Avatar) {
                public:
                    Avatar();
                    ~Avatar();
                    qBool ApproveFaultRecovery() {
                        return true;
                    }
                    //other application specific methods and variables
            };

    Note: By default, if the game does not implement this system callback,
    fault recovery is disabled for the given DOClass.
    See Also: Fault Tolerance Overview, ApproveEmigration
    */

    /*!
     * @brief   複製オブジェクトに障害回復機能をつけるかどうかを返します。
     *
     * 電源断などの障害発生時に、この仮想関数がシステム側から呼び出され、
     * 複製オブジェクトがセッション中に存在し続けるかどうかを返します。
     * 複製オブジェクトのすべてのインスタンス(複製マスター、およびデュプリカ)にて呼び出されますが、
     * 同一クラスのすべてのインスタンスすべてが同じ値を返す必要があります。
     *
     * この関数をアプリケーションでオーバーライドすることによって、
     * 複製オブジェクトクラスごとに挙動を変更することが出来ます。
     * デフォルトでは、このシステムコールバックは常にfalseを返す
     * (マイグレーションが発生しない)ようになっています。
     *
     * この機能は、電源断など、障害が発生したことに対してのみ有効です。
     * 正常シーケンスでのセッション離脱においても同様に複製オブジェクトを
     * セッションに残し続けるには、@ref AttemptEmigration(DOHandle) をオーバーライドしてください。
     *
     * ※この関数はアプリケーション側から呼び出すためではなく、
     * システム側から呼び出されるために用意されています。
     *
     * @retval  true  マイグレーションを発生させる場合に返ります。
     * @retval  false マイグレーションを発生させない場合に返します。
     * @see     ApproveEmigration, FaultRecoveryOperation
     */
    virtual qBool ApproveFaultRecovery();

    /*
    Description: Called by $PRODUCTNAME$ to determine if a duplicated object allows Emigration.

    $ifdef ETERNA$
    Parameters:
    uiReason: The reason for which an emigration is requested, which is either
    MIGRATION_REASON_LEAVING_SESSION or MIGRATION_REASON_LOAD_BALANCING.
    The uiReason parameter is usually set internally. However, if you wish to control an object's
    response to the ApproveEmigration call, you can add a member variable to the given object's class and use the
    value of this variable to decide whether or not this method returns true or false.
    $else$
    Parameters:
    uiReason: The reason for which an emigration is requested, which for NetZ can only be
    MIGRATION_REASON_LEAVING_SESSION. This value is set internally by $PRODUCTNAME$.
    $endif$

    Returns:  true if the given duplicated object can migrate meaning a new host will become the Duplication master
    of the object, false if $PRODUCTNAME$ will refuse any attempt to migrate the object or an error occurred.
    Remarks: If this system callback returns true, when the Station leaves the session, /i.e./ the uiReason is
    MIGRATION_REASON_LEAVING_SESSION, the objects will be migrated to another station in the Session. if the
    station quits the session cleanly then it migrates its objects itself to another station in the session,
    which is usually the first station it finds once the termination sequence commences. If you wish to control the
    station to which objects are migrated then call DuplicatedObject::AttemptEmigration before the station quits the game.
    On the other hand, if the station leaves the session due to a fault then the station to which objects will be migrated
    is determined by the PromotionReferee.

    The following code shows implementation of the ApproveEmigration system callback, which dictates
    that Avatar objects can migrate:

            class Avatar : Public DOCLASS(Avatar) {
                public:
                    Avatar();
                    ~Avatar();
                    qBool ApproveEmigration(qUnsignedInt32 uiReason) {
                        return true;
                    }
                    //other application specific methods and variables
            };

    Note: By default, if a class does not implement this system callback, Emigration
    is disabled for the given DOClass.

    See Also: Fault Tolerance Overview, Object Migration Overview, RemoveFromStoreOperation, ApproveFaultRecovery
    */

    /*!
     * @brief   複製オブジェクトがマイグレーションを行うかどうかを返します。
     *
     * マイグレーションとは、複製オブジェクトのコントロール権限が
     * 別のステーションに移行することを指します。
     * この関数は、正常シーケンスでステーションがセッションから抜けるとき、
     * その直前にシステム側から呼び出され、
     * 別のステーションにオブジェクトの権限を移行するかどうかを返します。
     * アプリケーションはこの関数をオーバーライドすることによって挙動を変更することが出来ます。
     *
     * 障害発生時(たとえば電源断などが原因でステーションが急にセッションから消えたときなど)に、
     * 複製オブジェクトが存在し続ける機能を実装する場合は、
     * この関数ではなく、@ref ApproveFaultRecovery 関数をオーバーライドします。
     *
     * ※この関数はアプリケーション側から呼び出すためではなく、
     * システム側から呼び出されるために用意されています。
     *
     * @param[in]   uiReason 常にMIGRATION_REASON_LEAVING_SESSIONが指定されます。
     * @retval  true    複製オブジェクトのマイグレーションを行います。
     *                  セッションに存在していた複製オブジェクトはそのコントロール権限をセッションに残るステーションに移譲します。
     * @retval  false   複製オブジェクトのマイグレーションを行いません。
     *                  今までセッションに存在していた複製オブジェクトは破棄されます。
     * @see     ApproveFaultRecovery, RemoveFromStoreOperation
     */
    virtual qBool ApproveEmigration(qUnsignedInt32 uiReason);

    /*
    Summary: Called by $PRODUCTNAME$ when the duplicated object has been initialized.
    Remarks: When InitDO is called, the Duplication role of the
    DuplicatedObject is fully defined.  If it is a Duplica, all its
    datasets have been appropriately initialized.

    This section explains how to use the InitDO system callback.

    Suppose we want to create a game where Avatars appear at random
    locations.  We also want to render the graphical representation of the
    Avatars at the same location in 3D space on all the stations
    immediately when they join the session.

    Lets say the object 'Avatar' contains a position dataset defined in the DDL file as follows:

            dataset Position {
                qReal x;
                qReal y;
                qReal z;
            };

    When creating an Avatar via DuplicatedObject::Create, a Duplication
    master is created, so all we need to do before rendering it is set the
    values of its position dataset. When the Avatar is published (via
    DuplicatedObject::Publish), its Duplica appear on all the stations that require a copy of the object
    which then need to be rendered at the correct position.  Thus, setting the
    Avatar position from within the InitDO function would be performed as follows:

            void Avatar::InitDO(void) {
                if (IsADuplicationMaster()) {
                    Position.x = SomeRandomValue();
                    Position.y = SomeRandomValue();
                    Position.z = SomeRandomValue();
                }
                RenderAtCurrentPosition();
            }

    Note: For this example, because the datasets have default values when
    the Avatar object is instantiated, setting the Avatar position from
    within the constructor will *not* work.

    See Also: DuplicatedObject::Create, DuplicatedObject::Publish, IsADuplicationMaster, IsADuplica
    */

    /*!
     * @brief   複製オブジェクトが初期化されたときにシステム側から呼び出されます。
     *
     * InitDOが呼び出されたとき、複製オブジェクトの役割(複製マスターであるのか、デュプリカであるのか)は決定された状態です。
     * もしこれがデュプリカである場合、すべてのデータセットは初期化された状態になっています。
     *
     * ※この関数はアプリケーション側から呼び出すためではなく、
     * システム側から呼び出されるために用意されています。
     */
    virtual void InitDO() {}

    /*
    Summary: Called by $PRODUCTNAME$ to determine the distance between two duplicated objects.
    Parameters:
    pOtherObject: A pointer to the duplicated object which you want to compute the
    distance to.
    Remarks: When distance-based dead reckoning is invoked, the desired precision of the extrapolation
    depends on a distance between the duplicated objects. The distance is a value
    specific to the game computed via this system callback and is typically the distance
    between the position of the objects in the virtual world.

    For distance-based dead reckoning $PRODUCTNAME$ needs to compute the distance
    between the registered observer (register via Station::RegisterObserver
    and get via Station::GetObserver) and the Duplica of the
    object for which it is doing the distance-based dead reckoning.  Thus, $PRODUCTNAME$
    calls this method on the Duplica of the observer and passes to the function a pointer
    to the duplicated object for which it is determining the allowable error in the
    extrapolation. The distance must be a positive value, or the special value
    UNKNOWN_DISTANCE. The default implementation returns UNKNOWN_DISTANCE.

    The following is an example that computes the distance between two
    Spheres in a virtual world.  The context of this example is found in the
    Dead Reckoning Overview.

            qReal Sphere::ComputeDistance(DuplicatedObject * pObservedDuplicatedObject) {
                if (pObservedDuplicatedObject->IsA(DOCLASSID(Sphere))) {
                    return sqrt( sqr(m_Pos.x-((Sphere*)pObservedDuplicatedObject)->m_Pos.x)
                                 +sqr(m_Pos.y-((Sphere*)pObservedDuplicatedObject)->m_Pos.y));
                } else {
                    return UNKNOWN_DISTANCE ;
                }
            }

    See Also: Dead Reckoning, Dead Reckoning Overview, ErrorToleranceFunction, UNKNOWN_DISTANCE,
    Station::RegisterObserver, PHBDRParameters
    */

    /*!
     * @brief   推測航法において、距離ベースの推定エラーを計算する際の距離計算を行います。
     *
     * 本関数は、距離ベースの推測航法を利用する際に
     * アプリケーション側で定義した複製オブジェクトクラスでオーバーライドしてください。
     *
     * アプリケーションから実行する必要はなく、Scheduler::Dispatch() 中にシステムからコールされます。
     *
     * 推測航法についてはプログラミングマニュアルを参照してください。
     *
     * @param[in]   pOtherObject            ステーションに登録されたオブザーバーである複製オブジェクトへのポインタが渡されます。
     *
     * @return  ComputeDistanceが呼び出された複製オブジェクトと、pOtherObject 間の距離を返すように実装してください。
     *
     * @see Station::RegisterObserver
     */
    virtual qReal ComputeDistance(DuplicatedObject* pOtherObject);

    // {secret}
    virtual void CallOperationOnDatasets(DOOperation* /*pOperation*/, Operation::Event /*eEvent*/);


    // {secret}
    virtual qBool ValidOperation(DOOperation* pOperation);

    /*
    Description: Called by $PRODUCTNAME$ each time a DOOperation begins.
    Parameters:
    pOperation: A pointer to the DOOperation that just began.
    Remarks: This method is called at the beginning of a DOOperation with a reference to
    the DOOperation that just began as a parameter.  Switch this parameter to find out what
    type of operation it is and then implement your functionality. See
    DuplicatedObject::OperationEnd for an example of how to use this method.
    See Also:  System Operations, OperationEnd, OperationManager
    */

    /*!
     * @brief   複製オブジェクトシステム操作が開始されるたびに呼び出されます。
     *
     * この関数は、複製オブジェクトシステム操作(@ref DOOperation)が開始されようとしているときに
     * その複製オブジェクトシステム操作インスタンスへのポインタとともにシステム側から呼び出されます。
     * @ref Operation::GetType 関数を呼び出すことによって
     * どの種類の操作が行われようとしているかを判定することが出来ます。
     *
     * ※この関数はアプリケーション側から呼び出すためではなく、
     * システム側から呼び出されるために用意されています。
     *
     * @param[in]   pOperation これから実行される操作オブジェクトへのポインタが渡されます。
     * @see     OperationEnd
     */
    virtual void OperationBegin(DOOperation* pOperation);


    /*
    Description: Called by $PRODUCTNAME$ each time a DOOperation ends.
    Parameters:
    pOperation: The DOOperation that just terminated.
    Remarks: This method is called at the end of a DOOperation with a reference to the DOOperation
    that just terminated as a parameter.  Do a switch/case on this parameter to find out what type of operation
    it is and to implement your functionality. The following is an example to capture the DOOperation
    ChangeMasterStationOperation. Thus, each time an AISphere object is locally promoted to the
    role of Duplication master, due to the Migration of the object, the local station will
    take over the control of its AI.
    Example:

            void AISphere::OperationEnd(DOOperation* pOperation) {
                switch (pOperation->GetType()) {
                    case Operation::ChangeMasterStation:
                        if ( ((ChangeMasterStationOperation*) pOperation)->IsAPromotion()){
                            // Take over control of AI
                        }
                    break;
                }
            }

    Note: To avoid deadlocks, while within an OperationEnd context you should not perform any blocking calls,
    such as CallContext::Wait.

    See Also: System Operations, OperationBegin, OperationManager
    */

    /*!
     * @brief   複製オブジェクトシステム操作が終了されるたびに呼び出されます。
     *
     * この関数は、複製オブジェクトシステム操作(@ref DOOperation)が終わった後で
     * その複製オブジェクトシステム操作インスタンスへのポインタとともにシステム側から呼び出されます。
     * @ref Operation::GetType 関数を呼び出すことによって
     * どの種類の操作が行われようとしているかを判定することが出来ます。
     *
     * ※この関数はアプリケーション側から呼び出すためではなく、
     * システム側から呼び出されるために用意されています。
     *
     * @param[in]   pOperation これから実行される操作オブジェクトへのポインタが渡されます。
     * @see     OperationEnd
     */
    virtual void OperationEnd(DOOperation* pOperation);


    /*
    $ifdef ETERNA$
    Description: Returns the weight of a given duplicated object for load balancing purposes.
    Returns: The hypothetical weight assigned to a duplicated object for load balancing purposes.
    Remarks: This method is used to assign a hypothetical weight to a given duplicated object
    that will participate in load balancing. The weight of an object is equivalent to how much
    the duplication master object contributes to a station's local load. An object will only
    be migrated if its weight is less than the total load to be transferred during a load balancing
    episode. The default weight of an object is 0. When this is the case the object will not
    participate in load balancing.

    Note: An object's weight must be equal to or greater than 0.

    See Also: LoadBalancingDomain, Object Migration Overview, LoadBalancingDomain::GetLocalLoad
    $else$
    {secret}
    $endif$
    */
    virtual qReal GetWeight()
    {
        return QREAL(0.0);
    }

    //group=DO Methods

    /*
    Description: Traces the default information for a duplicated object instance.
    Remarks: The default trace information for a duplicated object includes its class name,
    DOHandle, Duplication role ( /i.e./ Duplica or Duplication master), its ref count ( /i.e./ how many
    DORef s are presently created in the system for this object), whether or not
    it is flagged for destruction, as well as the station on which its Duplication master instance resides.
    Redefine the Trace method in the implementation of your DOCLASS in order to
    trace game specific information to that particular DOCLASS.
    However, in order to continue to trace the default information, ensure to include a call to
    DOCLASS(YourDOCLASS)::Trace().
    Example:
        class Map:DOCLASS(Map){
            [...]
            void Trace(qUnsignedInt64 uiTraceFlag) {
                DOCLASS(Map)::Trace(uiTraceFlag);
                QTRACE(uiTraceFlag,NEX_T("Map number is ") << dec << (GetMapNumber()));
            }
            [...]
        };

    See Also: DOSelections::Trace, Data Logging, TraceLog
    */

    /*!
     * @brief   この複製オブジェクトの状態をトレースします(デバッグ用)。
     * @param[in]   uiTraceFlag  どのフラグが有効であるときにトレースを実行するのか指定します。
     */
    virtual void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;

    //{secret}
    static void TraceOperationStack(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);


    // Discovery related methods...
    // {Secret}
    void SpecificAddDSToDiscoveryMessage(Message* pMsg, Station* pStation)
    {
        DUMMY_PARAM(pMsg);
        DUMMY_PARAM(pStation);
    };
    // {Secret}
    void SpecificExtractDSFromDiscoveryMessage(Message* pMsg)
    {
        DUMMY_PARAM(pMsg);
    };
    // {Secret}
    qBool SpecificRefresh(DataSet * pDataset, const Time& tRefreshTime);
    // {Secret}
    qBool SpecificExtractADataset(Message* pMsg, qByte byDatasetIndex);
    // {Secret}
    qResult SpecificUpdate(DataSet *pDataset, Time tUpdateTime);
    // {Secret}
    void FillDupSpaceInfo(DupSpace::Role eRole, DupSpace::ID* pArray, qUnsignedInt* puiIndex)
    {
        DUMMY_PARAM(eRole);
        DUMMY_PARAM(pArray);
        DUMMY_PARAM(puiIndex);
    }

    // Methods related to action calls.
    //{secret}
    Message* CreateStubMessage(MethodID* pidMethod);
    //{secret}
    qBool SendStubMessage(qBool bSendToDuplicas, Message* pMsg);
    //{secret}
    void SendToAllDuplicas(Message* pMsg, qUnsignedInt32 uiFlags);
    // {secret}
    void SendToSomeDuplicas(Selection* pTargets, Message* pMsg, qUnsignedInt32 uiFlags);

    qBool IsOverReliableSendBufferLimit(const Selection* pTargets, qUnsignedInt32 uiFlags, SubStreamID subStreamID) const;
    qBool IsInvalidSubStreamID(const Selection* pTargets, SubStreamID subStreamID) const;

    /*
     * Adapter methods
     */

    // {secret}
    // Generated code will override this method.  This method is
    // there to provide support for the adapter extension.
    virtual void CallOperationEndOnAdapters(DOOperation* /*pOperation*/) {}

public:

    /**********
     *
     * Operations entry points.
     *
     **********/

    // {secret}
    // TODO.  Cleanup up the arguments of this method.
    qBool RemoveFromStore(DOHandle hSourceStation, qBool bDeleted, qBool bRemoveDuplicas);

    // {secret}
    qBool AddToStoreAsDuplica(DOHandle hSourceStation, Message* pMessageToInitFrom=NULL);
    // {secret}
    qBool AddToStoreAsMaster();
    //{secret}
    qBool UndeleteMainRef();

    // {secret}
    qBool ChangeMasterStation(DOHandle hSourceStation,
                              DOHandle hOldStation,
                              const MasterStationRef& refNewMasterStation,
                              const qList<DOHandle> *plstDupSet,
                              qUnsignedInt uiContext);

    //{secret}
    static void UpdateDatasets(Message *pMessage, DOHandle hTargetDO, qByte byDatasetIndex);

    /*
    Description: Returns a pointer to the DOOperation that is currently in progress.
    Returns: A pointer to the DOOperation that is currently in progress, or a NULL pointer
    if no DOOperation is currently in progress or an error occurred.
    Remarks: This method is most commonly used to determine if the CallMethodOperation
    is in progress in order to control the calling of RMCs.
    See Also: DOOperation
    */

    /*!
     * @brief   現在実行している最中の、複製オブジェクトに対する操作オブジェクトへのポインタを取得します。
     *
     * この関数は、RMC(リモートメソッドコール)呼び出しを制御するために、
     * @ref CallMethodOperation が実行中であるかどうかを調べるときに利用します。
     *
     * @return  現在実行している最中の、DOOperationへのポインタが返ります。
     * @see     Operation::GetType
     */
    static DOOperation* GetCurrentOperation();
    // {secret}
    qBool ExecuteOperation(DOOperation& oOperation);

protected:

    // {secret}
    virtual void TestInvariants();

private:
    // {secret}
    qResult UpdateImpl(DataSet *pDataset, const Time& tUpdateTime);
    // {secret}
    qResult UpdateImplCore(DataSet *pDataset, const Time& tUpdateTime);

    // {secret}
    qBool RefreshImpl(DataSet* pDataset, const Time& tRefreshTime);
    // {secret}
    qBool RefreshImplCore(DataSet* pDataset, const Time& tRefreshTime);

    qBool ExecuteOperationImpl(DOOperation& oOperation);

    // {secret}
    qBool FaultRecoveryImpl(DOOperation* pOperation);

    // {secret}
    void InValidOperationLog(const DOOperation* pOperation)const;

    void ExecCallMethod(const CallMethodOperation& refOp);
    void ExecChangeMasterStation(const ChangeMasterStationOperation& refOperation);
    void ExecRemoveFromStore(const RemoveFromStoreOperation& refOperation);
    void ExecAddToStore(const AddToStoreOperation& refOp);
    void ExecUpdateDataSet(const UpdateDataSetOperation& refOperation);
    void ExecChangeDupSet(const ChangeDupSetOperation& refOperation);

    /**********
     *
     * Fetch/CreateDuplica methods
     *
     **********/
public:

    /*
    Description: Creates a Duplica of a given duplicated object.
    Parameters:
    hNewDuplicaStation: The DOHandle of the station on which a Duplica will be created.
    Returns: true if the Duplica is successfully created on the given station, false if not or an
    error occurred.
    Remarks: This method is one manner in which to create a Duplica of an object on a station.
    The method must be called on a Duplication master to create a Duplica of itself.
    See Also: FetchDuplica
    */

    /*!
     * @brief   この複製オブジェクトのデュプリカを作成します。
     *
     * この関数は、ステーションにオブジェクトのデュプリカを作成するひとつの方法です。
     * この関数は複製マスターが呼び出し、それ自身のデュプリカを作るために呼び出さなければなりません。
     *
     * @param[in]   hNewDuplicaStation デュプリカの作成先のステーション(@ref Station)の複製オブジェクトハンドル(@ref DOHandle)を指定します。
     * @retval  true    デュプリカの作成に成功したときに返ります。
     * @retval  false   デュプリカの作成に失敗したときに返ります。
     */
    qBool CreateDuplica(DOHandle hNewDuplicaStation);

    /*
    Description: Fetches a Duplica of a given duplicated object.
    Parameters:
    hFetchedObject: The DOHandle of the fetched object.
    pContext: A pointer to the FetchContext.

    Remarks: This method is one manner in which to create a Duplica of an object on a station.
    Example:
            bool PlayerManager::RequestPlayer(DOHandle hPlayerID, Player::Ref* prefPlayer) {
                FetchContext oFetchContext;

                // In this example, we assume that the session master holds a copy
                // of the desired player.
                oFetchContext.SetTargetStation(Session::GetInstance()->GetMasterStation());
                oFetchContext.ClearFlag(FetchContext::SynchronousCall);

                if (!DuplicatedObject::FetchDuplica(&oFetchContext, hPlayerID) {
                    printf("Unable to fetch duplica.\n" );
                    return false;
                }

                // We wait until the call completes.
                oFetchContext.Wait();

                // Once the call has completed, we grab a reference to the newly
                // fetched object.  This step is MANDATORY: if we do not do it,
                // then the object that has just been fetched is not considered
                // relevant (as there are no references to it), and will eventually be
                // removed by the system.
                *prefPlayer = oFetchContext.GetFetchedObject();
                if (!prefPlayer->IsValid()) {
                    printf("Invalid player reference.\n" );
                    return false;
                }
                return true;
            }

    See Also: CreateDuplica, FetchContext, FetchInProgress
    */

    /*!
     * @brief   複製オブジェクトのデュプリカを取ってきます。
     *
     * この関数は、ステーションにオブジェクトのデュプリカを作成するひとつの方法です。
     *
     * @param[in,out]   pContext    @ref FetchContext へのポインタを指定します。
     * @param[in]   hFetchedObject  取ってこられる複製オブジェクトへのハンドル(@ref DOHandle)を指定します。
     *
     * @retval  true    複製オブジェクトのデュプリカを取ってくるのに成功した場合に返ります。
     * @retval  false   複製オブジェクトのデュプリカを取ってくるのに失敗した場合に返ります。
     */
    static qBool FetchDuplica(FetchContext* pContext, DOHandle hFetchedObject);

    /*
    Description: Returns whether or not a given object is in the process of being fetched.
    Parameters:
    hTargetStation: The DOHandle of the target station from which the Duplica will be fetched.
    hFetchedObject: The DOHandle of the fetched object.

    Returns: true if the specified object is in the process of being fetched from the target
    station (if specified), false if not or an error occurred.
    See Also: FetchDuplica

    */
    static qBool FetchInProgress(DOHandle hFetchedObject);
    static qBool FetchInProgress(DOHandle hFetchedObject, DOHandle hTargetStation);


    /**********
     *
     * FaultRecovery methods.
     *
     **********/

    // {secret}
    qBool PerformFaultRecovery(DOHandle hNewMaster, LogicalClock lcNewMasterVersion);

    /**********
     *
     * Migration methods
     *
     **********/
    //group=DO Migration Methods

    /*
    Description: Emigrates a duplicated object to a specific station.
    Parameters:
    idNewLocation: The StationID to which the duplicated object will emigrate ( /i.e./ the destination
    StationID for the duplicated object).
    pContext: A pointer to the MigrationContext of this call.
    hTargetStation: The DOHandle of the target station to which the object will migrate.
    Remarks:  This method must be called on the Duplication master.  The signature of this method that
    does not contain a MigrationContext locks until the emigration completes or the error
    SYSTEMERROR_OBJDUP_CANNOT_EMIGRATE is generated due to the unavailability of station idNewLocation.
    On the other hand, the second signature returns immediately before ensuring successful migration to
    hTargetStation or encountering a system error.

    Note: Alternatively, you may use the method AttemptEmigration, which returns immediately but
    does not guarantee a successful emigration.

    Returns: true if the object is successfully emigrated, false if not or an error occurred.
    The errors SYSTEMERROR_OBJDUP_CANNOT_EMIGRATE and SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA may be signalled by this call.
    See Also: AttemptEmigration, MigrationInProgress
    */
    // obsolete
    qBool Emigrate(StationID idNewLocation);
    /*!
     * @brief   この複製オブジェクトを指定されたステーションに対してエミグレーションさせます。
     *
     * マイグレーションとは、複製オブジェクトのコントロール権限がステーション間で移転することをいいます。
     * マイグレーションには大きく分けて二つのプロセスがあり、
     * 最初に実行される複製オブジェクトの権限が移行するプロセスのことをエミグレーション(emigration)、
     * 次に実行される複製オブジェクトを受け入れるプロセスをイミグレーション(immigration)と呼びます。
     * この関数では前者の処理を実行することになります。
     *
     * この関数は複製マスター(@ref IsADuplicationMaster)が呼び出す必要があります。
     * 指定された@a pContextが初期化されたときに、コンストラクタに同期実行を指定すると、
     * この関数呼び出しは即座に終了せずに実行が完了するかエラーが発生するまで返ってこなくなります。
     * 非同期実行を指定すると、この関数呼び出しは即座に終了します。
     *
     * @param[in,out]   pContext        この呼び出しの@ref MigrationContext へのポインタを指定します。
     * @param[in]   hTargetStation      複製オブジェクトの移転先ステーションの複製オブジェクトハンドルを指定します。
     * @retval  true    呼び出しが正常に開始された場合に返ります。
     * @retval  false   呼び出しが正常に開始されなかった場合に返ります。
     *                  SYSTEMERROR_OBJDUP_CANNOT_EMIGRATEまたはSYSTEMERROR_OBJDUP_INVALID_ON_DUPLICAがシグナルされます。
     * @see     AttemptEmigration(DOHandle), MigrationInProgress
     */
    qBool Emigrate(MigrationContext* pContext, DOHandle hTargetStation);

    /*
    Description: Returns whether or not an object is in the process of migrating.
    Returns: true if the object is in the process of migrating, false if not or an error occurred.
    Remarks: An object may migrate due to Load Balancing, due to a fault (if the object is fault-tolerant), or
    due to a call to either AttemptEmigration or Emigrate.
    See Also: Fault Tolerance Overview, Emigrate, AttemptEmigration
    */

    /*!
     * @brief   この複製オブジェクトがマイグレーションを行っている最中かどうかを取得します。
     *
     * マイグレーションとは、複製オブジェクトのコントロール権限がステーション間で移転することをいいます。
     * 複製オブジェクトは、以下の理由によってマイグレーションを行う場合があります。
     *
     * @li ロードバランス(NPCのAIや物理計算などの処理などにかかるCPU時間を均等化するためにセッション内で行う負荷分散)
     * @li 障害回復(@ref ApproveFaultRecovery)
     * @li @ref AttemptEmigration(DOHandle) 関数か@ref Emigrate(MigrationContext*, DOHandle) 関数の呼び出しによる移転
     *
     * @retval  true    マイグレーションを実行している最中のときに返ります。
     * @retval  false   マイグレーションを実行していないか、エラーが発生した場合に返ります。
     * @see     Emigrate, AttemptEmigration(DOHandle)
     */
    qBool MigrationInProgress() const;
    void MigrationFault() const;

    /*
    Summary: The method attempts to emigrate a duplicated object to a specific station.
    Parameters:
    idNewLocation: The StationID to where the duplicated object will attempt to emigrate
    ( /i.e./ the destination StationID for the duplicated object).
    hNewLocation: The DOHandle of the station to where the duplicated object will attempt to emigrate.
    Returns: true if the attempt to migrate an object was successful, false if not or an error occurred.
    The errors SYSTEMERROR_OBJDUP_CANNOT_EMIGRATE and SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA may be signalled by this call.
    Remarks:  This method must be called on the Duplication master of an object. The method returns immediately
    before ensuring successful migration or encountering a system error. If a fault occurs on the destination
    station then migration will not proceed. If you wish to control to which stations fault tolerant objects
    are migrated, for load balancing for example, then you should call this method before the Station quits the Session.

    Note: Alternatively, you may use the method Emigrate, one signature of which locks until the emigration process
    is successful or an error is generated.

    See Also: Emigrate , MigrationInProgress
    */
    // obsolete
    qBool AttemptEmigration(StationID idNewLocation);
    /*!
     * @brief   この複製オブジェクトを指定されたステーションに対してエミグレーションさせます。
     *
     * マイグレーションとは、複製オブジェクトのコントロール権限がステーション間で移転することをいいます。
     * マイグレーションには大きく分けて二つのプロセスがあり、
     * 最初に実行される複製オブジェクトの権限が移行するプロセスのことをエミグレーション(emigration)、
     * 次に実行される複製オブジェクトを受け入れるプロセスをイミグレーション(immigration)と呼びます。
     * この関数では前者の処理を実行することになります。
     *
     * この関数は複製マスター(@ref IsADuplicationMaster)が呼び出す必要があります。
     * 関数の実行は即座に終了します(非同期実行)。
     *
     * @param[in]   hNewLocation    複製オブジェクトの移転先ステーションの複製オブジェクトハンドルを指定します。
     * @retval  true    呼び出しが正常に開始された場合に返ります。
     * @retval  false   呼び出しが正常に開始されなかった場合に返ります。
     * @see     Emigrate, MigrationInProgress
     */
    qBool AttemptEmigration(DOHandle hNewLocation);

    // {secret}
    void PrepareToLeave();
    //{secret}
    DOHandle SelectNewLocation(qUnsignedInt32 uiReason);

    //group=DO Methods

    // Misc methods...
    //{secret}
    DOClass* GetDOClass() const
    {
        return GetDOClass(GetDOClassID());
    }
    // {secret}
    static DOClass* GetDOClass(DOClassID idDOClass);

    /*
    Description:  Returns the DOHandle of the Station that has this object's Duplication master in its store.
    Returns:  The DOHandle of the Station on which the Duplication master for this duplicated object resides.
    Remarks: In order to determine the Session master of a given Session, call this method on the Session object.

    Remarks: Refer to FetchDuplica for a code sample.
    */

    /*!
     * @brief   この複製オブジェクトの複製マスターとなっている@ref Station オブジェクトの複製オブジェクトハンドルを取得します。
     *
     * @ref Session クラスに対してこの関数を呼び出せば、ゲームセッションのマスターが誰かを調べることが出来ます。
     *
     * @return  この複製オブジェクトの複製マスターとなっている@ref Station オブジェクトの@ref DOHandle が返ります。
     */
    DOHandle GetMasterStation() const
    {
        return m_refMasterStation.GetStationHandle();
    }
    // {secret}
    const Station* GetMasterStationPtr() const
    {
        return m_refMasterStation.GetStation();
    }
    // {secret}
    // TODO - Still required?
    const LogicalClock& GetMasterVersion() const
    {
        return m_refMasterStation.GetVersion();
    }
    // {secret}
    void SetMasterStation(const MasterStationRef& refNewMaster);

    //Description:  Returns the StationID of the station that has this object's Duplication master in its store.
    //Returns:  The StationID of the station on which the Duplication master for this duplicated object resides.
    /*!
     * @brief   この複製オブジェクトのマスターのステーションのStationIDを取得します。
     *
     * @return  この複製オブジェクトのマスターのステーションのStationIDが返ります。
     */
    StationID GetMasterID() const;

    /*
    Description: Returns whether or not DuplicatedObject is a Duplica.
    Returns: true if the DuplicatedObject is a Duplica, false if it is a Duplication master or unknown.
    Remarks: If the method is called before the object's Duplication role has been
    initialized, the object is neither a Duplication master nor a Duplica.
    Typically, this occurs if the methods IsADuplicationMaster or
    IsADuplica are called in the object constructor.  When the
    initialization of an object depends on its role, it should be performed
    via the InitDO system callback, where the role has been defined.
    See Also: Duplication role, IsADuplicationMaster
    */

    /*!
     * @brief   この複製オブジェクトがデュプリカであるか、そうでないかを取得します。
     *
     * デュプリカとは、複製オブジェクトの中でも、複製マスターのただのコピーである状態のことを言います。
     * 複製マスター(コピー元)に加えられた値の変化は、ネットワークを通じてセッション中に存在する
     * デュプリカの値を更新し、コピーされた状態を保ち続けようとします。
     * この関数がtrueを返したとき、この複製オブジェクトのインスタンスはデュプリカです。
     *
     * 複製オブジェクトに対する役割が初期化されているときは、複製オブジェクトは
     * 複製マスターでもデュプリカでもありません。この関数がコンストラクタなどで
     * 呼び出された場合がその典型的なケースです。どちらの役割になるかは
     * @ref InitDO コールバックによって決定されます。
     *
     * @retval  true  この複製オブジェクトはデュプリカです。
     * @retval  false この複製オブジェクトは複製マスターであるか、不明な状態です。
     */
    qBool IsADuplica() const;

    /*
    Description: Returns whether or not the DuplicatedObject is the Duplication master.
    Returns: true if the DuplicatedObject is a Duplication master, false if it is a Duplica or unknown.
    Remarks: If the method is called before the object's Duplication role has been
    initialized, the object is neither a Duplication master nor a Duplica.
    Typically, this occurs if the methods IsADuplicationMaster or
    IsADuplica are called in the object constructor.  When the
    initialization of an object depends on its role, it should be performed
    through the InitDO system callback, where the role has been defined.
    See Also: Duplication role, IsADuplica
    */

    /*!
     * @brief   この複製オブジェクトが複製マスターであるか、そうでないかを取得します。
     *
     * 複製マスターとは、複製オブジェクトの中でも、現在実行しているマシンによって作成されたか、
     * あるいは現在実行しているマシンに権限が移転されてきたかによって、
     * オブジェクトのデータを提供している側の状態のことを言います。
     * このオブジェクトに加えられた値の変化は、ネットワークを通じてセッション中に存在するデュプリカ
     * (複製マスターから情報をコピーされたオブジェクト)の値を更新します。
     * この関数がtrueを返したとき、この複製オブジェクトのインスタンスは複製マスターです。
     *
     * 複製オブジェクトに対する役割が初期化されているときは、複製オブジェクトは
     * 複製マスターでもデュプリカでもありません。この関数がコンストラクタなどで
     * 呼び出された場合がその典型的なケースです。どちらの役割になるかは
     * @ref InitDO コールバックによって決定されます。
     *
     * @retval  true  この複製オブジェクトは複製マスターです。
     * @retval  false この複製オブジェクトはデュプリカであるか、不明な状態です。
     */
    qBool IsADuplicationMaster() const;

    /*
    Description: Returns whether or not the DuplicatedObject has authority.
    Returns: true if the DuplicatedObject has authority, false if it does not.
    Remarks: A DuplicatedObject can only have authority if it is the Duplication master instance
    of the object. A Duplication master that has authority can safely perform session critical
    operations. Refer to the Duplicated Object Overview for more information.
    See Also: Duplicated Object Overview
    */

    /*!
     * @brief   この複製オブジェクトが権限を持っているかどうかを取得します。
     *
     * 複製オブジェクトは複製マスターであるときのみ権限を持ちます。
     * 権限を持つ複製マスターはセッションクリティカルな操作
     * (たとえばオブジェクトの主参照を削除する、あるいは移行するなど)を
     * 行うことが出来ます。ただし、マイグレーション(複製オブジェクトのコントロール権限の移転)を
     * 行っている最中の複製オブジェクトは、
     * どのステーションもセッションクリティカルな操作が出来ない状態となります。
     * 複製マスターであったとしても、マイグレーション実行中はコントロール権限がありません。
     * この関数では、マイグレーションが実行されている最中かどうかも含めて判定することが出来ます。
     *
     * NEXプログラミングマニュアルに、マイグレーションに関する
     * タイムラインなども含めた詳しい解説を掲載していますので併せて参照してください。
     *
     * @retval  true  複製オブジェクトに対する権限を持っているときに返ります。
     * @retval  false 複製オブジェクトに対する権限を持っていないときに返ります。
     */
    qBool HasAuthority() const
    {
        return IsADuplicationMaster() && !MigrationInProgress();
    }

    /*
    Summary:  Returns whether or not a duplicated object is an instance of
    a well-known duplicated object class.
    Returns:  true if the DuplicatedObject instance is a member of a class that was declared in the DDL file as
    well-known, false if not or an error occurred.
    Remarks: A Well-known duplicated object is known globally across the Session and is guaranteed to be
    discovered by a Station before the Session commences on that Station.
    See Also:  DDL File Syntax, DOHandle, WKHandle, IsAUserDO, IsACoreDO, IsGlobal
    */

    /*!
     * @brief   この複製オブジェクトが既知オブジェクトクラスのインスタンスであるかどうかを取得します。
     *
     * 既知オブジェクトは、セッションの中で全体的に知られていて、
     * すべてのステーションはセッションに参加する前からオブジェクトの状態が共有されるような
     * オブジェクトのことを指します。
     *
     * @retval  true  このオブジェクトはDDL(データ定義言語)ファイルで定義されている既知オブジェクトのインスタンスです。
     * @retval  false このオブジェクトは既知オブジェクトのインスタンスではありません。
     */
    qBool IsAWellKnownDO() const;

    /*
    Description: Returns whether or not the DuplicatedObject is a CoreDO.
    Returns: true if the DuplicatedObject is a core object created by $PRODUCTNAME$, false if not or an error occurred.

    $ifdef ETERNA$
    Remarks:
    A CoreDO is a DuplicatedObject created by $PRODUCTNAME$ when a Session is created or when
    Station s join or leave the Session. These objects are essential for the correct operation of
    $PRODUCTNAME$ and include the root classes for all global and non-global DuplicatedObject s,
    the Session, SessionClock, Station, and IDGenerator classes, system classes involved
    in fault tolerance, fault recovery, and multicast messaging mechanisms, a default cell class if the
    DuplicationSpace uses Cell s, and the LoadBalancingDomain and other classes involved in load balancing.

    $else$
    Remarks:
    A CoreDO is a DuplicatedObject created by $PRODUCTNAME$ when a Session is created or when
    Station s join or leave the Session. These objects are essential for the correct operation of
    $PRODUCTNAME$ and include the root classes for all global and non-global DuplicatedObject s,
    the Session, SessionClock, Station, and IDGenerator classes, system classes involved
    in fault tolerance, fault recovery and a default cell class.
    $endif$
    See Also: IsAUserDO, IsAWellKnownDO, IsGlobal
    */

    /*!
     * @brief   この複製オブジェクトがCoreDOであるかどうかを取得します。
     *
     * CoreDOとは、Net-Zシステムによって作成される複製オブジェクトのこと言います。
     * アプリケーションのDDL(データ定義言語)によって宣言された複製オブジェクトクラスは
     * CoreDOではなくUserDOと呼ばれます。CoreDOには@ref Station 、@ref Session 、
     * @ref SessionClock などのクラスが含まれます。
     *
     * @retval  true  この複製オブジェクトがCoreDOであるときに返ります。
     * @retval  false この複製オブジェクトがCoreDOでないときに返ります。
     * @see     IsAUserDO
     */
    virtual qBool IsACoreDO() const;

    /*
    Description: Returns whether or not the DuplicatedObject is a UserDO.
    Returns: true if the DuplicatedObject is a user-defined object, false if not or an error occurred.
    Remarks: A UserDO is a user-defined DuplicatedObject declared by the user in the DDL.
    See Also: IsACoreDO, IsAWellKnownDO, IsGlobal
    */

    /*!
     * @brief   この複製オブジェクトがUserDOであるかどうかを取得します。
     *
     * UserDOとは、アプリケーションのDDL(データ定義言語)によって宣言された
     * 複製オブジェクトクラスのことを言います。
     *
     * @retval  true  この複製オブジェクトがUserDOであるときに返ります。
     * @retval  false この複製オブジェクトがUserDOでないときに返ります。
     * @see     IsACoreDO
     */
    qBool IsAUserDO() const
    {
        return !IsACoreDO();
    }

    // {secret}
    virtual qBool IsABootstrapDO() const
    {
        return false;
    }

    /*
    Description: Returns a pointer to an OperationManager.
    Returns: A pointer to an OperationManager.
    Remarks: This method returns a pointer to the OperationManager that manages all DOOperation s. The main
    purpose of the OperationManager is to manage the callbacks called at the beginning and end of the various
    $PRODUCTNAME$ System Operations.
    See Also: OperationManager
    */

    /*!
     * @brief   @ref OperationManager へのポインタを取得します。
     *
     * この関数は、すべての@ref DOOperation オブジェクトを管理する@ref OperationManager オブジェクトへのポインタを返します。
     * @ref OperationManager の主な目的は、さまざまなシステム操作の開始と終了時に呼び出されるコールバック関数の管理です。
     *
     * @return  @ref OperationManager へのポインタが返ります。
     */
    static OperationManager* GetOperationManager();

    /*
    Description: Instructs $PRODUCTNAME$ to delete the main reference of the duplicated object.
    Returns:  true if the main Ref of the object was successfully deleted, false if not or an error occurred.
    The error SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA is returned if this method is called on a Duplica.
    Remarks: This method deletes the main (system) reference to the duplicated object held in the
    duplicated object store. Once the main reference is deleted the object's datasets will cease to be updated
    and the object will no longer be "visible" when a selection of objects is iterated over, nor be listed by
    the Object Monitor. If the main reference was the only reference to the object, the destructor will be
    called and the object will be deleted with this call. If other references to the Duplication master or Duplica s
    of the object exist, /e.g./ created via the RefTemplate or Ref, the destructor will not be called until the last
    reference is released. When the destructor is called all the Duplica s of the object will be deleted.

    This method can only be called on the Duplication master of an object and may be called before an
    object is published.

    Example:
    // This is one way to delete all the Duplication master objects on a Station.
    {
        Avatar::SelectionIterator itAvatarDM(DUPLICATION_MASTER);
        while (!itAvatarDM.EndReached()) {
            itAvatarDM->DeleteMainRef();
            itAvatarDM++;
        }
    }

    See Also: MainRefIsDeleted, RemoveFromStoreOperation, Publish
    */

    /*!
     * @brief   複製オブジェクトの主参照を削除するように指示します。
     *
     * この関数を呼び出すと、複製オブジェクトストアに格納されている
     * 複製オブジェクト主参照(システム参照)を削除します。
     * オブジェクトの主参照が削除されると、オブジェクトの持つデータセットは更新することをやめ、
     * イテレーションされている選択からはほどなく見えなくなります。
     *
     * 主参照が唯一の参照であった場合、複製オブジェクトのデストラクタが呼び出され、
     * この呼び出しによって複製オブジェクトは削除されます。
     * この複製オブジェクトの複製マスターまたはデュプリカに対する参照がまだ残っている場合
     * (たとえば@ref RefTemplate によってこのオブジェクトへの参照が存在する場合など)、
     * デストラクタは最後の参照が解放されるまでは実行されません。
     * デストラクタが呼び出されたときに、このオブジェクトのすべてのデュプリカは削除されます。
     *
     * この関数は複製マスターのみが呼び出すことが出来ます。
     * 複製オブジェクトに対して@ref Publish 関数が呼び出されていない状態であっても呼び出すことは可能です。
     *
     * @see     ~DuplicatedObject, MainRefIsDeleted
     */
    qBool DeleteMainRef();
    // {secret}
    qBool DeleteDuplicaMainRef();

private:
    // {secret}
    qBool DeleteMainRefImpl();

public:

    // {secret}
    qBool FlagIsSet(qUnsignedInt16 ui16Flag) const
    {
        return (m_uiFlags & ui16Flag)==ui16Flag;
    }
    /*
    Description: Returns whether or not DeleteMainRef has been called on this object.
    Remarks: If the method DeleteMainRef has been called and therefore, the main reference to
    the object has been deleted, the destructor will be called on the object when all references
    to the object have been released, /i.e./ there are no Ref s nor iterators that point to the object.
    Returns: true if the method DeleteMainRef has been called on this object and therefore, the
    main reference to the object has been deleted, false if not or an error occurred.
    See Also: DeleteMainRef
    */

    /*!
     * @brief   @ref DeleteMainRef 関数がこのオブジェクトに対して呼び出されたかどうかを取得します。
     *
     * @ref DeleteMainRef 関数が呼び出されたら
     * (つまり、複製オブジェクトの主参照が削除されようとしたら)、
     * すべての参照が解放されたタイミングでデストラクタが呼ばれます。
     *
     * @retval  true  主参照が削除されたときに返ります。
     * @retval  false 主参照は削除されていないか、エラーが発生した場合に返ります。
     * @see     DeleteMainRef
     */
    qBool MainRefIsDeleted() const
    {
        return !FlagIsSet(MainReferenceInUse);
    }

    // {secret}
    qBool ConnectOrphanDuplica();
    // {Secret}
    static qUnsignedInt32 GetRealDOClassID()
    {
        return DuplicatedObject_ClassID;
    }

#ifdef FOR_DOC_PURPOSE
    /*
    Summary:  Creates a local instance of a specified doclass.
    Parameters:
    idDOClass: The DOClassID of the duplicated object that will be created.
    uiTimeout: The maximum time in milliseconds that the system
    will wait. The default uiTimeout is infinite.
    idUserDefinedID: (Optional). When used, assigns the specified DOID to the duplicated object,
    bypassing the automatic DOID generation mechanism. Note that you MUST assign a different
    DOID to each duplicated object within the same class regardless of its location.
    dohHandle: The DOHandle of the duplicated object that will be created.
    Returns: SYSTEMERROR_GEN_INVALID_WAIT if this method is called in a single thread context
    with a timeout value other than 0.
    Remarks:  The DDL generated duplicated object classes inherit from the DuplicatedObject class.
    A specific constructor for each doclass is generated by the DDL compiler into each DOCLASS.
    It is not possible to create generic DuplicatedObject instances and you need to specify the
    doclass of the instance that you want to create.

    For example, given that Avatar is declared as a doclass in the DDL file, an Avatar object can be created
    and published in any of the following ways:

            // 1. Let $PRODUCTNAME$ assign the IDs for the object.
            {
                Avatar* pNewAvatar=Avatar::Create();
                pNewAvatar->Publish();
            }

            // 2. Assign the DOID.
            {
                ID idNewAvatar=0;
                IDGenerator::Ref refIDGenerator(Avatar::GetIDGenerator());
                if (refIDGenerator->GenerateID(&idNewAvatar, 0)) {
                    Avatar* pNewAvatar=Avatar::Create(idNewAvatar);
                    pNewAvatar->Publish();
                } else {
                    return false;
                }
            }

            // 3. Assign the DOClassID and let $PRODUCTNAME$ assign
            //    the DOID.
            {
                DuplicatedObject* pNewAvatar=DuplicatedObject::Create(DOCLASSID(Avatar));
                pNewAvatar->Publish();
            }

            // 4. Assign both the DOClassID and DOID.
            {
                ID idNewAvatar=0;
                IDGenerator::Ref refIDGenerator(Avatar::GetIDGenerator());
                if (refIDGenerator->GenerateID(&idNewAvatar, 0)) {
                    Avatar* pNewAvatar=(Avatar*)DuplicatedObject::Create(DOCLASSID(Avatar), idNewAvatar);
                    pNewAvatar->Publish();
                } else {
                    return false;
                }
            }

            // 5. Assign a DOHandle.

            // Assume there is a database from which we
            // save/retrieve information.

            DOHandle hPreviouslySavedAvatar = ReadHandleFromDatabase();
            Avatar* pNewAvatar=(Avatar*) DuplicatedObject::Create(hPreviouslySavedAvatar);
            // Initializes the attributes of pNewAvatar with the
            // info saved previously in the database, and then publish
            // the new avatar.
            pNewAvatar->Publish();

    It should be noted that as an object's DOClassID and therefore, its DOHandle may change when you recompile
    your project, alter your DDL file, or upgrade to a new version of $PRODUCTNAME$, it is safer to store and
    retrieve DOID s rather than DOHandle s in the database.

    Note:

    o The method DuplicatedObject* Create(DOID idUserDefinedID) is defined by a specific class.
    o To create a Session duplicated object, Session::CreateSession should be called rather than DuplicatedObject::Create.
    o If Create is called in the context of an RMC a non-zero timeout cannot be specified.
    See Also: Publish, CreateWellKnown
    */

    /*!
     * @refmanoverload{name1}
     *
     * @brief   複製オブジェクトのローカルインスタンスを作成します。
     *
     * 本関数はNULLを返す可能性があります。
     * NULLが返ってきた場合には速やかにNetZインスタンスを破棄し、セッションから抜けてください。
     *
     * 通常はブロックせずにすぐに値が返されます。
     * しかし、セッション参加後の複製オブジェクトの初期化処理が完了するよりも前に、
     * 親と通信できなくなったレアなケースではNULLが返り、
     * このような場合に最大で uiTimeout のブロックが発生することがあります。
     *
     * uiTimeout に0を指定してノンブロックで本関数を利用することもできます。
     *
     * この静的関数は、DDL(データ定義言語)コンパイラによって作成される
     * すべての複製オブジェクトクラスごとに定義される関数です。
     * たとえば、Avatarクラスを作成する場合、Avatar::Create()と呼び出します。
     * DuplicatedObject::Createという関数はサポートされません。
     *
     * @param[in]   uiTimeout   作成されるのを待つタイムアウトをミリ秒単位で指定します。デフォルトは1000です。
     * @return  作成された複製オブジェクトローカルインスタンスへのポインタが返ります。
     * @retval  NULL    セッション参加後の複製オブジェクトの初期化処理が完了するよりも前に、
     *                  親と通信できなくなったレアなケースで返ります。
     *                  NULLが返ってきた場合、必ずNetZインスタンスを破棄し、セッションから抜けてください。
     * @see     Publish, CreateWellKnown
     */
    static DuplicatedObject* Create(qUnsignedInt32 uiTimeout == DEFAULT_CREATE_TIMEOUT)
    {
        // This method is provided by specific classes.
        SYSTEMCHECK(false);
        return NULL;
    }

    /*!
     * @refmanoverload{name2}
     *
     * @brief   複製オブジェクトのローカルインスタンスを作成します。
     *
     * この静的関数は、DDL(データ定義言語)コンパイラによって作成される
     * すべての複製オブジェクトクラスごとに定義される関数です。
     * たとえば、Avatarクラスを作成する場合、Avatar::Create( idUserDefinedID )と呼び出します。
     * DuplicatedObject::Createという関数はサポートされません。
     *
     * このオーバーロードでは明示的に、作成される複製オブジェクトのDOIDを指定します。
     * この値は、@ref IDGenerator::GenerateID 関数を呼び出すことによって生成されます。
     * 複製オブジェクトクラスごとのIDジェネレータの複製オブジェクトハンドルは、
     * DDLコンパイラによって自動的に生成されるソースコードの中に実装される、
     * GetIDGeneratorという関数にて取得することが出来ます。
     *
     * @param[in]   idUserDefinedID     作成する複製オブジェクトの@ref DOID を指定します。
     * @return  作成された複製オブジェクトローカルインスタンスへのポインタが返ります。
     * @see     Publish, CreateWellKnown
     */
    static DuplicatedObject* Create(DOID idUserDefinedID)
    {
        // This method is provided by specific classes.
        SYSTEMCHECK(false);
        return NULL;
    }

#endif
    /*!
     * @refmanoverload{name3}
     *
     * @brief   複製オブジェクトのローカルインスタンスを作成します。
     *
     * 指定された複製オブジェクトクラスID(@ref DOClassID)型の
     * 複製オブジェクトのローカルインスタンスを作成します。
     * 複製オブジェクトクラスIDはDDL(データ定義言語)ファイルによって自動的に生成されます。
     * したがって、DDLファイルを書き換えてプログラムをリビルドすると値が変化する可能性があることに注意してください。
     *
     * @param[in]   idDOClass 作成する型の複製オブジェクトクラスID
     * @param[in]   uiTimeout 作成されるのを待つためのタイムアウト時間を指定します。デフォルトでは無期限です。
     * @return  作成された複製オブジェクトローカルインスタンスへのポインタが返ります。
     *          もしもuiTimeoutで指定された数値が0の場合はSYSTEMERROR_GEN_INVALID_WAITがシグナルされます。
     * @see     Publish, CreateWellKnown
     */
    static DuplicatedObject* Create(DOClassID idDOClass, qUnsignedInt32 uiTimeout = WAIT_INFINITE_TIMEOUT);

    /*!
     * @refmanoverload{name4}
     *
     * @brief   複製オブジェクトのローカルインスタンスを作成します。
     *
     * 指定された複製オブジェクトクラスID(@ref DOClassID)型の
     * 複製オブジェクトのローカルインスタンスを作成します。
     * 複製オブジェクトクラスIDはDDL(データ定義言語)ファイルによって自動的に生成されます。
     * したがって、DDLファイルを書き換えてプログラムをリビルドすると値が変化する可能性があることに注意してください。
     *
     * このオーバーロードでは明示的に、作成される複製オブジェクトのDOIDを指定します。
     * この値は、@ref IDGenerator::GenerateID 関数を呼び出すことによって生成されます。
     * 複製オブジェクトクラスごとのIDジェネレータの複製オブジェクトハンドルは、
     * DDLコンパイラによって自動的に生成されるソースコードの中に実装される、
     * GetIDGeneratorという関数にて取得することが出来ます。
     *
     * @param[in]   idDOClass       作成する型の複製オブジェクトクラスIDを指定します。
     * @param[in]   idUserDefinedID 作成される複製オブジェクトインスタンスIDを指定します。
     * @return  作成された複製オブジェクトローカルインスタンスへのポインタが返ります。
     * @see     Publish, CreateWellKnown
     */
    static DuplicatedObject* Create(DOClassID idDOClass, DOID idUserDefinedID );

    /*!
     * @refmanoverload{name 5}
     * @brief   複製オブジェクトのローカルインスタンスを作成します。
     *
     * このオーバーロードでは明示的に作成される複製オブジェクトのハンドルを指定します。
     * 複製オブジェクトハンドル(@ref DOHandle)は、複製オブジェクトクラスID(@ref DOClassID)と
     * 複製オブジェクトID(@ref DOID)から作成されます。
     * それぞれを明示的に指定した場合のオーバーロードと同様の挙動になります。
     *
     * @param[in]   dohHandle   作成される複製オブジェクトのハンドルを指定します。
     * @return  作成された複製オブジェクトローカルインスタンスへのポインタが返ります。
     * @see     Publish, CreateWellKnown
     */
    static DuplicatedObject* Create(const DOHandle &dohHandle );

    // {secret}
    static DuplicatedObject* CreateDuplica(DOHandle hDO, const MasterStationRef& refMaster);


    /*
    Summary:  Creates a well-known doclass.
    Parameters:
    hWK: The name of the global variable that references the well-known instance.  This variable has to be
    the same as that specified in the DDL file.
    Returns: SYSTEMERROR_OBJDUP_WKHANDLE_EXISTS is returned if the well-known instance has already been created, and
    SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA if this method is called on a station that is not the Session master.
    Remarks:  The DDL generated doclasses inherit from the DuplicatedObject class.  A specific constructor for each
    doclass is generated by the DDL compiler into each DOCLASS.  It is not possible to create generic
    DuplicatedObject instances, you need to specify the doclass of the instance that you want to create.
    For example, given that World is declared as a well-known doclass and the global variable
    name g_hTheWorld is supplied in the DDL file the well-known object can be created as follows:

            World* pNewWorld=World::CreateWellKnown(g_hTheWorld);
            pNewWorld->Publish();

    This method must be called on the Session master.

    Note: As $PRODUCTNAME$ guarantees that all well-known objects are discovered before the game Session is created,
    this method must be called before a Session is created to ensure that all well-known objects are
    created at the appropriate time.

    See Also: Session::RegisterWellKnownDOsFactory, Session::CreateSession, Session::CreateOrJoinSession, Session::JoinSession
    */

    /*!
     * @brief   指定された既知オブジェクトを作成します。
     *
     * 既知オブジェクトとは、セッションが始まるときに参加者に必ず伝わっている複製オブジェクトのことを言います。
     * セッション中では全ステーションから参照することが出来、事実上のグローバルオブジェクトとして扱うことが出来ます。
     * たとえば、ゲームが行われている世界の情報(レベル、天候、敵AIの賢さ)など、
     * ゲームを開始するためには必ず必要となる情報を既知オブジェクトとして扱います。
     * どの情報を既知オブジェクトとするかはゲームの仕様により異なります。
     *
     * この関数を利用すると、既知オブジェクトを作成することが出来ます。
     * すべての既知オブジェクトはセッションが構築される前に作成されている必要があります。
     * この関数はセッションマスターが呼び出す必要があります。
     *
     * @param[in]   hWK 既知オブジェクトインスタンスへの参照を示すグローバル変数を指定します。
     *                      DDLファイルに宣言した名称と同一名でなければなりません。
     * @return  作成された既知オブジェクトへのポインタが返ります。
     */
    static DuplicatedObject* CreateWellKnown(WKHandle& hWK);

    /*
    Summary:  Publishes the doclass instance to all stations connected to the session
    that require the object.
    Parameters:
    uiTimeout: The maximum time in milliseconds that the system will wait. The default uiTimeout
    is infinite. This should be set to zero when $PRODUCTNAME$ is run in single-thread mode.
    Returns: true if the object was successfully published, false if not or an error occurred.
    The error SYSTEMERROR_GEN_INVALID_WAIT will be signalled if this method is called in a single thread context
    and the system cannot return immediately, and SYSTEMERROR_GEN_INVALID_OPERATION if this method is
    called after DeleteMainRef has been called on the object.

    $ifdef VOICE$
    When this method is called on a VoiceChannel object, the method will return false if either
    the CodecID or the packet size has not been set before Publish is called.
    $endif$
    Remarks: For example, given that Avatar is declared as a doclass in the DDL file, an
    instance of the Avatar class can be created and then published as follows. Refer to Create
    for other object creation options.

            Avatar* pNewAvatar=Avatar::Create();
            pNewAvatar->Publish();

    When this method is called in a single thread context, uiTimeout should be set to zero. Note
    that if you want the datasets of the published object to be up-to-date when Publish is called,
    you should initialize the object's datasets between the calls to Create and Publish rather
    than initializing them after calling Publish. If Publish returns false, then you must still
    delete the object (via DeleteMainRef) you were attempting to publish.
    See Also: Create, IsPublished, Threading Model
    */

    /*!
     * @brief   この複製オブジェクトのインスタンスをセッションにいるステーションに発行します。
     *
     * 複製オブジェクトは、作成された後に発行(パブリッシュ)されることで初めて
     * セッション内の他のステーションが感知することが可能になります。
     *
     * この関数が呼び出されたときに、発行される複製オブジェクトのデータセットが
     * 最新であってほしい場合には、@ref Create 関数が呼び出されたあと、@ref Publish 関数が
     * 呼び出される前に初期値をセットします。
     *
     * @param[in]   uiTimeout 発行完了まで待つタイムアウトをミリ秒単位で指定します。シングルスレッドモードの場合0になっている必要があります。デフォルトは1000msecです。
     * @retval  true    正常に作成された場合に返ります。
     * @retval  false   作成に失敗した場合に返ります。
     *                  シングルスレッドで呼び出され@a uiTimeoutの値が不正な場合はSYSTEMERROR_GEN_INVALID_WAITがシグナルされます。
     *                  @ref DeleteMainRef が呼び出された複製オブジェクトに対してこの関数が呼び出された場合、SYSTEMERROR_GEN_INVALID_OPERATIONがシグナルされます。
     *                  この値が返された場合には@ref DeleteMainRef 関数を呼び出して複製オブジェクトの主参照を削除する必要があります。
     */
    qBool Publish( qUnsignedInt32 uiTimeout = DEFAULT_PUBLISH_TIMEOUT );

    /*
    Description: Returns whether or not the given DuplicatedObject has been published.
    Returns: true if the object has already been published, false if the object is currently unpublished.
    Remarks: After a DuplicatedObject is created, it must then be published via Publish in order for it to
    be duplicated on all necessary remote stations in the session.
    See Also: Publish
    */

    /*!
     * @brief   この関数が発行されたかどうかを取得します。
     *
     * 複製オブジェクトは、作成された後に発行(パブリッシュ)されることで初めて
     * セッション内の他のステーションが感知することが可能になります。
     *
     * @retval  true  この複製オブジェクトがすでに発行済みである場合に返ります。
     * @retval  false この複製オブジェクトは正しく発行できていない場合に返ります。
     */
    qBool IsPublished() const
    {
        return FlagIsSet(Published);
    }


    /*
    Description: Returns a selection that contains the stations where the duplicas of this
    duplicated object are located.
    Returns: A selection of stations that can be used by a Station::SelectionIterator to iterate
    over all the stations where duplicas of this duplicated object are located.
    SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA will be returned if this method is called on a Duplica.
    Remarks: This method can only be called on a Duplication master. If called on a Duplica
    SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA will be returned.

    This method would typically be called as in the following example.

    Example:

        Station::SelectionIterator myIt( myDO->GetDuplicasStationLocations() );

        while(!myIt.EndReached()){
            ...
            }

    */
    Selection *GetDuplicasStationLocations()
    {
        return &m_setDuplicationSet;
    }

    //{secret}
    void FillDuplicaStationsList(qList<DOHandle> *plstDuplicaStations);

    //group=DO Templates

    /*
    Summary: Constructs a reference for safe access to the duplicated object.

    Remarks:  Defined by the duplicated object. Safely access the duplicated object
    by using this class to construct a reference to it from a DOHandle.

    A Ref is a typedef to an appropriate version of the RefTemplate
    template.  All duplicated object subclasses redefine this typedef
    and convert the DOHandle to a reference to the specific subclass of
    duplicated object.

    For example, create a Ref to an Avatar with a DOHandle hAvatar as follows:

            Avatar::Ref refAvatar(hAvatar);
            refAvatar->Render();

    Note: All references to a duplicated object must be released before $PRODUCTNAME$ can call the
    object's destructor. If not all references are released then $PRODUCTNAME$ will wait a certain
    amount of time and issue a warning before the object is deleted.
    See also: RefTemplate
    */

    /*!
     * @brief   複製オブジェクトへの安全な参照を行うためのtypdef
     */
    typedef RefTemplate<DuplicatedObject> Ref;

    /*
    Description: Use your duplicated object class name and the SelectionIteratorTemplate
    class to iterate over duplicated objects.
    Remarks: It is possible to create a SelectionIterator that holds all of the duplicated
    objects of a certain class on a host as follows:

            Avatar::SelectionIterator itALLAvatar;

    This is implemented via a template. You can find more details about how to use it
    under SelectionIteratorTemplate.
    See Also: Iterators Overview, SelectionIteratorTemplate
    */

    /*!
     * @brief   複製オブジェクトを列挙するためのイテレータのtypedef
     */
    typedef SelectionIteratorTemplate<DuplicatedObject> SelectionIterator;

    //group=DO Methods

#ifdef FOR_DOC_PURPOSE
    /*
    Description: Casts (performs explicit type conversion) during run time.
    Parameters:
    pDO: A pointer to a DuplicatedObject class.
    Returns: A valid pointer to the object if the object is of the expected type, a null
    pointer if it is not.
    Remarks: The DynamicCast method functions in the same manner as the standard C++ dynamic_cast
    operator but using $PRODUCTNAME$ run-time information.

    For example, say we have an Avatar class that inherits from a Duplicated3DObject class, which in
    turn inherits from the root DuplicatedObject class. We can return a pointer to an Avatar
    object, if /pObj/ points to an Avatar object, as follows:

        Avatar *pAvatar = Avatar::DynamicCast(pObj)

    Note: This method is defined by each specific DuplicatedObject class.
    */

    /*!
     * @brief   複製オブジェクトへのポインタの動的キャストを行います。
     *
     * 指定されたDuplicatedObject*型のポインタを、ランタイム情報を用いて動的にキャストします。
     * この静的関数はDDL(データ定義言語)コンパイラによって自動的に作成されます。
     * たとえば、Avatarという名前のクラスをDDLファイルに宣言した場合、
     * Avatar::DynamicCast関数を呼び出すことで、機能を利用することが出来ます。
     * DuplicatedObject::DynamicCast関数の直接呼出しはサポートされません。
     *
     * @param[in]   pDO 複製オブジェクトへのポインタを指定します。
     * @return  @a pDOで指定された複製オブジェクトへのポインタが適切な型にキャストされて返ります。
     * @retval  NULL    @a pDOで指定された複製オブジェクトへのポインタが、想定される型でなかった場合に返ります。
     *                  たとえば、Avatar::DynamicCastを呼び出したときに、@a pDOがAvatar型でなければNULLが返ります。
     */
    static DuplicatedObject* DynamicCast(DuplicatedObject* pDO)
    {
        // This method is provided by specific classes.
        SYSTEMCHECK(false);
        return NULL;
    }
    static const DuplicatedObject* DynamicCast(const DuplicatedObject* pDO)
    {
        // This method is provided by specific classes.
        SYSTEMCHECK(false);
        return NULL;
    }
#endif

    // Use this macro to secretize the flags.
#ifdef FOR_DOC_PURPOSE

#else
    typedef enum _Flag
    {
        MainReferenceInUse=1<<0, // Corresponds to bMainReferenceInUse, in doprotocol.spin
        ValidConstruction=1<<2,
        ValidDestruction=1<<3,
        OrphanDuplica=1<<4,
#ifdef NEX_DEBUG_VERSION
        Published=1<<5,
        InTheStore=1<<15
#else
        Published=1<<5
#endif
    } Flag;

#endif
    // Relevance count manipulation.
    //{secret}
    qUnsignedInt GetRelevanceCount() const
    {
        return m_uiRelevanceCount;
    }
    //{secret}
    void IncreaseRelevanceCount()
    {
        ++m_uiRelevanceCount;
    }
    //{secret}
    void DecreaseRelevanceCount()
    {
        SYSTEMCHECK(m_uiRelevanceCount>0);
        --m_uiRelevanceCount;
    }

    // Ref count manipulation.

    // {secret}
    qUnsignedInt GetRefCount() const
    {
        return m_uiRefCount;
    }
    // {secret}
    void IncreaseRefCount()
    {
        ++m_uiRefCount;
    }

    // Increase both counts...
    //{secret}
    void IncreaseRefAndRelevanceCounts()
    {
        // 二つの変数がアトミックに値変化する必要は無い。
        ++m_uiRefCount;
        ++m_uiRelevanceCount;
    }


    // {secret}
    qBool DecreaseCountsImplCore(qBool bDecreaseRelevanceCount)
    {
        qBool bQuickDecreaseDone=true; // Assume it is done...
        if (bDecreaseRelevanceCount)
        {
            // This one is decreased here, but does not go up
            // on failure, as does the uiRefCount.  That's ok -
            // the second call to decrease the refcount does
            // not care about relevance.
            SYSTEMCHECK(m_uiRelevanceCount>0);
            --m_uiRelevanceCount;
        }
        SYSTEMCHECK(m_uiRefCount>0);
        if (--m_uiRefCount==0)
        {
            // The DO must be deleted.  At this point, we can't
            // do it - the store isn't locked.  Call the
            // complete DecreaseRefCount.
            ++m_uiRefCount;
            bQuickDecreaseDone=false;
        }
        return bQuickDecreaseDone;
    }
    // {secret}
    void DecreaseCountsImpl(qBool bDecreaseRelevanceCount)
    {
        // Quick check - select the fast or the slow decrease ref
        // count...  Most of the time, the fast decrease ref count
        // will do the job - and the store will not have to be
        // locked.  If not - then call the real one - where the DO
        // can be removed from the store.

        // This piece of code is rather complex.  But for
        // performance reasons, it is better to do it the way it
        // is.
        qBool bQuickDecreaseDone = DecreaseCountsImplCore(bDecreaseRelevanceCount);
        if (!bQuickDecreaseDone)
        {
            CompleteDecreaseRefCount();
        }
    }

    // {secret}
    void DecreaseRefAndRelevanceCounts()
    {
        DecreaseCountsImpl(true);
    }
    // {secret}
    void DecreaseRefCount()
    {
        DecreaseCountsImpl(false);
    }

    // {secret}
    void CompleteDecreaseRefCount();

    // {secret}
    void ProtectedDecreaseRefCount()
    {
        // Called with store locked.
        SYSTEMCHECK(Scheduler::GetInstance()->SystemLock().HasOwnership());
        qBool bDestroyDO=false;
        {
            SYSTEMCHECK(m_uiRefCount>0);
            if (--m_uiRefCount==0)
            {
                bDestroyDO=true;
            }
        }
        if (bDestroyDO)
        {
            DestroyDO();
        }
    }

    // {secret}
    void DestroyDO();

    // Flags manipulation.

    // {secret}
    void SetFlag(qUnsignedInt16 ui16Flag);
    // {secret}
    void ClearFlag(qUnsignedInt16 ui16Flag);

    // {secret}
    void ReleaseReferenceToMaster();
    // {secret}
    void AcquireReferenceToMaster();
    // {secret}
    qBool OwnsReferenceToMaster() const;

    //{secret}
    virtual void UpdateCellStats(size_t nbPublishers,
                                 size_t nbSubscribers,
                                 size_t nbPublishersAndSubscribers)
    {
        DUMMY_PARAM(nbPublishers);
        DUMMY_PARAM(nbSubscribers);
        DUMMY_PARAM(nbPublishersAndSubscribers);
    };

    void PrepareToDestroy()
    {
        m_uiRefCount = 1;
        m_uiRelevanceCount = 1;
    }

protected:

    void SetStationSpecialRelevance();

private:
    // {secret}
    static DuplicatedObject *CreateMasterImpl(DOHandle hMaster, DOClassID idDOClass, DOID idUserSuppliedDOID);


    /******
     *
     * Access to DuplicationSets
     *
     ******/
public:

    // {secret}
    qBool IsDuplicatedOn(DOHandle hLocation);

    // {secret}
    qBool IsInCachedDuplicationSet(DOHandle hStation) const;
    // {secret}
    qBool AddToCachedDuplicationSet(const Station* pStation);
    // {secret}
    qBool RemoveFromCachedDuplicationSet(DOHandle hStation);

    // {secret}
    qBool IsInDuplicationSet(DOHandle hStation) const;

private:
    // Always called via the ChangeDupSetOperation operation.
    // {secret}
    qBool AddToDuplicationSet(Station* pStation);
    qBool RemoveFromDuplicationSet(DOHandle hStation);

protected:
    // When a duplica is fetched, even if the duplica is currently
    // in the DupSet, we re-add it to the DupSet.  We hide the
    // removal to the user (no corresponding operation), since
    // logically, there has been no removal.
    // {secret}
    qBool ForceRedundantAddDuplica(DOHandle hStation)
    {
        return RemoveFromDuplicationSet(hStation);
    }

public:
    // This method is called by a leaving station.
    // {secret}
    static void RemoveAllDuplicasOnLeavingStation(DOHandle hStation);


    /******
     *
     * Implementation of DOProtocol basic operations
     *
     * See doprotocol.spin for use in protocol.
     *
     ******/

private:

    // Implementation of DOProtocol basic operations.
    // See doprotocol.spin for use in protocol.
    //{secret}
    void AcquireMainReference()
    {
        SYSTEMCHECK(!FlagIsSet(MainReferenceInUse));
        IncreaseRefCount();
        SetFlag(MainReferenceInUse);
    };

public:
    // TODO - Make private!
    //{secret}
    void ReleaseMainReference()
    {
        SYSTEMCHECK(FlagIsSet(MainReferenceInUse));
        ClearFlag(MainReferenceInUse);
        // The object could be destroyed after this.
        DecreaseRefCount();
    };




protected:

    /******
     *
     * Prevent use of copy constructor and assignment operation on
     * DuplicatedObjects.
     *
     ******/

#ifdef FOR_DOC_PURPOSE

#else
    DuplicatedObject(const DuplicatedObject&); // Not implemented.
    DuplicatedObject& operator=(const DuplicatedObject&); // Not implemented.
#endif

protected:

    /*******
     *
     * States
     *
     *******/

    // For NEX Developer <wiki/Duplicated Object State Machine#DOStates>.

    void SetInitialState(const StateMachine::QEvent& e);

    QSTATE InvalidState(const StateMachine::QEvent& e);

    QSTATE ValidState(const StateMachine::QEvent& e);
    QSTATE InitialState(const StateMachine::QEvent& e);
    QSTATE DuplicationMasterState(const StateMachine::QEvent& e);
    QSTATE UnpublishedMasterState(const StateMachine::QEvent& e);
    QSTATE UnidentifiedMasterState(const StateMachine::QEvent& e);
    QSTATE InStoreMasterState(const StateMachine::QEvent& e);
    QSTATE DeletedMasterState(const StateMachine::QEvent& e);

    QSTATE DuplicaState(const StateMachine::QEvent& e);
    QSTATE InStoreDuplicaState(const StateMachine::QEvent& e);
    QSTATE OrphanDuplicaState(const StateMachine::QEvent& e);
    QSTATE ConnectedDuplicaState(const StateMachine::QEvent& e);
    QSTATE DeletedDuplicaState(const StateMachine::QEvent& e);

private:
    /**********************************************
     **********************************************
     ***
     *** MEMBER VARIABLES
     ***
     **********************************************
     **********************************************/

    /******
     *
     * Member variables used in ref counting
     *
     ******/

    AtomicValue m_uiRefCount;
    AtomicValue m_uiRelevanceCount;


    /******
     *
     * Member variables that compose the DO state
     *
     * All write access to these values are protected via the
     * system lock.  The method CheckStateAccess is called in all
     * methods that write to it, to system check if it isn't the case.
     *
     ******/

    MasterStationRef m_refMasterStation;

    qUnsignedInt16 m_uiFlags;

    Selection m_setDuplicationSet;

    DOHandle m_dohMyself;

    // Cached info, managed by the duplica.
    //{secret}
    Selection m_setCachedDuplicationSet;

public:
    /*!
     * @brief   DuplicatedObject::Create で指定されるデフォルトのタイムアウト値です。単位はミリ秒です。
     *
     * @see DuplicatedObject::Create
     */
    static const qUnsignedInt32 DEFAULT_CREATE_TIMEOUT = 1000;
    /*!
     * @brief   DuplicatedObject::Publish で指定されるデフォルトのタイムアウト値です。単位はミリ秒です。
     *
     * @see DuplicatedObject::Publish
     */
    static const qUnsignedInt32 DEFAULT_PUBLISH_TIMEOUT = 1000;
};

END_NAMESPACE_NEX


#include <NetZ/src/Core/DOCore/DOFilters.h>

