﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOFilter.h>
#include <NetZ/src/Core/DOCore/DORef.h>
#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <NetZ/src/Core/DOCore/Selection.h>

BEGIN_NAMESPACE_NEX

class IsAKindOfDOFilter;
//Group=Template Classes

/*
Description: Instances of this class are used to iterate methods over subsets of the DuplicatedObject s on the station.
Remarks: Use instances of this class to iterate methods over subsets of the DuplicatedObject s on the station.
*/

/*!
 * @brief   ステーションにある複製オブジェクトのサブセットに対するイテレータクラスです。
 */
class IteratorOverDOs: public RootObject
{
    //Group=Methods
public:
    // {secret}
    IteratorOverDOs(const Selection* pSelection=NULL, DOFilter* pFilter=NULL);

    // {secret}
    IteratorOverDOs(qBool bDuplica, qBool bMaster, qBool bGotoStart=true);

private:
    // {Secret}
    IteratorOverDOs(const IteratorOverDOs& oIt);
public:
    virtual ~IteratorOverDOs();

private:
    IteratorOverDOs& operator=(const IteratorOverDOs& oIt);
public:

    /*
    Summary: Forces the iterator's instance pointer to the first
    DuplicatedObject in the list.
    See Also: IteratorOverDOs::EndReached, GotoEnd
    */

    /*!
     * @brief   イテレータを強制的に複製オブジェクトリストの先頭に戻します。
     * @see     EndReached, GotoEnd
     */
    void GotoStart();

    /*
    Summary: Forces the iterator's instance pointer to point to the last
    DuplicatedObject of the list.
    See Also: IteratorOverDOs::EndReached, GotoStart
    */

    /*!
     * @brief   イテレータを強制的に複製オブジェクトリストの末尾に戻します。
     * @see     EndReached, GotoStart
     */
    void GotoEnd();

    /*
    Summary: Returns the number of DuplicatedObject s in the iterator.
    Returns: The number of DuplicatedObject s.
    Remarks: This method would typically be used as follows:

        // Create a Selection Iterator of Avatar duplicas
        Avatar::SelectionIterator itDupAvatar(DUPLICA);
        // Return the number of Avatar duplicas
        return (itDupAvatar.Count());

    NOTE: After this method has been called, the iterator points to the end
    of the sequence (EndReached() returns true).

    See Also:  IteratorOverDOs::EndReached, IteratorOverDOs
    */

    /*!
     * @brief   このイテレータが繰り返していく対象の複製オブジェクトの個数を取得します。
     *
     * この関数が呼び出された場合、イテレータはシーケンスの末尾を指すようになります。
     * このとき、@ref EndReached() が true を返します。
     *
     * @return  このイテレータが繰り返していく対象の複製オブジェクトの個数が返ります。
     */
    size_t Count();

    //#ifdef FOR_DOC_PURPOSE
    //      /*
    //      Remarks:  Advances the IteratorOverDOs's instance pointer to the next
    //      DuplicatedObject in the iterator. Both postfix and prefix versions are available.
    //
    //       However, since the postfix and prefix versions behave exactly the same, and to avoid
    //       errors of assumption when using the postfix operator in assignment statements, the
    //       postfix version forbids its use in assignments by returning Void and generating
    //       a compiler error.
    //
    //      Returns: An updated IteratorOverDOs reference to the next duplicated object in
    //      the set. Nothing for the postfix version when used in an assignment statement.
    //
    //      See Also:  IteratorOverDOs::EndReached, Count, IteratorOverDOs
    //      */
    //      IteratorOverDOs& operator++();
    //#else

    /*!
     * @brief   このイテレータを進め、次の複製オブジェクトを指すようにします。
     *
     * 前置演算子はイテレータが返ります。
     *
     * @return  更新され、次の複製オブジェクトを指すIteratorOverDOsが返ります。
     */
    IteratorOverDOs& operator++();
    /*!
     * @brief   このイテレータを進め、次の複製オブジェクトを指すようにします。
     *
     * 後置演算子はvoidを返すようになっているので、
     * それを代入しようとするとコンパイルエラーが発生します。
     */
    void operator++(qInt iDummy);
    //#endif

    /*
    Description: Returns a DOHandle to the DuplicatedObject presently referenced by the iterator.
    Returns: A DOHandle to the DuplicatedObject presently referenced by the iterator or
    INVALID_DOHANDLE if the call signals an error.
    See Also: DuplicatedObject, DOHandle
    */

    /*!
     * @brief   現在このイテレータが指し示している複製オブジェクトハンドルを取得します。
     * @return  現在このイテレータが指し示している複製オブジェクトハンドルが返ります。
     * @retval  INVALID_DOHANDLE エラーが発生したときに返ります。
     */
    DOHandle GetHandle() const;

    /*
    Description: Returns a pointer to the DuplicatedObject presently referenced by the iterator.
    Returns: A pointer to the DuplicatedObject presently referenced by the iterator.
    Remarks: In order to call methods of the specific doclass ( /e.g./ Render, DoPhysics) you must
    either cast this pointer into its appropriate doclass or use the SelectionIteratorTemplate.
    See Also: DuplicatedObject, SelectionIteratorTemplate
    */

    /*!
     * @brief   現在このイテレータが指し示している複製オブジェクトへのポインタを取得します。
     *
     * 物理演算を行ったり、描画処理を行うためには、返ってきた複製オブジェクトをキャストするか、
     * あるいは、@ref SelectionIteratorTemplate を使わなければなりません。
     *
     * @return  現在このイテレータが指し示している複製オブジェクトへのポインタが返ります。
     */
    DuplicatedObject* GetDOPtr() const;

    /*
    Remarks: In order to call methods of the specific doclass ( /e.g./ Render, DoPhysics) you must either cast
    this pointer into its appropriate doclass or use the SelectionIteratorTemplate.
    See Also: DuplicatedObject, SelectionIteratorTemplate
    */

    /*!
     * @brief   現在このイテレータが指し示している複製オブジェクトへのポインタを取得します。
     *
     * 物理演算を行ったり、描画処理を行うためには、返ってきた複製オブジェクトをキャストするか、
     * あるいは、@ref SelectionIteratorTemplate を使わなければなりません。
     *
     * @return  現在このイテレータが指し示している複製オブジェクトへのポインタが返ります。
     */
    DuplicatedObject* operator->() const;

    /*
    Description: Returns whether or not the end of the iterator list has been reached
    Returns: true if the end of the iterator has been reached, false if not or an error occurred.
    See Also: DuplicatedObject, GotoStart, GotoEnd
    */

    /*!
     * @brief   イテレータがリストの末尾に到達したかどうかを取得します。
     * @retval  true    イテレータがリストの末尾に到達したときに返ります。
     * @retval  false   イテレータがリストの末尾に到達していないときに帰ります。
     * @see     DuplicatedObject, GotoStart, GotoEnd
     */
    qBool EndReached() const;

    // {secret}
    qBool CurrentItemIsValid() const;

    //{secret}
    void AdvanceToValidItem(qBool bCurrentItemIsFine);

    //{secret}
    void SetCreationContext(void* pCreationContext);
    //{secret}
    void* GetCreationContext();

    /*
    Description: Finds a given object in the iterator.
    Parameters:
    hDO: A DOHandle to the DuplicatedObject in the iterator that you wish to find.
    Returns: true if the DuplicatedObject was found, false if not or an error occurred.
    Remarks: This method moves the iterator to the given object or to the end if the object is not found.
    See Also: DuplicatedObject
    */

    /*!
     * @brief   与えられた複製オブジェクトハンドルが参照する複製オブジェクトを、このイテレータの中で検索します。
     *
     * この関数は、与えられた複製オブジェクトハンドルが参照する複製オブジェクトまで
     * イテレータを動かすか、あるいはオブジェクトが見つからなかった場合に最後まで動かされます。
     * 最後までイテレータが動くと、@ref EndReached() が true を返すようになります。
     *
     * @param[in]   hDO 検索対象の複製オブジェクトハンドル。
     * @retval  true    複製オブジェクトが見つかったときに返ります。このとき、このイテレータは指定された複製オブジェクトを指します。
     * @retval  false   複製オブジェクトが見つからなかったときに返ります。このとき、@ref EndReached() はfalseを返します。
     */
    qBool Find(DOHandle hDO);

    // {secret}
    void DisableInStoreFilter()
    {
        m_bMustBeInStore = false;
    }

    // {secret}
    void SetFilter(DOFilter* pNewFilter);
protected:
    // {secret}
    void SetCacheFilter(IsAKindOfDOFilter* pNewFilter);
public:

    // Iteration over Selections
    //{secret}
    void SetSelection(Selection* pSelection);

    //{secret}
    DOHandle GetNext(DOHandle hDO) const;

#if NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION
    //{secret}
    void GetNext( DOHandle hDO, Selection::const_iterator *pIterator ) const;
#endif

    //{secret}
    void MoveNext();

    //{secret}
    void MoveStart();

    //{secret}
    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);

    /*
    Description: Instructs $PRODUCTNAME$ to create this iterator using soft references to the duplicated objects
    that it contains.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. A given reference may be changed from hard to soft and vice versa during run time.
    A Hard reference ensures that an object stays relevant and therefore, kept up-to-date on the local
    Station as long as DuplicatedObject::DeleteMainRef has not been called on the object. This sort of
    reference is generally used when you want to fetch an object (via DuplicatedObject::FetchDuplica) that
    is not currently relevant on the local Station. In contrast, a Soft reference has no effect on an object's
    Relevance and therefore, a duplica will only be updated if the object is considered relevant due to a
    Duplication space match condition. The choice of using a hard or soft reference depends on whether or
    not you wish to keep the referenced object updated.

    For example, if you want to keep a list of your
    teammates and their current status (position, inventory, /etc./) even when they are in a different part
    of the virtual world to your avatar, you will want to keep Hard reference s to their avatars so that
    the avatar duplicas will always be updated. On the other hand, if you wish to keep a list of all the objects
    of a given type, say enemy avatars, that exist in the world, you will probably want to use Soft reference s
    so you don't update these objects unecessarily on your station. When a Soft reference is used, a given
    object will only be updated when it becomes relevant on the station due, to a Duplication space match condition.
    See Also: UseHardReference, Soft reference
    */
    void UseSoftReference()
    {
        m_refCurrentItem.SetSoft();
    }

    /*
    Description: Instructs $PRODUCTNAME$ to create this iterator using hard references to the duplicated objects
    that it contains.
    Remarks: A reference to a duplicated object may be either a Hard reference, which is the default, or a
    Soft reference. A given reference may be changed from hard to soft and vice versa during run time.
    A Hard reference ensures that an object stays relevant and therefore, kept up-to-date on the local
    Station as long as DuplicatedObject::DeleteMainRef has not been called on the object. This sort of
    reference is generally used when you want to fetch an object (via DuplicatedObject::FetchDuplica) that
    is not currently relevant on the local Station. In contrast, a Soft reference has no effect on an object's
    Relevance and therefore, a duplica will only be updated if the object is considered relevant due to a
    Duplication space match condition. The choice of using a hard or soft reference depends on whether or
    not you wish to keep the referenced object updated.

    For example, if you want to keep a list of your
    teammates and their current status (position, inventory, /etc./) even when they are in a different part
    of the virtual world to your avatar, you will want to keep Hard reference s to their avatars so that
    the avatar duplicas will always be updated. On the other hand, if you wish to keep a list of all the objects
    of a given type, say enemy avatars, that exist in the world, you will probably want to use Soft reference s
    so you don't update these objects unecessarily on your station. When a Soft reference is used, a given
    object will only be updated when it becomes relevant on the station due, to a Duplication space match condition.
    See Also: UseSoftReference, Hard reference
    */
    void UseHardReference()
    {
        m_refCurrentItem.SetHard();
    }

private:
    //{secret}
    void SetCurrentItem(DOHandle hNewCurrentItem);
    //{secret}
    // Used by MoveNext() and AdvanceToValidItem()
    inline void MoveNextNoLock();   // n2208 fixed confliction against following definition

    DOHandle GetMinimumValidHandle() const;
    DOHandle GetMaximumValidHandle() const;

    void ComputeHandleRangeRestrictions();

private:

    const Selection *m_pSelection;

    DORef m_refCurrentItem;
    DOFilter* m_pFilter;

    DOHandle m_hMinimumValidHandle;
    DOHandle m_hMaximumValidHandle;

    qBool m_bMustBeInStore;

    // This context is used to release the iterator.
    //{secret}
    void* m_pCreationContext;
};


inline DOHandle IteratorOverDOs::GetMinimumValidHandle() const
{
    return m_hMinimumValidHandle;
}

inline DOHandle IteratorOverDOs::GetMaximumValidHandle() const
{
    return m_hMaximumValidHandle;
}

inline DOHandle IteratorOverDOs::GetHandle() const
{
    return m_refCurrentItem.GetHandle();
}

// TODO.  This should eventually be replaced with a refNewCurrentItem -
// but for now, the cost of locking/unlocking is way too expensive - and
// it isn't worth it (run Iteration perf test with DOHandle, and with
// DORef to see the cost).
//{secret}
inline void IteratorOverDOs::SetCurrentItem(DOHandle hNewCurrentItem)
{
    m_refCurrentItem=hNewCurrentItem;
}


inline void IteratorOverDOs::GotoEnd()
{
    SetCurrentItem(INVALID_DOHANDLE);
}

inline qBool IteratorOverDOs::EndReached() const
{
    // n2370:この関数の内容を変更する場合は、AdvanceToValidItemの最適化されたコードも更新する必要があります。
    return m_refCurrentItem.GetHandle() == INVALID_DOHANDLE;
}


inline DOHandle IteratorOverDOs::GetNext(DOHandle hDO) const
{
    SYSTEMCHECK(DOHandle(hDO.GetValue()+1).GetValue()!=0); // A condition that was there earlier - but I'm sure it makes no sense.
    Selection::const_iterator it=m_pSelection->lower_bound(hDO.GetValue()+1);
    if (it!=m_pSelection->end())
    {
        return (*it).first;
    }
    else
    {
        return INVALID_DOHANDLE;
    }
}


inline void IteratorOverDOs::MoveNext()
{
    ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
    MoveNextNoLock();
}

#if NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION

/*!
  @brief 指定されたDOハンドルよりも値が大きいものの中でもっとも値が小さいものに対するイテレータを取得します。
 */
inline void IteratorOverDOs::GetNext( DOHandle hDO, Selection::const_iterator *pIterator ) const
{
    SYSTEMCHECK(DOHandle(hDO.GetValue()+1).GetValue()!=0);
    SYSTEMCHECK(pIterator != NULL);
    *pIterator = m_pSelection->lower_bound(hDO.GetValue()+1);
}

/*!
  @brief 最適化された、次への移動関数です。
  @remarks DORefに対してDOHandleをセットする処理(SetCurrentItemを呼び出すと同様のことを中で行います)は、
           中でさらにクリティカルセクションによるロックと、Selectionからの検索処理が走ります。
           検索処理とロック処理が二重に行われることを、この最適化では避けるようにしています。
 */
inline void IteratorOverDOs::MoveNextNoLock()
{
    Selection::const_iterator it;
    GetNext( GetHandle(), &it );

    if ( it != m_pSelection->end() )
    {
        m_refCurrentItem = (*it).second;
    }
    else
    {
        m_refCurrentItem = INVALID_DOHANDLE;
    }
}

/*!
  @brief 最適化された、最初への移動関数です。
 */
inline void IteratorOverDOs::MoveStart()
{
    ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
    Selection::const_iterator it;
    GetNext( GetMinimumValidHandle(), &it );

    if ( it != m_pSelection->end() )
    {
        m_refCurrentItem = (*it).second;
    }
    else
    {
        m_refCurrentItem = INVALID_DOHANDLE;
    }
}


#else   // NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION

inline void IteratorOverDOs::MoveNextNoLock()
{
    SetCurrentItem(GetNext(GetHandle()));
}

inline void IteratorOverDOs::MoveStart()
{
    ScopedCS oCS(Scheduler::GetInstance()->SystemLock());
    SetCurrentItem(GetNext(GetMinimumValidHandle()));
}

#endif  // NEX_ENABLE_ITERATOR_DO_GETNEXT_OPTIMIZATION

inline void IteratorOverDOs::GotoStart()
{
    if (m_pSelection==NULL)
    {
        GotoEnd();
        return;
    }
    MoveStart();
    AdvanceToValidItem(true);
}

inline IteratorOverDOs& IteratorOverDOs::operator++()
{
    AdvanceToValidItem(false);
    return *this;
}

// Postfix version.  Since it is possible to derive from the
// IteratorOverDOs, we can't reliably implement a postfix version
// which returns the initial value.

inline void IteratorOverDOs::operator++(qInt /*iDummy*/ )
{
    AdvanceToValidItem(false);
}

inline DuplicatedObject* IteratorOverDOs::GetDOPtr() const
{
    // TODO: Is this necesary? Shouldn't the responsability lie on the part of the caller?
    if (EndReached())
    {
        SIGNAL_SYSTEMERROR(SYSTEMERROR_GEN_END_OF_COLLECTION,0);
        return NULL;
    }
    else
    {
#if defined(NEX_DEBUG_VERSION)
        if (m_refCurrentItem.GetHandle()!=INVALID_DOHANDLE)
        {
            if (!m_refCurrentItem.IsValid())
            {
                QLOG(EventLog::Fatal,NEX_T("About to systemcheck, iterator to ") << (m_refCurrentItem.GetHandle().GetClassNameString()) << NEX_T(" ") << hex << (
                         m_refCurrentItem.GetHandleValue()) << NEX_T(" is invalid"));
            }
            SYSTEMCHECK(m_refCurrentItem.IsValid());
            SYSTEMCHECK(m_refCurrentItem.GetDOPtr()!=NULL);
        }
#endif
        return m_refCurrentItem.GetDOPtr();
    }
}

inline DuplicatedObject* IteratorOverDOs::operator->() const
{
    DuplicatedObject* pDO=GetDOPtr();
    SYSTEMCHECK(pDO!=NULL);
    return pDO;
}

inline void IteratorOverDOs::SetCreationContext(void* pCreationContext)
{
    m_pCreationContext=pCreationContext;
}

inline void* IteratorOverDOs::GetCreationContext()
{
    return m_pCreationContext;
}

inline void IteratorOverDOs::Trace(qUnsignedInt64 uiTraceFlag)
{
    DUMMY_PARAM(uiTraceFlag);
    QTRACE(uiTraceFlag,NEX_T("IteratorOverDOs ") << hex << (this));
}

inline void IteratorOverDOs::SetSelection(Selection* pSelection)
{
    // This method can only be called once.
    SYSTEMCHECK(m_pSelection==NULL);
    m_pSelection=pSelection;
    if( pSelection==NULL )
    {
        GotoEnd();
    }
}

END_NAMESPACE_NEX

