﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/SessionOperation.h>
#include <NetZ/src/Core/DOCore/Station.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

//n1589:memo
/*
Description: This SessionOperation is performed whenever a Station leaves the Session.
Remarks: A LeaveSessionOperation will be called whenever a Station leaves the Session, either voluntarily
or due to a fault. This operation will only be called on the Duplication master of the leaving station's
Station object. Therefore, if a Station voluntarily quits a Session, the operation will be called on the leaving
station's Station object after the player's intention to quit has been indicated. On the other hand, if a fault
occurs the operation will be called on the station to which the faulting Station object migrates, as determined by
the PromotionReferee. When a fault occurs, the Station object of the faulting station will migrate to another station
so that $PRODUCTNAME$ can perform an internal clean-up before the Station object is actually deleted. Specifically,
the Station Duplication master migrates, the Station's fault-tolerant duplicated objects are migrated and the
non-fault-tolerant ones deleted, then the Station object is deleted.

When a Station leaves the Session the following sequence of events occurs:
    o   The leaving Station is removed from the Duplication master store via a RemoveFromStoreOperation,
    o   The method SessionOperation::Begin is called,
    o   An internal clean-up is performed to ensure that the leaving Station is no longer considered part of the Session,
    o   The method SessionOperation::End is called.

Note that the LeaveSessionOperation is only called on one Station. Therefore, if you wish to call a method on all
stations when a given Station leaves the Session due to a fault, you should trap the ChangeMasterStationOperation
on the leaving station's Station object, as this object will migrate before being removed from the store.
*/

/*!
 * @brief   このSessionOperationはステーションがセッションから離脱するときに実行されます。
 *
 *          LeaveSessionOperationはステーション(@ref Station)がセッション(@ref Session)から離脱するときに
 *          発生します(離脱意志を持って離脱した場合も、障害発生も同様)。
 *          この操作は、離脱しようとしているステーションの@ref Station オブジェクトの複製マスター
 *          (@ref DuplicatedObject::IsADuplicationMaster())のみが呼び出すことが出来ます。
 *          したがって、もしも@ref Station が自発的に@ref Session を抜けたばあい、この操作は
 *          離れようとしている@ref Station が、プレイヤーの切断意思が通知されたタイミングで実行されます。
 *
 *          一方で、もしも障害が発生したときは、操作は障害が発生した@ref Station オブジェクトが、他のステーションに
 *          マイグレート(複製オブジェクトのコントロール権転移)が発生した先で実行されます。
 *          これは、システムが内部的に@ref Station オブジェクトを削除される前にクリーンナップをするためです。
 *
 *          @ref Station が@ref Session から離れるとき、以下の流れに従ってイベントが発生します。
 *
 *          ・離れている@ref Station が複製マスターストアから@ref RemoveFromStoreOperation によって取り除かれます<br>
 *          ・SessionOperation::Begin()が呼び出されます。<br>
 *          ・離れた@ref Station が確かに@ref Session に存在しないことを確認するために内部的なクリーンナップ処理が行われます。<br>
 *          ・SessionOperation::End()が呼び出されます。<br>
 *
 *          LeaveSessionOperationはひとつの@ref Station に対してのみ実行されます。
 *          したがって、もしもこの方法を、障害のためにセッションを離脱したすべてのステーションに対して行いたいときは、
 *          @ref ChangeMasterStationOperation によって離れている@ref Station オブジェクトを捕らえるべきです。
 */
//TODO :    SessionOperation::Begin, End関数は存在しないようですが…
//          DuplicatedObject::OperationBegin, OperationEnd？
class LeaveSessionOperation: public SessionOperation
{
    //Group=Methods
public:
    //{secret}
    LeaveSessionOperation(Station* pStation);
    //{secret}
    virtual ~LeaveSessionOperation();

    NEX_IMPL_OPERATION_COMMON_METHODS(LeaveSession);

    /*
    Description: Returns a pointer to the Station that is leaving the Session.
    Returns: A pointer to the Station that is leaving the Session.
    */

    /*!
     * @brief   セッションから離脱しているステーションへのポインタを取得します。
     * @return  セッションから離脱しているステーションへのポインタが返ります。
     */
    const Station* GetLeavingStation() const;

private:

#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event eEvent, qUnsignedInt64 uiTraceFlags) const;
#endif  // NEX_TRACE_ENABLED

private:
    Station::Ref m_refLeavingStation;
};

END_NAMESPACE_NEX

