﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/MemberContainer.h>
#include <OnlineCore/src/Platform/Core/qQueue.h>

BEGIN_NAMESPACE_NEX

//Group = Member Containers

/*
Description:  This class contains methods to control the elements of a queue declared in a
dataset and associated with a doclass in the DDL file.

Remarks:
A MemberQueue instance is associated with a doclass in the DDL file in the following way:

        dataset QueueDataset{
            queue<double> que;
        };

        doclass MyDOClass{
            QueueDataset m_dsQueueOfDoubles;
        };

The MyDOClass implementation inherits from the DDL generated DOCLASS (MyDOClass) that associates the m_dsQueueOfDoubles to your class.
Thus you can access this dataset member and perform operations on it just like any other dataset member of your doclass. For example:

        class MyDOClass: public DOCLASS ( MyDOClass) {
        public:
            void PushToBackOfQueue(qReal dNewValue) {
                m_dsQueueOfDoubles.lst.Push(dNewValue);
                Update();
            }
        // other user-implemented code
        }
*/
template<class TDDL_Type>
class MemberQueue : public MemberContainer<TDDL_Type>
{
    //Group=Methods
public:
    // {secret}
    typedef typename TDDL_Type::CPPType Type;
    //{secret}
    typedef typename NAMESPACE_NEX MemberContainer<TDDL_Type>::ContOp ContOp_t;

    //{secret}
    MemberQueue()
    {
    }

    /*
    Description: Pops an element from the front of the queue.
    */
    void Pop()
    {
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::POP_OPCODE));
        m_stdqQueue.pop();
    }

    /*
    Description: Pushes an element to the back of the queue.
    Parameters:
    tElem: The element to insert.
    */
    void Push(const Type& tElem)
    {
        MemberContainer<TDDL_Type>::AddOperation(ContOp_t(ContOp_t::PUSH_OPCODE, tElem));
        m_stdqQueue.push(tElem);
    }

    /*
    Description: Returns the element at the front of the queue.
    Returns: The element at the front of the queue.
    */
    const typename qQueue<Type>::value_type& Front() const
    {
        return m_stdqQueue.front();
    }

    /*
    Description: Returns the element at the back of the queue.
    Returns: The element at the back of the queue.
    */
    const typename qQueue<Type>::value_type& Back() const
    {
        return m_stdqQueue.back();
    }

    /*
    Description:  Returns whether or not the queue is empty.
    Returns:  true if the MemberQueue is empty, false if not or an error occurred.
    */
    qBool Empty() const
    {
        return m_stdqQueue.empty();
    }

    /*
    Description:  Returns the number of elements in the queue.
    Returns: The number of elements in the MemberQueue
    */
    typename qQueue<Type>::size_type Size() const
    {
        return m_stdqQueue.size();
    }

private:
    ////////////////////////////
    // Replay operations (BEGIN)
    ////////////////////////////
    //{secret}
    qBool ReplayInnerAssign(typename qQueue<Type>::size_type iPos, const Type& tElem)
    {
        if(iPos==0)
        {
            m_stdqQueue.front() = tElem;
        }
        else if (iPos == (m_stdqQueue.size() - 1))
        {
            m_stdqQueue.back() = tElem;
        }
        else
        {
            return false; //Error
        }
        return true;
    }
    ////////////////////////////
    // Replay operations (END)
    ////////////////////////////

protected:
    //{secret}
    virtual void AddContentToMsg(Message *pMsg)
    {
        qQueue<Type> queueCopy = m_stdqQueue;

        // 32bitと64bitでバイナリフォーマットをそろえるためsize_tをqUnsignedInt32にキャスト
        const qUnsignedInt32 uiSize = static_cast<qUnsignedInt32>(m_stdqQueue.size());
        *pMsg << uiSize;
        while(!queueCopy.empty())
        {
            TDDL_Type::Add(pMsg, queueCopy.front() );
            queueCopy.pop();
        }
    }

    //{secret}
    virtual void ExtractContentFromMsg(Message *pMsg)
    {
        Type tElem;
        qUnsignedInt uiSize=0;
        *pMsg >> uiSize;
        while(!m_stdqQueue.empty())
        {
            m_stdqQueue.pop();
        }
        while(uiSize>0)
        {
            TDDL_Type::Extract(pMsg, &tElem);
            m_stdqQueue.push(tElem);
            uiSize--;
        }
    }

public:
    //{secret}
    qBool ReplayOperations()
    {
        qBool bRet=true;
        ContOp_t *pcoOpTmp=NULL;
        while(!MemberContainer<TDDL_Type>::m_coOperationList.empty())
        {
            pcoOpTmp = MemberContainer<TDDL_Type>::m_coOperationList.back().GetPtr();
            switch(pcoOpTmp->GetOpCode())
            {
            case ContOp_t::POP_OPCODE:
                m_stdqQueue.pop();
                break;
            case ContOp_t::PUSH_OPCODE:
                m_stdqQueue.push(pcoOpTmp->GetData());
                break;
            case ContOp_t::ASSIGN_OPCODE:
                bRet = ReplayInnerAssign(pcoOpTmp->GetPos(), pcoOpTmp->GetData());
                break;
            default:
                SYSTEMCHECK(false && NEX_T("Invalid container operation code for MemberQueue."));
            }
            MemberContainer<TDDL_Type>::m_coOperationList.pop_back();
        }
        return bRet;
    }

    //{secret}
    qQueue<Type>* GetStdqQueue()
    {
        return &m_stdqQueue;
    }

    const MemberQueue& operator=(const MemberQueue& mqSource)
    {
        MemberQueue<TDDL_Type>& mqQ = const_cast<MemberQueue&>(mqSource);

        // Clear the content (there is no Clear method/operation for MemberQueue)
        while(!Empty())
        {
            Pop();
        }

        const typename qQueue<Type>::iterator itEnd = mqQ.m_stdqQueue.end();
        for(typename qQueue<Type>::iterator it = mqQ.m_stdqQueue.begin(); it != itEnd; ++it)
        {
            Push(*it);
        }

        return *this;
    }

    virtual qBool operator==(const MemberQueue& uqt) const
    {
        return (this->m_stdqQueue == uqt.m_stdqQueue);
    }

    virtual qBool operator!=(const MemberQueue& uqt) const
    {
        return (this->m_stdqQueue != uqt.m_stdqQueue);
    }


#if defined(NEX_TRACE_ENABLED)
    // Warning: TraceContent assumes the template type is qReal
    //{secret}
    qBool TraceContent(qUnsignedInt64 uiTraceFlag) const
    {
        QTRACE(uiTraceFlag, NEX_T("\n\nTracing queue content: "));
        qQueue<Type> queueCopy = m_stdqQueue;
        while(!queueCopy.empty())
        {
            QTRACE(uiTraceFlag, queueCopy.front());
            queueCopy.pop();
        }
        return true;
    }
#endif

private:
    size_t SizeOfContent() const
    {
        return sizeof(Type)*m_stdqQueue.size();
    }

private:
    //{secret}
    qQueue<Type> m_stdqQueue;
};

END_NAMESPACE_NEX

