﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DORef.h>
#include <NetZ/src/Core/DOCore/ActiveDOCallContext.h>

BEGIN_NAMESPACE_NEX

// Group=Results Classes

#ifdef FOR_DOC_PURPOSE
/*
Summary: This class defines the context of a Migration call.
Remarks: Before the migration of an object is performed the context of the call must be
defined via this class.  Within this class you set the appropriate flags and can also
obtain information about the CallContext::State and DOCallContext::Outcome of the call.
Migration is performed via the calls DuplicatedObject::Emigrate or DuplicatedObject::AttemptEmigration.
*/

/*!
 * @brief   このクラスはマイグレーション呼び出しのための非同期管理機能を提供します。
 *
 * マイグレーションとは、複製オブジェクトのコントロール権限がステーション間で移転することを言います。
 *
 * 複製オブジェクトのマイグレーションを実行する前に、必ずこのクラスによって呼び出しを定義する必要があります。
 * このクラスを超えない範囲でフラグを設定したり、@ref CallContext::State や
 * @ref DOCallContext::Outcome 関数呼び出しによる情報の取得を行うことが出来ます。
 * マイグレーションは、@ref DuplicatedObject::Emigrate(MigrationContext*, DOHandle) または、
 * @ref DuplicatedObject::AttemptEmigration(DOHandle) を呼び出すことで実行することが出来ます。
 *
 * マイグレーションには大きく分けて二つのプロセスがあり、
 * 最初に実行される複製オブジェクトの権限が移行するプロセスのことをエミグレーション(emigration)、
 * 次に実行される複製オブジェクトを受け入れるプロセスをイミグレーション(immigration)と呼びます。
 */
class MigrationContext: public DOCallContext
{
#else
class MigrationContext: public ActiveDOCallContext
{
#endif

    //Group=Methods
public:

    /*
    Remarks: By default, object migration will be made synchronously, /i.e./ the system will
    wait for the migration call to return a value or an error before performing the next task.
    The method DOCallContext::Wait can be used if the migration call is to be made asynchronously.
    Migration can be performed via either of the calls DuplicatedObject::Emigrate
    or DuplicatedObject::AttemptEmigration.

    Parameters:
    bSynchronousCall: true if the call is made synchronously, which is the default, false if asynchronously.
    */

    /*!
     * @brief   オブジェクトを初期化します。
     *
     * デフォルトでは、マイグレーション(複製オブジェクトのコントロール間移転)は、同期実行されます。
     * 同期実行される場合、システムはマイグレーションが終了するか、あるいはエラーが発生するまで返ってこなくなります。
     * @ref DOCallContext::Wait 関数を利用することによって、非同期実行された呼び出しを終了するまで待機することが出来ます。
     *
     * オブジェクトを直接指定してマイグレーションを実行する場合、@ref DuplicatedObject::Emigrate(MigrationContext*, DOHandle) 関数を利用します。
     *
     * @param   bSynchronousCall [in] 同期実行するかどうかを指定します。デフォルトはtrue(同期実行する)となります。
     */
    MigrationContext(qBool bSynchronousCall=true);
    virtual ~MigrationContext();

    //Description: Returns the DOHandle of a migrating object.
    //Returns: The DOHandle of the object that is in the process of migrating or will migrate.
    /*!
     * @brief   マイグレーションを行っている複製オブジェクトへのハンドルを取得します。
     *
     * 複製オブジェクトハンドル(@ref DOHandle)とは、セッション中で複製オブジェクトを一意に表す値です。
     * この関数によって、マイグレーション(複製オブジェクトのコントロール間移転)を行っている
     * 複製オブジェクトへのハンドルを取得することが出来ます。
     *
     * @return  マイグレーションを行っている複製オブジェクトへのハンドルが返ります。マイグレーション終了・失敗後には、INVALID_DOHANDLEが返ります。
     */
    DOHandle GetMigratingObject()
    {
        return m_refTargetObject.GetHandle();
    }

    // {secret}
    DuplicatedObject* GetMigratingObjectPtr() const
    {
        return m_refTargetObject.GetDOPtr();
    }

#ifdef NEX_TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif  // NEX_TRACE_ENABLED

    // {secret}
    virtual Type GetType() const
    {
        return MigrationCall;
    }

    // {secret}
    qBool MigrateObject(DOHandle hMigratingObject, DOHandle hNewStation);
    // {secret}
    qBool MigrateObjectImpl(DOHandle hMigratingObject, DOHandle hNewStation);

    // {secret}
    virtual qResult SendCallMessage(Message* pCallMessage);

    // {secret}
    virtual void ProcessOutcome(DOHandle hSourceStation, Outcome uOutcome);

    // {secret}
    qByte GetNewMasterVersion()
    {
        return m_lcNewMasterVersion.GetValue();
    }

    // {secret}
    virtual void ProcessCallCompletion();

private:
    LogicalClock m_lcNewMasterVersion;

};


END_NAMESPACE_NEX

