﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/MethodID.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <NetZ/src/Core/DOCore/ActiveDOCallContext.h>
#include <OnlineCore/src/Platform/Core/EventHandler.h>

BEGIN_NAMESPACE_NEX

class Event;

// Group=Results Classes

#ifdef FOR_DOC_PURPOSE
/*
Summary: This class defines the context of a RMC.
Remarks: Before a RMC is invoked the context of the call must be defined via the
RMCContext class. Within this class you set the station you wish the method to be
called on and the appropriate flags. You can also obtain information about the
CallContext::State and DOCallContext::Outcome of the call. Refer to the Remote Method Calls Overview
for an example of the complete implementation of RMCs.

Refer to CallMethodOperation and the Remote Method Calls Overview for examples of a RMCContext in use.
See Also: Remote Method Calls Overview, DDL File Syntax
*/

/*!
 * @brief   このクラスはRMCのためのコンテキストを定義します。
 *
 * RMC(リモートメソッドコール)を呼び出す前に、この呼び出しのコンテキストが
 * RMCContextクラスによって定義されなければなりません。このクラスに対して、
 * 呼び出し先のステーションとフラグを設定することが出来ます。
 * また、この呼び出しに対する@ref CallContext::State と、DOCallContext::Outcome の情報を
 * 含めることも出来ます。
 *
 * RMCの実装方法について詳しくはNEXプログラミングマニュアルも参照してください。
 */
class RMCContext: public DOCallContext
{
#else
class RMCContext: public ActiveDOCallContext
{
#endif
    //Group=Methods
public:
    /*
    Remarks: If no arguments are passed to the constructor, the RMCContext instance will
    initialize the value of the hTargetStation to INVALID_DOHANDLE. By default the RMC
    will be made synchronously, /i.e./ the thread will wait for the RMC to return a value
    or an error before performing the next task. The method DOCallContext::Wait can be used
    if the RMC is made asynchronously.

    Parameters:
    hTargetStation: The station we wish to call the RMC on.
    bSynchronousCall: true if the call is made synchronously, which is the default, false if asynchronously.
    */

    /*!
     * @brief   コンテキストを初期化します。
     *
     * @param[in]   hTargetStation          呼び出し対象のステーション(@ref Station)の複製オブジェクトハンドル(@ref DOHandle)を指定します。
     * @param[in]   bSynchronousCall        呼び出しが同期実行されるか非同期実行されるかを指定します。
     *                                      同期実行されるとRMCが値を返すかエラーになるまで呼び出した関数から返ってこなくなります(デフォルト)。
     */
    RMCContext(DOHandle hTargetStation=INVALID_DOHANDLE,
               qBool bSynchronousCall=true);

    virtual ~RMCContext();

#ifdef NEX_MOVE_SEMANTICS_SUPPORT

    /*!
    @brief    ムーブコンストラクタです。

    @param[in]    rhs    ProtocolCallContext オブジェクト。
    */
    RMCContext(RMCContext&& rhs) NEX_NOEXCEPT
        : ActiveDOCallContext()
        , m_idTargetMethod(0)
        , m_vReturnValuePtr()
        , m_pucDummyBuffer(NULL)
        , m_pResponseMessage(NULL)
    {
        *this = std::move(rhs);
    }

    /*!
    @brief    ムーブ代入演算子です。

    @param[in]    rhs    ProtocolCallContext オブジェクト。
    */
    RMCContext& operator=(RMCContext&& rhs) NEX_NOEXCEPT
    {
        if (this == &rhs)
        {
            return *this;
        }
        this->Swap(rhs);
        return *this;
    }
#endif

    // {secret}
    MethodID GetTargetMethodID()
    {
        return m_idTargetMethod;
    }
    // {secret}
    DOHandle GetTargetObject()
    {
        return m_refTargetObject.GetHandle();
    }

    // {secret}
    qBool PrepareCallMessage(DOHandle hTarget, MethodID id);

    // {secret}
    virtual void ProcessResponse(UserContext byResponseContext, State* pResultingState, Outcome* pResultingOutcome);

    // Return value ptr manipulation.
    // {secret}
    void AddReturnValuePtr(void* pReturnValue);
    // {secret}
    void* GetReturnValuePtr(qUnsignedInt iIndex);
    // {secret}
    void ClearReturnValuePtr();
    // {secret}
    void SetOutcome(Outcome eOutcome)
    {
        CallContext::SetOutcome(qResult(eOutcome));
    }

    /*
    Description: Sets the outcome of this RMCContext.
    Parameters:
    rOutcome: The outcome to which this RMCContext will be set.
    Remarks: The outcome can be set to either an error or a success outcome.
    */

    /*!
     * @brief   このRMCContextに対する結果を設定します。
     *
     * 結果は、エラーの値か成功の値のどちらかをセットできます。
     *
     * @param[in]   rOutcome このコンテキストに設定する結果値を指定します。
     */
    void SetOutcome(qResult rOutcome)
    {
        CallContext::SetOutcome(rOutcome);
    }

#ifdef NEX_TRACE_ENABLED    // n2370
    // {secret}
    virtual void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS);
#endif  // NEX_TRACE_ENABLED

    // {secret}
    virtual Type GetType() const
    {
        return RMC;
    }

    // {secret}
    Message* GetResponseMessage();

    void Swap(RMCContext& rhs) NEX_NOEXCEPT
    {
        DOCallContext::Swap(rhs);
        std::swap(m_idTargetMethod, rhs.m_idTargetMethod);
        std::swap(m_vReturnValuePtr, rhs.m_vReturnValuePtr);
        std::swap(m_pucDummyBuffer, rhs.m_pucDummyBuffer);
        std::swap(m_pResponseMessage, rhs.m_pResponseMessage);
    }

    /*!
    @brief    この @ref RMCContext の完了時に実行するコールバックを設定します。

    設定したコールバックはこの @ref RMCContext が完了状態に遷移したときに呼び出されます。
    このコールバックを使用することで、 @ref RMCContext が完了したかどうかを確認するためにポーリングする手間を省くことができます。

    callback には @ref RMCContext* を引数に取る関数ポインタ、関数オブジェクト、ラムダ式を指定できます。
    AddCallContextArgumentAdapter() を併用すると引数を一つも取らない関数を渡すこともできます。

    関数オブジェクトを渡す場合は値もしくは参照型で渡す方法とポインタ型で渡す方法の二種類があり、それぞれ挙動が異なります。
    値もしくは参照型で渡した場合は内部で関数オブジェクトがコピーもしくはムーブされ、コールバックはそのコピーもしくはムーブされたオブジェクトに対して行われます。
    そのため、本関数呼び出し後は指定したオブジェクトを即座に破棄する事ができます。
    ポインタ型で渡した場合は、コールバックはそのポインタが指すオブジェクトに対して行われます。
    そのため、コールバックが呼び出されるまでその関数オブジェクトを呼び出し側で保持しておく必要があります。

    本関数を複数回呼び出すことで複数のコールバックを登録することが出来ます。
    複数のコールバックを登録した場合はデフォルトでは登録順にコールバックの呼び出しが行われ、最初に登録されたコールバックは最初に呼び出されます。
    この順序を変更したい場合は、コールバックを登録する際に bAddToEnd パラメータを設定する必要があります。

    callback に関数オブジェクトをポインタで渡した場合は deleteCallbackAfterCall を true にする事で
    callback の呼び出し後に自動的に callback を delete することができます。
    もし callback を delete するのではなくアプリケーション独自の解放処理を行いたい場合は、
    再度本関数を呼び出しそのための関数オブジェクトを追加する事で実装が可能です。

    コールバック関数内では NEX のブロック関数を呼び出さないようにしてください。

    @param[in]  callback コールバックする関数ポインタもしくは関数オブジェクトもしくはラムダ式。
    @param[in]  deleteCallbackAfterCall callback の実行が完了した後自動的に callback を delete するかどうか。
    @param[in]  bAddToEnd true（デフォルト）であれば、コールバックはリストの最後に追加されます。falseであればリストの先頭に追加されます。
    */
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(TCallback&& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<RMCContext>(std::forward<TCallback>(callback), deleteCallbackAfterCall, bAddToEnd);
    }
#else
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(const TCallback& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<RMCContext>(callback, deleteCallbackAfterCall, bAddToEnd);
    }
#endif

private:
    // Info filled by the caller - on call initialization.
    // {secret}
    MethodID m_idTargetMethod;

    // Result
    // {secret}
    qVector<void*> m_vReturnValuePtr;

    // Temporary ptr.
    // {secret}
    qByte* m_pucDummyBuffer;

    Message* m_pResponseMessage;
};


END_NAMESPACE_NEX

