﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/SystemError.h>
#include <NetZ/src/Core/DOCore/DORef.h>
#include <NetZ/src/Core/DOCore/DOClass.h>

BEGIN_NAMESPACE_NEX


class DuplicatedObject;

DOClass* GetDOClass(DuplicatedObject* pDO);

// Group=Template Classes

/*
Description: Creates a safe way to access a duplicated object.
Remarks: In $PRODUCTNAME$, the safe way to access a duplicated object is via a reference.  The
template RefTemplate provides a safe reference to a duplicated object. You may create a reference
to a duplicated object in order to call any of your C++ methods.  If you have a reference to an object,
you can perform actions on it just like any other C++ object.

Every duplicated object class defines a typedef to the appropriate RefTemplate.

As long as a reference to a duplicated object exists, the duplicated object pointer
will not be invalidated by $PRODUCTNAME$.  Typically a reference would be created
on the stack and destroyed as soon as it was no longer required. It is important to
ensure that a reference to an object is only kept for a short period of time as even
though an object wil be removed from a given station's object store (refer to RemoveFromStoreOperation),
the object's destructor cannot be called while there is still a vaild reference to the object.

A reference may be created by specifying a DOHandle to an object or by specifying a reference to
an object. The default constructor may also be used and then an object assigned to the reference
via the assignment operator.

For example, if the DOClass Sphere is defined in the DDL and hSphere is a DOHandle to a Sphere,
then the following code provides access to the Sphere object referenced by hSphere.

        {
            Sphere::Ref refSphere(hSphere);
                if (refSphere.IsValid()){
                    // Call the Sphere method SelectSound;
                    refSphere->SelectSound();
                    // Call the action PlaySound on all duplicas of this Sphere
                    refSphere->PlaySound_OnDuplicas();
                }
        }

Note: All references to a duplicated object must be released before $PRODUCTNAME$ can call the
object's destructor.
See also: DuplicatedObject::Ref, DORef
*/

/*!
 * @brief   複製オブジェクトへの安全な参照(アクセサ)を作成します。
 *
 * これは薄いテンプレートであり、共通部分は、@ref DORef に実装されます。
 * テンプレート引数には複製オブジェクトクラス名を指定しますが、
 * これは、複製オブジェクトクラスにtypedefされるRefを利用することもできます。
 * たとえば、Avatarという複製オブジェクトクラスに対する、この参照クラスはAvatar::Refでtypedefされます。
 *
 * このシステムでは、複製オブジェクトへの安全な方法でのアクセスは参照によるものです。
 * このRefTemplateは複製オブジェクトへの安全な参照を提供します。
 * 複製オブジェクトの参照を作ることでC++の関数を呼び出すことが出来るようになります。
 * もしもオブジェクトへの参照があれば、それをC++のオブジェクトのように扱うことが出来るようになります。
 *
 * 複製オブジェクトへの参照がある限り、複製オブジェクトポインターは
 * システムによって無効化されることはありません。典型的には、
 * オブジェクトへの参照はスタック上に構築され、不要になったら削除されます。
 * 重要なのは、オブジェクトがステーションの複製オブジェクトストアから
 * 取り除かれようとしているとき(@ref RemoveFromStoreOperation)でさえも、
 * オブジェクトに対する参照は短い期間だけに抑えることです。
 * システムは複製オブジェクトのデストラクタは参照が残っている限り呼び出すことが出来ません。
 *
 * @ref DOHandle を明示することによってか、オブジェクトへの参照を明示することによって
 * 参照を作成することができます。代入演算子を利用することによって後で指定するために、
 * デフォルトのコンストラクタも利用することが出来ます。
 *
 * @tparam  DO 参照する複製オブジェクトのクラスを指定します。
 */
template<class DO> class RefTemplate: public DORef
{
    // Group=Methods
public:
    /*
    Description: Creates a reference to a duplicated object.
    Parameters:
    hDO: DOHandle that refers to the duplicated object to which a
    reference is wanted.
    oRef: A reference to a reference of a duplicated object.
    Remarks: The method DORef::IsValid should be used to check if the reference was correctly
    constructed.
    See Also: Duplicated Object Overview, DORef
    */

    /*!
     * @refmanoverload{name1}
     *
     * @brief   無効な参照として初期化します。
     */
    RefTemplate() {}
    /*!
     * @refmanoverload{name2}
     *
     * @brief   複製オブジェクトハンドルを利用して初期化します。
     *
     * 複製オブジェクトハンドル(@ref DOHandle)は、セッション中に存在する複製オブジェクトを
     * 一意に表すことが出来るハンドルです。このコンストラクタでは複製オブジェクトハンドルから
     * 複製オブジェクトへの安全な参照を作成します。
     *
     * 正しく複製オブジェクトのハンドルの参照が構築できた場合、
     * @ref DORef::IsValid() がtrueを返します。
     *
     * @param   hDO [in] 参照対象の複製オブジェクトのハンドルを指定します。
     */
    RefTemplate(DOHandle hDO): DORef(hDO) {}
    /*!
     * @refmanoverload{name3}
     *
     * @brief   コピーコンストラクタです。
     *
     * 指定した参照の指す複製オブジェクトを参照して初期化します。
     *
     * 正しく複製オブジェクトのハンドルの参照が構築できた場合、
     * @ref DORef::IsValid() がtrueを返します。
     *
     * @param   oRef [in] 参照のコピー元を指定します。
     */
    RefTemplate(const DORef& oRef): DORef(oRef) {}

    virtual ~RefTemplate() {};

    DO* operator->()
    {
        if (!IsValid())
        {
            return NULL;
        }
        else
        {
            return (DO*) GetDOPtr();
        }
    }

    const DO* operator->() const
    {
        if (!IsValid())
        {
            return NULL;
        }
        else
        {
            return (const DO*) GetDOPtr();
        }
    }

    /*
    Summary: Provides access to the duplicated object.
    Remarks: Returns a pointer to the duplicated object of a specific doclass.  This
    pointer can safely be used to call actions or methods.
    See Also: Duplicated Object Overview, DORef
    */

    /*!
     * @brief   複製オブジェクトへのアクセスを提供します。
     *
     * この関数によって返されるポインタによって関数などに安全にアクセスすることが出来ます。
     *
     * @return  複製オブジェクトへのポインタが返ります。
     */
    DO* operator*()
    {
        if (!IsValid())
        {
            return NULL;
        }
        else
        {
            return (DO*) GetDOPtr();
        }
    }

    const DO* operator*() const
    {
        if (!IsValid())
        {
            return NULL;
        }
        else
        {
            return (const DO*) GetDOPtr();
        }
    }

    RefTemplate<DO>& operator=(DO* pDO)
    {
        DORef::operator=(pDO);
        return *this;
    }

    inline qBool IsValid() const
    {
        if (GetDOPtr()==NULL)
        {
            SIGNAL_SYSTEMERROR(SYSTEMERROR_OBJDUP_DO_NOT_FOUND, 0);
            return false;
        }
        else
        {
            if (!GetDOClass(GetDOPtr())->IsAKindOf(DO::GetRealDOClassID()))
            {
                SIGNAL_SYSTEMERROR(SYSTEMERROR_OBJDUP_MISMATCHED_DOCLASS, 0);
                return false;
            }
            return true;
        }
    }

};


END_NAMESPACE_NEX

