﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>

BEGIN_NAMESPACE_NEX

class Operation;

// Group = System Operations

/*
Description: Approves or denies $PRODUCTNAME$ System Operations.
Remarks: This class is used to approve or deny $PRODUCTNAME$ System Operations before they are called
to prevent a given Operation being called incorrectly. An incorrect call may be due to a bug in
the game code or due to a hack performed by a player. During internal or beta testing of your game,
where cheating is unlikely, incorrectly called System Operations will most probably be due to bugs in
the game code.

The SafetyExecutive class can be used to prevent specific operations, such as object creation, deletion, and migration,
being perfomed on a given Station under specific conditions. For example, you could allow object migration
to occur under normal circumstances but then prevent migration under your deemed abnormal conditions.
Refer to RegisterCustomOperationCheck for details as to how to restrict when System Operations may be called.
*/

/*!
 * @brief   システム操作実行の可否を決定するクラスです。
 *
 * このクラスはシステム操作が実行される前に、その操作を行うのか、あるいは
 * 不正な操作命令ということで操作の実行を行わないのかを決定するために利用されます。
 * ゲームコードのバグやチートを行うプレイヤーによって不正な操作が実行されることがあります。
 * 開発途中のゲーム(ベータ版)ではチートは発生しにくいため、不正なシステム操作の
 * 大部分はゲームコードのバグということになります。
 *
 * オブジェクトの作成、オブジェクトの削除、
 * マイグレーション(複製オブジェクトコントロールのステーション間移動)などの、
 * それぞれの操作について、適切な状態で実行されようとしているかどうかをみるために
 * SafetyExecutiveクラスを利用することが出来ます。
 * アプリケーションの仕様として決まっていること(たとえば、オブジェクトのマイグレーションは
 * 試合開始から30秒しか実行させない、など)がある場合に、
 * その条件を満たさない(先ほどの例だと、試合開始1分後にマイグレーションが実行されようとした)なら
 * 操作を拒否することで、不正なマイグレーションを防ぐことが可能です。
 *
 * システム操作がどのように制限されていくのかの詳細を知るためには、
 * @ref RegisterCustomOperationCheck 関数の解説を参照してください。
 *
 * このクラスのオブジェクトははシステムによって自動的に作成されます。
 * アプリケーション側からオブジェクトの作成を行わないでください。
 * オブジェクトインスタンスを取得するためには@ref GetInstance 関数を利用してください。
 */
class SafetyExecutive: public RootObject
{
    //Group =Type Definitions
public:
    /*
    Description: Defines a CustomOperationCheck.
    Parameters:
    /oOperation/ : The Operation that this CustomOperationCheck will be called on.
    Remarks: A CustomOperationCheck is registered via RegisterCustomOperationCheck. It sets the conditions
    under which a given Operation will be accepted or denied.
    See Also: RegisterCustomOperationCheck
    */

    /*!
     * @brief   操作が実行されてもよいかどうかを決定する関数ポインタ型の定義です。
     *
     * これは、@ref RegisterCustomOperationCheck 関数に設定されます。
     */
    typedef qBool (*CustomOperationCheck)(const Operation& oOperation);

    /*
    Description: Defines an InvalidOperationCallback.
    Parameters:
    /oOperation/ : The Operation that this InvalidOperationCallback will be called on.
    Remarks: An InvalidOperationCallback is registered via RegisterInvalidOperationCallback. It defines what
    action will be taken when an Operation is denied.
    See Also: RegisterInvalidOperationCallback
    */

    /*!
     * @brief   操作の実行を拒否された場合にどのような処理を実行するのかを決定する関数ポインタ型の定義です。
     *
     * これは、@ref InvalidOperationCallback 関数に設定されます。
     */
    typedef void (*InvalidOperationCallback)(const Operation& oOperation);

    //Group =Methods

    // A SafetyExecutive object should never be directly instantiated. Instead, it is created by
    // $PRODUCTNAME$ on all Station s when a session is created.
    /*!
     * @brief   このコンストラクタはアプリケーションから呼び出さないでください。
     *
     * セッションが作成されたときにシステムがすべてのステーションについてこのオブジェクトを自動的に作成します。
     */
    SafetyExecutive();

    // A SafetyExecutive object should never be directly destroyed. It is destroyed by $PRODUCTNAME$
    // when the Session terminates.
    /*!
     * @brief   このデストラクタはアプリケーションから呼び出さないでください。
     *
     * セッションが終了したときにシステムが自動的に解放処理を行います。
     */
    virtual ~SafetyExecutive();

    /*
    Description: Returns a pointer to the SafetyExecutive instance.
    Returns: A reference to the SafetyExecutive object.
    Remarks: Use the reference returned by this method to control the approval or denial
    of $PRODUCTNAME$ System Operations. For example, register a custom operation check as follows:

            SafetyExecutive::GetInstance()->RegisterCustomOperationCheck(MyOperationCheck);
    */

    /*!
     * @brief   SafetyExecutiveのインスタンスへのポインタを取得します。
     *
     * 操作実行の可否を決定するため、SafetyExecutiveクラスへアクセスするときは、
     * この関数を利用してインスタンスへのポインタを取得し、
     * それを経由して関数の呼び出しなどを行ってください。
     * SafetyExecutiveクラスのインスタンスはセッション開始時に自動的に作成され、
     * セッションが終了したときに自動的に解放されるようになっています。
     *
     * @return  SafetyExecutiveオブジェクトへのポインタが返ります。
     */
    static SafetyExecutive* GetInstance();

    /*
    Description: Sets the conditions under which an Operation will be accepted or denied.
    Parameters:
    pfCustomOperationCheck: A user-defined function that defines the conditions
    under which an Operation is approved. This function must return true if the Operation
    is approved and false otherwise.
    Remarks: This method prevents a given Operation being called incorrectly, which may be due
    to a bug in the game code or due to a hack performed by a player. Currently only the
    AddToStoreOperation and ChangeDupSetOperation can be approved or denied via the SafetyExecutive,
    all other System Operations are automatically accepted. However, in future versions of $PRODUCTNAME$
    all System Operations will pass via the SafetyExecutive before being processed.

    For example, this method could be used to prevent Avatar objects being created on a client station
    as follows. The action taken, if any, when an Operation is denied, /i.e./ when pfCustomOperationCheck returns
    false, is defined by the /pfInvalidOperationCallback/, which is registered via RegisterInvalidOperationCallback.

            // Define the pfCustomOperationCheck
            qBool DenyClientAvatarCreation(const Operation& oOperation) {
                // I will not create duplicas of my Avatar on a client station.
                if (oOperation.GetType()==Operation::ChangeDupSet) {
                    const ChangeDupSetOperation* pOp=ChangeDupSetOperation::DynamicCast(&oOperation);
                    if (pOp->IsADuplicaAddition() && pOp->GetAssociatedDO()->
                                                        IsA(DOCLASSID(Avatar))) {
                        Station::Ref refStation(pOp->GetTargetStation());
                        if (refStation->GetProcessType()==Station::ClientProcess) {
                            return false;
                        }
                    }
                }
                return true;
            }

            // Then register the custom check
            SafetyExecutive::GetInstance()->RegisterCustomOperationCheck(DenyClientAvatarCreation);

    See Also: RegisterInvalidOperationCallback
    */

    /*!
     * @brief   システム操作(@ref Operation)の可否を決定するためのコールバック関数を登録します。
     *
     * この関数に指定されたコールバック関数によってシステム操作(@ref Operation)の
     * 操作が不正なものであるかそうでないかの判定処理を行います。
     * 登録されたコールバック関数によって、アプリケーション側コードのバグを回避したり、
     * プレイヤーのチート行為を抑えることが出来るようになります。
     *
     * この関数を登録しても、デフォルトではローカルステーション(自分自身)からの
     * システム操作は可否判定処理が行われません。ローカルステーションから実行されようとしている
     * システム操作でも@a pfCustomOperationCheckで指定されたコールバック関数を
     * 呼び出すようにするためには、@ref TrustLocalStation 関数を呼び出す必要があります。
     *
     * @param   pfCustomOperationCheck [in] 操作の可否を決定するためのコールバック関数を登録します。
     *                                      与えられた操作について、受け入れる場合はtrueを、そうでない場合はfalseを返す必要があります。
     * @see     GetInstance, RegisterInvalidOperationCallback
     */
    void RegisterCustomOperationCheck(CustomOperationCheck pfCustomOperationCheck);

    /*
    Description: Determines what action will be taken when an Operation
    is denied.
    Parameters:
    pfInvalidOperationCallback: A user-defined function that dictates what action will be taken when
    an Operation is denied.
    Remarks: This method is called by $PRODUCTNAME$ when an Operation is denied by the SafetyExecutive,
    /i.e./ the /pfCustomOperationCheck/ as given by RegisterCustomOperationCheck returns false. When an
    Operation is denied you define what action, if any, should be taken via an pfInvalidOperationCallback.
    For example, you could implement a reporting system as illustrated below. Here, if we are in debug or
    testing mode we assume the invalid operation call is due to a bug and output a trace when it occurs.
    On the other hand, if we are in release mode we assume an invalid operation is possibly due to a cheat
    and report the event to the server where we keep stats for each station. If a station is deemed to be
    cheating, we can then take appropriate action.

            // Define the pfInvalidOperationCallback
            void ReportCheats(const Operation& oOperation) {
                // For example, at this point we could report to the server that
                // the player on station 'oOperation.GetOrigin()' has performed an
                // invalid operation and is potentially cheating. This event would
                // be reported via a RMC on a well-known 'CheatManager' object.
                // The server could then keep stats on the reports and if a large
                // number of people report that the same person is cheating,
                // appropriate action be taken. Note that an invalid operation may
                // also be an indication of a bug in the game and not just a cheat.
            }

            void TraceInvalidOperations(const Operation& oOperation) {
                // During development testing, we know that as no one cheats if an
                // invalid operation is called then it is due to a bug in the code.
                QTRACE(TRACE_ALWAYS,NEX_T("** An invalid operation has been performed **"));
                SYSTEMCHECK(false); // Stop here.
            }

            // Then register the custom callback.
            // This callback will be called in release mode,
            SafetyExecutive::GetInstance()->RegisterInvalidOperationCallback(ReportCheats);
            // and this callback will be called when testing in internal or debug mode.
            SafetyExecutive::GetInstance()->RegisterInvalidOperationCallback(TraceInvalidOperations);

    See Also: RegisterCustomOperationCheck
    */

    /*!
     * @brief   システム操作が拒否されたときに何を実行するのかを決定するためのコールバック関数を登録します。
     *
     * この関数を利用して登録されたコールバック関数は、SafetyExecutiveクラスによって
     * システム操作(@ref Operation)が拒否されたときにシステム側から呼び出されます。
     * システム操作は、@ref RegisterCustomOperationCheck 関数で登録された
     * コールバック関数にてfalseを返すことで実行を拒否することが出来ます。
     *
     * システム操作が拒否されるということは、ゲームのバグであるか、
     * ユーザーのチート行為である可能性があります。この関数を使ってコールバック関数を
     * 登録しておき、その中でそれぞれの場合に対処することが可能になります。
     * たとえば、ゲーム開発中に利用するデバッグビルドでは、
     * システム操作が拒否されたことをログレポートに表示した後にクラッシュさせ、
     * プログラマが修正できるようにしておくことが出来ます。
     * また、製品出荷用のリリースビルドでは、
     * システム操作が拒否されたことはユーザーのチート行為である可能性があるとみなして
     * サーバーに通知するなどの操作を行うことも出来ます。
     *
     * @param   pfInvalidOperationCallback [in] システム操作が拒否されたときにシステム側から実行されるコールバック関数を指定します。
     *
     * @see     GetInstance, RegisterCustomOperationCheck
     */
    void RegisterInvalidOperationCallback(InvalidOperationCallback pfInvalidOperationCallback);

    // {secret}
    qBool CheckOperationValidity(const Operation& oOperation);

    /*
    Description: Sets whether or not the local station is trusted.
    Parameters:
    bFlag: If true (the default), the local station is trusted.
    Remarks: If the local station is trusted then the validity of any System Operations that the
    station triggers will not be checked by the SafetyExecutive before being called. If the local
    station is not trusted then System Operations will be validated by the SafetyExecutive before being
    called.
    See Also: LocalStationIsTrusted
    */

    /*!
     * @brief   ローカルステーションを信用するかどうかを設定します。
     *
     * ローカルステーション(自分自身)が信用されていると設定された場合(デフォルトは信用する設定)、
     * ローカルステーションから実行されようとしているすべてのシステム操作は
     * SafetyExecutiveクラスによって事前チェックされなくなります。
     * 信用されていない設定の場合、ローカルステーションから実行されようとしているシステム操作も
     * SafetyExecutiveクラスによる事前チェックの対象となります。
     *
     * @param   bFlag [in] ローカルステーションを信用するかどうかを指定します(デフォルトではtrueが設定されている状態です)。
     *
     * @see     GetInstance, LocalStationIsTrusted
     */
    void TrustLocalStation(qBool bFlag)
    {
        m_bTrustLocalStation=bFlag;
    }

    /*
    Description: Returns whether or not the local station is trusted.
    Returns: true if the local Station is trusted, which is the default behaviour, false if not or an error occurred.
    Remarks: If the local station is trusted then the validility of any System Operations that the
    station triggers will not be checked by the SafetyExecutive before being called. If the local
    station is not trusted then System Operations will be validated by the SafetyExecutive before being
    called.
    See Also: TrustLocalStation
    */

    /*!
     * @brief   ローカルステーションを信用するかどうかを取得します。
     *
     * ローカルステーション(自分自身)が信用されていると設定された場合(デフォルトは信用する設定)、
     * ローカルステーションから実行されようとしているすべてのシステム操作は
     * SafetyExecutiveクラスによって事前チェックされなくなります。
     * 信用されていない設定の場合、ローカルステーションから実行されようとしているシステム操作も
     * SafetyExecutiveクラスによる事前チェックの対象となります。
     *
     * この関数を利用すると、@ref TrustLocalStation 関数にて設定した
     * 値を取得することが出来ます。
     *
     * @return  ローカルステーションを信用するかどうかを返します。
     */
    qBool LocalStationIsTrusted() const
    {
        return m_bTrustLocalStation;
    }

private:
    qBool ValidOperation(const Operation& oOperation);

private:
    CustomOperationCheck m_pfCustomOperationCheck;
    InvalidOperationCallback m_pfInvalidOperationCallback;
    qBool m_bTrustLocalStation;
};


END_NAMESPACE_NEX

