﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/String.h>




BEGIN_NAMESPACE_NEX

enum
{
    STRING_MAX_LENGTH=256
};

class ByteStream;
class SessionDiscoveryTable;

// Group=Session Management

/*
Summary: Contains a description of the session.
Remarks: The SessionDescription includes a variety of information that identifies the Session
such as the application title, session name, session ID, library version and build numbers, and
user-defined version numbers and attributes. The SessionDescription can be used as a parameter
of the method Session::JoinSession.

Refer to Session::JoinSession and Session::QuerySessions for examples of how to use the methods of this class.
*/

/*!
 * @brief   セッションに関する情報を含むクラスです。
 *
 * セッション記述子は、セッションを判別するためのさまざまな情報を含みます。
 * たとえば、アプリケーションのタイトル、セッションの名前、セッションID、
 * ライブラリのバージョン番号やビルド番号、そしてユーザー定義番号や属性などがあります。
 * このセッション記述子は@ref Session::JoinSession の引数としても利用することが出来ます。
 */
class SessionDescription : public RootObject
{

    // Group=Methods

public:

    SessionDescription();
    virtual ~SessionDescription();

    qBool operator==(const SessionDescription& oSession);

    //{secret}
    void SetApplicationTitle(const qChar *szApplicationTitle);

    /*
    Description: Sets the name of the Session.
    Parameters:
    szSessionName: The name that will be set for the Session associated to this SessionDescription.
    Remarks: Only the Session master may set the name of the Session. The session name may be set either via
    SetSessionName or declared upon creation within the Session::CreateSession call. You can retrieve the
    session name via GetSessionName.
    See Also: GetSessionName, Session::CreateSession
    */

    /*!
     * @brief   セッションの名称を設定します。
     *
     * セッションの名称は、@ref Session::CreateSession を呼び出してセッションを作成するときに設定されます。
     * @ref GetSessionName を利用することで、設定したセッション名を取得することが出来ます。
     *
     * @param[in]   szSessionName  設定するセッションの名称を指定します。
     */
    void SetSessionName(const qChar *szSessionName);

    //{secret}
    void SetStationURL(const qChar *szStationURL);

    // {secret}
    void SetSessionDiscoveryType(const qChar *szSessionDiscoveryType);

    // {secret}
    void SetSessionDiscoveryContext(const qChar *szSessionDiscoveryContext);

    //{secret}
    void SetLibraryVersionNumber(qUnsignedInt32 uiVersion, qUnsignedInt32 uiBuild)
    {
        m_uiLibraryVersionNumber = uiVersion;
        m_uiLibraryBuildNumber = uiBuild;
    }

    /*
    Summary: Returns the title of the application as declared in the DDL file.
    Returns: The title of the application as declared in the DDL file.
    Remarks: Only the Session master can return this information.
    */

    /*!
     * @brief   アプリケーションの名前を取得します。
     *
     * アプリケーションのタイトルは、DDL(データ定義言語)ファイルによって宣言されます。
     * セッションマスターのみがこの情報を返すことが出来ます。
     *
     * @return  DDLによって設定されたアプリケーション名が返ります。
     */
    const qChar *GetApplicationTitle() const
    {
        return m_szApplicationTitle.CStr();
    }

    /*
    Summary:  Returns the name of the Session.
    Returns: The name of the session.
    Remarks: Only the Session master can return this information. The session name may be set either via
    SetSessionName or declared upon creation within the Session::CreateSession call.
    See Also: SetSessionName
    */

    /*!
     * @brief   セッションの名前を取得します。
     *
     * @ref SetSessionName か@ref Session::CreateSession 関数を呼び出すことでセッション名がセットされます。
     * セッションマスターのみがこの情報を返すことが出来ます。
     *
     * @return  セッションの名前が返ります。
     */
    const qChar *GetSessionName() const
    {
        return m_szSessionName.CStr();
    }

    //Summary:  Returns the URL of the session.
    //Returns: The URL of the session.
    //See Also: StationURL
    /*!
     * @brief   セッションのURLオブジェクトを取得します。
     * @return  セッションに参加するための接続先URLオブジェクトが返ります。
     */
    const qChar *GetStationURL() const
    {
        return m_szStationURL.CStr();
    }

    //Summary:  Returns the library version number of the session.
    //Returns: The library version number of the session.
    /*!
     * @brief   このセッションに利用されているライブラリのバージョン番号を取得します。
     * @return  このセッションに利用されているライブラリのバージョン番号が返ります。
     */
    qUnsignedInt32 GetLibraryVersionNumber() const
    {
        return m_uiLibraryVersionNumber;
    }

    //Summary:  Returns the library build number of the session.
    //Returns: The library build number of the session.
    /*!
     * @brief   このセッションに利用されているライブラリのビルド番号を取得します。
     * @return  このセッションに利用されているライブラリのビルド番号が返ります。
     */
    qUnsignedInt32 GetLibraryBuildNumber() const
    {
        return m_uiLibraryBuildNumber;
    }

    /*
    Description: Returns the version number of the application.
    Returns: The version number of the application as an unsigned integer.
    Remarks: The version number is specific to the given application. It can be set via
    SetApplicationVersionNumber.
    See also: SetApplicationVersionNumber, GetApplicationVersionNumbers, SetApplicationVersionNumbers
    */

    /*!
     * @brief   アプリケーションのバージョン番号を取得します。
     *
     * アプリケーションのバージョン番号はアプリケーションによって与えられます。
     * この値は、@ref SetApplicationVersionNumber 関数を呼び出すことで設定することが出来ます。
     *
     * @return  アプリケーションのバージョン番号が符号なし値として返ります。
     *
     * @see     SetApplicationVersionNumber
     */
    qUnsignedInt32 GetApplicationVersionNumber() const
    {
        return m_uiApplicationVersionNumber;
    }

    /*
    Description: Returns the version number of the application.
    Returns: The version number of the application as four byte values.
    Remarks: The version number is specific to the given application. It can be set via
    SetApplicationVersionNumbers.
    See Also:  SetApplicationVersionNumbers, GetApplicationVersionNumber, SetApplicationVersionNumber
    */

    /*!
     * @brief   アプリケーションのバージョン番号を取得します。
     *
     * アプリケーションのバージョン番号はアプリケーションによって与えられます。
     * この値は、@ref SetApplicationVersionNumbers 関数を呼び出すことで設定することが出来ます。
     *
     * @param[out]  pbyV1  バージョン番号1をバイト値として取得するためのポインタを指定します。
     * @param[out]  pbyV2  バージョン番号2をバイト値として取得するためのポインタを指定します。
     * @param[out]  pbyV3  バージョン番号3をバイト値として取得するためのポインタを指定します。
     * @param[out]  pbyV4  バージョン番号4をバイト値として取得するためのポインタを指定します。
     *
     * @see     SetApplicationVersionNumbers
     */
    void GetApplicationVersionNumbers(qByte* pbyV1, qByte* pbyV2, qByte* pbyV3, qByte* pbyV4) const
    {
#if (defined(SN_COMPILER) && defined(NON_SUPPORT_A_TARGET))
#pragma control %push diag
#pragma diag_suppress=68
#endif

#ifdef CTR_TARGET
#pragma diag_suppress=61,68
#endif

        *pbyV1=qByte((m_uiApplicationVersionNumber & ((qUnsignedInt32)255<<24)) >> 24);
        *pbyV2=qByte((m_uiApplicationVersionNumber & (255<<16)) >> 16);
        *pbyV3=qByte((m_uiApplicationVersionNumber & (255<<8)) >> 8);
        *pbyV4=qByte((m_uiApplicationVersionNumber & (255)));

#ifdef CTR_TARGET
#pragma diag_default=61,68
#endif

#if (defined(SN_COMPILER) && defined(NON_SUPPORT_A_TARGET))
#pragma control %pop diag
#endif
    }

    /*
    Description: Sets the version number of the application.
    Parameters:
    uiApplicationVersionNumber: The application version number as an unsigned integer.
    Remarks: The version number is specific to the given application. This number has no meaning
    to $PRODUCTNAME$ and it is up to the user to assign a meaning to it. The version number may
    be set via this method or via SetApplicationVersionNumbers. The two methods are much the same
    except for the manner in which the version munber is specified.

    Refer to Session::CreateOrJoinSession for an example of how this method could be used.
    See Also:  GetApplicationVersionNumber, GetApplicationVersionNumbers, SetApplicationVersionNumbers
    */

    /*!
     * @brief   アプリケーションのバージョン番号を設定します。
     *
     * アプリケーションのバージョン番号はアプリケーションによって与えられます。
     * この番号はNet-Zシステムのバージョン番号とはなにも関係がなく、意味は開発者によって決められます。
     *
     * @param[in]   uiApplicationVersionNumber  アプリケーションのバージョン番号を指定します。
     *
     * @see     GetApplicationVersionNumber
     */
    void SetApplicationVersionNumber(qUnsignedInt32 uiApplicationVersionNumber)
    {
        m_uiApplicationVersionNumber=uiApplicationVersionNumber;
    }

    /*
    Description: Sets the version number of the application.
    Parameters:
    byV1: First variable of the version number as a byte.
    byV2: Second variable of the version number as a byte.
    byV3: Third variable of the version number as a byte.
    byV4: Fourth variable of the version number as a byte.
    Remarks: The version number is specific to the given application. This number has no meaning
    to $PRODUCTNAME$ and it is up to the user to assign a meaning to it. The version number may
    be set via this method or via SetApplicationVersionNumber. The two methods are much the same
    except for the manner in which the version munber is specified.

    Refer to Session::CreateOrJoinSession for an example of this method in use.
    See Also:  GetApplicationVersionNumbers, GetApplicationVersionNumber, SetApplicationVersionNumber
    */

    /*!
     * @brief   アプリケーションのバージョン番号を設定します。
     *
     * アプリケーションのバージョン番号はアプリケーションによって与えられます。
     * この番号はNet-Zシステムのバージョン番号とはなにも関係がなく、意味は開発者によって決められます。
     *
     * @param[in]   byV1  1番目のアプリケーションバージョン番号を指定します。
     * @param[in]   byV2  2番目のアプリケーションバージョン番号を指定します。
     * @param[in]   byV3  3番目のアプリケーションバージョン番号を指定します。
     * @param[in]   byV4  4番目のアプリケーションバージョン番号を指定します。
     *
     * @see     GetApplicationVersionNumbers
     */
    void SetApplicationVersionNumbers(qByte byV1, qByte byV2, qByte byV3, qByte byV4)
    {
        m_uiApplicationVersionNumber= byV1<<24 | byV2<<16 | byV3 << 8 | byV4;
    }

    /*
    Description: Returns the type of SessionDiscoveryProtocol.
    Returns: A string that identifies the type of SessionDiscoveryProtocol.
    Remarks: The SessionDiscoveryProtocol type is a SessionDiscoveryProtocol specific
    string that defines the type of SessionDiscoveryProtocol,
    /e.g./ LANSessionDiscovery. Each active SessionDiscoveryProtocol must have a
    unique SessionDiscoveryType.
    */
    const qChar *GetSessionDiscoveryType() const
    {
        return m_szSessionDiscoveryType.CStr();
    }

    /*
    Description: Returns the session ID of the SessionDiscoveryProtocol.
    Returns: A string that defines the session ID of the
    SessionDiscoveryProtocol.
    Remarks: The SessionDiscoveryProtocol session ID is defined by the
    SessionDiscoveryProtocol and is not interpreted by
    $PRODUCTNAME$. It is used by the SessionDiscoveryProtocol to identify a given Session.
    */
    const qChar *GetSessionDiscoveryContext() const
    {
        return m_szSessionDiscoveryContext.CStr();
    }

    /*
    Description: Returns the ID of the Session.
    Returns: A value that defines the ID of the Session.
    Remarks: The session ID is a random number generated by $PRODUCTNAME$
    used to indentify a given Session.
    */

    /*!
     * @brief   セッションIDを取得します。
     *
     * セッションIDはNet-Zによってランダムに生成される数値で、
     * セッションを識別することが出来ます。
     *
     * @return  セッションIDが返ります。
     */
    qUnsignedInt32 GetSessionID() const
    {
        return m_uiSessionID;
    }

    // {secret}
    void SetSessionID(qUnsignedInt32 uiSessionID)
    {
        m_uiSessionID=uiSessionID;
    }

    // {secret}
    qBool StreamAttributesOut(qChar* szAttributesString, qUnsignedInt uiLength) const;

    // {secret}
    qBool StreamAttributesIn(qChar* szAttributesString);

    // {secret}
    qBool ExtractFrom(ByteStream *pByteStream);

    // {secret}
    void AddSourceTo(ByteStream *pByteStream) const;

    /*
    Description: Traces the content of this SessionDescription object.
    Remarks: The default information traced includes the application title and version number,
    the session name and ID, the connecting URL, the SessionDiscoveryProtocol type,
    and the $PRODUCTNAME$ version and build numbers.
    */
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS) const;

    /*
    Description: Sets user-defined attributes in the SessionDescription.
    Parameters:
    szKey: The Key variable, as a string, of the attribute to set. The string is limited
    to 127 characters and cannot contain the '=' character.
    szValue: The Value variable, as a string, of the attribute to set. The string is limited
    to 127 characters and cannot contain the '|' character.
    Remarks: An attribute is a user-defined Key-Value pair of string variables. Attributes
    are used to add additional information to the SessionDescription.

    Refer to Session::CreateOrJoinSession for an example of this method in use.
    See Also: GetAttribute, RemoveAttribute, FindFirstAttribute, FindNextAttribute
    */
    qBool SetAttribute(const qChar* szKey, const qChar* szValue);

    /*
    Description: Returns the given attribute.
    Parameters:
    szKey: The Key variable, as a string, of the attribute to get.
    szValue: The Value variable, as a string, of the attribute to get.
    uiLength: The length assigned to the szKey variable.
    Returns: true if the given attribute will be returned, false if it does not exist.
    Remarks: Refer to FindFirstAttribute for an example of this method in use.
    See Also: SetAttribute, RemoveAttribute, FindFirstAttribute, FindNextAttribute
    */
    qBool GetAttribute(const qChar* szKey, qChar* szValue, qUnsignedInt uiLength) const;

    // This macro is never defined.  It is used only to get around a compilation problem with
    // DocJet.
#ifdef FOR_DOC_PURPOSE

#else
    String GetAttribute(const String& strKey) const;
#endif

    /*
    Description: Returns if the given attribute exists or not.
    Returns: true if the given attribute exists, false if not or an error occurred.
    Parameters:
    szKey: The Key variable, as a string, of the attribute to find.
    See Also: SetAttribute, GetAttribute, RemoveAttribute, FindFirstAttribute, FindNextAttribute
    */
    qBool FindAttribute(const qChar* szKey) const;

    /*
    Description: Returns if the given attribute will be removed or not.
    Returns: true if the given attribute will be removed from the list, false if not or an error occurred.
    Parameters:
    szKey: The Key variable, as a string, of the attribute to remove.
    See Also: SetAttribute, GetAttribute, FindAttribute, FindFirstAttribute, FindNextAttribute
    */
    qBool RemoveAttribute(const qChar* szKey);

    /*
    Description: Returns the first attribute in the list.
    Parameters:
    szKey: The Key variable, as a string, of the attribute to find.
    uiLength: The length assigned to the szKey variable.
    Returns: true if the first attribute in the list was found, false if not or an error occurred. If there are
    no attributes in the list then this method will return false.
    Remarks: In conjuction with FindNextAttribute this method can be used to iterate over
    the attributes of a SessionDescription object. For example, to query the sessions available
    on the network and to display the SessionDescriptions and attributes, we could do the
    following. A full implementation of FindFirstAttribute is given in the EnumerateSessions tool.

            {
                List<SessionDescription> lstResult;
                if (Session::QuerySessions(&lstResult,false)==0) {
                    Display(NEX_T("No session found.\n"));
                } else {
                    List<SessionDescription>::iterator it=lstResult.begin();
                    while (it!=lstResult.end()) {
                        // Display information from the session description
                        Display(NEX_T("Application Title : %s\n"),it->GetApplicationTitle());
                        Display(NEX_T("Application Version : %x\n"),it->GetApplicationVersionNumber());
                        Display(NEX_T("Session Name : %s\n"),it->GetSessionName());
                        Display(NEX_T("Session ID : %x\n"),it->GetSessionId());

                        // Display generic attributes
                        qChar szKey[128];
                        if (it->FindFirstAttribute(szKey, 128)) {
                            do {
                                qChar szValue[128];
                                it->GetAttribute(szKey, szValue, 128);
                                Display(NEX_T("\t%s = %s\n"), szKey, szValue);
                            } while (it->FindNextAttribute(szKey, 128));
                        }
                        it++;
                    }
                }
            }

    See Also: SetAttribute, GetAttribute, RemoveAttribute, FindAttribute, FindNextAttribute
    */
    qBool FindFirstAttribute(qChar* szKey, qUnsignedInt uiLength);

    /*
    Description: Returns the attribute that follows the one specified in this method.
    Parameters:
    szKey: The Key variable, as a string, of the attribute that proceeds the one to find.
    uiLength: The length assigned to the szKey variable.
    Returns: true if the next attribute was found, false if not or an error occurred. If the given szKey is the
    last in the list then this method will return false.
    Remarks: This method will return the next szKey in the list from the one given as a parameter.
    In conjuction with FindFirstAttribute this method can be used to iterate over the attributes
    of a SessionDescription object. Refer to FindFirstAttribute and the EnumerateSessions tool
    for an example of how to implement this method.

    Refer to FindFirstAttribute for an example of this method in use.
    See Also: SetAttribute, GetAttribute, RemoveAttribute, FindAttribute, FindFirstAttribute
    */
    qBool FindNextAttribute(qChar* szKey, qUnsignedInt uiLength);

    // {secret}
    size_t GetNbAttributes() const;

    /*
    Description: Propagates the contents of the SessionDescription object to its duplicas and to
    the connected lobbies.
    Remarks: This method is only valid on a Duplication master. It is used to synchronize changes
    made to the session description on the Duplication master with its duplicas and the connected
    lobbies. Once a Session is created, after any changes are made to the SessionDescription,
    via SetAttribute for example, Update must be called to propagate the change to the other
    instances of the SessionDescription object.
    Returns: true if the SessionDescription was successfully updated, false if not or an error occurred. If this method is
    called on a Duplica the error SYSTEMERROR_OBJDUP_INVALID_ON_DUPLICA will be returned.
    */
    qBool Update();

    // {secret}
    void SetUpdateCallback(void (*pfUpdateCallback)())
    {
        m_pfUpdateCallback=pfUpdateCallback;
    }

    // {secret}
    void RegisterSessionDiscoveryTable(SessionDiscoveryTable* pSessionDiscoveryTable)
    {
        m_pSessionDiscoveryTable=pSessionDiscoveryTable;
    }

protected:
    void AddString(ByteStream *pReceivedMessage, const String &refValue) const;
    void ExtractString(ByteStream *pReceivedMessage, String *pValue) const;

    // The Data from the SessionDescription comes from several sources.
    // For some variables, the SessionDescription will be
    // authoritatize - typically for which the system does not provide
    // any meaning .  For other variables, the variable is authoritative
    // in other parts of the system, and the value in the session
    // description is a copy of this other variable.

    /*
     *
     * Info about SessionDiscovery
     *
     */

    // SessionDiscoveryProtocol used to discover this session
    // description is registered (in addition to LAN).
    // SessionDescription is authoritative.
    // {secret}
    String m_szSessionDiscoveryType;
    // SessionDiscovery specific context - used to know where it must be
    // updated.  SessionDescription is authoritative.
    // {secret}
    String m_szSessionDiscoveryContext;

    /*
     *
     * Info about session master (local to session master - not shared)
     *
     * This info is filled by the upper layers automatically (TODO:
     * When?  What happens when it changes?)  It isn't replicated -
     * thus changes must be done on every station.
     *
     */

    // General game product info.  Core is authoritative (Static data
    // from the DDL files)
    // {secret}
    String          m_szApplicationTitle;
    // Application specific version number. SessionDescription is authoritative.
    // {secret}
    qUnsignedInt32  m_uiApplicationVersionNumber;

    // Connection data.
    // {secret}
    String          m_szStationURL;

    // Product info - from ProductInfo.
    // {secret}
    qUnsignedInt32  m_uiLibraryVersionNumber;
    qUnsignedInt32  m_uiLibraryBuildNumber;


    /*
     *
     * Info about session (shared accross session)
     *
     * This info is filled when we create a game - or by the user, on
     * the master.  When it changes (on the master), it is updated to
     * the clients.
     *
     */

    // General info about the session. Core is authoritative (set by
    // Session::Create).
    // {secret}
    String          m_szSessionName;
    // Random session Id.  Core is authoritative (set by Session::Create).
    // {secret}
    qUnsignedInt32  m_uiSessionID;

    // Open portion.  Session Description is authoritative.
    // {secret}
    qMap<String, String> m_mapSessionAttributes;

    // Callback on update.  Goes up to the core.
    // {secret}
    void (*m_pfUpdateCallback)();
    SessionDiscoveryTable   *m_pSessionDiscoveryTable;

};

END_NAMESPACE_NEX

