﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/JobChangeConnection.h>
#include <OnlineCore/src/Core/SystemComponent.h>

//n1589
#include <OnlineCore/src/Transport/PRUDP/PRUDPUserProtocol.h>

//n2788
#include <NetZ/src/Core/DOCore/DirectTransport.h>

BEGIN_NAMESPACE_NEX

class JobConnectStation;
class JobDisconnectStation;

class StationManager: public SystemComponent
{
public:
    NEX_DECLARE_SYSCOMPONENT_METHODS(StationManager,SystemComponent);

    StationManager();
    virtual ~StationManager();

    typedef enum _ConnectionResult
    {
        Success,
        Failure,
        Pending
    } ConnectionResult;

    // This class is simply a pair which take care of the jobs that change connection's
    // status to a station (JobChangeConnection). This job can only connect or disconnect
    // a station. Moreover, we found that it is best to let finish the job before doing
    // the inverse. Hence, a pair is perfect to keep track of this. For exemple, if we
    // are connecting and for a reason we want to disconnect, we can add a pending job
    // for a disconnection. If finally we want to stay connected, we just have to remove
    // the pending job. All this is done by StationManager and ConnectionJobs.
    class ConnectionJobs: public RootObject
    {
    public:
        ConnectionJobs()
        {
            m_pQueued=NULL;
            m_pPending=NULL;
        }

        // The job currently queued to the scheduler - in progress.
        JobChangeConnection* GetQueuedJob() const
        {
            return m_pQueued;
        }
        // The next job to do.  This job can be cancelled if we change
        // our mind (e.g. the next job is to disconnect, but
        // before is is queued, there is request to connect
        // again.  We simply cancel the disconnect job).
        JobChangeConnection* GetPendingJob() const
        {
            return m_pPending;
        }

        void SetQueuedJob(JobChangeConnection* pJob)
        {
            m_pQueued=pJob;
        }
        void SetPendingJob(JobChangeConnection* pJob)
        {
            m_pPending=pJob;
        }

    private:
        // Invalid operations.
        ConnectionJobs(const ConnectionJobs&);
        ConnectionJobs& operator=(const ConnectionJobs&);

    private:

        JobChangeConnection* m_pQueued;     // job currently runnning
        JobChangeConnection* m_pPending;    // next job to run
    };

    static StationManager* GetInstance();

    void RecordStationURLs(DOHandle hStation, const qList<StationURL>& lstURLs);
    qBool RetrieveStationURLs(DOHandle hStation, qList<StationURL>* plstURLs);

    virtual void StateTransition(State eFinal);

    // {secret}
    void AddBootstrapStationURLs(Message *pMessage);
    // {secret}
    qBool ExtractBootstrapStationURLs(Message *pMessage);

    void AddDeadStation(DOHandle hStation);
    qBool StationIsDead(DOHandle hStation);

public:

    void SetInitialConnectionPoint(DOHandle hStation, EndPoint* pEndPoint);
    void ClearInitialEndPoint();
    DOHandle GetInitialConnectionPoint() const
    {
        return m_hInitialConnectionPoint;
    }
    EndPoint* GetInitialEndPoint() const
    {
        return m_pInitialEndPoint;
    }

    /* Newest entry points :-) */

    Station::ConnectionState GetTargetConnectionState(DOHandle hStation);
    JobDisconnectStation *DisconnectStation(Station* pStation);
    ConnectionResult ConnectStation(DOHandle hStation);
    qBool ConnectionIsPossible(DOHandle hStation);

    JobConnectStation* GetLatestConnectionJob(DOHandle hStation) const;
    JobChangeConnection* GetLatestJob(DOHandle hStation) const;

#ifdef NEX_TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS) const;
#endif  // NEX_TRACE_ENABLED

    void ProcessCompletedJob(JobChangeConnection* pJob);
    void TraceState(DOHandle hStation, qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);


    //n1589
    void UpdateStationStatus(Station* pStation, DOOperation* pOperation);

    //n1589
    void UpdatePeerAddress();

    // n2370
    // {secret}
    void RegisterStationEventCallback( StationEventCallback callback );
    // {secret}
    void UnregisterStationEventCallback();

    // {secret}
    static void SetUpdatePeerAddressBypassCheck(qBool b)
    {
        s_bUpdatePeerAddressBypassCheck = b;
    }

private:
    void ActivateJob(JobChangeConnection* pJob);
#if NEX_ENABLE_OBJDUP_CONNECTSTATION_OPTIMIZATION
    Station::ConnectionState GetTargetConnectionState( const Station::Ref *pStationRef );
    qBool ConnectionIsPossible( const Station::Ref *pStationRef );
#endif  // NEX_ENABLE_OBJDUP_CONNECTSTATION_OPTIMIZATION

private:

    typedef qMap<DOHandle, qList<StationURL> > StationURLs;
    StationURLs m_mapURLs;

    qMap<DOHandle,ConnectionJobs*> m_mapStationJobs;

    DOHandle m_hInitialConnectionPoint;
    EndPoint* m_pInitialEndPoint;

    qList<DOHandle> m_lstDeadStations;

    // n2370
    /*
     * ステーションのステートが変化したことをアプリ側から取得できるコールバックです。
     * これを利用することで、アプリ側からステーションの状態が変化したタイミングを知ることができます。
     * ステーションがセッションに参加したタイミングや、あるいはセッションを抜けようとしているタイミングを取得できます。
     */
    StationEventCallback m_pfStationEventCallback;

    static qBool s_bUpdatePeerAddressBypassCheck;
};

END_NAMESPACE_NEX

