﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once
#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

#include <NetZ/src/Core/DOCore/DupSpace.h>
#include <NetZ/src/Core/DupSpace/MatchOperationTriggers.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/Selection.h>
#include <OnlineCore/src/Platform/Core/PerfCounter.h>
#include <OnlineCore/src/Core/SmoothedAverage.h>
#include <OnlineCore/src/Platform/Core/qVector.h>

// The includes are outside the file protection to solve problems related to a
// circular file inclusion.


BEGIN_NAMESPACE_NEX

class DupSpaceOperation;
class DuplicatedObject;
class IteratorOverDOs;
class DOOperation;
class DOFilter;
class Station;

//{secret}
// Match policy flags.
#define MATCH_PUBLISHER_MASTER      1<<0
#define MATCH_PUBLISHER_DUPLICA     1<<1
#define MATCH_SUBSCRIBER_MASTER     1<<2
#define MATCH_SUBSCRIBER_DUPLICA    1<<3
#define MATCH_PS_MASTER             1<<4
#define MATCH_PS_DUPLICA            1<<5
#define MATCH_CELL_MASTER           1<<6
#define MATCH_CELL_DUPLICA          1<<7

//group=DDL Output

/*
Description: The root class of all user-defined Duplication space s.
Remarks: Duplication Spaces are disabled by default. If you want to use this very complex feature
with your game, please contact {html:<a href="mailto:eval@quazal.com">}eval@quazal.com{html:</a>}
for information on how to obtain a version of Net-Z that does support it.

Duplication space s are one mechanism used by $PRODUCTNAME$ to enable a game to be scaled
as they ensure that only the objects that a Station requires are duplicated to that station.
The periodic matching behaviour of $PRODUCTNAME$ is specified via the MatchOperationTriggers
class using the GetLocalTriggers and GetGlobalTriggers methods. The periodic matching behaviour of
each space that results in Duplica creation is specified via GetLocalTriggers since a successful
match within a single duplication space is a sufficient condition for Duplica creation. GetGlobalTriggers
is used for periodic Relevance checks that result in Duplica deletion since non-relevance within one
duplication space is not a sufficient condition for deletion. The non-relevance within all duplication
spaces has to be assured prior to Duplica deletion. This policy is set for all duplication spaces
via the GetGlobalTriggers method.

For example, suppose you declare a duplication space in the DDL file as follows:

        dupspace InterestSpace;

You then need to define an InterestSpace class that will uitimately inherit, via the DDL generated
InterestSpaceDDL.h file, from the DuplicationSpace class. Once a duplication space is defined, you then
need to implement the PSMatch function. In the PSMatch function, you specify the conditions that determine
whether or not a duplicated object will be discovered by another object within the InterestSpace.

        class InterestSpace : public DUPSPACE (InterestSpace) {
            public:
            virtual qBool PSMatch(DuplicatedObject * oPublisher,
                                    DuplicatedObject * oSubscriber){
                // User implementation of the PSMatch function for InterestSpace
            }
            // Other methods
        }

The DuplicationSpace is instantiated by $PRODUCTNAME$ prior to session creation as a result of a
DDL dupspace definition.  To access the DuplicationSpace from within your game, use the GetInstance method.
For example, if InterestSpace is a DuplicationSpace declared in the DDL file, the following method returns
a pointer to that DuplicationSpace:

        DuplicationSpace* dupSInterestSpace = InterestSpace::GetInstance();

For examples of how to implement a duplication space, refer to the DupSpaces sample available in the distribution.
Refer also to the duplication space samples in the User Guide and the Duplication Space Overview in this manual.
See Also: DDL File Syntax, MatchOperationTriggers, Duplication Space Overview
*/
class DuplicationSpace
{
    //group=Methods
public:

    /*
    Remarks: A DuplicationSpace object should never be directly instantiated. Instead, it is created by $PRODUCTNAME$
    prior to session creation as a result of a DDL dupspace declaration. To access the DuplicationSpace object from
    within your game, use the GetInstance method. For example, if InterestSpace is a duplication
    space declared in the DDL file, the following method returns a pointer to that DuplicationSpace:

            DuplicationSpace* dupSInterestSpace = InterestSpace::GetInstance();

    See Also: DDL File Syntax
    */
    DuplicationSpace();

    /*
    Remarks: A DuplicationSpace object should never be directly destroyed.  It is destroyed
    by $PRODUCTNAME$ when the session terminates.
    See Also: DDL File Syntax
    */
    virtual ~DuplicationSpace();


    //{secret}
    DupSpace::ID GetID()
    {
        return m_id;
    }

public:

    // Methods that are redefined by specific DuplicationSpaces.
    //{secret}
    virtual const qChar* GetClassNameString()=0;

    //{secret}
    virtual qBool UsesDefaultCells()
    {
        return false;
    }

    //{secret}
    virtual IteratorOverDOs* CreateSubscriberIterator(qUnsignedInt uiPSRelation);
    //{secret}
    IteratorOverDOs* CreateSubscriberIterator()
    {
        return CreateSubscriberIterator(m_uiPSRelation);
    }

    //{secret}
    virtual IteratorOverDOs* CreatePublisherIterator(qUnsignedInt uiPSRelation);
    //{secret}
    IteratorOverDOs* CreatePublisherIterator()
    {
        return CreatePublisherIterator(m_uiPSRelation);
    }

    //{secret}
    IteratorOverDOs* CreatePSIterator();
    // {secret}
    IteratorOverDOs* CreatePSIterator(Selection* pSelection);

    //{secret}
    IteratorOverDOs* CreateCellIterator(qUnsignedInt uiCellRelation);
    //{secret}
    IteratorOverDOs* CreateCellIterator()
    {
        return CreateCellIterator(m_uiCellRelation);
    }

    //{secret}
    void ReleaseIterator(IteratorOverDOs* pIt) const;


    //{secret}
    Selection* SelectAppropriateSelection(qBool bMaster, qBool bDuplica);

    //{secret}
    Selection* SelectAppropriateSelection(qUnsignedInt uiAxis,qUnsignedInt uiMasterFlag, qUnsignedInt uiDuplicaFlag)
    {
        return SelectAppropriateSelection((uiAxis & uiMasterFlag)==uiMasterFlag, (uiAxis & uiDuplicaFlag)==uiDuplicaFlag);
    }


    //{secret}
    qBool IsAPublisher(DuplicatedObject* pDO)
    {
        return IsAPublisher(pDO, m_uiPSRelation);
    }


    //{secret}
    qBool IsAPublisher(DuplicatedObject* pDO, qUnsignedInt uiPSRelation);


    //{secret}
    qBool IsASubscriber(DuplicatedObject* pDO)
    {
        return IsASubscriber(pDO, m_uiPSRelation);
    }


    //{secret}
    qBool IsASubscriber(DuplicatedObject* pDO, qUnsignedInt uiPSRelation);


    //{secret}
    qBool IsACell(DuplicatedObject* pDO)
    {
        return IsACell(pDO, m_uiCellRelation);
    }


    //{secret}
    qBool IsACell(DuplicatedObject* pDO, qUnsignedInt uiCellRelation);


    //{secret}
    qBool IsAPS(DuplicatedObject* pDO)
    {
        return IsAPS(pDO, m_uiCellRelation);
    }


    //{secret}
    qBool IsAPS(DuplicatedObject* pDO, qUnsignedInt uiCellRelation);

    //{secret}
    virtual void OperationEndMatchTrigger(DOOperation* pOperation);

    // {secret}
    void SetPSRelation(qUnsignedInt uiNewRelation);
    // {secret}
    void SetCellRelation(qUnsignedInt uiNewRelation);


public:

    //group=DupSpace System Callbacks

    /*
    Description: Called by $PRODUCTNAME$ whenever a DupSpaceOperation begins.
    Remarks: The method is called at the beginning of a DupSpaceOperation with
    a reference to the DupSpaceOperation that just began as a parameter. Switch
    this parameter to find out what kind of operation it is and then implement
    your own functionality.
    Parameters:
    pOperation: a pointer to a DupSpaceOperation that has begun.
    See Also: OperationEnd, MatchOperation, OperationManager
    */
    virtual void OperationBegin(DupSpaceOperation* pOperation)
    {
        DUMMY_PARAM(pOperation);
    }

    /*
    Description: Called by $PRODUCTNAME$ whenever a DupSpaceOperation ends.
    Remarks: The method is called at the end of a DupSpaceOperation with a
    reference to the DupSpaceOperation that just terminated as a parameter.
    Switch this parameter to find out what kind of operation it is and then
    implement your own functionality.
    Parameters:
    pOperation: a pointer to a DupSpaceOperation that has ended.
    See Also: OperationBegin, MatchOperation, OperationManager
    */
    virtual void OperationEnd(DupSpaceOperation* pOperation)
    {
        DUMMY_PARAM(pOperation);
    }

    /*
    {Group:DupSpace Match Methods}

    Description: Determines whether, for any given publisher-subscriber pair of DOs in a duplication space,
    the Publisher DO is discovered by the subscriber's station.
    Remarks: This user implemented function is iterated over the possible pairs of publishers and subscribers in the duplication
    space on each station to determine if a discovery is to take place or an already discovered object is still relevant.
    $PRODUCTNAME$ divides the space into cells and, on each station, for all of the cells for which the station is the master,
    it matches all of the Publisher duplicated objects with all of the duplicas of the Subscriber objects.

    The match decision will typically be based on a computation involving the datasets of the two duplicated objects,
    such as distance, location, interest, etc.  If the PSMatch function returns true, a Duplica of the Publisher is created
    on the station where the Duplication master of the Subscriber resides. The match condition must be written so
    that it will return the same value if it is called on either the Duplication master or the Duplica of a
    Publisher or Subscriber.

    The following is an example of the PSMatch implementation for an InterestSpace populated with User
    duplicated objects. User duplicated objects have an Interest dataset and they discover each other
    only if that Interest is the same. If the object has already been discovered, the example here also
    checks whether an existing object was recently created and, if so, assumes that it is still relevant.
    If you were checking for the distance between the publisher and the subscriber, you could implement
    the addition of a buffer to the destruction threshold (in this case, the distance) to prevent a
    creation/destruction cycle if the object oscillates in and out of the threshold.

            qBool InterestSpace::PSMatch (DuplicatedObject* pPublisher, DuplicatedObject* pSubscriber, qBool bOnRelevanceCheck){
                if (pPublisher->IsAKindOf(DOCLASSID(User)) && pSubscriber->IsAKindOf(DOCLASSID(User))){
                    User *pUserPublisher = (User*) pPublisher;
                    User *pUserSubscriber = (User*) pSubscriber;
                    if (bOnRelevanceCheck) {
                        // In the object constructor, we have recorded the creation
                        // time (this would be custom code in the User
                        // class).  Here, we check how long ago the object was
                        // created and assume a match if the object was
                        // created recently.
                        //
                        if (Time::GetTime() - pUserPublisher->GetCreationTime() < 1000) {
                            return true;
                        }
                    }
                    if (pUserPublisher->mdsInterest.iName == pUserSubscriber->mdsIntrest.iName) {
                        return true;
                    }
                }
                return false;
            }

    Parameters:
    pPublisher: a pointer to a publishing DuplicatedObject in the DuplicationSpace.
    pSubscriber: a pointer to a subscribing DuplicatedObject in the DuplicationSpace.
    bOnRelevanceCheck: PSMatch is called with this parameter set to true when the check is a relevance
    check on a previously discovered object; that is, if the check is to determine if the object
    should be destroyed.
    Returns: This function should be defined to return true if a Publisher duplicated object is to be
    discovered by the Subscriber duplicated object and false if otherwise.  If bOnRelevanceCheck is true,
    the condition should be a little more relaxed, to prevent oscillation in the creation/destruction of objects.
    See Also: CellMatch, SetSymmetricPSMatch
    */
    virtual qBool PSMatch(DuplicatedObject* pPublisher, DuplicatedObject* pSubscriber, qBool bOnRelevanceCheck)=0;

    //group=DupSpace Cell Methods

    /*
    Description: Specifies when a duplicated object belongs to a Cell and when it does not.
    Remarks: This function has to be redefined only if custom cells are created.  If you create custom cells,
    then this is the function that decides when a duplicated object is to belong to a Cell.

    The following is an example of a CellMatch implementation for an InterestSpace populated
    with User duplicated objects and split up into InterestArea cells.  User duplicated objects have an
    InterestArea dataset and they belong to a Cell only if that InterestArea is the same as the cell's
    InterestAreaDescription:

            qBool InterestSpace::CellMatch (DuplicatedObject* pPS, DuplicatedObject* pCell, qBool bOnRelevanceCheck)=0){
                if (pPS->IsAKindOf(DOCLASSID(User)) && pCell->IsAKindOf(DOCLASSID(InterestArea))){
                    User *pUser = (User*) pPS;
                    if (bOnRelevanceCheck) {
                        // In the object constructor, we record the creation
                        // time (this would be custom code in the User
                        // class).  Here, we check how long ago the object was
                        // created, and assume a match  if the object was
                        // created recently.
                        //
                        if (Time::GetTime() - pUser->GetCreationTime() < 1000) {
                            return true;
                        }
                    }
                    InterestArea *pCellInterestArea = (InterestArea*) pCell;
                    if (pUser->mdsInterestArea.iName == pCellInterestArea->mdsInterestAreaDesc.iName) {
                        return true;
                    }
                }
                return false;
            }

    Parameters:
    pPS: a pointer to an object that is a publisher or a subscriber in the DuplicationSpace.
    pCell: a pointer to a cell in the DuplicationSpace.
    bOnRelevanceCheck: true when this CellMatch is a relevance check with
    the intention of deleting objects that are not relevant anymore.
    Returns: This function should be defined so as to return true if a duplicated object belongs to a Cell and
    false if otherwise.  If bOnRelevanceCheck is true, the condition should
    be a little more relaxed, to prevent oscillation in the creation/destruction of objects.
    */
    virtual qBool CellMatch(DuplicatedObject* pPS, DuplicatedObject* pCell, qBool bOnRelevanceCheck)=0;

    /*
    Description: Called when the DuplicationSpace is constructed. It
    constructs the cells declared in the DDL file or default $PRODUCTNAME$ cells.
    Remarks: If you do not manually specify cells in your duplication space then you
    do not need to redefine this virtual function as it will be called by $PRODUCTNAME$
    to create default cells. However, if you did specify cells in the DDL file this is
    where you should create and publish all of the Cell duplicated objects. This is also
    an ideal place to define your match policy, such as MatchOperationTriggers::DisablePeriodicMatch.
    See Also: DuplicatedObject::Create, MatchOperationTriggers, DDL File Syntax
    */
    virtual void CreateCells()=0;

    /*
    {Group:DupSpace Match Methods}

    Description: Checks if a Duplica or all duplicas are still relevantly duplicated.
    Parameters:
    pDuplicaToCheck:  Optional pointer to a DuplicatedObject instance whose Relevance needs to be checked.
    Returns: true if the parameter is a valid Duplica, false if not or an error occurred.  If the method
    is called on a Duplication master, the call returns false and SYSTEMERROR_OBJDUP_INVALID_ON_MASTER
    is signalled.
    Remarks: This method can be called by $PRODUCTNAME$ or the user to check if a given
    Duplica should still be duplicated as it is, or if it should be deleted on some station(s).
    The CheckRelevance signature without a specific Duplica to check as a parameter is used by
    $PRODUCTNAME$ or the user to perform a Relevance check for all of the local duplicas.
    With the default Relevance check policy, the method is called by $PRODUCTNAME$ whenever
    a ChangeMasterStationOperation or RemoveFromStoreOperation takes place. A duplica is considered
    relevant as long as it matches a local object in the Duplication space or as long as there
    is a Hard reference or iterator pointing to it.
    */
    static qBool CheckRelevance(DuplicatedObject* pDuplicaToCheck);
    static void CheckRelevance();

    /*
    Description: Matches publishers and subscribers (MatchPSRelationPair) for all of the
    duplicated objects that belong to the Cell.
    Remarks: The pCell parameter must be a valid Duplication master cell.  The matching is
    performed based on the user implemented PSMatch function.
    Parameters:
    pCell:  a DuplicatedObject pointer to the Duplication master of a Cell that is to perform
    matching of its publishers and subscribers.
    Returns: true if pCell is a Duplication master cell, false if not or an error occurred.
    */
    qBool MatchAcrossPSRelation(DuplicatedObject* pCell);

    // {secret}
    qBool NewMatchAcrossSymmetricPSRelation(DuplicatedObject* pCell,
                                            qVector<DORef>& refPublishers,
                                            qVector<DORef>& refSubscribers,
                                            qVector<DORef>& refPS);

    /*
    Description: Calls MatchAcrossPSRelation for all of the cells for which this station is
    a Duplication master.
    Remarks: This function calls MatchAcrossPSRelation for all of the local station's duplication
    master cells.
    */
    void MatchAcrossPSRelations();

    /*
    Description: Matches duplicated objects with cells.
    Remarks: The function performs a match between all of  the cells in the duplication space
    with all duplicated objects.
    */
    void MatchAcrossCellRelation();

    //group=DupSpace Match Methods

    /*
    Description: Called automatically and periodically by default to update the state of the distributed system.
    Remarks: This function is performed periodically by default, unless you turn off the
    periodic match with the MatchOperationTriggers::DisablePeriodicMatch function. It calls
    MatchAcrossCellRelation and MatchAcrossPSRelation to ensure a complete
    duplication state update of the distributed system.
    */
    void Match();

    /*
    Description: Calls MatchPSRelationPair for the provided publisher across all the subscribing
    duplicated objects of the space.
    Remarks: This function is called automatically by $PRODUCTNAME$ whenever a new publishing
    duplicated object is added to the duplication space.  Call this function to perform
    MatchPSRelationPair for a specific Publisher and all subscribers.
    Parameters:
    pPublisher:  Duplicated Object pointer to the Publisher that is to be matched with all
    subscribers in the space.
    Returns: true if the parameter is a valid publishing duplicated object, false if not or an error occurred.
    See Also: MatchPSRelationPair, Match
    */
    qBool MatchAcrossSubscribers(DuplicatedObject* pPublisher);

    /*
    Description: Calls MatchPSRelationPair for the provided Subscriber across all publishing duplicated objects in the space.
    Remarks: This function is called automatically by $PRODUCTNAME$ whenever a new subscribing
    duplicated object is added to the duplication space.  Call this function to perform
    MatchPSRelationPair for a specific Subscriber and all publishers.
    Parameters:
    pSubscriber:  Duplicated Object pointer to the Subscriber that is to be matched with all
    publishers in the space.
    Returns: true if the parameter is a valid subscribing duplicated object, false if not or an error occurred.
    See Also: MatchPSRelationPair, Match
    */
    qBool MatchAcrossPublishers(DuplicatedObject* pSubscriber);

    //group=DupSpace Cell Methods

    /*
    Description: Calls MatchCellRelationPair for the specified Cell across all of the publishing
    and subscribing duplicated objects in the duplication space.
    Remarks: This function is called automatically by $PRODUCTNAME$ whenever a new subscribing
    duplicated object is added to the duplication space. Call this function to perform
    MatchCellRelationPair for a specific Cell and all publishers and subscribers.
    Parameters:
    pCell:  DuplicatedObject pointer to the Cell that is to be matched with all publishers
    and subscribers in the space.
    Returns: true if the parameter is a valid Cell, false if not or an error occurred.
    See Also: MatchCellRelationPair, CellMatch
    */
    qBool MatchAcrossPS(DuplicatedObject* pCell);

    /*
    Description: Takes a PS duplicated object (Publisher and/or Subscriber) and calls
    MatchCellRelationPair for all of the cells in the space.
    Remarks: This function is called automatically by $PRODUCTNAME$ whenever a new subscribing duplicated
    object is added to the duplication space.  Call this function to perform MatchCellRelationPair for
    a specific PS and all of the cells in the space.
    Parameters:
    pPS: Either a publishing and/or subscribing DuplicatedObject pointer to the duplicated object that is
    to be matched via the CellMatch function.
    Returns: true if the parameter is a valid duplicated object PS (Publisher and/or Subscriber),
    false if not or an error occurred.
    See Also: CellMatch, MatchCellRelationPair
    */
    qBool MatchAcrossCells(DuplicatedObject* pPS);

    //group=DupSpace Match Methods

    /*
    Description: Calls the appropriate methods to update the duplication space with respect to
    the specified duplicated object.
    Remarks: Based on the role of the duplicated object in the space as well as the state of
    the duplication space, one or several of the following methods will be called by $PRODUCTNAME$:
    CellMatch, MatchAcrossSubscribers, MatchAcrossPublishers, or MatchAcrossPS.
    Parameters:
    pDO: Either a publishing and/or subscribing DuplicatedObject pointer to the duplicated object
    according to which the state of the duplication space is to be updated.
    Returns: true if the parameter is a valid DuplicatedObject pointer, false if not or an error occurred.
    See Also: CellMatch, Match, MatchAcrossSubscribers, MatchAcrossPublishers, MatchAcrossPS.
    */
    qBool MatchSingleObject(DuplicatedObject* pDO);

    /*
    Description: Calls PSMatch and creates a Duplica, if necessary.
    Remarks: This function takes a publisher-subscriber pair of DuplicatedObject instances,
    calls PSMatch to determine if the Subscriber should discover the Publisher, and then creates
    the apropriate Duplica on the station on which the Subscriber duplicated object has its Duplication master.
    Parameters:
    pPublisher:  The publishing duplicated object instance to be matched with pSubscriber.
    pSubscriber:  The subscribing duplicated object instance that is to be matched with pPublisher.
    Returns: true if the parameters are valid, false if not or an error occurred.
    See Also: PSMatch, Match
    */
    qBool MatchPSRelationPair(DuplicatedObject* pPublisher, DuplicatedObject* pSubscriber);

    /*
    {Group:DupSpace Cell Methods}

    Description: Calls CellMatch and creates a Duplica, if necessary.
    Remarks: This function takes a PS (Publisher or Subscriber) and a Cell duplicated object and does the
    following:
    (1) Calls CellMatch to determine if the particular duplicated object belongs to the Cell or not, and
    (2) Creates appropriate duplicas on Subscriber stations in case the PS belongs to the Cell
    as a previously unpublished Publisher or a previously unsubscribing Subscriber.
    Parameters:
    pPS:  The duplicated object pointer to the PS (Publisher and/or Subscriber) that
    is to be matched in its relation with a pCell.
    pCell:  The Cell duplicated object pointer to the Cell that is to be updated with respect to pPS.
    Returns: true if the parameters are valid, false if not or an error occurred.
    See Also: CellMatch, Match
    */
    qBool MatchCellRelationPair(DuplicatedObject* pPS, DuplicatedObject* pCell);

    //{secret}
    void NoCellRequired();

    /*
    Description: Returns the MatchOperationTriggers for this particular duplication space
    Remarks: The MatchOperationTriggers class is used to fine-tune the periodic match behaviour
    that results in Duplica creation.
    Returns: The MatchOperationTriggers instance returned by this function can be used to control
    the behaviour of this space's periodic match activities that result in Duplica creation.
    See Also: MatchOperationTriggers
    */
    MatchOperationTriggers* GetLocalTriggers()
    {
        return &m_oLocalTriggers;
    }

    /*
    Description: Returns the global MatchOperationTriggers for all duplication spaces in the session.
    Remarks: The MatchOperationTriggers class instance returned by this method is used to fine-tune
    the periodic Relevance check behaviour that results in Duplica deletion.
    Returns: A pointer to a MatchOperationTriggers instance that can be used to fine-tune the
    periodic Relevance check activities that result in Duplica deletion.
    See Also: MatchOperationTriggers
    */
    static MatchOperationTriggers* GetGlobalTriggers();

    //group=DupSpace General Methods

    /*
    Description: Returns whether or not a given Duplica is relevant in this DuplicationSpace.
    Parameters:
    pDO: A pointer to the Duplica whose Relevance will be determined.
    Returns: true if the specified Duplica pDO is relevant in this DuplicationSpace, false if not or an
    error occurred.
    Remarks: This method can be used to determine if a Duplica is duplicated on a given Station due to a
    DuplicationSpace::PSMatch or DuplicationSpace::CellMatch, /i.e./ the object is relevant on the Station.
    If this method returns false then the Duplica is duplicated on the Station due to another reason such as
    being a Global duplicated object or duplicated due to Fault tolerance requirements.
    See Also: DuplicationSpace::PSMatch, DuplicationSpace::CellMatch, Relevance
    */
    virtual qBool DuplicaIsRelevant(DuplicatedObject *pDO);

    // This macro is never defined.  It is only used to create a dummy method where the
    // documentation for GetInstance can be located.
#ifdef FOR_DOC_PURPOSE

    /*
    Description: Returns a pointer to the DuplicationSpace instance.
    Returns: A pointer to the DuplicationSpace.
    Remarks: Use the reference returned by this method to access the DuplicationSpace.
    For example, get the local triggers as follows:

            DuplicationSpace::GetInstance()->GetLocalTriggers();

    */
    static DuplicationSpace* GetInstance()
    {
        SYSTEMCHECK(false);
    }
#endif
    //group=DupSpace Match Methods

    /*
    Description: Returns whether or not the PSMatch function is symmetric.
    Returns: true if the PSMatch function is symmetric, false if it is not symmetric.
    See Also: SetSymmetricPSMatch
    */
    qBool PSMatchIsSymmetric()
    {
        return m_bSymmetricPSMatch;
    }

    /*
    Description: Sets whether or not the PSMatch function is symmetric.
    Parameters:
    bSymmetricPSMatch: If true (the default), the PSMatch function is symmetric and will only be called once on complementary
    Publisher/Subscriber pairs. If false, the PSMatch function will be called on all combinations
    of Publisher/Subscriber pairs.
    Remarks: This method sets how the PSMatch function is called on all objects within this
    Duplication space. It should only be set to true when all objects called by the PSMatch
    function are both publishers and subscribers. When this is the case, setting this method to
    true can decrease the amount of time and resources needed for the PSMatch operation.

    When PSMatch is called on a Station, $PRODUCTNAME$ matches all Publisher duplicated objects
    (masters and duplicas) with all Subscriber object duplicas. Therefore, when objects are both
    Publishers and Subscribers, PSMatch is called twice on a given object, once as a Publisher
    and once as a Subscriber. If the result of the match is the same regardless of whether the PSMatch
    function is called on the Publisher or on the Subscriber, /i.e./ the function is symmetric, then
    calling PSMatch a second time is redundant. Therefore, to avoid making this redundant call, you can
    set SetSymmetricPSMatch to true. For example, the following is called to set the PSMatch function
    to be symmetric.

        DuplicationSpace::GetInstance()->SetSymmetricPSMatch;

    The default behaviour of a Duplication space is to assume that the PSMatch function is not
    symmetric and to call PSMatch over all Publisher/Subscriber pair combinations.
    See Also: PSMatchIsSymmetric, PSMatch
    */
    void SetSymmetricPSMatch(qBool bSymmetricPSMatch=true);

    //group=DupSpace General Methods

    /*
    Description: Returns whether or not object datasets are automatically refreshed
    before the Duplication space performs either PSMatch or CellMatch operations.
    Returns: true if DuplicatedObject::Refresh is called automatically before match
    operations are performed, false if DuplicatedObject::Refresh is not called automatically.
    See Also: SetAutomaticRefresh
    */
    qBool RefreshIsAutomatic()
    {
        return m_bAutomaticRefresh;
    }

    /*
    Description: Sets whether or not object datasets will be automatically refreshed before
    the Duplication space performs PSMatch or CellMatch operations.
    Remarks: The default behaviour of a Duplication space is to automatically refresh all
    object datasets before performing PSMatch or CellMatch operations to ensure that
    match operations are performed on the most accurate data. If you set this method to false
    you need to call DuplicatedObject::Refresh manually. For example, say within your main loop
    you call DuplicatedObject::Refresh before performing object physics and then call PSMatch.
    In this case SetAutomaticRefresh should be set to false as there is no need to call
    DuplicatedObject::Refresh before performing the match operation as datasets were refreshed
    in the proceeding step and doing so would only increase processing time and resource usage.
    Parameters:
    bAutomaticRefresh: If true (the default), $PRODUCTNAME$ will call DuplicatedObject::Refresh
    before performing Duplication space match operations.
    If false, $PRODUCTNAME$ does not automatically call DuplicatedObject::Refresh.
    See Also: RefreshIsAutomatic, PSMatch, DuplicatedObject::Refresh
    */
    void SetAutomaticRefresh(qBool bAutomaticRefresh=true)
    {
        m_bAutomaticRefresh=bAutomaticRefresh;
    }

    // {secret}
    void AssignID();
    // {secret}
    static void ResetIDGenerator();

    //group=DupSpace Role Methods

    /*
    Description: Declares the specified duplicated object class as a Subscriber in this DuplicationSpace.
    Parameters:
    idClassID: The ID of the DuplicatedObject class that will be declared a Subscriber in this DuplicationSpace.
    bIncludeDerivedClasses: (Required) If true, all classes derived from the specified class (idClassID) will also be
    declared as Subscriber s in this DuplicationSpace.
    Remarks: This method can be used to add a given duplicated object class as a Subscriber in this
    DuplicationSpace. It is the equivalent of a DLL file subscriber declaration. However, contrary
    to DDL declarations, this method may be called at run time.
    This method must be called on all stations to ensure that the DuplicationSpace is replicated in the
    same manner on all stations. It must also be called after the $PRODUCTNAME$ object has been created.
    If the class given by idClassID is already declared as a Subscriber, then further calls to this method are ignored.

    Refer to the Duplication Space Overview page for more information about how and when to use this method.
    See Also: AddPublisherDOClass, AddCellDOClass, ClearAllDOClassRoles, Duplication Space Overview
    */
    void AddSubscriberDOClass(DOClassID idClassID, qBool bIncludeDerivedClasses);

    /*
    Description: Declares the specified duplicated object class as a Publisher in this DuplicationSpace.
    Parameters:
    idClassID: The ID of the DuplicatedObject class that will be declared a Publisher in this DuplicationSpace.
    bIncludeDerivedClasses: (Required) If true, all classes derived from the specified class (idClassID)
    will also be declared as Publisher s in this DuplicationSpace.
    Remarks: This method can be used to add a given duplicated object class as a Publisher in this
    DuplicationSpace. It is the equivalent of a DLL file publisher declaration. However, contrary
    to DDL declarations, this method may be called at run time.
    This method must be called on all stations to ensure that the DuplicationSpace is replicated in the
    same manner on all stations. It must also be called after the $PRODUCTNAME$ object has been created.
    If the class given by idClassID is already declared as a Publisher, then further calls to this method are ignored.

    Refer to the Duplication Space Overview page for more information about how and when to use this method.
    See Also: AddSubscriberDOClass, AddCellDOClass, ClearAllDOClassRoles, Duplication Space Overview
    */
    void AddPublisherDOClass(DOClassID idClassID, qBool bIncludeDerivedClasses);

    /*
    {Group:DupSpace Cell Methods}

    Description: Declares the specified duplicated object class as a Cell in this DuplicationSpace.
    Parameters:
    idClassID: The ID of the DuplicatedObject class that will be declared a Cell in this DuplicationSpace.
    bIncludeDerivedClasses: (Required) If true, all classes derived from the
    specified class (idClassID) will also be declared as Cell s in this DuplicationSpace.
    Remarks: This method can be used to add a given duplicated object class as a Cell in this
    DuplicationSpace. It is the equivalent of a DLL file cell declaration. However, contrary
    to DDL declarations, this method may be called at run time.
    This method must be called on all stations to ensure that the DuplicationSpace is replicated in the
    same manner on all stations. It must also be called after the $PRODUCTNAME$ object has been created.
    If the class given by idClassID is already declared as a Cell, then further calls to this method are ignored.

    Refer to the Duplication Space Overview page for more information about how and when to use this method.
    See Also: AddSubscriberDOClass, AddPublisherDOClass, ClearAllDOClassRoles, Duplication Space Overview
    */
    void AddCellDOClass(DOClassID idClassID, qBool bIncludeDerivedClasses);

    /*
    Description: Clears all Publisher s, Subscriber s, and Cell s associated with this DuplicationSpace.
    Remarks: This method clears all Publisher, Subscriber, and Cell declarations that have previously
    been made for this DuplicationSpace whether in a DDL file or by means of the AddSubscriberDOClass,
    AddPublisherDOClass, or AddCellDOClass methods.
    This method must be called on all stations to ensure that the DuplicationSpace is replicated in the
    same manner on all stations. It must also be called after the $PRODUCTNAME$ object has been created.

    Refer to the Duplication Space Overview page for more information about how and when to use this method.
    See Also: AddSubscriberDOClass, AddPublisherDOClass, AddCellDOClass, Duplication Space Overview
    */
    void ClearAllDOClassRoles();

    // {secret}
    static qBool IsUsingSessionSpace()
    {
        return s_bUseSessionSpace;
    }
    // {secret}
    static void DisableSessionSpace()
    {
        s_bUseSessionSpace = false;
    }
    // {secret}
    static void EnableSessionSpace()
    {
        s_bUseSessionSpace = true;
    }

protected:
    // {secret}
    DOFilter *GetCellFilter() const
    {
        return m_pfCellFilter;
    }
    // {secret}
    DOFilter *GetPublisherFilter() const
    {
        return m_pfPublisherFilter;
    }
    // {secret}
    DOFilter *GetSubscriberFilter() const
    {
        return m_pfSubscriberFilter;
    }
    // {secret}
    DOFilter *GetPSFilter() const
    {
        return m_pfPSFilter;
    }

    // {secret}
    void MatchAcrossPSRelations_SingleCellVariant();

protected:
    // Default cells implementation.

    void CreateDefaultCells();
    qBool DefaultCellMatch(DuplicatedObject* pPS, DuplicatedObject* pCell);

private:
    void MatchDOPair(DuplicatedObject* pA, DuplicatedObject* pB, qBool bCellMatch, qBool bSymmetric=false);

    const Station* GetLocation(const DuplicatedObject* pDO);

    qBool IndicateDiscovery(DuplicatedObject* pPublisher, DuplicatedObject* pSubscriber);
    void ProcessMatch(DuplicatedObject* pPublisher, DuplicatedObject* pSubscriber, const Station* pSubscriberLocation);

private:
    // This class is allowed to manipulate the list.
    //{secret}
    friend class DuplicationSpaceTable;
    void SetID(DupSpace::ID id)
    {
        m_id=id;
    }

    // After a call to this method, the methods IsAPublisher,
    // Subscriber or Cell will return correct values, and the
    // CreatePublisherIterator, ... methods will also return
    // correct values.
    //{secret}
    void AddDOClassToFilter(DOFilter** ppFilter, DOClassID id, qBool bIncludeDerivedClasses);
    void InitFilter(DOFilter** ppFilter, DupSpace::Role iRole);
    void InitRelationToDOClasses();
    void DestroyRelationToDOClasses();

    DupSpace::ID m_id;
    static DupSpace::ID s_idNext;

    static DuplicationSpace* s_pFirstDupSpace;
    DuplicationSpace* m_pNextDupSpace;

    // An instance can be registered several times, if several
    // facades are used in a process.
    //{secret}
    qInt m_iNbRegistration;

    DOFilter *m_pfCellFilter;
    DOFilter *m_pfPublisherFilter;
    DOFilter *m_pfSubscriberFilter;
    DOFilter *m_pfPSFilter;

    // Match policy.
    //{secret}
    MatchOperationTriggers m_oLocalTriggers;

    qBool m_bCellRequired;
    qUnsignedInt m_uiPSRelation;
    qUnsignedInt m_uiCellRelation;
    qBool m_bSymmetricPSMatch;
    qBool m_bAutomaticRefresh;

    size_t m_uiReservedPublishers;
    size_t m_uiReservedSubscribers;
    size_t m_uiReservedPS;

    static qBool s_bUseSessionSpace;

public:

    // {secret}
    TimeInterval m_tiLastMatchDuration;
    // {secret}
    static TimeInterval s_tiLastRelevanceCheck;
    // {secret}
    // With recent changes to the PerfCounters, to fix a leak, this
    // now crashes.
    //PerfCounter m_pcMatch;

    // {secret}
    SmoothedAverage<TimeInterval> m_tiMatchDuration;
    // {secret}
    static SmoothedAverage<TimeInterval> s_tiRelevanceCheck;
};

END_NAMESPACE_NEX

