﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <OnlineCore/src/Core/SystemComponent.h>

BEGIN_NAMESPACE_NEX

//Group=Host Migration

/*
Description: This class indicates to Net-Z that a client-side Host Migration extension package will be used.
Remarks: If you wish to use the Host Migration extension package to synchronise the Net-Z session master and Rendez-Vous
Gathering host for a given session, then you must register HostMigrationExtension with NetZ using the Register
method of this class and the HostMigrationCallback must be registered with a specific Net-Z-based gathering using the Register
method of its class.
*/

/*!
 @brief    ホストマイグレーション拡張機能を利用するためのクラスです。

 ホストマイグレーション拡張機能を利用すると、 P2P セッション内でセッションホストが移行されると、
 該当するマッチメイクセッションについて、適切な MatchMakingClient::UpdateSessionHost() の呼び出しが行われます。
 これにより、 P2P セッションのホストとゲームサーバーに登録されたホストを簡単に同期させることができます。

 ホストマイグレーション拡張機能の利用手順は以下の通りです。

 @li (1) DDLファイル内で、&#35;use &#139;NetZ/src/Core/HostMigration/HostMigrationExt.ddl&#155;     を記述する。<br>
 @li (2) NetZ オブジェクト生成後、 P2P セッション生成・参加前に @ref HostMigrationExtension::Register() を実行。<br>
 @li (3) 生成した MatchmakeExtensionClient を使って HostMigrationCallback を生成。<br>
 @li (4) P2P セッションの生成・参加<br>
 @li (5) HostMigrationCallback::Register() で参加したギャザリングIDを登録<br>

 ホストマイグレーション拡張機能はネットワークトポロジーがClientServerの以外の場合で使用できます。

 @see HostMigrationCallback
 */
class HostMigrationExtension: public SystemComponent
{
public:
    NEX_DECLARE_SYSCOMPONENT_METHODS(HostMigrationExtension,SystemComponent);

    //group=Methods

    HostMigrationExtension();
    virtual ~HostMigrationExtension();

    //{secret}
    virtual qBool BeginInitialization();
    //{secret}
    virtual qBool EndInitialization();
    //{secret}
    virtual qBool BeginTermination();

    /*
    Description: Registers HostMigrationExtension with Net-Z.
    Returns: true if HostMigrationExtension was successfully registered with Net-Z, false if not or an error occurred.
    Remarks: This method must be called after the NetZ object is created and before a session is either created or joined.
    HostMigrationExtension is automatically unregistered when the NetZ object is destroyed.
    Example:
            #include <Services/Extensions/HostMigration.h>

            // Create NetZ object
            NetZ oNetZ;
            // Register the Host Migration extension
            HostMigrationExtension::Register()
            // Create the Net-Z session
            Session::CreateSession();
    See Also: HostMigrationCallback,
    */

    /*!
     @brief    NetZ にホストマイグレーション拡張機能を登録します。

     本関数は NetZ オブジェクトの生成後、 P2P セッションを生成または参加
     する前に呼び出してください。NetZ オブジェクトが破棄される前に自動的に削除されますので、 Unregister は不要です。
     同一のROMでインターネット通信モードとローカル通信モードを使用し、かつインターネット通信モードでホストマイグレーション拡張機能を使用する場合は、
     ローカル通信モードで  Disable()を実行することで、本関数の呼び出しを省略できます。

     @retval    true    成功
     @retval    false    失敗

     @see Disable()
     */
    static qBool Register();

    /*!
     @brief    NetZ のホストマイグレーション拡張機能を無効にします。

     本関数は NetZ オブジェクトの生成後、 P2P セッションを生成または参加
     同一のROMでインターネット通信モードとローカル通信モードを使用し、かつインターネット通信モードでホストマイグレーション拡張機能を使用する場合は、
     ローカル通信モードで本関数を実行することで、 Register()の呼び出しを省略できます。

     @retval    true    成功
     @retval    false    失敗

     @see Register()
     */
    static qBool Disable();
};

END_NAMESPACE_NEX

