﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once
#include <NetZ/src/Core/DOCore/DOClassTemplate.h> // special case
#include <NetZ/src/Core/DOCore/DataSetTemplate.h> // special case


#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Core/DOCore/DOClass.h>
#include <NetZ/src/Core/DOCore/DuplicatedObject.h>
#include <NetZ/src/Core/SessionClock/SessionClockDDL.h>

#include <OnlineCore/src/Core/SmoothedAverage.h>
#include <OnlineCore/src/Core/LatestMedian.h>
#include <OnlineCore/src/Platform/Core/Time.h>

//n2967 セッションクロックを移動平均の代わりに中央値を使うオプション
#define NEX_SESSIONCLOCK_USE_MEDIAN 1

BEGIN_NAMESPACE_NEX

// Group=Time Management

// This macro is never defined.  It is used only to get around a compilation problem with
// DocJet, which does not like the inheritance to the DOCLASS(SessionClock).
#ifdef FOR_DOC_PURPOSE

/*
Description:  The Session's synchronized time provider.
Remarks:  The time provided by the session clock is synchronised across all stations that belong to
the session. The session clock is automatically set to zero on the session master when Session::CreateSession
is called. Only the session master may reset the session clock via the method SetTime.

Use the session clock to synchronize or compare events on different stations. However,
in order to measure the time it takes to perform a set of operations on a single station
use an instance of SystemClock or Chrono. Note that poorly synchronized SessionClock s will affect various
$PRODUCTNAME$ processes, especially dead reckoning.

The accuracy of the synchronization is determined by the latency and jitter of
the network. Latency is the time lag before a message reaches its destination,
while jitter is the variation in latency. The synchronization algorithm
assumes that the latency between two stations is symmetrical. Therefore, the
non-symmetry of latency between two stations will cause an error in the session
clock on the local station. Non-symmetrical latency can be caused by one Station
performing additional tasks, such as loading files (especially upon a join), which delay synchronization
pulses leading to poorly synchronized SessionClock s.
See also: SystemClock, Chrono, Time
*/

/*!
 * @brief   セッションの同期された時刻を提供するクラスです。
 *
 * SessionClockクラスによって提供される時刻(セッションクロック)は、セッションに参加しているすべての
 * 参加者の間で同期されます。セッションクロックは、セッションマスターが
 * @ref Session::CreateSession 関数を呼び出したタイミングに自動的に0で初期化されます。
 * セッションマスターは@ref SetTime 関数を呼び出すことによってこのクラスが提供する時刻を
 * 変更することが出来ます。
 *
 * セッションクロックは異なった参加者同士でイベントの同期や比較を行うために利用できます。
 * 単一マシン上での時刻取得は@ref SystemClock またはChronoを利用してください。
 * 同期がうまくとれていないセッションクロックは、Net-Zのいくつかのシステム(特に、推測航法)に
 * 悪影響を及ぼします。
 *
 * セッションクロックの誤差はレイテンシ(メッセージが相手に届くまでのラグ)と、
 * ジッタ(レイテンシのばらつき)です。時刻同期を行うためのアルゴリズムは、
 * 二つの参加者間のレイテンシが対称的であることを仮定します。したがって、
 * 対称的でないレイテンシをもつ二つの参加者間でセッションクロックは誤差を生じます。
 * 非対称的なレイテンシは、参加者がそれ以外の追加タスク(たとえば、ファイルをロードするなど)を
 * 実行して同期パルスが遅延することで発生することがあります。
 *
 * セッションクロックについての詳細はNEXプログラミングマニュアルも参照してください。
 * @see     SystemClock, Chrono, Time
 */
class SessionClock : DuplicatedObject
{

#else

class SessionClock : public DOCLASS(SessionClock)
{
    friend void SessionClockTest();
#endif
    //group=Methods
public:

    // API for the class.

    /*
    Summary: Returns the current time of the session clock on the local station
    Remarks:  The local session clock is synchronized with the session master's session clock.
    Any connected station can call this method.
    Returns:  The current time, in milliseconds, of the session clock on the local station.
    */

    /*!
     * @brief   セッションクロックを取得します。
     *
     * この関数を呼び出すと、自分自身が保持しているSessionClockオブジェクトから、
     * セッションクロック情報を取得することが出来ます。
     * この、SessionClockオブジェクトは、セッションマスターのSessionClockオブジェクトと
     * 同期されています。セッションに参加しているすべての参加者はこの関数を呼び出すことが可能です。
     *
     * @return  現在のセッションクロック値がミリ秒単位で返ります。
     */
    static Time GetTime();

    /*
    Description: Sets the time, in milliseconds, of the session clock on the
    session master.
    Remarks: Only the session master can set the time of the session clock.
    Returns: true if the time was successfully set, false if not or an error occurred.
    The error SYSTEMERROR_DC_NOT_CLUSTER_MASTER is signalled if the method is called
    on a station that is not the session master.
    Parameters:
    tNow: The time in milliseconds to which the session clock on the session master will be set.
    */

    /*!
     * @brief   現在のセッションクロック値を設定します。
     *
     * セッションマスターのみが現在のセッションクロック値を設定することが出来ます。
     * セッションマスターであるかどうかは、@ref Session::GetInstance で取得した
     * セッションオブジェクトインスタンスのポインタに対して
     * @ref DuplicatedObject::IsADuplicationMaster 関数を呼び出すことで調べることが出来ます。
     *
     * @param[in]   tNow 設定するセッションクロック値をミリ秒単位で指定します。
     * @retval  true    正常に設定できた場合に返ります。
     * @retval  false   設定に失敗した場合に返ります。
     *                  セッションマスター以外がこの関数を呼び出したときは
     *                  SYSTEMERROR_DC_NOT_CLUSTER_MASTERエラーがシグナルされます。
     */
    static qBool SetTime(Time tNow);


    //n1769:memo この関数は実体が存在していないようなので、関数リファレンスマニュアルには記述されないようにしておきます。
    /*
    Description: Sends a message to all SessionClock duplicas.
    Parameters:
    pMsg: A pointer to the message that will be broadcasted.
    Returns: true if the message was successfully broadcasted, false if not or an error occurred.
    */

    /*!
    @internal
     * @brief   すべてのSessionClockのデュプリカに対してメッセージを送信します。
     *
     * @param[in] pMsg  ブロードキャストされるメッセージオブジェクトへのポインタを指定します。
     *
     * @retval  true    正確にブロードキャストできた場合に返ります。
     * @retval  false   送信に失敗した場合に返ります。
     */
    static qBool BroadcastMessage(Message *pMsg);

    // group=Fine Tuning Methods

    /*
    Summary:  Sets the initial minimum number of synchronization pulses sent to synchronize
    the SessionClock before leaving the Session::JoinSession method.
    Remarks: The default number of synchronization pulses is s_uiDefaultMinimumNbSyncPulse.
    This means that at least s_uiDefaultMinimumNbSyncPulse messages will be sent between the
    host and session master to synchronize the clock before we leave the Session::JoinSession method.
    Increase the value of this parameter for a more precise time synchronization, although the
    synchronization will take a longer time to complete and use more bandwidth.
    Parameters:
    uiMinSyncPulse:  The minimum number of synchronization pulses that will be sent
    before we leave Session::JoinSession. uiMinSyncPulse can take any value greater than 0.
    Returns:  true if the uiMinSyncPulse was successfully set, false if not or an error occurred.
    The error SYSTEMERROR_GEN_OUT_OF_RANGE is signalled if uiMinSyncPulse is not greater than 0.
    */

    /*!
     * @internal
     * @brief   初期同期パルス回数を設定します。
     *
     * この関数によって、セッション参加時(@ref Session::JoinSession 関数の終了時)に、
     * セッションマスターと参加者の間で最低何回クロックの同期パルスを送信するかを設定します。
     * 多くのパルス数を設定すればその分だけセッションクロックの正確性が向上しますが、
     * 多くの帯域を圧迫するデメリットもあります。
     *
     * @param[in]   uiMinSyncPulse  初期同期パルス回数を指定します。
     * @retval  true    指定されたパルス回数が正常に設定されたときに返ります。
     * @retval  false   設定に失敗した場合に返ります。
     *                  @a uiMinSyncPlusが0よりも大きくない場合にはSYSTEMERROR_GEN_OUT_OF_RANGEがシグナルされます。
     * @see     s_uiDefaultMinimumNbSyncPulse
     */
    static qBool SetMinimumNbSyncPulse(size_t uiMinSyncPulse);

    static size_t GetMinimumNbSyncPulse()
    {
        return s_uiSyncPulseSampleSize;
    }

    /*
    Summary:  Sets the delay between consecutive pulses sent to keep the session clock
    synchronized after session creation.
    Remarks: The default value is s_tiDefaultSyncPulseDelay. You can set the tiPulseDelay to any value
    greater than zero or to INFINITE. Setting the minimum synchronization pulse delay to INFINITE
    means that the SessionClock will synchronize initially using the minimum number of synchronization
    pulses at Session::JoinSession, but will not send any pulses after that to prevent drift. Decrease
    the value of this parameter to prevent drift, decrease the time taken to synchronize stations, and
    to improve the accuracy of synchronization, at the cost of increased bandwidth usage.

    This method can be called on any station and also at run time. Therefore, it is possible to set a certain
    initial pulse delay and then to reduce it once the initial clock synchronization is complete, for example.
    Parameters:
    tiPulseDelay:  The amount of delay, in milliseconds, between two consecutive pulses
    sent. tiPulseDelay can take any value greater than 0.
    Returns:  true if the tiPulseDelay was successfully set, false if not or an error occurred.
    The error SYSTEMERROR_GEN_OUT_OF_RANGE is signalled if tiPulseDelay is not greater than 0.
    */

    /*!
     * @internal
     * @brief   セッションクロック同期パルス送信間隔を設定します。
     *
     * この関数を利用することで、@ref Session::JoinSession 関数によってセッションに参加した後、
     * どの程度の周期でセッションクロックの同期パルスを送信するかを設定します。
     * tiPlusDelayにINFINITEを指定すると、セッション参加後はドリフトを回避するための
     * 同期パルスを送信しなくなります。
     *
     * 同期パルス送信間隔を短くすることでドリフトを抑制し、
     * 正確なセッションクロックを保つことが可能になりますが、帯域を多く消費することに注意してください。
     * 逆に、長くすると帯域はあまり消費しなくなりますが正確でなくなる可能性があります。
     * セッションクロックが正確でなくなると推測航法などに悪影響が出る場合があります。
     *
     * この関数は初期化時だけでなく実行時でも呼び出すことが出来ます。
     * したがって、セッションクロックが確かな数値を保持していると感じた場合に
     * 同期間隔を動的に長めに設定しなおすということが可能です。
     *
     * @param[in]   tiPulseDelay 送信間隔を指定します。0からINFINITEの間の時間を指定することが出来ます。
     * @retval  true    指定されたパルス間隔が正常に設定されたときに返ります。
     * @retval  false   設定に失敗した場合に返ります。
     *                  @a uiMinSyncPlusが0よりも大きくない場合にはSYSTEMERROR_GEN_OUT_OF_RANGEがシグナルされます。
     * @see     s_tiDefaultSyncPulseDelay
     */
    static qBool SetSyncPulseDelay(TimeInterval tiPulseDelay);

    /*
    Description: Signals whether or not to take station induced latency into account
    when performing time synchronization calculations.
    Parameters:
    bAdjust: If false (the default), the time synchronization calculations will not take station induced
    latency into account.
    Remarks: AdjustForStationLatency affects how this Station synchronizes its SessionClock
    with the session master. When bAdjust is set to true, this Station will take into
    account the values of the input and output latency (set via Station::SetLatencyInfo) of
    this Station and of the session master in order to more accurately calculate the session
    clock time on this Station. For the majority of cases it is not necessary to adjust for station
    latency as the input and output latency of one station's connection will often be similar to
    that of the session master.
    See Also: Station::SetLatencyInfo
    */
    static void AdjustForStationLatency(qBool bAdjust)
    {
        s_bAdjustForStationLatency=bAdjust;
    }

    // {secret}
    static void SendSyncRequest();

    /*
    Summary:  Sets the size (number) of pulses used to synchronize the local session
    clock with that of the session master.
    Remarks: The default is s_uiDefaultSyncPulseSampleSize. The session clock on the local
    station is synchronized by sending a series of pulses to the session master at a set time
    interval. This method sets the size (in number of pulses) of each sample that will be sent.
    Increase the value of this parameter to improve the accuracy of synchronization, at the cost of greater
    bandwidth usage and an increase in the time taken to synchronize stations.
    Parameters:
    uiSyncPulseSampleSize: The number of pulses sent per sample. s_uiDefaultSyncPulseSampleSize can take
    any value greater than 0.
    Returns:  true if the uiSyncPulseSampleSize was successfully set, false if not or an error occurred.
    The error SYSTEMERROR_GEN_OUT_OF_RANGE is signalled if uiSyncPulseSampleSize is not greater than 0.
    */

    /*!
    @internal
    @brief   ローカルステーションとセッションマスターで同期を取るときに利用するパルス数を指定します。

    数を増やすと、高い精度で同期できますが、より多くの帯域幅の使用が必要です。
    デフォルト値は、  s_uiDefaultMinimumNbSyncPulseです。

    @param[in]    uiSyncPulseSampleSize パルス数。0より大きい値である必要があります。

    @retval true 設定が成功した
    @retval false 設定が失敗した

    */
    static qBool SetSyncPulseSampleSize(size_t uiSyncPulseSampleSize);

    // {secret}
    static void Enable();
    // {secret}
    static void Disable();

    //group=Default Constants

    // Summary: The default value for the minimum number of initial synchronization pulses is 5.
    /*!
    * @brief   初期同期パルス回数のデフォルト値(5)です。
     */
    static const size_t s_uiDefaultMinimumNbSyncPulse;

    // Summary: The default value for the pulse delay to keep the SessionClock synchronized is
    // 2000 milliseconds ( = 2 seconds).
    /*!
    * @brief   同期パルス送信間隔のデフォルト値(2000msec)です。
     */
    static const TimeInterval s_tiDefaultSyncPulseDelay;

    // Summary: The default value for the sample size of the synchronization pulse is 10 pulses.
    static const size_t s_uiDefaultSyncPulseSampleSize;


#ifdef NEX_SESSIONCLOCK_USE_MEDIAN
    /*!
     * @brief   セッションクロック同期計測結果の中央値計算にどのくらい過去の値を使うか設定します。
     *
     * @param[in]   s 計算に用いる過去の値数を設定します。
     */
    static void SetDefaultMedianSize(size_t s)
    {
        SYSTEMCHECK(s > 0);
        s_uiMedianSize = s;
    }

    /*!
     * @brief   セッションクロック同期計測結果の中央値計算にどのくらい過去の値を使うかを取得します。
     *
     * @return 計算に用いる過去の値数を取得します。
     */

    static size_t GetDefaultMedianSize()
    {
        return s_uiMedianSize;
    }
#endif

public:
    // Interface to the class - as seen by the system

    /*
    Remarks: A SessionClock object should never be directly instantiated. Instead, it is created by
    $PRODUCTNAME$ when a session is created and initialized to 0 ms.
    */

    /*!
     * @brief   このコンストラクタは呼び出さないでください。
     *
     * SessionClockオブジェクトはセッションが作成され、
     * 0ミリ秒として初期化されるときにシステムによって自動的に作成されます。
     */
    SessionClock();

    //{Secret}
    static qBool IsInitializedOrDisabled();

    // {secret}
    virtual void OperationEnd(DOOperation* pOperation);


    /*
    Remarks: A SessionClock object should never be directly destroyed. It is destroyed by $PRODUCTNAME$
    when the session terminates.
    */

    /*!
     * @brief   このデストラクタは呼び出さないでください。
     *
     * SessionClockオブジェクトはシステムによってセッション終了時に自動的に破棄されます。
     */
    virtual ~SessionClock();

    // {Secret}
    void GetFrequency(Time* pliFrequency);

    // {Secret}
    void StartTheTime();

    // {Secret}
    static void RestoreRealTimeRatio();

    // {Secret}
    void _Sync(Time liCurrentTime);


    // {Secret}
    qBool SetTimeOnInstance(Time liCurrentTime);

    // {Secret}
    Time _GetTime();
    //{secret}
    void AdjustTime(LargeTimeInterval tiDiff);
    //{secret}
    void SyncRequest(Time uiTimeStamp);
    //{secret}
    void SyncResponse(Time uiTimeStamp, Time uiCurrentTime, TimeInterval iServerDispatchTime);

private:
    // {Secret}
    Time _GetTimeImpl();

    class JobSendPulse;
#if !defined(DOT_NET_COMPILER)
    /* The following must be specified */
    friend class JobSendPulse;
#endif
    void StartSyncPulse();
    void StopSyncPulse();


    // {Secret}
    qBool _SetRealTimeRatio(qReal dRatio);

    qBool _SendSyncRequest();
    qBool _Pause();
    qBool _Resume();

public:

    // {secret}
    static void ApplyDefaultsIfRequired();
    // {secret}
    static void ApplyDefaults();
    // TODO: Should use a ref.
    //{secret}
    static SessionClock* GetInstance();

private:
    // {Secret}
#ifdef NEX_SESSIONCLOCK_USE_MEDIAN
    LatestMedian<TimeInterval> m_tiOffsetStatsMedian;
    SmoothedAverage<TimeInterval> m_tiOffsetStats;
    static const size_t s_uiDefaultMedianSize;
#else
    SmoothedAverage<TimeInterval> m_tiOffsetStats;
#endif
    TimeInterval m_tiOffset;
    qBool m_bSet;
    qBool m_bPaused;

    qReal m_dRealTimeRatio;
    qReal m_dRealTimeRatioBeforePause;

    static  qBool s_bDefaultsApplied;

private:
    qUnsignedInt m_uiTimeSyncReceived;
    Time m_tLastSyncPulse;

private:
    // Fine tuning values.
    // {secret}
    static TimeInterval s_tiPulseDelay;
    static size_t s_uiMinNbSyncPulse;
    static size_t s_uiSyncPulseSampleSize;
    static qBool s_bAdjustForStationLatency;
    static qBool s_bEnabled;
#ifdef NEX_SESSIONCLOCK_USE_MEDIAN
    static size_t s_uiMedianSize;
#endif

public:
    // {Secret}
    void FakeTheTime();//For tests

public :
    // {secret}
    static qBool s_bSimulateDelayOnReception;

private :
    qBool m_bInitialized;

    // {secret}
    qBool m_boIsSyncing;

    Time m_tiLastSyncRequest;

    JobSendPulse *m_pJobSendPulse;

};

// For the TraceOutput - to trace the session time.
// {secret}
extern qUnsignedInt32 GetSessionTime();

END_NAMESPACE_NEX

