﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>
#include <NetZ/src/Extensions/Adapter/AdapterEvent.h>
#include <OnlineCore/src/Core/DispatchQueue.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>

BEGIN_NAMESPACE_NEX

//group=Adapters

class DOOperation;
class Adapter;

/*
Description: Deprecated. Dispatches system events that relate to an Adapter.
Remarks: This class has been deprecated. Events can now be caught by a system callback or a DO callback for a specific object. See System Operations
and information on Duplicated Objects for more information. This class is used to control when system events that relate to an Adapter are processed.
See Also: Net-Z Integration, System Operations, Duplicated Object Overview
*/
// {noAutoLink}
class AdapterEventDispatcher: public DispatchQueue<AdapterEvent>
{
    // group=Methods

public:

    /*
    Description: An AdapterEventDispatcher object is instantiated by $PRODUCTNAME$ before the session is created.
    Remarks: An AdapterEventDispatcher object should never be directly instantiated. Instead, it is created
    by $PRODUCTNAME$ prior to session creation.
    */
    AdapterEventDispatcher();

    /*
    Description: The AdapterEventDispatcher object is destroyed by $PRODUCTNAME$ when the session terminates.
    Remarks: An AdapterEventDispatcher object should never be directly destroyed. It is destroyed by
    $PRODUCTNAME$ when the session terminates.
    */
    virtual ~AdapterEventDispatcher();

    /*
    Description: Ensures that AdapterEvents are processed when they occur rather than when queued.
    Remarks: The methods IsProcessingEventsDirectly and IsQueuingEvents can be used to determine if AdapterEvents
    are processed directly or queued.
    See Also: QueueEvents, IsProcessingEventsDirectly, IsQueuingEvents
    */
    void ProcessEventsDirectly();

    /*
    Description: Ensures that AdapterEvents are queued rather than processed when they occur.
    Remarks: The methods IsProcessingEventsDirectly and IsQueuingEvents can be used to determine if AdapterEvents
    are processed directly or queued. When AdapterEvents are queued, the events are dispatched via the dispatch
    methods of the DispatchQueue class. Note that contrary to when events are processed directly (ProcessEventsDirectly),
    if events are queued the Adapter may be created after DuplicatedObject::OperationEnd is called at the end of
    the AddToStoreOperation. This is because even though the event is queued before the Operation ends, it may be
    dispatched after the end of the Operation.
    See Also: ProcessEventsDirectly, IsProcessingEventsDirectly, IsQueuingEvents, DispatchQueue::Dispatch,
    DispatchQueue::DispatchAll, DispatchQueue::DispatchFirstItem
    */
    void QueueEvents();

    /*
    Description: Returns whether or not the AdapterEventDispatcher is processing events directly.
    Returns: true if the AdapterEventDispatcher is processing events directly, false if it is queuing them.
    Remarks: AdapterEvents may be either processed directly as they occur or queued.
    See Also: ProcessEventsDirectly, QueueEvents
    */
    qBool IsProcessingEventsDirectly()
    {
        return m_bProcessEventsDirectly;
    }

    /*
    Description: Returns whether or not the AdapterEventDispatcher is queuing events.
    Returns: true if the AdapterEventDispatcher is queuing events, false if it is processing them directly.
    Remarks: AdapterEvents may be either processed directly as they occur or queued.
    See Also: ProcessEventsDirectly, QueueEvents
    */
    qBool IsQueuingEvents()
    {
        return !IsProcessingEventsDirectly();
    }

protected:

    /*
    Description:
    Remarks:  Callbacks executed in the system thread context.  When this
    method is called, the event can be ignored, queued or processed.
    See Also: ProcessAdapterEvent
    */
    //{secret}
    void FilterAdapterEvent(AdapterEvent* pEvent);

    /*
    Description:
    Remarks: Callbacks for the events queued and dispatched in a
    different thread context (typically the user's thread
    context), or those processed directly in FilterAdapterEvent.
    See Also: FilterAdapterEvent
    */
    //{secret}
    void ProcessAdapterEvent(AdapterEvent* pEvent);

public:

    /* System API.  Methods called by the system. */

    // {secret}
    void DispatchDOOperation(DOOperation* pOperation, Adapter** ppAdapter, qUnsignedInt uiAdapterIndex);

    /*
    Description: Returns a pointer to the AdapterEventDispatcher instance.
    Returns: A reference to the AdapterEventDispatcher object.
    Remarks: Use the reference returned by this method to control how Adapter events are dispatched.
    For example, ensure that Adapter events are processed when they occur as follows:

            AdapterEventDispatcher::GetInstance()->ProcessEventsDirectly();

    */
    static AdapterEventDispatcher* GetInstance();

private:
    // virtual callbacks.
    //{secret}
    virtual void ProcessItem(AdapterEvent* pEvent);

private:
    qBool m_bProcessEventsDirectly;

};

END_NAMESPACE_NEX

