﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

BEGIN_NAMESPACE_NEX

class DuplicatedObject;
class Station;
class DOOperation;
class DatasetOperation;

//Group=Custom Dead Reckoning

/*
Description: (Template Class) Instantiates the required custom dead reckoning methods for a dataset.
Remarks: For each dataset that specifies a custom dead reckoning dataset property, this template class instantiates
the various required methods. These methods are called by $PRODUCTNAME$ when required however, they can also be called
directly in order to access your MyOwnModel class.
See Also: The Marbles sample
*/
template<class DS, class MODEL, class MODELPOLICY >
class DeadReckoningDSDecl: public DS
{
public:
    //group=Methods

    DeadReckoningDSDecl() {}
    // Inherits members from DS.
    // Inherits AddSourceTo from DS.

    // This method is called when a duplica receives an update from its master
    // {secret}
    void ExtractFrom(Message *pMessage)
    {
        DS oTmp;
        oTmp.ExtractFrom(pMessage);
        MODEL* pMyOwnModel = m_oModelPolicy.GetLocalDeadReckoningModel();
        pMyOwnModel->UpdateModelOnDuplica(oTmp);
    }

    // This method is called when DuplicatedObject::Refresh is called
    // {secret}
    qBool Refresh(const Time& tRefreshTime)
    {
        MODEL* pMyOwnModel = m_oModelPolicy.GetLocalDeadReckoningModel();
        SYSTEMCHECK(pMyOwnModel!=NULL);

        DS* pMyselfAsDS = static_cast<DS*>(this);
        pMyOwnModel->ComputeValue(pMyselfAsDS, tRefreshTime);
        return true;
    }

    /*
    Description: Returns a Duplication master's local instance of one of its Duplica's dead reckoning models.
    Parameters:
    pTargetStation: A pointer to the remote Station for which we wish to return the dead reckoning model.
    Returns: A pointer to the Duplication master's local instance of the dead reckoning model, that you defined as described
    in the MyOwnModel class, which is associated to the Duplica that resides on pTargetStation, or a NULL pointer
    if no model is found.
    Remarks: This method is called when DuplicatedObject::Update is called and an update is required, in order to
    access a Duplication master's local instance of a given Duplica's dead reckoning model. If this method returns a NULL
    pointer then an update will not be sent to the remote Station.

    If you wish to indicate continuity breaks, one way to implement them is to modify the implementation of this
    method so that when the method is called and a continuity break is indicated, then the model's behaviour changes.
    For example, you could force updates to be sent (IsAccurateEnough returns false) for the next three Update calls.
    Alternatively, you could implement continuity breaks as part of the dataset. Another way to use this method is to
    set the error threshold between the Duplication master's dataset values and its Duplica's via this method's implementation.
    See Also: GetLocalDeadReckoningModel, the Marbles sample
    */
    MODEL* GetRemoteDeadReckoningModel(const Station* pTargetStation)
    {
        return m_oModelPolicy.GetRemoteDeadReckoningModel(pTargetStation);
    }

    /*
    Description: Returns a Duplica's local instance of the dead reckoning model.
    Returns: A pointer to the Duplica's local instance of the dead reckoning model that you defined as described
    in the MyOwnModel class.
    Remarks: This method is called when DuplicatedObject::Refresh is called and an update has been received,
    in order to access a given Duplica's local instance of the dead reckoning model.

    If you wish to implement
    local corrections, you could do so by modifying the implementation of this method so that when the method is called
    corrections are made to the duplica's local model to ensure the object does not pass through walls, floors, and the like.
    See Also: GetRemoteDeadReckoningModel, the Marbles sample
    */
    MODEL* GetLocalDeadReckoningModel()
    {
        return m_oModelPolicy.GetLocalDeadReckoningModel();
    }

    /*
    Description: Returns the number of dead reckoning model class instances that are currently
    instantiated by the Duplication master.
    Returns: The number of instances of the given dead reckoning model class that are
    currently instantiated by the Duplication master.
    Remarks: If a SharedModelPolicy is used then the number of models will be equal to one, and if a
    PerStationModelPolicy is used then the number of models will be equal to the number of duplicas as
    one model is instantiated for each Duplica.
    */
    qUnsignedInt32 GetNbModels() const
    {
        return m_oModelPolicy.GetNbModels();
    }

    /*
    Description: Returns the dataset's model policy.
    Returns: A reference to the model policy.
    Remarks: Provides access to the model policy and makes it possible to provide information to the model policy which can be
    used, for example, when the policy creates, destroys, or changes models and their association with duplicas.
    */
    MODELPOLICY & GetDeadReckoningModelPolicy()
    {
        return m_oModelPolicy;
    }

    //{secret}
    void OperationEnd(DOOperation* pOperation)
    {
        DS::OperationEnd(pOperation);
        // When a duplica is deleted, if necessary we remove its dead reckoning model on its master's station
        if (pOperation->GetType()==Operation::ChangeDupSet)
        {
            ChangeDupSetOperation* pChangeDupSetOp=ChangeDupSetOperation::DynamicCast(pOperation);
            if (pChangeDupSetOp->IsADuplicaRemoval())
            {
                m_oModelPolicy.ClearRemoteDeadReckoningModel(pChangeDupSetOp->GetTargetStationPointer());
            }
        }
    }

private:
    // No copy ctor nor assignement.

    DeadReckoningDSDecl(DeadReckoningDSDecl<DS,MODEL,MODELPOLICY>&);
    DeadReckoningDSDecl<DS,MODEL,MODELPOLICY>* operator=(DeadReckoningDSDecl<DS,MODEL,MODELPOLICY>&);

private:
    MODELPOLICY m_oModelPolicy;
};


END_NAMESPACE_NEX

