﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

BEGIN_NAMESPACE_NEX

//Group=Distance-based Migration

/*
Description: This class implements distanced-based duplicated object migration.
Remarks: This extension implements migration of NPC objects according to the proximity of observer
objects as follows. The distance between the observer on the local station and the Duplication
master of any NPC objects ( /i.e./ NPC objects controlled on the local station) is calculated.
If this distance is less than the minimum distance then no further calculations are made and the
NPC object does not migrate. However, if the distance between the observer on the local station
and the NPC object is greater than the minimum distance, then the distance between the NPC object
and the closest observer of a remote station is calculated. If the distance between the NPC object
and the remote observer is less than that between the NPC object and the local observer and that the
difference between these two distances is greater than the neutral zone width, then the NPC object
will migrate to the remote station. If not it will stay on the local station.
See Also: Object Migration Overview, Station::RegisterObserver
*/
class DistanceBasedMigrationPolicy
{
public:
    //Group=Methods:

    /*
    Description: Enables distance-based migration of duplicated objects.
    Remarks: When distance-based migration is enabled NPC duplicated objects will migrate
    according to the proximity of observer objects. If you Enable distance-based migration
    then you must Disable it before you destroy the $PRODUCTNAME$ object.
    See Also: Disable, Station::RegisterObserver
    */
    static void Enable();

    /*
    Description: Disables distance-based migration of duplicated objects.
    Remarks: Once distance-based migration is disabled, NPC duplicated objects will migrate
    according to whatever other migration scheme, if any, is in place. If you Enable distance-based
    migration then you must Disable it by calling this method before you destroy the $PRODUCTNAME$ object.
    See Also: Enable
    */
    static void Disable();

    /*
    Description: Sets the minimum migration distance of objects.
    Parameters:
    dDistance: The minimum distance from the observer on the local station to a given NPC
    Duplication master object at which the NPC object will not migrate to another station.
    The default value is 5 units.

    Remarks: If the Duplication master of a given NPC object is closer to the observer on the
    local station than the minimum distance, the NPC object will not migrate, even if an
    observer object from another station moves closer to the NPC object. Setting a minimum
    migration distance ensures that a NPC object will not migrate just before it collides
    with another object.

    The units of dDistance are arbitrary. Therefore, when distance-based migration
    is implemented, the units used will be the same as the units of distance defined in your game.
    For instance, if in your game you describe distance using metres, then the minimum
    migration distance will be calculated in metres.

    Typically this method should be called after the $PRODUCTNAME$ object is created and before Enable
    is called however, it may be called at any time as the minimum migration distance can be set dynamically.
    See Also: GetMinMigrationDistance
    */
    static void SetMinMigrationDistance( qReal dDistance );

    /*
    Description: Sets the neutral zone width for the migration of objects.
    Parameters:
    dWidth: The width of a neutral zone located halfway between the observer object on the
    local station and the closest observer object of a remote station. If the Duplication master
    of a NPC object is located within this zone, it will not migrate to the remote station.
    The default value is 5 units.

    Remarks: Setting the neutral zone prevents objects continually migrating between stations when
    the NPC object is roughly equal distances from two observer objects.

    The units of dWidth are arbitrary. Therefore, when distance-based migration
    is implemented, the units used will be the same as the units of distance defined in your game.
    For instance, if in your game you describe distance using metres, then the neutral zone
    width will be calculated in metres.

    Typically this method should be called after the $PRODUCTNAME$ object is created and before Enable
    is called however, it may be called at any time as the neutral zone width can be set dynamically.
    See Also: GetNeutralZoneWidth
    */
    static void SetNeutralZoneWidth( qReal dWidth );

    /*
    Description: Returns the minimum migration distance.
    Returns: The minimum migration distance in arbitary units, which is set via SetMinMigrationDistance.
    See Also: SetMinMigrationDistance
    */
    static qReal GetMinMigrationDistance()
    {
        return s_dMinMigrationDistance;
    }

    /*
    Description: Returns the neutral zone width.
    Returns: The neutral zone width in arbitary units, which is set via SetNeutralZoneWidth.
    See Also: SetNeutralZoneWidth
    */
    static qReal GetNeutralZoneWidth()
    {
        return s_dNeutralZoneWidth;
    }

private:
    static qReal s_dMinMigrationDistance;
    static qReal s_dNeutralZoneWidth;
};

END_NAMESPACE_NEX

