﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <NetZ/src/Core/DOCore/ObjDupDecl.h>

BEGIN_NAMESPACE_NEX


// Group = Dead Reckoning

#define DEFAULT_PARAMETER       QREAL(-1.0)

/*
Description: Used in ErrorToleranceFunction::SetParameters() to indicate that the error tolerance has no maximum value.
See Also: ErrorToleranceFunction::SetParameters
*/
#define UNLIMITED_ERROR         QREAL(-1.0)


/*
 Description:  This class is used to compute the maximum extrapolation error tolerated
 before the datasets of a Duplica will be updated.
 Remarks: The error tolerance function is used to calculate the maximum extrapolation
 error tolerated before the datasets of a Duplica will be updated.
 The extrapolation error is defined as the difference between the current
 dataset values of an object, /i.e./ that of the Duplication master, and the
 extrapolated dataset values of the object, /i.e./ that of a Duplica. The
 error tolerance may be constant or a function of the distance between
 the observer and an object, depending on the parameters chosen, and may be freely reset at run time. The observer
 is set via the method Station::RegisterObserver. For example, the error tolerance function can be set such that a larger
 error is tolerated for objects displayed a long distance from the
 observer, /i.e./ those objects in the background of a scene, and a small
 error is tolerated for those objects closer to the observer, /i.e./
 those in the foreground. When this is the case, extrapolation of the datasets
 is more precise and therefore, a greater portion of the bandwidth is used for
 objects that are close to the observer. For objects that appear further away from
 the observer, extrapolation of the datasets is less precise and proportionally
 less bandwidth is used. $PRODUCTNAME$ does not impose a definition of close or far. It is
 up to you to implement the method DuplicatedObject::ComputeDistance, which calculates
 the distance between two duplicated objects.
*/

/*!
 * @brief   推測航法におけるエラー許容のパラメータを管理するクラスです。
 *
 * DataSet::GetErrorToleranceFunction() でデータセットごとに取得できます。
 *
 * 推測航法機能とは、他のプレイヤーの動き（位置や向きなど）を自然に見えるように補間する機能です。
 * DDL定義のデータセットで extrapolation_filter を指定することで使用可能になります。
 *
 * 詳細はプログラミングマニュアルを参照してください。
 */
class ErrorToleranceFunction: public RootObject
{
    //group=Methods
public:

    /*
    Remarks: An ErrorToleranceFunction object should never be directly instantiated. Instead, it is created by
    $PRODUCTNAME$ prior to session creation.
    See Also: DataSet::GetErrorToleranceFunction
    */
    ErrorToleranceFunction();

    /*
    Remarks: An ErrorToleranceFunction object should never be directly destroyed. It is destroyed
    by $PRODUCTNAME$ when the session terminates.
    */
    virtual ~ErrorToleranceFunction();

    /*
    Summary: Sets the parameters of the error tolerance function.
    Parameters:
    dConstantFactor: Constant factor in the error tolerance function, which is
    also equal to the minimum value of the error tolerance.
    dLinearFactor: Linear factor in the error tolerance function.
    dQuadraticFactor: Quadratic factor in the error tolerance function.
    dMaximumError: Maximum value of the error tolerance. This optional
    parameter has a default value of UNLIMITED_ERROR.
    Remarks: The error tolerance function is used by $PRODUCTNAME$ to determine if the
    extrapolation error on a dataset is acceptable or not. If the
    extrapolation error exceeds the error tolerance the dataset of a Duplica
    will be updated from the Duplication master.

    The error tolerance may be constant or a function of the distance between
    the object and a user-defined observer, which means that the tolerated
    error will change according to the location of an object.

    The parameters of the error tolerance function are used to compute the
    error tolerance using a formula that corresponds to the following
    pseudo code:

            qReal dError=dConstantFactor +
                          dLinearFactor * dDistance +
                          dQuadraticFactor * dDistance * dDistance;
            if (dMaximumError!= UNLIMITED_ERROR && dError>dMaximumError) {
                return dMaximumError;
            } else {
                return dError;
            }


    The parameters of the error tolerance function are game
    specific. Their selection involves a trade-off between bandwidth used
    and precision of the dataset variables extrapolation. A higher error
    tolerance leads to lower bandwidth usage.

    $PRODUCTNAME$ defines the following defaults:
    dConstantFactor=dLinearFactor=dQuadraticFactor=0 and
    dMaximumError=UNLIMITED_ERROR.

    See Also: DDL File Syntax, SetConstantError, SetLinearError, SetQuadraticError
    */

    /*!
     * @brief   許容エラーを計算するパラメータを設定します。
     *
     * より簡単にパラメータを設定できるように以下の関数が用意されています。通常は以下の関数の使用で問題ありません。
     *
     * @li ErrorToleranceFunction::SetConstantError()
     * @li ErrorToleranceFunction::SetLinearError()
     * @li ErrorToleranceFunction::SetQuadraticError()
     *
     * 詳細はプログラミングマニュアルを参照してください。
     *
     * @param[in]   dConstantFactor 0次係数のパラメータです。
     * @param[in]   dLinearFactor   1次係数のパラメータです。
     * @param[in]   dQuadraticFactor    2次係数のパラメータです。
     * @param[in]   dMaximumError   エラーの最大値を指定します。これを超えるものは指定した値にクリップされます。
     *
     * @see ErrorToleranceFunction::SetConstantError, ErrorToleranceFunction::SetLinearError, ErrorToleranceFunction::SetQuadraticError
     */
    void SetParameters(qReal dConstantFactor,
                       qReal dLinearFactor,
                       qReal dQuadraticFactor,
                       qReal dMaximumError=UNLIMITED_ERROR);

    /*
    Summary: Sets the default distance used to calculate the error tolerance.
    Remarks: This distance is used if the distance between two duplicated objects is unknown
    (UNKNOWN_DISTANCE) or if the observer on the station being updated has not been defined.
    The initial value for the default distance is 0.
    Parameters:
    dDefaultDistance: The default distance $PRODUCTNAME$ uses to calculate the error tolerance.
    The default value is 0.

    See also: ValueAt, UNKNOWN_DISTANCE, UNLIMITED_ERROR, DuplicatedObject::ComputeDistance,
    Dead Reckoning, Dead Reckoning Overview, DataSet
    */

    /*!
     * @brief   距離ベースの推定エラーを使用する際の、デフォルトの距離を設定します。推測航法で利用します。
     *
     * 距離ベースの推測航法を利用する際に、
     * DuplicatedObject::ComputeDistance() をアプリケーション側の複製オブジェクトでオーバーライドしなかった場合の
     * デフォルトの距離を設定します。
     *
     * @param[in]   dDefaultDistance    デフォルトの距離。本関数のコール前は0が設定されています。
     */
    void SetDefaultDistance(qReal dDefaultDistance);

    /*
    Summary: Sets a constant error tolerance.
    Parameters:
    dError: The constant tolerated error.
    Remarks:  This methods calls SetParameters and is equivalent to the call SetParameters(dError, 0, 0).
    For example, a constance error of 0.1 units on the Position dataset could be set as follows:

            Position::GetErrorToleranceFunction()->SetConstantError(0.1);

    See Also: SetParameters, SetLinearError, SetQuadraticError, Dead Reckoning Overview,
    Dead Reckoning
    */

    /*!
     * @brief   距離に関係なく一定のエラー許容を設定します。
     *
     * @param[in]   dError  エラー許容の値です。
     *
     * ErrorToleranceFunction::SetParameters(dError, 0, 0)
     * と同じ動作です。
     *
     * 詳細はプログラミングマニュアルを参照してください。
     *
     * @see ErrorToleranceFunction::SetParameters, ErrorToleranceFunction::SetLinearError, ErrorToleranceFunction::SetQuadraticError
     */
    void SetConstantError(qReal dError);

    /*
    Summary:  Sets a linear relationship between the error tolerance and the distance
    between the observer and object.
    Parameters:
    dMinimumValue: The minimum tolerated error.
    dMaximumValue: The maximum tolerated error.
    dDistance: The total distance over which the tolerated error is to vary linearly.
    Remarks: These three parameters determine the minimum and maximum tolerated error as well as
    the slope of the linear relation between the error tolerance and the distance. The slope is
    equal to (dMaximumValue- dMinimumValue)/dDistance.

    This methods calls SetParameters and is equivalent to the call
    SetParameters(dMinimumValue, (dMaximumValue- dMinimumValue)/dDistance, 0, dMaximumValue)

    Returns: true if the error tolerance is set successfully, false if not or an error occurred.
    The error SYSTEMERROR_OBJDUP_INVALID_PARAMETER is signalled if dDistance is less than or equal to 0
    or if dMaximumValue is less than dMinimumValue.
    See Also:  SetParameters, SetConstantError, SetQuadraticError, DuplicatedObject::ComputeDistance,
    Dead Reckoning, Dead Reckoning Overview.
    */

    /*!
     * @brief   エラー許容を決定するパラメータを、エラー許容と距離をリニアな一次関係として設定します。
     *
     * SetParameters(dMinimumValue, (dMaximumValue-dMinimumValue)/dDistance, 0, dMaximumValue);
     * と同じ動作です。
     *
     * 詳細はプログラミングマニュアルを参照してください。
     *
     * @param[in]   dMinimumValue   最小許容エラーを指定します。
     * @param[in]   dMaximumValue   最大許容エラーを指定します。
     * @param[in]   dDistance       最大許容エラーに到達する際の距離を指定します。
     *
     * @return  設定が成功したかどうかを返します。<br>
     *          dDistanceが負の値である時、または dMinimumValue が dMaximumValue より大きいときに false が返り、
     *          SYSTEMERROR_OBJDUP_INVALID_PARAMETER がシグナルされます。
     *
     * @see ErrorToleranceFunction::SetParameters, ErrorToleranceFunction::SetConstantError, ErrorToleranceFunction::SetQuadraticError
     */
    qBool SetLinearError(qReal dMinimumValue, qReal dMaximumValue, qReal dDistance);

    /*
    Summary:  Sets a quadratic relationship between the error tolerance and the distance
    between the observer and object.
    Parameters:
    dMinimumValue: The minimum tolerated error.
    dMaximumValue: The maximum tolerated error.
    dDistance: The total distance over which the tolerated error is to vary in a quadratic manner.
    Returns: true if the error tolerance is set successfully, false if not or an error occurred.
    The error SYSTEMERROR_OBJDUP_INVALID_PARAMETER is signalled if dDistance is less than or equal to 0
    or if dMaximumValue is less than dMinimumValue.
    Remarks: These three parameters determine the minimum and maximum tolerated error, as well as
    the quadratic equation of the relation between the error tolerance and the distance.
    The quadratic coefficient is equal to (dMaximumValue- dMinimumValue)/(dDistance*dDistance).

    This methods calls SetParameters and is equivalent to the call
    SetParameters(dMinimumValue, 0, (dMaximumValue- dMinimumValue)/(dDistance*dDistance), dMaximumValue)

    For example, a quadratic error could be set on the Position dataset as follows:

            Position::GetErrorToleranceFunction()-&gt;SetQuadraticError(0.01, 1, 10);

    See Also: SetParameters, SetConstantError, SetLinearError,
    DuplicatedObject::ComputeDistance, Dead Reckoning, Dead Reckoning Overview.
    */

    /*!
     * @brief   エラー許容を決定するパラメータを、エラー許容と距離を一次関係のない二次関係として設定します。
     *
     * SetParameters(dMinimumValue, 0, (dMaximumValue-dMinimumValue)/(dDistance*dDistance), dMaximumValue);
     * と同じ動作です。
     *
     * 詳細はプログラミングマニュアルを参照してください。
     *
     * @param[in]   dMinimumValue   最小許容エラーを指定します。
     * @param[in]   dMaximumValue   最大許容エラーを指定します。
     * @param[in]   dDistance       最大許容エラーに到達する際の距離を指定します。
     *
     * @return  設定が成功したかどうかを返します。<br>
     *          dDistanceが負の値である時、または dMinimumValue が dMaximumValue より大きいときに false が返り、
     *          SYSTEMERROR_OBJDUP_INVALID_PARAMETER がシグナルされます。
     *
     * @see ErrorToleranceFunction::SetParameters, ErrorToleranceFunction::SetConstantError, ErrorToleranceFunction::SetLinearError
     */
    qBool SetQuadraticError(qReal dMinimumValue, qReal dMaximumValue, qReal dDistance);

    /*
    Summary: Returns the error tolerance at a given distance.
    Parameters:
    dDistance: Is the distance at which this method will return the error tolerance.
    Returns: The value of the error tolerance at the given distance.
    Remarks: This method is called internally by $PRODUCTNAME$ to determine if an update
    is required. If the value UNKNOWN_DISTANCE is passed the default distance is used.
    See also: SetDefaultDistance
    */
    qReal ValueAt(qReal dDistance) const;
    // {Secret}
    qReal SquaredValueAt(qReal dDistance) const;

    // {Secret}
    qBool DistanceMatters();

private:
    qReal m_pdParameters[5];
    qReal m_dDefaultDistance;
    qInt m_iLastParameterIndex;
    qReal m_dMaximumSquaredError;
};

END_NAMESPACE_NEX

