﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <limits>
#include <OnlineCore/src/Core/PluginsDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Core/FixedSizeMemoryPool.h>

BEGIN_NAMESPACE_NEX

class Buffer: public RefCountedObject
{

public:

    enum Flag
    {
        FLAG_NONE           = (qByte)0x00,
        FLAG_UPDATABLE      = (qByte)0x01, // 過去の互換性用に定義。使われていない。
        FLAG_EXPIRED        = (qByte)0x02, // 過去の互換性用に定義。使われていない。
        FLAG_PBPOOL         = (qByte)0x04, // PacketBufferPoolに確保される(主に P2P 通信)
        FLAG_MAX    = (qByte)0x80,
        FLAG_MASK   = (qByte)0xFF
    };

    void SetFlag(qByte uiFlag)
    {
        m_byFlags |= (uiFlag & FLAG_MASK);
    }

    void ClearFlag(qByte uiFlag)
    {
        m_byFlags &= ~(uiFlag & FLAG_MASK);
    }

    qBool FlagSet(qByte uiFlag) const
    {
        return (m_byFlags & uiFlag) != 0;
    }

    qByte GetFlags() const
    {
        return m_byFlags;
    }

    qUnsignedInt8 GetTTL() const
    {
        return m_uiTTL;
    }

    void SetTTL(qUnsignedInt8 uiTTL)
    {
        m_uiTTL = uiTTL;
    }

    /*
    Remarks: The default buffer size is 1024 bytes.
    */
    static void SetDefaultBufferSize(size_t size);
    static size_t GetDefaultBufferSize();

    Buffer()
    {
        Initialize(GetDefaultBufferSize(), FLAG_NONE);    // Default size allocate
    }
    Buffer(size_t uiBufSize)
    {
        Initialize(uiBufSize, FLAG_NONE);
    }
#ifdef NEX_CLIENT_PBPOOL // クライアントのみ
protected:
    /*! @internal
    @brief PacketBufferから呼び出すためだけに用意。 protected。
     */
    Buffer(size_t uiBufSize, Flag flag)
    {
        Initialize(uiBufSize, static_cast<qByte>(flag));
    }
#endif // NEX_CLIENT_PBPOOL

private:
    /*! @internal
     */
    void Initialize(size_t uiBufSize, qByte flags);
public:
    // Copy constructor.
    Buffer(const Buffer &oBuffer);
    // Creates a binary buffer from a string representation (see
    // ToString)
    Buffer(const String &oString);

#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    // ムーブコンストラクタ
    Buffer(Buffer&& rhs) NEX_NOEXCEPT;

    // ムーブ代入演算子
    Buffer& operator=(Buffer&& rhs) NEX_NOEXCEPT;
#endif

    virtual ~Buffer();

    /*! @internal
     @brief 現在のサイズと要求されるサイズから、次にメモリ確保するサイズを決定する。
     */
    static size_t  GetAllocateSize(size_t nowSize, size_t requiredSize);
    // Appends some data to the end of a buffer, or at uiStartOffset,
    // if specified.
    qBool   AppendData(const void *pBuffer, size_t uiNbBytes, size_t uiStartOffset = (std::numeric_limits<size_t>::max)());

    // Copy uiNbytes from the buffer, starting at offset uiStartOffset,
    // into a buffer pointed to by pBuffer.
    qBool   CopyContent(void *pBuffer, size_t uiNbBytes, size_t uiStartOffset = 0) const;
    // Clears the content of a buffer.
    void    Clear()
    {
        m_uiContentSize = 0;
        m_uiHeadShiftSize = 0;
        m_uiTTL = 0;
        m_bySubStreamID = 0;

        // フラグはパケットバッファフラグのみ保持する
        m_byFlags = m_byFlags & FLAG_PBPOOL;
    }
    // Returns the size of the internal buffer currently used by
    // buffer.  Always >= ContentSize.  A buffer automatically
    // increases its size, when needed.
    // n2967: Bufferの確保されたサイズを返す
    inline size_t   GetSize() const
    {
        return m_uiBufSize - m_uiHeadShiftSize;
    }

    // n2967:ヘッドシフトしていない本当のサイズを返す
    inline size_t   GetRealSize() const
    {
        return m_uiBufSize;
    }

    // n2967:ヘッドシフトしていない本当のサイズでResize
    void            ResizeByRealSize(size_t uiSize);
    void            Resize(size_t uiSize)
    {
        ResizeByRealSize(uiSize + m_uiHeadShiftSize);
    }
#ifdef NEX_CLIENT_PBPOOL // クライアントのみ
    /*! @internal
     @brief 指定したサイズ分サイズ拡張を試す。 拡張できない場合、falseが返る。 その場合、リサイズされないまま。
     */
    qBool           AttemptExpand(size_t uiSize);
#endif // NEX_CLIENT_PBPOOL

    /*! @internal
     @brief Bufferの中身を交換します。サイズ、フラグ、その他全ての内部変数が置き換わります。
     */
    void            Swap(Buffer& oBuffer);

    inline qByte    *GetContentPtr() const
    {
        return m_pData + m_uiHeadShiftSize;
    }

    // n2967:ヘッドシフトしていない本当のContentSize
    inline size_t  GetRealContentSize() const
    {
        return m_uiContentSize + m_uiHeadShiftSize;
    }

    // Returns the size of content of the buffer.
    inline size_t   GetContentSize() const
    {
        SYSTEMCHECK( GetRealContentSize() <= m_uiBufSize );
        return m_uiContentSize;
    }

    void            SetContentSize(size_t uiContentSize)
    {
        m_uiContentSize = uiContentSize;
        SYSTEMCHECK( GetRealContentSize() <= m_uiBufSize );
        /*checkContentSize();*/
    }

    // n2967:ヘッドシフトサイズを取得
    inline size_t GetHeadShiftSize() const
    {
        return m_uiHeadShiftSize;
    }

    // n2967:ヘッドシフトサイズを設定。設定された分のずれた位置がヘッドになる
    void SetHeadShiftSize(size_t shiftSize);

    // ヘッドシフトサイズをadditionSize分増やす。
    void AddHeadShiftSize(size_t additionSize)
    {
        SetHeadShiftSize(GetHeadShiftSize() + additionSize);
    }

    // n2967:ヘッドシフトをリセット
    void ResetHeadShiftSize()
    {
        SetHeadShiftSize(0);
    }

    qByte   ComputeCheckSum(size_t uiSize, qByte startChecksum=0);
    qByte   GetCheckSum();
    void    AppendCheckSum(qByte startChecksum=0);
    void    StripCheckSum();
    qBool   IsCheckSumValid(qByte startChecksum=0);

    // Access the buffer content by index.
    qByte   &operator[](size_t uiIndex);
    // Access the buffer content by index.
    qByte   &operator[](size_t uiIndex) const
    {
        // Should never read past content size... and nor past buffer size.
        SYSTEMCHECK(uiIndex < m_uiContentSize);
        SYSTEMCHECK(uiIndex + m_uiHeadShiftSize < m_uiBufSize);

        return GetContentPtr()[uiIndex + m_uiHeadShiftSize];
    }
    // Assignment operator.
    Buffer& operator=(const Buffer& oBuffer);
    // Compares two buffers.
    qBool   operator==(const Buffer& oBuffer) const;
    // Compares two buffers.
    qBool   operator!=(const Buffer& oBuffer) const
    {
        return !(*this==oBuffer);
    }
    Buffer& operator+(const Buffer &oBuffer);
    Buffer& operator+=(const Buffer& oBuffer);

    // Creates a string representation of a buffer - can be used to
    // construct a buffer.
    String ToString() const;

    void    Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS) const;

private:
    qByte   *AllocateDataBuffer(size_t uiBufferSize);
    void    FreeDataBuffer(qByte *pBuffer, size_t uiBufferSize);
public:

    //  The purpose is to not use GetContentSize in some circumstances, because of
    //  the underlying SYSTEMCHECK.
    size_t GetContentSizeValue() const
    {
        return m_uiContentSize;
    }

    qByte GetSubStreamID() const
    {
        return m_bySubStreamID;
    }

    void SetSubStreamID(qByte subStreamID)
    {
        m_bySubStreamID = subStreamID;
    }
#ifdef NEX_CLIENT_PBPOOL // クライアントのみ
    /* @internal
     @brief Validでない場合は、このオブジェクトを使用してはいけない。
            現状メモリ確保できていない場合false。
     */
    qBool IsValid() const
    {
        if (!m_pData)
        {
            QTRACE(D_TRACE_MEMORY, NEX_T("Buffer::IsValid() false"));
            // コンストラクタでバッファ確保されるはずなのに無い(バッファ枯渇)。
            return false;
        }

        return true;
    }
#endif // NEX_CLIENT_PBPOOL

protected:
    qByte  *m_pData;
    size_t m_uiContentSize;
    size_t m_uiBufSize;
    size_t m_uiHeadShiftSize; // ヘッドをシフトさせるサイズ

    qByte m_byFlags;

    qUnsignedInt8 m_uiTTL;

    qByte m_bySubStreamID;

};

END_NAMESPACE_NEX

