﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#if defined(WIN32_TARGET) || defined(__i386__) || defined(i386) || defined(__x86_64) || defined(__x86_64__)
//n2967: CPUがアライメント違反に寛容かつLittle Endianなアーキテクチャの場合(x86系)、じかにメモリをアクセスする
#define NEX_USE_CPU_UNALIGNED_LITTLE_ENDIAN_ACCESS 1

#endif

BEGIN_NAMESPACE_NEX

/** !private
 *  @defgroup ByteOrder ホストオーダとNEXのネットワークオーダ変換ライブラリ
 *  @brief プリミティブ型をNEXの伝送路に用いられるエンディアン(リトルエンディアン)へ変換します。
 *
 *  プリミティブ型として、qUnsignedInt8〜qUnsignedInt64、qInt8〜qInt64、qFloat、qDouble
 *  qByte、qChar8、qChar16を指定できます。
 *  Socketのネットワークオーダと定義が異なることに注意してください。
 **/



/** !private
 * @ingroup ByteOrder
 * @brief 値をNEXのネットワークオーダからホストオーダへ変換します。
 *
 * @param[in] p 変換前のバイト配列を指定します
 * @param[out] v 変換後のプリミティブ型(Type)の格納するポインタを指定します
 */
template <typename Type> void NexNetworkOrderToHostOrder(const qByte* p, Type *v);

/** !private
 * @ingroup ByteOrder
 * @brief 値をNEXのネットワークオーダからホストオーダへ変換します。
 *
 * @param[in] v 変換前のプリミティブ型(Type)の値を指定します
 * @return 変換後の値を返します
 */
template <typename Type> Type NexNetworkOrderToHostOrder(Type v);

/** !private
 * @ingroup ByteOrder
 * @brief 値をホストオーダからNEXのネットワークオーダへ変換します。
 *
 * @param[in] p 変換前のバイト配列を指定します
 * @param[out] v 変換後のプリミティブ型(Type)の格納するポインタを指定します
 */
template <typename Type> void HostOrderToNexNetworkOrder(Type v, qByte* p);

/** !private
 * @ingroup ByteOrder
 * @brief 値をホストオーダからNEXのネットワークオーダへ変換します。
 *
 * @param[in] v 変換前のプリミティブ型(Type)の値を指定します
 * @return 変換後の値を返します
 */
template <typename Type> Type HostOrderToNexNetworkOrder(Type v)
{
    return NexNetworkOrderToHostOrder(v);
}



// internal use


//for strict aliasing
union UnionDoubleUnsignedInt64
{
    qDouble d;
    qUnsignedInt64 ui;
};

//for strict aliasing
union UnionFloatUnsignedInt32
{
    qFloat f;
    qUnsignedInt32 ui;
};


inline void GetUnalignedNexNetworkOrder16(const qByte* p, qUnsignedInt16 *v)
{
    *v = ((qUnsignedInt16)p[0]) | ((qUnsignedInt16)p[1] << 8);
}

inline void GetUnalignedNexNetworkOrder32(const qByte* p, qUnsignedInt32 *v)
{
    *v = ((qUnsignedInt32)p[0]) |
         (((qUnsignedInt32)p[1]) << 8) |
         (((qUnsignedInt32)p[2]) << 16) |
         (((qUnsignedInt32)p[3]) << 24);
}

inline void GetUnalignedNexNetworkOrder64(const qByte* p, qUnsignedInt64 *v)
{
    *v= ((qUnsignedInt64)p[0]) |
        (((qUnsignedInt64)p[1]) << 8) |
        (((qUnsignedInt64)p[2]) << 16) |
        (((qUnsignedInt64)p[3]) << 24) |
        (((qUnsignedInt64)p[4]) << 32) |
        (((qUnsignedInt64)p[5]) << 40) |
        (((qUnsignedInt64)p[6]) << 48) |
        (((qUnsignedInt64)p[7]) << 56);
}


inline void SetUnalignedNexNetworkOrder16(qUnsignedInt16 v, qByte* p)
{
    p[0] = v & 0x00FFU;
    p[1] = (v & 0xFF00U) >> 8;
}

inline void SetUnalignedNexNetworkOrder32(qUnsignedInt32 v, qByte* p)
{
    p[0] = (qByte)(v & 0x000000FFUL);
    p[1] = (qByte)((v & 0x0000FF00UL) >> 8);
    p[2] = (qByte)((v & 0x00FF0000UL) >> 16);
    p[3] = (qByte)((v & 0xFF000000UL) >> 24);
}

inline void SetUnalignedNexNetworkOrder64(qUnsignedInt64 v, qByte* p)
{
    p[0] =  (qByte)(v & 0x00000000000000FFULL);
    p[1] = (qByte)((v & 0x000000000000FF00ULL) >> 8);
    p[2] = (qByte)((v & 0x0000000000FF0000ULL) >> 16);
    p[3] = (qByte)((v & 0x00000000FF000000ULL) >> 24);
    p[4] = (qByte)((v & 0x000000FF00000000ULL) >> 32);
    p[5] = (qByte)((v & 0x0000FF0000000000ULL) >> 40);
    p[6] = (qByte)((v & 0x00FF000000000000ULL) >> 48);
    p[7] = (qByte)((v & 0xFF00000000000000ULL) >> 56);
}



inline qUnsignedInt16 ByteOrderSwap16(qUnsignedInt16 v)
{
    return ((v & 0x00FFU) << 8) |  ((v & 0xFF00U) >> 8);
}


inline qUnsignedInt32 ByteOrderSwap32(qUnsignedInt32 v)
{
    return ((v & 0x000000FFUL) << 24) | ((v & 0x0000FF00UL) << 8) |
           ((v & 0x00FF0000UL) >> 8) | ((v & 0xFF000000UL) >> 24);
}


inline qUnsignedInt64 ByteOrderSwap64(qUnsignedInt64 v)
{
    return ((v & 0x00000000000000FFULL) << 56) | ((v & 0x000000000000FF00ULL) << 40) |
           ((v & 0x0000000000FF0000ULL) << 24) | ((v & 0x00000000FF000000ULL) <<  8) |
           ((v & 0x000000FF00000000ULL) >>  8) | ((v & 0x0000FF0000000000ULL) >> 24) |
           ((v & 0x00FF000000000000ULL) >> 40) | ((v & 0xFF00000000000000ULL) >> 56);
}



#if defined(NEX_USE_CPU_UNALIGNED_LITTLE_ENDIAN_ACCESS)
//n2967: Host Orderが、Little Endianの場合は無変換でアクセスできる
inline void NexNetworkOrderToHostOrder64(const qByte* p, qUnsignedInt64 *v)
{
    *v = *reinterpret_cast<const qUnsignedInt64 *>(p);
}

inline void NexNetworkOrderToHostOrder32(const qByte* p, qUnsignedInt32 *v)
{
    *v = *reinterpret_cast<const qUnsignedInt32 *>(p);
}

inline void NexNetworkOrderToHostOrder16(const qByte* p, qUnsignedInt16 *v)
{
    *v = *reinterpret_cast<const qUnsignedInt16 *>(p);
}

#else
//n2967: アライメント違反を考慮して少し無駄なコードになっている
inline void NexNetworkOrderToHostOrder64(const qByte* p, qUnsignedInt64 *v)
{
    GetUnalignedNexNetworkOrder64(p, v);
}

//n2967: アライメント違反を考慮して少し無駄なコードになっている
inline void NexNetworkOrderToHostOrder32(const qByte* p,  qUnsignedInt32 *v)
{
    GetUnalignedNexNetworkOrder32(p, v);
}

//n2967: アライメント違反を考慮して少し無駄なコードになっている
inline void NexNetworkOrderToHostOrder16(const qByte* p, qUnsignedInt16 *v)
{
    GetUnalignedNexNetworkOrder16(p, v);
}
#endif

//qUnsignedInt64

template <>
inline
void NexNetworkOrderToHostOrder<qUnsignedInt64>(const qByte* p, qUnsignedInt64 *v)
{
    return NexNetworkOrderToHostOrder64(p, v);
}


template <>
inline
void HostOrderToNexNetworkOrder<qUnsignedInt64>(qUnsignedInt64 v, qByte *p)
{
    return SetUnalignedNexNetworkOrder64(v, p);
}

template <>
inline
qUnsignedInt64 NexNetworkOrderToHostOrder<qUnsignedInt64>(qUnsignedInt64 v)
{
#if NN_NEX_BYTE_ORDER==NN_NEX_LITTLE_ENDIAN
    return v;
#else
    return ByteOrderSwap64(v);
#endif
}


//qUnsignedInt32

template <>
inline
void NexNetworkOrderToHostOrder<qUnsignedInt32>(const qByte* p, qUnsignedInt32 *v)
{
    NexNetworkOrderToHostOrder32(p, v);
}

template <>
inline
void HostOrderToNexNetworkOrder<qUnsignedInt32>(qUnsignedInt32 v, qByte *p)
{
    return SetUnalignedNexNetworkOrder32(v, p);
}

template <>
inline
qUnsignedInt32 NexNetworkOrderToHostOrder<qUnsignedInt32>(qUnsignedInt32 v)
{
#if NN_NEX_BYTE_ORDER==NN_NEX_LITTLE_ENDIAN
    return v;
#else
    return ByteOrderSwap32(v);
#endif
}

//qUnsignedInt16

template <>
inline
void NexNetworkOrderToHostOrder<qUnsignedInt16>(const qByte* p, qUnsignedInt16 *v)
{
    NexNetworkOrderToHostOrder16(p, v);
}

template <>
inline
void HostOrderToNexNetworkOrder<qUnsignedInt16>(qUnsignedInt16 v, qByte *p)
{
    return SetUnalignedNexNetworkOrder16(v, p);
}

template <>
inline
qUnsignedInt16 NexNetworkOrderToHostOrder<qUnsignedInt16>(qUnsignedInt16 v)
{
#if NN_NEX_BYTE_ORDER==NN_NEX_LITTLE_ENDIAN
    return v;
#else
    return ByteOrderSwap16(v);
#endif
}

// qInt64
template <>
inline
void NexNetworkOrderToHostOrder<qInt64>(const qByte* p, qInt64 *v)
{
    NexNetworkOrderToHostOrder64(p, reinterpret_cast<qUnsignedInt64 *>(v));
}

template <>
inline
void HostOrderToNexNetworkOrder<qInt64>(qInt64 v, qByte *p)
{
    return SetUnalignedNexNetworkOrder64(static_cast<qUnsignedInt64>(v), p);
}

template <>
inline
qInt64 NexNetworkOrderToHostOrder<qInt64>(qInt64 v)
{
    return static_cast<qInt64>( NexNetworkOrderToHostOrder( static_cast<qUnsignedInt64>(v) ) );
}


// qInt32

template <>
inline
void NexNetworkOrderToHostOrder<qInt32>(const qByte* p, qInt32 *v)
{
    NexNetworkOrderToHostOrder32(p, reinterpret_cast<qUnsignedInt32 *>(v));
}

template <>
inline
void HostOrderToNexNetworkOrder<qInt32>(qInt32 v, qByte* p)
{
    return SetUnalignedNexNetworkOrder32(static_cast<qUnsignedInt32>(v), p);
}

template <>
inline
qInt32 NexNetworkOrderToHostOrder<qInt32>(qInt32 v)
{
    return static_cast<qInt32>( NexNetworkOrderToHostOrder( static_cast<qUnsignedInt32>(v) ) );
}


// qInt16

template <>
inline
void NexNetworkOrderToHostOrder<qInt16>(const qByte* p, qInt16 *v)
{
    NexNetworkOrderToHostOrder16(p, reinterpret_cast<qUnsignedInt16 *>(v));
}

template <>
inline
void HostOrderToNexNetworkOrder<qInt16>(qInt16 v, qByte* p)
{
    return SetUnalignedNexNetworkOrder16(static_cast<qUnsignedInt16>(v), p);
}

template <>
inline
qInt16 NexNetworkOrderToHostOrder<qInt16>(qInt16 v)
{
    return static_cast<qInt16>( NexNetworkOrderToHostOrder( static_cast<qUnsignedInt16>(v) ) );
}

// qDouble

template <>
inline
void NexNetworkOrderToHostOrder<qDouble>(const qByte* p, qDouble *v)
{
    UnionDoubleUnsignedInt64 value;
    NexNetworkOrderToHostOrder64(p, &value.ui);
    *v = value.d;
}


template <>
inline
void HostOrderToNexNetworkOrder<qDouble>(qDouble v, qByte* p)
{
    UnionDoubleUnsignedInt64 value;
    value.d = v;
    SetUnalignedNexNetworkOrder64(value.ui, p);
}


template <>
inline
qDouble NexNetworkOrderToHostOrder<qDouble>(qDouble v)
{
#if NN_NEX_BYTE_ORDER==NN_NEX_LITTLE_ENDIAN
    return v;
#else
    UnionDoubleUnsignedInt64 value;
    UnionDoubleUnsignedInt64 ret;
    value.d = v;
    ret.ui = ByteOrderSwap64(value.ui);
    return ret.d;
#endif

}


//qFloat


template <>
inline
void NexNetworkOrderToHostOrder<qFloat>(const qByte* p, qFloat *v)
{
    UnionFloatUnsignedInt32 value;
    NexNetworkOrderToHostOrder32(p, &value.ui);
    *v = value.f;
}


template <>
inline
void HostOrderToNexNetworkOrder<qFloat>(qFloat v, qByte* p)
{
    UnionFloatUnsignedInt32 value;
    value.f = v;
    SetUnalignedNexNetworkOrder32(value.ui, p);
}


template <>
inline
qFloat NexNetworkOrderToHostOrder<qFloat>(qFloat v)
{
#if NN_NEX_BYTE_ORDER==NN_NEX_LITTLE_ENDIAN
    return v;
#else
    UnionFloatUnsignedInt32 value;
    UnionFloatUnsignedInt32 ret;
    value.f= v;
    ret.ui = ByteOrderSwap32(value.ui);
    return ret.f;
#endif

}


//qChar8


template <>
inline
void NexNetworkOrderToHostOrder<qChar8>(const qByte* p, qChar8 *v)
{
    *v = static_cast<qChar8>(*p);
}


template <>
inline
void HostOrderToNexNetworkOrder<qChar8>(qChar8 v, qByte* p)
{
    *p = static_cast<qByte>(v);
}


template <>
inline
qChar8 NexNetworkOrderToHostOrder<qChar8>(qChar8 v)
{
    return v;
}

//qInt8

template <>
inline
void NexNetworkOrderToHostOrder<qInt8>(const qByte* p, qInt8 *v)
{
    *v = static_cast<qInt8>(*p);
}

template <>
inline
void HostOrderToNexNetworkOrder<qInt8>(qInt8 v, qByte* p)
{
    *p = static_cast<qByte>(v);
}


template <>
inline
qInt8 NexNetworkOrderToHostOrder<qInt8>(qInt8 v)
{
    return v;
}

//qByte

template <>
inline
void NexNetworkOrderToHostOrder<qByte>(const qByte* p, qByte* v)
{
    *v = static_cast<qByte>(*p);
}

template <>
inline
void HostOrderToNexNetworkOrder<qByte>(qByte v, qByte* p)
{
    *p = static_cast<qByte>(v);
}

template <>
inline
qByte NexNetworkOrderToHostOrder<qByte>(qByte v)
{
    return v;
}

//qBool
template <>
inline
void NexNetworkOrderToHostOrder<qBool>(const qByte* p, qBool *v)
{
    if (*p)
    {
        *v =true;
    }
    else
    {
        *v = false;
    }
}

template <>
inline
void HostOrderToNexNetworkOrder<qBool>(qBool v, qByte* p)
{
    if (v)
    {
        *p =1;
    }
    else
    {
        *p = 0;
    }
}


template <>
inline
qBool NexNetworkOrderToHostOrder<qBool>(qBool v)
{
    return v;
}

#if defined (NEX_QCHAR16_IS_NOT_UINT16_T)
template <>
inline
void NexNetworkOrderToHostOrder<qChar16>(const qByte* p, qChar16* v)
{
    NexNetworkOrderToHostOrder16(p, reinterpret_cast<qUnsignedInt16 *>(v));
}

template <>
inline
void HostOrderToNexNetworkOrder<qChar16>(qChar16 v, qByte* p)
{
    return SetUnalignedNexNetworkOrder16(static_cast<qUnsignedInt16>(v), p);
}

template <>
inline
qChar16 NexNetworkOrderToHostOrder<qChar16>(qChar16 v)
{
    return static_cast<qChar16>( NexNetworkOrderToHostOrder( static_cast<qUnsignedInt16>(v) ) );
}


#endif

END_NAMESPACE_NEX

