﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/Buffer.h>
#include <OnlineCore/src/Core/ByteOrder.h>

BEGIN_NAMESPACE_NEX

//#define NEX_AVOID_NAT_CHECK_BUG 1

#if defined(NEX_AVOID_RVCT40_BUG) || defined(NEX_AVOID_NAT_CHECK_BUG)

#define NEX_SIMPLE_TYPE_BYTE_STREAMING(TYPE)                \
qBool Append(const TYPE *pValue, size_t uiSize=1) {return Append((const qByte*) pValue, sizeof(TYPE), uiSize);} \
qBool Extract(TYPE *pValue, size_t uiSize=1) {return Extract((qByte*) pValue, sizeof(TYPE), uiSize);} \
qBool Append(const TYPE &refValue) {return Append(&refValue);} \
qBool Extract(TYPE &refValue) {return Extract(&refValue);} \
ByteStream& operator<<(const TYPE &refValue) {Append(&refValue); return *this;} \
ByteStream& operator>>(TYPE &refValue) {Extract(&refValue); return *this;}

#else

#define NEX_SIMPLE_TYPE_BYTE_STREAMING(TYPE)                \
qBool Append(const TYPE *pValue, size_t uiSize=1) {return Append((const qByte*) pValue, sizeof(TYPE), uiSize);} \
qBool Extract(TYPE *pValue, size_t uiSize) {return Extract((qByte*) pValue, sizeof(TYPE), uiSize);}   \
qBool Extract(TYPE *pValue) {return ExtractPrimitive(pValue);} \
qBool Append(const TYPE &refValue) {return AppendPrimitive(refValue);} \
qBool Extract(TYPE &refValue) {return Extract(&refValue);} \
ByteStream& operator<<(const TYPE &refValue) {Append(&refValue); return *this;} \
ByteStream& operator>>(TYPE &refValue) {Extract(&refValue); return *this;}

#endif
/*
Description: Implements streaming to and from a ByteStream for a custom data type.
Remarks: This class is used to implement streaming to and from a ByteStream for custom (user-defined) data types.
See Also: DDL File Syntax
*/
class ByteStream: public RootObject
{
public:
    ByteStream();
    ByteStream(Buffer *pBuffer);
    ByteStream(size_t uiBufSize);

    virtual ~ByteStream();

    void Clear();
    size_t GetLength() const
    {
        return m_pBuffer->GetContentSize();
    }
    qBool SetLength(size_t uiLength);
    size_t GetSize() const
    {
        return m_pBuffer->GetSize();
    }

    size_t GetPosition() const
    {
        return m_uiPosition;
    }

    qBool SetPosition(size_t uiPosition)
    {
        if (uiPosition > GetLength())
        {
            return false;
        }
        else
        {
            m_uiPosition = uiPosition;
            return true;
        }
    }

    qBool AddPosition(size_t addPos)
    {
        return SetPosition(GetPosition() + addPos);
    }

    Buffer *GetBuffer()
    {
        return m_pBuffer;
    }
    const Buffer *GetBuffer() const
    {
        return m_pBuffer;
    }

    qBool ErrorHasOccured() const
    {
        return m_bErrorHasOccured;
    }
    void ClearErrorFlag()
    {
        m_bErrorHasOccured=false;
    }
    void SetErrorFlag()
    {
        m_bErrorHasOccured=true;
    }

    //  Similar to CanExtract, but logs an error if a request of uiNbBytes bytes
    //  overruns the buffer currently.
    //  Also, sets the error flag if it is the case.
    qBool ValidateBufferLimit(size_t uiNbBytes)
    {
        if (m_pBuffer->GetContentSizeValue() - GetPosition() >= uiNbBytes)
        {
            return true;
        }
        else
        {
            QLOG(EventLog::Error, NEX_T("ByteStream limit is exceeded by request of ") << uiNbBytes
                 << NEX_T(" bytes. Length:") << m_pBuffer->GetContentSizeValue()
                 << NEX_T(" Position: ") << GetPosition());

            SetErrorFlag();

            return false;
        }
    }

    // API to stream operators.

    qBool AppendString(const qChar* pBuffer);
    qBool AppendString(const qChar* pBuffer, size_t uiMaxSize);
    qBool AppendString(const qChar* pBuffer, size_t uiMaxSize, size_t uiStrLength);
    qBool ExtractString(qChar *strBuffer,qUnsignedInt16 uiBufSize);

#if NN_NEX_BYTE_ORDER == NN_NEX_LITTLE_ENDIAN
    qBool Append(const qByte* pBuffer, size_t uiTypeSize, size_t uiNb)
    {
        return AppendRaw(pBuffer, uiTypeSize*uiNb);
    }
    qBool Extract(qByte* pBuffer, size_t uiTypeSize, size_t uiNb)
    {
        return ExtractRaw(pBuffer, uiTypeSize*uiNb);
    }
#else
    qBool Append(const qByte* pBuffer, size_t uiTypeSize, size_t uiNb);
    qBool Extract(qByte* pBuffer, size_t uiTypeSize, size_t uiNb);
#endif // NN_NEX_BYTE_ORDER == NN_NEX_LITTLE_ENDIAN

    qBool AppendRaw(const qByte* pBuffer, size_t uiNbBytes);
    qBool ExtractRaw(qByte* pBuffer, size_t uiNbBytes);

private:
    //Cannot use these fuction. Please use << or >>
    qBool Extract(qBool *)
    {
        return false;
    }
    qBool Append(const qBool &)
    {
        return false;
    }
    qBool Extract(qBool &)
    {
        return false;
    }
    qBool Append(const qBool *)
    {
        return false;
    }
    //n2967
    template<typename T> qBool ExtractPrimitive(T* ptr)
    {
        if (! ValidateBufferLimit(sizeof(T)))
        {
            *ptr = (T) 0;
            return false;
        }

        NexNetworkOrderToHostOrder<T>(m_pBuffer->GetContentPtr()+m_uiPosition, ptr);
        m_uiPosition+=sizeof(T);
        return true;
    }

    template<typename T> qBool AppendPrimitive(T v)
    {
        qByte temp[sizeof(qUnsignedInt64)];
        HostOrderToNexNetworkOrder<T>(v, temp);
        m_pBuffer->AppendData(temp, sizeof(T));
        return true;
    }
public:

    NEX_SIMPLE_TYPE_BYTE_STREAMING(qChar8);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qInt8);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qByte);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qInt16);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qUnsignedInt16);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qInt32);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qUnsignedInt32);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qFloat);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qDouble);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qInt64);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qUnsignedInt64);
#if !defined(NEX_LONG_IS_UINT64_T)
    NEX_SIMPLE_TYPE_BYTE_STREAMING(long);
    NEX_SIMPLE_TYPE_BYTE_STREAMING(unsigned long);
#endif
#if defined (NEX_QCHAR16_IS_NOT_UINT16_T)
    NEX_SIMPLE_TYPE_BYTE_STREAMING(qChar16);
#endif

    qBool Append(const Time *pbValue);
    qBool Extract(Time *Ptr);
    ByteStream& operator<<(const Time &refValue)
    {
        Append(&refValue);
        return *this;
    }
    ByteStream& operator>>(Time &refValue)
    {
        Extract(&refValue);
        return *this;
    }

    qBool   operator==(const ByteStream& oStream) const;

    ByteStream& operator<<(const Buffer& oBuffer);
    ByteStream& operator>>(Buffer& oBuffer);

    ByteStream& operator<<(const qBool& boolVal);
    ByteStream& operator>>(qBool& boolVal);

    ByteStream& operator<<(ByteStream& oValue)
    {
        return *this << *oValue.GetBuffer();
    }
    ByteStream& operator>>(ByteStream& oValue);

    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, qBool bTraceBytes=false);
    static void TraceBytes(qUnsignedInt64 uiTraceFlags, qByte* pBuffer, size_t uiSize);

private:
    void Init();

    qBool CanAppend(qUnsignedInt32 uiBytesToAppend) const;
    qBool CanExtract(qUnsignedInt32 uiBytesToExtract) const;

protected:
    qBool m_bErrorHasOccured;
    Buffer *m_pBuffer;

    size_t m_uiPosition;
};

#undef NEX_SIMPLE_TYPE_BYTE_STREAMING


END_NAMESPACE_NEX


