﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Core/CallbackHolder.h>

BEGIN_NAMESPACE_NEX

class CallbackRoot;
class CallContextRegister;
class CallContext;

/*!
    @brief    引数を持たない関数を受け取り、CallContext* を引数に取る関数に変換する関数アダプタです。

    CallContext::RegisterCompletionCallbackGeneric() に引数を一つも取らない関数を渡したい場合に使用します。

    function には引数を一つも取らない関数ポインタ、関数オブジェクトなどを指定できます。
    関数オブジェクトを渡す場合は値もしくは参照型で渡す方法とポインタ型で渡す方法の二種類があり、それぞれ挙動が異なります。
    値もしくは参照型で渡した場合は内部で関数オブジェクトがコピーもしくはムーブされます。
    そのため、本関数呼び出し後は指定したオブジェクトを即座に破棄する事ができます。
    ポインタ型で渡した場合はその関数オブジェクトの利用が完了するまで呼び出し側で保持しておく必要があります。
    本関数にポインタ渡しして得た関数オブジェクトを CallContext::RegisterCompletionCallbackGeneric() に渡し、かつ第二引数を true としても
    元の関数オブジェクトは delete されません。ご注意ください。

    @param[in] function 引数を一つも取らない関数ポインタや関数オブジェクト。
    @return CallContext* を引数に取る関数オブジェクト。
*/
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
template<typename TFunction>
AddArgumentAdapterHelper<TFunction, CallContext*> AddCallContextArgumentAdapter(TFunction&& function)
{
    return std::forward<AddArgumentAdapterHelper<TFunction, CallContext*> >(
        AddArgumentAdapter<TFunction, CallContext*>(std::forward<TFunction>(function)));
}
#else
template<typename TFunction>
AddArgumentAdapterHelper<TFunction, CallContext*> AddCallContextArgumentAdapter(const TFunction& function)
{
    return AddArgumentAdapter<TFunction, CallContext*>(function);
}
#endif

/*
Description: The root class of all call contexts.
Remarks: This root class defines the context of all call contexts. You may make any number of
non-blocking calls using different call contexts and $PRODUCTNAME$ will guarantee that the
order of execution is maintained on the remote station. Nevertheless, if you create a dependency
chain between calls, /i.e./ a given call depends on the result returned by a previous call,
you must Wait between calls so that the system has time to complete the first request before calling
the second.

The State diagram for calls is as follows, where the State may be returned via GetState.

{image:RMCStateDiagram}

If an Asynchronous call returns false, /i.e./ call initiation failed, then the State of the call's
CallContext will be either Available or CallFailure depending on when $PRODUCTNAME$ detected the error.
*/

/*!
@brief    非同期処理の管理クラスのルートクラスです。

NEX でサーバーとの通信などで非同期処理を実行するには、非同期関数の引数に、 @ref CallContext クラスやそのサブクラス
（サーバーとの通信が発生する場合は @ref ProtocolCallContext クラス）を渡します。 @ref CallContext クラスを利用することで、
非同期処理の状態や結果に関する情報の取得や、キャンセルやリセット、タイムアウトの設定などもできます。
*/
class CallContext: public RefCountedObject
{
    //Group=Type Definitions
public:


    // Description: Enumerates the various States of a CallContext.
    // See Also: State

    /*!
    @brief     @ref CallContext の状態を列挙します。

       非同期処理を開始すると Available から CallInProgress に遷移し、処理が終了すると
       CallSuccess、CallFailure、CallCancelled のいずれかの完了状態に遷移します。
    */
    enum State
    {
        // The call can be called. When this is done the State will change to
        // CallInProgress. If Reset is called when the State is one of CallSuccess,
        // CallFailure, or CallCancelled the State will change to Available.
        Available,    //!< 実行可能。
        /*
        $ifdef RENDEZVOUS$
        The call is in progress.
        $else$
        The call is in progress. The method CallContext::Cancel may be called.
        $endif$
        */
        CallInProgress, //!< 処理を実行中。この状態のときは他の非同期処理の Context としては使用できません。
        // The call has terminated. The method Reset may be called.
        CallSuccess,    //!< 処理が成功して終了しました。
        // The call failed. The method Reset may be called.
        CallFailure,    //!< 処理が失敗して終了しました。
        /*
        $ifdef RENDEZVOUS$
        The call was cancelled by the user. The method Reset may be called.
        $else$
        The call was cancelled by the user via the method CallContext::Cancel. The method
        Reset may be called.
        $endif$
        */
        CallCancelled    //!< 処理がキャンセルされました。
    };

    // Description: Enumerates the various Flags that may be used with a CallContext.
    /*!
    @brief     @ref CallContext に使用可能な各種フラグを列挙します。
    */
    enum Flags
    {
        /*
        $ifdef RENDEZVOUS$
        {secret}
            $else$
        The call cannot be cancelled. This flag can only be set for a MigrationContext.
         $endif$
        */
        CancellationNotAllowed  = 0x01,
        // The CallContext of this call will be deleted upon completion of the call that uses this CallContext.
        // When you dynamically create a CallContext, you need to ensure that it is deleted once it is no longer
        // needed.
        /*!
         @brief    処理が完了したタイミングで ReleaseRef() が呼ばれます。
         */
        DeleteOnCompletion      = 0x02,
        /*
        $ifdef RENDEZVOUS$
        {secret}
            $else$
           The call will be unidirectional. It will not return the result of the called method. This
           flag can only be set for a RMCContext. If the remote method returns a value, this value will be ignored
           and not returned to the caller (no return message will be sent). However, as always, the call will still
           return a Boolean that indicates whether or not the message was sent to the target station. If this flag
           is set, the call can be made on multiple targets and the flags SendUnreliableMessage, CallOnDuplicas,
           CallOnMaster, CallOnNeighbouringStations, and CallOnLocalStation may also be set.
           $endif$
        */

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
        /*!
         * @brief   呼び出しが一方通行になります。この呼び出しに対する結果は返されません。
         *          このフラグは@ref RMCContext のためだけに用意されています。もしも相手先のメソッドが値を返したとき、
         *          値は無視されて返答されません。しかし、常に、メッセージが相手に対して送信されたのかどうかは
         *          ブール値として返ります。もしもこのフラグがセットされていたら、この呼び出しは複数の対象を取ることができ、
         *          SendUnreliableMessage, CallOnDuplicas, CallOnMaster, CallOnNeighbouringStations, CallOnLocalStationの
         *          各フラグと一緒に設定しても問題ありません。
         */
#endif
        OneWayCall              = 0x04,
        /*
        $ifdef RENDEZVOUS$
        {secret}
            $else$
         The call will be sent over an unreliable channel. This flag can only be set for a RMCContext and is only
         valid when the OneWayCall flag is set.
          $endif$
        */

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
        /*!
         * @brief   呼び出しはアンリライアブル(到達保証性のない)チャンネルを利用します。
         *          このフラグは@ref RMCContext のために用意されています。
         *          また、OneWayCallフラグも同時にセットされていなければなりません。
         */
#endif
        SendUnreliableMessage   = 0x08,
        /*
        $ifdef RENDEZVOUS$
        {secret}
            $else$
         The call message will not be bundled and will therefore be sent as soon as possible.
         This flag can only be set for a RMCContext.
        $endif$
        */
        SendUnbundledMessage    = 0x10,
        // The first available flag that a derived class may use.
        FirstFlagAvailable      = 0x20
    };

    /*
    Description: Defines a CompletionCallback.
    Parameters:
    o /CallContext/ : A pointer to the CallContext of this CompletionCallback.
    o /UserContext/ : A pointer to the UserContext associated with this CompletionCallback.
    Remarks: A CompletionCallback may be registered via RegisterCompletionCallback to ensure that a
    given user-defined function is called upon completion of the CallContext.
    See Also: RegisterCompletionCallback, RegisterCompletionCallback
    */

    /*!
    @brief    CompletionCallback を定義します。

     処理完了時に特定のユーザー定義関数が呼び出されるように、CompletionCallback を @ref RegisterCompletionCallback() で登録します。

    @see     RegisterCompletionCallback,  RegisterCompletionCallback
    */
    typedef void (*CompletionCallback)(CallContext*, const UserContext*);

    /*!
    @brief    コンストラクタです。
    */
    CallContext()
        : m_uiFlags(0)
        , m_eState(Available)
        , m_uiID(0)
        , m_lstCompletionCallback()
        , m_vUserContext()
        , m_pCompletionCallbacks(NULL)
        , m_rCallOutcome(QRESULT_ERROR(Core, Unknown))
        , m_pDependentConnection(NULL)
        , m_idConnection()
        , m_pCancellationCallback(NULL)
        , m_tDeadline(0)
    {
    }

    // Remarks: If the DeleteOnCompletion flag is set, the destructor is automatically called upon
    // completion of the call that uses this CallContext.

    /*!
    @brief    デストラクタです。

    状態が CallInProgress である場合、関連付けられた非同期処理はキャンセルされます。

    DeleteOnCompletionフラグが設定された場合、この @ref CallContext を使用する非同期処理の完了時にデストラクタが自動的に呼び出されます。
    */
    virtual ~CallContext();

#ifdef NEX_MOVE_SEMANTICS_SUPPORT

    /*!
    @brief    ムーブコンストラクタです。

    @param[in]    rhs    CallContext オブジェクト。
    */
    CallContext(CallContext&& rhs) NEX_NOEXCEPT
        : m_uiFlags(0)
        , m_eState(Available)
        , m_uiID(0)
        , m_lstCompletionCallback()
        , m_vUserContext()
        , m_pCompletionCallbacks(NULL)
        , m_rCallOutcome(QRESULT_ERROR(Core, Unknown))
        , m_pDependentConnection(NULL)
        , m_idConnection()
        , m_pCancellationCallback(NULL)
        , m_tDeadline(0)
    {
        *this = std::move(rhs);
    }

    /*!
    @brief    ムーブ代入演算子です。

    @param[in]    rhs    CallContext オブジェクト。
    */
    CallContext& operator=(CallContext&& rhs) NEX_NOEXCEPT
    {
        if (this == &rhs)
        {
            return *this;
        }
        this->Swap(rhs);
        return *this;
    }
#endif
    /* Description: A timeout interval which "fails" a call if the call
    exceeds the interval. That is, the call's state changes to
    CallContext::CallFailure the reason being Core::Timeout.
    Parameters: The interval in milliseconds.
    */

    //Do not expose this API publicly as it does not work as advertised in
    // all the places a call context can be used. See CallContext.SetDeadline()
    // {secret}
    /*!
     @brief    非同期処理のタイムアウトをセットします。

     非同期処理の開始後 tiTimeout ミリ秒後に非同期処理が完了していない( CallInProgress )場合に、
     自動的に状態が CallFailure に移行し、 @ref GetOutcome() で取得できる qResult が QRESULT(Core, Timemout) になります。

     @param[in]    tiTimeout    タイムアウト時間(ミリ秒）
     */
    void SetTimeout(TimeInterval tiTimeout)
    {
        SetDeadline(Time::ConvertTimeoutToDeadline(tiTimeout));
    };

    /* Description: A timeout deadline which "fails" a call if the call
    exceeds the deadline. That is, the call's state changes to
    CallContext::CallFailure the reason being Core::Timeout.
    Parameters: The deadline which takes the time in the form of the
        number of milliseconds since the session started.
    */

    //Do not expose this API publicly as it does not work as advertised in
    // all the places a call context can be used. In fact in most places
    // this deadline is ignoreded by jobs. Some exceptions to this which do
    // use the call context deadline: SyncSim.JoinSimulation,
    // AuthenticationClient.Login
    // {secret}
    void SetDeadline(Time tDeadline)
    {
        m_tDeadline = tDeadline;
    };

    // {secret}
    Time GetDeadline() const
    {
        return m_tDeadline;
    };

    // {secret}
    qBool HasDeadline() const
    {
        return m_tDeadline != Time(0);
    };

    //group=Methods

    /*
    Description: Returns the current State of the call.
    Returns: The current State of the call which is one of Available,
    CallInProgress, CallSuccess, CallFailure, or CallCancelled.
    See Also: State
    */

    /*!
    @brief    CallContext の現在の状態を返します。

    NEX のスレッドモードがスレッドアンセーフモードであっても、本関数は Scheduler::Dispatch() を含む任意の他の関数と同時に呼び出すことができます。

       @return  現在の @ref State 。 Available 、 CallInProgress 、 CallSuccess 、 CallFailure または CallCancelled のいずれかを返します。

    @see      State
    */
    State GetState() const
    {
        // 厳密にはデータ競合するが、現在サポートしているプラットフォームにおいては問題ない。
        // 必要があればm_eStateをAtomicValueに置き換える。
        return m_eState;
    }

    /*
    Description: Sets the UserContext associated to this CallContext.
    Parameters:
    iIndex: The user-defined index of the UserContext that is associated to this CallContext.
    oUserContext: A pointer to the UserContext associated with this CallContext.
    Remarks: A UserContext can be associated to a CallContext via SetUserContext and returned via
    GetUserContext.
    See Also: GetUserContext
    */

    /*!
    @brief    この CallContext に関連付けられている @ref UserContext を設定します。

    @param[in]    iIndex    この  @ref CallContext に関連付けられている @ref UserContext のユーザー定義インデックスです。
    @param[in]    oUserContext    この @ref CallContext に関連付けされている @ref UserContext へのポインタ。

    @see     GetUserContext
    */
    void SetUserContext(size_t iIndex, const UserContext& oUserContext);

    /*
    Description: Returns the UserContext associated to this CallContext.
    Parameters:
    iIndex: The user-defined index of the UserContext that will be returned.
    Returns: A pointer to the UserContext associated with this CallContext.
    Remarks: A UserContext can be associated to a CallContext via SetUserContext and returned via
    GetUserContext.
    See Also: SetUserContext
    */

    /*!
    @brief    この  @ref CallContext に関連付けられている @ref UserContext を返します。

    @return    この @ref CallContext に関連付けされている @ref UserContext へのポインタ。

    @param[in]    iIndex    返される @ref UserContext のユーザー定義インデックスです。

    @see     SetUserContext
    */
    const UserContext& GetUserContext(size_t iIndex) const;

public:
    // {secret}
    qBool InitiateCall();
    // {secret}
    qBool InitiateSpecialCall();

    // InitiateCall()が成功する状態か確認する
    qBool CanInitiate()
    {
        return GetState() != CallInProgress;
    }

    /*!
        @internal
        @brief 初期化と同時にScheduler::Dispatch()からSignalするようにする。

        内部でInitiateCall()とSignalCompletionDelay()と同等の処理を行います。
        RMCのAPIが呼び出されたときに即座にエラーとするユーティリティ関数。
        @retval true シグナルに成功。
        @retval false ステートがCallInProgressだった。
     */
    qBool InitiateCallAndSignalCompletionDelay(qResult rOutcome);
private:
    // {secret}
    qBool SetCallInProgressState();

public:
    // {secret}
    void SignalFailure()
    {
        // 第三引数のbSetOutcomeをfalseにしているのはおそらくm_rCallOutcomeの初期値が
        // QRESULT_ERROR(Core, Unknown)のため、同じものをセットしないという最適化を意図している。
        // もしくは事前にSetOutcome()でセット済みのケースを想定しているが、現状ではそういう使い方はしていなさそう。
        SetStateImpl(CallContext::CallFailure, QRESULT_ERROR(Core,Unknown), false);
    }
    // {secret}
    void SignalFailure(qResult rOutcome)
    {
        SYSTEMCHECK(rOutcome == false);
        SetStateImpl(CallContext::CallFailure, rOutcome, true);
    }
    // {secret}
    void SignalSuccess()
    {
        SetStateImpl(CallContext::CallSuccess, QRESULT_SUCCESS(Core,Success), true);
    }
    // {secret}
    void SignalSuccess(qResult rOutcome)
    {
        SYSTEMCHECK(rOutcome == true);
        SetStateImpl(CallContext::CallSuccess, rOutcome, true);
    }
    // {secret}
    // Calls into SignalSuccess/Failure, based on outcome.
    void SignalCompletion(qResult rOutcome)
    {
        rOutcome == true ? SignalSuccess(rOutcome) : SignalFailure(rOutcome);
    }

    /*!
        @internal
        @brief Scheduler::Dispatch()からSignalする。

        コールバックのタイミングをScheduler::Dispatch()まで遅延したいときに使う。
     */
    void SignalCompletionDelay(qResult rOutcome);

    // {secret}
    // Signals special terminations (e.f. Cancellation)
    // This is not the prefered interface to use.
    void SignalSpecialCompletion(State eNewState, qResult rOutcome)
    {
        SetStateImpl(eNewState,rOutcome,true);
    }

public:

    /*
    Description: Sets the flag to be used with the call.
    Parameters:
    uiFlag: The flag with which the call is called.
    See Also: ClearFlag, FlagIsSet, Flags
    */

    /*!
    @brief    非同期処理に使用するフラグを設定します。

    @param[in]    uiFlag    CallContext::Flags で定義されるフラグ。

    @see     ClearFlag, FlagIsSet, Flags
    */
    void SetFlag(qUnsignedInt32 uiFlag);

    /*
    Description: Returns whether or not the combination of flags currently in use is valid.
    Returns: true if the combination of flags currently in use is valid, false if not or an error occurs.
    See Also: Flags
    */

    /*!
    @brief    現在使用されているフラグの組み合わせが有効かどうかを返します。

    @return    現在使用されているフラグの組み合わせが有効なら true 、そうでないかエラーが起きる場合は false 。

    @see     Flags
    */
    virtual qBool FlagsAreValid() const;

    /*
    Description: Clears the flag set via SetFlag.
    Parameters:
    uiFlag: The flag with which the call is called.
    See Also: SetFlag, FlagIsSet, Flags
    */

    /*!
    @brief     @ref SetFlag を使用して設定されたフラグをクリアします。

    @param[in]    uiFlag    コールが呼び出されるフラグ。

    @see     SetFlag, FlagIsSet, Flags
    */
    void ClearFlag(qUnsignedInt32 uiFlag);

    /*
    Description: Returns whether or not a given flag is set.
    Parameters:
    uiFlag: The flag with which the call is called.
    Returns: true if the stated flag is set, false if not or an error occurred.
    See Also: SetFlag, ClearFlag, Flags
    */

    /*!
    @brief    特定のフラグが設定されているかどうかを返します。

    @return    指定のフラグが設定されれば true 、そうでないかエラーが起きた場合は false 。

    @param[in]    uiFlag    コールが呼び出されるフラグ。

    @see     SetFlag, ClearFlag, Flags
    */
    qBool FlagIsSet(qUnsignedInt32 uiFlag) const;
    // {secret}
    qUnsignedInt32 GetFlags() const
    {
        return m_uiFlags;
    }

private:
    // {secret}
    void SetState(State eNewState)
    {
        SetStateImpl(eNewState,QRESULT_ERROR(Core,Unknown),false);
    }

    // {secret}
    void SetStateImpl(State eNewState, qResult rOutcome, qBool bSetOutcome);

public:

    // {secret}
    static const qChar* GetStateString(State eState);

    /*
    Description: Sets the operations to perform when this CallContext completes.
    Parameters:
    pfCompletionCallback: A pointer to the call back function for the CallContext.
    oContext: The UserContext associated with this CallContext.
    pCallback: A pointer to the Callback object to be called on completion.
    bCallOnSuccess: If true (the default), the Callback object will be called only if
    the CallContext succeeds.
    bAddToEnd: If true (the default), the pCallback will be added to the end of the list
    of callbacks. If false, the callback will be added to the beginning of the list.

    Remarks: A CompletionCallback presents an easy and efficient way to ensure a given function
    is called upon completion of the CallContext. When methods are called asynchronously, use of this
    callback avoids having to continually poll the CallContext to determine whether or not it has
    completed. If multiple callbacks are registered, then by default they are stored and called in the order that
    they were registered, with the first registered callback being called first. If you wish to change the order
    you need to set the bAddToEnd parameter when you register a given callback.

    A complete example of the implementation of a completion callback is given in the Inventory and Item classes
    of the Real-time 3D Framework, part of which is given below. Refer to the framework source code for full details.
    First we declare the ChangeOwner RMC in the DDL:

                // DDL declarations for Item class.
                doclass Item : Duplicated3DObject {
                    ItemOwnership m_dsOwner;
                    bool ChangeOwner(in dohandle hCurrentOwner, in dohandle hNewOwner,
                    dohandle hTargetItemMaster);
                };

    Then in the /Inventory.cpp/ file we implement the PickUp method that uses the ChangeOwner RMC:

    $import CallContextRegister1 TOP/Frameworks\Eterna\RealTime3D\FoundationClasses\Inventory.cpp$

    Then we call the RMC as follows. When the RMC completes, the method Inventory::AddItem will be called.

    $import CallContextRegister2 TOP/Frameworks\Eterna\RealTime3D\FoundationClasses\Inventory.cpp$

    $ifdef RENDEZVOUS$
    $else$
    See Also: Callback
    $endif$
    */

    /*!
    @refmanoverload{name1}

    @brief    この @ref CallContext の完了時に実行するコールバック関数を設定します。

    設定したコールバックはこの @ref CallContext が完了状態に遷移したときに呼び出されます。
    このコールバックを使用することで、 @ref CallContext が完了したかどうかを確認するためにポーリングする手間を省くことができます。

    本関数を複数回呼び出すことで複数のコールバックを登録することが出来ます。
    複数のコールバックを登録した場合はデフォルトでは登録順にコールバックの呼び出しが行われ、最初に登録されたコールバックは最初に呼び出されます。
    この順序を変更したい場合は、コールバックを登録する際に bAddToEnd パラメータを設定する必要があります。

    コールバック関数内では NEX のブロック関数を呼び出さないようにしてください。

    @param[in]    pfCompletionCallback     完了時に呼び出されるコールバック関数へのポインタ。
    @param[in]    oContext     呼び出されるコールバック関数へ渡される @ref UserContext 。
    @param[in]    bAddToEnd    true（デフォルト）であれば、コールバックはリストの最後に追加されます。falseであればリストの先頭に追加されます。

    */
    void RegisterCompletionCallback(CompletionCallback pfCompletionCallback, const UserContext& oContext, qBool bAddToEnd=true);

    /*!
    @refmanoverload{name2}

    @brief    この  @ref CallContext の完了時に実行するコールバックオブジェクトを設定します。

    設定したコールバックはこの @ref CallContext が完了状態に遷移したときに呼び出されます。
    このコールバックを使用することで、 @ref CallContext が完了したかどうかを確認するためにポーリングする手間を省くことができます。

    本関数を複数回呼び出すことで複数のコールバックを登録することが出来ます。
    複数のコールバックを登録した場合はデフォルトでは登録順にコールバックの呼び出しが行われ、最初に登録されたコールバックは最初に呼び出されます。
    この順序を変更したい場合は、コールバックを登録する際に bAddToEnd パラメータを設定する必要があります。

    コールバック関数内では NEX のブロック関数を呼び出さないようにしてください。

    @param[in]    pCallback    完了時に呼び出されるコールバックオブジェクトへのポインタ。
    @param[in]    bCallOnSuccess    真（デフォルト）であれば、 コールバックは非同期処理の成功時にのみ呼び出されます。
    @param[in]    bAddToEnd    true（デフォルト）であれば、コールバックはリストの最後に追加されます。falseであればリストの先頭に追加されます。

    */
    void RegisterCompletionCallback(CallbackRoot* pCallback, qBool bCallOnSuccess=true, qBool bAddToEnd=true);

    // {secret}
    void RegisterCancellationCallback(CallbackRoot* pCallback);



private:
    // {secret}
    // Called when the call is initiated.
    void Register();

    // {secret}
    // Called when the CallContext reaches a final state.
    void Unregister();

public:
    // {secret}
    qBool IsRegistered() const;

    /*
    Description: Resets the State of the call to Available.
    Remarks: This method may only be called if the current State of the call is one of
    Sucess, Failure, or Cancelled. The error SYSTEMERROR_GEN_INVALID_OPERATION will
    result if this method is called when the current State is
    CallInProgress.
    If the State is Available, the method returns true, but it behaves as a no-op.
    See Also: State
    */

    /*!
    @brief    CallContext の状態を利用可能に再設定します。

    このメソッドは、 @ref State が Sucess 、 Failure または Cancelled のいずれかひとつである場合のみ、呼び出すことができます。

    @ref State が CallInProgress のときに本関数が呼び出されると、エラー SYSTEMERROR_GEN_INVALID_OPERATION になります。
    @ref State が Available の場合、本関数は true を返しますが、何も起こりません。

    @see     State
    */
    qBool Reset();

    /*
    Description: Causes the system to wait until the Outcome of the CallContext is known or the timeout is exceeded.
    Parameters:
    uiTimeout: The maximum time in milliseconds that the system will wait. The default
    uiTimeout is infinite.
    Returns: true if the outcome of the CallContext is known, false if not or
    the timeout is exceeded. The possible outcomes of a given call are specific to the call
    that is made.
    Remarks: This method may only be called if the current CallContext::State of the call is
    CallInProgress and the flag (DOCallContext::Flags) SynchronousCall is set to false. The method
    returns true if the Wait has succeeded, /i.e./ the state is no longer CallInProgress. The error
    SYSTEMERROR_GEN_TIMEOUT will be generated if the timeout is exceeded.

    In order to prevent deadlocks, if this method is called on an embeded RMC then the only valid
    uiTimeout is zero. If any other timeout is set then the error SYSTEMERROR_GEN_INVALID_WAIT will
    be signalled. Refer to CallMethodOperation for an example of how to use this method with embeded RMCs.

    $ifdef ETERNA$
    Refer to DuplicatedObject::FetchDuplica for an example of this method in use.
    $endif$

    See Also: State, GetOutcome
    */

    /*!
    @brief     非同期処理が完了する、またはタイムアウトを越えるまで内部処理を進め、待機します。

    本関数は、 CallContext の @ref State が CallInProgress の場合に呼び出すことができます。
    非同期処理が完了する、またはタイムアウトを越えるまで本関数の内部で @ref Scheduler::Dispatch() が呼び出されます。

    タイムアウトを越えた場合は、エラー SYSTEMERROR_GEN_TIMEOUT が発生します。
    タイムアウトを越えた場合でも非同期処理が継続するため、非同期処理のout引数は解放できません。
    out引数を解放したい場合は明示的に @ref CallContext::Cancel() を呼び出してください。

    @param[in]    uiTimeout    待機する最大の時間（ミリ秒）。デフォルトのuiTimeoutは無限大です。

    @return     非同期処理が完了した場合は true を、タイムアウトを越えた場合は false を返します。
                false を返した場合の CallContext::GetOutcome() の値は不定です。

    @see     State,  CallContext::GetOutcome
    */
    qBool Wait(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    /*!
    @brief     非同期処理が完了する、またはタイムアウトを越えるまで待機します。

    本関数は、 CallContext の @ref State が CallInProgress の場合に呼び出すことができます。
    非同期処理が完了する、またはタイムアウトを越えるまで処理を戻しません。
    CallContext::Wait() とは異なり、本関数内部で Scheduler::Dispatch() は呼び出されません。
    内部処理を進めるには他のスレッドで Scheduler::Dispatch() などを呼び出す必要があります。
    NEX のスレッドモードがスレッドアンセーフモードであっても、
    本関数は Scheduler::Dispatch() を含む任意の他の関数と同時に呼び出すことができます。

    タイムアウトを越えた場合でも非同期処理が継続するため、非同期処理のout引数は解放できません。
    out引数を解放したい場合は明示的に @ref CallContext::Cancel() を呼び出してください。

    @param[in]    uiTimeout    待機する最大の時間（ミリ秒）。デフォルトのuiTimeoutは無限大です。

    @return     非同期処理が完了した場合は true を、タイムアウトを越えた場合は false を返します。
                false を返した場合の CallContext::GetOutcome() の値は不定です。

    @see     State,  CallContext::GetOutcome
    */
    qBool WaitWithoutDispatch(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);

    /*
    Description: Cancels the call that is associated to this CallContext.
    Parameters:
    eNewState: The new State of this CallContext. The default value is CallCancelled.
    Returns: true if the call was successfully cancelled, false if not or an error occurred.
    The error SYSTEMERROR_GEN_INVALID_OPERATION will be returned if this method is called when the
    current State of the call is any state other than CallInProgress.
    Remarks: When this method is called the CallContext::State of the call will be set to CallCancelled
    and the place holder for the results deleted. If the call associated with the context has not yet been
    issued, it will be cancelled, if it has already been issued it cannot then be cancelled but
    the issuing station will ignore the return message, if any. This method may only be called
    if the current state of the call is CallInProgress.

    See Also: State, Reset
    */

    /*!
    @brief    この  @ref CallContext に関連付けられている非同期処理をキャンセルします。

    CallContext に関連付けされた非同期処理のうち、まだ実行されていない処理を放棄します。
    本関数から戻った後は非同期処理開始時にセットした out 引数を解放する事ができます。

    すでに実行されている処理については取り消す事は出来ず、どこまで処理されたかを知る方法はありません。
    例えば、サーバーに対して何らかのリクエストを行う処理をキャンセルした場合、サーバーへのリクエストを送信した後に
    キャンセルが行われてもそのリクエストはサーバーで処理されます。その結果を呼び出し側で受信してもそれを無視するという動作になります。

    主に非同期処理の結果を無視して良い状況で非同期処理開始時にセットしたout引数を解放する目的で使用してください。

    本関数で非同期処理がキャンセルされた場合、返値は true となり非同期処理の結果は QERROR(Core, Cancelled) になります。
    ステートが CallInProgress 以外だった場合にキャンセルした場合は返値は false となり、キャンセル処理は行われません。

    @retval true  非同期処理がキャンセルされた。(ステートが CallInProgress だった場合)
    @retval false 非同期処理が開始されていないものをキャンセルしようとした。(ステートが Available だった場合。)
                  もしくはすでに非同期処理が完了していた。(ステートが CallSuccess もしくは CallFailure もしくは CallCancelled だった場合。)

    @see     State ,  Reset
    */
    qBool Cancel();

    // {secret}
    qUnsignedInt32 GetCallID() const
    {
        return m_uiID;
    }

    // {secret}
    void SetDependentConnection(void* pConnection, ConnectionID idConnection);
    // {secret}
    void* GetDependentConnection() const
    {
        return m_pDependentConnection;
    }
    // {secret}
    ConnectionID GetDependentConnectionID() const
    {
        return m_idConnection;
    }

    /*
    Description: Returns the Outcome of the call as a qResult.
    Returns: The Outcome of the call as a qResult.
    Remarks: The possible outcomes of a given call are specific to the call
    that is made. Refer to the documentation for the specific calls that may take a
    CallContext as a parameter for a description of the possible outcomes for that call.
    */

    /*!
    @refmanoverload{name1}

    @brief    非同期処理の結果を @ref qResult として取得します。

    特定の非同期処理に対する結果を取得します。
    結果の内容については @ref CallContext をパラメータとしてとる関数を参照してください。

    @return     @ref qResult としての非同期処理の結果。
    */
    qResult GetOutcome() const
    {
        return m_rCallOutcome;
    }

    void Swap(CallContext& rhs) NEX_NOEXCEPT
    {
        std::swap(m_uiFlags, rhs.m_uiFlags);
        std::swap(m_eState, rhs.m_eState);
        std::swap(m_lstCompletionCallback, rhs.m_lstCompletionCallback);
        std::swap(m_vUserContext, rhs.m_vUserContext);
        std::swap(m_rCallOutcome, rhs.m_rCallOutcome);
        std::swap(m_uiID, rhs.m_uiID);
        std::swap(m_pDependentConnection, rhs.m_pDependentConnection);
        std::swap(m_idConnection, rhs.m_idConnection);
        std::swap(m_pCancellationCallback, rhs.m_pCancellationCallback);
        std::swap(m_tDeadline, rhs.m_tDeadline);
    }
    // {secret}
    qBool CallIsComplete() const
    {
        return StateIsTerminal(m_eState);
    }

    // {secret}
    qBool HasExpired(Time tNow)
    {
        if  (m_tDeadline == NAMESPACE_NEX Time(0))
        {
            return false;
        }
        else
        {
            return (tNow > m_tDeadline ? true :false);
        }
    }

#ifdef NEX_TRACE_ENABLED    // n2370
    /*!
    @brief  この @ref CallContext オブジェクトの内容をトレースします。
            リリースモードで実行されると自動的に無効になります。
    @param[in]  uiTraceFlags    @ref TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);
#else
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS)
    {
        DUMMY_PARAM(uiTraceFlags);
    }
#endif  // NEX_TRACE_ENABLED

    /*!
    @brief    この @ref CallContext の完了時に実行するコールバックを設定します。

    設定したコールバックはこの @ref CallContext が完了状態に遷移したときに呼び出されます。
    このコールバックを使用することで、 @ref CallContext が完了したかどうかを確認するためにポーリングする手間を省くことができます。

    callback には @ref CallContext* を引数に取る関数ポインタ、関数オブジェクト、ラムダ式を指定できます。
    AddCallContextArgumentAdapter() を併用すると引数を一つも取らない関数を渡すこともできます。

    関数オブジェクトを渡す場合は値もしくは参照型で渡す方法とポインタ型で渡す方法の二種類があり、それぞれ挙動が異なります。
    値もしくは参照型で渡した場合は内部で関数オブジェクトがコピーもしくはムーブされ、コールバックはそのコピーもしくはムーブされたオブジェクトに対して行われます。
    そのため、本関数呼び出し後は指定したオブジェクトを即座に破棄する事ができます。
    ポインタ型で渡した場合は、コールバックはそのポインタが指すオブジェクトに対して行われます。
    そのため、コールバックが呼び出されるまでその関数オブジェクトを呼び出し側で保持しておく必要があります。

    本関数を複数回呼び出すことで複数のコールバックを登録することが出来ます。
    複数のコールバックを登録した場合はデフォルトでは登録順にコールバックの呼び出しが行われ、最初に登録されたコールバックは最初に呼び出されます。
    この順序を変更したい場合は、コールバックを登録する際に bAddToEnd パラメータを設定する必要があります。

    callback に関数オブジェクトをポインタで渡した場合は deleteCallbackAfterCall を true にする事で
    callback の呼び出し後に自動的に callback を delete することができます。
    もし callback を delete するのではなくアプリケーション独自の解放処理を行いたい場合は、
    再度本関数を呼び出しそのための関数オブジェクトを追加する事で実装が可能です。

    コールバック関数内では NEX のブロック関数を呼び出さないようにしてください。

    @param[in]  callback コールバックする関数ポインタもしくは関数オブジェクトもしくはラムダ式。
    @param[in]  deleteCallbackAfterCall callback の実行が完了した後自動的に callback を delete するかどうか。
    @param[in]  bAddToEnd true（デフォルト）であれば、コールバックはリストの最後に追加されます。falseであればリストの先頭に追加されます。
    */
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(TCallback&& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<CallContext>(std::forward<TCallback>(callback), deleteCallbackAfterCall, bAddToEnd);
    }
#else
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(const TCallback& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<CallContext>(callback, deleteCallbackAfterCall, bAddToEnd);
    }
#endif

protected:

    // {secret}
    static qBool TransitionIsValid(State eInitialState, State eNewState);
    // {secret}
    static qBool StateIsTerminal(State eState)
    {
        return (eState==CallSuccess || eState==CallFailure || eState==CallCancelled);
    }
    // Used by the RMCContext - for now.
    //{secret}
    void SetOutcome(qResult rOutcome)
    {
        m_rCallOutcome = rOutcome;
    }

    //{secret}
    qBool CancelImpl();

    template<typename TRealCallContextType, typename TCallback>
    void RegisterCompletionCallbackGenericImpl(const TCallback& callback, qBool deleteCallbackAfterCall, qBool bAddToEnd = true)
    {
        if (!m_pCompletionCallbacks)
        {
            m_pCompletionCallbacks = qNew qVector<CallbackHolderBase*>();
        }
        m_pCompletionCallbacks->push_back(qNew CallbackHolder<TCallback, TRealCallContextType*>(
            callback,
            deleteCallbackAfterCall));
        UserContext uc(qUnsignedInt32(m_pCompletionCallbacks->size() - 1));
        RegisterCompletionCallback(MyCompletionCallback, uc, bAddToEnd);
    }
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    template<typename TRealCallContextType, typename TCallback>
    void RegisterCompletionCallbackGenericImpl(TCallback&& callback, qBool deleteCallbackAfterCall, qBool bAddToEnd = true)
    {
        if (!m_pCompletionCallbacks)
        {
            m_pCompletionCallbacks = qNew qVector<CallbackHolderBase*>();
        }
        m_pCompletionCallbacks->push_back(qNew CallbackHolder<TCallback, TRealCallContextType*>(
            std::forward<TCallback>(callback),
            deleteCallbackAfterCall));
        UserContext uc(qUnsignedInt32(m_pCompletionCallbacks->size() - 1));
        RegisterCompletionCallback(MyCompletionCallback, uc, bAddToEnd);
    }
#endif

private:

    // {secret}
    virtual void BeginTransition(State /*eNewState*/, qResult /*rOutcome*/, qBool /*bSetOutcome*/) {}

    // Deactivate copy-ctor + op=
    // {secret}
    CallContext(const CallContext&);
    // {secret}
    CallContext& operator=(const CallContext&);

    // RegisterCompletionCallbackGeneric用のユーティリティ関数
    void CallCallback(qUnsignedInt32 index)
    {
        SYSTEMCHECK(m_pCompletionCallbacks != NULL);
        SYSTEMCHECK(index < m_pCompletionCallbacks->size());
        (*m_pCompletionCallbacks)[index]->Call(this);
    }

    // RegisterCompletionCallbackGeneric用のユーティリティ関数
    static void MyCompletionCallback(CallContext* cc, const UserContext* uc)
    {
        cc->CallCallback(uc->GetUIntValue());
    }

    void DestroyCompletionCallbacks();

protected:
    // {secret}
    virtual void ProcessCallCompletion() {}

    qUnsignedInt32 m_uiFlags;

private:
    //{secret}
    struct InternalData
    {
        CompletionCallback m_Callback;
        UserContext        m_Context;

        InternalData( const CompletionCallback &cb,
                      const UserContext &c ) : m_Callback(cb), m_Context(c) {}
    };

    State m_eState;
    /**
     * @brief identifier of the CallContext instance.
     * @remarks this value is zero if the instance is not registered to CallContextRegister.
     *          otherwise, the instance is registered to CallContextRegister.
     *          to check reigstered or not registered, call IsRegistered function.
     */
    qUnsignedInt32 m_uiID;

    qList<InternalData> m_lstCompletionCallback;
    qVector<UserContext> m_vUserContext;
    qVector<CallbackHolderBase*>* m_pCompletionCallbacks;   // RegisterCompletionCallbackGenericで設定したコールバックを覚えるvector

    qResult m_rCallOutcome;

    void* m_pDependentConnection;
    ConnectionID m_idConnection;
    CallbackRoot* m_pCancellationCallback;
    Time          m_tDeadline;
};

END_NAMESPACE_NEX
