﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/CallbackRoot.h>


BEGIN_NAMESPACE_NEX

//Group=Template Classes

/*
Description: Use this template class to construct custom callbacks.
Remarks: Via this template class you can construct and define user-defined completion callbacks that can
be called as part of a CallContext, for example. Typically, a Callback is constructed as follows:

        Callback* pCallback = qNew Callback<Inventory, DOHandle>
                            (pInventory, &Inventory::AddItem, hItem);

An example of the use of a completion callback is given in the Inventory class of the Real-time 3D Framework.
Refer to the source code of this class for further details.

See Also: CallContext, CallContext::RegisterCompletionCallback
*/

/*!
 @brief    ユーザー定義コールバックのテンプレートクラスです。

 このテンプレートクラスを用いることで、非同期処理完了時に呼び出されるコールバック関数を容易にカスタマイズできます。
 使用方法については、CallContextBasicサンプルデモを参照してください。

 @tparam    TargetClass    ターゲットクラス
 @tparam    ArgumentType    コールバック関数に引数として渡されるクラス

 @see    CallContext, CallContext::RegisterCompletionCallback
 */
template<class TargetClass, class ArgumentType> class Callback: public CallbackRoot
{
public:

    /*
    Parameters:
    pTargetObject: The duplicated object that the callback method, pMethod, will be called on.
    pMethod: The callback method that is part of the TargetClass class.
    arg: The arguments of the callback method, pMethod.
    */

    /*!
     @brief    コンストラクタです。

     @param[in]    pTargetObject    ターゲットオブジェクト
     @param[in]    pMethod    コールバック関数
     @param[in]    arg    コールバック関数に渡す引数
     */
    Callback(TargetClass* pTargetObject,
             void (TargetClass::*pMethod)(ArgumentType arg),
             ArgumentType arg):
        m_pTargetObject(pTargetObject),
        m_pMethod(pMethod),
        m_arg(arg)
    {
    }

    /*!
     @brief    デストラクタです。

     非同期処理完了後のコールバック関数実行後に自動的にデストラクタが呼ばれるため、
     CallContext::RegisterCompletionCallback でセットした場合はデストラクタを明示的に
     呼ばないようにしてください。
     （非同期処理成功時にのみコールバックが実行される場合はこの限りではありません）
     */
    virtual ~Callback() {}

private:

    virtual void CallObjectMethod()
    {
        void (TargetClass::*pMethod)(ArgumentType arg)=m_pMethod;
        (m_pTargetObject->*pMethod)(m_arg);
    }

private:
    TargetClass* m_pTargetObject;
    void (TargetClass::*m_pMethod)(ArgumentType arg);
    ArgumentType m_arg;
};

END_NAMESPACE_NEX

