﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/Checksum/Interface/ChecksumDecl.h>
#include <OnlineCore/src/Core/Checksum/Interface/SignatureBytes.h>
#include <OnlineCore/src/Core/PluginObject.h>
#include <OnlineCore/src/Core/Key.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>
#include <OnlineCore/src/Core/Job.h>

//Group=Checksum Algorithms

/*
Summary: The default length of the checksum.
Remarks: The default length of the various checksums provided with $PRODUCTNAME$ are:
    o   CRC -- 32 bits
    o   RSA MD5 -- 128 bits
    o   HMAC -- 16 bits
    o   UMAC -- 8 bits
*/

BEGIN_NAMESPACE_NEX

class Buffer;
class Plugin;
class CallContext;
class KeyCache;

/*
Description: Specifies atributes for the checksum algorithm.
Remarks: This class is used to specify specific atributes of the checksum algorithm used by
$PRODUCTNAME$. The chosen checksum algorithm will be used to calculate the checksum for all
transmitted messages.

The available checksum algorithms are: CRC, RSA MD5, HMAC, and UMAC, with the latter two
being keyed algorithms. Keyed algorithms offer a more secure checksum for a given checksum
length than non-keyed ones. The choice of checksum is a trade-off between the checksum
length and message security. A long checksum is more secure than a short one. However, it takes
more time and resources to calculate, which increases the delay between when a message is
sent and when the target station receives it. Comparing the checksums provided with
$PRODUCTNAME$; the CRC algorithm is the fastest but least secure checksum. The RSA MD5
algorithm is the most secure. However, due to its length it is very time consuming to calculate.
The two keyed algorithms offer greater security than the CRC algorithm. The UMAC algorithm
is almost as fast as the CRC algorithm and the HMAC algorithm approximately an order of
magnitude slower.

For further information on the checksum algorithms used by $PRODUCTNAME$, refer to standard
cryptography and authentication literature references.
See Also: CRCChecksum, MD5Checksum, HMACChecksum, UMACChecksum
*/
class ChecksumAlgorithm: public PluginObject
{

public:
    //Group=Methods
    static const size_t NATURAL_CHECKSUM_LENGTH=0;
    //{secret}
    ChecksumAlgorithm();
    //{secret}
    virtual ~ChecksumAlgorithm();

    /*
    Summary: Reduces the checksum length to the byDesiredChecksumLength.
    Parameters:
    byDesiredChecksumLength: The desired checksum length in bytes. This value must be
    between 0 and the NATURAL_CHECKSUM_LENGTH. If a value of 0 or one greater than the
    NATURAL_CHECKSUM_LENGTH is given, then the NATURAL_CHECKSUM_LENGTH of the checksum
    algorithm is used.

    Remarks: Decreases the checksum length to the byDesiredChecksumLength.
    Returns: The actual length of the checksum that will be transmitted.
    See Also: GetChecksumLength
    */
    qByte ReduceChecksumLength(qByte byDesiredChecksumLength);  //n1769 memo: チェックサムを通常より短くする。

    // {secret}
    void CompressChecksum(Buffer* pChecksum);  //n1769 memo: 「圧縮」と呼ぶより、「切り詰める」と表現した方が正確な気がする。
    // {secret}
    virtual qBool ComputeChecksum(const Buffer &oBuffer, Buffer *pChecksum) =0;

    virtual qBool ComputeChecksum(const qByte** pBuffer, const size_t nSize[], size_t length, SignatureBytes &signature) =0;
    // {secret}
    virtual qBool IsReady() const
    {
        return true;
    }

    // {secret}
    virtual qUnsignedInt32 ComputeChecksumForTransport( const qByte* pBuffer, size_t nSize )
    {
        return ComputeChecksumForTransportArray(&pBuffer, &nSize, 1);
    }
    // {secret}
    virtual qUnsignedInt32 ComputeChecksumForTransportArray( const qByte** pBuffer, const size_t nSize[], size_t length ) = 0;

    //Summary: Returns the checksum length.
    //Returns: The checksum length in bytes.
    //Remarks: This value will be equal to the natural checksum length unless the length of
    //the checksum has been changed via ReduceChecksumLength.
    //See Also: ReduceChecksumLength
    virtual qByte GetChecksumLength() =0;

    //{secret}
    qBool AppendChecksum(Buffer *pBufferToChecksum);  //n1769 memo: pBufferToChecksumの末尾に、計算したチェックサムをくっつける。
    //{secret}
    qBool RemoveChecksum(Buffer *pBuffer);            //n1769 memo: pBufferからチェックサム部分を取り除く。チェックサムが正当ならばtrue。チェックサムが不当であったり、内部不整合を起こしている場合はfalseが返る。

    /*
    Summary: Derives a key from a password.
    Parameters:
    strPassword: The password as a string.
    uiNbIterations: The number of times that the password will be hashed using the
    chosen checksum algorithm. The higher this number, the harder it is to reverse
    calculate the password and the safer the authentication.

    Remarks: This method uses the checksum algorithm specifed via *which method?* to perform the hash.
    Returns: A key derived from the given password as a Key object.
    */
    Key DeriveKey(const qChar8 *szPassword, qUnsignedInt32 uiNbIterations=0xFFFF);  //n1769 memo: パスワードからキーを得る。
    Key DeriveKey(const Buffer & oBuffer, qUnsignedInt32 uiNbIterations=0xFFFF);    //n1769 memo: パスワードからキーを得る。

    static qBool DeriveKey(CallContext* pCallContext,
                           ChecksumAlgorithm* pChecksumAlgo,
                           const qChar8 *szPassword, qUnsignedInt32 uiNbIterations,
                           PrincipalID pid, qUnsignedInt32 uiSecondNbIterations,
                           Key* pkeyResult, Job::JobType jobTypeToUse = Job::RV
                          );

    static qBool DeriveKey(CallContext* pCallContext,
                           ChecksumAlgorithm* pChecksumAlgo,
                           const Buffer & oBuffer, qUnsignedInt32 uiNbIterations,
                           const Buffer & oSecondBuffer, qUnsignedInt32 uiSecondNbIterations,
                           Key* pkeyResult, Job::JobType jobTypeToUse = Job::RV
                          );

private:

    // Number of bytes appended to the buffer.
    //{secret}
    qByte m_byTransmittedChecksumLength;  //n1769 memo: 名前が紛らわしいが、実際に使われるチェックサムの長さが格納される（デフォルト長から切り詰められることがある。その値）。
};

END_NAMESPACE_NEX





