﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/UtilityDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Core/LocalClock.h>

/*
X = The function call is not allowed
O = The function call is allowed

State/Function  | Start | Pause | Resume | Check | Stop | Reset | GetState
---------------------------------------------------------------------------
Paused          |   X   |   X   |   O    |   O   |   O   |  O   |    O
Started         |   X   |   O   |   X    |   O   |   O   |  O   |    O
Stopped         |   O   |   X   |   X    |   O   |   X   |  O   |    O
*/





BEGIN_NAMESPACE_NEX

//Group = Time Management

/*
Description:  Measures the elapsed time from Start to Stop.
Remarks:  An instance of this class measures the elapsed time, in milliseconds, on
the local station. Use an instance of this class to precisely measure time in the
same manner as you would with a stopwatch.
*/

/*!
 *  @brief Start()からStop()の間の経過時間を図ります。
 *
 *  ローカルステーション上の経過時間をミリ秒単位で計測します。
 *  ストップウォッチのように中止と再開との間の停止時間を除いた経過時間の計測が可能です。
 *
 */
class Chrono: public RootObject
{

    //Group =Methods

public:
    enum
    {
        CHRONO_IS_STARTED=0,
        CHRONO_IS_PAUSED =1,
        CHRONO_IS_STOPPED=2,
        INVALID_TIME_INTERVAL=-1
    };

    //Remarks:  By default, upon creation the Chrono is stopped. Use Start to start it.

    /*! @brief コンストラクタです。
     *
     *  Chronoクラスは、ストップした状態になります。
     */

    Chrono();

    /*! @brief デストラクタです。
     *
     */
    virtual ~Chrono();

    //Description: Starts the Chrono at an initial time of zero. This method is
    //equivalent to pushing 'start' on a stopwatch.
    //Remarks:  You can only Start a Chrono that is presently stopped.
    //Returns: true if the Chrono was successfully started, false if not or an error occurred.
    // SYSTEMERROR_GEN_INVALID_OPERATION is signalled if the Chrono is not stopped.

    /*! @brief 時間計測を0[msec]からスタートさせます。
     *
     *  ストップウォッチで、スタートを押した場合と同じ処理です。
     *
     *  @retval true スタートできたときに返します。
     *  @retval false ストップしておらず、スタートできなかった場合に返します。
     *
     *  @see Stop()
     */
    qBool Start();

    //Description: Pauses the Chrono if it was currently running.
    //Remarks: Pauses the Chrono until you resume it via Resume. You can resume the Chrono
    //after pausing it, which you cannot do if you Stop the Chrono.
    //Returns: The current elapsed time of the Chrono. The error SYSTEMERROR_GEN_INVALID_OPERATION is
    //signalled if this method is called and the Chrono is not started.

    /*! @brief 時間計測を中断します。
     *
     *  Stop()しない限り、Resume()によって、中断した時間計測を再開できます。
     *
     *  @retval 0以上 中断し、時間計測結果を返します。
     *  @retval -1 スタートしておらず、中断できなかった場合に返します。
     *
     *  @see Resume(), Stop()
     *
     */
    TimeInterval Pause();

    //Description: Resumes the Chrono if it was temporarily paused.
    //Remarks: Resumes the operation of the Chrono if it was in a paused state.
    //You can Resume the Chrono after pausing it, which you cannot do if you stop the Chrono.
    //Returns: true if the Chrono successfully resumed operation, false if not or an error occurred.
    //The error SYSTEMERROR_GEN_INVALID_OPERATION is signalled if this method is called and the Chrono is not paused.
    /*! @brief 時間計測を再開します。
     *
     *  Pause()により中断した時間計測を、再開できます。
     *
     *  @retval true 再開できたときに返します。
     *  @retval false 中断しておらず、再開できなかった場合に返します。
     *
     *  @see Pause()
     *
     */
    qBool Resume();

    //Description: Returns the current elapsed time without stopping the Chrono.
    //Returns: The current elapsed time of the Chrono.
    //Remarks:  You can check the Chrono at anytime.

    /*! @brief 時間計測結果を取得します。
     *
     *  いつでも取得可能です。
     *
     *  @return 経過時間[msec]
     */
    TimeInterval Check();

    //Description: Stops the chrono and returns the elapsed time since the Chrono was started.
    //Remarks: The time given by the Chrono is the elapsed time since the Chrono
    //was started excluding the time that the Chrono was paused.
    //Returns: The current elapsed time of the Chrono. The error SYSTEMERROR_GEN_INVALID_OPERATION is signalled
    // if this method is called on a Chrono that is not started.
    /*! @brief 時間計測をストップさせます。
     *
     *  ストップウォッチで、ストップを押した場合と同じ処理です。
     *
     *  @retval true ストップできたときに返します。
     *  @retval false スタートしておらず、ストップできなかった場合に返します。
     *
     *  @see Start()
     */
    TimeInterval Stop();

    //Description:  Resets the Chrono to zero.
    //Remarks: Sets the state of the Chrono to stopped and resets the time to zero.
    //You may Reset the Chrono at anytime.

    /*! @brief 時間計測をリセットして、ストップした状態にさせます。
     *
     */
    void Reset();

    // {secret}
    // Description:
    // Return the present state of the Chrono.
    // Remarks:  The state if one of: CHRONO_IS_STARTED, CHRONO_IS_STOPPED, CHRONO_IS_PAUSED.
    qUnsignedInt32 GetState() const;

private:
    // {secret}
    TimeInterval Elapsed() const;

    // {secret}
    void UpdateAccumulatedTime();


private:
    LocalClock m_oClock;
    Time m_tInitialTime;
    TimeInterval m_tiAccumulatedTime;

    qUnsignedInt32 m_uiState;
};

END_NAMESPACE_NEX

