﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/Compression/Interface/CompressionDecl.h>
#include <OnlineCore/src/Core/PluginObject.h>
#include <OnlineCore/src/Platform/Core/ProfilingUnit.h>

BEGIN_NAMESPACE_NEX

class Buffer;
class Plugin;

//group=Compression

/*
Description: The root class of any compression algorithm that $PRODUCTNAME$ uses.
Remarks: This is the root class of any compression algorithm used by $PRODUCTNAME$. The
compression algorithm used by $PRODUCTNAME$, if any, is set via StreamSettings::SetCompressionAlgorithm.
If you wish to implement your own compression algorithm, it must inherit from this class.
For further details on how this is done, contact Quazal Technical Support.
See Also: Key, StreamSettings::SetCompressionAlgorithm
*/
class CompressionAlgorithm: public PluginObject
{
public:
    //group=Methods

    CompressionAlgorithm();
    virtual ~CompressionAlgorithm();

    //  TODO: more explicit error codes eventually

    //{secret}
    enum CompressStatus
    {
        CompressSuccess,
        CompressFailure //  Generic failure for now
    };

    //{secret}
    static const qChar* GetCompressStatusText(CompressStatus eStatus);

    //{secret}
    CompressStatus Compress(const Buffer &oInBuffer, Buffer *pOutBuffer);
    //{secret}
    CompressStatus Compress(Buffer *pInOutBuffer);

    //  TODO: more explicit error codes eventually

    //{secret}
    enum DecompressStatus
    {
        DecompressSuccess,
        DecompressFailure,  //  Algorithm returned some error code
        DecompressOverflow  //  Through to undefined behavior, e.g. buffer to unpack
        //  may have been compressed by a totally different algorithm.
    };

    //{secret}
    static const qChar* GetDecompressStatusText(DecompressStatus eStatus);

    //{secret}
    DecompressStatus Decompress(const Buffer &oInBuffer, Buffer *pOutBuffer);
    //{secret}
    DecompressStatus Decompress(Buffer *pInOutBuffer);

    /*
    Description: Returns the compression ratio of this CompressionAlgorithm.
    Returns: The compression ratio of this CompressionAlgorithm.
    Remarks: The compression ratio quantifies the reduction in data quantity achieved by a data compression algorithm.
    It is equal to the ratio of the uncompressed data to the compressed data.
    The returned value is based upon the last time data was compressed by using the current instance.
    It is set to zero initially.
    */
    qReal GetCompressionRatio();

    //{secret}
    ProfilingUnit *GetCompressionProfiling()
    {
        return &m_puCompression;
    }
    //{secret}
    ProfilingUnit *GetDecompressionProfiling()
    {
        return &m_puDecompression;
    }

private:

    void RecordCompressionStats(const Buffer &oInBuffer, const Buffer &oOutBuffer);

    virtual CompressStatus CompressImpl(const Buffer &oInBuffer, Buffer *pOutBuffer)=0;
    virtual DecompressStatus DecompressImpl(const Buffer &oInBuffer, Buffer *pOutBuffer)=0;

    size_t m_uiUncompressedDataSize;
    size_t m_uiCompressedDataSize;

    CriticalSection m_csCompression;

private:
    ProfilingUnit   m_puCompression;
    ProfilingUnit   m_puDecompression;
};

END_NAMESPACE_NEX





