﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>
#include <OnlineCore/src/Core/SystemSetting.h>

BEGIN_NAMESPACE_NEX

// For NEX Developer <wiki/Pseudo Singleton#CorePseudoSingleton>
// The instances it manages.

class CallContextRegister;
class Scheduler;
class SystemComponents;
class SecurityContextManager;

#define NEX_MANAGEINSTANCE_(TYPE,NAME)      \
    public:                             \
        const TYPE* Get##NAME() const {return m_p##NAME;}       \
        TYPE* Get##NAME() {return m_p##NAME;}               \
    private:                                \
        TYPE *m_p##NAME

#define NEX_MANAGEGLOBALINSTANCE_(TYPE,NAME)    \
    public:                                 \
        static TYPE* Get##NAME() {return s_p##NAME;}                \
    private:                                \
        static TYPE *s_p##NAME


#define NEX_MANAGEINSTANCE(C)   NEX_MANAGEINSTANCE_(C,C)
#define NEX_MANAGEGLOBALINSTANCE(C) NEX_MANAGEGLOBALINSTANCE_(C,C)


// $ifdef SPARKFAMILY$
// group=Lobby
// $else$
// group=Miscellaneous Classes
// $endif$
// $ifdef WII_COMMON$
// group=Net-Z API
// $endif$

/*
Description: Sets the threading mode that will be used when $PRODUCTNAME$ is created.
Remarks: This class is mainly used to define whether $PRODUCTNAME$ uses internal or user threads for processing.
When run in internal-thread mode, $PRODUCTNAME$ automatically dispatches Jobs for processing. However, when
user-thread mode is used, (/i.e./ UseThreads is set to false) you need to dispatch Jobs yourself using the Scheduler
class.
Note: A Core object is created and destroyed by the system.

$ifdef WII_COMMON$
Note: Use the SetDefaultThreadPriority method when integrating $PRODUCTNAME$ into your games. In multi-threaded mode,
the threads that Quazal products spawn are set to priority 17. Hence, you should ensure that your game threads do not
starve the Quazal threads.
$endif$

$ifdef WII_SPARKXBL$

$else$
Note: When using NetZ and Rendez-Vous together, the threading model specified by the first created product will be used
by the other product regardless of the threading model that the second product specifies. For example, if
Rendez-Vous is created first and specifies that internal threads are used, then when a NetZ session is created, it will also
use internal threads even if NetZ sets UseThreads to false.
See Also: Scheduler
$endif$
*/

/*!
 * @brief   システムを動作させるスレッドモードを設定するためのクラスです。 また、NgsFacade の廃棄モードの設定も行います。
 * 
 *
 * 
 * このクラスは NgsFacade もしくは VSocket もしくは NetZ もしくは MatchmakeExtensionClient などの
 * 各種クライアントオブジェクトが生成されたタイミングでインスタンスが作成され、全て破棄されたタイミングで Core オブジェクトは破棄されます。
 * 
 *
 * 
 *
 * Core を生成する前に @ref Core::SetThreadMode() でスレッドモードを設定してください。
 *
 * @see NgsFacade, NetZ
 */
class Core: public RefCountedObject
{
public:
#if NEX_ENABLE_IMPROVE_DO_CORRUPT
    static qBool IsFatalState()
    {
        return s_bFatalState;
    }

    static void SetFatalState(qBool o);
#endif

    //  Note: Both AcquireInstance and ReleaseInstance are reserved for product facade objects, only.
    //  Calling either one of them directly may result into undefined behavior.

    //{secret}
    static void AcquireInstance();
    //{secret}
    static void ReleaseInstance();

    // コンストラクタでCore::AcquireInstance()し、デストラクタでCore::ReleaseInstance()するRAIIクラス。
    class ScopedAcquireInstance: public RootObject
    {
    public:
        ScopedAcquireInstance()
        {
            Core::AcquireInstance();
        }

        ~ScopedAcquireInstance()
        {
            Core::ReleaseInstance();
        }
    };

    /*
    Description: Returns a pointer to the Core instance.
    Returns: A pointer to the Core object.
    */

    /*!
     * @brief   Coreインスタンスへのポインタを返します。
     *
     * Coreオブジェクトを操作するためには、この関数が返す参照を使用して下さい。
     * @return  Coreインスタンスへのポインタ。
     */
    static Core* GetInstance()
    {
        InstanceControl* pInstance = InstanceControl::FindInstance(IC_TYPE_CORE, PseudoSingleton::GetCurrentContext());
        if (pInstance!=NULL)
        {
            return static_cast<Core*>(pInstance->GetDelegatorInstance());
        }
        else
        {
            return NULL;
        }
    }
    typedef void (*TerminateImmediatelyCallback)(void *context);

    /*!
     * @brief   強制終了モードを設定します。
     *
     * ライブラリ内部の状態やネットワークの状態によっては、 NEXの終了時に
     * @ref NgsFacade::Terminate() や @ref NgsFacade のデストラクタが長時間ブロックしてしまうことがあります。
     * 強制終了モードに設定することで、これらの処理時間を減らすことができます。
     * NetZ::SetTerminateImmediately()をtrueにした場合の処理も行い、全体的に終了処理が早くなります。
     * 通常の切断処理が行われないため、通信相手にはキープアライブのタイムアウトが発生してこちらのステーションがネットワークから切断されたように見える場合があります。
     * そのため、通常の終了シーケンスで設定することは控えてください。
     * また、サーバーと通信するAPIは全て失敗します。
     * AC によるネットワーク切断時に、自動的に強制終了モードを設定します。
     *
     * nn::nex::Core::GetInstance()->SetTerminateImmediately()とすることで設定可能です。
     *
     * この関数は、シャットダウン/スリープ時やネットワークに繋がっていない場合に、NEX全体を破棄するときに利用可能です。
     * NetZのみの終了の高速化は、@ref NetZ::SetTerminateImmediately() を実行してください。
     *
     * 初期値は通常終了モードです。
     * 本設定は、Coreインスタンス毎に保持されます。
     * ゲームサーバーへのログイン時に、通常終了モードへ設定されます。
     *
     *
     * @param[in]   bTerminate  trueを指定するとき、強制終了モードになります。
     *                              falseを指定するとき、通常モードに戻ります。
     *
     * @see NetZ::SetTerminateImmediately
     */
    void SetTerminateImmediately(qBool bTerminate);
    void SetTerminateImmediatelyCallback(TerminateImmediatelyCallback callback, void *context);
    void ResetTerminateImmediatelyCallback();

    /*!
     * @brief   強制終了モードが設定されているかどうかを取得します。
     *
     * nn::nex::Core::GetInstance()->GetTerminateImmediately()とすることで参照可能です。
     *
     *
     * @retval  true    強制終了モードの場合に返ります。
     * @retval  false   通常モードの場合に返ります。
     */
    qBool GetTerminateImmediately()
    {
        return m_bTerminateImmediately;
    }

    Time GetTerminateImmediatelyTime()
    {
        return m_tiTerminateImmediatelyAt;
    }

    /*
    Description: Returns whether or not $PRODUCTNAME$ is using internal threads for processing.
    Returns: true if $PRODUCTNAME$ is using internal threads for processing, false if it is only
    using user threads.
    See Also: UseThreads
    */

    /*!
     * @internal
     * @brief   システムが内部スレッドモードで動作しているかどうかを取得します。
     * @retval  true    内部スレッドモードの場合に返ります。
     * @retval  false   ユーザースレッドモードの場合に返ります。
     */
    static qBool IsUsingThreads()
    {
        return s_bUsesThreads;
    }

    /*!
     * @brief   インターナルスレッドが有効かどうかを取得します。
     *
     * インターナルスレッドとは NEX の内部処理を行うスレッドです。
     * Core::SetThreadMode() に Core::ThreadModeInternal, Core::ThreadModeInternalTransportBuffer
     * を設定した場合に本関数は true を返します。
     * NetZ もしくは NgsFacade オブジェクトを生成した後から本関数を使用できます。
     *
     * @return  true    インターナルスレッドが有効かどうか
     * @see     SetThreadMode
     */
    static qBool IsUsingInternalThread()
    {
        return IsUsingThreads();
    }

    //For NEX Developer anchor from <wiki/Core Threading Model#UseThreads>

    /*
    Description: Sets whether or not $PRODUCTNAME$ will use internal threads for processing.
    Parameters:
    bUsesThreads: If true (the default), $PRODUCTNAME$ will use internal threads for processing. If false,
    $PRODUCTNAME$ will use only user thread(s).
    Remarks: A call to this method should be made before $FACADENAME$ is created. If this method is
    called after $FACADENAME$ created, the call will be ignored.
    When run in user-thread mode all processing will occur sequentially in user thread(s),
    whereas in internal-thread mode, a certain amount of processing, such as that associated with
    callbacks and operations, will be performed in internal threads in parallel with the user thread.

    When run in internal-thread mode $PRODUCTNAME$ will automatically dispatch Jobs for processing however,
    when user-thread mode is used, /i.e./ bUsesThreads is false, you *must* dispatch Jobs yourself by calling
    either Scheduler::Dispatch or Scheduler::DispatchAll regularly.

    For example, to set $PRODUCTNAME$ to use only user threads we could call the following to set the core
    to user thread mode:

            Core::UseThreads(false)
            Core::SelectThreadSafeExecution(false)

    Then create the $PRODUCTNAME$ object.

    See Also: IsUsingThreads, Scheduler, Threading Model
    */

    /*!
     * @internal
     * @brief   システムの動作スレッドモードを設定します。
     *
     * この関数は、システムが構築される前に呼び出される必用があります。
     * システムが構築されたあとでこの関数を利用しても、設定は反映されません(途中で変更することも出来ません)。
     * インターナルスレッドモード以外を利用する場合、@ref Scheduler::Dispatch を定期的に実行して
     * ジョブを実行する必要があります。
     *
     * @param[in]   bUsesThreads    trueを指定するとき、内部スレッドモードで動作します(デフォルト)。
     *                              falseを指定するとき、ユーザースレッドモードで動作します。
     */
    static void UseThreads(qBool bUsesThreads=true)
    {
        s_bUsesThreads=bUsesThreads;
    }

    /*!
     * @brief   スレッドモードを表す列挙体です。@ref Core::SetThreadMode() の引数に用いられます。
     *
     *          Core::ThreadModeUndefined 以外を設定する必要があります。
     */
    enum ThreadMode
    {
        /*!
         * @brief   初期値です。
         *
         * Core::ThreadModeInternal と同じ動作となります。
         */
        ThreadModeUndefined,

        /*!
         * @brief   スレッドセーフトランスポートバッファスレッドモードに設定します。
         *          送受信処理をライブラリ内部のトランスポートバッファスレッドで行うことでディスパッチ処理のブロック時間（送受信時の長いブロック時間）が改善されます。
         *          ライブラリ内での排他が有効となり、一部の API を除きスレッドセーフとなります。詳しくはNEXプログラミングマニュアルの初期化設定(スレッドモデル、排他制御）を参照してください。
         *          @ref Scheduler::Dispatch() をアプリケーション側で定期的に実行する必要があります。
         *          このスレッドモードを指定した場合は Socket::Initialize の maxSessions を 4 にする必要があります。
         *          また、それに応じてSocketのバッファ領域も増やす必要があります。 
         */
         ThreadModeSafeTransportBuffer,

        /*!
         * @brief   スレッドアンセーフトランスポートバッファスレッドモードに設定します。
         *          送受信処理をライブラリ内部のトランスポートバッファスレッドで行うことでディスパッチ処理のブロック時間（送受信時の長いブロック時間）が改善されます。
         *          ライブラリ内での排他が一部を除いて無効となり、処理速度が向上します。
         *          排他が行われないため、すべての NEX の API を同一スレッドから使用するか、アプリケーション側で適切に排他処理を行ってください。
         *          @ref Scheduler::Dispatch() をアプリケーション側で定期的に実行する必要があります。
         *          処理速度、ブロック時間共に、優れているスレッドモードです。
         *          このスレッドモードを指定した場合は Socket::Initialize の maxSessionsを 4 にする必要があります。
         *          また、それに応じて Socket のバッファ領域も増やす必要があります。 
         */
        ThreadModeUnsafeTransportBuffer,

        /*!
         * @brief   スレッドアンセーフユーザースレッドモードに設定します。
         *          ディスパッチ処理、通信処理共にユーザーのスレッドで行われることになります。
         *          ライブラリ内での排他が無効となり、処理速度が向上します。
         *          排他が行われないため、すべての NEX の API を同一スレッドから使用するか、アプリケーション側で適切に排他処理を行ってください。
         *          @ref Scheduler::Dispatch() をアプリケーション側で定期的に実行する必要があります。
         *          処理速度の点で優れているスレッドモードです。
         *          ただし、ソケット処理で数 msec ブロックする可能性があるので、メインスレッドとは別スレッドから NEX の API を呼び出してください。
         */
        ThreadModeUnsafeUser,

#ifndef FOR_DOC_PURPOSE
        /*!
         * @internal
         * @brief   スレッドセーフユーザースレッドモードに設定します。
         *          ディスパッチ処理、通信処理共にユーザーのスレッドで行われることになります。
         *          ライブラリ内での排他が有効となり、一部の API を除きスレッドセーフとなります。
         *          @ref Scheduler::Dispatch() をアプリケーション側で定期的に実行する必要があります。
         *          ソケット処理で数 msec ブロックする可能性があるので、メインスレッドとは別スレッドから NEX の API を呼び出してください。
         */
        ThreadModeSafeUser,
#endif
        /*!
         * @brief   インターナルスレッドモードに設定します。
         *          ディスパッチ処理、通信処理共にライブラリ内部のスレッドで自動的に行われます。
         *          ライブラリ内での排他が有効となり、一部の API を除きスレッドセーフとなります。
         *          アプリケーションが NEX の API を呼び出すタイミングによってはその API 呼び出しが長時間ブロックする可能性があります。
         *          これを避けるには Core::ThreadModeInternalTransportBuffer を使用してください。
         */
        ThreadModeInternal,

        /*!
         * @brief   インターナルトランスポートバッファスレッドモードに設定します。
         *          ディスパッチ処理、通信処理共にライブラリ内部のスレッドで自動的に行われます。
         *          ライブラリ内での排他が有効となり、一部の API を除きスレッドセーフとなります。
         *          送受信処理をライブラリ内部のトランスポートバッファスレッドで行うことで内部のディスパッチ処理のブロック時間（送受信時の長いブロック時間）が改善されます。
         *          その結果、Scheduler::SystemLock() を内部で確保する時間が減少し、アプリケーションが NEX の各種 API を呼び出した際のブロック時間が減少します。
         *          このスレッドモードを指定した場合は Socket::Initialize の maxSessionsを 4 にする必要があります。
         *          また、それに応じて Socket のバッファ領域も増やす必要があります。 
         */
        ThreadModeInternalTransportBuffer
    };

    /*!
     * @brief   NEXのスレッドモードを設定します。
     *
     * NEX を初期化する前 (Initialize() などを呼び出す前) に、
     * 本関数でスレッドモードを設定してください。
     *
     * @param[in]   threadMode  @ref ThreadMode 型の列挙子です。
     */
    static void SetThreadMode(ThreadMode threadMode)
    {
        s_bThreadMode = threadMode;
    }

    /*!
     * @brief   現在設定しているスレッドモードを取得します。
     *
     * @return  現在設定しているスレッドモードが返ります。
     *          @ref ThreadMode 列挙体で定義される値のうちのいずれかになります。
     */
    static ThreadMode GetThreadMode()
    {
        return s_bThreadMode;
    }

    /*!
    * @internal
    * @brief   現在設定しているスレッドモードを文字列で取得します。
    *
    * @return  現在設定しているスレッドモードが文字列で返ります。
    */
    static const qChar* GetThreadModeString();

    /*!
     * @brief   トランスポートバッファスレッドが有効かどうかを取得します。
     *
     * トランスポートバッファスレッドとは送受信処理を行うスレッドです。
     * Core::SetThreadMode() に Core::ThreadModeUnsafeTransportBuffer, Core::ThreadModeSafeTransportBuffer, Core::ThreadModeInternalTransportBuffer
     * を設定した場合に本関数は true を返します。
     * NetZ もしくは NgsFacade オブジェクトを生成した後から本関数を使用できます。
     *
     * @return  true    トランスポートバッファスレッドが有効かどうか
     * @see     SetThreadMode
     */
    static qBool IsUsingTransportBufferThread()
    {
        return s_bUsesTransportBufferThread;
    }

    /*!
     * @internal
     * @brief   送受信処理のみを別のスレッドで動作させるかどうかを設定します。
     *
     * この関数は、システムが構築される前に呼び出される必要があります。
     * システムが構築されたあとでこの関数を利用しても、設定は反映されません(途中で変更することも出来ません)。
     *
     * 下位レイヤーの送受信APIでブロック時間が発生するため、
     * 送受信処理のみを別スレッドに分離することで全体的なパフォーマンスを
     * 高めることが出来ます。
     *
     * @param[in]   o   送受信処理のみを別スレッドで動作させるかどうかを指定します。
     */
    static void UseTransportBufferThread(qBool o)
    {
        s_bUsesTransportBufferThread = o;
    }

#ifdef CTR_TARGET
    /*!
        @brief スレッドのスタックメモリに AutoStack を使用するかどうかを取得します。デフォルト値は、falseです。

        @retval true   AutoStack を使用します。
        @retval false  AutoStack を使用しません。

        @see UseThreadAutoStack()
    */
#endif
    static qBool IsUsingThreadAutoStack()
    {
        return ObjectThreadRoot::IsUsingAutoStack();
    }

#ifdef CTR_TARGET
    /*!
        @brief スレッドのスタックメモリに AutoStack を使用するかどうかを設定します。

        @param[in]  usingAutoStack  AutoStack を使用するなら true。 デフォルトは、false。

        AutoStack を使用する場合、システムは nn::os::Thread::TryStartUsingAutoStack() でスレッドを実行します。@n
        使用しない場合、システムは nn::os::Thread::TryStart() でスレッドを実行します。

        @see UseThreadAutoStack()
    */
#endif
    static void UseThreadAutoStack(qBool usingAutoStack = true)
    {
        ObjectThreadRoot::UseAutoStack(usingAutoStack);
    }

    /*!
     * @brief   NEX がスレッドセーフに設定されているかどうかを取得します。
     *
     * Core::SetThreadMode() に Core::ThreadModeSafeTransportBuffer, Core::ThreadModeInternal, Core::ThreadModeInternalTransportBuffer
     * を設定した場合に本関数は true を返します。
     * NetZ もしくは NgsFacade オブジェクトを生成した後から本関数を使用できます。
     *
     * @retval  true    スレッドセーフのときに返ります。
     * @retval  false   スレッドアンセーフのときに返ります。
     */
    static qBool IsThreadSafe()
    {
        return s_bIsThreadSafe;
    }

    /*!
     * @internal
     * @brief   NEX の処理をスレッドセーフとするか、スレッドアンセーフとするか設定します。
     *
     * この関数は、システムが構築される前に呼び出される必用があります。
     * システムが構築されたあとでこの関数を利用しても、設定は反映されません(途中で変更することも出来ません)。
     *
     * スレッドセーフとした場合、各スレッドモードごとに必要な排他が行われますが、その分処理速度が遅くなります。
     *
     * 内部スレッドモードを使用している場合、必ずスレッドセーフに設定しなければなりません。
     * しかし、ユーザースレッドモードを使用している時には、処理速度を上げるためにスレッドセーフを無効にできます。
     * その際には、必ず以下のことを守って下さい。
     *
     * -(i) ユーザースレッドモードにするために @ref UseThreads() で false に設定すること
     * -(ii) アプリケーション側で「スレッドを全く使わないこと」または「2つのスレッドから同時に NetZ の API を呼ばないこと」を確実にすること。
     *
     * また、後者が起こらないことを確実にするため、APIの呼び出し群を保護するためにクリティカルセクションを使用すること
     *
     * 内部スレッドモードを使用している( @ref UseThreads() でtrueを設定している)ときにスレッドアンセーフに設定した場合、アサート停止します。
     *
     * @param[in]   bThreadSafe NetZの処理をスレッドセーフとするかどうかを指定します。
     * @see         IsThreadSafe, UseThreads, CriticalSection
     */
    static void SelectThreadSafeExecution(qBool bThreadSafe=true)
    {
        s_bIsThreadSafe=bThreadSafe;
    }
    //$ifdef NOTTHREADAFFINITY$
    //{secret}
    //$endif$
    //$ifdef NON_SUPPORT_E$
    /* Description: For NON_SUPPORT_E platforms only. Sets the thread affinity of the calling thread and of any other system thread that
              the $PRODUCTNAME$ client creates.
      Parameters:
      uiDefaultThreadAffinity: This parameter specifies the hardware thread number (0 - MAXIMUM_PROCESSORS-1)
      available to the client on an NON_SUPPORT_E console.

      Remarks: Set this at the bootstrap, before any system threads have been created. As a result,
      therefore, core (system) threads will call XSetThreadProcessor (see the NON_SUPPORT_E XDK
      Reference Manual) using the hardware thread specified with this method.

    */
    //$endif$
    //$ifdef WIN32$
    /* Description: For Windows and NON_SUPPORT_E platforms only. Sets the thread affinity of the calling thread,
            and of any other system thread that the $PRODUCTNAME$ client creates.
      Parameters:
      uiDefaultThreadAffinity: On a Windows platform, setting this parameter to 1, forces $PRODUCTNAME$ to run all
      system threads on one CPU. On the NON_SUPPORT_E Platform, this parameter sets the hardware thread number
      (0 - MAXIMUM_PROCESSORS-1).

      Remarks: For Windows and NON_SUPPORT_E platforms only. Set this mask at the bootstrap, before any system
      threads are created.

      If you are developing on a Windows platform, a thread affinity mask is a bit vector in which each bit
      represents the processors that a thread is allowed to run on. Refer to the Windows SDK function
      SetThreadAffinityMask for more information on this mask. Note that this must be a subset of the
      process affinity mask for the containing process of a thread. That is, a thread can only run on the
      processors its process can run on. See BasicSample for an example of how this is done.


    */
    //$endif$
    static void SetDefaultThreadAffinity(qUnsignedInt32 uiDefaultThreadAffinity);

    /* Description: Sets the priority of threads that are created after this call.
      Parameters:
      uiDefaultThreadPriority: The choice of thread priority values is platform dependant. Please refer
      to the platform documentation for possible values.
      Remarks: To take effect, this function must be called before instantiating the NetZ object.
      $ifdef WII_COMMON$
        Note: Use this method when integrating $PRODUCTNAME$ into your games. In multi-threaded mode,
        the threads that Quazal products spawn are set to priority 17. Hence, you need to ensure that
        your game threads do not starve the Quazal threads.
      $endif$
    */
    static void SetDefaultThreadPriority(qUnsignedInt32 uiDefaultThreadPriority);

    /*!
        @brief インターナルスレッドの優先度を設定します。

        スレッドモードに Core::ThreadModeInternal もしくは Core::ThreadModeInternalTransportBuffer
        を指定した場合に NEX 内部で生成されるインターナルスレッドの優先度を設定します。
        NetZ や NgsFacade オブジェクトを生成する前に設定する必要があります。

        @param[in] priority スレッド優先度。
        @see Core::ThreadModeInternal, Core::ThreadModeInternalTransportBuffer
    */
    static void SetInternalThreadPriority(qUnsignedInt priority);

    /*!
        @brief インターナルスレッドの優先度を取得します。

        @return スレッド優先度。
        @see Core::ThreadModeInternal, Core::ThreadModeInternalTransportBuffer
    */
    static qUnsignedInt GetInternalThreadPriority();

    /*!
     * @brief インターナルスレッドが利用する CPU の コア番号を設定します。
     *
     * ObjectThreadRoot::DEFAULT_CORE_NO が、デフォルト値です。
     * CTR では、デフォルト値を使用した場合は CORE_NO_USE_PROCESS_VALUE が設定されます。
     *
     * SDK 3.0 以降の CTR では、1 を指定することによりシステムコアを選択できます。
     * システムコアの利用の際には、nn::os::CTR::SetApplicationCpuTimeLimit() で、CPU時間を割り当てる必要があります。
     * システムコアの利用は、動作保証外です。
     * 
     *
     * 
     *
     * @param[in] no CPU コア番号
     */
    static void SetInternalThreadCoreNo(qInt32 no);

    /*!
     * @brief インターナルスレッドが利用する CPU の コア番号を取得します。
     *
     * @return CPU コア番号
     */
    static qInt32 GetInternalThreadCoreNo();

    /* Description: Sets the thread stack size.
      Parameters:
      uiThreadSize: The size of the thread stack. The default size is usually between 16 to 32k
      (depending upon the platform).
      Remarks:  Use with care. This is the system thread stack. This method allows you to configure
      the thread stack size at run-time without having to recompile the libraries.
    */

    /*!
        @brief スレッドスタックのサイズを設定します。

        NEX で内部的に作成されるスレッドのスタックサイズを設定します。
        デフォルト値で動作するように設定されているため、変更する必要はありません。

        @param[in] uiThreadSize スレッドスタックのサイズ。
    */
    static void SetThreadStackSize(size_t uiThreadSize);

    /*  Description: Sets the thread stack size.
        Returns: The current size of the thread stack.
        Remarks:  This method allows you to retrieve the thread stack size at
        run time.
    */
#ifdef CTR_TARGET
    /*!
        @brief スレッドスタックのサイズを取得します。

        CTRでは、スタックバッファを4096バイトアライメントに揃える必要があるため、
        実際には本関数で取得した値＋4095バイトのメモリを確保しています。

        @return スレッドスタックのサイズ。
    */
    static size_t GetThreadStackSize();
#else
    /*!
        @brief スレッドスタックのサイズを取得します。

        @return スレッドスタックのサイズ。
    */
    static size_t GetThreadStackSize();
#endif

    //{secret}
    // Encoding for strings sent over wire
    // This is here only for backwards compatibility and
    // should not be exposed.
    static qUnsignedInt32 GetConnectionEncoding()
    {
        return s_oConnectionEncoding;
    }
    //{secret}
    static void SetConnectionEncoding(qUnsignedInt32 uiCode);

    // Coreインスタンスの個数を返す。(PseudoSingleton利用時に複数になる)
    static qUnsignedInt32 GetCoreCount() { return s_uiCoreCount; }

    // GetScheduler
    NEX_MANAGEINSTANCE(Scheduler);
    // GetCallContextRegister
    NEX_MANAGEINSTANCE(CallContextRegister);
    // GetSystemComponents
    NEX_MANAGEINSTANCE(SystemComponents);
    // GetSecurityContextManager
    NEX_MANAGEINSTANCE(SecurityContextManager);

private:
    static void RefreshThreadMode();
    void CreateGlobals();
    void DeleteGlobals();

#if NEX_ENABLE_IMPROVE_DO_CORRUPT
    static qBool s_bFatalState;
#endif

    /*
    Remarks: A Core object should never be directly instantiated. Instead, it is created
    by the system before $PRODUCTNAME$ is created.
    */
    //{secret}
    Core();

    /*
    Remarks: A Core object should never be directly destroyed. It is destroyed by
    $PRODUCTNAME$ when the session terminates.
    */
    //{secret}
    virtual ~Core();

    // Cannot copy
    //{secret}
    Core(const Core &);
    //{secret}
    Core &operator=(const Core &);

    // Settings...
    //{secret}
    static qBool s_bUsesThreads;

    static ThreadMode s_bThreadMode;
    static qBool s_bUsesTransportBufferThread;

    static qBool s_bIsThreadSafe;
    static qUnsignedInt32 s_uiCoreCount;
    static SystemSetting    s_oConnectionEncoding;
    PseudoSingleton m_psInstance;

    qBool m_bTerminateImmediately;
    Time m_tiTerminateImmediatelyAt;
    TerminateImmediatelyCallback m_pfTerminateImmediatelyCallback;
    void* m_pTerminateImmediatelyContext;
    qBool m_bNetZTerminateImmediately;
    CriticalSection m_csTerminateImmediately;
};

/*!
    @brief   NEX を初期化します。
 
    NEX の Core オブジェクトを生成し、Scheduler などのコア機能を開始します。
    この呼び出し以降 Scheduler::Dispatch() の呼び出しが可能になります。
    すでに初期化済みの場合は何も起こりません。
    Initialize() が呼び出された回数分 Finalize() を呼び出す必要があります。本関数を呼び出さない場合、NEX の初期化処理は NgsFacade, NetZ, VSocket, ***Client などのオブジェクトなどを生成したときに自動的に行われます。本関数の呼び出しは必須ではありません。
  
    ensureFirstInitialization を true にする事により、NEX の初期化が最初に行われたことを保証することが出来ます。
    すでに NEX が初期化されている状態で ensureFirstInitialization を true にして本関数を呼び出した場合は処理を停止します。
    この機能を利用するとアプリケーションの意図したタイミングで NEX が初期化されていることを確認できます。
 
    @param ensureFirstInitialization NEX が最初に初期化されたことを保証するかどうか。
 */
void Initialize(qBool ensureFirstInitialization);

/*!
    @brief   NEX が初期化済みかどうかを確認します。すでに Initialize() を呼び出しているか、もしくは NgsFacade, NetZ, VSocket, ***Client などのオブジェクトを生成している場合に true が返ります。@return すでに初期化済みかどうか。

*/
qBool IsInitialized();

/*!
    @brief   NEX の終了処理を行います。

    実際の終了処理は Initialize() を呼び出した回数分 Finalize() を呼び出しかつ
    NgsFacade, ***Client などのオブジェクトを全て破棄した時点で行われます。
    ensureLastFinalization に true を指定すると実際の終了処理が行われる事を保証できます。
    ensureLastFinalization に true を指定しているにもかかわらず Initialize() を呼び出した回数分に達していない、もしくは NgsFacade, NetZ, VSocket, ***Client などのオブジェクトを破棄していない場合は処理を停止します。この機能を利用するとアプリケーションの意図したタイミングで NEX の終了処理が行われていることを確認できます。
    
    @param ensureLastFinalization 実際の終了処理が行われる事を保証するかどうか。
*/
void Finalize(qBool ensureLastFinalization);

/*!
    @brief   スコープで NEX の初期化と終了処理を行うクラスです。

*/
class ScopedInitializer : public RootObject
{
public:
    /*!
        @brief   コンストラクタです。 NEX を初期化します。

        Initialize() と同等の処理を行います。
        詳細は Initialize() を参照してください。

        @param ensureFirstInitialization NEX が最初に初期化されたことを保証するかどうか。
        @param ensureLastFinalization 実際の終了処理が行われる事を保証するかどうか。
     */
    ScopedInitializer(qBool ensureFirstInitialization, qBool ensureLastFinalization)
        : m_ensureFirstInitialization(ensureFirstInitialization)
        , m_ensureLastFinalization(ensureLastFinalization)
    {
        NAMESPACE_NEX Initialize(m_ensureFirstInitialization);
    }

    /*!
        @brief   デストラクタです。 NEX の終了処理を行います。

        Finalize() と同等の処理を行います。
        詳細は Finalize() を参照してください。
     */
    ~ScopedInitializer()
    {
        NAMESPACE_NEX Finalize(m_ensureLastFinalization);
    }
private:
    qBool m_ensureFirstInitialization;
    qBool m_ensureLastFinalization;
};

#undef _NEX_MANAGEINSTANCE
#undef _NEX_MANAGEGLOBALINSTANCE
#undef NEX_MANAGEINSTANCE
#undef NEX_MANAGEGLOBALINSTANCE

END_NAMESPACE_NEX

