﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/Encryption/Interface/EncryptionDecl.h>
#include <OnlineCore/src/Core/PluginObject.h>
#include <OnlineCore/src/Core/Key.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>

BEGIN_NAMESPACE_NEX

class Buffer;
class Plugin;

//Group=Encryption

/*
Description: The root class of any encryption algorithm that $PRODUCTNAME$ uses.
Remarks: This is the root class of any encryption algorithm used by $PRODUCTNAME$. The
encryption algorithm used by $PRODUCTNAME$, if any, is set via StreamSettings::SetEncryptionAlgorithm.
If you wish to implement your own encryption algorithm, it must inherit from this class.
For further details on how this is done, contact Quazal Technical Support.
See Also: Key, StreamSettings::SetEncryptionAlgorithm
*/
class EncryptionAlgorithm: public PluginObject
{

public:
    //Group=Methods

    /*
    Parameters:
    uiMinimumKeyLength: The minimum Key length of this EncryptionAlgorithm. The default value is 8 units.
    uiMaximumKeyLength: The maximum Key length of this EncryptionAlgorithm. The default value is 64 units.
    */
    EncryptionAlgorithm(qUnsignedInt uiMinimumKeyLength=8, qUnsignedInt uiMaximumKeyLength=64);

    virtual ~EncryptionAlgorithm();

    //{secret}
    virtual qUnsignedInt32 Encrypt(const Buffer &oInBuffer, Buffer *pOutBuffer) = 0;
    //{secret}
    virtual qUnsignedInt32 Encrypt(Buffer *pInOutBuffer);

    //{secret}
    virtual qUnsignedInt32 Decrypt(const Buffer &oInBuffer, Buffer *pOutBuffer) = 0;
    //{secret}
    virtual qUnsignedInt32 Decrypt(Buffer *pInOutBuffer);

    //{secret}
    virtual qBool GetErrorString(qUnsignedInt32 uiError, qChar *szString, size_t uiStringSize);

    /*
    Summary: Sets the key used by the encryption algorithm.
    Parameters:
    pbyContent: A pointer to a buffer of bytes that contains the key.
    uiLength: The length of the key as a qUnsignedInt.
    oKey: The key as a Key object.
    Returns: true if the Key was successfully set, false if not or an error occurred.
    See Also: Key
    */
    qBool SetKey(const Key& oKey);
    qBool SetKey(qByte *pbyContent, size_t uiLength)
    {
        return SetKey(Key(pbyContent, uiLength));
    }

    const Key& GetKey()
    {
        return m_keyEncryption;
    }

    //Summary: Returns the minimum key length used by the encryption algorithm.
    //Returns: The minimum key length as an unsigned integer.
    //See Also: GetMaximumKeyLength
    size_t GetMinimumKeyLength() const
    {
        return m_uiMinimumKeyLength;
    }
    //Summary: Returns the maximum key length used by the encryption algorithm.
    //Returns: The maximum key length as an unsigned integer.
    //See Also: GetMinimumKeyLength
    size_t GetMaximumKeyLength() const
    {
        return m_uiMaximumKeyLength;
    }

protected:
    virtual void KeyHasChanged() {};

protected:
    Key m_keyEncryption;
    size_t m_uiMinimumKeyLength;
    size_t m_uiMaximumKeyLength;
};

END_NAMESPACE_NEX





