﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/UserContext.h>
#include <OnlineCore/src/Platform/Core/SystemError.h>
#include <OnlineCore/src/Core/StateMachine.h>

BEGIN_NAMESPACE_NEX

//Group = System Operations

/*
Description: This is the root class of all $PRODUCTNAME$ operations.
Remarks:
$ifdef ETERNA$
The operations in $PRODUCTNAME$ fall into one of the categories DOOperation,
DupSpaceOperation, and SessionOperation, which respectively incorporate the operations
relating to duplicated objects, duplication spaces, and the game session.

The Operation class contains methods that enable the extraction of information about the type
of operation that it is and if there is a user-defined context associated to the operation.
$else$
The operations in $PRODUCTNAME$ fall into either of the categories DOOperation
or SessionOperation, which respectively incorporate the operations
relating to duplicated objects and the game session.

The Operation class contains methods that enable the extraction of information about the type
of operation that it is and if there is a user-defined context associated to the operation.
$endif$
*/

/*!
 * @brief   操作オブジェクトの基底クラスです。
 *
 * 操作とは、システムの中で発生するイベントのことを指します。
*
 * Net-Zにおける操作は、ゲーム中に出てくるオブジェクトを操作するための
 * 複製オブジェクト操作(@ref DOOperation)と、
 * ゲームセッションを操作するためのセッション操作(@ref SessionOperation)の
 * カテゴリに分けられます。このクラスはインスタンス化できません。
 *
 * このOperationクラスは、その操作がどの種類であるかの情報(@ref GetType 関数)と、
 * 操作に関連付けられているユーザー定義のコンテキスト情報(@ref GetUserData 関数)を提供しています。
 */
#ifdef FOR_DOC_PURPOSE
class Operation: public RootObject
#else
class Operation: public StateMachine::QEvent
#endif
{
    //Group=Type Definitions
public:

    // This macro is never defined.  It is used only to get around a compilation problem with
    // DocJet, as Connect and Disconnect cannot be made secret.
#ifdef FOR_DOC_PURPOSE

    //$ifdef NETZ$
    //Description: This enumerates all types of operations (duplicated objects and game session)
    // that can be called by $PRODUCTNAME$.
    //Remarks: The defined operations fall into either the DOOperation or SessionOperation categories:
    //operations for duplicated objects and operations for the game session (JoinSessionOperation
    //and LeaveSessionOperation) respectively.
    //See Also: Type
    /*!
     * @brief   システムから呼び出される操作の種類を表す列挙体です。
     * @see     GetType
     */
    enum Type
    {
        /* See ChangeMasterStationOperation*/

        /*!
         * @brief @ref ChangeMasterStationOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        ChangeMasterStation,
        /* See ChangeDupSetOperation*/

        /*!
         * @brief @ref ChangeDupSetOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        ChangeDupSet,
        /* See FaultRecoveryOperation*/

        /*!
         * @brief @ref FaultRecoveryOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        FaultRecovery,
        /* See AddToStoreOperation*/

        /*!
         * @brief @ref AddToStoreOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        AddToStore,
        /* See RemoveFromStoreOperation*/

        /*!
         * @brief @ref RemoveFromStoreOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        RemoveFromStore,
        /* See CallMethodOperation */

        /*!
         * @brief @ref CallMethodOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        CallMethod,
        /* See UpdateDataSetOperation */

        /*!
         * @brief @ref UpdateDataSetOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        UpdateDataSet,
        /* See JoinSessionOperation */

        /*!
         * @brief @ref JoinSessionOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        JoinSession,
#ifndef FOR_DOC_PURPOSE
        /* See LeaveSessionOperation */

        /*!
         * @internal
         * @brief LeaveSessionOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        LeaveSession,
#endif
        /*!
         * @brief @ref JoinStationOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        JoinStation,
        /*!
         * @brief @ref LeaveStationOperation クラスのインスタンスであるときに、この列挙値になります。
         */
        LeaveStation
    };

#else

    enum Type
    {
        MasterIDChange = StateMachine::FirstUserSignal,
        FaultRecovery,
        AddToStore,
        CallMethod,
        UpdateDataSet,
        RemoveFromStore,
        Match,
        Connect,
        Disconnect,
        ChangeMasterStation,
        ChangeDupSet,
        JoinSession,
        LeaveSession,
        JoinStation,
        LeaveStation,
        CallProtocolMethod,
        CreateMaster,
        RendezVousLogin,
        RendezVousLogout,
        Test
    };


    enum Event
    {
        Begin, End, Approval, Denial, CustomTrace
    };

#endif

    //Group=Methods

    //{secret}
    Operation(qUnsignedInt32 uiOrigin=0);

    //{secret}
    virtual ~Operation()
    {
    }

    // {secret}
    void SetOrigin(qUnsignedInt32 uiOrigin)
    {
        m_uiOrigin = uiOrigin;
    }
    // {secret}
    qUnsignedInt32 GetOrigin() const
    {
        return m_uiOrigin;
    }

    //n1589:実装はcppへ
#ifdef NEX_TRACE_ENABLED
    void Trace(Event oEvent) const;
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS) const;
#else
    void Trace(Event oEvent) const
    {
        DUMMY_PARAM(oEvent);
    }
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS) const
    {
        DUMMY_PARAM(uiTraceFlags);
    }
#endif

private:
    virtual StateMachine::Signal GetSignal() const
    {
        return static_cast<StateMachine::Signal>(GetType());
    }

public:

    /*
    Description: Returns the Type of the operation.
    Returns: The Type of operation that has been called.
    Remarks: Refer to Type for the different types of operations that $PRODUCTNAME$ may call and to
    SetTraceFilter for an example of this method in use.
    See Also: Type
    */
    //n1589:memo:GetType is defined by NEX_IMPL_OPERATION_COMMON_METHODS
    /*!
     * @brief   この操作オブジェクトの種類を返します。
     * @return  この操作オブジェクトインスタンスの種類が返ります。
     */
    virtual Type GetType() const =0;

    /*
    Description: Returns the user-defined value that was set via the method SetUserData.
    Returns: The user-defined value set via the method SetUserData. The value may be either
    a qUnsignedInt32, qReal, or pointer to an object.

    $ifdef ETERNA$
    Remarks: The user-defined data should be set when one of the methods DataSet::OperationBegin,
    DuplicatedObject::OperationBegin, or DuplicationSpace::OperationBegin is called and retrieved
    when one of the methods DataSet::OperationEnd, DuplicatedObject::OperationEnd, or
    DuplicationSpace::OperationEnd is called.
    $endif$

    $ifdef NETZ$
    Remarks: The user-defined data should be set when either of the methods DataSet::OperationBegin
    or DuplicatedObject::OperationBegin is called and retrieved when either of the methods
    DataSet::OperationEnd or DuplicatedObject::OperationEnd is called.
    $endif$

    Note: The user must ensure that the methods SetUserData and GetUserData are called in
    relation to the same operation.

    See Also: SetUserData, UserContext
    */

    /*!
     * @brief   ユーザー定義値を取得します。
     *
     * ユーザー定義データは、@ref DataSet::OperationBegin または
     * @ref DuplicatedObject::OperationBegin 関数にて設定(@ref SetUserData)し、
     * @ref DataSet::OperationEnd または@ref DuplicatedObject::OperationEnd 関数にて
     * この関数を通じて呼び出して使用するために用意されています。
     *
     * @return  @ref SetUserData でこの操作に設定されたユーザー定義値が返ります。
     * @see     SetUserData
     */
    UserContext GetUserData();

    /*
    Description: Sets a user-defined value that can be obtained via the method GetUserData.
    Parameters:
    uUserData: A user-defined value which may be either a qUnsignedInt32, qReal, or pointer to an
    object.

    $ifdef ETERNA$
    Remarks: The user-defined value may be either a qUnsignedInt32, qReal, or pointer to an
    object. It should be set when one of the methods DataSet::OperationBegin,
    DuplicatedObject::OperationBegin, or DuplicationSpace::OperationBegin is called and retrieved
    when one of the methods DataSet::OperationEnd, DuplicatedObject::OperationEnd, or
    DuplicationSpace::OperationEnd is called.
    $endif$

    $ifdef NETZ$
    Remarks: The user-defined value may be either a qUnsignedInt32, qReal, or pointer to an
    object. It should be set when either of the methods DataSet::OperationBegin or
    DuplicatedObject::OperationBegin is called and retrieved
    when either of the methods DataSet::OperationEnd or DuplicatedObject::OperationEnd is called.
    $endif$

    Note: The user must ensure that the methods SetUserData and GetUserData are called in
    relation to the same operation.

    See Also: GetUserData, UserContext
    */

    /*!
     * @brief   ユーザー定義値を設定します。
     *
     * ユーザー定義値は、@ref qUnsignedInt32 か@ref qReal か、
     * 何かしらのオブジェクトへのポインタです。この関数は、
     * @ref DataSet::OperationBegin または@ref DuplicatedObject::OperationBegin 関数にて
     * 呼び出し、ユーザー定義値をセットしたあと、
     * @ref DataSet::OperationEnd または@ref DuplicatedObject::OperationEnd 関数にて
     * @ref GetUserData 関数を呼び出してユーザー定義値を取り出して利用することが出来ます。
     */
    void SetUserData(UserContext uUserData);

    /*
    Description: Returns the class name of this Operation as a string.
    Returns: A string that represents the class name of the Operation.
    Remarks: Use this method to get the class name of the operation as a string. Refer to Type for the
    different types of operations that $PRODUCTNAME$ may call.
    See Also: Type
    */

    /*!
     * @brief   この操作オブジェクトのクラス名称を取得します。
     * @return  クラス名称へのポインタが返ります。
     */
    virtual const qChar* GetClassNameString() const = 0;

    // {secret}
    virtual void ForceImplOperationCommonMethodsMacro() = 0;


#ifdef FOR_DOC_PURPOSE
    /*
    Description: Use this method for casting (explicit type conversion) during run time.
    Parameters:
    pOperation: A pointer to an Operation class.
    Returns: A valid pointer to the Operation if the Operation is of the expected type, a null
    pointer if it is not.
    Remarks: The DynamicCast method functions in the same manner as the standard C++ dynamic_cast
    operator but using $PRODUCTNAME$ run-time information.

    For example, we can return a pointer to the AddToStoreOperation object, if /pOperation/ points to a
    AddToStoreOperation object, as follows:

        AddToStoreOperation *pAddToStoreOperation = AddToStoreOperation::DynamicCast(pOperation)

    Note: This method is defined by each specific Operation class.
    */

    /*!
     * @brief   ランタイム情報を用いた動的キャストを行います。
     *
     * この関数はNetZの動的情報を用いて、C++のdynamic_castに相当する機能を提供します。
     * Operationクラスとその派生クラスのみに利用することが可能です。
     * この機能を利用するためにRTTIを有効にする必要はありません。
     *
     * この関数を呼び出すには、キャストしたい対象のクラスにある静的関数を呼び出します。
     * たとえば、Operation型のポインタpOperationを、@ref AddToStoreOperation 型の
     * ポインタに動的変換するためには、AddToStoreOperation::DynamicCast 関数を利用します。
     *
     * @param[in]   pOperation 変換元の操作オブジェクトへのポインタを指定します。
     * @return  動的キャストされたポインタが返ります。
     * @retval  NULL    指定された@a pOperationがNULLである場合、
     *                  あるいは、@a pOperationがキャスト対象のクラスインスタンスではなかったときに返ります。
     */
    static SpecificOperation *DynamicCast( Operation *pOperation );
    static const SpecificOperation *DynamicCast( const Operation *pOperation );

#endif

    /*
    Description: Sets which operations will be traced via the TraceLog.
    Parameters:
    pfTraceFilter: A user defined function that takes a pointer to an Operation and returns a Boolean. If this
    is set to NULL, all operations for all UserDO s will be traced.
    Remarks: This method is called by $PRODUCTNAME$ to determine which operations should be traced via the TraceLog.
    To trace operations, you need to call the SetTraceFilter method that will enable the
    registration of a callback, set the TraceLog flag to trace operations, and then register a callback
    via the pfTraceFilter function that defines how the traces are filtered. If this function is not
    defined then by default all operations for all objects except CoreDO s are traced, /i.e./ all UserDO s.

    Below is an example of how to call this method and how to register a callback to trace selected operations
    on a user object class.

        // Register a callback that defines how the traces will be filtered
        qBool FilterAvatarTraces(const Operation* pOperation) {
            DuplicatedObject* pAssociatedDO=NULL;
            // Select the operations to be traced
            switch (pOperation->GetType()) {
                case Operation::UpdateDataSet:
                case Operation::ChangeMasterStation:
                case Operation::FaultRecovery:
                case Operation::AddToStore:
                case Operation::RemoveFromStore:
                case Operation::CallMethod:
                case Operation::ChangeDupSet:
                    pAssociatedDO=((DOOperation*)pOperation)->GetAssociatedDO();
                    break;
                default:
                    break;
            }
            if (pAssociatedDO==NULL) {
                // Keep traces that are not related to a specific DO.
                return true;
            }

            // Trace the operation if the object is of the Avatar class.
            if (pAssociatedDO->IsA(DOCLASSID(Avatar))) {
                return true;
            } else {
                return false;
            }
        }

        // To trace operations, call the SetTraceFilter method and
        // then set the TraceLog flag to trace operations
        Operation::SetTraceFilter(FilterAvatarTraces);
        TraceLog::GetInstance()->SetFlag(TRACE_OPERATION);

    See Also: TraceLog
    */
    //n1589:memo:called from DOCore::DOCore
    /*!
     * @brief   操作オブジェクトのトレース実行をどのタイミングで実行するかを指定するためのコールバック関数を設定します。
     *
     * これで指定されたコールバック関数は、システム側から呼びだされ、
     * @ref TraceLog でトレースすべき操作かどうかを指定します。
     * 指定されたコールバック関数は、トレース対象となっている操作オブジェクトのポインタとともに
     * 呼び出され、qBool値を返します。これがtrueであればトレースを行い、
     * falseであればトレースを行いません。
     *
     * TraceLogにトレースフラグをセットしたあとで、
     * この関数を利用して操作ごとのフィルタコールバックを設定します。
     * もしなにも設定されていなければ、
     * CoreDO(システム側で自動的に用意される複製オブジェクト。たとえば@ref Station, @ref Session など。)を
     * 除くすべての複製オブジェクト(つまり、UserDO)に対してトレースが行われるように設定されます。
     *
     * @param[in]   pfTraceFilter トレースフィルタ用のコールバック関数を指定します。
     * @see     DuplicatedObject::IsACoreDO, DuplicatedObject::IsAUserDO
     */
    static void SetTraceFilter(qBool (*pfTraceFilter)(const Operation*))
    {
        s_pfTraceFilter=pfTraceFilter;
    }

    // {secret}
    void AbortOperation();

    // {secret}
    qBool OperationIsAborted() const
    {
        return m_bOperationAborted;
    }

protected:
    // {secret}
    const qChar* GetEventString(Event eEvent) const;

private:
#ifdef NEX_TRACE_ENABLED    // n2370
    virtual void TraceImpl(Event oEvent, qUnsignedInt64 uiTraceFlags) const =0;
#endif  // NEX_TRACE_ENABLED

private:
    // {secret}
    qBool m_bOperationAborted;

    // {secret}
    UserContext m_uUserData;

    static qBool (*s_pfTraceFilter)(const Operation*);

    //{secret}
    qUnsignedInt32 m_uiOrigin; // TODO - Change this to CID/PID
};

#define NEX_IMPL_OPERATION_COMMON_METHODS( TYPE )                               \
    virtual void ForceImplOperationCommonMethodsMacro(){};                                          \
    static TYPE##Operation *DynamicCast( Operation *pOperation ){                       \
        if( pOperation == NULL || pOperation->GetType() != TYPE ){ return NULL; }   \
        return static_cast< TYPE##Operation * >( pOperation );                          \
    }                                                                                   \
    static const TYPE##Operation *DynamicCast( const Operation *pOperation ){                       \
        if( pOperation == NULL || pOperation->GetType() != TYPE ){ return NULL; }   \
        return static_cast< const TYPE##Operation * >( pOperation );                            \
    }                                                                                   \
    virtual Type GetType() const {return TYPE;}                                         \
    virtual const qChar* GetClassNameString() const {return NEX_T(#TYPE);}

END_NAMESPACE_NEX

