﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RootObject.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>

BEGIN_NAMESPACE_NEX

class Operation;

//group=System Operations

/*
Description: Defines callbacks that are called on specified System Operations.
Remarks: To implement your own callbacks on a given System Operation, you need to sub class
this class, implement the CallMethod method, and then register the callback to an
OperationManager via OperationManager::RegisterCallback. OperationCallback s are useful for retrieving
information such as the reason why a Station leaves a Session or why a Duplication master migrates.
See Also: OperationManager
*/

/*!
 * @brief   システム操作に対するコールバック関数を定義します。
 *
 * 与えられたシステム操作(@ref Operation)に対するコールバック関数を実装するために、
 * このクラスから派生したクラスを作成して、@ref CallMethod() を実装したものを
 * @ref OperationManager に@ref OperationManager::RegisterCallback() を使って登録する
 * 必要があります。OperationCallbackは、ステーションがセッションを離れた理由や、
 * 複製オブジェクトがマイグレーション(複製オブジェクトのコントロール権のステーション間転移)が
 * 発生したのかなどの情報を取得するために便利です。
 *
 * @see     OperationManager
 */
class OperationCallback: public RootObject
{
public:
    // group=Methods

    /*
    Parameters:
    iPriority: The Priority allocated to this OperationCallback. User-defined priorities are
    non-zero positive or negative integers in the range -1024 to 1024.
    Remarks: OperationCallback s are called in numerical order according to their allocated Priority.
    The Priority is a positive or negative integer that determines the sequence in
    which operation-related events occur within the system. The priority zero is reserved for
    when an Operation takes place. User-defined priorities may be in the ranges -1024 to -1 and
    1 to 1024, and system priorities are outside of these ranges. Refer to OperationManager for
    further details.
    See Also: OperationManager
    */

    /*!
     * @brief   オブジェクトを初期化します。
     *
     * OperationCallbackは数値による優先順にしたがって呼び出されます。
     * 優先度は正の値または負の値の整数で、操作に関係のあるイベントをシステムが実行する順序を決定します。
     * 優先度0は、操作の実行のために予約されています。そのため、アプリケーション側で利用できる
     * 優先度は-1024から-1までと、1から1024までとなります。システム側で利用する優先度はこれ以外を利用します。
     *
     * @ref     OperationManager
     */
    OperationCallback(qInt32 iPriority): m_iPriority(iPriority) {}
    virtual ~OperationCallback() {}

    /*
    Description: Defines the implementation of this OperationCallback.
    Parameters:
    pOperation: A pointer to the Operation that this OperationCallback will be called on.
    Remarks: This method defines what action will be taken when this OperationCallback is called.
    */

    /*!
     * @brief   このOperationCallbackの実装です。
     *
     * この関数は、このOperationCallbackが呼び出されたときにどのような処理を実行するのかを定義します。
     *
     * @param   pOperation [in] このOperationCallbackが呼び出されたシステム操作へのポインタ
     */
    virtual void CallMethod(Operation* pOperation)=0;

    /*
    Description: Returns the Priority of this OperationCallback.
    Returns: The Priority of this OperationCallback.
    Remarks: The Priority is a positive or negative integer that determines the sequence in
    which operation-related events occur within the system. The priority zero is reserved for
    when an Operation takes place. User-defined priorities may be in the ranges -1024 to -1 and
    1 to 1024, and system priorities are outside of these ranges. Refer to OperationManager for
    further details.
    See Also: OperationManager
    */

    /*!
     * @brief   OperationCallbackの優先度を取得します。
     *
     * OperationCallbackは数値による優先順にしたがって呼び出されます。
     * 優先度は正または負の整数で、操作に関係のあるイベントをシステムが実行する順序を決定します。
     * 優先度0は、操作の実行のために予約されています。そのため、アプリケーション側で利用できる
     * 優先度は-1024から-1までと、1から1024までとなります。システム側で利用する優先度はこれ以外を利用します。
     * 詳細につきましては、@ref OperationManager を参照して下さい。
     * @return  このOperationCallbackの優先度が返ります。
     */
    qInt32 GetPriority() const
    {
        return m_iPriority;
    }

private:
    qInt32 m_iPriority;
};


//group=System Operations

/*
Description: Manages the System Operations that occur in $PRODUCTNAME$.
Remarks: Several instances of this class may be created that manage the various different
$PRODUCTNAME$ System Operations. For an OperationCallback to be called by the OperationManager,
it must first be registered to the manager via RegisterCallback.

When an OperationCallback is created it must be assigned a priority within the range -1024 to 1024,
excluding those priorities used by the system. OperationCallback s are then called in numerical order
according to their allocated priority. If two callbacks are allocated the same priority then both are
executed, but $PRODUCTNAME$ does not provide any guarantees as to which one is executed first.
The priority is a positive or negative integer that determines
the sequence in which operation-related events occur within the system. All registered callbacks are
called from lowest priority to highest priority, with the actual operation taking place at a priority
of zero. Priorities enable you to control the order in which your own OperationCallback s are called,
such as ensuring a callback is called immediately before or after OperationBegin is called, for example.
The priorities used by $PRODUCTNAME$ and the order in which they
are called are listed in the _Call type definition and also illustrated below.

$ifdef NETZ$
{image: NetZOpCallbackPriority}
$endif$
$ifdef ETERNA$
{image: EternaOpCallbackPriority}
$endif$

*A* *timeline* *that* *indicates* *the* *order* *in* *which* *the* OperationCallback *s* *defined* *by* *$PRODUCTNAME$*
*are* *called.*

See Also: OperationCallback
*/

/*!
 * @brief   システム側で発生するシステム操作(@ref Operation)を管理します。
 *
 * システム側で発生するいくつかの異なるシステム操作を管理するために
 * このクラスのインスタンスが作成される可能性があります。
 * OperationManagerから呼び出される@ref OperationCallback については、
 * @ref RegisterCallback() によって最初にマネージャーに登録されなければなりません。
 *
 * @ref OperationCallback が作成されたときに設定できる優先度には、-1024から1024の値のうち
 * システムが利用しない優先度を設定する必要があります。
 *
 */
class OperationManager: public RootObject
{
public:
    //group=Methods

    OperationManager();
    virtual ~OperationManager();

    //{secret}
    Operation* GetCurrentOperation() const;
    //{secret}
    void TraceOperationStack(qUnsignedInt64 uiTraceFlag);
    //{secret}
    void OperationBegins(Operation* pOperation);
    //{secret}
    void OperationEnds(Operation* pOperation);

public:
    //group=Type Definitions

    // Enumerates the OperationCallback s that may be called by this OperationManager.
    /*!
     * @brief   OperationManagerによって呼び出されるOperationCallbackの列挙体です。
     */
    enum _Call
    {
        // The first user-defined OperationCallback that will be called before $PRODUCTNAME$ calls OperationBegin.
        /*!
         * @brief   システムが@ref DuplicatedObject::OperationBegin() または@ref DataSet::OperationBegin() を呼び出す前に
         *          実行されるユーザー定義@ref OperationCallback のうち、最初の優先度です。
         */
        FirstOpBeginCall = -1024,
        // $PRODUCTNAME$ calls DuplicatedObject::OperationBegin or DataSet::OperationBegin.
        /*!
         * @brief   システムが@ref DuplicatedObject::OperationBegin() または@ref DataSet::OperationBegin() を
         *          呼び出す優先度です。
         */
        ObjectOwnOpBeginCallback = -512,
        // An internal $PRODUCTNAME$ callback that is called immediately before the Operation takes place.
        /*!
         * @brief   OperationBeginが実行されたあと、実際に操作が行われる直前のうち、最後の優先度です。
         */
        LastOpBeginCall = -1,
        // An internal $PRODUCTNAME$ callback that is called immediately after the Operation takes place.
        /*!
         * @brief   実際にシステム操作が行われたあとの最初の優先度です。
         */
        FirstOpEndCall = 1,
        /*
        $ifdef ETERNA$
        An internal Eterna callback that is called if this Operation requires Duplica s to be created, /i.e./
        AddToStoreOperation or ChangeMasterStationOperation.
        $else$
        {secret}
        $endif$
        */
        DupSpaceMatchCallback = 500,
        /*
        $ifdef NETZ$
        An internal NetZ callback that is called if this Operation requires Duplica s to be created, /i.e./
        AddToStoreOperation or ChangeMasterStationOperation.
        $else$
        {secret}
        $endif$
        */
        DiscoveryCallback = 500,
        // An internal $PRODUCTNAME$ callback.
        ReleaseStationReferences = 600,
        // $PRODUCTNAME$ calls DuplicatedObject::OperationEnd or DataSet::OperationEnd.
        /*!
         * @brief   システムが、@ref DuplicatedObject::OperationEnd() またはDataSet::OperationEnd() を
         *          呼び出す優先度です。
         */
        ObjectOwnOpEndCallback = 512,
        // The last user-defined OperationCallback that will be called after $PRODUCTNAME$ calls OperationEnd.
        /*!
         * @brief   システムがOperationEndを実行したあとに実行されるなかで、最後の優先度です。
         */
        LastOpEndCall = 1024
    };


    //group=Methods

    // The caller is responsible to manage the pointer - we do not
    // take ownership of the pointer.

    /*
    Description: Registers an OperationCallback to this OperationManager.
    Parameters:
    pCallback: A pointer to the OperationCallback that will be registered.
    Remarks: Registered callbacks will be called in numerical order according to their
    allocated Priority, which is defined when the OperationCallback object is created.
    See Also: UnregisterCallback, OperationCallback
    */

    /*!
     * @brief   このOperationManagerに対してコールバック関数へのポインタを指定します。
     *
     * 登録された@ref OperationCallback は作成時に指定された優先度に従って
     * 実行されていきます。
     *
     * @see     UnregisterCallback, Operationcallback
     */
    void RegisterCallback(OperationCallback* pCallback);

    /*
    Description: Unregisters a previously registered OperationCallback from this OperationManager.
    Parameters:
    pCallback: A pointer to the OperationCallback that will be unregistered.
    See Also: RegisterCallback, OperationCallback
    */

    /*!
     * @brief   以前登録したOperationCallbackをマネージャーから登録解除します。
     *
     *
     * @param   pCallback [in] 登録解除をする対象のコールバック関数オブジェクトを指定します。
     * @see     RegisterCallback
     */
    qBool UnregisterCallback(OperationCallback* pCallback);

    // {secret}
    void InvokeCallbacks(qInt iMinPriority, qInt iMaxPriority, Operation* pOperation);
    // {secret}
    void PushOperation(Operation* pOperation)
    {
        m_stackOperations.push_back(pOperation);
    }
    // {secret}
    void PopOperation(Operation* pOperation);

private:
    qList<OperationCallback*> m_lstCallbacks;
    qList<Operation*> m_stackOperations;
};

END_NAMESPACE_NEX

