﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/UtilityDecl.h>
#include <OnlineCore/src/Core/Chrono.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Platform/Core/Condition.h>

BEGIN_NAMESPACE_NEX

// Group=Time Management

/*
Summary:  Provides an easy way to ensure an activity goes no faster than a specified frequency.
Remarks:  The Pacer class provides an easy way to pace a particular activity
to ensure that it goes no faster than a specified frequency. It is important to note
that the pacer can only sleep to affect the frequency and therefore, can slow down but not
speed up processing time.
*/

/*!
 *  @brief 設定された最大周波数の間隔より早く処理が進まないように、必要に応じてビジーウェイトをするクラスです。
 *
 *  設定された周波数より遅い場合は、ウェイトしません。
 *
 */
class Pacer: public RootObject
{

    // Group=Methods

public:

    /*
    Parameters:
    dFrequency: The maximum frequency at which you want the activity to be performed. The default value
    is zero.
    Remarks: This constructor constructs a Pacer object and calls SetFrequency(dFrequency) on it.
    */

    /*!
     *  @brief コンストラクタです。
     *
     *
     *  @param dFrequency 最大周波数[Hz]を設定します。デフォルトは0で、ウェイトなしです。
     *
     */
    Pacer(qReal dFrequency=0);

    /*!
     *  @brief デストラクタです。
     *
     */
    virtual ~Pacer();

    // Description: Sets the maximum frequency, in Hertz, at which
    // the paced activity will be performed.
    // Parameters:
    // dFrequency: Frequency, in Hertz, at which the Pacer runs. If
    // the value of dFrequency is 0 the Pacer will not delay the calls to Pace.
    // Remarks:  A frequency of 10 Hz ( /i.e./ 10 intervals per second) will cause
    // the Pacer to remain in the Pace method long enough to ensure that the calls
    // to perform the paced activity do not occur without at least a 0.1 second
    // delay ( /i.e./ 1/10). Therefore, if the processing time between two consecutive
    // calls to Pace is 0.02 seconds, Pace will sleep for 0.08 seconds.
    // See Also: GetFrequency
    /*!
     *  @brief 最大周波数を設定します。
     *
     *
     *  @param[in] dFrequency 最大周波数[Hz]を設定します。0は、ビジーウェイトなしです。
     *
     *
     *  @see GetFrequency()
     *
     */
    void SetFrequency(qReal dFrequency);

    // Description: Returns the frequency of the Pacer.
    // Returns: The pacer frequency, in HertZ, that was set via SetFrequency or upon construction.
    // See also: SetFrequency
    /*!
     *  @brief 最大周波数を取得します。
     *
     *  @return 最大周波数[Hz]を取得します。
     *
     *  @see SetFrequency()
     *
     */
    qReal GetFrequency() const
    {
        return m_dFrequency;
    }

    // Description: Causes the pacer to return immediately from the Pace call.
    // Remarks: This method is called from another thread to cause the thread
    // that called Pace to exit from the method. Use this function to exit
    // from Pace while it is sleeping to pace an activity's frequency of occurrence.
    /*!
     *  @brief Pace()実行時のビジーウェイトをキャンセルします。
     *
     */
    void Interrupt();

    /*
    Description: Causes a delay to ensure that the paced activity is called at a rate
    no faster than a set frequency.
    Remarks:  This is the main method of the pacer. It actually does the pacing.
    Assume that the frequency is set via SetFrequency to 10Hz ( /i.e./
    10 intervals/second or 0.1 seconds/interval). If the delay between two
    calls to Pace is 0.06 s, the second call will sleep for 0.04 s to ensure
    that the method is called at a rate no faster than the set frequency.

    The Pacer also records statistics about the processing time, which is the
    time between two consecutive calls to the method Pace. For the above
    example the processing time is equal to 0.06 s.

    Note:  When Pace is called, the thread goes to sleep if a delay shorter
    than 1/dFrequency has occured since the last call. If the value of
    dFrequency is 0, it returns immediately.
    */

    /*!
     *  @brief 最大周波数より早くならないペースになるように、必要に応じてビジーウェイトします。
     *
     *  Pacerクラスのメインメソッドです。
     */
    virtual void Pace();

    // Description: Returns the statistics associated with the Pacer.
    // Returns: The statistics associated with the Pacer. The
    // statistics are calculated since the last time the method ResetStats
    // was called, or since the Pacer was created, whichever happened last.
    // Parameters:
    //   ptiTotal: Total elapsed time since the creation of the Pacer or
    //   since the last call to ResetStats.
    //   ptiTotalProcessing: Total time spent between the calls to Pace.
    //   ptiMinProcessing: Minimum time spent between two calls to Pace.
    //   ptiMaxProcessing: Maximum time spent between two calls to Pace.
    //   puiLoops: The number of times Pace was called since the creation
    //   of the Pacer or a call to ResetStats.
    //See Also: ResetStats



    /*!
     *  @brief 統計情報を取得します。
     *  @param[out] ptiTotal ResetStats()もしくは、生成からの経過時間を返します。
     *  @param[out] ptiTotalProcessing Pace()とPace()の間の他の処理合計時間を返します。
     *  @param[out] ptiMinProcessing 二つのPace()間の他の処理時間の最小値を返します。
     *  @param[out] ptiMaxProcessing 二つのPace()間の他の処理時間の最大値を返します。
     *  @param[out] puiLoops ResetStats()もしくは、生成からのPace()が呼ばれた回数を返します。
     */
    void GetStats(TimeInterval* ptiTotal,
                  TimeInterval* ptiTotalProcessing,
                  TimeInterval* ptiMinProcessing,
                  TimeInterval* ptiMaxProcessing,
                  qUnsignedInt32 *puiLoops) const;

    /*
    Description: Resets all the statistics for the Pacer to zero.
    Remarks: Refer to GetStats for a description of the Pacer statistics that can be returned.
    See Also: GetStats
    */

    /*!
     * @brief 統計情報をリセットします。
     *
     */
    void ResetStats();

private:
    qReal m_dFrequency;
    Chrono m_oChrono;
    qBool m_bFirstLoop;
    qBool m_bInterrupted;
#ifdef NEX_CONDITION_TIMEOUT_SUPPORT
    Condition m_oCondition;
#else
    EmulatedCondition m_oCondition;
#endif
    NonReentrantableForcedCriticalSection m_oCsForCondition;

protected:
    TimeInterval m_tiDesiredLoopTime;

    TimeInterval m_tiTotal, m_tiTotalProcessing, m_tiMinProcessing, m_tiMaxProcessing;

    qUnsignedInt32 m_uiLoops;
    qBool m_bStatsRecorded;
};

END_NAMESPACE_NEX


