﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>

#include <OnlineCore/src/Core/Job.h>
#include <OnlineCore/src/Platform/Core/qChain.h>
#include <OnlineCore/src/Platform/Core/qReservedQueue.h>
#include <OnlineCore/src/Platform/Core/qMultiMap.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/Condition.h>
#include <OnlineCore/src/Platform/Core/ProfilingUnit.h>
#include <OnlineCore/src/Platform/Core/HighResolutionChrono.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Core/WorkerThreads.h>
#include <OnlineCore/src/Core/CallPolicy.h>
#include <OnlineCore/src/Core/Core.h>
#include <OnlineCore/src/Core/Pacer.h>

#ifdef NEX_WATERMARK_ENABLED
#include <OnlineCore/src/Platform/Core/WaterMark.h>
#endif


BEGIN_NAMESPACE_NEX

class Core;

//group=Miscellaneous Classes

typedef qBool (*PreventBlockCallCallback)(RootObject* object, qUnsignedInt32 uiTimeout);

/*
Description: Dispatches Jobs when $PRODUCTNAME$ is run in user-thread mode.
Remarks: This class is used to dispatch Jobs when $PRODUCTNAME$ is run in user-thread mode. If
you direct $PRODUCTNAME$ to use internal threads for processing you should not call any of
the methods of this class as Job scheduling is performed automatically.
See Also: Threading Model, Core::UseThreads
*/

/*!
 * @brief   ジョブの実行をスケジューリングするクラスです。
 *
 * SchedulerクラスはNEXのジョブを実行するために利用します。
 * アプリケーションから必ず @ref Scheduler::Dispatch 、または @ref Scheduler::DispatchAll を定期的に呼び出してジョブを実行してください。
 *
 * このクラスはシングルトンです。
 * システムによってインスタンスが自動的に作成、解放されます。
 */
class Scheduler: public RootObject
{
    //group=Methods
public:
    /*!
     * @internal
     * @brief   Scheduler のワーカースレッドクラスです。インターナルスレッドで利用されます。
     */
    class SchedulerWorkerThread: public WorkerThreads
    {
    public:
        SchedulerWorkerThread()
            : m_pScheduler(NULL)
            , m_pacer()
        {
            if (s_tiInternalThreadDispatchInterval == 0)
            {
                // インターバルが0の場合はウェイトなし。
                m_pacer.SetFrequency(0);
            }
            else
            {
                m_pacer.SetFrequency(1000.0f / s_tiInternalThreadDispatchInterval);
            }
        }
        virtual ~SchedulerWorkerThread() {};

        void AssociateScheduler(Scheduler* pScheduler)
        {
            m_pScheduler=pScheduler;
        }

        virtual void Initialize()
        {
            m_pScheduler->InitializeThread();
        }

        virtual void Work()
        {
            // インターナルスレッドでのディスパッチ
            // オリジナルはm_pScheduler->Dispatch(100, ContinueWhenEmpty);
            // として100ms必ず待っていたが、こうするとトランスポートスレッドを外したときに
            // ジョブが無いときは100msごとにしかパケットの転送が行われなくなるのでContinueWhenEmptyを無くした。
            m_pScheduler->DispatchInternal(100);
            m_pacer.Pace();
        }

    protected:
        // This accessor is there in case we want to change the
        // way that the dispatch work is done.
        Scheduler* GetScheduler()
        {
            return m_pScheduler;
        }

    private:
        Scheduler* m_pScheduler;
        Pacer m_pacer;
    };

    // Description: Enumerates the flags you can use to control the Scheduler's behavior.
    // See Also: DispachFlag
    /*!
     * @brief   @ref Dispatch が呼び出されたときにどのように振舞うかを指定するためのフラグです。
     */
    enum DispachFlag
    {
        // This flag keeps the Scheduler running until the uiDispatchTimeout expires, even if
        // all jobs have already been processed. If any new jobs arrive before uiDispatchTimeout
        // expires, they will be processed.
        // このフラグが有効であれば、なにもジョブがなかったとしても
        // 指定されたタイムアウト時間が終了するまで Dispatch で待機します。
        // もし待機中に新しいジョブが途中で登録された場合、それらは実行されます。
        // 待機中はSpecialJobは実行されないため、待機中は通信処理は行われません。
        // ThreadSafeCallPolicy使用時(=非インターナルスレッド時)に複数のスレッドがDispatchした場合は
        // 二つ目のスレッドではディスパッチ処理がスキップされるため待機しません。
        /*!
         * @brief   利用できません。
         */
        ContinueWhenEmpty = 0x01,
        // This flag keeps the Scheduler running to prevent fault detection. With this flag, the Scheduler
        // will not dispatch any jobs for processing; it only provides a heartbeat to keep the
        // client connected.
        /*!
         * @brief   クライアントとの接続を維持するためのキープアライブ信号処理のみを行うフラグです。
         *          これを指定することで、接続が切れることを防ぐことが出来ます。
         */
        DispatchKeepAliveOnly = 0x02
    };

    // Dispatch()関数内のどのステージを実行中か
    // TransportJobで送信向けの処理を行うか受信向けの処理を行うかの判断に使う。
    enum DispatchStage
    {
        DISPATCH_STAGE_NOT_DISPATCHING,         // Dispatch()関数に入っていない
        DISPATCH_STAGE_PRE_EXECUTE_JOB,         // 通常ジョブ実行前(SpecialDispatch中)
        DISPATCH_STAGE_EXECUTING_NORMAL_JOB,    // 通常ジョブ実行中
        DISPATCH_STAGE_POST_EXECUTE_JOB         // 通常ジョブ実行後(SpecialDispatch中)
    };

#ifdef NEX_CLIENT_OPTIMIZATION
    static const TimeInterval DEFAULT_INTERNAL_THREAD_SLEEP_TIME_MS = 8;  // クライアントでは1フレームに二回呼ぶ程度
#else
    static const TimeInterval DEFAULT_INTERNAL_THREAD_SLEEP_TIME_MS = 2;  // サーバではなるべく短くなるように
#endif

private:
    // Not allowed, not implemented.
    //{secret}
    Scheduler(const Scheduler&);
    Scheduler& operator=(const Scheduler&);

public:
    friend class Core;

    /*
    Remarks: A Scheduler object should never be directly instantiated. Instead, it is created
    by $PRODUCTNAME$ prior to session creation.
    */

    /*!
     * @internal
     * @brief   このコンストラクタはアプリケーション側から呼び出さないでください。
     *          Schedulerオブジェクトはシステムによって自動的に作成されます。
     */
    Scheduler(const qChar* pName = NULL, qUnsignedInt32 uiExecutionLockGroup = LockMasks::SystemLock, SchedulerWorkerThread* pSchedulerWorkerThread = NULL);

    /*
    Remarks: A Scheduler object should never be directly destroyed. It is destroyed by
    $PRODUCTNAME$ when the session terminates.
    */

    /*!
     * @internal
     * @brief   このデストラクタはアプリケーション側から呼び出さないでください。
     *          Schedulerオブジェクトはシステム終了時、自動的に解放されます。
     */
    virtual ~Scheduler();

public:

    // {secret}
    void SetName(const qChar* szName);

    //{secret}
    void Queue(Job* pJob, qBool bFront = false);

    //{secret}
    qBool Cancel(Job *pJob);

    /*
    Description: Dispatches all Jobs in the dispatch queue for processing.
    Returns: An error will be returned if this method is called when $PRODUCTNAME$ is set to "internal threads" mode.
    Refer to the Threading Model overview for threading details.
    Remarks: This method dispatches all Jobs in the dispatch queue for processing. The method will
    return once all Jobs ready to execute have been dispatched.

    $ifdef NETZ$
    Typically this method should be
    called each frame as given below to ensure all objects receive the most recent information
    before their datasets are updated or refreshed. Within the main loop the recommended sequence
    of calls to minimize latency is given below. It is possible to call DispatchAll or Dispatch
    at other points in the sequence or only once within the main loop however, this will add some
    latency to the system. In addition, dispatching jobs twice within this loop has no negative
    effect on performance.

        1.  Dispatch jobs
        2.  Refresh all duplicas
        3.  Perform game state change (physics, AI, etc.)
        4.  Update all duplication masters
        5.  Dispatch jobs
        6.  Render the game state

    Even if you call DispatchAll in the main loop, depending on how you write your code you may also need to call this
    method at other times. The most common situation is during load time, or any other time consuming sequence for that
    matter, where if messages are not regularly dispatched it will be assumed that a given Station has faulted and left
    the Session as it is not sending messages.

    Refer to the Threading Model overview for an example of how to use this method.
    $endif$
    $ifdef RENDEZVOUS$
    Note: If Dispatch or DispatchAll is not called at least each minute, a fault will be declared by Rendez-Vous.
    $endif$

    See Also: Dispatch
    */

    /*!
     * @brief   登録されているすべてのジョブを実行します。
     *
     * この関数を呼び出すと、スケジューラに登録されているすべてのジョブを、空になるまで実行します。
     * ジョブの登録数が多いと、多くのCPU時間を消費してしまう可能性があります。
     * フレームレートの揺らぎを防ぐためには、タイムアウトを指定することが出来る
     * @ref Dispatch 関数を利用します。
     *
     * この関数は必ず定期的に呼び出してください。
     * レイテンシ(通信遅延)を最低限に抑えるために、以下のようなシーケンスでゲームループを
     * 構成することを推奨します。
     *
     * @li @ref DispatchAll 、@ref Dispatch の実行（受信想定の処理）
     * @li 受信データをアプリケーションでハンドリング。複製オブジェクトの場合はすべてDOデュプリカのリフレッシュ（受信データの反映）
     * @li ゲーム状態の変更（物理計算、AI など）
     * @li 送信データをセット。複製オブジェクトの場合はすべてのDOマスターの更新（送信データの準備）
     * @li @ref DispatchAll 、@ref Dispatch の実行（送信想定の処理）
     * @li レンダリングなどアプリケーション側の処理
     *
     * もしも極度にブロックするような処理(たとえば画面暗転中のファイルロード)を
     * メインスレッドで実行する場合は注意が必要です。メインループの処理が滞り、
     * ジョブの実行が行われなくなると、メッセージの送受信に悪影響が出て、
     * 結果的にタイムアウトが発生したと判定されてしまう場合があります。
     *
     * インターナルスレッドを利用している場合は本関数を呼び出しても何も行いません。
     *
     */
    void DispatchAll()
    {
        Dispatch(WAIT_INFINITE_TIMEOUT, 0);
    }

    /*
    Description: Returns a pointer to the Scheduler instance.
    Returns: A reference to the Scheduler object.
    Remarks: Use the reference returned by this method to control the Scheduler object.
    For example, all Jobs in the dispatch queue could be dispatched for processing as follows:

            Scheduler::GetInstance()->DispatchAll();
    */

    /*!
     * @brief   唯一のSchedulerオブジェクトのインスタンスを取得します。
     *
     * @return  Schedulerオブジェクトのインスタンスへのポインタが返ります。
     */
    static Scheduler* GetInstance()
    {
        Core* pCore = Core::GetInstance();
        if (pCore==NULL)
        {
            return NULL;
        }
        else
        {
            return pCore->GetScheduler();
        }
    }

    /*
    Description: Starts a dispatcher thread on this Scheduler object.
    Returns: true if a dispatcher thread was successfully started, false if not or an error occurred.
    Remarks: If Core::UseThreads is set to true, then when $PRODUCTNAME$ is created a Schedular object is automatically
    created and a dispatcher thread started. This thread may then be stopped (StopDispatcherThread) and restarted
    (StartDispatcherThread) at will during run time. For example, if you want a specific section of code to be run in
    user threads but the rest run in internal threads, then you can stop the dispatcher thread while that specific code
    is being executed.
    See Also: StopDispatcherThread, HasDispatcherThread, Core::UseThreads
    */
    qBool StartDispatcherThread();

    /*
    Description: Stops the dispatcher thread on this Scheduler object.
    Returns: true if the dispatcher thread was successfully stopped, false if not or an error occurred.
    Remarks: If Core::UseThreads is set to true, then when $PRODUCTNAME$ is created a Schedular object is automatically
    created and a dispatcher thread started. This thread may then be stopped (StopDispatcherThread) and restarted
    (StartDispatcherThread) at will during run time. For example, if you want a specific section of code to be run in
    user threads but the rest run in internal threads, then you can stop the dispatcher thread while that specific code
    is being executed.
    See Also: StartDispatcherThread, HasDispatcherThread, Core::UseThreads
    */
    qBool StopDispatcherThread();

    /*
    Description: Returns whether or not this Scheduler object uses a dispatcher thread.
    Returns: true if this Scheduler object uses a dispatcher thread, false if not or an error occurred.
    Remarks: The Scheduler will use dispatcher threads if $PRODUCTNAME$ has been instructed to uses threads,
    which is achieved by via setting Core::UseThreads to true.
    See Also: StartDispatcherThread, StopDispatcherThread, Core::UseThreads
    */
    qBool HasDispatcherThread();

    //n1589:memo
    /*
    Description: Dispatches Jobs in the queue for processing for a given time.
    Parameters:
    uiDispatchTimeout: The maximum time in milliseconds that the system will wait. The default
    uiDispatchTimeout is infinite.
    uiFlags: _DispatchFlag provides a method for keeping the Scheduler running even if there are no
    threads to process.
    uiFlags: You can set DispatchFlag s to determine behavior if all Jobs have been dispatched before
    the timeout is exceeded. You can set multiple flags using AND.
    Returns: An error will be returned if this method is called when $PRODUCTNAME$ is set to "internal threads" mode.
    Refer to the Threading Model overview for threading details.
    Remarks: This method dispatches Jobs in the dispatch queue for processing until either the timeout
    is exceeded or the queue is empty. The ContinueWhenEmpty flag keeps the Scheduler running until the
    uiDispatchTimeout expires, even if all jobs have already been processed. If any new jobs arrive before
    uiDispatchTimeout expires, they will be processed. The DispatchKeepAliveOnly flag keeps the Scheduler
    running to prevent fault detection. With this flag, the Scheduler will not dispatch any jobs for processing;
    it only provides a heartbeat to keep the client connected.

    $ifdef NETZ$
    Typically this method should be called each frame as given below to ensure all objects receive the
    most recent information before their datasets are updated or refreshed. Within the main loop the
    recommended sequence of calls to minimize latency is given below. It is possible to call DispatchAll
    or Dispatch at other points in the sequence or only once within the main loop however, this will add
    some latency to the system. In addition, dispatching jobs twice within this loop has no negative
    effect on performance.

        1.  Dispatch jobs
        2.  Refresh all duplicas
        3.  Perform game state change (physics, AI, etc.)
        4.  Update all duplication masters
        5.  Dispatch jobs
        6.  Render the game state

    Even if you call Dispatch in the main loop, depending on how you write your code you may also need to call this
    method at other times. The most common situation is during load time, or any other time consuming sequence for that
    matter, where if messages are not regularly dispatched it will be assumed that a given Station has faulted and left
    the Session as it is not sending messages.

    Refer to the Threading Model overview for an example of how to use this method.
    $endif$
    $ifdef RENDEZVOUS$
    Note: If Dispatch or DispatchAll is not called at least each minute, a fault will be declared by Rendez-Vous.
    $endif$

    See Also: DispatchAll, DispatchFlag
    */

    /*!
     * @brief   登録されているジョブをタイムアウトを設けて実行します。
     *
     * この関数を実行すると、登録されているジョブを実行します。
     * ただし、指定されたタイムアウトが経過した場合、この関数はジョブの実行を中断してゲームに制御を戻します。
     * 実行されなかったジョブは、次回以降@ref Dispatch() を呼び出したときに実行されます。
     *
     * この関数は必ず定期的に呼び出してください。
     * レイテンシ(通信遅延)を最低限に抑えるために、以下のようなシーケンスでゲームループを
     * 構成することを推奨します。
     *
     * @li @ref DispatchAll 、@ref Dispatch の実行（受信想定の処理）
     * @li 受信データをアプリケーションでハンドリング。複製オブジェクトの場合はすべてDOデュプリカのリフレッシュ（受信データの反映）
     * @li ゲーム状態の変更（物理計算、AI など）
     * @li 送信データをセット。複製オブジェクトの場合はすべてのDOマスターの更新（送信データの準備）
     * @li @ref DispatchAll 、@ref Dispatch の実行（送信想定の処理）
     * @li レンダリングなどアプリケーション側の処理
     *
     * もしも極度にブロックするような処理(たとえば画面暗転中のファイルロード)を
     * メインスレッドで実行する場合は注意が必要です。メインループの処理が滞り、
     * ジョブの実行が行われなくなると、メッセージの送受信に悪影響が出て、
     * 結果的にタイムアウトが発生したと判定されてしまう場合があります。
     *
     * また、この関数に指定するタイムアウトが短すぎる場合、
     * 登録されていくジョブの数が処理可能なジョブの数を上回ってしまうことがあります。
     * このとき、いつまで経ってもジョブの実行が先に進まなくなり、
     * 上記のような状況に陥る場合があります。
     * アプリケーションの仕様とあわせてタイムアウト時間やトラフィック量の調整を行ってください。
     * 残っているジョブの数は Scheduler::GetReadyJobsSize() で確認できます。
     *
     * タイムアウトは正確な数値ではありません。
     * ジョブの内容や数、タイミングによって指定した時間をオーバーする可能性がありますのでご注意ください。
     *
     * インターナルスレッドを利用している場合は本関数を呼び出しても何も行いません。
     *
     * @param[in]   uiDispatchTimeout    タイムアウトをミリ秒単位で指定します。0 を渡すとタイムアウトがなしになり、Scheduler::DispatchAll() と同様に全てのジョブを実行します。
     * @param[in]   uiFlags              ジョブの実行方式をビットフラグ形式で指定します(デフォルトはフラグなし)。 実行方式については @ref DispachFlag を参照してください。
     * @return 常に 0 を返します。
     * @see Scheduler::DispachFlag, Scheduler::GetReadyJobsSize()
     */
    size_t Dispatch(qUnsignedInt32 uiDispatchTimeout, qUnsignedInt32 uiFlags = 0);

    // インターナルスレッドを使っていてもディスパッチする。ライブラリ内部からディスパッチする場合に使用する。
    // DispatcherWorkerThreadから呼び出せるようにpublicにする。
    size_t DispatchInternal(qUnsignedInt32 uiDispatchTimeout, qUnsignedInt32 uiFlags = 0);

    //{secret}
    void SpecialDispatch(const Time& tDeadline, Time currentTime);
    //{secret}
    void InitializeThread();

    //{secret}
    qBool WorkerThreadIsRunning() const
    {
        return m_pWorkerThreads->IsRunning();
    }


    //{secret}
    WorkerThreads* GetWorkerThreads()
    {
        return m_pWorkerThreads;
    }

    //{secret}
    size_t GetNbWorkers() const
    {
        return m_pWorkerThreads->GetNbWorkers();
    }

    //{secret}
    qBool IsAWorker( ThreadID id ) const
    {
        return m_pWorkerThreads->IsAWorker( id );
    }

    //{secret}
    static void GlobalSingleThreadDispatch(qUnsignedInt32 uiTimeout=WAIT_INFINITE_TIMEOUT);
    //{secret}
    static void GlobalSingleThreadSpecialDispatch();
    //{secret}
    static void BypassGlobalSingleThreadDispatch();
    //{secret}
    static void UseGlobalSingleThreadDispatch();

    /*!
     * @brief   インターナルスレッドが Scheduler::Dispatch() を呼び出す間隔を設定します。初期値は 8 ms です。
     *
     * Core::SetThreadMode() に Core::ThreadModeInternal もしくは Core::ThreadModeInternalTransportBuffer を設定した場合にこの設定が使用されます。
     * NetZ もしくは NgsFacade オブジェクトを生成する前に設定してください。
     *
     * @param[in] interval Scheduler::Dispatch() を呼び出す間隔
     */
    static void SetInternalThreadDispatchInterval(TimeInterval interval) { s_tiInternalThreadDispatchInterval = interval; }

    /*!
     * @brief   インターナルスレッドが Scheduler::Dispatch() を呼び出す間隔を取得します。
     *
     * @return Scheduler::Dispatch() を呼び出す間隔
     */
    static TimeInterval GetInternalThreadDispatchInterval() { return s_tiInternalThreadDispatchInterval; }
private:
    static void SingleThreadDispatch(qUnsignedInt32 uiTimeout);
    static void SingleThreadSpecialDispatch();

public:

    //{secret}
    void EnableActivityTrace(qBool bState = true);

    //{secret}
    qBool DoesTraceActivity() const
    {
        return m_bTraceActivity;
    }

#ifdef NEX_TRACE_ENABLED
    /*
    Description: Traces the contents of this Scheduler object.
    Remarks: This method traces the list of jobs that are currently scheduled.
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const;
    //{secret}
    void TraceJobState(const qChar* szEvent, Job* pJob);
#else
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        DUMMY_PARAM(uiTraceFlags);
    };
    void TraceJobState(const qChar* /*szEvent*/, Job* /*pJob*/) {}
#endif

    // 一つのJobStep実行時間が指定した時間以上となったときにログでWarningを出す。INFINITE_LARGE_TIME_INTERVALを指定するとログを出さなくなります。
    // 引数はマイクロ秒
    static void SetJobStepExecutionBusyIntervalThreshold(LargeTimeInterval interval);

    void WaitForAllJobsToComplete(TimeInterval timeout, qBool immidiately=true);
    //{secret}
    void WaitForEmptyQueues() const;
    //{secret}
    size_t GetTotalQueueSize() const;
    //{secret}
    size_t GetReadyQueueSize() const;
    /** \internal */
    size_t GetOneTypeJobsCount(Job::JobType jobType) const;
    //{secret}
    void ClearAllQueues();

    /*!
     * @brief   現在 Scheduler が実行可能なジョブの個数を取得します。
     *
     * 実行可能なジョブの個数が増加傾向である場合は Scheduler::Dispatch() のタイムアウト時間が短いか、より高頻度で呼び出す必要があることを示します。
     * 
     * @return 現在実行可能なジョブの個数。
     */
    size_t GetReadyJobsSize() const
    {
        return GetReadyQueueSize();
    }
    
    //{secret}
    static qBool CurrentThreadCanWaitForJob();

    //{secret}
    qBool CurrentThreadIsDispatchingJobs() const
    {
        return m_pDispatchCallPolicy->CurrentThreadIsCalling();
    }

    qBool OtherThreadIsDispatchingJobs() const
    {
        return m_pDispatchCallPolicy->OtherThreadIsCalling();
    }

    // {secret}
    void IgnoreWaitTime()
    {
        m_bIgnoreWaitTime=true;
    }
    // {secret}
    void RestoreWaitTime()
    {
        m_bIgnoreWaitTime=false;
    }

    //{secret}
    void RegisterSpecialDispatchJob(Job* pJob, qBool bFront = false);
    //{secret}
    void UnregisterSpecialDispatchJob(Job *pJob);
    //{secret}

    /*
    Description: Returns a reference to the CriticalSection that currently locks the system.
    Returns: A reference to the CriticalSection that is currently used to lock the system.
    Remarks: Once you have a reference to a CriticalSection, you may then enter and leave it
    as required. Note that once a CriticalSection is entered and therefore, the system locked,
    you cannot call any blocking functions, such as CallContext::Wait or Dispatch, until you
    leave the CriticalSection. Refer to Threading Model for more information on blocking functions.
    Example:
        void MyFunction() {
            // Lock the system. No state change will occur until
            // the block is exited.
            ScopedCS oCS(Scheduler::GetInstance()->SystemLock());

            // Protected block of code. You cannot call any blocking
            // function while the system is locked.
        }

    See Also: CriticalSection, ScopedCS, Threading Model
    */

    /*!
     * @brief   システムをロックするためのクリティカルセクションの参照を返します。
     *
     * 本関数で取得した @ref CriticalSection に入った後、システムがロックされます。@n
     * システムのロック中に @ref CallContext::Wait() や @ref Dispatch() などのブロッキング関数を呼ぶことは出来ません。
     *
     * @return  システムをロックするためのクリティカルセクションの参照を返します。
     *
     * @see     CriticalSection, ScopedCS
     */
    CriticalSection& SystemLock()
    {
        // We don't offten check the validity of the Scheduler::GetInstance()
        // pointer before we call this method and we are therefore able to
        // return a refrence to an invalid CriticalSection which
        // then leads to hard to diagnose problems since the program will
        // only crash once the member of some invalid object is actually
        // refrenced. Checking that the 'this' pointer is valid will catch
        // this error.
        SYSTEMCHECK(this != NULL);

        return m_csSystemLock;
    }

#ifdef NEX_LOCK_DEBUG_ENABLED
    //Note for bug 2531: if a SYSTEMCHECK happens upon returned value of IsLockedByCurrentThread(),
    //even if no internal threads are used (e.g. Core::UseThreads(false); ), this means that
    //another thread, somewhere, has entered the scope of the system lock.
    //In this case, we might recommend any given user to not make calls of RV or NetZ from another
    //thread, unless he is implementing his own locking mechanism.
    //See bug 2531 for details.

    //{secret}
    qBool IsLockedByCurrentThread()
    {
        return m_csSystemLock.HasOwnership();
    }

#endif //NEX_LOCK_DEBUG_ENABLED

    //{secret}
    static CriticalSection& GlobalSystemLock()
    {
        return s_csGlobalSystemLock;
    }

    //n1589
#ifdef NEX_LOCK_DEBUG_ENABLED
    qBool IsGlobalLockedByCurrentThread()
    {
        return s_csGlobalSystemLock.HasOwnership();
    }
#endif

    //{secret}
    qBool IsQueued(const Job* pJob) const;

    size_t GetNbSpecialJobs() const;

    void ProcessJobsInTermination(Job::JobType jobType);
#if NEX_ENABLE_IMPROVE_DO_CORRUPT
    void TerminateDOCoreJobs();
    void DeleteDOCoreJobs(qChain<Job*>* m_lstQueue);
    void DeleteDOCoreJobs(qMultiMap<Time, Job*>* m_mapQueue);
#endif
    void ForceTerminateJobs();

    void RequeueWaitingJob(Job* pWaitingJob, TimeInterval waitTime);

    // 現在のDispatch()がどのステージを実行中かを返す。
    DispatchStage GetDispatchStage() const
    {
        return m_dispatchStage;
    }
private:
    void QueueImpl(Job* pJob, qBool bFront);

    void IncrementNbJobs();
    void DecrementNbJobs();

    void DispatchBegins();
    void DispatchEnds();

    void PopFirstJob();

    void WaitForJobToExecute(qUnsignedInt32 uiTimeout);

    void ProcessJob(Job* pJob, const Time& tDeadline, Time currentTime);
    void DeleteJob(Job* pJob);

    // {secret}
    void ProcessCurrentJob(const Time& tDeadline, Time currentTime);
    //{secret}
    qBool QueueNewlyReadyJobs(Time currentTime);
    // {secret}
    qBool JobIsReady(Job* pJob, const Time& tExecutionTime, const Time& tNow);

    size_t GetTotalNumberOfJobs() const;

    //{secret}
    size_t DispatchImpl(Time tDeadline, Time currentTime);

    // デッドラインまでの時間(ミリ秒)を返す。
    // currentTimeにTime(0)を指定すると現在時刻からのデッドラインまでの時間を返す。
    qUnsignedInt32 TimeToDeadline(Time tDeadline, Time currentTime=Time(0)) const;
    // デッドラインが過ぎているかどうかを返す。
    // currentTimeにTime(0)を指定すると現在時刻がデッドラインを過ぎているかどうかを返す。
    qBool DeadlineExpired(Time tDeadline, Time currentTime=Time(0)) const
    {
        return TimeToDeadline(tDeadline, currentTime)==0;
    }

#if NEX_DISABLE_SCHEDULER_PERFCOUNTER_OPTIMIZATION
#else
    // {secret}
    ProfilingUnit& DispatchProfilingUnit()
    {
        return m_puDispatchInterval;
    }
    // {secret}
    const ProfilingUnit& DispatchProfilingUnit() const
    {
        return m_puDispatchInterval;
    }
    // {secret}
    void SetMaxDispatchInterval(TimeInterval tiInterval)
    {
        s_tiMaxDispatchInterval = tiInterval;
    }
#endif  // !defined(NEX_DISABLE_SCHEDULER_PERFCOUNTER_OPTIMIZATION)

private:
    Job*  m_pCurrentJob;
    volatile qBool m_bCurrentJobValid;
    SchedulerWorkerThread* m_pWorkerThreads;
#ifdef NEX_CONDITION_TIMEOUT_SUPPORT
    Condition m_oCondition;
#else
    EmulatedCondition m_oCondition;
#endif
    NonReentrantableForcedCriticalSection m_oCsForCondition;

    //{secret}
    mutable CriticalSection m_csSystemLock;

    //{secret}
    mutable CriticalSection m_csQueue;


    // For NEX Developer <wiki/Core Threading Model#queues>
    //{secret}
    qChain<Job*> m_lstReadyQueue;
    qChain<Job*> m_lstSuspendedQueue;
    qMultiMap<Time, Job*> m_mapWaitingQueue;

    // Special jobs.  These special dispatch job is used to
    // integrate the transport work, in single threaded operation mode.
    // This is a hack - as no generalized form of such a special job is
    // required.  This mecanism should eventually be generalized.
    // {secret}
    qChain<Job*> m_lstSpecialDispatchJobs;

    qBool m_bIgnoreWaitTime;

#ifdef NEX_WATERMARK_ENABLED
    WaterMark m_oSchedulerJobs;
    WaterMark m_oBusyRatio;
#endif

    static qBool s_bGlobalDispatch;
    Time m_tiLastDispatchStart;
    Time m_tiLastDispatchEnd;
    // A lock valid accross all instances of the scheduler (when using multiple contextes)
    //{secret}
    static CriticalSection s_csGlobalSystemLock;

    // Instance context in which this scheduler must run.
    //{secret}
    ICContext m_icContext;

    // ディスパッチのポリシー。
    // 非インターナルスレッドモードではThreadSafeCallPolicy、
    // (非インターナルスレッドモードでは複数のスレッドから同時にDispatch()される可能性があるため。)
    // インターナルスレッドモードではSingleThreadCallPolicyが使用される。
    // (インターナルスレッドモードではインターナルスレッド(ワーカースレッド)からのみDispatch()されるため。)
    CallPolicy* m_pDispatchCallPolicy;

    Time m_prevDispatchTime;

#if NEX_DISABLE_SCHEDULER_PERFCOUNTER_OPTIMIZATION
#else
    ProfilingUnit m_oJobProcessing;

    // (Perfcounter) Measures the frequency at which dispatch
    // is called.  This is useful to debug situations where it
    // isn't called often enough.
    ProfilingUnit m_puDispatchInterval;
    HighResolutionChrono m_oChrono; // to report info to the DispatchInverval.
    qBool m_bRecordDispatchInterval;

#endif  // !defined(NEX_DISABLE_SCHEDULER_PERFCOUNTER_OPTIMIZATION)
    static TimeInterval s_tiMaxDispatchInterval;
    static TimeInterval s_tiInternalThreadDispatchInterval; // インターナルスレッドのDispatch間隔

    qBool m_bTraceActivity;

    // Dispatch()のどのステージを実行中かを示す。
    DispatchStage m_dispatchStage;

public:
    static qBool PreventBlockCall(
        qBool bPotentialBlockTest,
        qUnsignedInt32 uiTimeout,
        qBool bErrorValue,
        PreventBlockCallCallback cbCallback,
        RootObject* object,
        qBool* pbReturn
    );

    static qBool PreventRegularBlockCall(
        qBool bPotentialBlockTest,
        qUnsignedInt32 uiTimeout,
        PreventBlockCallCallback cbCallback,
        RootObject* object,
        qResult* prReturn
    );
};

END_NAMESPACE_NEX

