﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Core/Key.h>
#include <OnlineCore/src/Core/Buffer.h>

BEGIN_NAMESPACE_NEX

class EncryptionAlgorithm;

/**
    \ingroup group_onlinecore_core
    \brief This class defines a secure storage thats take charge of encrypting
           and decrypting the data on Read and Write.

    The methods ReadImpl and WriteImpl needs to be implemented and must handle the read/write
    access to the storage. We use the NVI (Non-Virtual Interface) Idiom for this implementation.
    Refer to http://www.gotw.ca/publications/mill18.htm for more information.
*/
class OSDK_API SecureStorage : public RootObject
{
public:
    /**
        \brief Constructor
        \param[in] key Sets the key used by the encryption algorithm.
    */
    SecureStorage(const Key &key);

    /** \brief Destructor */
    virtual ~SecureStorage();

    /**
        \brief Decrypts and validates (checksum) the data read from ReadImpl.
        \returns The decrypted data
        \sa ReadImpl
    */
    String Read();

    /**
        \brief Encrypt the data to save. This method calls WriteImpl for the IO access.
        \param[in] dataToSave The data to encrypt
        \sa WriteImpl
    */
    void Write(const String& dataToSave);

private:
    /**
        \brief This method is called by Read method. This method is in charge of the IO access.
        \returns The data from IO
        \sa Read
    */
    virtual String ReadImpl() = 0;

    /**
        \brief This method is called by Write method. This method is in charge of the IO access.
        \param[in] dataToSave The data to save
        \sa Write
    */
    virtual void WriteImpl(const String& dataToSave) = 0;

    EncryptionAlgorithm* m_encryptionAlgorithm;
};

END_NAMESPACE_NEX

