﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <cstring>
#include <limits>
#include <OnlineCore/src/Core/UtilityDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/String.h>

BEGIN_NAMESPACE_NEX

//  Convert a string to uppercase
//  Returns szStr
qChar* StrUpper(qChar* szStr);

//  Convert a string to lowercase
//  Returns szStr
qChar* StrLower(qChar* szStr);

//  Duplicate a string
//  qSpecialDeleteArray MUST be called afterwards on returned value
qChar* StrDup(const qChar* szStr);

qVector<String> StringSplit(const String& str, qChar delimiter);

/*
    @brief  文字型 T1 配列 *pDst ( count 文の領域)へ 文字列 *pSrc を先頭から 最大 count - 1 文字コピーします。
            最後は、\\0 で終端することが保証されます。

    @details
    count が 0 のときは何もしません。
    pSrc の長さが count - 1 以上のときは、 count - 1 文字コピーします。 終端に \\0 の付加を行います。
    pSrc の長さが count - 1 より少ない場合には、残りの文字を \\0 で埋めます。

    @tparam T1 *pDstの文字の型
    @tparam T2 *psrcの文字の型
    @param[out] pDst    コピー先のバッファの先頭へのポインタを指定します。
    @param[in]  pSrc    コピー元の文字列の先頭へのポインタを指定します。
    @param[in]  count   コピー先のバッファの要素数を指定します。
    @return     実際にコピーした文字列数を返します。\\0を含みません。
 */


template<typename T1, typename T2>
size_t StrCopy(T1* pDst, const T2* pSrc, size_t count)
{
    SYSTEMCHECK( pDst != NULL );
    SYSTEMCHECK( pSrc != NULL );

    // マルチキャラクタとワイドキャラクタは単純コピーできないのでコンパイルエラーとする
    NEX_STATIC_ASSERT(!((sizeof(T1) == sizeof(qChar8) || sizeof(T2) == sizeof(qChar8)) && sizeof(T1) != sizeof(T2)));

    size_t length = 0;

    for(;count > 1 && *pSrc; --count, ++pDst, ++pSrc, ++length)
    {
        *pDst = static_cast<T1>(*pSrc);
    }

    for(;count > 0; --count, ++pDst)
    {
        *pDst = static_cast<T1>(0);
    }

    return length;
}


template<typename T>
size_t StrLen(const T* pStr)
{
    SYSTEMCHECK( pStr != NULL );

    size_t length = 0;
    for(; *pStr; ++pStr, ++length)
    {
    }

    return length;
}

/*
    @brief      バッファの要素数や終端に配慮した文字数の比較をおこないます。

    @details    与えられた二つの文字列に対し、最大で count - 1 番目までの文字を比較します。
                最初に発見した異なる文字の値が、 pStr1 の指す文字列のほうが小さかった場合は負の値が、大きかった場合は正の値が返ります。
                非 0 の値が返った場合に双方の文字列が異なることは保証されますが、 0 が返った場合に双方の文字列が一致することは保証されません。
                双方の文字列が一致していることを保証するには、両者の長さが count 未満であることを確認する必要があります。

    @tparam     T               文字の型です。
    @param[in]  pStr1           比較する文字列の一方の先頭へのポインタを指定します。
    @param[in]  pStr2           比較する文字列の他方の先頭へのポインタを指定します。
    @param[in]  count           文字列の比較をおこなう最大の要素数を指定します。
    @return     文字列が一致すれば 0 、一致しなければ非 0 の値を返します。
*/
template<typename T>
int StrNCmp( const T* pStr1, const T* pStr2, size_t count )
{
    SYSTEMCHECK( pStr1 != NULL );
    SYSTEMCHECK( pStr2 != NULL );

    if( count == 0 )
    {
        return 0;
    }

    T c1 = *pStr1;
    T c2 = *pStr2;
    for (;c1 && c1 == c2 && count > 1; count --)
    {
        c1 = *(++pStr1);
        c2 = *(++pStr2);
    };

    return c1 - c2;
}

/*
    @brief      大文字小文字を無視して、バッファの要素数や終端に配慮した文字数の比較をおこないます。

    @details    与えられた二つの文字列に対し、最大で count - 1 番目までの大文字小文字を無視して文字を比較します。
                最初に発見した異なる文字の値が、 pStr1 の指す文字列のほうが小さかった場合は負の値が、大きかった場合は正の値が返ります。
                非 0 の値が返った場合に双方の文字列が異なることは保証されますが、 0 が返った場合に双方の文字列が一致することは保証されません。
                双方の文字列が一致していることを保証するには、両者の長さが count 未満であることを確認する必要があります。

    @tparam     T               文字の型です。
    @param[in]  pStr1           比較する文字列の一方の先頭へのポインタを指定します。
    @param[in]  pStr2           比較する文字列の他方の先頭へのポインタを指定します。
    @param[in]  count           文字列の比較をおこなう最大の要素数を指定します。
    @return     文字列が一致すれば 0 、一致しなければ非 0 の値を返します。
*/
template<typename T>
int StrNCaseCmp( const T* pStr1, const T* pStr2, size_t count )
{
    SYSTEMCHECK( pStr1 != NULL );
    SYSTEMCHECK( pStr2 != NULL );

    if( count == 0 )
    {
        return 0;
    }

    int c1 = ::toupper(*pStr1);
    int c2 = ::toupper(*pStr2);
    for (;c1 && c1 == c2 && count > 1; count--)
    {
        c1 = ::toupper(*(++pStr1));
        c2 = ::toupper(*(++pStr2));
    };
    return c1 - c2;
}

/*
     @brief 部分文字列を検索します。

     @details 文字列 pBig 中で、部分文字列 pLittle が最初に出現する文字列 pBig 中の位置を検索します。

     @param pBig    検索対象となる文字列
     @param pLittle 検索する文字列
     @param limit   pBig を検索する文字数。pBig 内の limit 文字目までに pLittle が見つからない場合は NULL を返す。(pLittle の先頭文字が limit 文字目までに含まれていれば良い)
     @return 文字列 pLittle が最初に出現する文字列 pBig 中のポインタ。 見つからない場合は NULL 。 pLittle が空文字の場合は、 pBig。
 */
template<typename T>
T* StrStr(const T *pBig, const T* pLittle, size_t limit = (std::numeric_limits<size_t>::max)())
{
    SYSTEMCHECK( pBig != NULL );
    SYSTEMCHECK( pLittle != NULL );

    if (*pLittle == 0)
    {
        return const_cast<T *>(pBig);
    }
    size_t littleLen = StrLen(pLittle);

    for(size_t i = 0; *pBig && i < limit; pBig++, i++)
    {
        if (StrNCmp(pBig, pLittle, littleLen) == 0)
        {
            return const_cast<T *>(pBig);
        }
    }

    return NULL;
}

/*
     @brief 部分文字列を大文字小文字無視して、検索します。

     @details 文字列 pBig 中で、部分文字列 pLittle が最初に出現する文字列 pBig 中の位置を大文字小文字を無視して検索します。

     @param pBig    検索対象となる文字列
     @param pLittle 部分検索する文字列
     @param limit   pBig を検索する文字数。pBig 内の limit 文字目までに pLittle が見つからない場合は NULL を返す。(pLittle の先頭文字が limit 文字目までに含まれていれば良い)
     @return 文字列 pLittle が最初に出現する文字列 pBig 中のポインタ。 見つからない場合は NULL 。 pLittle が空文字の場合は、 pBig。
 */
template<typename T>
T* StrCaseStr(const T *pBig, const T* pLittle, size_t limit = (std::numeric_limits<size_t>::max)())
{
    SYSTEMCHECK( pBig != NULL );
    SYSTEMCHECK( pLittle != NULL );

    if (*pLittle == 0)
    {
        return const_cast<T *>(pBig);
    }
    size_t littleLen = StrLen(pLittle);

    for(size_t i = 0; *pBig && i < limit; pBig++, i++)
    {
        if (StrNCaseCmp(pBig, pLittle, littleLen) == 0)
        {
            return const_cast<T *>(pBig);
        }
    }

    return NULL;
}

/*
     @brief 部分文字列を検索します。

     @details 文字列 pSrc 中で、部分文字列 pSubstr が最初に出現する位置を返します。

     @param pSrc    検索対象となる文字列
     @param pSubstr 検索する部分文字列
     @param startOffset  検索対象となる文字列の開始位置
     @return 先頭を0とする見つかった位置。-1の場合は、見つからなかった。
 */
template<typename T>
std::ptrdiff_t FindSubstring(const T* pSrc, const T* pSubstr, size_t startOffset  = 0)
{
    if (pSrc == NULL || pSubstr == NULL || *pSrc == 0 || *pSubstr == 0)
    {
        return -1;
    }

    T* ptr = StrStr(pSrc + startOffset , pSubstr);
    if (ptr == NULL)
    {
        return -1;
    }
    return ptr - pSrc;
}

/*
     @brief 部分文字列を大文字小文字を無視して検索します。

     @details 文字列 pSrc 中で、部分文字列 pSubstr が最初に出現する位置を返します。

     @param pSrc    検索対象となる文字列
     @param pSubstr 検索する部分文字列
     @param startOffset  検索対象となる文字列の開始位置
     @return 先頭を0とする見つかった位置。-1の場合は、見つからなかった。
 */
template<typename T>
std::ptrdiff_t FindSubstringNoCase(const T* pSrc, const T* pSubstr, size_t startOffset  = 0)
{
    if (pSrc == NULL || pSubstr == NULL || *pSrc == 0 || *pSubstr == 0)
    {
        return -1;
    }

    T* ptr = StrCaseStr(pSrc + startOffset , pSubstr);
    if (ptr == NULL)
    {
        return -1;
    }
    return ptr - pSrc;
}

/*
     @brief 文字列がワイルドカード文字列とマッチするかを調べます。

     @details 文字列 pTarget が ワイルドカード文字列 pWildcard とマッチするかどうかを返します。
              '*'は任意の0文字以上の文字列、'?'は任意の1文字とマッチします。

     @param pTarget   検索対象となる文字列
     @param pWildcard 検索するワイルドカード文字列
     @return マッチしたかどうか。
 */
template<typename T1, typename T2>
qBool WildcardMatch(const T1* pTarget, const T2* pWildcard)
{
    const T1* pCurrentTarget = pTarget;
    const T2* pCurrentWildcard = pWildcard;

    while(true)
    {
        if (*pCurrentTarget == 0)
        {
            // 検索対象が終端まで来たらワイルドカード文字列側が終端ならマッチ
            if (*pCurrentWildcard == 0)
            {
                return true;
            }
            
            // ワイルドカード文字列側が*ならワイルドカード文字列を読み進める
            if (*pCurrentWildcard == '*')
            {
                return WildcardMatch(pCurrentTarget, pCurrentWildcard + 1);
            }

            // それ以外の文字ならミスマッチ
            return false;
        }
        
        if (*pCurrentWildcard == 0)
        {
            return false;
        }
        
        if (*pCurrentWildcard == '*')
        {
            return *(pCurrentWildcard + 1) == 0
                || WildcardMatch(pCurrentTarget + 1, pCurrentWildcard)
                || WildcardMatch(pCurrentTarget, pCurrentWildcard + 1);
        }
        
        if (*pCurrentWildcard == '?' || (*pCurrentWildcard == *pCurrentTarget))
        {
            pCurrentWildcard++;
            pCurrentTarget++;
            continue;
        }
        return false;
    }
}

END_NAMESPACE_NEX
