﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/
#pragma once

#include <OnlineCore/src/Core/CoreDecl.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>
#include <OnlineCore/src/Core/SingleStepJob.h>

BEGIN_NAMESPACE_NEX

// 任意の返値を保持するクラス。
template<typename TReturnValue>
class HandleReturnValueAdapterBase : public RootObject
{
public:
    HandleReturnValueAdapterBase()
        : m_returnValue()
    {}
    virtual ~HandleReturnValueAdapterBase(){}
    const TReturnValue& GetReturnValue()
    {
        return m_returnValue;
    }
    virtual void operator()() = 0;
protected:
    TReturnValue m_returnValue;
};

// TReturnValue型の返値を返す関数オブジェクトを返値を返さない関数オブジェクトに変換し、
// 後でその返値を取得できるようにする関数アダプタ
template<typename TFunction, typename TReturnValue>
class HandleReturnValueAdapter : public HandleReturnValueAdapterBase<TReturnValue>
{
public:
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    HandleReturnValueAdapter(TFunction&& function)
        : m_function(std::forward<TFunction>(function))
    {}
#else
    HandleReturnValueAdapter(const TFunction& function)
        : m_function(function)
    {}
#endif
    virtual ~HandleReturnValueAdapter(){}
    virtual void operator()()
    {
        this->m_returnValue = m_function();
    }
private:
    TFunction m_function;
};

// 指定されたスレッドが完了するまで待つジョブ
class JobWaitForThreadFinished : public SingleStepJob
{
public:
    // 受け取ったpHandleReturnValueAdapterとpThreadの所有権は自分自身になる
    JobWaitForThreadFinished(HandleReturnValueAdapterBase<qResult>* pHandleReturnValueAdapter, ObjectThreadGeneric* pThread)
        : m_pHandleReturnValueAdapter(pHandleReturnValueAdapter)
        , m_pThread(pThread)
    {
    }
    virtual ~JobWaitForThreadFinished()
    {
        // スレッドが完了するまで待つ。
        // 関連づけられたCallContextがキャンセルされた場合はここでスレッドが完了するまで待つことになる。
        m_pThread->Wait();  
        qDelete m_pHandleReturnValueAdapter;
        qDelete m_pThread;
    };

    virtual qBool OnStep(qResult* result)
    {
        if (!m_pThread->Finished())
        {
            return false;
        }
        *result = m_pHandleReturnValueAdapter->GetReturnValue();
        return true;
    }
private:
    HandleReturnValueAdapterBase<qResult>* m_pHandleReturnValueAdapter;
    ObjectThreadGeneric* m_pThread;
};

/*!
    @brief 任意の処理をスレッド化し、それを非同期処理として実行するためのクラスです。
 */
class ThreadedJob : public RootObject
{
public:
    /*!
        @brief コンストラクタです。
     */
    ThreadedJob();

    /*!
        @brief デストラクタです。

     */
    virtual ~ThreadedJob(){}

    /*!
        @brief 任意の処理をスレッド化し、それを非同期処理として実行します。

        本関数を呼び出すと内部で一つスレッドを生成し、そのスレッドが threadedCallback を呼び出します。
        threadedCallback から処理が戻ると非同期処理の完了となり、pContext に結果が格納されます。

        threadedCallback には引数が空で返値の型が qResult もしくは qBool となる関数ポインタ、関数オブジェクト、ラムダ式を指定できます。
        threadedCallback が返却した qResult 値は pContext->GetOutcome() で取得可能です。
        threadedCallback から返却する qResult 値には NEX の各種非同期処理を呼び出した結果である CallContext::GetOutcome()
        から取得した qResult オブジェクトをそのまま使用したり、qResult(true) もしくは qResult(false) を使うことができます。
        アプリケーション定義のエラーを返却したい場合は QRESULT_ERROR(<qResultFacility::FirstUserFacility 以降の値>, <0 ～ 0xFFFF>) を使用できます。
        threadedCallback の返値の型が qBool の場合は、true を返却すると QSUCCESS(Core, Success)が、 false を返却すると QERROR(Core, Unknown) がそれぞれ
        pContext->GetOutcome() から返却されます。

        関数オブジェクトを渡す場合は値もしくは参照型で渡す方法とポインタ型で渡す方法の二種類があり、それぞれ挙動が異なります。
        値もしくは参照型で渡した場合は内部で関数オブジェクトがコピーもしくはムーブされ、コールバックはそのコピーもしくはムーブされたオブジェクトに対して行われます。
        そのため、本関数呼び出し後は指定したオブジェクトを即座に破棄する事ができます。
        ポインタ型で渡した場合は、コールバックはそのポインタが指すオブジェクトに対して行われます。
        そのため、コールバックが呼び出されるまでその関数オブジェクトを呼び出し側で保持しておく必要があります。

        threadedCallback に関数オブジェクトをポインタで渡した場合は deleteCallbackAfterCall を true にする事で
        threadedCallback の呼び出し後に自動的に threadedCallback を delete することができます。

        すでに実行した非同期処理が完了したかどうかにかかわらず、一つの ThreadedJob オブジェクトを使って何度でも本関数を呼び出すことが出来ます。
        非同期処理完了前に ThreadedJob オブジェクトを破棄しても問題ありません。

        本関数呼び出し後に CallContext::Cancel() を呼び出す場合は、事前に起動したスレッドから呼び出された threadedCallback の処理が完了している必要があります。
        threadedCallback の処理が完了していない場合は、完了するまで Scheduler::Dispatch() 内でブロックが発生します。

        NEX の関数を複数のスレッドで並列に呼び出す場合は NEX のスレッドモードがスレッドセーフである必要があることに注意してください。

        @param[in,out] pContext 非同期処理の実行状況を管理するためのオブジェクトです。
        @param[in] threadedCallback 生成したスレッドから呼び出されるコールバック。
        @param[in] deleteCallbackAfterCall threadedCallback の実行が完了した後自動的に threadedCallback を delete するかどうか。

        @return 非同期処理の開始に成功したかどうか。pContext が CallContext::CallInProgress の場合のみ開始に失敗し false を返します。
     */
    template<typename TCallback>
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    qBool Start(CallContext* pContext, TCallback&& threadedCallback, qBool deleteCallbackAfterCall=false)
#else
    qBool Start(CallContext* pContext, const TCallback& threadedCallback, qBool deleteCallbackAfterCall=false)
#endif
    {
        // CallContextを初期化できる状態かどうかをスレッドを作る前にチェックしておく
        if (!pContext->CanInitiate())
        {
            return false;
        }

        // スレッドの作成
        ObjectThreadGeneric* pThread = qNew ObjectThreadGeneric(m_threadName, m_threadPriority);
        pThread->SetCoreNo(m_threadCoreNo);

        QLOG(EventLog::Info, m_threadName << NEX_T(" will be invoked. Priority: ") << pThread->GetPriority());

        // threadedCallbackは返値がqResultの関数、一方ObjectThreadGeneric::Launch()は返値なしの関数を受け付けるので、
        // 適合させるためにHandleReturnValueAdapterでコールバックをラップする
        HandleReturnValueAdapter<TCallback, qResult>* pHandleReturnValueAdapter =
            qNew HandleReturnValueAdapter<TCallback, qResult>(NEX_STD_FORWARD(TCallback, threadedCallback));

        // pHandleReturnValueAdapterとpThreadの所有権はJobWaitForThreadFinishedに移る
        JobWaitForThreadFinished* jobWaitForThreadFinished = qNew JobWaitForThreadFinished(pHandleReturnValueAdapter, pThread);
        QVERIFY(jobWaitForThreadFinished->Start(pContext, true));   // 先にCanInitiate()でチェックしてるので必ず成功する

        // スレッドの起動
        // 関数オブジェクト(pHandleReturnValueAdapter)にはスレッドの処理結果を書き込んでもらう必要があるので、
        // 値渡ししてコピーするのではなくポインタで渡す。
        pThread->Launch(pHandleReturnValueAdapter, deleteCallbackAfterCall, m_threadStackSize);

        return true;
    }

    /*!
        @brief Start() で生成されるスレッドの優先度を設定します。

        @param[in] threadPriority スレッド優先度。
    */
    void SetThreadPriority(qUnsignedInt32 threadPriority)
    {
        m_threadPriority = threadPriority;
    }

    /*!
        @brief Start() で生成されるスレッドの優先度を取得します。

        @return スレッド優先度。
    */
    qUnsignedInt32 GetThreadPriority() const
    {
        return m_threadPriority;
    }

    /*!
        @brief Start() で生成されるスレッドが利用する CPU の コア番号を設定します。

        ObjectThreadRoot::DEFAULT_CORE_NO が、デフォルト値です。
        CTR では、デフォルト値を使用した場合は CORE_NO_USE_PROCESS_VALUE が設定されます。

        SDK 3.0 以降の CTR では、1 を指定することによりシステムコアを選択できます。
        システムコアの利用の際には、nn::os::CTR::SetApplicationCpuTimeLimit() で、CPU時間を割り当てる必要があります。
        システムコアの利用は、動作保証外です。
        

        

        @param[in] threadCoreNo CPU コア番号
     */
    void SetThreadCoreNo(qInt32 threadCoreNo)
    {
        m_threadCoreNo = threadCoreNo;
    }

    /*!
        @brief Start() で生成されるスレッドが利用する CPU の コア番号を取得します。

        @return CPU コア番号
     */
    qInt32 GetThreadCoreNo() const
    {
        return m_threadCoreNo;
    }

    /*!
        @brief Start() で生成されるスレッドのスタックサイズを設定します。

        @param[in] threadStackSize スレッドのスタックサイズ。
    */
    void SetThreadStackSize(size_t threadStackSize)
    {
        m_threadStackSize = threadStackSize;
    }

    /*!
        @brief Start() で生成されるスレッドのスタックサイズを取得します。

        @return スレッドのスタックサイズ。
    */
    size_t GetThreadStackSize() const
    {
        return m_threadStackSize;
    }

    /*!
        @brief Start() で生成されるスレッドの名前を設定します。

        @param[in] threadName スレッド名。
    */
    void SetThreadName(String threadName)
    {
        m_threadName = threadName;
    }

    /*!
        @brief Start() で生成されるスレッドの名前を取得します。

        @return スレッド名。
    */
    String GetThreadName() const
    {
        return m_threadName;
    }
private:
    size_t m_threadStackSize;
    qUnsignedInt32 m_threadPriority;
    qInt32 m_threadCoreNo;
    String m_threadName;
};

END_NAMESPACE_NEX

