﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/Message.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/qQueue.h>
#include <OnlineCore/src/Platform/Core/Variant.h>
#include <OnlineCore/src/Core/Buffer.h>
#include <OnlineCore/src/Core/PacketBuffer.h>
#include <OnlineCore/src/Core/qBuffer.h>
#include <OnlineCore/src/DDLTypes/TypeCGMacros.h>
#include <OnlineCore/src/DDLTypes/DDLType.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>

////////////////////////////////////////////////////////////////////////////////
// This file defines the built-in types of the DDL compiler. Note that those
// types must also be declared in the following files:
//
// - \src\ddl\CompilerDriver\CompilerDriver.cpp (DeclareBuiltInTypes method)
////////////////////////////////////////////////////////////////////////////////

BEGIN_NAMESPACE_NEX

//==============================================================================
// DECLARE_TYPE_DDLTYPE macro defines a C++ DDLType class for the specified
// DDLTYPENAME DDL custom variable type (native C++ types). The class allows to
// parse a variable of that type to and from (Add/Extract) a NetZ message.
//==============================================================================
#define DECLARE_TYPE_DDLTYPE(DDLTYPENAME, CPPTYPENAME, FORMAT)                  \
class OSDK_API DDLTYPE(DDLTYPENAME) : public DDLType {                          \
public:                                                                         \
    typedef CPPTYPENAME CPPType;                                                \
    static void Add(Message* pMessageToSend, const CPPType& oValue) {           \
        (*pMessageToSend) << oValue;                                            \
    }                                                                           \
    static void Extract(Message* pReceivedMessage, CPPType* pValue) {           \
        (*pReceivedMessage) >> *pValue;                                         \
    }                                                                           \
    static qBool FormatVariableValue(const void* pVariable, String* pstrValue) { \
        SYSTEMCHECK(pVariable != NULL);                                         \
        CPPTYPENAME *pVar = (CPPTYPENAME*)pVariable;                            \
        pstrValue->Format(FORMAT, *pVar);                                       \
        return true;                                                            \
    }                                                                           \
};

DECLARE_TYPE_DDLTYPE(int64,qInt64,NEX_T("%d"))
DECLARE_TYPE_DDLTYPE(uint64,qUnsignedInt64,NEX_T("%d"))

DECLARE_TYPE_DDLTYPE(int32,qInt32,NEX_T("%d"))
DECLARE_TYPE_DDLTYPE(uint32,qUnsignedInt32,NEX_T("%d"))
DECLARE_TYPE_DDLTYPE(int16,qInt16,NEX_T("%d"))
DECLARE_TYPE_DDLTYPE(int8,qInt8,NEX_T("%d"))
DECLARE_TYPE_DDLTYPE(uint16,qUnsignedInt16,NEX_T("%d"))
DECLARE_TYPE_DDLTYPE(byte,qByte,NEX_T("%d"))
DECLARE_TYPE_DDLTYPE(bool,qBool,NEX_T("%d"))
DECLARE_TYPE_DDLTYPE(double,qDouble,NEX_T("%f"))
DECLARE_TYPE_DDLTYPE(float,qFloat,NEX_T("%f"))

DECLARE_TYPE_DDLTYPE(cstr,qChar,NEX_T("N/A"))
DECLARE_TYPE_DDLTYPE(char,qChar,NEX_T("N/A"))
DECLARE_TYPE_DDLTYPE(char8,qChar8,NEX_T("N/A"))
DECLARE_TYPE_DDLTYPE(char16,qChar16,NEX_T("N/A"))

//==============================================================================
// The DDLTYPE(real) class allows to parse a variable of the "real" type to and
// from (Add/Extract) a NetZ message. The DDL "real" type is a special type
// because it always passes C++ floats on the network.
//==============================================================================
class OSDK_API DDLTYPE(real): public DDLType
{
public:
    typedef qReal CPPType;
    static void Add(Message* pMessageToSend, const qReal& oValue)
    {
        (*pMessageToSend) << (qFloat)oValue;
    }
    static void Extract(Message* pReceivedMessage, qReal* pValue)
    {
        qFloat fValue;
        (*pReceivedMessage) >> fValue;
        *pValue = static_cast< qReal >( fValue );
    }
    static qBool FormatVariableValue(const void *pVariable, String* pstrValue)
    {
        SYSTEMCHECK(pVariable != NULL);
        const qReal rValue = *(qReal*)pVariable;
        pstrValue->Format(NEX_T("%f"), FormatCast(rValue));
        return true;
    }
};

//==============================================================================
// The DDLTYPE(datetime) class allows to parse a variable of the "datetime" type
// to and from (Add/Extract) a NetZ message. The DDL "datetime" type is a special
// type because it always passes C++ uint64 on the network.
//==============================================================================
class OSDK_API DDLTYPE(datetime): public DDLType
{
public:
    typedef DateTime CPPType;
    static void Add(Message* pMessageToSend, const DateTime& oValue)
    {
        (*pMessageToSend) << static_cast<qUnsignedInt64>(oValue);
    }
    static void Extract(Message* pReceivedMessage, DateTime* pValue)
    {
        qUnsignedInt64 ui64Value;
        (*pReceivedMessage) >> ui64Value;
        *pValue = DateTime( ui64Value );
    }
    static qBool FormatVariableValue(const void *pVariable, String* pstrValue)
    {
        SYSTEMCHECK(pVariable != NULL);
        const qUnsignedInt64 ui64Value = *(qUnsignedInt64*)pVariable;
        pstrValue->Format(NEX_T("%d"), ui64Value);
        return true;
    }
};


//==============================================================================
// The DDLTYPE(qresult) class allows to parse a variable of the "qresult" type
// to and from (Add/Extract) a NetZ message. The DDL "qresult" type is a special
// type because of the way it implements the Add and Extract methods.
//==============================================================================
class OSDK_API DDLTYPE(qresult): public DDLType
{
public:
    typedef qResult CPPType;
    static void Add(Message* pMessageToSend, const qResult& oValue);
    static void Extract(Message* pReceivedMessage, qResult* pValue);
    static qBool FormatVariableValue(const void *pVariable, String* pstrValue);

};

//==============================================================================
// The DDLTYPE(variant) class allows to parse a variable of the "variant" type
// to and from (Add/Extract) a NetZ message. The DDL "variant" type is a special
// type because of the way it implements the Add and Extract methods.
//==============================================================================
class OSDK_API DDLTYPE(variant): public DDLType
{
public:
    typedef Variant CPPType;
    static void Add(Message* pMessageToSend, const Variant& oValue);
    static qBool Extract(Message* pReceivedMessage, Variant* pValue);

};

//==============================================================================
// The DDLTYPE(qBuffer) class allows to parse a variable of the "qBuffer" type
// to and from (Add/Extract) a NetZ message. The DDL "qBuffer" type is a special
// type because of the way it implements the Add and Extract methods.
//==============================================================================
class OSDK_API DDLTYPE(qBuffer): public DDLType
{
public:
    typedef qBuffer CPPType;
    static void Add(Message* pMessageToSend, const qBuffer& oValue);
    static qBool Extract(Message* pReceivedMessage, qBuffer* pValue);
};

//==============================================================================
// The DDLTYPE(string) class allows to parse a variable of the "string" type
// to and from (Add/Extract) a NetZ message. The DDL "string" type is a special
// type because of the way it implements the Add and Extract methods.
//==============================================================================
class OSDK_API DDLTYPE(string): public DDLType
{
public:
    typedef String CPPType;

    static void Add(Message *pMessageToSend, const String &oValue);
    static qBool Extract(Message *pReceivedMessage, String *pValue);
    static qBool FormatVariableValue(const void *pVariable, String* pstrValue)
    {
        SYSTEMCHECK(pVariable != NULL);
        const qChar* szValue=((String*)pVariable)->CStr();
        pstrValue->Format(NEX_T("%s"), szValue!=NULL? szValue: NEX_T(""));
        return true;
    }
};

//==============================================================================
// The DDLTYPE(buffer) class allows to parse a variable of the "buffer" type
// to and from (Add/Extract) a NetZ message. The DDL "buffer" type is a special
// type because of the way it implements the Add and Extract methods.
//==============================================================================
class OSDK_API DDLTYPE(buffer): public DDLType
{
public:
    typedef Buffer CPPType;

    static void Add(Message *pMessageToSend, const Buffer &oValue)
    {
        // 32bitと64bitでバイナリフォーマットを同一にするためsize_tをqUnsignedInt32にキャスト
        *pMessageToSend << static_cast<qUnsignedInt32>(oValue.GetContentSize());
        pMessageToSend->Append(oValue.GetContentPtr(), oValue.GetContentSize());
    }
    static qBool Extract(Message *pReceivedMessage, Buffer *pValue)
    {
        // 32bitと64bitでバイナリフォーマットを同一にするためサイズはsize_tではなくqUnsignedInt32で読み取る
        qUnsignedInt32 uiSize = 0;
        if(!pReceivedMessage->Extract(uiSize))
        {
            return false;
        }

        if (! pReceivedMessage->ValidateBufferLimit(uiSize))
        {
            return false;
        }

        qByte *pMsgBufPtr =
            pReceivedMessage->GetBuffer()->GetContentPtr()+
            pReceivedMessage->GetCurrentOffset();

        pValue->AppendData(pMsgBufPtr,uiSize);
        pReceivedMessage->SetOffset(pReceivedMessage->GetCurrentOffset()+uiSize);
        return true;
    }
    static qBool FormatVariableValue(const void *pVariable, String* pstrValue)
    {
        SYSTEMCHECK(pVariable != NULL);
        Buffer *pbufVar = (Buffer*)pVariable;
        pstrValue->Format(NEX_T("%d bytes"),pbufVar->GetContentSize());
        return true;
    }
};

//==============================================================================
// The DDLTYPE(buffertail) class allows to parse a variable of the "buffertail"
// type to and from (Add/Extract) a NetZ message. The DDL "buffertail" type is a
// special type because of the way it implements the Add and Extract methods.
//==============================================================================
class OSDK_API DDLTYPE(buffertail): public DDLType
{
public:
    typedef Buffer CPPType;

    static void Add(Message *pMessageToSend, const Buffer &oValue);
    static void Extract(Message *pReceivedMessage, Buffer *pValue);
    static qBool FormatVariableValue(const void *pVariable, String* pstrValue);
};

//==============================================================================
// The following template class is used to integrate the qList type.
//==============================================================================
template<class TDDL_Type>
class DDLTYPE(qlist) : public DDLType
{
public:
    typedef qList<typename TDDL_Type::CPPType> CPPType;

    static void Add(Message* pMessageToSend, const CPPType &stdList)
    {
        // 互換性を保つため、送信するサイズは4byteに固定
        qUnsignedInt32 size = static_cast<qUnsignedInt32>(stdList.size());
        (*pMessageToSend) << size;

        typename CPPType::const_iterator i;
        typename CPPType::const_iterator end = stdList.end();
        for( i = stdList.begin(); i != end; i++ )
        {
            TDDL_Type::Add( pMessageToSend, *i );
        }
    };

    static void Extract(Message *pReceivedMessage, CPPType *pstdList )
    {
        pstdList->clear();
        qUnsignedInt32 uiSize;
        if(!pReceivedMessage->Extract(uiSize))
        {
            return;
        }
        qUnsignedInt32 i;
        for(i = 0; i < uiSize && !pReceivedMessage->ErrorHasOccured(); i++)
        {
            typename TDDL_Type::CPPType v;
            TDDL_Type::Extract( pReceivedMessage, &v );
            if (pReceivedMessage->ErrorHasOccured())
            {
                pstdList->clear();
                return;
            }
            pstdList->push_back( v );
        }
        return;
    };
};

//==============================================================================
// The following template class is used to integrate the qVector type.
//==============================================================================
template<class TDDL_Type>
class DDLTYPE(qvector) : public DDLType
{
public:
    typedef qVector<typename TDDL_Type::CPPType> CPPType;

    static void Add(Message* pMessageToSend, const CPPType& stdVector)
    {
        // 互換性を保つため、送信するサイズは4byteに固定
        qUnsignedInt32 size = static_cast<qUnsignedInt32>(stdVector.size());
        (*pMessageToSend) << size;

        typename CPPType::const_iterator i;
        typename CPPType::const_iterator end = stdVector.end();
        for( i = stdVector.begin(); i != end; i++ )
        {
            TDDL_Type::Add( pMessageToSend, *i );
        }
    };

    static void Extract(Message* pReceivedMessage, CPPType* pStdVector)
    {
        pStdVector->clear();
        qUnsignedInt32 uiSize;
        if(!pReceivedMessage->Extract(uiSize))
        {
            return;
        }
        pStdVector->reserve(uiSize);
        qUnsignedInt32 i;
        for(i = 0; i < uiSize && !pReceivedMessage->ErrorHasOccured(); i++)
        {
            typename TDDL_Type::CPPType v;
            TDDL_Type::Extract( pReceivedMessage, &v );
            if (pReceivedMessage->ErrorHasOccured())
            {
                pStdVector->clear();
                return;
            }
            pStdVector->push_back(v);
        }
        return;
    };
};

//==============================================================================
// The following template class is used to integrate the qQueue type.
//==============================================================================
template<class TDDL_Type>
class DDLTYPE(qqueue) : public DDLType
{
public:
    typedef qQueue<typename TDDL_Type::CPPType> CPPType;

    static void Add(Message* pMessageToSend, const CPPType& stdVector)
    {
        // 32bitと64bitで同一のバイナリフォーマットにするためにsize_tからqUnsignedInt32にキャスト
        qUnsignedInt32 size = static_cast<qUnsignedInt32>(stdVector.size());
        (*pMessageToSend) << size;

        typename CPPType::const_iterator i;
        typename CPPType::const_iterator end = stdVector.end();
        for( i = stdVector.begin(); i != end; i++ )
        {
            TDDL_Type::Add( pMessageToSend, *i );
        }
    };

    static void Extract(Message* pReceivedMessage, CPPType* pStdVector)
    {

        qUnsignedInt32 uiSize;
        if(!pReceivedMessage->Extract(uiSize))
        {
            return;
        }
        pStdVector->resize(uiSize); //queueは、reserveを使えない
        pStdVector->clear();
        qUnsignedInt32 i;
        for(i = 0; i < uiSize && !pReceivedMessage->ErrorHasOccured(); i++)
        {
            typename TDDL_Type::CPPType v;
            TDDL_Type::Extract( pReceivedMessage, &v );
            if (pReceivedMessage->ErrorHasOccured())
            {
                pStdVector->clear();
                return;
            }
            pStdVector->push_back(v);
        }
        return;
    };
};

//==============================================================================
// The DDLTYPE(stationurl) class allows to parse a variable of the "stationurl"
// type to and from (Add/Extract) a NetZ message. The DDL "stationurl" type is a
// special type because of the way it implements the Add and Extract methods.
//==============================================================================
class DDLTYPE(stationurl): public DDLType
{
public:
    typedef StationURL CPPType;
    static void Add(Message* pMessageToSend, const StationURL& oValue);
    static qBool Extract(Message* pReceivedMessage, StationURL* pValue);
};

END_NAMESPACE_NEX

