﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include "HttpConnection.h"
#include "HttpEventListener.h"

BEGIN_NAMESPACE_NEX

class CallContext;

/*!
    @brief      HTTP リクエストを行うためのクライアントです。
*/
class HttpClient : public RootObject, public NonCopyable
{
public:
    HttpClient();
    virtual ~HttpClient();

    /*!
        @brief      HTTP リクエストを開始します

        @param[in]  pCC 呼び出し情報。
        @param[in]  url リクエスト先URL。
        @param[in]  method リクエストメソッド。
        @param[in]  eventListener データの送受信を行うイベントリスナーオブジェクト。
        @param[in]  relayBufferSize HttpEventListener::PostChunkedBuffer() 及び HttpEventListener::ProcessResponse() において一度にバッファに読み書きする最大サイズ。
                                    これらの関数に渡される最大バッファサイズを指定します。
    */
    qResult Start(CallContext* pCC, const String& url, HttpConnection::Method method, HttpEventListener* eventListener,
                  size_t relayBufferSize=HttpConnection::DEFAULT_RELAY_BUFFER_SIZE);

    /*!
        @brief      HTTP リクエストを開始します

            各種設定を省いた簡易版です。
            buffer と statusCode は RMC が成功したとき(pCC が ProtocolCallContext::GetState()==CallContext::CallSuccess の状態になったとき)のみその内容を利用してください。

        @param[in]  pCC 呼び出し情報。
        @param[in]  url リクエスト先URL。
        @param[in]  method リクエストメソッド。
        @param[in]  maxDownloadSize 最大ダウンロードサイズ。このサイズを超えてボディをダウンロードしようとした場合キャンセルされます。
        @param[out] buffer ダウンロードしたデータを格納するバッファ。
        @param[out] statusCode HTTP ステータスコード。
    */
    qResult Start(CallContext* pCC, const String& url, HttpConnection::Method method, size_t maxDownloadSize,
                  qVector<qUnsignedInt8>* buffer, qInt* statusCode);

    /*!
        @brief      HTTP リクエストを開始します

            HTTPリクエストヘッダとリクエストボディの送信に対応しています。
            buffer と statusCode は RMC が成功したとき(pCC が ProtocolCallContext::GetState()==CallContext::CallSuccess の状態になったとき)のみその内容を利用してください。

        @param[in]  pCC 呼び出し情報。
        @param[in]  url リクエスト先URL。
        @param[in]  method リクエストメソッド。
        @param[in]  httpThreadPriority HTTPスレッドの優先度。
        @param[in]  postBuffer 送信するリクエストボディの内容。
        @param[in]  requestHeader 送信するリクエストヘッダの内容。
        @param[in]  maxDownloadSize 最大ダウンロードサイズ。このサイズを超えてボディをダウンロードしようとした場合キャンセルされます。
        @param[out] buffer ダウンロードしたデータを格納するバッファ。
        @param[out] statusCode HTTP ステータスコード。
    */
    qResult Start(CallContext* pCC, const String& url, HttpConnection::Method method, qUnsignedInt32 httpThreadPriority,
                  const qVector<qByte>& postBuffer, const qMap<String, String>& requestHeader, size_t maxDownloadSize,
                  qVector<qUnsignedInt8>* buffer, qInt* statusCode);
};

END_NAMESPACE_NEX

