﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>

BEGIN_NAMESPACE_NEX

class HttpSessionEventListener;

class HttpConnection : public RootObject, public NonCopyable
{
public:
    enum Method
    {
        GET = 0,
        HEAD = 1,
        POST = 2,
        PUT = 3
    };

    static HttpConnection* GetHttpConnection();

    static void ReleaseHttpConnection(HttpConnection* pConnection);

    // HttpSessionEventListenerの登録を行います。NULLを指定すると解除を行います。
    static void RegisterHttpSessionEventListener(HttpSessionEventListener* eventListener)
    {
        s_pHttpSessionEventListener = eventListener;
    }

    explicit HttpConnection()
        : m_Method(POST)
        , m_ThreadPriority(DEFAULT_THREAD_PRIORITY)
        , m_HttpSendSocketBufferSize(0)
        , m_HttpRecvSocketBufferSize(0)
        , m_UseChunkedPost(false)
        , m_UseOldNintendoRootCaCert(true)
        , m_UseNintendoClientCert(true)
    {
        memset(m_BoundaryString, 0, MAX_BOUNDARY_SIZE);
    }

    virtual ~HttpConnection() {};

    // エラーコードはプラットフォーム依存なので、特定の理由がない限りは
    // Success, Failぐらいが分かるレベルのResultを返す
    // 返すエラーコードがプラットフォームで違うということのないように！

    virtual qResult Initialize(const qChar8 *url, Method method = POST) = 0;

    // リクエストヘッダを設定する
    virtual qResult AddHeaderField(const qChar8* pLabel, const qChar8* pValue) = 0;

    // POST データを x-www-form-urlencoded 形式でセットする。 ConnectAsync 前に呼ぶ必要がある。
    virtual qResult AddPostData(const qChar8* pLabel, const qChar8* pValue) = 0;

    // 遅延 POST モードを使用する。 ConnectAsync 後に WritePostChunk で送信データをセットする。
    virtual void UseChunkedPost(size_t contentLength) = 0;

    // 遅延 POST モードか。
    virtual qBool IsUseChunkedPost() const;

    // 遅延 POST モード時に全てのデータを登録した事を知らせる。
    virtual void CompleteChunkedPost() = 0;

    // 遅延 POST モード時に全てのデータを登録したか。
    virtual qBool IsCompleteChunkedPost() const = 0;

    // HTTP リクエストを開始する。
    virtual qResult ConnectAsync(void) = 0;

    // HTTP リクエストをキャンセルする。
    virtual qResult Cancel(void) = 0;

    // ステータスコードを返す
    // ヘッダ受信前にステータスコードを取ろうとすると false を返す。
    // IsReceivingHeaderCompleted 関数でヘッダの受信を確認した後はエラーがない限りステータスコードが取得できる
    virtual qBool GetStatusCode(qInt32* pStatusCode) const = 0;

    // 受信データを取得する
    virtual qResult Read(qUnsignedInt8* pBodyBuf, size_t* bodySize, size_t bufLen) = 0;

    // 遅延 POST モード時にデータを登録する
    virtual qResult WritePostChunk(qUnsignedInt8* pPostBuf, size_t size) = 0;

    // WritePostChunk で登録できるデータサイズを取得する
    virtual size_t GetWritableSize() const = 0;

    // 未使用
    virtual qResult GetError() const = 0;

    // ヘッダ部を全て受信したか
    virtual qResult IsReceivingHeaderCompleted() const = 0;

    // HTTP 通信が完了したか
    virtual qBool IsFinished() const = 0;

    // POST データを multipart/form-data 形式でセットする。 ConnectAsync 前に呼ぶ必要がある。
    virtual qResult AddFormData(const qChar8* pLabel, const void* pValue, size_t valueLen) = 0;

    // レスポンスヘッダを取得する
    virtual qBool GetResponseHeader(const qChar8* pLabel, String* pValue) const = 0;

    // ヘッダの受信が完了し、もうヘッダを読まないことを伝える。(CTRでFinalize()時にブロックしなくなる)
    virtual void NoMoreResponseHeader(){};

    // HTTP スレッドの優先度を指定する。ConnectAsync関数呼び出し前にセットしてください。 CTR でのみ有効。
    void SetThreadPriority(qUnsignedInt32 priority)
    {
        m_ThreadPriority = priority;
    };

    // HTTP スレッドの優先度を取得する
    qUnsignedInt32 GetThreadPriority()
    {
        return m_ThreadPriority;
    };
    /*
     * @brief HTTP 通信で使用するソケットの送信バッファサイズを指定します。
     *
     *  主に DataStore ストレージへのアクセスで使われます。
     *  0 の場合は、 システム標準値が使われます。
     *  この値は、NXでのみ使用されます。Windows では使用されず自動設定されます。
     *
     */
    void SetHttpSendSocketBufferSize(size_t size)
    {
        m_HttpSendSocketBufferSize = size;
    }

    /*
     * @brief HTTP 通信で使用するソケットの送信バッファサイズを取得します。
     *
     *  主に DataStore ストレージへのアクセスで使われます。
     *  0 の場合は、 システム標準値が使われます。
     *
     */
    size_t GetHttpSendSocketBufferSize() const
    {
        return m_HttpSendSocketBufferSize;
    }

    /*
     * @brief HTTP 通信で使用するソケットの受信バッファサイズを指定します。
     *
     *  主に DataStore ストレージへのアクセスで使われます。
     *  0 の場合は、 システム標準値が使われます。
     *  この値は、NXでのみ使用されます。Windows では使用されず自動設定されます。
     *
     */
    void SetHttpRecvSocketBufferSize(size_t size)
    {
        m_HttpRecvSocketBufferSize = size;
    }

    /*
     * @brief HTTP 通信で使用するソケットの受信バッファサイズを取得します。
     *
     *  主に DataStore ストレージへのアクセスで使われます。
     *  0 の場合は、 システム標準値が使われます。
     *
     */
    size_t GetHttpRecvSocketBufferSize() const
    {
        return m_HttpRecvSocketBufferSize;
    }

    // 使用するRootCA証明書をセットします。 ConnectAsync 前に呼ぶ必要がある。
    virtual qBool AddRootCa(const qByte* pCertData, size_t size) = 0;

    // HTTP通信に使用されるバッファのサイズを設定します。 ConnectAsync 前に呼ぶ必要がある。 CTR でのみ有効。
    virtual void SetTransferBufferSize(size_t size) = 0;

    // Boundary文字列を取得する。boundaryはNULL終端される。NULL終端を含んだ最大サイズはMAX_BOUNDARY_SIZE。
    // 返値はHttpConnectionオブジェクトが破棄されるまで有効なBoundary文字列へのポインタ。
    const qChar8* GetBoundaryString() const;

    // 古い任天堂ルートCA証明書を使用するかどうかを設定する。初期値は使用する(true)
    void SetUseOldNintendoRootCaCert(qBool use)
    {
        m_UseOldNintendoRootCaCert = use;
    }

    // 任天堂のクライアント証明書を使用するかどうかを設定する。初期値は使用する(true)
    void SetUseNintendoClientCert(qBool use)
    {
        m_UseNintendoClientCert = use;
    }

    // HTTP通信で使用するデフォルトのスレッド優先度です。 CTR/Cafe でのみ有効。
    static const qUnsignedInt32 DEFAULT_THREAD_PRIORITY = 16;

    // デフォルトの送受信バッファサイズです。CURL_MAX_WRITE_SIZE(16KB)以上にする必要がある。
    static const qUnsignedInt32 DEFAULT_TRANSFER_BUFFER_SIZE = 16 * 1024;

    // デフォルトのリレー用バッファサイズです。
    static const qUnsignedInt32 DEFAULT_RELAY_BUFFER_SIZE = 16 * 1024;

    // NULL終端を含んだ最大のバウンダリサイズです。
    static const qUnsignedInt32 MAX_BOUNDARY_SIZE = 41;
private:
    static qInt GetConnectionCount();
protected:
    Method m_Method;
    qUnsignedInt32 m_ThreadPriority;
    size_t m_HttpSendSocketBufferSize;  // HTTP通信送信用ソケットバッファサイズ
    size_t m_HttpRecvSocketBufferSize;  // HTTP通信受信用ソケットバッファサイズ
    static const qUnsignedInt32 MAX_PROXY_STRING_SIZE = 256;
    volatile qBool m_UseChunkedPost;
    qBool m_UseOldNintendoRootCaCert;   // 古い任天堂ルートCA証明書を除外するか
    qBool m_UseNintendoClientCert;      // 任天堂のクライアント証明書を除外するか
private:
    mutable qChar8 m_BoundaryString[MAX_BOUNDARY_SIZE];

private:
    static qInt s_ConnectionCount;
    static HttpSessionEventListener* s_pHttpSessionEventListener;
};

END_NAMESPACE_NEX

