﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/CriticalSection.h>

BEGIN_NAMESPACE_NEX

/*!
@internal
@brief    NEXの内部状態にかかわらず必ず排他制御を行うクリティカルセクションクラスです。

NEXの内部状態にかかわらず必ず排他制御を行う@ref CriticalSection を継承した強制的なクリティカルセクションです。
通常の@ref CriticalSection は@ref Core::SelectThreadSafeExecution にfalseを指定すると排他制御を行いませんが、
ForcedCriticalSection はどんなNexの内部状態の場合でも必ず排他制御を行います。
誤って CriticalSection を永久的にロックすることを避けるため、 @ref ScopedCS クラスを使用することができます。
詳細については、このクラスを参照してください。

@see     ScopedCS
*/
class ForcedCriticalSection: public CriticalSection
{
public:

    ForcedCriticalSection(qUnsignedInt32 uiLockGroup=LockMasks::Default, qBool reentrantable=true)
        : CriticalSection(uiLockGroup, reentrantable)
    {
    }

    virtual void Enter()
    {
        // MutexPrimitive::Enter()でs_bNoOpを参照して無効になるのを防ぐためForceEnter()する
        ForceEnter();
    }

    virtual void Leave()
    {
        ForceLeave();
    }

    // ForcedCriticalSectionならtrueを返す。
    virtual qBool IsForce() const { return true; }

    // 再入可能なクリティカルセクションかどうかを返す。
    // 以下のように返される。
    // CriticalSection:                       true
    // ForcedCriticalSection:                 true
    // NonReentrantableForcedCriticalSection: false
    virtual qBool IsReentrantable() const { return true; }
};

/*
@brief    NEXの内部状態にかかわらず必ず排他制御を行うクリティカルセクションクラスです。再帰的にロックすることはできません。

ForcedCriticalSection と同様の動作をしますが再帰的にロックすることはできません。
Condition::WaitForEvent() 向けのクラスです。

@see     ScopedCS
*/
class NonReentrantableForcedCriticalSection: public ForcedCriticalSection
{
public:
    NonReentrantableForcedCriticalSection(qUnsignedInt32 uiLockGroup=LockMasks::Default)
        : ForcedCriticalSection(uiLockGroup, false)
    {
    }

    // 再入可能なクリティカルセクションかどうかを返す。
    // 以下のように返される。
    // CriticalSection:                       true
    // ForcedCriticalSection:                 true
    // NonReentrantableForcedCriticalSection: false
    virtual qBool IsReentrantable() const { return false; }
};

END_NAMESPACE_NEX
