﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>
#include <stdarg.h>

#include <OnlineCore/src/Platform/Core/StringStream.h>

BEGIN_NAMESPACE_NEX

class LogDevice;
class OutputFormat;
class LogFilter;

//group=Data Logging

/*
Description: The root class for the logging system.
Remarks: This is the root class for the logging system and should not be directly instantiated.
Instead, you need to instantiate either the EventLog or TraceLog classes.
*/

/*!
 @brief ログを扱う EventLog、TraceLog の基底クラスです。

 @see   EventLog, TraceLog
 */
class Log : public RootObject
{
public:
    //Group=Methods

    Log();

    virtual ~Log();

    /*
    Description: Sets the LogDevice attached to this Log object.
    Remarks: This method sets the type of LogDevice that will be used with this Log.
    Parameters:
    pLogDevice: The LogDevice that will be attached to this Log object. MUST be dynamically allocated with qNew.
    bDeleteCurrentDevice: If true, the current log device will be deleted. If false, the caller will be responsible for the ownership of the current log device.
    See Also: GetLogDevice
    */

    /*!
     * @brief LogDeviceを設定します。
     *
     * @param[in] pLogDevice LogDeviceを設定します。qNewで動的確保されている必要があります。
     * @param[in] bDeleteCurrentDevice true 現在のLogDeviceを自動削除します。false 現在のLogDeviceを削除しません。
     *
     * @see GetLogDevice()
     */
    void SetLogDevice( LogDevice *pLogDevice, qBool bDeleteCurrentDevice = true );

    // デフォルトのログデバイスに戻す。
    void SetDefaultLogDevice(qBool bDeleteCurrentDevice = true);

    /*
    Description: Sets the OutputFormat for this Log object.
    Remarks: This method sets the type of OutputFormat that will be used with this Log.
    Parameters:
    pOutputFormat: The OutputFormat that will be attached to this Log object. MUST be dynamically allocated with qNew.

    See Also: GetOutputFormat
    */

    /*!
     * @brief LogDeviceを設定します。
     *
     * @param[in] pOutputFormat OutputFormatを設定します。qNewで動的確保されている必要があります。。
     *
     * @see GetLogDevice()
     */
    void SetOutputFormat( OutputFormat *pOutputFormat  );

    // デフォルトのスタティックに用意してあるOutputFormatに戻す。
    void SetDefaultOutputFormat();

    /*
    Description: Returns the LogDevice attached to this Log object.
    Returns: A pointer to the LogDevice object.
    See Also: SetLogDevice
    */

    /*!
     * @brief LogDeviceを取得します。
     *
     * @return 設定されているLogDeviceのポインタを返します。
     */
    LogDevice *GetLogDevice();

    // {secret}
    // Note.  As opposed to the LogDevice, the Log is not responsible
    // to release the LogFilter - the creator is.
    void SetLogFilter(LogFilter *pLogFilter);
    // {secret}
    LogFilter *GetLogFilter();

    /*
    Description: Returns the OutputFormat attached to this Log object.
    Returns: A pointer to the OutputFormat object.
    See Also: SetOutputFormat
    */

    /*!
     @brief    OutputFormatへのポインタを取得します。

     @return    @ref OutputFormat へのポインタ
     */
    OutputFormat *GetOutputFormat();

    // {secret}
    virtual void Output( const qChar *szFormat, ... );

    // {secret}
    virtual void OutputString( const qChar *szString, ... );

    // {secret}
    virtual void AddCustomPrefix( qChar *, qInt, void *) {}

protected:
    friend class EventLog;

    void DeleteLogDevice(LogDevice* logDevice);
    void DeleteOutputFormat(OutputFormat* outputFormat);
    void OutputImpl(const qChar *szFormat, va_list vlArgPtr, void *pUserData );

private:
    LogDevice *m_pLogDevice;

    OutputFormat *m_pOutputFormat;
    LogFilter *m_pLogFilter;

    // Cannot copy
    Log(const Log &);
    Log &operator=(const Log &);
    void PrepareOutput(qChar *outputBuffer, qUnsignedInt32 logSize,void *pUserData);

};

END_NAMESPACE_NEX

