﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

// See jwiki:MemoryAllocation for an overview of memory allocation in the system.

#ifdef __cplusplus  // see else below...

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>

// The amount of 0 padding to add to each block
#ifndef HEAP_TRAILER_PADDING_SIZE
// {secret}
// If not specified, defaults to the same size as the signature contained in the
// trailer since anything less would end up taking that much in order for the
// Trailer to be aligned.
#define HEAP_TRAILER_PADDING_SIZE (sizeof(qUnsignedInt32))
#endif //HEAP_TRAILER_PADDING_SIZE

#ifdef NEX_GLOBALNEW_DETECTION_ENABLED
// Global operator.

// We define the standard operator new/delete.  There is a total of 6 versions
// of each - the version with exceptions or not, array and not, plus
// the placement version.

// We do not redefine the placement new/delete.  This is typically
// defined inline by the compiler includes, and we do not need to
// redefine it (see also bug:4675 for a related question).

// {secret}
#ifdef CTR_TARGET
void *operator new(size_t) throw(std::bad_alloc);
#else
void *operator new(size_t);
#endif
// {secret}
#ifdef CTR_TARGET
void *operator new[](size_t size) throw(std::bad_alloc);
#else
void *operator new[](size_t size);
#endif
// {secret}
void* operator new(std::size_t, const std::nothrow_t&) throw();
// {secret}
void* operator new[](std::size_t, const std::nothrow_t&) throw();

// {secret}
void operator delete(void* uptr) throw();
// {secret}
void operator delete[](void *uptr) throw();
// n2605: warning 831-D: support for placement delete is disabled. (see ARM\include\new.)
// {secret}
//void operator delete(void*, const std::nothrow_t&) throw();
// {secret}
//void operator delete[](void*, const std::nothrow_t&) throw();

// We now redefine our own versions of the operator new.  This is the
// one that gets called when qNew is used, in debug mode.  qNew adds
// extra debug info.

//{secret}
void *operator new(size_t size, const NAMESPACE_NEX qChar *szFile, NAMESPACE_NEX qUnsignedInt32 uiLine) throw();
//{secret}
void *operator new[](size_t size, const NAMESPACE_NEX qChar *szFile, NAMESPACE_NEX qUnsignedInt32 uiLine) throw();




#endif //NEX_GLOBALNEW_DETECTION_ENABLED

BEGIN_NAMESPACE_NEX

#ifdef NEX_WATERMARK_ENABLED
class WaterMark;
#endif

class MutexPrimitive;

//group=Miscellaneous Classes
//$ifdef WII_COMMON$
//group=Net-Z API
//$endif$

/*
Description: Overrides the default $PRODUCTNAME$ memory allocation functions.
Remarks: This class is used to override the default $PRODUCTNAME$ memory allocation functions.

$ifdef WII_COMMON$
Note: Memory access on the Wii is not thread-safe. So it is assumed that you will create your own
thread-safe memory allocation functions. If you wish to use them with Net-Z objects, you must register
them using the SetBasicMemoryFunctions method.
$endif$

*/

/*!
 @brief    メモリ確保・開放処理の管理クラスです。

 NEXライブラリは内部で動的なメモリ確保を頻繁に行います。
 デフォルトでは、NEXライブラリは標準のmalloc()/free()関数を利用してメモリ確保を行いますが、
 @ref SetBasicMemoryFunctions でメモリマネージャーを差し替えることが可能です。

 差し替えるメモリマネージャーをスレッドセーフにするか、スレッドアンセーフにするかはアプリケーション側の
 判断に委ねられますが、いくつか注意点があります。@ref SetBasicMemoryFunctions() をご参照下さい。
 */
class MemoryManager: public RootObject
{
public:

    typedef enum _InstructionType
    {
        Unknown,
        DirectCall,
        UntrackedDirectCall,
        RootObjectNewDelete,
        RootObjectNewDeleteArray,
        GlobalNewDelete,
        GlobalNewDeleteArray,
        AllocatorClass,
        qSpecialNewDelete,
        qSpecialNewDeleteArray,
        qAllocFree
    } InstructionType;

    //n1769
    const qChar* GetInstructionTypeString(InstructionType t) const;

#ifdef NEX_HEAP_DEBUGGING_ENABLED
    typedef struct _Trailer
    {
        qUnsignedInt32 m_uiSignature;
        qByte m_pPadding[HEAP_TRAILER_PADDING_SIZE];
    } Trailer;
#endif


    /*!
    @brief メモリ確保関数の関数型です。

       stSize には確保するメモリサイズが渡されます。
       返値には確保したメモリの先頭を指すポインタを指定してください。

    @param[in] stSize  確保するメモリサイズ
    @see  MemoryManager::SetBasicMemoryFunctions, FreeFunction
    @return  確保したメモリの先頭を指すポインタ。
    */
    typedef void* (* MallocFunction)(size_t stSize);


    /*!
    @brief メモリ解放関数の関数型です。

       pMemory には解放すべきメモリの先頭を指すポインタが渡されます。

    @param[in] pMemory  解放すべきメモリの先頭を指すポインタ。
    @see  MemoryManager::SetBasicMemoryFunctions, MallocFunction
    */
    typedef void (*FreeFunction)(void *pMemory);



#ifdef DBG_STACKWALK_SUPPORT
    enum { MaxDbgStackSize = 12 };
#endif

    typedef struct _Header
    {
#ifdef NEX_HEAP_DEBUGGING_ENABLED
        qUnsignedInt32  m_uiSignature;
        qUnsignedInt32  m_uiLine;
        qUnsignedInt64  m_tiTime;
        qInt32          m_count;
        InstructionType m_eSourceInstruction;
        MemoryManager   *m_pMemoryManager;
        Trailer         *m_pTrailer;
        struct _Header  *m_pPrevious;
        struct _Header  *m_pNext;
        size_t          m_stRealSize;
        const qChar     *m_szFile;
#ifdef DBG_STACKWALK_SUPPORT
        qUnsignedInt32  m_pStack[MaxDbgStackSize];
#endif
#endif
        FreeFunction    m_fcnFree;
        void*           m_pInfo; // 追加の情報
    } Header;

protected:
    /*
    Parameters:
    szName: The name of this MemoryManager.
    Remarks: A MemoryManager object should never be directly instantiated. It is created
    by $PRODUCTNAME$ when required.
    */
    MemoryManager(const qChar* szName);
public:
    /*
    Remarks: A MemoryManager object should never be directly destroyed. It is destroyed
    by $PRODUCTNAME$ when no longer required.
    */
    virtual ~MemoryManager();

#ifdef NEX_HEAP_DEBUGGING_ENABLED

    //{secret}
    static void *Allocate(size_t stSize, InstructionType eSourceInstruction=DirectCall)
    {
        return Allocate( stSize, NEX_T("Unknown"), 0, eSourceInstruction );
    }
    //{secret}
    static void *Allocate(size_t stSize, const qChar *szFile, qUnsignedInt32 uiLine, InstructionType eSourceInstruction=DirectCall);
    //{secret}
    static void Free(void *pPtr, InstructionType eSourceInstruction=DirectCall);

#else   // NEX_HEAP_DEBUGGING_ENABLED

    //{secret}
    static void *Allocate(size_t stSize, const qChar *szFile, qUnsignedInt32 uiLine, InstructionType eSourceInstruction=DirectCall)
    {
        DUMMY_PARAM( szFile );
        DUMMY_PARAM( uiLine );
        DUMMY_PARAM( eSourceInstruction );
        return Allocate( stSize );
    }
    //{secret}
    static void *Allocate(size_t stSize, InstructionType eSourceInstruction)
    {
        DUMMY_PARAM( eSourceInstruction );
        return Allocate( stSize );
    }
    //{secret}
    static void Free(void *pPtr, InstructionType eSourceInstruction)
    {
        DUMMY_PARAM( eSourceInstruction );
        Free( pPtr );
    }
    //{secret}
    static void *Allocate(size_t stSize);
    //{secret}
    static void Free(void *pPtr);

#endif  // NEX_HEAP_DEBUGGING_ENABLED

    static void *AllocateForPbPool(void* p, FreeFunction fcnFree, void* pInfo);

    static void *AllocateThreadSafe(size_t stSize);
    static void FreeThreadSafe(void *pPtr);
protected:
    // Methods used by the manager.

    static void* GenericMalloc(size_t stRealSize);
    static void GenericFree(FreeFunction fcnFree, void* pPtr);

#if defined(NEX_HEAP_DEBUGGING_ENABLED)
    virtual void BeginProtection();
    virtual void EndProtection();
#else
    virtual void BeginProtection() {}
    virtual void EndProtection() {}
#endif
    void IncreaseMemUsage(size_t stRealSize);
    void DecreaseMemUsage(size_t stRealSize);

public:
#ifndef NEX_DEFAULT_ALIGNMENT
#define NEX_DEFAULT_ALIGNMENT   (sizeof(void*))
#endif
    static const size_t NEX_MAX_ALIGNMENT = 8;

    // 指定されたサイズのメモリを確保したいときに実際に確保すべき量を計算する
    static size_t GetAlignedSize(size_t size)
    {
        // ビット演算によるアライメントサイズ計算。アライメントサイズが2の累乗である必要がある。
        return (size + (NEX_DEFAULT_ALIGNMENT - 1)) & ~(NEX_DEFAULT_ALIGNMENT - 1);
    }

    // 指定されたサイズのメモリを確保したいときに実際に確保すべき量を計算する
    static size_t GetAlignedSize(size_t size, size_t alignmentSize)
    {
        // ビット演算によるアライメントサイズ計算。アライメントサイズが2の累乗である必要がある。
        return (size + (alignmentSize - 1)) & ~(alignmentSize - 1);
    }

    //{secret}
    static size_t GetHeaderSize()
    {
        return GetAlignedSize(sizeof(Header));
    }
    //{secret}
    static Header* GetHeaderPtr(void* pUserPtr)
    {
        return (Header*)((qByte*)pUserPtr-GetHeaderSize());
    }

    // To cleanup.

#ifdef NEX_HEAP_DEBUGGING_ENABLED
    //{secret}
    qBool MemoryLeakDetected(qUnsignedInt32 uiDesiredMemoryUsage=0)
    {
        return m_uiMemUsage!=uiDesiredMemoryUsage;
    }
    // 現在メモリアロケートされている個数を取得する
    // この値をDumpAllocatedMemory()の第一引数に使用し、ある時点からアロケートされたメモリ一覧を取得することができる。
    qInt32 GetCurrentAllocationCount() const
    {
        return m_pLastHeader->m_count;
    }
    //{secret}
    void DumpAllocatedMemory(qInt32 startMemoryAllocateCount = 0, qBool boShowStack = false);

    //{secret}
    static size_t GetRealAllocationSize(Header* pHeader);
    //{secret}
    static size_t GetUserAllocationSize(Header* pHeader);
    //{secret}
    qUnsignedInt32 GetMemoryUsage()
    {
        return m_uiMemUsage;
    }
#endif

#ifdef NEX_HEAP_DEBUGGING_ENABLED
    //{secret}
    static void CheckPointer(MemoryManager* pManager, void* pUserPtr, InstructionType eSourceInstruction=Unknown)
    {
        CheckBlock(pManager, GetHeaderPtr(pUserPtr), eSourceInstruction);
    }

    //{secret}
    static void CheckPointer(void* pUserPtr, InstructionType eSourceInstruction=Unknown)
    {
        CheckBlock(GetDefaultMemoryManager(), GetHeaderPtr(pUserPtr), eSourceInstruction);
    }

    //{secret}
    static void CheckBlock(MemoryManager* pManager, Header* pHeader, InstructionType eSourceInstruction=Unknown);
    //{secret}
    qBool ValidBlock(Header *pHeader, InstructionType eSourceInstruction=Unknown);
    //{secret}
    qBool ValidPointer(void* pPtr, InstructionType eSourceInstruction=Unknown)
    {
        return ValidBlock(GetHeaderPtr(pPtr), eSourceInstruction);
    }

    //{secret}
    void CheckMemoryState();

    //{secret}
    void SetLog(Log *pLog)
    {
        m_pLog = pLog;
    }

    //{secret}
    void GetMemoryBlockInfo(Header *pHeader, qChar *szInfo, qBool bDumpStack = false);

    //{secret}
    void LogSnapshot(Log *pLog, qInt32 tStartCount = 0, qInt32 tEndCount = 0, qBool bDumpStack = false);
    
    void LogSnapshotByTime(Log *pLog, qUnsignedInt64 tStartTime = 0, qUnsignedInt64 tEndTime = 0, qBool bDumpStack = false);

    qInt32 GetMemoryAllocateCount() { return m_memoryAllocateCount; }
#endif

    static MemoryManager* GetDefaultMemoryManager();

    //{secret}
    static void ShutdownDefaultMemoryManager();

    //group=Methods

    /*
    Description: Enables the developer to override the default memory allocation functions used
    by $PRODUCTNAME$.
    Parameters:
    fcnMalloc: A user-defined Malloc function.
    fcnFree: A user-defined Free function.
    Remarks: This method enables the developer to override the memory allocation functions used
    by the $PRODUCTNAME$ memory manager. By default $PRODUCTNAME$ calls the /malloc/ and /free/
    functions to allocate memory. Using this method, however, you can change these functions to your
    own memory allocation functions. This method may be called at any time. However, it is normally
    called before the $PRODUCTNAME$ object is created.

    By redefining the MallocFunction and FreeFunction methods, SetBasicMemoryFunctions can be used,
    for example, to allocate memory via your own memory manager or to calculate variables such as
    how much memory $PRODUCTNAME$ uses. If you redefine the fcnMalloc and fcnFree functions, they
    must use the same signatures as given in the example below.

    For example, we could redefine the MallocFunction and FreeFunction to count the number
    of allocated memory blocks as follows:

        qUnsignedInt g_uiAllocatedBlocksCnt=0;

        void* SpecialMalloc(size_t stSize) {
            g_uiAllocatedBlocksCnt++;
            return malloc(stSize);
        }

        void SpecialFree(void* pBlock) {
            g_uiAllocatedBlocksCnt--;
            free(pBlock);
        }

        MemoryManager::SetBasicMemoryFunctions(SpecialMalloc, SpecialFree);

    */


    /*!
     @brief    NEXライブラリ内部で使用されるメモリ確保／解放関数を設定します。

     NEXライブラリは、内部で動的なメモリ確保を頻繁に行いますが、
     本関数でメモリマネージャーを差し替えることができます。

     

     この関数を使用すると、NEXライブラリが使用するメモリマネージャーをデベロッパ独自のコードに
     変更することができます。この関数はいつでも呼び出すことができますが、通常はNEXのオブジェクトが
     作成される前に呼び出します。

     差し替えるメモリマネージャーをスレッドセーフなものとするか、スレッドアンセーフなものとするかは
     アプリケーション側の判断に委ねられます。

     もしNEXライブラリを非シングルスレッドモードで利用しているのであれば、差し替えるメモリマネージャーは
     スレッドセーフでなくてはなりません。

     NEXライブラリをシングルスレッドモードで使用している場合であっても、アプリケーション側が
     マルチスレッド環境で動作し、NEXライブラリのオブジェクトを複数のスレッドから利用しているような場合は
     注意が必要です。
     NEXライブラリのオブジェクトを自動変数として使用する（オブジェクトをスタック上に作成する）ケースであっても、
     オブジェクトの内部で動的なメモリ確保が実行されるケースもありますので、もしスレッドアンセーフな
     メモリマネージャーを使用する場合は、適切な排他制御が必要となります。

     NEXライブラリ内部では、動的メモリ確保は必ず成功することを前提としたコードが記述されています。
     そのため、必ず有効なメモリへのアドレスを返すように実装してください。
     具体的には、以下のような実装をしていただく必要があります。

     ・予定してあるメモリ量を超えてメモリを確保したらライブラリをシャットダウンする。

     ・事前に数KB程度(正常にライブラリシャットダウンまで実行できるであろう量)のメモリをAllocしておき、Allocできなくなったらその数KBのメモリを解放する。その後ライブラリをシャットダウンする。

     ・予めメモリに十分余裕を持っておく（通信環境が悪くなった場合などでもメモリが足りなくなることがないように十分なデバッグを行なってください）。

     @param[in]    fcnMalloc  ユーザー定義のメモリ確保関数。
     @param[in]    fcnFree    ユーザー定義のメモリ解放関数。
     */
    static void SetBasicMemoryFunctions(MallocFunction fcnMalloc, FreeFunction fcnFree)
    {
        s_fcnMalloc = fcnMalloc;
        s_fcnFree = fcnFree;
    }

#ifdef CAFEIOP_TARGET
    /*!
     @brief   IPCで使用するアロケータを指定します。

         NetZやRendezVousの生成前やCore::AcquireInstance()実行前に設定してください。
         MemoryManager::SetBasicMemoryFunctions()とは違い途中での差し替えには対応していません。

     @param[in]    fcnMalloc  ユーザー定義のメモリ確保関数。IPC用のメモリ確保を行う関数を指定してください。
     @param[in]    fcnFree    ユーザー定義のメモリ解放関数。
     */
    static void SetIpcMemoryFunctions(MallocFunction fcnMalloc, FreeFunction fcnFree)
    {
        s_fcnIpcMalloc = fcnMalloc;
        s_fcnIpcFree = fcnFree;
    }
    static void *AllocateIpc(size_t stSize)
    {
        return s_fcnIpcMalloc(stSize);
    }
    static void FreeIpc(void *pPtr)
    {
        s_fcnIpcFree(pPtr);
    }
#endif



#ifdef WII_TARGET
    // {secret}
    static qBool IsMemoryFunctionSet()
    {
        return (s_fcnMalloc != NULL);
    }
#endif
    // {secret}
    void Trace();

private:

#ifdef NEX_HEAP_DEBUGGING_ENABLED
    //{secret}
    static void LinkHeader(MemoryManager* pManager, Header* pHeader);
    //{secret}
    static void UnlinkHeader(MemoryManager* pManager, Header* pHeader);
#endif
    qUnsignedInt32 m_uiMemUsage;
#ifdef NEX_HEAP_DEBUGGING_ENABLED
    Header        *m_pLastHeader;
    Log           *m_pLog;
    AtomicValue   m_memoryAllocateCount;
#endif
#ifdef NEX_WATERMARK_ENABLED
    WaterMark   *m_pMemoryWaterMark;
#endif
    static enum eShutDownState
    {
        ShutDownPending,
        ShutDownInProcess,
        ShutDownDone
    } s_eShutDownState;

    MutexPrimitive* m_pMutex;

    //{secret}
    static MallocFunction s_fcnMalloc;
    //{secret}
    static FreeFunction s_fcnFree;
#ifdef CAFEIOP_TARGET
    //{secret}
    static MallocFunction s_fcnIpcMalloc;
    //{secret}
    static FreeFunction s_fcnIpcFree;
#endif
};

//{secret}
// NB_OBJECT_TYPEはsize_tやqUnsignedInt32やqInt32などの整数値
template <class T, class NB_OBJECT_TYPE> T* SpecialNewArray(NB_OBJECT_TYPE uiNbObjects, const qChar* szFile, qUnsignedInt32 uiLine)
{
    // Allocate the memory.
#ifdef NEX_HEAP_DEBUGGING_ENABLED
    void *pMemoryBlock=MemoryManager::Allocate(sizeof(T)*uiNbObjects + sizeof(qUnsignedInt32), szFile, uiLine, MemoryManager::qSpecialNewDeleteArray);

#else /* NEX_HEAP_DEBUGGING_ENABLED */
    DUMMY_PARAM(szFile);
    DUMMY_PARAM(uiLine);
    void *pMemoryBlock=MemoryManager::Allocate(sizeof(T)*uiNbObjects + sizeof(qUnsignedInt32), MemoryManager::qSpecialNewDeleteArray);
#endif /* NEX_HEAP_DEBUGGING_ENABLED */

    // 先頭4byteに確保する個数を入れる
    qUnsignedInt32 *pNbObjects=(qUnsignedInt32*) pMemoryBlock;
    T* pTypePtr=(T*) (pNbObjects+1);
    // Store the number of objects.
    *pNbObjects=static_cast<qUnsignedInt32>(uiNbObjects);
    // Create the objects.
#ifndef SN_COMPILER
    for (NB_OBJECT_TYPE ui=0; ui<uiNbObjects; ui++)
    {
        new ((qChar*)(pTypePtr+ui)) T;
    }
#endif
    return pTypePtr;
}

//{secret}
template <class T> void SpecialDeleteArray(T *pPtr)
{
    // Retrieve number of objects.
    qUnsignedInt32 *pNbObjects=((qUnsignedInt32*) pPtr)-1;
    void *pMemoryBlock=pNbObjects;
    // Destroy the objects.
    T* pTypePtr=(T*) pPtr;
    // This pointer is actually not used at all if T is a simple type
    // that has no destructor - the compiler can get rid of the loop
    // that follows.
    DUMMY_PARAM(pTypePtr);
    for (qUnsignedInt ui=0; ui<*pNbObjects; ui++)
    {
        pTypePtr[ui].~T();
    }
    // Free the memory.
    MemoryManager::Free(pMemoryBlock, MemoryManager::qSpecialNewDeleteArray);
}

//{secret}
template <class T> T* SpecialNew(const qChar* szFile, qUnsignedInt32 uiLine)
{
    // Allocate the memory.
#ifdef NEX_HEAP_DEBUGGING_ENABLED
    void *pObjectLocation=MemoryManager::Allocate(sizeof(T), szFile, uiLine, MemoryManager::qSpecialNewDelete);

#else /* NEX_HEAP_DEBUGGING_ENABLED */
    DUMMY_PARAM(szFile);
    DUMMY_PARAM(uiLine);
    void *pObjectLocation=MemoryManager::Allocate(sizeof(T), MemoryManager::qSpecialNewDelete);
#endif /* NEX_HEAP_DEBUGGING_ENABLED */

#ifdef SN_COMPILER
    return (T*)pObjectLocation;
#else /* SN_COMPILER */
    return new (pObjectLocation) T;
#endif /* SN_COMPILER */
}

//{secret}
template <class T> void SpecialDelete(T *pPtr)
{
    // Call the object destructor.
    pPtr->~T();
    // Free the memory.
    MemoryManager::Free(pPtr, MemoryManager::qSpecialNewDelete);
}


/*!
  @defgroup nex_define nex_define
  @{
 */

// New macros
#ifdef NEX_HEAP_DEBUGGING_ENABLED

#define qAlloc(TYPE,NUMBER)             (TYPE*) NAMESPACE_NEX MemoryManager::Allocate(sizeof(TYPE)*(NUMBER), NEX_T(__FILE__), __LINE__, NAMESPACE_NEX MemoryManager::qAllocFree)
#define qSpecialNew(TYPE)               NAMESPACE_NEX SpecialNew<TYPE>(NEX_T(__FILE__), __LINE__)
#define qSpecialNewArray(TYPE,NUMBER)   NAMESPACE_NEX SpecialNewArray<TYPE>(NUMBER, NEX_T(__FILE__), __LINE__)
#define qNew                            new(NEX_T(__FILE__), __LINE__)
#define qNewMayFail(pool)               new(pool, NEX_T(__FILE__), __LINE__)

#else /* NEX_HEAP_DEBUGGING_ENABLED */

#define qAlloc(TYPE,NUMBER)             (TYPE*) NAMESPACE_NEX MemoryManager::Allocate(sizeof(TYPE)*(NUMBER), NAMESPACE_NEX MemoryManager::qAllocFree)
#define qSpecialNew(TYPE)               NAMESPACE_NEX SpecialNew<TYPE>(NEX_T(""), 0)
#define qSpecialNewArray(TYPE,NUMBER)   NAMESPACE_NEX SpecialNewArray<TYPE>(NUMBER, NEX_T(""), 0)

/*
 @brief    new演算子をラップするマクロです。
 */
#define qNew                            new
#define qNewMayFail(pool)               new(pool)

#endif /* NEX_HEAP_DEBUGGING_ENABLED */

#define qFree(PTR)                      NAMESPACE_NEX MemoryManager::Free(PTR, NAMESPACE_NEX MemoryManager::qAllocFree)
#define qSpecialDelete(PTR)             NAMESPACE_NEX SpecialDelete(PTR)
#define qSpecialDeleteArray(PTR)        NAMESPACE_NEX SpecialDeleteArray(PTR)

/*
 @brief    delete演算子をラップするマクロです。
 */
#define qDelete                         delete


#ifdef NEX_HEAP_DEBUGGING_ENABLED
#define NEX_STARTSNAPSHOT(Var) \
    Log *pLog##Var = qNew Log(); \
    LogDeviceFile *pLogDevice##Var = qNew LogDeviceFile(); \
    pLogDevice##Var->SetFile(NEX_T(#Var) NEX_T(".log")); \
    pLog##Var->SetLogDevice(pLogDevice##Var); \
    Platform::Sleep(1); \
    qInt32 startCount##Var = MemoryManager::GetDefaultMemoryManager()->GetMemoryAllocateCount(); \
 

#define NEX_STOPSNAPSHOT(Var) \
    MemoryManager::GetDefaultMemoryManager()->LogSnapshot(pLog##Var,startCount##Var,MemoryManager::GetDefaultMemoryManager()->GetMemoryAllocateCount(),true); \
    qDelete pLog##Var;

#define NEX_SNAPSHOT(Var) \
    qInt32 endCount##Var = MemoryManager::GetDefaultMemoryManager()->GetMemoryAllocateCount(); \
    Platform::Sleep(1); \
    Log *pLog##Var = qNew Log(); \
    LogDeviceFile *pLogDevice##Var = qNew LogDeviceFile(); \
    pLogDevice##Var->SetFile(NEX_T(#Var) NEX_T(".log")); \
    pLog##Var->SetLogDevice(pLogDevice##Var); \
    MemoryManager::GetDefaultMemoryManager()->LogSnapshot(pLog##Var,0,endCount##Var,true); \
    qDelete pLog##Var;

#define NEX_STARTLOG(Var) \
    Log *pLog##Var = qNew Log(); \
    LogDeviceFile *pLogDevice##Var = qNew LogDeviceFile(); \
    pLogDevice##Var->SetFile(NEX_T(#Var) NEX_T(".log")); \
    pLog##Var->SetLogDevice(pLogDevice##Var); \
    MemoryManager::GetDefaultMemoryManager()->SetLog(pLog##Var);


#define NEX_STOPLOG(Var) \
    MemoryManager::GetDefaultMemoryManager()->SetLog(NULL); \
    qDelete pLog##Var;
#endif /* NEX_HEAP_DEBUGGING_ENABLED */

/*!
  @}
 */


END_NAMESPACE_NEX

#else /* __cplusplus */
//  C allocation routines

//  void *malloc( size_t size );
//{secret}
void*  QuazalCRTAlloc( size_t stSize );

//  void *realloc(void *memblock, size_t size );
//{secret}
void* QuazalCRTRealloc( void *pPtr, size_t stSize );

//  void* calloc( size_t stNum, size_t stSize );
//{secret}
void* QuazalCRTCalloc( size_t stNum, size_t stSize );

//  void free( void* pPtr )
//{secret}
void QuazalCRTFree( void* pPtr );

#endif // #ifdef __cplusplus

