﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/Platform.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
// Group=Platform Subsystem

#ifdef NEX_SIGLO_FAMILY
#include <nn/os.h>
#elif defined(UNIX_FAMILY)
#include <pthread.h>
#elif defined( CTR_TARGET )
#include <nn/os/os_CriticalSection.h>
#elif defined( CAFE_TARGET )
#include <cafe/os.h>
#elif defined(CAFEIOP_TARGET)
#include <utl_mutex.h>
#endif

BEGIN_NAMESPACE_NEX

//==============================================================================
// Mutex primitive (Quazal internal class)
//==============================================================================
class MutexPrimitive: public RootObject, public NonCopyable
{
    class Attributes;
    friend class Condition;

    // Group=
public:
    MutexPrimitive(qBool reentrantable=true);
    virtual ~MutexPrimitive();

    void Enter()
    {
        if (!s_bNoOp)
        {
            EnterImpl();
        }
    }
    qBool TryEnter()
    {
        if (!s_bNoOp)
        {
            return TryEnterImpl();
        }
        return true;
    }
    void Leave()
    {
        if (!s_bNoOp)
        {
            LeaveImpl();
        }
    }
    // 強制ロック
    void ForceEnter()
    {
        EnterImpl();
    }
    qBool ForceTryEnter()
    {
        return TryEnterImpl();
    }
    // 強制アンロック
    void ForceLeave()
    {
        LeaveImpl();
    }

#if !defined(NEX_SIGLO_FAMILY) && (defined(WIN_FAMILY))
    // WIN_FAMILY もしくは UNIX_FAMILY の場合
    void EnterImpl();
    qBool TryEnterImpl();
    void LeaveImpl();
#else
    void EnterImpl()
    {
#ifdef NEX_SIGLO_FAMILY
        nn::os::LockMutex(&m_hMutex);
#elif defined(UNIX_FAMILY)
        pthread_mutex_lock(&m_hMutex);
#elif defined(CAFE_TARGET)
        OSLockMutex(&m_hMutex);
#elif defined(CAFEIOP_TARGET)
        UTL_LockMutex(&m_hMutex);
#elif defined(CTR_TARGET)
        m_hMutex.Enter();
#else
#error not supported platform
#endif
#ifdef NEX_LOCK_DEBUG_ENABLED
        m_iCounter++;
#endif
    }

    qBool TryEnterImpl()
    {
#ifdef NEX_SIGLO_FAMILY
        qBool isSucceeded = nn::os::TryLockMutex(&m_hMutex);
#elif defined(UNIX_FAMILY)
        int ret = pthread_mutex_trylock(&m_hMutex);
        SYSTEMCHECK(ret == 0 || EBUSY);
        qBool isSucceeded = ret == 0;
#elif defined(CAFE_TARGET)
        qBool isSucceeded = OSTryLockMutex(&m_hMutex);
#elif defined(CAFEIOP_TARGET)
        qBool isSucceeded = UTL_TryLockMutex(&m_hMutex);
#elif defined(CTR_TARGET)
        qBool isSucceeded = m_hMutex.TryEnter();
#else
#error not supported platform
#endif
#ifdef NEX_LOCK_DEBUG_ENABLED
        m_iCounter++;
#endif
        return isSucceeded;
    }

    void LeaveImpl()
    {
#ifdef NEX_LOCK_DEBUG_ENABLED
        // EnterしてないのにLeaveはNG
        SYSTEMCHECK(m_iCounter != 0 && NEX_T("MutexPrimitive::LeaveImpl Not Entered Mutex"));
        m_iCounter--;
#endif
#ifdef NEX_SIGLO_FAMILY
        nn::os::UnlockMutex(&m_hMutex);
#elif defined(UNIX_FAMILY)
        pthread_mutex_unlock(&m_hMutex);
#elif defined(CAFE_TARGET)
        OSUnlockMutex(&m_hMutex);
#elif defined(CAFEIOP_TARGET)
        UTL_UnlockMutex(&m_hMutex);
#elif defined(CTR_TARGET)
        m_hMutex.Leave();
#else
#error not supported platform
#endif
    }
#endif

    static qBool IsANoOp()
    {
        return s_bNoOp;
    }
    static void SetNoOpFlag()
    {
        s_bNoOp=true;
    }
    static void ClearNoOpFlag()
    {
        s_bNoOp=false;
    }
#ifdef NEX_LOCK_DEBUG_ENABLED
    qInt32 GetCount() const
    {
        return m_iCounter;
    }
#endif

private:
#ifdef NEX_SIGLO_FAMILY
    nn::os::MutexType m_hMutex;
#elif defined(WIN_FAMILY)
    // windows.hをヘッダからインクルードしないためにここではCRITICAL_SECTION構造体を使わない。
#ifdef NEX_64BIT_ARCH
    static const qUnsignedInt32 CRITICAL_SECTION_SIZE = 40; // sizeof(CRITICAL_SECTION)
#else
    static const qUnsignedInt32 CRITICAL_SECTION_SIZE = 24; // sizeof(CRITICAL_SECTION)
#endif
    qByte m_hMutex[CRITICAL_SECTION_SIZE];
#elif defined(UNIX_FAMILY)
    pthread_mutex_t m_hMutex;
#elif defined(CTR_TARGET)
    nn::os::CriticalSection m_hMutex;
#elif defined(CAFE_TARGET)
    OSMutex m_hMutex;
#elif defined(CAFEIOP_TARGET)
    UTLMutex m_hMutex;
#else
#error MutexPrimitive is not implemented.
#endif

#ifdef NEX_LOCK_DEBUG_ENABLED
    qInt32 m_iCounter; // デバッグ用
#endif
    static qBool s_bNoOp;
};

END_NAMESPACE_NEX

