﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

// <desc: Class that abstracts the notion of threads>

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/ObjectThreadRoot.h>
#include <OnlineCore/src/Core/CallbackHolder.h>

BEGIN_NAMESPACE_NEX

// Group=Platform Subsystem
template<class TargetClass, class ArgumentType> class ObjectThread: public ObjectThreadRoot
{
    // Group=
public:
    ObjectThread(const String& strName, qUnsignedInt threadPrio=GetDefaultPrio())
        : ObjectThreadRoot(strName, threadPrio)
        , m_pMethod(NULL)
        , m_pTargetObject(NULL)
    {
    }

    virtual ~ObjectThread()
    {
    }

    /*!
    @brief      オブジェクトスレッドを起動します。

    @param[in]  pTargetObject スレッド関数をメンバに持つオブジェクト
    @param[in]  pMethod       スレッド関数
    @param[in]  arg           スレッド関数の引数
    @param[in]  uiStackSize   スタックのサイズ。デフォルト0を指定すると ObjectThreadRoot::s_uiDefaultThreadStackSize が使用されます。
    */
    qBool Launch(TargetClass* pTargetObject,
                 void (TargetClass::*pMethod)(ArgumentType arg),
                 ArgumentType arg,
                 size_t uiStackSize = 0)
    {
        m_pTargetObject=pTargetObject;
        m_pMethod=pMethod;
        m_arg=arg;
        return ObjectThreadRoot::Launch(uiStackSize);
    }

private:

    virtual void CallObjectMethod()
    {
        //SYSTEMCHECK(m_pTargetObject!=NULL);
        //SYSTEMCHECK(m_pMethod!=NULL);
        TargetClass* pTargetObject=m_pTargetObject;
        void (TargetClass::*pMethod)(ArgumentType arg)=m_pMethod;
        ArgumentType arg=m_arg;
        ReadyToRun(); // When this is called, we cannot use any class variables anymore...
        (pTargetObject->*pMethod)(arg);
    }

private:
    void (TargetClass::*m_pMethod)(ArgumentType arg);
    TargetClass* m_pTargetObject;
    ArgumentType m_arg;
};


class ObjectThreadGeneric: public ObjectThreadRoot
{
public:
    ObjectThreadGeneric(const String& strName, qUnsignedInt threadPrio=GetDefaultPrio())
        : ObjectThreadRoot(strName, threadPrio)
        , m_pCallbackHolder(NULL)
    {
    }

    virtual ~ObjectThreadGeneric()
    {
        qDelete m_pCallbackHolder;
    }

    /*!
    @brief      オブジェクトスレッドを起動します。

    @param[in]  callback 起動したスレッドから呼び出される関数。
    @param[in]  deleteCallbackAfterCall callbackの実行が完了後にcallbackをdeleteするかどうか。
    @param[in]  uiStackSize   スタックのサイズ。デフォルト0を指定すると ObjectThreadRoot::s_uiDefaultThreadStackSize が使用されます。
    */
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    template<typename TCallback>
    qBool Launch(TCallback&& callback,
                 qBool deleteCallbackAfterCall,
                 size_t uiStackSize=0)
    {
        m_pCallbackHolder = qNew CallbackHolder<TCallback, void>(std::forward<TCallback>(callback), deleteCallbackAfterCall);
        return ObjectThreadRoot::Launch(uiStackSize);
    }
#else
    template<typename TCallback>
    qBool Launch(const TCallback& callback,
                 qBool deleteCallbackAfterCall,
                 size_t uiStackSize = 0)
    {
        m_pCallbackHolder = qNew CallbackHolder<TCallback, void>(callback, deleteCallbackAfterCall);
        return ObjectThreadRoot::Launch(uiStackSize);
    }
#endif

private:

    virtual void CallObjectMethod()
    {
        ReadyToRun(); // When this is called, we cannot use any class variables anymore...
        m_pCallbackHolder->Call(NULL);
    }

private:
    CallbackHolderBase* m_pCallbackHolder;
};

END_NAMESPACE_NEX

