﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <OnlineCore/src/Platform/Core/ProfilingUnit.h>

#include <stdlib.h>  // mbstowcs()用。

/*!
    @brief このヘッダは、Quazal::ProfilingUnitを使いやすくするために用意されたマクロです。

    使い方：
    例えば以下のような関数 foo() があったとします。

    void foo()
    {
        NN_LOG("This is foo().\n");
    }

    この関数のプロファイルをしたい、というときは、
    関数の先頭にCTR_PROFILING_UNITマクロを記述してください。
    これでfoo()が計測の対象となります。
    このとき、引数としてオーダー番号を与えます。
    プロファイリング結果のリストを作成する際、このオーダー番号で
    ソートすることができます。

    void foo()
    {
        CTR_PROFILING_UNIT(1000);
        NN_LOG("This is foo().\n");
    }

    関数全体ではなく、特定のブロックの計測を行う場合は、そのブロックの先頭に
    名前指定のマクロ、CTR_PROFILING_UNIT_WITH_NAME()を使って下さい。

    void goo()
    {
        ...
        ...

        while(1)
        {
            CTR_PROFILING_UNIT_WITH_NAME(1000, "main loop");

            ...
            ...
        }
    }


    また、CPUのコストではなく、なんらかの「量」をカウントしておきたい場合に、
    CTR_PROFILING_UNIT_WITH_SIZEマクロを利用できます。
    例えば送受信を行う関数において、転送した量を記録しておきたい場合などです。
    この場合は、以下のようにマクロを用いると、転送量の合計を記録しておくことができます。
    void send(const u8 *buf, size_t sz)
    {
        CTR_PROFILING_UNIT_WITH_SIZE(sz);

        ...
        ...
    }


    プロファイリング機能を無効にする（実行バイナリに含まれないようにする）ときは、
    明示的に NEX_DISABLE_USING_CTR_PROFILING_UNIT マクロを 1 で #define してください。
 */
//#define NEX_DISABLE_USING_CTR_PROFILING_UNIT 1

#if NEX_DISABLE_USING_CTR_PROFILING_UNIT

#define CTR_PROFILING_UNIT(order)                   (void)0
#define CTR_PROFILING_UNIT_WITH_NAME(order, name)   (void)0
#define CTR_PROFILING_UNIT_WITH_SIZE(order, amount) (void)0

#else
//n1769
// UNICODEビルドで多少躓いた。
// NEX_T()マクロが展開され、Lとなるが、このLと__PRETTY_FUNCTION__が結合してしまい、
// 変なシンボルを生成してしまっていた。
// http://social.msdn.microsoft.com/Forums/en-US/vcgeneral/thread/82213969-1796-4000-9656-3e692e3ec634
// ↑のサイトに記載されていたテクニックを試し、以下のようなマクロを書いてはみたが、
// これでもダメだった。
//  #define MY_TEXT_MACRO2(x) L##x
//  #define MY_TEXT_MACRO(x) MY_TEXT_MACRO2(x)
//  #define MY_PRETTY_FUNCTION MY_TEXT_MACRO(__PRETTY_FUNCTION__)
// _UNICODEマクロの有無による場合分けを導入し、UNICODEビルドの時は、いったんワイド文字配列に
// コピーするようにしてみた。これでとりあえずビルドは通った。

// 順序付き。SetOrder()が何度も呼び出され、ムダであるが、目をつぶる。
#define CTR_PROFILING_UNIT_WITH_NAME(order, name)                               \
          static NAMESPACE_NEX ProfilingUnit s_profilingUnitWithNameInstance(NEX_T(name)); \
          s_profilingUnitWithNameInstance.SetOrder(order);                        \
          NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnitWithName(s_profilingUnitWithNameInstance);

#ifndef _UNICODE
#define CTR_PROFILING_UNIT(order)                                                  \
            static NAMESPACE_NEX ProfilingUnit s_profilingUnitInstance(__PRETTY_FUNCTION__); \
            s_profilingUnitInstance.SetOrder(order);                                   \
            NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnit(s_profilingUnitInstance);

#define CTR_PROFILING_UNIT_WITH_SIZE(order, amount)                                                \
            static NAMESPACE_NEX ProfilingUnit s_profilingUnitWithSizeInstance(__PRETTY_FUNCTION__);         \
            s_profilingUnitWithSizeInstance.SetOrder(order);                                           \
            NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnitWithSize(s_profilingUnitWithSizeInstance); \
            s_profilingUnitWithSizeInstance.IncAmount(amount);

#else

#define CTR_PROFILING_UNIT(order)                                                    \
            static bool  s_first_time_flag = true;                                       \
            static NAMESPACE_NEX qChar s_function_name[128];                                   \
            if(s_first_time_flag)mbstowcs(s_function_name, __PRETTY_FUNCTION__, 128);    \
            s_first_time_flag = false;                                                   \
            static NAMESPACE_NEX ProfilingUnit s_profilingUnitInstance(s_function_name);       \
            s_profilingUnitInstance.SetOrder(order);                                     \
            NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnit(s_profilingUnitInstance);

#define CTR_PROFILING_UNIT_WITH_SIZE(order, amount)                                                \
            static bool  s_first_time_flag = true;                                                     \
            static NAMESPACE_NEX qChar s_function_name[128];                                                 \
            if(s_first_time_flag)mbstowcs(s_function_name, __PRETTY_FUNCTION__, 128);                  \
            s_first_time_flag = false;                                                                 \
            static NAMESPACE_NEX ProfilingUnit s_profilingUnitWithSizeInstance(s_function_name);             \
            s_profilingUnitWithSizeInstance.SetOrder(order);                                           \
            NAMESPACE_NEX ProfilingUnit::Scope scopedProfilingUnitWithSize(s_profilingUnitWithSizeInstance); \
            s_profilingUnitWithSizeInstance.IncAmount(amount);
#endif  // end of _UNICODE

#endif  // end of NEX_DISABLE_USING_CTR_PROFILING_UNIT

