﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Platform/Core/AtomicValue.h>

#define ACQUIREREF(a) a->AcquireRef()
#define RELEASEREF(a) a->ReleaseRef()
BEGIN_NAMESPACE_NEX

/*!
 @brief    参照カウントオブジェクトの基底クラスです。

 参照カウントオブジェクトの基底クラスです。
 AcquireRef() で参照カウントを +1, ReleaseRef() で参照カウントを -1 し、
 カウントが 0 になるときに自分自身のデストラクタを呼びます。
 */
class RefCountedObject: public RootObject
{

public:
    RefCountedObject()
        : RootObject()
#ifdef NEX_DEBUG_VERSION
        , m_boTraceRefs(false)
#endif
        , m_RefCount(1)
        , m_bIsDeleting(false)
    {
    }
    virtual ~RefCountedObject()
    {
        SYSTEMCHECK(m_RefCount==0 || m_RefCount==1);
    }

    class Ref
    {
    private:
        void AcquireRef()
        {
            if (m_pObject!=NULL)
            {
                m_pObject->AcquireRef();
            }
        }
        void ReleaseRef()
        {
            if (m_pObject!=NULL)
            {
                m_pObject->ReleaseRef();
                m_pObject=NULL;
            }
        }
    public:
        Ref(RefCountedObject* pObject=NULL): m_pObject(pObject)
        {
        }
        Ref(const Ref& oRef): m_pObject(oRef.m_pObject)
        {
            AcquireRef();
        }
        ~Ref()
        {
            ReleaseRef();
        }

        const Ref& operator=(const Ref& oRef)
        {
            ReleaseRef();
            m_pObject=oRef.m_pObject;
            AcquireRef();
            return *this;
        }

        RefCountedObject* operator->()
        {
            return m_pObject;
        }
        const RefCountedObject* operator->() const
        {
            return m_pObject;
        }
    private:
        RefCountedObject* m_pObject;
    };

    /*!
     @brief    参照カウントを 1 増加させます。
     */
    inline RefCountedObject *AcquireRef()
    {
        ++m_RefCount;
        return this;
    }
    /*!
     @brief    参照カウントを 1 減少させます。0 になるときに自分自身を delete します。
     */
    inline void ReleaseRef()
    {
        qInt32 ref = (--m_RefCount);
        SYSTEMCHECK(ref >= 0);
        if (ref == 0 && !m_bIsDeleting)
        {
            m_bIsDeleting = true;
            qDelete this;
        }
    }

    inline void TraceRefs( qBool boTraceRefs )
    {
#ifdef NEX_DEBUG_VERSION
        m_boTraceRefs = boTraceRefs;
#else
        DUMMY_PARAM(boTraceRefs);
#endif
    }

    qInt32 GetRefCount() const
    {
        return m_RefCount.Get();
    }



private:
#ifdef NEX_DEBUG_VERSION
    qBool m_boTraceRefs;
#ifdef CTR_TARGET
    qByte padding1[3];  // to avoid compiler warning.
#endif
#endif

    AtomicValue m_RefCount;

    qBool m_bIsDeleting;
};


END_NAMESPACE_NEX

