﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDeclInternal.h>
#include <OnlineCore/src/Platform/Core/TraceLog.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>

///////////////////////////////////////////////////////////////////////////////
// IMPORTANT NOTE: A filter translates certain portions of this file into a Python
// script. This filter picks up only C-style comments (comments delimited by
// an opening slash/star and a closing star/slash) where the opening and closing
// delimiters must be alone on their line as seen below. The Python filter ignores
// C++ style comments (as seen on this line).
// Therefore, please note that the comments in this file have been carefully inserted
// so that the translation works as expected.
// See the results in ...src\Services\Constants\qResultErrors.py
///////////////////////////////////////////////////////////////////////////////

#ifdef NEX_SIGLO_FAMILY
#include <nn/nn_Result.h>
#include <nn/err/err_Types.h>
#endif

BEGIN_NAMESPACE_NEX

class String;

// group=Error Codes

// Description: A 32 bit integer that describes the return result.
// Remarks: The breakdown of the significance of each bit in the integer is as follows:
//
//  @ReturnResultTable
//  Bit number   Value
//  ----------   -----
//  0-15         The return code, which may be either a SuccessCode or an ErrorCode and is specific to a given facility
//  16-24        The facility (refer to qResultFacility for possible values)
//  31           0 if success, 1 if error
//
#define QRESULT_ERROR_BIT           0x80000000

//group=Error Code Enumerations

// Description: Enumerates the facilities from which a return result may originate.
/*!
 @brief    エラーコードのファシリティ用の名前空間です
 */
#ifdef SWIG
class qResultFacility
{
public:
#else
namespace qResultFacility
{
#endif

    /*
    Description: Enumerates the facilities from which a return result may originate.
    Remarks: A facility is the specific area of the system from which a return result may originate.
    Each facility defines its own success codes and error codes specific for that facility. Therefore,
    an error code of 1, for example, will mean one thing in the Core facility but may mean something
    completely different in the DDL or any other facility.

    You cannot modify the built-in Facilities. However, you may add an almost unlimited number of user-defined
    Facilities.
    */

    /*!
     @brief    qResult の発生元となるファシリティを定義する列挙子です

     ファシリティは、qResult の発生元となるシステムの特定のエリア(モジュール)を表します。
     各ファシリティは、そのファシリティ固有の独自の成功およびエラーコードを定義します。
     そのため、例えば、エラーコード１は Core ファシリティにおいてあるエラー要因を意味していても、
     DDL またはその他のファシリティでは、それとは完全に別のことを意味することがあります。

     既存のファシリティは変更することができませんが、ユーザー定義ファシリティを追加することは可能です。
     */
    enum Facilities
    {
        /*
        The origin of the return result is the $PRODUCTNAME$ core.
        The error and success codes of this facility are defined in CoreErrorCode and
        CoreSuccessCode respectively.
        */
        Core              = 1,    //!<    ファシリティ共通コード
        /*
        The origin of the return result is the DDL compiler.
        The error codes of this facility are defined in DDLErrorCode. Currently
        no success codes are defined.
        */
        DDL               = 2,    //!<    DDL コンパイラ
        // $ifdef RENDEZVOUS$
        /*
        The origin of the return result is Rendez-Vous specific code.
        The error codes of this facility are defined in RendezVousErrorCode. Currently
        no success codes are defined.
        */
        RendezVous        = 3,    //!<    NGS
        /*
        The origin of the return result is the Python scripts.
        The error codes of this facility are defined in PythonCoreErrorCode. Currently
        no success codes are defined.
        */
        PythonCore        = 4,    //!<    NGS（主にサーバー側で発生したエラー）
        //$endif$

        /*
        The origin of the return result is the network transport.
        The error codes of this facility are defined in TransportErrorCode. Currently
        no success codes are defined.
        */
        Transport         = 5,    //!<    通信処理起因のエラー
        //$ifdef NETZ$
        /*
        The origin of the return result is the duplicated object core.
        The error and success codes of this facility are defined in DOCoreErrorCode and
        DOCoreSuccessCode respectively.
        */
        DOCore            = 6,    //!<    NetZ 起因のエラー
        //$endif$

        //$ifdef RENDEZVOUS$
        /*
        The origin of the return result is Spark! specific code. The error codes
        of this facility are defined in SparkErrorCode which can be fond in the Spark! Reference Manual.
        Currently the success codes are defined in CoreSuccessCode.
        */
        Spark             = 8,    //!<    使用されません
        //$endif$

        //n1769
        TransportSocket   = 9,    //!<    ソケットレイヤのエラー


        ZZZAuthentication = 10,    //!<    使用されません
        Tracking = 11,    //!<    使用されません
        Telemetry = 12,    //!<    使用されません
        TitleStorage = 13,    //!<    使用されません
        Privilege = 14,    //!<    使用されません
        OnlineConfig = 15,    //!<    使用されません
        GameSession = 16,    //!<    使用されません
        Localization = 17,    //!<    使用されません
        News = 18,    //!<    使用されません
        PlayerStats = 19,    //!<    使用されません
        UserStorage = 20,    //!<    使用されません
        SkillRating = 21,    //!<    使用されません
        ZZZFriends = 22,    //!<    使用されません
        SubAccountManagement = 23,    //!<    使用されません
        Friends = 24,    //!<    使用されません


        LAST_QUAZAL_INTERNAL = 50,    //!<    使用されません

        /*
        The end of Quazal defined error.
        */
        FirstNintendoFacility = 101,    //!<    使用されません

        /*
        For Nintendo Friend and Presence
        The error codes of this facility are defined in FPDErrorCode.
        */
        FPD = 101,    //!<    フレンドプレゼンス

        /*
        For Nintendo Matchmaking
        */
        Matchmaking = 102,    //!<    使用されません

        /*
        For Nintendo Ranking APIs
        */
        Ranking = 103,    //!<    ストレージアカウント／ランキング

        /*
        For Nintendo Authentication Error (NAS and Nintendo specific RV login error)
        defined in AuthenticationErrorCode(AuthenticationErrors.h)
         */
        Authentication = 104,    //!<    認証処理

        /*
        For Nintendo DataStore APIs defined in DataStoreErrorCode.
        */
        DataStore = 105,    //!<    DataStore

        //{secret}
        Subscription = 106,

        //{secret}
        Rating = 107,

        /*
        For Nintendo ServiceItem APIs defined in ServiceItemErrorCode.
        */
        ServiceItem = 108,    //!<    ServiceItem

        //{secret}
        Shop = 109,

        //{secret}
        MusicRequest = 110,

        /*
        For Nintendo MatchmakeReferee APIs defined in MatchmakeRefereeErrorCode.
        */
        MatchmakeReferee = 111, //!<    MatchmakeReferee

        /*
        For Nintendo Subscriber APIs defined in SubscriberErrorCode.
        */
        Subscriber = 112, //!<    Subscriber

        /*
        For Nintendo Ranking2 APIs defined in Ranking2ErrorCode.
        */
        Ranking2 = 113, //!<    Ranking2

        /*
        For Nintendo Coral APIs defined in CoralErrorCode.
        */
        Coral = 114, //!<    Coral

        //{secret}
        LAST_INTERNAL     = 114,    //!<    使用されません

        /*
        The ID of the first user-defined facility, which by default is equal to 200.
        */
        FirstUserFacility = 200    //!<    ユーザー定義のファイシティを使用する場合はこれ以降を使用してください
    };
#ifdef SWIG
};
#else
}
#endif

//TODO: This macro is misused (only once) and misnamed and does the same as QERROR
// {secret}
// Internal macro used to change the facility of a qResult.
#define QRESULT_SET_FACILITY(code,facilityID) (code & 0x8000FFFF) | (facilityID << 16)

//group=Error Code Enumerations

// Description: This enumeration defines the success codes that may be returned for the Core facility.
/*!
 @brief    Core モジュールのサクセスコードを定義する名前空間です。
 */
#ifdef SWIG
class CoreSuccessCode
{
public:
#else
namespace CoreSuccessCode
{
#endif

    /*
    Description: Enumerates the success codes for the Core facility that an operation may return.
    Remarks: Details as to the cause of each SuccessCode are detailed in the documentation
    of each individual $PRODUCTNAME$ method that may return a result.
    */

    /*!
     @brief    Core モジュールのサクセスコードです。
     */
    enum SuccessCode
    {
        /*
        The operation completed successfully.
        */
        Success                     = 1,    //!<    成功
        /*
        Success of the operation is pending, /i.e./ the operation is still in progress and has not
        yet returned one of the other SuccessCode s nor a CoreErrorCode.
        */
        SuccessPending              = 2    //!<    処理が進行中。成功でもエラーでもない状態です。
    };
#ifdef SWIG
};
#else
}
#endif

//group=Error Code Enumerations

// Description: This enumeration defines the error codes that may be returned for the Core facility.
/*!
 @brief    Core モジュールのエラーコードを定義する名前空間です。
 */
#ifdef SWIG
class CoreErrorCode
{
public:
#else
namespace CoreErrorCode
{
#endif

    /*
    Description: Enumerates the error codes for the Core facility that a call may return.
    Remarks: Details as to the cause of each ErrorCode are detailed in the documentation
    of each individual $PRODUCTNAME$ method that may return a result.
    */

    /*!
     @brief    Core モジュールのエラーコードです。
     */
    enum ErrorCode
    {
        START = 1,
        /*
        The reason for the error is unknown.
        */
        Unknown                     = 1,    //!<    定義されていないエラー
        /*
        The operation is currently not implemented.
        */
        NotImplemented              = 2,    //!<    未実装
        /*
        The operation specifies or accesses an invalid pointer.
        */
        InvalidPointer              = 3,    //!<    不正なポインタ
        /*
        The operation was aborted.
        */
        OperationAborted            = 4,    //!<    処理が中断された
        /*
        The operation raised an exception. Note that Rendez-Vous does not log traces for exceptions
        that are registered with the python script engine (qResult and data base exceptions).
        On the other hand, if an unexpected exception occurs (an assert, for example), Rendez-Vous
        dumps the call stack before returning the exception.
        */
        Exception                   = 5,    //!<    例外発生
        /*
        An attempt was made to access data in an incorrect manner. This may be due to
        inadequate permission or the data, file, /etc./ not existing.
        */
        AccessDenied                = 6,    //!<    アクセスが許可されていない
        /*
        The operation specifies or accesses an invalid DOHandle.
        */
        InvalidHandle               = 7,    //!<    不正な DOHandle
        /*
        The operation specifies or accesses an invalid index.
        */
        InvalidIndex                = 8,    //!<    不正な index
        /*
        The system could not allocate or access enough memory or disk space to perform the specified operation.
        */
        OutOfMemory                 = 9,    //!<    メモリ不足
        /*
        Invalid argument were passed with the operation. The argument(s) may be out of range or invalid.
        */
        InvalidArgument             = 10,    //!<    不正な引数
        /*
        The operation did not complete within the specified timeout for that operation.
        */
        Timeout                     = 11,    //!<    タイムアウトで終了
        /*
        Initialization of the component failed.
        */
        InitializationFailure       = 12,    //!<    初期化に失敗
        /*
        The call failed to initialize.
        */
        CallInitiationFailure       = 13,    //!<    非同期処理の初期化に失敗
        /*
        An error occurred during registration.
        */
        RegistrationError           = 14,    //!<    レジストレーションに失敗（NEXではレジストレーションは不要です）
        /*
        The buffer is too large to be sent.
        */
        BufferOverflow              = 15,    //!<    送信できないサイズのバッファ
        /*
        A lock is needed for a perticular process but it is already locked (the lock
        should be released beforehand) or not already locked (we cannot take the
        lock now because it might cause deadlocks).
        */
        InvalidLockState            = 16,    //!<    不正なロック状態（ロックが必要な状態で別のスレッドがロックしているなど）
        /** Invalid sequence of method calls. */
        InvalidSequence             = 17,    //!<    不正なシーケンス
        /** System error. */
        SystemError                 = 18,    //!<    システムエラー
        Cancelled                   = 19     //!<    キャンセルされた
     };

    //note: 新規result追加時には、OnlineCore/src/Platform/Core/ResulDefine.hのNEX_ERROR_RESULT_DEFINEの変更も必要
    //Wiki エラーコード追加時に行う作業 を参照
    //アプリケーションがハンドリングするべきエラーを増やした場合には、commonmanualのoverall.rstの「アプリケーションがハンドリングするべきエラー一覧」に追加すること
#ifdef SWIG
};
#else
}
#endif

//group=Error Code Macros

/*!
 @defgroup    nex_define nex_define
 @{
 */
//Description: Tests if the operation result code is a success.
//Remarks: This macro is typically used in an "if" statement.

/*!
 @brief    リターンコードが成功を示すものかどうかを返します。

 @param[in]    RETURNCODE    qResult::GetReturnCode() で取得できるリターンコード
 */
#define QSUCCEEDED(RETURNCODE)  ((RETURNCODE & QRESULT_ERROR_BIT) == 0)
//Description: Tests if the operation result code is an error.
//Remarks: This macro is typically used in an "if" statement.
/*!
 @brief    リターンコードが失敗を示すものかどうかを返します。

 @param[in]    RETURNCODE    qResult::GetReturnCode() で取得できるリターンコード
 */
#define QFAILED(RETURNCODE)     ((RETURNCODE & QRESULT_ERROR_BIT) != 0)
//Description: Builds a specific error code.
//Remarks: This macro is typically used in the "case" statement of a "switch" statement.
/*!
 @brief    ファシリティとエラー列挙子からリターンコードを生成します

 @param[in]    FACILITY    qResultFacilityに定義されている列挙子。
 @param[in]    RETURNCODE    ファシリティ名+ErrorCode に定義されている列挙子。
 */
#define QERROR(FACILITY,RETURNCODE) (NAMESPACE_NEX qInt32)(QRESULT_ERROR_BIT | NAMESPACE_NEX qResultFacility::FACILITY<<16 | NAMESPACE_NEX FACILITY##ErrorCode::RETURNCODE)
//Description: Builds a specific success code.
//Remarks: This macro is typically used in the "case" statement of a "switch" statement.
/*!
 @brief    ファシリティと成功列挙子からリターンコードを生成します

 @param[in]    FACILITY    qResultFacilityに定義されている列挙子。
 @param[in]    RETURNCODE    ファシリティ名+SuccessCode に定義されている列挙子。
 */
#define QSUCCESS(FACILITY,RETURNCODE) (NAMESPACE_NEX qInt32)(NAMESPACE_NEX qResultFacility::FACILITY<<16 | NAMESPACE_NEX FACILITY##SuccessCode::RETURNCODE)


#ifdef NEX_DEBUG_VERSION
//Description: Accesses an error return result as a qResult.
// Parameters:
// FACILITY: The qResultFacility from which the error originates.
// RETURNCODE: The error Return Codes.
//Remarks: The file name and line number where the error occurred are only returned when this macro is used in
//Debug mode.
#define QRESULT_ERROR(FACILITY,RETURNCODE) NAMESPACE_NEX qResult(QERROR(FACILITY,RETURNCODE), NEX_T(__FILE__), __LINE__)
//Description: Accesses a success return result as a qResult.
// Parameters:
// FACILITY: The qResultFacility from which the success result originates.
// RETURNCODE: The success Return Codes.
//Remarks: The file name and line number where the error occurred are only returned when this macro is used in
//Debug mode.
#define QRESULT_SUCCESS(FACILITY,RETURNCODE) NAMESPACE_NEX qResult(QSUCCESS(FACILITY,RETURNCODE), NEX_T(__FILE__), __LINE__)
#else
/*!
 @brief    ファシリティとエラー列挙子からqResultを生成します

 @param[in]    FACILITY    qResultFacilityに定義されている列挙子。
 @param[in]    RETURNCODE    ファシリティ名+ErrorCode に定義されている列挙子。
 */
#define QRESULT_ERROR(FACILITY,RETURNCODE) NAMESPACE_NEX qResult(QERROR(FACILITY,RETURNCODE))
/*!
 @brief    ファシリティと成功列挙子からqResultを生成します

 @param[in]    FACILITY    qResultFacilityに定義されている列挙子。
 @param[in]    RETURNCODE    ファシリティ名+SuccessCode に定義されている列挙子。
 */
#define QRESULT_SUCCESS(FACILITY,RETURNCODE) NAMESPACE_NEX qResult(QSUCCESS(FACILITY,RETURNCODE))
#endif // NEX_DEBUG_VERSION
    
    
#ifdef NEX_SIGLO_FAMILY
/*!
 @brief    ファシリティと成功列挙子からnn::Resultを継承したエラークラス名を生成します
    
 nn::nex:: Result + FACILITY + RETURNCODE というクラス名を生成します。
 
 例: NEX_NNRESULT(Core, OperationAborted)::%Includes(result) = nn::nex::ResultCoreOperationAborted::%Includes(result)

 @param[in]    FACILITY    qResultFacilityに定義されている列挙子。
 @param[in]    RETURNCODE    ファシリティ名+SuccessCode に定義されている列挙子。

 @see nn::nex::qResult::GetNnResult(), nnResult
*/    
#define NEX_NNRESULT(FACILITY, RETURNCODE) NAMESPACE_NEX Result##FACILITY##RETURNCODE
    
#endif

/*!
 @}
 */

//group=Return Results Classes

// Description: This class defines the result that a call may return.
//
//   Remarks: The results that are returned by this class are defined in the ErrorCode and SuccesCode
//   classes defined for each specific facility as detailed in the table below. Refer to these classes
//   for details as to the individual error and success codes that each facility may return. A return code
//   is a 32 bit integer defined by QRESULT_ERROR_BIT. Note that the return result is accessed via one of
//   the QRESULT macros.
//
//  $ifdef RENDEZVOUS$
//    @table
//    qResultFacility       ErrorCode               SuccessCode
//    /Core/                CoreErrorCode           CoreSuccessCode
//    /DDL/                 DDLErrorCode            ---
//    /PythonCore/          PythonCoreErrorCode     ---
//    /RendezVous/          RendezVousErrorCode     ---
//    /Transport/           TransportErrorCode      ---
//
//  $else$
//    @table
//    qResultFacility       ErrorCode               SuccessCode
//    /Core/                CoreErrorCode           CoreSuccessCode
//    /DDL/                 DDLErrorCode            ---
//    /DOCore/              DOCoreErrorCode         DOCoreSuccessCode
//    /Transport/           TransportErrorCode      ---
//  $endif$
//
// Note that as the qResult class uses some explicit constructors the following call is invalid:
//
//              // This will cause an error
//              qResult result1 = true;
//
// Instead, you may call either of the  following:
//
//              // One possibility
//              qResult result1(true);
//              // Another way to do the same thing
//              qResult result1;
//              result1 = true;
//
//  In addition, as the qResult class does not support implicit conversion, the macros that return a qResult
// always need to be prefixed with QRESULT. For example:
//
//             // This call will produce an error
//             return QSUCCESS(Core,Success);
//             // This is the correct way to call the macro
//             return QRESULT_SUCCESS(Core,Success);
//
//
// Example:
//
//          qResult AccountManagementHelper::CreatePlayer(const String &strPlayerName, const String &strPassword,
//                                                        const String &strEmail) {
//                  AccountManagementClient oAccountManagementClient;
//                  oAccountManagementClient.Bind(pGuest);
//                  ProtocolCallContext ccCreateAccount;
//
//                  if (!oAccountManagementClient.CreateAccount(&ccCreateAccount, &rResult, strPlayerName,
//                                                              strPassword, BuiltInGroups::Players, strEmail)) {
//                      return QRESULT_ERROR(Core, CallInitiationFailure);
//                  }
//                  if (!ccCreateAccount.Wait(5000)) {
//                      return QRESULT_ERROR(Core, Timeout);
//                  }
//                  return rResult;
//          }
//
/*!
@brief    NEXライブラリが返す処理の結果を示すクラスです。

このクラスによって返される結果は、各特定のファシリティに定義されているErrorCodeおよびSuccesCodeで定義されています。
各ファシリティが返す可能性のある個々のエラーおよび成功コードに関する詳細については、そちらを参照してください。


qResult::GetReturnCode() で取得できる値を直接エラー・EULAアプレットに渡さないでください。
qResult からネットワークエラーコードに変換するには ErrorCodeConverter を使用してください。



@see ErrorCodeConverter

*/
class qResult : public RootObject
{
public:
    //group= Methods

    /*!
     @refmanoverload{name1}
     @brief    コンストラクタです。
     */
    qResult();
    /*!
     @refmanoverload{name2}
     @brief    コンストラクタです。
     @param[in]    iResult    qResult の 32 bit 値
     */
    explicit qResult(const qInt32& iResult);
    /*!
     @refmanoverload{name3}
     @brief    コンストラクタです。
     @param[in]    bResult    true の場合、QSUCCESS(Core, Success), false の場合、QERROR(Core, Unknown) がセットされます。
     */
    explicit qResult(const qBool& bResult);

    qResult(const qInt32& iResult, const qChar *szFilename, const qInt32& iLineNumber);

    /*!
     @brief    ファシリティを取得します。
     @return   ファシリティ
     */
    qResultFacility::Facilities GetFacility() const;

    // {secret}
    qBool Equals(const qInt32& iResult) const;
    // {secret}
    qBool Equals(const qBool& bResult) const;

private:
    // Safe bool idiom
    // 意図しないboolへの変換を禁止した安全なboolへの変換
    // https://ja.wikibooks.org/wiki/More_C%2B%2B_Idioms/%E5%AE%89%E5%85%A8%E3%81%AA_bool%28Safe_bool%29
    typedef void (qResult::*bool_type)() const;
    void this_type_does_not_support_comparisons() const {}
public:
    operator bool_type() const
    {
        // QRESULT_ERROR_BITがたっていないときのみtrueを返す
        return (m_iReturnCode & QRESULT_ERROR_BIT) == 0 ? &qResult::this_type_does_not_support_comparisons : 0;
    }
#ifdef FOR_DOC_PURPOSE
    // Safe bool idiomをそのまま関数リファレンスに出すと煩雑になるので、簡易的なシグネチャのものをドキュメントに出す。
    /*!
     @brief    サクセスであれば true, エラーであれば false を返します。
     @retval    true    サクセス
     @retval    false   エラー
     */
    operator qBool () const;
#endif

    /*!
       @refmanoverload{name1}
       @brief    代入演算子です
     */
    qResult& operator=(const qBool& bEq);
    /*!
       @refmanoverload{name2}
       @brief    代入演算子です
     */
    qResult& operator=(const qInt32& uiEq);
    /*!
       @refmanoverload{name3}
       @brief    代入演算子です
     */
    qResult& operator=(const qUnsignedInt32& uiEq);
    /*!
       @refmanoverload{name4}
       @brief    代入演算子です
     */
    qResult& operator=(const qResult& oEq);

    /*!
     @brief    比較演算子(等号)です

     @param[in]    rhs    比較対象
     @retval    true    リターンコードが完全に一致する場合
     @retval    false    リターンコードが一致しない場合
     */
    bool operator==(const qResult& rhs) const
    {
        return m_iReturnCode == rhs.m_iReturnCode;
    }

    /*!
     @brief    比較演算子(不等号)です

     @param[in]    rhs    比較対象
     @retval    true    リターンコードが一致しない場合
     @retval    false    リターンコードが完全に一致する場合
     */
    bool operator!=(const qResult& rhs) const
    {
        return m_iReturnCode != rhs.m_iReturnCode;
    }

    //Description: Returns the result return code as a qInt32.
    //Returns: The result return code as a qInt32.
    //Remarks: As any given method may return a variety of error or success codes, if you wish to test the qResult
    // of a method we recommend that you test the qResult against success or error ( /i.e./ the last bit of the
    // QRESULT_ERROR_BIT) rather than specific Return Codes. This ensures that if the return code of the method
    // is changed, your code will still function correctly.
    /*!
     @brief    リターンコードを取得します

     qResult の種別を一意に識別するコードを取得します。この値を直接エラー・EULAアプレットに渡さないでください。

     @return    qResult の種別を一意に識別するコード
     */
    qInt32 GetReturnCode() const
    {
        return m_iReturnCode;
    }

    //n1769
    const qChar* GetFileName() const
    {
        return m_cszFilename;
    }

    //n1769
    qInt32 GetLineNumber() const
    {
        return m_iLineNumber;
    }

    //1769
    /*!
     @brief    qResultのエラー名をC文字列として取得します（デバッグ用）

     qResult::GetReturnCodeString() と同じ結果を返します。
     これは互換性のために残されている古い関数です。 qResult::GetReturnCodeString() を使用して下さい。

     @return    NUL終端する文字列の先頭を指すポインタ。
     */
    const qChar* GetErrorName() const;

    /*!
     @brief    リターンコードの文字列表記をC文字列として取得します（デバッグ用）

     成功のリターンコードの場合は "qResult Success Return Code"が、
     存在しないリターンコードの場合は "Unknown qResult Return Code"が返ります。

     @return    NUL終端する文字列の先頭を指すポインタ。
     */
    const qChar* GetReturnCodeString() const;

    //n1769: qResultの深刻度を判断するための関数。
    // ただし、これらの関数は基本的にNEX P2Pに関わるqResult値しか
    // カバーできていません。
    qBool IsError(void) const;
    qBool IsWarning(void) const;
    qBool IsReinitializeRequired(void) const;
    qBool IsResetRequired(void) const;
    qBool IsProgrammingError(void) const;

    /* Description: Traces the content of this Result object.
       Remarks: This method traces the result return code, file origin, and line number. Note that Rendez-Vous
       does not log traces for exceptions that are registered with the python script engine
       (qResult and data base exceptions). If an unexpected exception occurs
       (an assert, for example), however, Rendez-Vous dumps the call stack before returning the exception.
    */

    /*!
        @brief      qResult オブジェクトの内容をトレースします。

        リターンコードの数値と文字列表記を出力します。
        出力されるリターンコードは qResult の種別を一意に識別するコードであり、エラーコードではありません。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
#ifdef NEX_TRACE_ENABLED    // n2370
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const;
#else
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const { (void)uiTraceFlags; }
#endif  // NEX_TRACE_ENABLED

    // {secret}
    void Format(String* pstrDescription) const;

    friend StringStream& operator<<(StringStream& oStream, const qResult& rhs);
    
#ifdef NEX_SIGLO_FAMILY
    /*!
        @brief  qResult オブジェクトの内容を nn::Result オブジェクトに変換します。
    
        @return nn::Result オブジェクト

        @see GetErrorCodeStruct(), NEX_NNRESULT(), nnResult
    */
    nn::Result GetNnResult() const;

    /*!
        @brief  qResult オブジェクトの内容を nn::Result オブジェクトに変換します。

        @return nn::Result オブジェクト

        @see GetNnResult(), NEX_NNRESULT(), nnResult
    */
    operator nn::Result () const { return GetNnResult(); }

    /*!
        @brief  qResult オブジェクトの内容を nn::err::ErrorCode 構造体に変換します。

        @return nn::err::ErrorCode 構造体

        @see GetNnResult(), NEX_NNRESULT(), nnResult
    */
    nn::err::ErrorCode GetErrorCodeStruct() const;
#endif
    
    
    
private:
    qInt32      m_iReturnCode;
    const qChar *m_cszFilename;
    qInt32      m_iLineNumber;
};

// String streaming.
// {secret}
extern StringStream& operator<<(StringStream& oStream, const qResult& rhs);

//{secret}
template <typename T>
bool operator == (const T& lhs, const qResult& rhs)
{
    return rhs.Equals(lhs);
}
//{secret}
template <typename T>
bool operator == (const qResult& lhs, const T& rhs)
{
    return lhs.Equals(rhs);
}
//{secret}
template <typename T>
bool operator != (const T& lhs, const qResult& rhs)
{
    return !rhs.Equals(lhs);
}
//{secret}
template <typename T>
bool operator != (const qResult& lhs, const T& rhs)
{
    return !lhs.Equals(rhs);
}

END_NAMESPACE_NEX
