﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/qVector.h>

/** Lock-Free Ring Buffer
 * This ring buffer is implemented without any luck so that it can be used
 * between two threads without lock, as long as one thread only push to it and
 * one thread only pop from it at any time.  This can allow also a lock-free consumer
 * with lock-stepped multiple producers or similar schemes.
 *
 */
template <typename T>
class RingBuffer : public RootObject
{
public:
    explicit RingBuffer(qUnsignedInt32 size)
        : m_vector(size+1)
        , m_read(0)
        , m_write(0)
    {
        m_vector.resize(size+1);
    }
    qBool Pop(T& elem)
    {
        if (IsEmpty())
        {
            return false;
        }
        elem = m_vector[m_read];
        if (++m_read >= m_vector.size())
        {
            m_read = 0;
        }
        return true;
    }
    qBool Push(const T& elem)
    {
        if (IsFull())
        {
            return false;
        }
        m_vector[m_write] = elem;
        if (++m_write >= m_vector.size())
        {
            m_write = 0;
        }
        return true;
    }
    qBool IsFull() const
    {
        return !IsEmpty() && GetCapacity() == GetSize();
    };
    qBool IsEmpty() const
    {
        return m_read == m_write;
    };
    size_t GetSize() const
    {
        if (m_write < m_read)
        {
            return m_vector.size() - m_read + m_write;
        }
        return m_write - m_read;
    }
    size_t GetCapacity() const
    {
        return m_vector.size() - 1;
    }
private:
    qVector<T> m_vector;
    qUnsignedInt32 m_read;
    qUnsignedInt32 m_write;
};

