﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/CriticalSection.h>


BEGIN_NAMESPACE_NEX

// Group=Miscellaneous Classes

/*
Description: This class scopes the use of a CriticalSection.
Remarks: This class scopes a CriticalSection in the sense that when the ScopedCS constructor
or destructor is called, the ScopedCS object respectively enters or leaves the specified CriticalSection.
As the destructor is called when the block where the constructor was called is exited, using this class is
a good way to avoid forgetting to leave a CriticalSection whereby permanently locking it.
Refer to Scheduler::SystemLock for an example of how to use a ScopedCS.
See Also: CriticalSection
*/

/*!
@brief    スコープ内に限定した @ref CriticalSection です。

コンストラクタで引数で渡された @ref CriticalSection に入り、デストラクタで @ref CriticalSection から抜けます。
@ref CriticalSection から出るのを忘れてデッドロックに入るのを避けるには、このクラスの使用が有効です。

@see     CriticalSection
*/
class ScopedCS: public RootObject
{
    // Group=Methods
public:
    /*
    Parameters:
    pCS: A pointer to the CriticalSection that will be scoped.
    Remarks: When the ScopedCS constructor is called, it enters the CriticalSection specified
    via pCS.
    See Also: CriticalSection::Enter
    */

    /*!
    @brief    コンストラクタです。

    @ref ScopedCS  コンストラクタが呼び出されると、pCSによって指定された @ref CriticalSection に入ります。

    @param[in]    pCS    範囲指定される @ref CriticalSection への参照。

    @see     CriticalSection::Enter
    */
    ScopedCS(CriticalSection &pCS) :m_pCS(&pCS)
    {
        m_pCS->Enter();
    }

    /*
    Remarks: When the ScopedCS destructor is called, it leaves the CriticalSection specified
    upon construction.
    See Also: CriticalSection::Leave
    */

    // デストラクタは非virtualなので継承禁止
    /*!
    @brief    デストラクタです。

     @ref ScopedCS  デストラクタが呼び出されると、コンストラクタ時に指定された @ref CriticalSection から抜けます。

    @see     CriticalSection::Leave
    */
    ~ScopedCS()
    {
        m_pCS->Leave();
    }
private:
    CriticalSection *m_pCS;
};

/*!
@internal
@brief    スコープ内に限定した @ref CriticalSection です。

コンストラクタで引数で渡された @ref CriticalSection に入り、デストラクタで @ref CriticalSection から抜けます。
@ref CriticalSection から出るのを忘れてデッドロックに入るのを避けるには、このクラスの使用が有効です。
ただし、コンストラクタで与えた pCS が NULL の場合は @ref CriticalSection に入りません。

@see     CriticalSection
*/
class ScopedPtrCS: public RootObject
{
public:
    /*!
    @brief    コンストラクタです。

    コンストラクタが呼び出されると、pCSによって指定された @ref CriticalSection に入ります。
    ただし、pCS が NULL の場合は @ref CriticalSection に入りません。

    @param[in]    pCS    範囲指定される @ref CriticalSection への参照。

    @see     CriticalSection::Enter
    */
    ScopedPtrCS(CriticalSection* pCS)
        :m_pCS(pCS)
    {
        if (m_pCS)
        {
            m_pCS->Enter();
        }
    }

    // デストラクタは非virtualなので継承禁止
    /*!
    @brief    デストラクタです。

    デストラクタが呼び出されると、コンストラクタに指定された @ref CriticalSection から抜けます。
    ただし、コンストラクタで与えた pCS が NULL の場合は何も行いません。

    @see     CriticalSection::Leave
    */
    ~ScopedPtrCS()
    {
        if (m_pCS)
        {
            m_pCS->Leave();
        }
    }
private:
    CriticalSection *m_pCS;
};

END_NAMESPACE_NEX

