﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/StringStream.h>
#include <OnlineCore/src/Platform/Core/TimeDefs.h>
#ifdef UNIX_FAMILY
#include <sys/time.h>
#include <unistd.h>
#endif

//{secret}
BEGIN_NAMESPACE_NEX

class TimeProvider;

/*
$ifdef VOICE$
{secret}
$else$

Description: Expresses a time in milliseconds.
Remarks: The SystemClock and SessionClock classes can be used to get Time values.
The following example sets the value of the variable /tNow/ to
the number of milliseconds elapsed since the session started.

    Time tNow = SessionClock::GetTime();

See also: SystemClock, SessionClock, TimeInterval
$endif$
*/

/*!
 * @brief   ミリ秒単位での時間を表すクラスです。
 *
 * @ref SystemClock と@ref SessionClock クラスにて、時間の値を取得するためにこのクラスを利用できます。
 */
class Time: public RootObject
{
public:
    //Group=Methods

    /*
    Parameters:
    ui64Value: The initial time.
    tTime: The initial time.
    Remarks: The copy constructor, Time(const Time& tTime), is not valid on Win32 platforms.
    */

    /*!
     * @refmanoverload{name1}
     *
     * @brief   オブジェクトを初期化します。
     */
    Time() : m_ui64Value(0) {}

    /*!
     * @brief   オブジェクトを初期化します。
     * @param   ui64Value   [in] 時刻の初期値を64bitで指定します。
     */
    Time(const qUnsignedInt64 ui64Value) : m_ui64Value(ui64Value) {}

#ifdef UNIX_FAMILY

    // WARNING: This constructor was removed from win32 platform
    // since it reveals a bug introduced in VC++ SP4.  See Bug 211 for
    // details
    /*!
     * @refmanoverload{name2}
     *
     * @brief   オブジェクトを初期化します。
     * @param   tTime       [in] コピー元のオブジェクトを指定します。
     */
    Time(const Time& tTime)
    {
        m_ui64Value = tTime.m_ui64Value;
    }

    // {secret}
    Time(const struct timeval oTimeval);
    // {secret}
    Time(const struct timespec oTimeSpec);

    // {Secret}
    void GetTime(struct timespec &oTimeSpec) const;
    // {secret}
    void GetTime(struct timeval &oTimevla) const;
#endif

    //n1589:memo:returns milli
    // {secret}
    /*!
     @brief ミリ秒単位の時間を表すTimeオブジェクトを取得します。

     SystemClock::GetTime()の呼び出しと同じです。
     時間取得の精度やパフォーマンスは、タイムプロバイダの性能に左右されます。
     時間取得の詳細については、@ref SystemClock を参照してください。

     @return ミリ秒単位の時間を表すTimeオブジェクトを返します。
     @see SystemClock::GetTime()
     */
    static Time GetTime();
    // {secret}
    static void Reset();
    // {secret}
    static Time GetSessionTime()
    {
        if (s_pfGetSessionTime!=NULL)
        {
            return s_pfGetSessionTime();
        }
        else
        {
            return 0;
        }
    }

    // {secret}
    static void GetFrequency(Time* pliFrequency)
    {
        *pliFrequency=FREQUENCY;
    }
    static qUnsignedInt64 GetFrequency()
    {
        return FREQUENCY;
    }

    /*! @refmanoverload{name1}
     @brief 代入演算子です。
     @param[in] tTime Timeオブジェクト。
     @return 代入後のTimeオブジェクトが返ります。
     */
    Time& operator=(const Time& tTime)
    {
        m_ui64Value=tTime.m_ui64Value;
        return *this;
    }
    /*! @refmanoverload{name2}
     @brief 代入演算子です。 64bit整数値をTimeオブジェクトに変換して代入します。
     @param[in] ui64Value 64bit整数値。
     @return 代入後のTimeオブジェクトが返ります。
     */
    Time& operator=(qUnsignedInt64 ui64Value)
    {
        m_ui64Value=ui64Value;
        return *this;
    }
    /*! @refmanoverload{name1}
     @brief Timeオブジェクトを符号なし64bit整数値として返します。
     @return 符号なし64bit整数値が返ります。
     */
    operator qUnsignedInt64() const
    {
        return m_ui64Value;
    }
    operator qUnsignedInt64()
    {
        return m_ui64Value;
    }

    /*! @refmanoverload{name1}
     @brief 減算演算子です。
     @param[in] tiTime 減算する値を指定します。
     @return 減算後のTimeオブジェクトが返ります。
     */
    Time operator-(TimeInterval tiTime) const
    {
        if (tiTime>0)
        {
#ifdef NEX_DEBUG_VERSION
            if (m_ui64Value>0)
            {
                SYSTEMCHECK(m_ui64Value>=(qUnsignedInt64)tiTime);
            }
#endif
            return m_ui64Value-tiTime;
        }
        else
        {
            // CodeWarrior promotes the tiTime to Unsigned prior to computing the -.
            return m_ui64Value+(-tiTime);
        }
    }

    /*! @refmanoverload{name1}
     @brief 加算演算子です。
     @param[in] tiTime 加算する値を指定します。
     @return 加算後のTimeオブジェクトが返ります。
     */
    Time operator+(TimeInterval tiTime) const
    {
        if (tiTime<0)
        {
#ifdef NEX_DEBUG_VERSION
            // We check if the operation is valid.
            if (tiTime==-tiTime)
            {
                // If tiTime is 0x80000000, the tiTime==-tiTime, and we cannot use the
                // regular check.
                SYSTEMCHECK(m_ui64Value>=0x8000000);
            }
            else
            {
                SYSTEMCHECK(m_ui64Value>=(qUnsignedInt64)-tiTime);
            }
#endif
            return m_ui64Value-(-tiTime);
        }
        else
        {
            return m_ui64Value+tiTime;
        }
    }

    /*! @refmanoverload{name2}
     @brief 減算演算子です。
     @param[in] tiTime 減算する値を指定します。
     @return 減算後のTimeオブジェクトが返ります。
     */
    Time operator-(LargeTimeInterval tiTime) const
    {
#ifdef NEX_DEBUG_VERSION
        if (tiTime>0 && m_ui64Value>0)
        {
            SYSTEMCHECK(m_ui64Value>=(qUnsignedInt64)tiTime);
        }
#endif
        return m_ui64Value-tiTime;
    }

    /*! @refmanoverload{name2}
     @brief 加算演算子です。
     @param[in] tiTime 加算する値を指定します。
     @return 加算後のTimeオブジェクトが返ります。
     */
    Time operator+(LargeTimeInterval tiTime) const
    {
#ifdef NEX_DEBUG_VERSION
        if (tiTime<0)
        {
            if (tiTime==-tiTime)
            {
                // for the lowest negative value, tiTime==-tiTime, and we cannot use the
                // regular check.
                // TODO... We should add a test, to be really safe...
            }
            else
            {
                SYSTEMCHECK(m_ui64Value>=qUnsignedInt64(-tiTime));
            }
        }
#endif
        return m_ui64Value+tiTime;
    }
    /*! @refmanoverload{name1}
     @brief 減算代入演算子です。
     @param[in] tiTime 減算する値を指定します。
     @return 減算後のTimeオブジェクトが返ります。
     */
    Time& operator-=(TimeInterval tiTime)
    {
        m_ui64Value-=tiTime;
        return *this;
    }
    /*! @refmanoverload{name1}
     @brief 加算代入演算子です。
     @param[in] tiTime 加算する値を指定します。
     @return 加算後のTimeオブジェクトが返ります。
     */
    Time& operator+=(TimeInterval tiTime)
    {
        m_ui64Value+=tiTime;
        return *this;
    }
    /*! @refmanoverload{name2}
     @brief 減算代入演算子です。
     @param[in] tiTime 減算する値を指定します。
     @return 減算後のTimeオブジェクトが返ります。
     */
    Time& operator-=(LargeTimeInterval tiTime)
    {
        m_ui64Value-=tiTime;
        return *this;
    }
    /*! @refmanoverload{name2}
     @brief 加算代入演算子です。
     @param[in] tiTime 加算する値を指定します。
     @return 加算後のTimeオブジェクトが返ります。
     */
    Time& operator+=(LargeTimeInterval tiTime)
    {
        m_ui64Value+=tiTime;
        return *this;
    }

    /*! @refmanoverload{name3}
     @brief 減算演算子です。
     @param[in] tTime 減算する値を持つTimeオブジェクト値を指定します。
     @return 減算後の値を64bit整数値で返します。
     */
    LargeTimeInterval operator-(const Time& tTime) const
    {
        // (符号無し整数 - 符号無し整数)におけるラップアラウンドは定義済み動作なので問題ない
        LargeTimeInterval tiResult = static_cast<LargeTimeInterval>(m_ui64Value - tTime.m_ui64Value);
        SYSTEMCHECK(m_ui64Value == qUnsignedInt64(tTime.m_ui64Value + tiResult));
        return tiResult;
    }

    /*! @refmanoverload{name1}
     @brief 比較演算子です。
     @param[in] tTime 比較するTimeオブジェクトを指定します。
     @return 比較結果を返します。
     */
    qBool operator==(const Time& tTime) const
    {
        return m_ui64Value==tTime.m_ui64Value;
    }
    /*! @refmanoverload{name1}
     @brief 比較演算子です。
     @param[in] tTime 比較するTimeオブジェクトを指定します。
     @return 比較結果を返します。
     */
    qBool operator!=(const Time& tTime) const
    {
        return m_ui64Value!=tTime.m_ui64Value;
    }
    /*! @refmanoverload{name1}
     @brief 比較演算子です。
     @param[in] tTime 比較するTimeオブジェクトを指定します。
     @return 比較結果を返します。
     */
    qBool operator<(const Time& tTime) const
    {
        return m_ui64Value<tTime.m_ui64Value;
    }
    /*! @refmanoverload{name1}
     @brief 比較演算子です。
     @param[in] tTime 比較するTimeオブジェクトを指定します。
     @return 比較結果を返します。
     */
    qBool operator>(const Time& tTime) const
    {
        return m_ui64Value>tTime.m_ui64Value;
    }
    /*! @refmanoverload{name1}
     @brief 比較演算子です。
     @param[in] tTime 比較するTimeオブジェクトを指定します。
     @return 比較結果を返します。
     */
    qBool operator<=(const Time& tTime) const
    {
        return m_ui64Value<=tTime.m_ui64Value;
    }
    /*! @refmanoverload{name1}
     @brief 比較演算子です。
     @param[in] tTime 比較するTimeオブジェクトを指定します。
     @return 比較結果を返します。
     */
    qBool operator>=(const Time& tTime) const
    {
        return m_ui64Value>=tTime.m_ui64Value;
    }

    /*
    Description: Converts a time interval in seconds to milliseconds.
    Parameters:
    fTimeIntervalInSeconds: A time interval in seconds.
    Returns: The given time interval in milliseconds.
    See Also: ConvertToSeconds
    */

    /*!
     * @brief   秒単位の時間範囲から、ミリ秒単位の時間範囲オブジェクトへ変換します。
     * @param   fTimeIntervalInSeconds [in] 変換される、秒単位の時間範囲を指定されます。
     * @return  変換された結果のミリ秒単位の時間範囲オブジェクトが返ります。
     */
    static TimeInterval ConvertFromSeconds(qReal fTimeIntervalInSeconds)
    {
        return TimeInterval(fTimeIntervalInSeconds * QREAL(1000.0));
    }

    /*
    Description: Converts a time interval in milliseconds to seconds.
    Parameters:
    tiTimeInterval: A time interval in milliseconds.
    Returns: The given time interval in seconds.
    See Also: ConvertFromSeconds
    */

    /*!
     * @brief   ミリ秒単位の時間範囲オブジェクトから、秒単位の時間範囲オブジェクトへ変換します。
     * @param   tiTimeInterval [in] 変換される、ミリ秒単位の時間範囲オブジェクトを指定します。
     * @return  変換された、秒単位の時間範囲値が返ります。
     */
    static qReal ConvertToSeconds(TimeInterval tiTimeInterval)
    {
        return qReal(tiTimeInterval) / QREAL(1000.0);
    }

    //{secret}
    static Time ConvertTimeoutToDeadline(qUnsignedInt32 uiTimeout);
    //{secret}
    static qUnsignedInt32 ConvertDeadlineToTimeout(Time tDeadline);

    // {Secret}
    Time Scale(qReal fRatio) const;

    // {Secret}
    Time Multiply(qReal fFactor) const;
    // {Secret}
    Time Divide(qReal fDividend) const;

    // {secret}
    typedef qUnsignedInt64 (*SessionTimeCallback)();
    // {secret}
    static void RegisterSessionTimeCallback(SessionTimeCallback pfGetSessionTime)
    {
        s_pfGetSessionTime = pfGetSessionTime;
    }

    /*
    Description: This method has been deprecated. Use the SystemClock::RegisterTimeProvider method instead.
    Parameters:
    pTimeProvider: A pointer to the TimeProvider that will be registered to this Time object.
    By default the $PRODUCTNAME$ internal, platform specific TimeProvider is registered.
    Remarks: Registers the specified TimeProvider to this Time object. A custom TimeProvider may be used to
    provide the time that $PRODUCTNAME$ uses for time related computations, such as those associated with the
    SessionClock and SystemClock. Before the time can be retrieved from a TimeProvider other than the default,
    the TimeProvider must be registered to a Time object
    via this method. Note that RegisterTimeProvider must be called before the $PRODUCTNAME$ object is created as
    follows:

            // Call these two lines before you create the NetZ object
            MyTimeProvider oTimeProvider;
            Time::RegisterTimeProvider(&oTimeProvider);

    See Also: GetProviderTime
    */
    static void RegisterTimeProvider(TimeProvider *pTimeProvider);

private:

    qUnsignedInt64 m_ui64Value;
    static SessionTimeCallback s_pfGetSessionTime;
    static const qUnsignedInt64 FREQUENCY;
};

// Time streaming.
//{secret}
extern StringStream& operator<<(StringStream& oStream, const Time& tValue);

//{secret}
END_NAMESPACE_NEX

