﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <iterator>
#include <algorithm>

#if defined(NON_SUPPORT_C_TARGET)

// Where ptrdiff_t is defined.

#include <stddef.h>
#endif

BEGIN_NAMESPACE_NEX

// Where T is a pointer (or a type with a pointer-like interface)

#define CHAINABLE(T)    mutable qChain<T*>::Links m_oLinks

// A ChainPolicy suitable for use with objects declared as CHAINABLE.

template<class T>
class DefaultChainPolicy
{
public:
    static T& Next(const T& pIt)
    {
        return pIt->m_oLinks.m_pNext;
    }
    static T& Previous(const T& pIt)
    {
        return pIt->m_oLinks.m_pPrevious;
    }
};


#define CHAINPOLICY_DEFINE(NAME)  \
template<class T> \
class ChainPolicy##NAME { \
    public: \
        static T& Next(const T& pIt) {return pIt->m_oLinks##NAME .m_pNext;} \
        static T& Previous(const T& pIt) {return pIt->m_oLinks##NAME .m_pPrevious;} \
}
#define CHAINABLE_POLICY(T, NAME) mutable qChain<T*>::Links m_oLinks##NAME


// class qChain: The usefulness of this container is that it works
// without using dynamically allocated memory like most STL containers.
// Where T is a pointer (or a type with a pointer-like interface)
template<class T, class ChainPolicy=DefaultChainPolicy<T> >
class qChain: public RootObject
{
private:
public:
#ifdef NEX_STD_BASE_ITERATOR_SUPPORT
    class iterator: public std::iterator<std::bidirectional_iterator_tag,T>
#else
    class iterator
#endif
    {
    public:
        iterator(): m_link(NULL) {}
        iterator(const iterator &it): m_link(it.m_link) {}
        iterator(const T& link) : m_link(link) {}

        const iterator& operator++() const
        {
            m_link = ChainPolicy::Next(m_link);
            return *this;
        }
        iterator& operator++()
        {
            m_link = ChainPolicy::Next(m_link);
            return *this;
        }
        const iterator& operator--() const
        {
            m_link = ChainPolicy::Previous(m_link);
            return *this;
        }
        iterator& operator--()
        {
            m_link = ChainPolicy::Previous(m_link);
            return *this;
        }

        // The postfix versions are not implemented for now.
        // Anyways, most of the time, this is not the right
        // version to use.  They can't be implemented with a
        // by-reference return value are are less efficient.

        //              iterator& operator++(int);
        //              iterator& operator--(int);

        qBool operator==(const iterator &it) const
        {
            return m_link == it.m_link;
        }
        qBool operator!=(const iterator &it) const
        {
            return m_link != it.m_link;
        }
        iterator &operator=(const iterator &it)
        {
            m_link = it.m_link;
            return *this;
        }

        T &operator*()
        {
            return m_link;
        }
        const T &operator*() const
        {
            return m_link;
        }

#ifndef NEX_STD_BASE_ITERATOR_SUPPORT
        typedef std::bidirectional_iterator_tag iterator_category;
        typedef T value_type;
        typedef ptrdiff_t difference_type;
        typedef T* pointer;
        typedef T& reference;
#endif

    private:
        mutable T m_link;
    };

    typedef const iterator const_iterator;

    qChain() : m_itFirst(NULL), m_itLast(NULL), m_itEnd(NULL), m_nbLinks(0) {}
    virtual ~qChain()
    {
        clear();
    }

    // For convenience, to implement the Links.  Does nothing
    // except the initialization of the pointers.
    class Links
    {
    public:
        Links(): m_pNext(NULL), m_pPrevious(NULL) {}
        mutable T m_pNext;
        mutable T m_pPrevious;
    };

    const iterator& begin() const
    {
        return m_itFirst;
    }
    iterator& begin()
    {
        return m_itFirst;
    }

    const iterator& end() const
    {
        return m_itEnd;
    }
    iterator& end()
    {
        return m_itEnd;
    }

    //最後尾の要素のiteratorを返します。次はendになります。
    const iterator& last() const
    {
        return m_itLast;
    }
    iterator& last()
    {
        return m_itLast;
    }

    const T& front() const
    {
        return *m_itFirst;
    }
    T& front()
    {
        return *m_itFirst;
    }

    void pop_front()
    {
        erase(m_itFirst);
    }


    const T& back() const
    {
        return *m_itLast;
    }
    T& back()
    {
        return *m_itLast;
    }

    void pop_back()
    {
        erase(m_itLast);
    }


    // Set links between i and j
    void LinkItems(const T& i, const T& j)
    {
        SYSTEMCHECK(ChainPolicy::Next(i)==NULL);
        SYSTEMCHECK(ChainPolicy::Previous(j)==NULL);
        ChainPolicy::Next(i)=j;
        ChainPolicy::Previous(j)=i;
    }

    void push_back(const T& newItem)
    {
        SYSTEMCHECK(ChainPolicy::Previous(newItem)==NULL);
        SYSTEMCHECK(ChainPolicy::Next(newItem)==NULL);
        if (begin()!=end())
        {
            LinkItems(*m_itLast,newItem);
            m_itLast = newItem;
        }
        else
        {
            SYSTEMCHECK(m_itFirst==m_itEnd);
            SYSTEMCHECK(m_itLast==m_itEnd);

            m_itFirst = iterator(newItem);
            m_itLast = iterator(newItem);
        }
        m_nbLinks++;
    }

    void push_front(const T& newItem)
    {
        SYSTEMCHECK(ChainPolicy::Previous(newItem)==NULL);
        SYSTEMCHECK(ChainPolicy::Next(newItem)==NULL);
        if (begin()!=end())
        {
            LinkItems(newItem,*m_itFirst);
            m_itFirst = newItem;
        }
        else
        {
            SYSTEMCHECK(m_itFirst==m_itEnd);
            SYSTEMCHECK(m_itLast==m_itEnd);
            m_itFirst = iterator(newItem);
            m_itLast = iterator(newItem);
        }
        m_nbLinks++;
    }

    // Inserts newItem before pos and returns an iterator pointing on newItem.
    iterator insert(iterator pos, const T& newItem)
    {
        SYSTEMCHECK(ChainPolicy::Previous(newItem)==NULL);
        SYSTEMCHECK(ChainPolicy::Next(newItem)==NULL);
        iterator result(newItem);
        // Check if chain is not empty
        if( begin()!=end() )
        {
            //Fix for inserting at end
            // inserting at end is just like a push_back
            if (m_itEnd == pos)
            {
                LinkItems(*m_itLast, newItem);
                m_itLast = newItem;
            }
            else
            {
                // Check if newItem has been inserted before m_itFirst
                if( m_itFirst != pos )
                {
                    // Must set links between pos previous item and newItem
                    T& posPreviousNode = ChainPolicy::Previous(*pos);
                    ChainPolicy::Previous(newItem)     = posPreviousNode;
                    ChainPolicy::Next(posPreviousNode) = newItem;
                    ChainPolicy::Previous(*pos)        = NULL;
                }
                else
                {
                    m_itFirst = result;
                }
                LinkItems(newItem,*pos);
            }
        }
        else
        {
            SYSTEMCHECK(m_itFirst==m_itEnd);
            SYSTEMCHECK(m_itLast==m_itEnd);
            m_itFirst = result;
            m_itLast  = result;
        }
        m_nbLinks++;
        return result;
    }

    template <typename InputIterator>
    void insert(iterator pos, InputIterator itStart, InputIterator itEnd)
    {
        while(itStart != itEnd)
        {
            pos = ++insert(pos, *itStart);
            ++itStart;
        }
    }

    iterator erase(iterator it);

    iterator erase(iterator first, iterator last)
    {
        iterator it = first;
        while (it != last)
        {
            it = erase(it);
        }
        return it;
    }

    void remove(const T& item)
    {
        iterator start = begin();

        while(start != end())
        {
            start = std::find(start, end(), item);
            if(start != end())
            {
                start = erase(start);
            }
        }
    }

    size_t size() const
    {
        return m_nbLinks;
    }
    qBool empty() const
    {
        return size()==0;
    }
    void clear()
    {
        erase(begin(),end());
    }
    void swap(qChain& other)
    {
        std::swap(m_itFirst, other.m_itFirst);
        std::swap(m_itLast, other.m_itLast);
        std::swap(m_nbLinks, other.m_nbLinks);
    }

private:
    iterator m_itFirst;
    iterator m_itLast;
    iterator m_itEnd;
    size_t  m_nbLinks;
};

template<class T, class ChainPolicy>
typename qChain<T,ChainPolicy>::iterator qChain<T,ChainPolicy>::erase(iterator it)
{
    // This must be a copy, since we are going to modify
    // m_itLast/m_itFirst.

    // Create a copy of the previous/next - since they will be
    // changed.
    iterator itPrevious = ChainPolicy::Previous(*it);
    iterator itNext = ChainPolicy::Next(*it);

    // Unlink to the left.
    if (itPrevious!=NULL)
    {
        ChainPolicy::Next(*itPrevious)=*itNext;
    }
    else
    {
        SYSTEMCHECK(m_itFirst==it);
    }
    ChainPolicy::Previous(*it)=NULL;

    // Unlink ot the right.
    if (itNext!=NULL)
    {
        ChainPolicy::Previous(*itNext)=*itPrevious;
    }
    else
    {
        SYSTEMCHECK(m_itLast==it);
    }
    ChainPolicy::Next(*it)=NULL;

    // Recompute first/last
    if (m_itFirst==it)
    {
        m_itFirst = itNext;
    }
    if (m_itLast==it)
    {
        m_itLast = itPrevious;
    }

    m_nbLinks--;
    return itNext;
}

END_NAMESPACE_NEX

