﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Platform/Core/ScopedCS.h>
#include <OnlineCore/src/Platform/Core/SystemChecker.h>

BEGIN_NAMESPACE_NEX

//This is a list class which forces the user to lock a critical section (to prevent concurrent access) before manipulating it
//If the user manipulates this list without locking a critical section, the class will force an error
template <typename T> class qProtectedVector: public RootObject
{
public:
    qProtectedVector(): m_csVector(LockMasks::Ignored) {}
    qProtectedVector(const qProtectedVector& oOther):RootObject(), m_csVector(LockMasks::Ignored)
    {
        *this = oOther;
    }
    //n1589:CriticalSection:Ignored
    qProtectedVector(const qVector<T>& oOther): m_csVector(LockMasks::Ignored)
    {
        *this = oOther;
    }

    virtual ~qProtectedVector() {}

    qProtectedVector& operator=(const qProtectedVector& oOther)
    {
        ScopedCS oCS(m_csVector);
        m_oVector = oOther.m_oVector;
        return *this;
    }
    qProtectedVector& operator=(const qVector<T>& oOther)
    {
        ScopedCS oCS(m_csVector);
        m_oVector = oOther;
        return *this;
    }

    qBool operator==(const qVector<T>& oOther) {
        ScopedCS oCS(m_csVector);
        return m_oVector == oOther;
    }

    qBool operator==(const qProtectedVector& oOther) {
        ScopedCS oCS(m_csVector);
        return m_oVector == oOther.m_oVector;
    }

    qBool operator!=(const qVector<T>& oOther) {
        return !(*this == oOther);
    }

    qBool operator!=(const qProtectedVector& oOther) {
        return !(*this == oOther);
    }

    typedef typename qVector<T>::iterator iterator;
    typedef typename qVector<T>::const_iterator const_iterator;

    void push_back(const T& e)
    {
        ScopedCS oCS(m_csVector);
        m_oVector.push_back(e);
    }
    void pop_back()
    {
        ScopedCS oCS(m_csVector);
        m_oVector.pop_back();
    }

    void push_front(const T& e)
    {
        ScopedCS oCS(m_csVector);
        m_oVector.push_front(e);
    }
    void pop_front()
    {
        ScopedCS oCS(m_csVector);
        m_oVector.pop_front();
    }

    T& front()
    {
        SYSTEMCHECK(m_csVector.HasOwnership());
        return m_oVector.front();
    }
    const T& front() const
    {
        SYSTEMCHECK(m_csVector.HasOwnership());
        return m_oVector.front();
    }

    iterator begin()
    {
        SYSTEMCHECK(m_csVector.HasOwnership());
        return m_oVector.begin();
    }
    const_iterator begin() const
    {
        SYSTEMCHECK(m_csVector.HasOwnership());
        return m_oVector.begin();
    }
    iterator end()
    {
        SYSTEMCHECK(m_csVector.HasOwnership());
        return m_oVector.end();
    }
    const_iterator end() const
    {
        SYSTEMCHECK(m_csVector.HasOwnership());
        return m_oVector.end();
    }

    iterator erase(iterator pos)
    {
        SYSTEMCHECK(m_csVector.HasOwnership());
        return m_oVector.erase(pos);
    }
    iterator erase(iterator first, iterator last)
    {
        SYSTEMCHECK(m_csVector.HasOwnership());
        return m_oVector.erase(first,last);
    }

    void remove(const T& x)
    {
        ScopedCS oCS(m_csVector);
        m_oVector.remove(x);
    }

    /* API is different than the standard: it would be unsafe to
     * return an iterator */
    void safe_erase(iterator pos)
    {
        ScopedCS oCS(m_csVector);
        m_oVector.erase(pos);
    }
    void safe_erase(iterator first, iterator last)
    {
        ScopedCS oCS(m_csVector);
        m_oVector.erase(first, last);
    }

    void clear()
    {
        ScopedCS oCS(m_csVector);
        m_oVector.clear();
    }
    qBool empty() const
    {
        ScopedCS oCS(m_csVector);
        return m_oVector.empty();
    }
    qUnsignedInt size() const
    {
        ScopedCS oCS(m_csVector);
        return m_oVector.size();
    }

    void copy(qVector<T> & vectorCopy) const
    {
        ScopedCS oCS(m_csVector);
        vectorCopy = m_oVector;
    }

    CriticalSection& CS() const
    {
        return m_csVector;
    }

private:

    mutable NAMESPACE_NEX CriticalSection m_csVector;

    qVector<T> m_oVector;
};

END_NAMESPACE_NEX

