﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once


#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Platform/Core/qMap.h>
#include <OnlineCore/src/Platform/Core/qReservedQueue.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Stack/Core/SocketDriver.h>
#include <OnlineCore/src/Platform/Stack/Core/ConnectionOrientedSocketManager.h>
#include <OnlineCore/src/Core/SharedPtr.h>
#include <OnlineCore/src/Core/ByteOrder.h>

BEGIN_NAMESPACE_NEX

class IOCompletionContext;
class Buffer;
class InterfaceInfo;

class Socket: public RootObject
{
    friend class QueuingSocketTest;
    class ImplAttributes;

    friend StringStream& operator<<(StringStream& oStr, Socket& oSocket);

public:
    // ソケットの状態
    enum SocketStatus
    {
        SOCKET_STATUS_AVAILABLE,                                // 利用可能な状態
        SOCKET_STATUS_DISCONNECTED_GENERAL,                     // 切断された状態
        SOCKET_STATUS_DISCONNECTED_UNDER_MAINTENANCE,           // 切断された状態(メンテナンス中エラーが返された)
        SOCKET_STATUS_DISCONNECTED_SERVICE_NO_LONGER_AVAILABLE, // 切断された状態(サービス終了エラーが返された)
        SOCKET_STATUS_DISCONNECTED_APPLICATION_VERSION_IS_OLD,  // 切断された状態(アプリケーションバージョンエラーが返された)
        SOCKET_STATUS_DISCONNECTED_OUT_OF_SERVICE,              // 切断された状態(サービス休止中)
        SOCKET_STATUS_DISCONNECTED_MAX_CONNECTIONS_REACHED       // 切断された状態(コネクション数限界)
    };

    // Winsock2 においては、GetLastSocketErrorの値が入る
    enum CALL_STATE
    {
        CALL_SUCCESS= 0,
        CALL_ERROR = 0xFFFFFFFFUL
    };

    // Poll()で監視するイベント。SocketDriver::PollEventと同値。
    enum PollEvent
    {
        // No events raised or to check for.
        POLL_EVENT_NONE = 0,
        // Normal data may be read without blocking.
        POLL_EVENT_READY_TO_READ = 1,
        // Normal data may be written without blocking or a connection has been successfully established.
        POLL_EVENT_READY_TO_WRITE = 2
    };

    // SendやRecvの返値
    static const qInt SOCKET_SUCCESS;   // 成功
    static const qInt SOCKET_ERR;       // 失敗

    Socket(TransportProtocol::Type transportProtocolType = DEFAULT_TRANSPORT_PROTOCOL_TYPE, qUnsignedInt32 uiMtu = SocketConstants::DEFAULT_MTU);
    virtual ~Socket();

    qBool Open();

    void Shutdown();
    void Close();
    void AcquireAcceptedSocket(const InetAddress& remoteAddr);
    void ReleaseAcceptedSocket(const InetAddress& remoteAddr);

    virtual qBool Bind(InetAddress *pBindAddress, qBool bReuse = false);

    qInt Recv(qByte* pReceivedBuffer, size_t receiveBufferSize, size_t* pWrittenSize, InetAddress* pInetAddress);
    qInt RecvNonBlock(qByte* pReceivedBuffer, size_t receiveBufferSize, size_t* pWrittenSize, InetAddress* pInetAddress);
    virtual qInt Recv(Buffer* receivedBuffer, InetAddress* pInetAddress);
    virtual qInt RecvNonBlock(Buffer* receivedBuffer, InetAddress* pInetAddress);
    qInt Send(const qByte* pSendBuffer, size_t sendBufferSize, const InetAddress& inetAddress);
    virtual qInt Send(const Buffer& sendBuffer, const InetAddress& inetAddress);

    // 自身が保持している一つのSocketDriver::SocketをPollする。複数個をPollしたい場合はstatic版を使用してください。
    qBool Poll(PollEvent event, PollEvent* revent, qUnsignedInt32 uiTimeoutInMs);

    // 保持しているSocketDriver::SocketのためのPollInfoを作成する。
    SocketDriver::PollInfo CreatePollInfo(SocketDriver::PollEvent event)
    {
        SocketDriver::PollInfo pollInfo(m_pDriverSocket.get(), event);
        return pollInfo;
    }

    // コネクション指向(TCP, WebSocket)のソケットを利用する場合のための接続関数
    qBool Connect(const InetAddress& connectAddress);
    qBool Listen();

    // Listen()を呼び出したかどうか。
    qBool IsListening() const
    {
        return m_pDriverSocket->IsListening();
    }

    void DoWork();

    Socket::CALL_STATE GetIOResult(IOCompletionContext *pIOCompletionContext);
    qBool SetMulticastAddress(qUnsignedInt32 uiAddress);
    InetAddress *GetAddress();

    qUnsignedInt16 GetBoundPort();

    TransportProtocol::Type GetTransportProtocolType()
    {
        return m_transportProtocolType;
    }

    // ソケットの状態が利用可能状態かどうかを返す。
    qBool IsSocketValid() const
    {
        return m_socketStatus == SOCKET_STATUS_AVAILABLE;
    }

    // ソケットの状態を返す。
    SocketStatus GetSocketStatus() const
    {
        return m_socketStatus;
    }

    // UDP用のデフォルトのSocketDriverを取得する。
    static SocketDriver* GetDefaultSocketDriver()
    {
        return s_pDefaultUdpSocketDriver;
    }

    // UDP用のSocketDriverを取得する
    static SocketDriver* GetSocketDriver()
    {
        return s_pSocketDrivers[TransportProtocol::TYPE_UDP];
    }
    // UDP用のSocketDriverをセットする
    static void SetSocketDriver(SocketDriver* pFactory)
    {
        s_pSocketDrivers[TransportProtocol::TYPE_UDP] = pFactory;
    }

    // WebSocket用のSocketDriverを取得する
    static WebSocketDriver* GetWebSocketDriver()
    {
        return static_cast<WebSocketDriver*>(s_pSocketDrivers[TransportProtocol::TYPE_WEB_SOCKET]);
    }

    // WebSocket用のSocketDriverを設定する
    static void SetWebSocketDriver(WebSocketDriver* webSocketDriver)
    {
        s_pSocketDrivers[TransportProtocol::TYPE_WEB_SOCKET] = webSocketDriver;
    }

    // 現在使われていない。(未使用のIOCompletionNotifierからのみ使用されている)
    SocketDriver::Socket* GetDriverSocket()
    {
        return m_pDriverSocket.get();
    }

    // トランスポートプロトコルによっては必要となるURI情報をセットする。
    // WebSocketの場合に接続先として利用する。Connect前に呼び出す必要がある。
    void SetUri(const String& uri)
    {
        m_pDriverSocket->SetUri(uri);
    }

    void Trace(qUnsignedInt64 uiTraceFlag=TRACE_ALWAYS);
    qBool SetTTL(qUnsignedInt8 uiTTL);

    // accept可能な最大ソケット数をセットする。
    static void SetMaxNumAcceptableSockets(size_t num)
    {
        if (num == 0)
        {
            // 0ならデフォルトにする。
            s_numAcceptableSockets = DEFAULT_NUM_ACCEPTABLE_SOCKETS;
            return;
        }
        s_numAcceptableSockets = num;
    }

    // 複数個のSocketをPollする。
    // SocketDriver::PollInfoはCreatePollInfo()で作成できます。
    // 指定するSocketは全て同じTransportProtocol::Typeである必要があります。
    static qBool Poll(TransportProtocol::Type transportProtocolType, SocketDriver::PollInfo aPollInfo[], qUnsignedInt uiPollInfoSize, qUnsignedInt32 uiTimeoutInMs)
    {
        return s_pSocketDrivers[transportProtocolType]->Poll(aPollInfo, uiPollInfoSize, uiTimeoutInMs);
    }
protected:
    // 各トランスポートプロトコルのデフォルトのSocketDriver
    static SocketDriver* s_pDefaultUdpSocketDriver;
    static SocketDriver* s_pDefaultTcpSocketDriver;
    static SocketDriver* s_pDefaultWebSocketDriver;

    InetAddress         m_oInetAddress; // Bindした自身のアドレス(ポート番号のみ利用可能)
    qInt                m_iRecvAddrLen;
    TransportProtocol::Type m_transportProtocolType;
    qUnsignedInt32      m_uiMTU;

    static SocketDriver *s_pSocketDrivers[TransportProtocol::TYPE_COUNT];
    SharedPtr<SocketDriver::Socket> m_pDriverSocket;
    SocketDriver*       m_pSocketDriver;    // m_pDriverSocketを生成したSocketDriver

    SocketStatus m_socketStatus;
    qBool m_bConnected;     // 一度でもConnectに成功するとtrueになる
    qBool m_bClosed;        // 一度でもCloseするとtrueになる。
#ifdef NEX_SYSTEMCHECK_ENABLED
    InetAddress m_connectedRemoteAddress;   // 接続した相手のアドレス。コネクションを共有する場合に同じ相手にConnectしていることを確認するために使う。
#endif

private:
    qInt RecvImpl(qByte* pReceivedBuffer, size_t receivecBufferSize, size_t* pWrittenSize, InetAddress* pInetAddress, SocketDriver::SocketFlag flag);
    static qUnsignedInt32 GetLastSocketError();
    qBool ProcessDriverSocketResult(SocketDriver::Result eResult, IOCompletionContext *pIOCompletionContext);
    qBool _Bind(InetAddress *pBindAddress);

    SharedPtr<ConnectionOrientedSocketManager> m_pConnectionOrientedSocketManager;

    static const TransportProtocol::Type DEFAULT_TRANSPORT_PROTOCOL_TYPE;
    static const size_t DEFAULT_NUM_ACCEPTABLE_SOCKETS = 10240;  // Acceptできる最大ソケット数のデフォルト値
    static size_t s_numAcceptableSockets;                        // Acceptできる最大ソケット数

};

StringStream& operator<<(StringStream& oStr, Socket& oSocket);



END_NAMESPACE_NEX
