﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/PseudoSingleton.h>
#include <OnlineCore/src/Core/SystemComponent.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Transport/Interface/StationProbeList.h>
#include <OnlineCore/src/Transport/Interface/ConnectivityTester.h>
#include <OnlineCore/src/Core/Buffer.h>



BEGIN_NAMESPACE_NEX

class NATTransportAdapter;

class CallContext;

//$ifdef RENDEZVOUS$
//group=NAT Traversal Service
//$else$
//group=Session Management
//$endif$

/*
Description: This singleton manages $PRODUCTNAME$ connectivity testing.
Remarks:

$ifdef NETZ$
If you are also using Rendez-Vous, use this class to manage and perform
connectivity testing, including correct NAT traversal, between the Stations participating
in a gaming Session.

If you are also using Rendez-Vous, see the /Rendez-Vous Reference Manual/ for more information
on how it expects you to use this class.

If you are not using Rendez-Vous or any of the platforms mentioned in
RegisterConnectivityTester below, you can use this class to manage connectivity testing
between stations in a session, in which case, you must implement and register your own
ConnectivityTester::ProbeStations method.
$endif$

$ifdef RENDEZVOUS$
Use this class to test, establish and manage connectivity between Stations
participating in a peer-to-peer game Session. Note that Rendez-Vous uses its own
ConnectivityTester which should not be overridden. The Rendez-Vous connectivity
tester calls into its NATTraversal Service to provide a high degree of connectivity
between players. See the NAT Overview for information on the NATTraversal Service.
See Also: NATTraversalClient, VSocket, NATProperties, UPnPProperties, RootTransport
$endif$
*/

/*!
 @brief    インターネット P2P 通信の管理クラスです。
 */
class ConnectivityManager: public PseudoSingleton
{
    //Group=Methods
public:
    ConnectivityManager();
    virtual ~ConnectivityManager();

    /*
    Description: Returns a pointer to the ConnectivityManager instance.
    Returns: A reference to the ConnectivityManager instance.
    Remarks: Use the reference returned by this method to control the ConnectivityManager object.
    */

    /*!
     @brief    @ref ConnectivityManager のインスタンスを取得します。

     @return @ref ConnectivityManager オブジェクトのポインタ
     */
    static ConnectivityManager *GetInstance();

    // {secret}
    static ConnectivityManager *GetInstanceWithoutNetworkCheck();

    // {secret}
    qBool RegisterTransportAdapter(NATTransportAdapter *pTransportAdapter);

    // {secret}
    qBool UnregisterTransportAdapter();


    /*
    $ifdef NETZ$
    Description: Registers a custom ConnectivityTester object with the ConnectivityManager.
    Parameters:
    pConnectivityTester: A pointer to the ConnectivityTester object.
    Returns: True if the specified ConnectivityTester was successfully installed, false otherwise.
    Remarks:
    To establish a player's connectivity with all stations in the session, JoinSession() or CreateOrJoinSession()
    call into the ConnectivityManager::ProbeStations() method which, in turn, calls into the
    ConnectivityTester::ProbeStations method. It is important to register your ConnectivityTester before
    calling any of these methods. If the connectivity test fails, the originating calls will also fail.

    Note:
        o Rendez-Vous has its own ConnectivityTester::ProbeStations method which calls into
          its NATTraversal Service.
        o Net-Z for the NON_SUPPORT_E, NON_SUPPORT_C (using UPnP) and the WII (using DWC) have default custom
          ConnectivityTester classes which are automatically created on startup. They have been
          written to utilize the NAT traversal capabilities of their native platforms and platform
          services.

    Only one ConnectivityTester object can be registered at any given time. If you try to
    register more than one ConnectivityTester, an error is returned.

    See Also: UnregisterConnectivityTester, ConnectivityTester, JoinSession, CreateOrJoinSession
    $else$
    {secret}
    $endif$
    */
    qBool RegisterConnectivityTester(ConnectivityTester *pConnectivityTester);

    /*
    $ifdef NETZ$
    Description: Unregisters the ConnectivityTester object.
    Parameters:
    Remarks: Only one ConnectivityTester object can be registered to a Session at any given time.
    See Also: RegisterConnectivityTester, ConnectivityTester
    $else$
    {secret}
    $endif$
    */
    qBool  UnregisterConnectivityTester();

    // {secret}
    ConnectivityTester *GetConnectivityTester()
    {
        return m_pConnectivityTester;
    }

    // {secret}
    qBool FetchPublicURL(CallContext *pCallContext, StationURL::URLType eType);
    // {secret}
    qBool GetPublicURL(StationURL::URLType eType, StationURL *pPublicStationURL);

    /*
    Description: Sets the probe lifetime in ms before it expires.
    Parameters:
    tiProbeKeepAlive: The probe lifetime in ms.
    Remarks: This value must be configured if you expect a longer period of time than the default value of 60000 ms
    between the time you call the ProbeStations() method and another ProbeStations() (or JoinSession() if you are using
    NetZ).
    */
    void SetProbeLifetime(TimeInterval tiProbeLifetime);

    /*
    Description: Sets the probe keep alive frequency in ms.
    Parameters:
    tiProbeKeepAlive: The probe keep alive frequency in ms.
    Remarks: This value must be equal to or lower than the probe lifetime. The default value is 60000 ms.
    */
    void SetProbeKeepAlive(TimeInterval tiProbeKeepAlive);

    /*
    Description: Probes (pings) all stations in the specified StationProbeList.
    Parameters:
    pContext: A pointer to the CallContext of this call.
    preflstStationProbe: The list of stations that will be probed.
    ui32Timeout: The maximum time in milliseconds that the system will wait to receive probe reply from all stations.
    The default value is 5000 ms.
    Returns: true if the call was successfully initiated, false if not or an error occurred. The state of the CallContext
    (CallContext::State) will be /CallSuccess/ if all specified stations could be probed and /CallFailure/ if one or more
    stations could not be probed.

    Remarks: This method probes all stations in the specified StationProbeList (preflstStationProbe) in order to determine
    if the caller ( /i.e./ the local station) can connect to all stations. Note that each time this method is called, the
    StationProbe data for each station is reset to the default values before the probe is actually performed.

    $ifdef RENDEZVOUS$
    Note: Rendez-Vous has a custom ConnectivityTester which also uses Rendez-Vous' NATTraversal Service to establish
    connectivity between stations. If using Net-Z, do not attempt to register any custom implementation of
    ProbeStations.

    Note: The StationURL s in the StationProbeList must all have the same URLType StationURL::prudp. Also, the Rendez-Vous
    Connection ID must be present in each StationURL.

    If also using Net-Z, refer to the /Net-Z Reference Manual/ for information about this command that is
    specific to Net-Z.
    See Also: StationURL, StationProbeList, StartNATSession
    $else$
    This method calls into a registered implementation of ConnectivityTester::ProbeStations. If not using Rendez-Vous,
    you are expected to implement this method to probe all stations in the specified StationProbeList to determine if
    the caller (/i.e./ the local station) can connect to all of them.

    The Station::JoinSession and Station::CreateOrJoinSession methods call into this method to determine
    whether the calling station can connect to all other stations in the session. In turn, this method calls into
    the registered ConnectivityTester::ProbeStations.

    StationURLs in the StationProbeList must all have the same URLType (StationURL::prudp).

    If also using Rendez-Vous, refer to the /Rendez-Vous Reference Manual/ for information about this command that is
    specific to Rendez-Vous.

    See Also: StatonURL, StationProbeList, RegisterConnectivityTester, StartNATSession
    $endif$
    */
    //n2781:デフォルト値を変更
    /*! @internal
     * @brief 指定された URL に対して NAT トラバーサルを行い、RTT(Ping) 値を取得します
     *
     * 本関数を呼び出す前に、 StartNatSession() を呼んでおく必要があります。
     *
     * @param[in,out]  pContext          呼び出し情報。
     * @param[in]  preflstStationProbe
     * @param[in]  ui32Timeout           NAT トラバーサルのタイムアウト値。 0 を指定した場合、 RootTransport::RootTransport::SetNATTraversalTimeout() で指定した値が使われます。
     * @param[in]  bWaitForAllResult     全てのクライアントに対する NAT トラバーサルの結果を待つかどうか。true の場合、全ての結果を待つ。false の場合、一つでも失敗した段階で非同期処理を終了する。
     */
    qBool ProbeStations(CallContext *pContext, StationProbeList* preflstStationProbe, qUnsignedInt32 ui32Timeout = 0, qBool bWaitForAllResult = false);

    /*
    $ifdef RENDEZVOUS$
    Description: Starts a NAT session.
    Returns: True if the NAT session successfully started; false otherwise.
    Remarks: Traditionally, Net-Z and Rendez-Vous easily shared the same port (by default, this is 9103).
    Any other networking engine shared its port with Rendez-Vous for NAT purposes through a NATTransportAdapter.
    Now, Rendez-Vous detects a player's NAT device during the login and may determine that the networking engine
    (such as Net-Z) may need to use a different port from the one that Rendez-Vous uses and it may need to open
    (and close) this port each time a new game session is created or joined. We have therefore added the concept
    of a NATSession to assign the networking engine a port for the "lifetime" of a game session (Net-Z Session).
    This method can reset the well-known port number (RootTransport::SetWellKnownPortNumber).

    Call this method to start a NAT session after a successfully loging in to Rendez-Vous.
    The lifetime of a NAT session is that of a game session and not a Rendez-Vous session. However, if you plan
    on calling ProbeStations on a list of stations before creating your game engine object, it is necessary to
    call StartNATSession before doing so to use the future game port for NAT traversal.

    If you are using NetZ as your network engine, you need to call StartNATSession as in the following example.

    Example:

                Rendez-Vous oRV;
                oRV.Login(...);
                ConnectivityManager::GetInstance()->StartNATSession();

                //ProbeStations

                // First game session
                NetZ pNetZ = qNew NetZ();
                Session::CreateSession();

                ...

                oNetZ->Terminate();
                qDelete pNetZ();
                ConnectivityManager::GetInstance()->StopNATSession();

                // Second game session
                pNetZ = qNew NetZ();
                ConnectivityManager::GetInstance()->StartNATSession();
                Session::CreateSession();
                ...

    See Also: StopNATSession
     $else$
    {secret}
    $endif$
    */
    qBool StartNATSession();
    void OnStartNATSession(CallContext *cc = NULL);

    /*!
     @brief    NAT セッションを開始します。

     NAT セッションを開始し、インターネット P2P 通信に必要な前処理を行います。

     インターネット P2P 通信を行う場合、NetZ オブジェクトを作成する前、もしくは VSocket::Open() を呼び出す前に必ず本関数を実行してください。
     なお、本関数を実行するたびに P2P 通信で使用されるポート番号が変化します。

     NgsFacade オブジェクト作成後の初回呼び出し時にのみ、 NAT 種別判定用サーバーの DNS による名前解決処理を別スレッドで行います。
     名前解決前にルータの WAN 側のケーブルを抜いたときなどには、名前解決処理のタイムアウトを待つため、本非同期処理の完了に 30 秒以上かかることがあります。
     また、名前解決はキャンセルできないため、StartNATSession() 後、アドレス解決が完了していない状態で NEX を終了させようとすると、30 秒以上かかることがあります。
     本関数呼び出し前に NATTraversalClient オブジェクトの生成と、そのオブジェクトへの Credentials のバインドが必要です。

     

     非同期処理の結果は、@ref CallContext::GetOutcome() で取得できます。取得できる値は以下の通りです。
     
     @li    QSUCCESS(Core, Success) 成功
     @li    QERROR(Core, InvalidSequence) NEXが初期化されていない。
                                          NATTraversalClient が 生成されていない。もしくは NATTraversalClient オブジェクトに Credentials がバインドされていない。
                                          すでに StartNATSession() が実行中。
                                          Bind()済みのVSocketオブジェクトが存在する。(VSocket オブジェクトの Close() が必要です。)
     @li    QERROR(Transport, DnsError) DNS タイムアウト
     @li    QERROR(Transoport, NatCheckError) NAT チェックのエラー

     @param[in,out]    cc    呼び出し情報。
     @retval    true    非同期処理の開始に成功。
     @retval    false   指定した cc の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

     @see RootTransport::SetNameResolveThreadPriority(), RootTransport::GetNameResolveThreadPriority()
     @see RootTransport::SetNameResolveThreadCoreNo(), RootTransport::GetNameResolveThreadCoreNo()
     */
    qBool StartNATSession(CallContext *cc);


    /*
    $ifdef RENDEZVOUS$
    Description: Stops a running NAT session.
    Returns: True if the NAT session successfully stopped; false otherwise.
    Remarks: It is necessary to call this method after the game session is over as the lifetime of a NAT session is
    for a game session and not for a Rendez-Vous session. If you are using NetZ, StopNATSession must be called after
    a call to NetZ::Terminate().

    Example:
    If you are using NetZ as your network engine, you need to call StopNATSession as follows:

                Rendez-Vous oRV;
                oRV.Login(...);
                ConnectivityManager::GetInstance()->StartNATSession();

                // First game session
                NetZ pNetZ = qNew NetZ();
                Session::CreateSession();

                ...

                oNetZ->Terminate();
                ConnectivityManager::GetInstance()->StopNATSession();
                qDelete pNetZ();

    See Also: StartNATSession
    $else$
    {secret}
    $endif$
    */

    /*!
     @brief    NAT セッションを終了します。

     NetZ::Terminate() 完了後、本関数を呼び出してください。

     @retval    true    成功
     @retval    false    失敗
     */
    qBool StopNATSession();

    //{secret}
    qBool IsNATSessionStarted();

    //{secret}
    qBool InitiateProbes(StationProbeList &refProbeList);

    qBool CheckUnupdateNATPort();

#ifdef CAFE_TARGET
    /*!
     * @internal
     * @brief NATセッション有効時( P2P 通信中)にシステムによるバックグラウンドダウンロード機能を停止するかを設定します。
     *
     * true が指定されている場合、StartNATSession() 呼び出し時にシステムによるバックグラウンドダウンロード機能を停止し、
     * StopNATSession() 呼び出し時に有効化します。falseが指定されている場合はバックグラウンドダウンロード機能の挙動には影響ありません。
     * また、どちらを指定しても、BOSSライブラリのフォアグランドダウンロード機能には影響を与えません。
     *
     * true が指定されている場合、NEXライブラリ内で自動的に ndm の Initialize(), Finalize() を呼び出します。
     * デフォルトは「true（停止する）」です。アプリケーションが直接 ndm の機能を使用する場合は、本関数を用いて
     * StartNATSession() を呼び出す前に設定を false に変更してください。
     *
     * @param[in] isSuspendBossDaemon NATセッション有効時にシステムによるバックグラウンドダウンロード機能を停止するかどうかを指定します。trueの場合、停止します。
     */
#endif
    static void SetSuspendBossDaemonOnNATSession(qBool isSuspendBossDaemon);

#ifdef CAFE_TARGET
    /*!
     * @internal
     * @brief NATセッション有効時( P2P 通信中)に システムによるバックグラウンドダウンロード機能を停止を停止するかを設定を取得します。
     *
     * @return StartNatSession()時にシステムによるバックグラウンドダウンロード機能を停止するかどうか
     */
#endif
    static qBool GetSuspendBossDaemonOnNATSession();

    // RMCによる中継情報の解決をするかを設定します。デフォルト TrueでON
    void SetIsEnabledResolveRelayInfo(qBool flag)
    {
        m_isEnabledResolveRelayInfo = flag;
    }

    qBool IsEnabledResolveRelayInfo()
    {
        return m_isEnabledResolveRelayInfo;
    }

    qBool StartResolveRelayInfo(CallContext *pContext);

    static void SetRelayPublicAddress();

    void FailResolvedRelayInfo();
    void ResolvedRelayInfo(CallContext *pContext);

    // Data members
private:
    qBool m_bIsNATSessionStarted;
    qBool m_isEnabledResolveRelayInfo;
    qUnsignedInt16 m_ui16NetZPortNumber;
    qUnsignedInt16 m_ui16PrevNetZPortNumber;
    ConnectivityTester *m_pConnectivityTester;

    qUnsignedInt32 m_NATCheckCallID;

    qUnsignedInt32 m_relayResolveCallID;

    qUnsignedInt32 m_SuspendBossDaemonCount;
    static qBool s_bSuspendBossDaemonOnNATSession;
#ifdef NEX_CLIENT_PBPOOL
    qBool m_bIsPacketBufferManagerInitialized;
#endif // NEX_CLIENT_PBPOOL

    qBool SetRelayAddress();

    qBool StartNATSessionInternal(CallContext *cc);
};

END_NAMESPACE_NEX

