﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>

BEGIN_NAMESPACE_NEX

/// 無制限の帯域幅を表します。
static const qUnsignedInt32 INFINITE_BANDWIDTH = 0xFFFFFFFF;


//Group = Network Emulation

/*
Description: Enables various network conditions to be emulated.
Remarks: The two types of emulation devices InputEmulationDevice
and OutputEmulationDevice both inherit from the EmulationDevice class.
Reference an emulation device through the RootTransport singleton. For example,

        RootTransport::GetInstance()->GetInputEmulationDevice()

        RootTransport::GetInstance()->GetOutputEmulationDevice().

It should be noted that some transport protocols  may not support all the functionality of the emulation device.
Enable must be called before an emulation device can be used.
See Also: EmulationDevice, InputEmulationDevice, OutputEmulationDevice
*/

/*!
 * @brief   さまざまなネットワーク状態をエミュレーションするクラスです。デバッグ用機能です。
 *
 * NEXでは、InputEmulationDevice、OutputEmulationDeviceという二つのエミュレーションデバイス
 * が定義されています。
 * これらクラスのオブジェクトは、RootTransportシングルトンより以下のように取得可能です。
 *
 * RootTransport::GetInstance()->GetInputEmulationDevice()
 * RootTransport::GetInstance()->GetOutputEmulationDevice()
 *
 * 各種設定値は、Enableメソッドを実行した後に設定してください。
 *
 *
 * @see EmulationDevice, InputEmulationDevice, OutputEmulationDevice
 * @see RootTransport::GetInputEmulationDevice, RootTransport::GetOutputEmulationDevice
 */
class EmulationDevice: public RootObject
{
public:

    //Group=Methods

    /*
    Remarks: An EmulationDevice object should never be directly instantiated. Instead, it is created by
    $PRODUCTNAME$ before a session is created. To access the EmulationDevice object from within your
    game use the Enable method.
    */
    EmulationDevice();

    /*
    Remarks: An EmulationDevice object should never be directly destroyed. It is destroyed by $PRODUCTNAME$
    when the session terminates.
    */
    virtual ~EmulationDevice();

    /*
    Description: Sets the simulated latency in milliseconds.

    Parameters:
    ui16Latency: The latency you want to emulate, in milliseconds. The
    default is 0.

    Remarks: Latency is the amount of time before a message sent from
    one station is received by another station. Enable must
    be called before this parameter is set. The only way to reference
    an emulation device is to use RootTransport::GetInputEmulationDevice or
    RootTransport::GetOutputEmulationDevice.

    See Also: GetLatency
    */


    /*!
     *  @brief 伝送遅延をmsec単位で指定します。
     *  @param[in] ui16Latency 伝送遅延[msec]を設定します。初期値は0です。
     *
     *  @remarks 設定する前に必ずEnableメソッドを実行してください。
     *           遅延を設定した場合は、遅延分のメモリ消費量が増えます。
     *  @see GetLatency, Enable
     *
     */
    void        SetLatency(qUnsignedInt ui16Latency);

    /*
    Description: Sets the simulated network jitter, in milliseconds.

    Parameters:
    ui16Jitter: Set this value to increase the latency in a random way
    in the range of 0 to ui16Jitter milliseconds. The default value is 0.

    Remarks: Jitter is the variation in latency. Use this method
    to adjust the jitter. As a result, the latency
    will be increased randomly by a value between 0 and ui16Jitter.
    Enable must be called before setting this parameter.
    The only way to reference an emulation device is to use
    RootTransport::GetInputEmulationDevice or
    RootTransport::GetOutputEmulationDevice.

    See Also: GetJitter
    */

    /*!
     *  @brief 遅延ジッターをmsec単位で指定します。
     *  @param[in] ui16Jitter ジッター遅延[msec]を設定します。初期値は0です。
     *                     0から設定値までの値がランダムに遅延が設定されれます。
     *
     *  @remarks 設定する前に必ずEnableメソッドを実行してください。
     *           ジッターを設定した場合は、遅延分のメモリ消費量が増えます。
     *  @see GetJitter, Enable
     *
     */
    void        SetJitter(qUnsignedInt ui16Jitter);
    // {secret}
    void        SetProtocolHeaderSize(size_t uiHeaderSize);

    /*
    Description: Sets the simulated modem bandwidth in bits per second.
    Parameters:
    uiBandwidth: The emulated bandwidth in bits per second. The default value
    is INFINITE_BANDWIDTH.
    Remarks:  Adjust the emulated bandwidth with this call. The default value is
    INFINITE_BANDWIDTH and therefore, performance is limited only by the actual physical
    bandwidth of the system. Change the emulated bandwidth to test the
    performance of your game at various conditions. Enable
    must be called before this parameter is set. The only way to reference
    an emulation device is to use RootTransport::GetInputEmulationDevice or
    RootTransport::GetOutputEmulationDevice.

    See Also: GetBandwidth
    */

    /*!
     *  @brief 帯域制限速度をbps単位で指定します。
     *  @param[in] uiBandwidth 帯域制限速度[bps]を設定します。初期値は、nn::nex::INFINITE_BANDWIDTHで、
     *                     帯域制限なしです。
     *
     *  @remarks 設定する前に必ずEnableメソッドを実行してください。
     *           INFINITE_BANDWIDTHの場合は、システムや物理帯域で制限されます。
     *  @see GetBandwidth, Enable
     *
     */
    void        SetBandwidth(qUnsignedInt32 uiBandwidth);

    // {secret}
    size_t        GetProtocolHeaderSize();


    //Description: Returns the simulated latency in milliseconds.
    //Returns: The simulated latency in milliseconds.
    //Remarks:  You can set this value via SetLatency.
    //See Also: SetLatency
    /*!
     *  @brief 伝送遅延をmsec単位で取得します。
     *  @return 伝送遅延[msec]を取得します。
     *  @see SetLatency, Enable
     */
    qUnsignedInt        GetLatency();

    //Description: Returns the simulated jitter in milliseconds.
    //Returns: The simulated jitter in milliseconds.
    //Remarks:  You can set this value via SetJitter.
    //See Also: SetJitter
    /*!
     *  @brief 遅延ジッターをmsec単位で取得します。
     *  @return ジッター遅延[msec]を取得します。
     *  @see SetJitter, Enable
     */
    qUnsignedInt        GetJitter();

    //Description: Returns the simulated modem bandwidth in bits per second.
    //Returns: The simulated modem bandwidth in bits per second.
    //Remarks: You can set this value via SetBandwidth.
    //See Also: SetBandwidth

    /*!
     *  @brief 帯域制限速度をbps単位で取得します。
     *  @return 帯域制限速度[bps]を取得します。
     *  @see SetBandwidth, Enable
     *
     */
    qUnsignedInt32      GetBandwidth();

    /*
    Description: Sets the packet drop probability to simulate poor network
    conditions where packets are dropped by routers on the way to their destination.
    Parameters:
    dProbability: The probability that a packet will be dropped on the way to
    its destination. dProbability must be in the range 0-1 and the default value,
    given by DROP_PROBABILITY_DEFAULT, is 0.
    Returns: true if the packet drop probability was successfully set, false if not or an error occurred.
    SYSTEMERROR_GEN_OUT_OF_RANGE is signalled if dProbability is not between 0-1.
    See Also: GetPacketDropProbability
    */

    /*!
     * @brief パケットロス率を設定します。
     * @param[in] dProbability パケットロス率を設定します。0以上1以下の範囲で有効です。デフォルト値は、0です。
     * @remarks 設定する前に必ずEnableメソッドを実行してください。
     * @see GetPacketDropProbability
     */
    void    SetPacketDropProbability(qReal dProbability);

    /*
    Description: Returns the packet drop probability.
    Returns: The packet drop probability that was set with SetPacketDropProbability.
    See Also: SetPacketDropProbability
    */

    /*!
     * @brief パケットロス率を取得します。
     * @return   パケットロス率[0-1]を返します。
     * @see SetPacketDropProbability
     */
    qReal   GetPacketDropProbability();

    //Description: Returns whether or not an emulation device is enabled.
    //Returns: true if the either the input or output emulation device that the method is
    //called on is enabled, false if not or an error occured.
    //Remarks: By default an EmulationDevice is disabled.
    //See Also: Enable

    /*!
     * @brief ネットワークエミュレーションが有効か無効を取得します。
     * @retval true ネットワークエミュレーションが有効です。
     * @retval false ネットワークエミュレーションが無効です。初期値です。
     * @see Enable
     */
    qBool           IsEnabled()
    {
        return m_bEnabled;
    }

    /*
    Description: Enables an emulation device.
    Remarks: This method must be called before either the input or output emulation
    devices are used.

    Due to the way in which the EmulationDevice operates, it cannot be disabled as such. Nevertherless,
    if you wish to disable an EmulationDevice, then you can effectively do so by setting the bandwidth
    very high and both the jitter and latency to 0.
    See Also: IsEnabled
    */

    /*!
     * @brief ネットワークエミュレーションを有効にします。
     *
     * 一度有効にすると無効にはできません。また、各種パラメータを設定する前に有効にする必要があります。
     *
     * @see IsEnabled
     */
    virtual void    Enable()
    {
        m_bEnabled = true;
    };

protected:
    qBool                   m_bEnabled;
    size_t                  m_uiProtocolHeaderSize;
    qUnsignedInt            m_uiBandwidth;
    qUnsignedInt            m_uiLatency;
    qUnsignedInt            m_uiJitter;
    qReal                   m_rPacketDropProbability;
};

END_NAMESPACE_NEX

