﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/StepSequenceJob.h>
#include <OnlineCore/src/Platform/Core/NonCopyable.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>
#include <OnlineCore/src/Platform/Core/ObjectThread.h>


BEGIN_NAMESPACE_NEX
    
class ResolveMethodContainerInterface
{
public:
    ResolveMethodContainerInterface(){}
    virtual ~ResolveMethodContainerInterface(){}
    virtual qBool operator()() = 0;
};

template<typename ResolveMethod>
class ResolveMethodContainer : public RootObject, public ResolveMethodContainerInterface
{
public:
    ResolveMethodContainer(ResolveMethod resolveMethod)
        : m_resolveMethod(resolveMethod){}
    virtual ~ResolveMethodContainer(){}
    virtual qBool operator()()
    {
        return m_resolveMethod();
    }
private:
    ResolveMethod m_resolveMethod;
};

class JobNameResolve : public StepSequenceJob, public NonCopyable
{
public:
    static const qUnsignedInt32 DEFAULT_JOB_DELAY_INTERNAL_THREAD = 10;
    static const qUnsignedInt32 THREAD_RUNNING_CHECK_WAIT = 50;

    template<typename ResolveMethod>
    JobNameResolve(qUnsignedInt threadPriority, qInt32 coreNo, ResolveMethod resolveMethod)
        : StepSequenceJob(Job::Core,NEX_T("JobNameResolve"))
        , m_AssociatedCallID(0)
        , m_result(QRESULT_ERROR(Core, Unknown))
        , m_pThread(NULL)
        , m_lockResult(LockMasks::Ignored)
        , m_threadPriority(threadPriority)
        , m_coreNo(coreNo)
        , m_pResolveMethodContainer(qNew ResolveMethodContainer<ResolveMethod>(resolveMethod))
    {
        // #17880 このJobの途中に行われるDNS解決はNEXのメインスレッドとは別スレッドで動作する。
        // 呼び出し元の一部(JobStartNATSession, JobNATCheckBase)はその途中でRootTransportにアクセスする。
        // 処理が完了するまでの間、RootTransportのインスタンスが解放されないようにNetworkへの参照を維持する。
        // TODO: JobStartNATSession, JobNATCheckBaseでRootTransportを直接使ってDNS解決しないようにする。
        //   DNS解決後に↑のジョブからRootTransportの変数にDNS解決結果を書き込むようにする。
        SYSTEMCHECK(Network::GetInstance() != NULL);
        Network::AcquireInstance();
    }
    virtual ~JobNameResolve();

    void Initialize(qUnsignedInt32 uiAssociatedCallID);


protected:
    void StepRunThread();
    void StepCheck();
    void StepCancelWait();

    void CheckThread(int);

    void CompleteJob(const qResult& result);

    qUnsignedInt32 m_AssociatedCallID;
    qResult m_result;
    ObjectThread<JobNameResolve, int>* m_pThread;
    ForcedCriticalSection m_lockResult;

    qUnsignedInt m_threadPriority;
    qInt32 m_coreNo;
    ResolveMethodContainerInterface* m_pResolveMethodContainer;

private:
    static PseudoGlobalVariable<qBool> g_isJobNameResolveRunning;
};


END_NAMESPACE_NEX


