﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>

BEGIN_NAMESPACE_NEX

class Buffer;
class StationURL;
class NATTraversalExtension;

//group=NAT Traversal Service

/*
Description: This class forms the interface between Rendez-Vous and a client's in-game networking.
Remarks: A NATTransportAdapter defines how messages are sent and received on the game network socket.

If your game performs networking via a means other than Quazal Net-Z, then you need to implement an appropriate
NATTransportAdapter to ensure that Rendez-Vous can correctly perform NAT traversal for the game via your game sockets.
To do so, sub class this class and implement the virtual methods, then register the NATTransportAdapter to the
ConnectivityManager (via ConnectivityManager::RegisterTransportAdapter) so that Rendez-Vous will call into your
NATTransportAdapter class.
Example:
class MyNATTransportAdapter: public NATTransportAdapter {

public:

    MyNATTransportAdapter(MyGameSocket *pSocket) {
        m_pMyGameSocket = pSocket;
        // This is the message that enables my game socket to recognize
        // NATTransportAdapter messages when they are received.
        m_uiMyHeader = 0x80;
    }

    qBool GetPacketHeader(qByte** pBuffer, qUnsignedInt32* uiBufferSize)
    {
        *pBuffer = (qByte*)&m_uiMyHeader;
        *uiBufferSize = sizeof(qUnsignedInt32);
        return true;
    }

    qBool Send(const qByte *pBuffer, qUnsignedInt32 uiSize, const StationURL *pStationURL)
    {
        // Retrieve the IP address of the destination process
        String strIPAddress = pStationURL->GetAddress();
        // Retrieve the port number of the destination process
        qUnsignedInt16 ui16PortNumber = pStationURL->GetPortNumber();
        // Send the buffer to the specified IPAddress/PortNumber using my game socket;
        return m_pMyGameSocket->Send(strIPAddress, ui16PortNumber, pBuffer, uiSize);
    }

    // Don't forget that when I receive a message on MyGameSocket, I need to call
    // MyNATTransportAdapter::Receive for each message that starts with the provided
    // header returned by GetPacketHeader.

public:
    qUnsignedInt32  m_uiMyHeader;
    MyGameSocket*   m_pMyGameSocket;

};

*/
class NATTransportAdapter: public RootObject
{
    //group=Methods
public:
    NATTransportAdapter():m_oPublicURL(NEX_T("")) {}
    virtual     ~NATTransportAdapter() {}


#ifdef FOR_DOC_PURPOSE
    /*
    Description: Returns the packet header that is affixed to messages sent via this NATTransportAdapter.
    Parameters:
    ppBuffer: A pointer to the buffer that contains the packet header.
    uiBufferSize: The size in bytes of the ppBuffer.
    Returns: The packet header that is affixed to messages sent via this NATTransportAdapter.
    Remarks: The packet header is used to indicate that the message is a NAT traversal message. It is prefixed to
    messages sent to your game socket on another station via Send.

    Refer to NATTransportAdapter for an example of this method in use.
    See Also: Send
    */
    virtual qBool GetPacketHeader(qByte** ppBuffer, size_t* uiBufferSize)
    {
        return false;
    }
#endif

    //{secret}
    virtual qBool GetPacketHeader(qByte** /*ppBuffer*/, size_t* /*uiBufferSize*/)
    {
        return false;
    }

    /*
    Description: Defines how the local station sends NAT related messages.
    Parameters:
    refStationURL: The StationURL of the station to whom the message will be sent. The StationURL should be of
    type UDP ("udp:/address=209.236.43.23;port=9103").
    pBuffer: A pointer to the buffer that is to be sent.
    uiBufferSize: The size in bytes of the pBuffer.
    Remarks: Whenever the local station is required to send a NAT related message, either to the Rendez-Vous back end or
    to your game socket on another station, it will call this method. Therefore, you must implement this method in order
    to define how the specified pBuffer should be sent to the specified StationURL via your game socket.

    Any message ( /i.e./ pBuffer) sent via this method to your game socket on another station will be prefixed with the buffer
    returned by GetPacketHeader. This ensures that the receiving station correctly recognizes the message as one related to NAT
    traversal.

    Refer to NATTransportAdapter for an example of this method in use.
    See Also: StationURL, GetPacketHeader
    */
    virtual void Send(const StationURL &refStationURL, const qByte *pBuffer, size_t uiBufferSize, qUnsignedInt8 uiTTL = 0) = 0;

    /*
    Description: Defines how the local station processes received NAT related messages.
    Parameters:
    refStationURL: The StationURL of the station from whom the message was received.
    pBuffer: A pointer to the buffer that is received.
    uiBufferSize: The size in bytes of the pBuffer.
    Remarks: Whenever a NAT traversal related message is received by a station, this method should be called to ensure that
    the message is relayed to Rendez-Vous and that the NAT related data is corectly processed. NAT traversal messages can be
    recognized via their packet header, retrieved via GetPacketHeader.  IMPORTANT: You should strip your packet header
    from the buffer passed to ReceiveMessage.  Not doing so will result in the packet being ignored since the packet
    will be considered invalid and a proper trace will be display.
    See Also: GetPacketHeader
    */
    void Receive(const StationURL &refStationURL, const qByte *pBuffer, size_t uiBufferSize);

    // {secret}
    const StationURL& GetPublicURL()
    {
        return m_oPublicURL;
    }
    //{secret}
    void SetPublicURL(StationURL& refPublicURL)
    {
        m_oPublicURL = refPublicURL;
    }

public:
    //{secret}
    StationURL  m_oPublicURL;
};

END_NAMESPACE_NEX

