﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Transport/Interface/NATRelayInterface.h>
#include <OnlineCore/src/Transport/Interface/URLProbeList.h>
#include <OnlineCore/src/Transport/Interface/NATEchoInterface.h>
#include <OnlineCore/src/Transport/Interface/UnsupportedNATReferee.h>
#include <OnlineCore/src/Transport/Interface/StationProbeList.h>
#include <OnlineCore/src/Transport/Interface/NATTraversalResult.h>
#include <OnlineCore/src/Transport/Interface/ConnectivityTester.h>
#include <OnlineCore/src/Core/CallContext.h>

BEGIN_NAMESPACE_NEX

class NATTransportAdapter;
class NATTraversalStream;
class RootTransport;
class CallContext;
class JobCheckAndRelay;

class NATTraversalEngine : public RootObject 
{
    enum Msg
    {
        Probe = 0,
        ProbeReply = 1,
        EchoURL = 2
    };

public:

    /* =========================
      重要なメソッド
     ========================= */
    // 他のモジュールから NAT トラバーサル要求を受けるためのメソッド(StationURL を書きかえるのでコピー渡し)
    qBool PrepareNATTraversal(StationURL urlTargetStation);
    qBool PrepareNATTraversal(const qList<StationURL> &lstTargetURL);

    // RV から NAT トラバーサル要求を受信
    void ProcessRequestProbe(const StationURL &oURL);

    void ReceiveMessage(const StationURL &refStationURL, const qByte *pBuffer, size_t uiSize);

    // NAT越え中のProbeRequestやProbe、完了情報を全てクリアする
    void ClearProbeList();

    //ダミーパケット
    void SendDummyPacket(qUnsignedInt8 uiTTL = 0);
    void StartSendDummyPacket();
    void StopSendDummyPacket();

    //P2Pの接続相手にダミーパケットを投げる場合
    void SendDummyPacket(const StationURL& urlTarget, qUnsignedInt8 uiTTL);

    //中継サーバにダミーパケットを投げる
    void SendDummyPacket(const InetAddress& target);

    //現在の環境で同時送信できるProbeの数を取得する
    qUnsignedInt GetNumOfMaxProbes();

    //現在送信中の外側Probeの数を取得する
    qUnsignedInt GetNumOfSendingProbes();

    //タイプとURLを指定してProbeを送信する
    void SendProbe(NATTraversalEngine::Msg eMsgType, const StationURL &refStationURL, Time tiTime, qUnsignedInt8 uiTTL = 0);
    //Probeを送信する
    void SendProbes();
    //送信が不要になったProbeを止める
    void StopProbes();
    //Probeの送信を開始する
    void StartSendProbe(const StationURL &urlTarget);
    //ProbeRequestを送信する
    void SendProbeRequests();
    //ProbeRequestのタイムアウト判定
    void TimeoutProbeRequest(const Time& tiCurrent);

    qBool GetUpdatedURL(const StationURL& urlTargetStation, StationURL *pUpdatedURL);
    void CheckPortAndRelay(const qList<StationURL> &lstTarget, const StationURL &urlRequester);

    /* =========================
      実装しないとダメなメソッド
     ========================= */
    NATTraversalEngine();
    virtual ~NATTraversalEngine();

    qBool StartNATStream(RootTransport *pTransport = NULL);
    qBool StopNATStream();

    //RVNATRelayを受け渡しするためのメソッド(NATTraversalClientから呼ばれる)
    virtual qBool RegisterRelay(NATRelayInterface *pRelay);
    virtual qBool UnregisterRelay();
    NATRelayInterface* GetRelay() const
    {
        return m_pRelay;
    }

    //RVNATEchoを受け渡しするためのメソッド
    virtual qBool RegisterEcho(NATEchoInterface *pEcho);
    virtual qBool UnregisterEcho();

    //NATTraversalClient から移動
    virtual qBool RegisterConnectivityTester(ConnectivityTester* pTester);
    virtual qBool UnregisterConnectivityTester();

    //CID
    void SetLocalCID(const ConnectionID cidLocal)
    {
        m_cidLocal = cidLocal;
    }
    ConnectionID GetLocalCID() const
    {
        return m_cidLocal;
    }

    //トランスポートアダプタ
    qBool RegisterTransportAdapter(NATTransportAdapter *pTransportAdapter);
    qBool UnregisterTransportAdapter();

    // NAT トラバーサル失敗/成功報告
    UnsupportedNATReferee* GetUnsupportedNATReferee()
    {
        return &m_UnsupportedNATReferee;
    }
    void ReportNATTraversalResult(ConnectionID cid, qBool bResult, qUnsignedInt32 rtt = 0);

    // NATタイプ判定報告
    void ReportNATProperties(NATProperties::MappingProperties mapping, NATProperties::FilteringProperties filtering, qUnsignedInt32 rtt = 0);

    //
    qBool Initialize()
    {
        return true;
    }
    qBool Terminate();

    //Probeの生存期間
    static void SetProbeLifetime(TimeInterval tiProbeLifetime)
    {
        s_tiProbeLifetime = tiProbeLifetime;
    }
    static void SetProbeFrequency(TimeInterval tiFrequency)
    {
        s_tiFrequency = tiFrequency;
    }
    static void SetProbeKeepAlive(TimeInterval tiProbeKeepAlive)
    {
        s_tiProbeKeepAlive = tiProbeKeepAlive;
    }
    static TimeInterval GetProbeLifetime()
    {
        return s_tiProbeLifetime;
    }
    static TimeInterval GetProbeFrequency()
    {
        return s_tiFrequency;
    }
    static TimeInterval GetProbeKeepAlive()
    {
        return s_tiProbeKeepAlive;
    }

    void SetRelayServerAddress(const String& relayServerAddress)
    {
        m_RelayServerAddress = relayServerAddress;
    }

    const String& GetRelayServerAddress() const
    {
        return m_RelayServerAddress;
    }

    void SetRelayServerPort(qUnsignedInt16 relayServerPort)
    {
        m_RelayServerPort = relayServerPort;
    }

    qUnsignedInt16 GetRelayServerPort() const
    {
        return m_RelayServerPort;
    }

    /*! @internal
     *  @brief 中継サーバ強制利用フラグを指定します
     *
     *  通常、一度 NAT トラバーサルに失敗した後に中継サーバを介した NAT トラバーサルを試みますが、
     *  このフラグが有効である(true)場合、初回から中継サーバを使用するようになります。
     *  自分が Target である場合も、中継サーバを指示します(Requester が中継サーバを指示した場合はそちらを優先)
     *
     *  アプリケーション側から指定する以外に、EDM でかつ NAT トラバーサルに失敗した場合にライブラリ内でフラグが有効化されます。
     *
     *  このフラグは @ref ConnectivityManager::StopNatSession() で自動的に無効化されます。
     *  アプリケーション側でこのフラグを有効化する場合にはセッション毎に呼び出すようにしてください。
     *
     *  @param[in] forceToUseRelayFlag 中継サーバ強制利用フラグ(true : 有効, false : 無効)
     */
    void SetForceToUseRelay(qBool forceToUseRelayFlag)
    {
        m_ForceToUseRelay = forceToUseRelayFlag;
    }

    qBool GetForceToUseRelay() const
    {
        return m_ForceToUseRelay;
    }

    // 中継サーバを利用すべきかどうか
    qBool ShouldUseRelay(const StationURL& url, qBool checkURL = true) const;

    // 中継サーバを利用できるかどうか
    qBool CanUseRelay(const StationURL& url, qBool checkURL = true) const;

    //
    qUnsignedInt32 GetURLPingTime(const StationURL& urlTargetStation);
    qUnsignedInt32 GetCIDPingTime(const ConnectionID cid);
    //失敗の早期検出
    qBool GetProbeFailed(const StationURL& urlTargetStation);

    NATTraversalResult GetNATTraversalResult(ConnectionID cid);

    //時間毎の処理
    void Execute();

    /* =========================
      使っていないのでとりあえず未実装なメソッド
     ========================= */
    qBool FetchPublicURL(CallContext *, StationURL::URLType)
    {
        SYSTEMCHECK(false && NEX_T("NOT IMPLEMENTED"));
        return false;
    }

    qBool GetPublicURL(StationURL::URLType, StationURL *)
    {
        SYSTEMCHECK(false && NEX_T("NOT IMPLEMENTED"));
        return false;
    }

    void SendEchoRequest()
    {
        SYSTEMCHECK(false && NEX_T("NOT IMPLEMENTED"));
    }

    void SetJobGetPublicURLDone()
    {
        SYSTEMCHECK(false && NEX_T("NOT IMPLEMENTED"));
    }

    // for internal
    static qUnsignedInt GetMaxProbesDefault()
    {
        return  g_uiMAX_PROBES_DEFAULT;
    }
    void SetMaxProbesDefault(qUnsignedInt value)
    {
        g_uiMAX_PROBES_DEFAULT = value;
    }
    static qUnsignedInt8 GetLowTTLValue()
    {
        return g_ui8LowTTLValue;
    }
    void SetLowTTLValue(qUnsignedInt8 value)
    {
        g_ui8LowTTLValue = value;
    }
    static qUnsignedInt16 GetDummyPortNumber()
    {
        return g_ui16DUMMY_PORT_NUMBER;
    }
    void SetDummyPortNumber(qUnsignedInt16 value)
    {
        g_ui16DUMMY_PORT_NUMBER = value;
    }
    static qUnsignedInt32 GetSendDummyPacketInterval()
    {
        return  g_uiSendDummyPacketInterval;
    }
    void SetSendDummyPacketInterval(qUnsignedInt32 value)
    {
        g_uiSendDummyPacketInterval = value;
    }
    static qUnsignedInt32 GetProbeRequestTimeout()
    {
        return g_uiProbeRequestTimeout;
    }
    void SetProbeRequestTimeout(qUnsignedInt32 value)
    {
        g_uiProbeRequestTimeout = value;
    }
    static qUnsignedInt32 GetNumberOfDummyPacket()
    {
        return  g_uiNUMBER_OF_DUMMY_PACKET;
    }
    void SetNumberOfDummyPacket(qUnsignedInt32 value)
    {
        g_uiNUMBER_OF_DUMMY_PACKET = value;
    }

private:
    UnsupportedNATReferee  m_UnsupportedNATReferee;
    NATRelayInterface     *m_pRelay;
    NATEchoInterface      *m_pEcho;
    NATTraversalStream    *m_pStream;
    ConnectionID           m_cidLocal;
    NATTransportAdapter   *m_pTransportAdapter;

    /* =========================
      重要なデータ
     ========================= */
    // 「ProbeRequest送信予定リスト」
    qList<StationURL> m_lstProbeRequest;

    // 「Probe送信中リスト」
    // NATトラバーサルを要求されたオリジナルのURLとIPアドレスが同じである必要がある。
    // (オリジナルIPアドレスと同じものが
    //  m_lstProbeRequest, m_mpWaitingProbeRequest, m_lstURLProbeSending, m_lstURLProbeFinished
    //  のいずれにもない場合は失敗と判定される。NATTraversalEngine::GetProbeFailed()参照。)
    URLProbeList           m_lstURLProbeSending;

    // 「Probe送信完了リスト」
    // Probeが成功したものは GetPingTime() != Infinite になる。
    // Probeが失敗したものは GetPingTime() == Infinite になる。
    // NATトラバーサルを要求されたオリジナルのURLとIPアドレスが同じである必要がある。
    URLProbeList           m_lstURLProbeFinished;

    Time m_tNextProbeTime;
    JobCheckAndRelay *m_pJobCheckAndRelay;
    CallContext m_oCallContext;
    qBool m_bStartSendDummyPacket;
    Time m_tNextSendDummyPacket;
    Time m_tNextCleanupTime;

    qUnsignedInt32 m_uiCurrentProbeRequestID;

    // 「ProbeRequest返信待ちリスト」
    // 自分がRequesterとして送信したProbeRequestのリスト
    typedef std::pair<Time, StationURL> WaitingProbeRequestEntry;
    qMap<qUnsignedInt32, WaitingProbeRequestEntry> m_mpWaitingProbeRequest;

    String m_RelayServerAddress;
    qUnsignedInt16 m_RelayServerPort;
    qBool m_ForceToUseRelay;

    qList<ConnectionID> m_tempCidStopList;    // Probeに成功したRVCIDを入れるリスト。StopProbes()で毎回生成すると負荷になるのでメンバに持つ。
    qList<StationURL> m_tempRelayUrlList;     // Probeに失敗し、かつ中継サーバが有効なURLのリストStopProbes()で毎回生成すると負荷になるのでメンバに持つ。

    // NAT Traversalの共通パラメータ
    static TimeInterval s_tiProbeLifetime;
    static TimeInterval s_tiFrequency;
    static TimeInterval s_tiProbeKeepAlive;

    //EIMの場合に同時送信できるProbeの最大数
    static qUnsignedInt g_uiMAX_PROBES_DEFAULT;
    static qUnsignedInt8 g_ui8LowTTLValue;
    static qUnsignedInt16 g_ui16DUMMY_PORT_NUMBER;
    static qUnsignedInt32 g_uiSendDummyPacketInterval;
    static TimeInterval g_uiCleanupInterval;

    //ProbeRequestの返信待ち時間
    static qUnsignedInt32 g_uiProbeRequestTimeout;
    static qUnsignedInt32 g_uiNUMBER_OF_DUMMY_PACKET;
};

END_NAMESPACE_NEX

