﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Stack/Core/InetDecl.h>
#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/NATProperties.h>
#include <OnlineCore/src/Transport/Interface/Network.h>
#include <OnlineCore/src/Transport/Interface/PacketRecognizer.h>
#include <OnlineCore/src/Transport/Interface/TransportPerfCounters.h>
#include <OnlineCore/src/Transport/Interface/OutputEmulationDevice.h>
#include <OnlineCore/src/Transport/Interface/InputEmulationDevice.h>
#include <OnlineCore/src/Transport/Interface/TransportStreamManager.h>
#include <OnlineCore/src/Transport/Interface/UPnPProperties.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Core/SystemPorts.h>

BEGIN_NAMESPACE_NEX

class Stream;
class Buffer;
class PacketOut;
class Packet;
class VirtualNATDevice;
#if NEX_ENABLE_VIRTUAL_FILTER_DEVICE
class VirtualFilterDevice;
#endif

#ifdef NEX_ROUTING_SUPPORT
class Router;
#endif
class Relay;

//group=Transport Protocol

/*
Description: This singleton is the base class of the $PRODUCTNAME$ transport layer.
$ifdef NETZ$
Remarks: Sets various transport parameters such as the connection timeout and well-known
port number. Can also set and retrieve various network parameters for third party
networking engines, for example.
$else$
Remarks: Sets various Network and NAT parameters. Can also set and retrieve various
network parameters for third party networking engines, for example.
See Also: NATTraversalClient, NATProperties, ConnectivityManager,
$endif$
*/

/*!
 * @brief   タイムアウトや Transport に関する各種設定を行うためのクラスです。
 * このクラスは接続/切断/NAT トラバーサルのタイムアウトや、Transport のチューニングパラメータを設定します。
 */
class RootTransport: public RootObject
{
public:
    friend class Network;

    //n1589:Stream共通の通信用パラメータはここで宣言
public:

    enum
    {
        DEFAULT_TIMEOUT_NAT_TRAVERSAL = (60 * 1000),
        DEFAULT_TIMEOUT_CONNECT_END_POINT = (15 * 1000),
        DEFAULT_TIMEOUT_DISCONNECT_END_POINT = (15 * 1000),
        /*
        n1589:このターミネートタイムアウトは何らかの致命的なエラーによりDOのターミネート処理を進められなくなった場合に
        強制中断するためのものです。
        その場合、致命的なエラーが発生します。
        上の切断タイムアウトよりも十分に長くする必要があります。
        */
        DEFAULT_TIMEOUT_TERMINATE_DO_CORE = (30 * 1000),

        DEFAULT_MTU = SocketConstants::DEFAULT_MTU,

#ifdef FPD_BUILD
        DEFAULT_WELL_KNOWN_PORT_NUMBER = 9103,
#else
        DEFAULT_WELL_KNOWN_PORT_NUMBER = 9203,
#endif

        DEFAULT_TIMEOUT_TERMINATE_IMMEDIATELY = 2000,

#ifdef NON_SUPPORT_B_TARGET
        DEFAULT_IO_WAIT_TIME = 50,
#else
        //n1589:todo1:DEFAULT_IO_WAIT_TIME = 10
        DEFAULT_IO_WAIT_TIME = 10,
#endif
#ifdef WIN32_TARGET
        DEFAULT_PACKET_TTL = 128,
#else // WIN32_TARGET
        DEFAULT_PACKET_TTL = 64,
#endif // WIN32_TARGET

#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
        /*! デフォルト無効 */
        DEFAULT_ROUTING_HISTORY_UPDATE_SPAN = 0,
        DEFAULT_ROUTING_HISTORY_TTL = 10000,
        DEFAULT_ROUTING_LIMIT = 0,
#endif

        DEFAULT_RECEIVE_BUFFER_SIZE = 8192,
        DEFAULT_SEND_BUFFER_SIZE    = 8192,

        DEFAULT_IDLE_SLEEP_TIME = 0,

        /*! トランスポートバッファスレッドで使用するメモリプールの数 */
        DEFAULT_TRANSPORT_BUFFER_POOL_NUM = 64,
        /*! トランスポートバッファスレッドのデフォルトの優先度 */
        DEFAULT_TRANSPORT_BUFFER_THREAD_PRIORITY = NEX_THREAD_PRIORITY_HIGH,
        /*! トランスポートバッファスレッドがデフォルトで使う CPU CORE NO */
        DEFAULT_TRANSPORT_BUFFER_CORE_NO = ObjectThreadRoot::DEFAULT_CORE_NO,
        /*! RootTransport::SetUpDefaultNATCheckServerInternalでDNSを解決するために使うスレッドのデフォルトプライオリティ*/
        DEFAULT_NAME_RESOLVE_THREAD_PRIORITY = NEX_THREAD_PRIORITY_HIGH,
        /*! RootTransport::SetUpDefaultNATCheckServerInternalでDNSを解決するために使うスレッドのデフォルトCPU CORE NO */
        DEFAULT_NAME_RESOLVE_THREAD_CORE_NO = ObjectThreadRoot::DEFAULT_CORE_NO,
        /*! PacketHistoryBundlingで履歴として保持する時間[msec]}*/
        DEFAULT_PACKET_HISTORY_BUNDLING_HOLDING_TIME = 182,
        /*! PacketHistoryBundlingで履歴として保持する最大パケット数*/
        DEFAULT_PACKET_HISTORY_BUNDLING_MAX_HOLDING_PACKETS = 32,
        /*! PacketHistoryBundlingでバンドリングして送信する過去履歴の最大数*/
        DEFAULT_PACKET_HISTORY_BUNDLING_MAX_RETRANSMIT_PACKTES = QINT16_MAX,
        /*! PacketHistoryBundlingでバンドリングして送信する再送回数の最大数*/
        DEFAULT_PACKET_HISTORY_BUNDLING_MAX_RETRANSMIT_COUNTS = 2,
        /*! PacketHistoryBundlingでバンドリングされなかったパケットを強制再送するまでの保留時間*/
        DEFAULT_PACKET_HISTORY_BUNDLING_FORCED_RETRANSMIT_TIMEOUT = 150,
        /*! PacketHistoryBundlingで1パケットにバンドリングするパケットサイズの上限*/
        DEFAULT_PACKET_HISTORY_BUNDLING_MAXIMUM_PACKET_BYTES = RootTransport::DEFAULT_MTU
    };

    /*!
     * @brief PacketHistoryBundling用のパケット数の統計種別値
     *
     * GetPacketHistoryBundlingPacketCounts()が返す配列の添え字、
     * GetPacketHistoryBundlingPacketCount()の引数として利用する値です。
     */
    enum PacketHistoryCount
    {
        /*! @brief バンドルしないと判断された回数 */
        PACKET_HISTORY_UNBUNDLING_COUNT = 0,
        /*! @brief キューにバンドリング対象Packetがなく、バンドリングできなかった回数 */
        PACKET_HISTORY_BUNDLING_COUNT_NONE,
        /*! @brief 1パケット以上がバンドリングされた回数 */
        PACKET_HISTORY_BUNDLING_COUNT_ONE_AND_MORE,
        /*! @brief SetPacketHistoryBundlingMaxRetransmitPackets値/2分のパケット以上がバンドリングされた回数 */
        PACKET_HISTORY_BUNDLING_COUNT_A_HALF_AND_MORE,
        /*! @brief SetPacketHistoryBundlingMaxRetransmitPackets値分のパケットがバンドリングされた回数 */
        PACKET_HISTORY_BUNDLING_COUNT_MAX,
        /*! @brief 総送信パケット数
         * PACKET_HISTORY_UNBUNDLING_COUNT + PACKET_HISTORY_BUNDLING_COUNT_NONE +
         * PACKET_HISTORY_BUNDLING_COUNT_ONE_AND_MOREの和と等しいです。*/
        PACKET_HISTORY_TOTAL_COUNT,
#ifndef FOR_DOC_PURPOSE
        /*! @internal enum値の最大値を示します。 */
        MAX_PACKET_HISTORY_COUNT = PACKET_HISTORY_TOTAL_COUNT
#endif
    };

    // 伝送路の状態
    // Socket::SocketStatusと同じ定義
    enum TransportStatus
    {
        TRANSPORT_STATUS_AVAILABLE,                                 // 利用可能な状態
        TRANSPORT_STATUS_DISCONNECTED_GENERAL,                      // 切断された状態
        TRANSPORT_STATUS_DISCONNECTED_UNDER_MAINTENANCE,            // 切断された状態(メンテナンス中エラーが返された)
        TRANSPORT_STATUS_DISCONNECTED_SERVICE_NO_LONGER_AVAILABLE,  // 切断された状態(サービス終了エラーが返された)
        TRANSPORT_STATUS_DISCONNECTED_APPLICATION_VERSION_IS_OLD,   // 切断された状態(アプリケーションバージョンエラーが返された)
        TRANSPORT_STATUS_DISCONNECTED_OUT_OF_SERVICE,               // 切断された状態(サービス休止中エラーが返された)
        TRANSPORT_STATUS_DISCONNECTED_MAX_CONNECTIONS_REACHED        // 切断された状態(コネクション数限界が返された)
    };

    static const qUnsignedInt32 ADDRESS_CHAR_SIZE=128;

    static const qUnsignedInt16 DEFAULT_RELAY_SERVER_PORT;

    /*!
     * @brief   NAT トラバーサルのタイムアウト時間を取得します。
     * @return NAT トラバーサルのタイムアウト時間が返ります。
     */
    static qUnsignedInt32 GetNATTraversalTimeout()
    {
        return s_uiNATTraversalTimeout;
    }

    /*!
     * @brief   NAT トラバーサルのタイムアウト時間を設定します。
     * ゲームに参加できる最大人数×[3000-5000]を目安に設定してください。
     *
     * @param[in] o タイムアウトをミリ秒単位で指定します。
     */
    static void SetNATTraversalTimeout(qUnsignedInt32 o)
    {
        s_uiNATTraversalTimeout = o;
    }

    static qUnsignedInt32 GetConnectEndPointTimeout()
    {
        return s_uiConnectEndPointTimeout;
    }

    static void SetConnectEndPointTimeout(qUnsignedInt32 o)
    {
        s_uiConnectEndPointTimeout = o;
    }

    static qUnsignedInt32 GetDisconnectEndPointTimeout()
    {
        return s_uiDisconnectEndPointTimeout;
    }

    static void SetDisconnectEndPointTimeout(qUnsignedInt32 o)
    {
        if(o >= s_uiTerminateDOCoreTimeout - 5000)
        {
            _QLOG(
                EventLog::Warning,
                NEX_T("DisconnectEndPointTimeout(%d) should be shorter than TerminateDOCoreTimeout(%d) - 5000"),
                o,
                s_uiTerminateDOCoreTimeout
            );
        }
        s_uiDisconnectEndPointTimeout = o;
    }

    /*!
     * @internal
     * @brief NetZオブジェクトの終了処理におけるタイムアウト時間を取得します。
     *
     * @return タイムアウト時間がミリ秒単位で返ります。
     * @see NetZ::Terminate, SetTerminateDOCoreTimeout
     */
    static qUnsignedInt32 GetTerminateDOCoreTimeout()
    {
        return s_uiTerminateDOCoreTimeout;
    }


    /*!
     * @internal
     * @brief NetZオブジェクトの終了処理におけるタイムアウト時間を設定します。
     *
     *
     * @param[in] o タイムアウト時間をミリ秒単位で指定します。
     * @see NetZ::Terminate, GetTerminateDOCoreTimeout
     */
    static void SetTerminateDOCoreTimeout(qUnsignedInt32 o)
    {
        if(o <= s_uiDisconnectEndPointTimeout + 5000)
        {
            _QLOG(
                EventLog::Warning,
                NEX_T("TerminateDOCoreTimeout(%d) should be longer than DisconnectEndPointTimeout(%d) + 5000"),
                o,
                s_uiDisconnectEndPointTimeout
            );
        }
        s_uiTerminateDOCoreTimeout = o;
    }


    static void SetTerminateImmediatelyTimeout(qUnsignedInt32 o)
    {
        s_uiTerminateImmediatelyTimeout = o;
    }
    static qUnsignedInt32 GetTerminateImmediatelyTimeout()
    {
        return s_uiTerminateImmediatelyTimeout;
    }

    static void EnableTransportBufferThread(qBool o)
    {
        Core::UseTransportBufferThread(o);
    }

    static qBool IsEnabledTransportBufferThread()
    {
        return Core::IsUsingTransportBufferThread();
    }

    /*!
     * @internal
     * @brief トランスポートバッファスレッドで使用されるメモリを予め確保するかどうかを設定します。
     *
     * true を設定することで、トランスポートバッファスレッドでメモリの確保・開放を行いません。
     * false を設定すると、トランスポートバッファスレッドでメモリの確保・開放が行われます。
     * スレッドアンセーフなメモリアロケーターを使う場合に、トランスポートバッファスレッドを有効にするときにはかならず true を設定してください。
     *
     * @param[in] o トランスポートバッファスレッドで必要なメモリを予め確保するかどうかを設定します。デフォルトは true です。
     * @see GetTransportBufferPoolNum, SetTransportBufferPoolNum, IsEnabledTransportBufferThreadBufferPool
     */
    static void EnableTransportBufferThreadBufferPool(qBool o)
    {
        s_bEnableTransportBufferThreadBufferPool = o;
    }

    /*!
     * @internal
     * @brief トランスポートバッファスレッドで使用されるメモリを予め確保するかどうかを取得します。
     *
     * @return トランスポートバッファスレッドで使用されるメモリを予め確保するかどうかを返します。デフォルトは true です。
     * @see GetTransportBufferPoolNum, SetTransportBufferPoolNum, EnableTransportBufferThreadBufferPool
     */
    static qBool IsEnabledTransportBufferThreadBufferPool()
    {
        return s_bEnableTransportBufferThreadBufferPool;
    }

    /*!
     * @brief トランスポートバッファスレッドで使用されるバッファのためのメモリプール数を返します。
     *
     * @return メモリプール数。デフォルトは64です。
     * @see SetTransportBufferPoolNum
     */
    static size_t GetTransportBufferPoolNum()
    {
        return s_TransportBufferPoolNum;
    }

    /*!
     * @brief トランスポートバッファスレッドで使用されるバッファのためのメモリプール数を設定します。
     *
     * メモリプール１つのサイズは
     * MTU(Maximum Transmission Unit : ネットワークにおいて1回の転送に送信できる最大サイズ)
     * です。
     * MTUサイズのメモリが、本関数で指定したメモリプール数ほど確保されます。
     * なお、このメモリはNetZ生成時に確保され、NetZ破棄時に解放されます。
     *
     * @param[in] poolNum  メモリプール数。デフォルトは64です。
     * @see GetTransportBufferPoolNum
     */
    static void SetTransportBufferPoolNum(qUnsignedInt poolNum)
    {
        s_TransportBufferPoolNum = poolNum;
    }

    /*!
     * @brief トランスポートバッファスレッドの受信用スレッドの優先度を返します。
     *
     * @return 優先度
     *
     * @see SetTransportBufferThreadSendPriority, GetTransportBufferThreadSendPriority, SetTransportBufferThreadRecvPriority, GetTransportBufferThreadRecvPriority, SetTransportBufferThreadPriority
     */
    static qUnsignedInt GetTransportBufferThreadPriority()
    {
        return s_TransportBufferThreadPriority;
    }

    /*!
     * @brief トランスポートバッファスレッドの送信用と受信用のスレッド優先度をまとめて設定します。
     *
     * @param[in] priority 優先度
     *
     * @see SetTransportBufferThreadSendPriority, GetTransportBufferThreadSendPriority, SetTransportBufferThreadRecvPriority, GetTransportBufferThreadRecvPriority, GetTransportBufferThreadPriority
     */
    static void SetTransportBufferThreadPriority(qUnsignedInt priority)
    {
        s_TransportBufferThreadPriority = priority;
        s_TransportBufferThreadSendPriority = priority;
    }

    /*!
     * @brief トランスポートバッファスレッドの受信用優先度を返します。
     *
     * @return 優先度
     *
     * @see SetTransportBufferThreadSendPriority, GetTransportBufferThreadSendPriority, SetTransportBufferThreadRecvPriority, SetTransportBufferThreadPriority, GetTransportBufferThreadPriority
     */
    static qUnsignedInt GetTransportBufferThreadRecvPriority()
    {
        return s_TransportBufferThreadPriority;
    }

    /*!
     * @brief トランスポートバッファスレッドの受信用優先度を設定します。
     *
     * @param[in] priority 優先度
     *
     * @see SetTransportBufferThreadSendPriority, GetTransportBufferThreadSendPriority, GetTransportBufferThreadRecvPriority, SetTransportBufferThreadPriority, GetTransportBufferThreadPriority
     */
    static void SetTransportBufferThreadRecvPriority(qUnsignedInt priority)
    {
        s_TransportBufferThreadPriority = priority;
    }

    /*!
     * @brief トランスポートバッファスレッドの送信用優先度を返します。
     *
     * @return 優先度
     *
     * @see SetTransportBufferThreadSendPriority, SetTransportBufferThreadRecvPriority, GetTransportBufferThreadRecvPriority, SetTransportBufferThreadPriority, GetTransportBufferThreadPriority
     */
    static qUnsignedInt GetTransportBufferThreadSendPriority()
    {
        return s_TransportBufferThreadSendPriority;
    }

    /*!
     * @brief トランスポートバッファスレッドの送信用優先度を設定します。
     *
     * @param[in] priority 優先度
     *
     * @see GetTransportBufferThreadSendPriority, SetTransportBufferThreadRecvPriority, GetTransportBufferThreadRecvPriority, SetTransportBufferThreadPriority, GetTransportBufferThreadPriority
     */
    static void SetTransportBufferThreadSendPriority(qUnsignedInt priority)
    {
        s_TransportBufferThreadSendPriority = priority;
    }

    /*!
     * @brief トランスポートバッファスレッドが利用する CPU の コア番号を設定します。
     *
     * DEFAULT_TRANSPORT_BUFFER_CORE_NO が、デフォルト値です。
     * CTR では、デフォルト値を使用した場合は CORE_NO_USE_PROCESS_VALUE が設定されます。
     *
     * SDK 3.0 以降の CTR では、1 を指定することによりシステムコアを選択できます。
     * システムコアの利用の際には、nn::os::CTR::SetApplicationCpuTimeLimit() で、CPU時間を割り当てる必要があります。
     * システムコアの利用は、動作保証外です。
     * 
     *
     * 
     *
     * @param[in] no CPU コア番号
     */
    static void SetTransportBufferThreadCoreNo(qInt32 no)
    {
        s_TransportBufferThreadCoreNo = no;
    }

    /*!
     * @brief トランスポートバッファスレッドが利用する CPU の コア番号を取得します。
     *
     * @return CPU コア番号
     * DEFAULT_TRANSPORT_BUFFER_CORE_NO が、デフォルト値です。
     * 他の値はプラットフォームの SDK の定義に従います。
     */
    static qInt32 GetTransportBufferThreadCoreNo()
    {
        return s_TransportBufferThreadCoreNo;
    }

    /*!
     * @brief 名前解決用スレッドの優先度を返します。
     * @return 優先度
     *
     * @see ConnectivityManager::StartNATSession(), SetNameResolveThreadPriority()
     * @see SetNameResolveThreadCoreNo(), GetNameResolveThreadCoreNo()
     */
    static qUnsignedInt GetNameResolveThreadPriority()
    {
        return s_NameResolveThreadPriority;
    }

    /*!
     * @brief 名前解決用スレッドの優先度を設定します。
     *
     * @param[in] priority 優先度
     *
     * @see ConnectivityManager::StartNATSession(), GetNameResolveThreadPriority()
     * @see SetNameResolveThreadCoreNo(), GetNameResolveThreadCoreNo()
     */
    static void SetNameResolveThreadPriority(qUnsignedInt priority)
    {
        s_NameResolveThreadPriority = priority;
    }

    /*!
     * @brief 名前解決用スレッドが利用する CPU の コア番号を設定します。
     *
     * DEFAULT_NAME_RESOLVE_THREAD_CORE_NO が、デフォルト値です。
     * CTR では、デフォルト値を使用した場合は CORE_NO_USE_PROCESS_VALUE が設定されます。
     *
     * SDK 3.0 以降の CTR では、1 を指定することによりシステムコアを選択できます。
     * システムコアの利用の際には、nn::os::CTR::SetApplicationCpuTimeLimit() で、CPU時間を割り当てる必要があります。
     * システムコアの利用は、動作保証外です。
     * 
     *
     * 
     *
     * @param[in] no CPU コア番号
     *
     * @see ConnectivityManager::StartNATSession(), SetNameResolveThreadCoreNo()
     * @see SetNameResolveThreadPriority(), GetNameResolveThreadPriority()
     */
    static void SetNameResolveThreadCoreNo(qInt32 no)
    {
        s_NameResolveThreadCoreNo = no;
    }

    /*!
     * @brief 名前解決用スレッドが利用する CPU の コア番号を取得します。
     *
     * @return CPU コア番号
     * DEFAULT_NAME_RESOLVE_THREAD_CORE_NO が、デフォルト値です。
     * 他の値はプラットフォームの SDK の定義に従います。
     *
     * @see ConnectivityManager::StartNATSession(), GetNameResolveThreadCoreNo()
     * @see SetNameResolveThreadPriority(), GetNameResolveThreadPriority()
     */
    static qInt32 GetNameResolveThreadCoreNo()
    {
        return s_NameResolveThreadCoreNo;
    }

// WebSocket対応プラットフォームのみドキュメントに出す
#ifdef NEX_TRANSPORT_PROTOCOL_TYPE_WEB_SOCKET_SUPPORT
    /*!
     * @brief ゲームサーバーとの接続用スレッドの優先度を返します。
     *
     * @return 優先度
     *
     * @see SetConnectThreadPriority()
     */
#endif
    static qUnsignedInt GetConnectThreadPriority()
    {
        return SocketDriver::GetConnectThreadPriority();
    }

#ifdef NEX_TRANSPORT_PROTOCOL_TYPE_WEB_SOCKET_SUPPORT
    /*!
     * @brief ゲームサーバーとの接続用スレッドの優先度を設定します。
     *
     * @param[in] priority 優先度
     *
     * @see GetConnectThreadPriority()
     * @see GetConnectThreadCoreNo(), SetConnectThreadCoreNo()
     */
#endif
    static void SetConnectThreadPriority(qUnsignedInt priority)
    {
        SocketDriver::SetConnectThreadPriority(priority);
    }

#ifdef NEX_TRANSPORT_PROTOCOL_TYPE_WEB_SOCKET_SUPPORT
    /*!
     * @brief ゲームサーバーとの接続用スレッドが利用する CPU の コア番号を設定します。
     *
     * ObjectThreadRoot::DEFAULT_CORE_NO が、デフォルト値です。
     * CTR では、デフォルト値を使用した場合は CORE_NO_USE_PROCESS_VALUE が設定されます。
     *
     * SDK 3.0 以降の CTR では、1 を指定することによりシステムコアを選択できます。
     * システムコアの利用の際には、nn::os::CTR::SetApplicationCpuTimeLimit() で、CPU時間を割り当てる必要があります。
     * システムコアの利用は、動作保証外です。
     * 
     *
     * 
     *
     * @param[in] no CPU コア番号
     *
     * @see SetConnectThreadPriority(), GetConnectThreadPriority()
     * @see GetConnectThreadCoreNo()
     */
#endif
    static void SetConnectThreadCoreNo(qInt32 no)
    {
        SocketDriver::SetConnectThreadCoreNo(no);
    }

#ifdef NEX_TRANSPORT_PROTOCOL_TYPE_WEB_SOCKET_SUPPORT
    /*!
     * @brief ゲームサーバーとの接続用スレッドが利用する CPU の コア番号を取得します。
     *
     * @return CPU コア番号
     * ObjectThreadRoot::DEFAULT_CORE_NO が、デフォルト値です。
     * 他の値はプラットフォームの SDK の定義に従います。
     *
     * @see SetConnectThreadPriority(), GetConnectThreadPriority()
     * @see SetConnectThreadCoreNo()
     */
#endif
    static qInt32 GetConnectThreadCoreNo()
    {
        return SocketDriver::GetConnectThreadCoreNo();
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送する機能のON/OFFを返します。
     *
     * @return 有効・無効フラグ
     * @see EnablePacketHistoryBundling
     */
#endif
    static qBool IsEnablePacketHistoryBundling()
    {
        return s_bEnablePacketHistoryBundling;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたReliableパケットをバンドリングするか否かを返します。
     *
     * @return 有効・無効フラグ
     * @see EnableReliablePacketHistoryBundling
     */
#endif
    static qBool IsEnableReliablePacketHistoryBundling()
    {
        return s_bEnableReliablePacketHistoryBundling;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたUnreliableパケットをバンドリングするか否かを返します。
     *
     * @return 有効・無効フラグ
     * @see EnableUnreliablePacketHistoryBundling
     */
#endif
    static qBool IsEnableUnreliablePacketHistoryBundling()
    {
        return s_bEnableUnreliablePacketHistoryBundling;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られた DO のパケットをバンドリングするか否かを返します。
     *
     * @return 有効・無効フラグ
     * @see EnableDOPacketHistoryBundling
     */
#endif
    static qBool IsEnableDOPacketHistoryBundling()
    {
        return s_bEnableDOPacketHistoryBundling;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送するために、パケットを保持する時間を返します。
     *
     * @return 保持時間[msec]
     * @see SetPacketHistoryBundlingHoldingTime
     */
#endif
    static TimeInterval GetPacketHistoryBundlingHoldingTime()
    {
        return s_packetHistoryBundlingHoldingTime;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送するために、保持する最大パケット数を返します。
     *
     * @return 最大パケット数
     * @see SetPacketHistoryBundlingMaxHoldingPackets
     */
#endif
    static size_t GetPacketHistoryBundlingMaxHoldingPackets()
    {
        return s_packetHistoryBundlingMaxHoldingPackets;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送する時に、同時にバンドルする最大パケット数を返します。
     *
     * @return 最大パケット数
     * @see SetPacketHistoryBundlingMaxRetransmitPackets
     */
#endif
    static size_t GetPacketHistoryBundlingMaxRetransmitPackets()
    {
        return s_packetHistoryBundlingMaxRetransmitPackets;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送する最大再送回数を返します
     *
     * @return 最大再送回数
     * @see SetPacketHistoryBundlingMaxRetransmitCounts
     */
#endif
    static size_t GetPacketHistoryBundlingMaxRetransmitCounts()
    {
        return s_packetHistoryBundlingMaxRetransmitCounts;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングされなかったパケットを強制再送するまでの保留時間
     *
     * @return 保留時間[msec]
     */
#endif
    static TimeInterval GetPacketHistoryBundlingForcedRetransmitTimeout()
    {
        return s_packetHistoryBundlingForcedRetransmitTimeout;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットバンドリングする時の、パケットの上限サイズを返します。
     *
     * @return  上限サイズ[byte]
     */
#endif
    static size_t GetPacketHistoryBundlingMaximumPacketBytes()
    {
        return s_packetHistoryBundlingMaximumPacketBytes;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   バンドリングされたパケット数の統計値を返します。(デバッグ用)
     *
     * enum PacketHistoryCountを要素に持つ配列ポインタを返します。
     *
     * @return パケット数統計値の配列
     */
#endif
    static const size_t* GetPacketHistoryBundlingPacketCounts()
    {
        return s_packetHistoryBundlingPacketCounts;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   バンドリングされたパケット数の統計値を返します。(デバッグ用)
     *
     * @param[in] phc PacketHistoryCount統計種別値を指定します。
     *
     * @return パケット数の統計値
     */
#endif
    static size_t GetPacketHistoryBundlingPacketCount(PacketHistoryCount phc)
    {
        return s_packetHistoryBundlingPacketCounts[phc];
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送する機能をON/OFFします
     *
     * 複製オブジェクトの新規パケット送信時に、送信先が同じ過去に送られたパケットを一つのパケットに含めて送信します。
     * Unreliableパケットの到達性向上につながります。最低1回は再送信されることを保証するため、バンドリングされる契機がなくても、
     * 1回は再送信します。Unreliableパケットに関しては、常にバンドリングされて、
     * Reliableパケットに関しては、EnableReliablePacketHistoryBundlingの指定に従います。
     * パケットのペイロードサイズが、1364バイト以上にはバンドリングされることはありません。
     *
     * パケット数がすくない場合には、パケットを送信する機会を増やすため、
     * nn::nex::Stream::GetSettings().GetBundling().Disable();
     * (バンドリングなし)と設定する必要があります。
     * パケット数が多い場合には、負荷を下げるため、
     * nn::nex::Stream::GetSettings().GetBundling().Enable(0);
     * (タイムアウトなしで、バッファに溜まっているパケットをバンドリングする)
     * と設定する必要があります。
     *
     * @param[in] b バンドリングするか設定します。デフォルトは、OFF(False)です。
     * @see IsEnablePacketHistoryBundling, EnableReliablePacketHistoryBundling
     */
#endif
    static void EnablePacketHistoryBundling(qBool b)
    {
        s_bEnablePacketHistoryBundling = b;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたReliableパケットをバンドリングして再送するか指定します。
     *
     * @param[in] b Reliableパケットをバンドリングするか設定します。デフォルトは、ON(True)です。
     * @see IsEnableReliablePacketHistoryBundling
     */
#endif
    static void EnableReliablePacketHistoryBundling(qBool b)
    {
        s_bEnableReliablePacketHistoryBundling = b;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたUnreliableパケットをバンドリングして再送するか指定します。
     *
     * @param[in] b Unreliableパケットをバンドリングするか設定します。デフォルトは、ON(True)です。
     * @see IsEnableUnreliablePacketHistoryBundling
     */
#endif
    static void EnableUnreliablePacketHistoryBundling(qBool b)
    {
        s_bEnableUnreliablePacketHistoryBundling = b;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られた DO のパケットをバンドリングして再送するか指定します。
     *
     * DO のパケットに対するバンドリングを無効にすると、 DirectStream のパケットのみが
     * バンドリングの対象になります。
     *
     * DO のデータセットの同期だけでなく、RMC も指定の対象となることに注意してください。
     *
     * @param[in] b DO のパケットをバンドリングするか設定します。デフォルトは、ON(True)です。
     * @see IsEnableDOPacketHistoryBundling
     */
#endif
    static void EnableDOPacketHistoryBundling(qBool b)
    {
        s_bEnableDOPacketHistoryBundling = b;
    }


#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送するためにパケットを保持する時間を設定します。
     *
     * @param[in] t 保持時間をmsec単位で設定します。
     * @see GetPacketHistoryBundlingHoldingTime, EnablePacketHistoryBundling
     */
#endif
    static void SetPacketHistoryBundlingHoldingTime(TimeInterval t)
    {
        s_packetHistoryBundlingHoldingTime = t;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送するために、保持する最大パケット数を設定します。
     *
     * @param[in] n 最大パケット数
     * @see GetPacketHistoryBundlingMaxHoldingPackets, EnablePacketHistoryBundling
     */
#endif
    static void SetPacketHistoryBundlingMaxHoldingPackets(size_t n)
    {
        s_packetHistoryBundlingMaxHoldingPackets = n;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送する時に、同時にバンドルする最大パケット数を設定します。
     *
     * @param[in] n 最大パケット数
     *
     * 設定する値は、1+ Stream::GetSettings().GetBundling().GetMaxUnreliableBundlingPackets()以上でないと、
     * 伝送効率が悪くなります。
     * 伝送スループットを調整する場合には、SetPacketHistoryBundlingMaxRetransmitCountsの方が調整しやすいです。
     *
     * @see GetPacketHistoryBundlingMaxRetransmitPackets, EnablePacketHistoryBundling, SetPacketHistoryBundlingMaxRetransmitCounts
     */
#endif
    static void SetPacketHistoryBundlingMaxRetransmitPackets(size_t n)
    {
        s_packetHistoryBundlingMaxRetransmitPackets = n;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングして再送する最大再送回数します。
     *
     * 強制再送も1回とカウントされます。
     *
     * @param[in] n 最大再送回数
     *
     * @see GetPacketHistoryBundlingMaxRetransmitCounts, EnablePacketHistoryBundling, SetPacketHistoryBundlingForcedRetransmitTimeout
     */
#endif
    static void SetPacketHistoryBundlingMaxRetransmitCounts(size_t n)
    {
        s_packetHistoryBundlingMaxRetransmitCounts = n;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングされなかったパケットを強制再送するまでの保留時間を設定します。
     *
     * 強制再送により最低1回送信されることが保証されます。GetPacketHistoryBundlingHoldingTimeより大きい値の場合は、強制再送されません。
     *
     * @param[in] timeout 保留時間[msec] インターネット通信の場合は、往復遅延時間が100msec～133msecに値が集中するので、133msecより大きい値を推奨します。
     *
     * @see GetPacketHistoryBundlingForcedRetransmitTimeout, EnablePacketHistoryBundling, GetPacketHistoryBundlingHoldingTime
     */
#endif
    static void SetPacketHistoryBundlingForcedRetransmitTimeout(TimeInterval timeout)
    {
        s_packetHistoryBundlingForcedRetransmitTimeout = timeout;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   過去に送られたパケットをバンドリングする時の、バンドリング上限サイズを設定します。デフォルトは1364です。
     *
     * @param[in] bytes 上限サイズ[byte]
     */
#endif
    static void SetPacketHistoryBundlingMaximumPacketBytes(size_t bytes)
    {
        s_packetHistoryBundlingMaximumPacketBytes = bytes;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   Unreliable通信において、リオーダリングもしくは重複発生パケットをドロップする機能をON/OFFします。
     *
     * パケットが届く順番がおかしくなってしまう、同じパケットが何度か届く問題を解消し、パケットの順番を保証します。
     * 届く順番がおかしいパケットに関しては、ロスとなり、順番入れ替えを行いません。
     *
     * EnablePacketHistoryBundling=trueの時には、このフラグにかかわらず、リオーダリングもしくは重複パケットはドロップされます
     *
     * @param[in] b ドロップするか設定します。デフォルトは、OFF(False)です。
     *
     * @see IsEnableDropDuplicateReorderingUnreliablePackets, EnablePacketHistoryBundling
     */
#endif
    static void EnableDropDuplicateReorderingUnreliablePackets(qBool b)
    {
        s_bEnableDropDuplicateReorderingUnreliablePackets = b;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   Unreliable通信において、リオーダリングもしくは重複発生パケットをドロップする機能のON/OFFを返します。
     *
     * パケットが届く順番がおかしくなってしまう、同じパケットが何度か届く問題を解消し、パケットの順番を保証します。
     * 届く順番がおかしいパケットに関しては、ロスとなり、順番入れ替えを行いません。
     *
     * EnablePacketHistoryBundling=trueの時には、このフラグにかかわらず、リオーダリングもしくは重複パケットはドロップされます
     *
     * @return ドロップするかフラグを返します。デフォルトは、OFF(False)です。
     *
     * @see EnableDropDuplicateReorderingUnreliablePackets, EnablePacketHistoryBundling
     */
#endif
    static qBool IsEnableDropDuplicateReorderingUnreliablePackets()
    {
        return s_bEnableDropDuplicateReorderingUnreliablePackets;
    }

    static void UpdatePacketHistoryBundlingPacketCounts(PacketHistoryCount phc)
    {
        s_packetHistoryBundlingPacketCounts[phc]++;
    }

#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief マスター経由通信履歴の更新間隔を設定します。
     *
     * @param[in] ui 更新間隔[msec]
     */
#endif
    static void SetRoutingHistoryUpdateSpan(qUnsignedInt32 ui)
    {
        s_uiRoutingHistoryUpdateSpan = ui;
    }
#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   マスター経由通信履歴の更新間隔を取得します。
     * @return マスター経由通信履歴の更新間隔が返ります。
     */
#endif
    static qUnsignedInt32 GetRoutingHistoryUpdateSpan()
    {
        return s_uiRoutingHistoryUpdateSpan;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief マスター経由通信履歴の生存時間を設定します。
     *
     * @ref StreamSettings::GetMaxSilenceTime() で取得した沈黙時間の2倍程度の値に設定してください。
     * マスター経由通信を持つステーション同士がここで設定した時間分通信しなかった場合、その通信は @ref Session::GetNumOfRoutingConnections() 及び @ref Station::GetNumOfRoutingConnections() でカウントされません。
     *
     * @param[in] ui 生存時間[msec]
     */
#endif
    static void SetRoutingHistoryTTL(qUnsignedInt32 ui)
    {
        s_uiRoutingHistoryTTL = ui;
    }
#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief   マスター経由通信履歴の生存時間を取得します。
     * @return マスター経由通信履歴の生存時間が返ります。
     */
#endif
    static qUnsignedInt32 GetRoutingHistoryTTL()
    {
        return s_uiRoutingHistoryTTL;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief マスター経由通信の数を制限します。
     *
     * マスター経由通信の数を制限します。 この制限値を超えるマスター経由の通信は、転送せずに破棄されます。
     * マスター経由通信の数を制限することで、マスター経由通信が一定数に収まるネットワークを構築することができます。
     * 0を設定すると、制限なしとなります。初期値は0、制限なしです。
     * 現在のマスター経由通信の数は、@ref Session::GetNumOfRoutingConnections で取得できます。
     *
     * @param[in] ui 制限数
     */
#endif
    static void SetRoutingLimit(qUnsignedInt32 ui)
    {
        s_uiRoutingLimit = ui;
    }
#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief マスター経由通信の制限数を取得します。
     * @return マスター経由通信の制限数が返ります。 0は制限無しを表します。
     */
#endif
    static qUnsignedInt32 GetRoutingLimit()
    {
        return s_uiRoutingLimit;
    }

    /*!
     * @internal
     *
     * 中継モードが有効化・無効化かを設定にします。 ConnectivityManager::StartNATSession()より前、
     * ConnectivityManager::StopNATSession()より後に設定してください。
     *
     * ここで有効に設定してもサーバー側で許可されていないと、中継モードが有効になりません。
     * デフォルトは、無効です。
     *
     * @param[in] flag 中継モードが有効か、無効かを設定します。
     *
     */
    static void SetEnabledRelay(qBool flag) { s_bEnableRelay = flag; }

    /*!
     * @internal
     *
     * 中継モードが有効化・無効化を取得します。
     * @return 中継モードが有効か、無効かを返します。
     */
    static qBool IsEnabledRelay() { return s_bEnableRelay; }


    /*!
     * @internal
     *
     * 中継用テーブルにエントリーが存在するかを返します。
     * @retval true 中継テーブルにエントリーが存在して、中継動作中です。
     * @retval false 中継テーブルにエントリーが存在しないか、中継モードが無効です。
     */
    static qBool ExistsRelayTable();
#endif

protected:
    static qUnsignedInt32 s_uiNATTraversalTimeout;
    static qUnsignedInt32 s_uiConnectEndPointTimeout;
    static qUnsignedInt32 s_uiDisconnectEndPointTimeout;
    static qUnsignedInt32 s_uiTerminateDOCoreTimeout;

    static qBool s_bEnableDropDuplicateReorderingUnreliablePackets;

    static qBool s_bEnablePacketHistoryBundling;
    static qBool s_bEnableReliablePacketHistoryBundling;
    static qBool s_bEnableUnreliablePacketHistoryBundling;
    static qBool s_bEnableDOPacketHistoryBundling;
    static TimeInterval s_packetHistoryBundlingHoldingTime;
    static size_t s_packetHistoryBundlingMaxHoldingPackets;
    static size_t s_packetHistoryBundlingMaxRetransmitPackets;
    static size_t s_packetHistoryBundlingMaxRetransmitCounts;
    static TimeInterval s_packetHistoryBundlingForcedRetransmitTimeout;
    static size_t s_packetHistoryBundlingMaximumPacketBytes;
    static size_t s_packetHistoryBundlingPacketCounts[MAX_PACKET_HISTORY_COUNT + 1];
    static qUnsignedInt32 s_uiTerminateImmediatelyTimeout;
    static qBool s_bEnableRelay;

public:
    // n2370: why public? this class is singleton.
    //group=Network Interface
    RootTransport();
    virtual ~RootTransport();
    //{secret}
    virtual qResult         Initialize()
    {
        return QRESULT_SUCCESS(Core, Success);
    }

    //{secret}
    qBool                   RegisterStream(Stream *pStream,
                                           Stream::ID idDesiredStream,
                                           Stream::ID *pAcquiredStreamID = NULL)
    {
        return m_oTransportStreamManager.RegisterStream(pStream,idDesiredStream,pAcquiredStreamID);
    }
    //{secret}
    qBool                   UnregisterStream(Stream::Type oType, Stream::ID idStream)
    {
        return m_oTransportStreamManager.UnregisterStream(oType,idStream);
    }
    //{secret}
    size_t                  GetNbStream()
    {
        return m_oTransportStreamManager.GetNbStream();
    }
    
    virtual qBool           Connect(const StationURL *pRemoteStationURL, TransportProtocol::Type transportProtocolType, qUnsignedInt16 uiOutgoingPort) = 0;
    //{secret}
    virtual qBool           OpenSocket(qUnsignedInt16 ui16Port,
                                        qUnsignedInt16 *pui16ListeningPort,
                                        qBool bPreferred,
                                        TransportProtocol::Type transportProtocolType,
                                        qBool usePacketBufferPool,
                                        qBool acceptIncomingConnections,
                                        qBool bReuse,
                                        qBool bForceUseExistenceSocket) = 0;

    virtual qBool           OpenUdpSocket(qUnsignedInt16 ui16Port,
                                           qUnsignedInt16 *pui16ListeningPort=NULL) = 0;

    virtual qBool           NotifyCreateEndPoint(TransportProtocol::Type transportProtocolType, qUnsignedInt16 localPort, const InetAddress& remoteAddr) = 0;
    virtual qBool           NotifyDeleteEndPoint(TransportProtocol::Type transportProtocolType, qUnsignedInt16 localPort, const InetAddress& remoteAddr) = 0;
    //{secret}
    virtual qBool           CloseSocket(TransportProtocol::Type transportProtocolType, qUnsignedInt16 ui16Port) = 0;
    virtual qBool           CloseUdpSocket(qUnsignedInt16 ui16Port) = 0;
    //{secret}
    virtual qBool           CloseSocket() = 0;
    //{secret}
    virtual size_t          GetNbListeningPorts() = 0;

    virtual TransportStatus GetTransportStatus(TransportProtocol::Type transportProtocolType, qUnsignedInt16 localPort) = 0;

    // {secret}
    void StopTransportThread();
    //{secret}
    virtual qBool           Send(TransportProtocol::Type transportProtocolType, qUnsignedInt16 ui16Port, Stream::Type oType,
                                 Stream::ID idSource, Stream::ID idDestination,
                                 PacketOut *pPacket, qBool bQueue = false) = 0;
    //{secret}
    virtual qBool           Receive(TransportProtocol::Type transportProtocolType, qUnsignedInt16 uiPort, Buffer *pBuffer, const InetAddress *pSourceAddress) = 0;

#ifdef NEX_ROUTING_SUPPORT
    //{secret}
    virtual Router         *GetRouter();
#endif

    virtual Relay          *GetRelay();

    //{secret}
    void                    SetMTU(qUnsignedInt32 uiMTU)
    {
        m_uiMTU = uiMTU;
    }
    //{secret}
    qUnsignedInt32          GetMTU()
    {
        return m_uiMTU;
    }

    //{secret}
    void                    SetPreferredPortNumber(qUnsignedInt16 uiPreferredPortNumber)
    {
        SYSTEMCHECK(GetNbListeningPorts()>0);
        m_uiPreferredPortNumber = uiPreferredPortNumber;
    }
    //{secret}
    // UDPで送信するときに優先的に送信元となるポート
    qUnsignedInt16          GetPreferredPortNumber()
    {
        return m_uiPreferredPortNumber;
    }

    //group=NAT Traversal

    /*
    $ifdef RENDEZVOUS$
    Description: Sets the network interface that $PRODUCTNAME$ will bind to.
    Parameters:
    pszInterfaceAddress: The network interface address to use. The default value is 0.0.0.0.
    Remarks: The network interface address to use. The default value is 0.0.0.0.
    Remarks: By default, $PRODUCTNAME$ will bind on a specific port on 0.0.0.0 and listen
    on that port on all configured addresses on all interfaces. That is, if a station has two
    configured network interfaces, $PRODUCTNAME$ will capture all packets addressed to its
    port on any of the two interfaces. However, if $PRODUCTNAME$ is bound to port 9103 (the
    default port) on the network interface specified here, it will only capture data sent
    to that interface/port and will discard all others. This method was created to facilitate
    testing. It must be called before the $PRODUCTNAME$ object is created.
    See Also: GetInterfaceAddress
    $else$
    {secret}
    $endif$
    */
    static void SetInterfaceAddress(const qChar* pszInterfaceAddress);

    /*
    $ifdef RENDEZVOUS$
    Description: Returns the network interface bound to $PRODUCTNAME$ by SetInterfaceAddress.
    Returns: The network interface used by $PRODUCTNAME$.
    Remarks: The network interface to bind to, set by SetInterfaceAddress.
    See Also: SetInterfaceAddress
    $else$
    {secret}
    $endif$
    */
    static const qChar* GetInterfaceAddress();

    //group=Network Interface

    /*
    Description: Sets the well-known port number used by $PRODUCTNAME$.
    Parameters:
    uiWellKnownPort: The well-known port number that will be used. The default is port 9103.
    Remarks: By default, $PRODUCTNAME$ will use port 9103 as the default well-known port.
    If the well-known port cannot be used, the system will open a dynamic port. The default well-known
    port can be changed via this method, which must be called before the $PRODUCTNAME$ object is created.
    In general you should not need to change the well-know port number, except where necessary
    in order to avoid a clash with another application that uses the same port number, such as if you wish
    to create more than one session on the same station.
    See Also: GetWellKnownPortNumber
    */

    /*!
     * @brief  既知ポート番号を設定します。
     *
     * デフォルトの既知ポート番号は9103ですが、この関数でポート番号を変更できます。
     * 既知ポートが利用できない場合、システムはdynamic port (49152 ～ 65535)をオープンします。
     * Net-Zのインスタンスを作成する前にこの関数を呼ぶことで、デフォルトの既知ポート番号を変更できます。
     * 通常、既知ポート番号を変更する必要はありませんが、別アプリケーションが同一のポート番号を
     * 使用しているとき（複数のセッションを同一のステーションで作成する場合など）、ポート番号の
     * 衝突を回避するためにこの関数を利用できます。
     * @see    GetWellKnownPortNumber
     * @param[in] uiWellKnownPort 設定するポート番号。デフォルト値は9103です。
     */
    static void             SetWellKnownPortNumber(qUnsignedInt16 uiWellKnownPort);

    /*
    Description: Returns the well-known port number used by $PRODUCTNAME$.
    Returns: The well-known port number used by $PRODUCTNAME$.
    Remarks: The well-known port number is set via SetWellKnownPortNumber.
    See Also: SetWellKnownPortNumber
    */

    /*!
     * @brief  既知ポート番号を取得します。
     *
     * 既知ポート番号は @ref SetWellKnownPortNumber で設定できます。
     * @see    SetWellKnownPortNumber
     * @return 使用されている既知ポート番号が返ります。
     */
    static qUnsignedInt16   GetWellKnownPortNumber();

    static qUnsignedInt16   GetNextPortNumber();

    // group=Emulation Devices

    //Description: Returns a pointer to an instance of the OutputEmulationDevice.
    //Returns: A pointer to an instance of the OutputEmulationDevice.
    //See Also: GetInputEmulationDevice, OutputEmulationDevice, InputEmulationDevice
    /*!
     * @brief 出力ネットワークエミュレータデバイスを取得します。[デバック用]
     *
     * @return 出力ネットワークエミュレータデバイスが返ります
     * @remark RootTransport::GetInstance()->GetOutputEmulationDevice()とすることで取得可能です。
     * @see GetInputEmulationDevice, OutputEmulationDevice, InputEmulationDevice
     */
    OutputEmulationDevice   *GetOutputEmulationDevice()
    {
        return &m_oOutputEmulationDevice;
    }

    //Description: Returns a pointer to an instance of the InputEmulationDevice.
    //Returns: A pointer to an instance of the InputEmulationDevice.
    //See Also: GetOutputEmulationDevice, OutputEmulationDevice, InputEmulationDevice
    /*!
     * @brief 入力ネットワークエミュレータデバイスを取得します。[デバック用]
     *
     * @return 入力ネットワークエミュレータデバイスが返ります
     * @remark RootTransport::GetInstance()->GetInputEmulationDevice()とすることで取得可能です。
     * @see GetOutputEmulationDevice, OutputEmulationDevice, InputEmulationDevice
     */
    InputEmulationDevice    *GetInputEmulationDevice()
    {
        return &m_oInputEmulationDevice;
    }

    //group=Transport Parameters

    /*
    Description: Returns a pointer to an instance of the transport performance counters.
    Returns: A pointer to an instance of TransportPerfCounters.
    Example:
            TransportPerfCounters *pCounters = RootTransport::GetInstance()->GetPerfCounters();
            // Return and print the BPS sent and received on a station
            qUnsignedInt32 uiBPSRecv = pCounters->GetValue(PERF_TRANSPORT_BPS_RECV, PERF_FREQUENCY_LAST);
            qUnsignedInt32 uiBPSSent = pCounters->GetValue(PERF_TRANSPORT_BPS_SENT, PERF_FREQUENCY_LAST);
            printf("BPS Sent: %d, Recv %d\n",uiBPSRecv,uiBPSSent);
    See Also: TransportPerfCounters,
    */
#ifdef NEX_PERFCOUNTER_ENABLED
    TransportPerfCounters   *GetPerfCounters()
    {
        return &m_oPerfCounters;
    }
#endif // NEX_PERFCOUNTER_ENABLED

    //{secret}
    void                    AttachVNATDevice(VirtualNATDevice *pDevice);
    //{secret}
    void                    DetachVNATDevice(VirtualNATDevice *pDevice);
    //{secret}
    VirtualNATDevice*       GetVNATDevice()
    {
        return m_pVNATDevice;
    }

#if NEX_ENABLE_VIRTUAL_FILTER_DEVICE
    VirtualFilterDevice*     GetVirtualFilterDevice()
    {
        return m_pFilterDevice;
    }
#endif

    //{secret}
    TransportStreamManager *GetTransportStreamManager()
    {
        return &m_oTransportStreamManager;
    }

    //group=Methods

    /*
    Description:  Returns a pointer to the RootTransport instance.

    */

    /*!
     * @brief RootTransportのインスタンスを取得します。
     * @return RootTransportのインスタンス
     */
    static RootTransport* GetInstance()
    {
        Network* pNetwork = Network::GetInstance();
        if (pNetwork == NULL)
        {
            //n1589
            QTRACE(D_TRACE_CONSTRUCT, NEX_T("RootTransport::GetInstance:pNetwork == NULL"));
            return NULL;
        }
        else
        {
            return pNetwork->GetRootTransport();
        }
    }

    //{secret}
    static void SetRecvBufferSize(qUnsignedInt32 uiSize)
    {
        s_uiRecvBufferSize = uiSize;
    }

    //{secret}
    static qUnsignedInt32 GetRecvBufferSize()
    {
        return s_uiRecvBufferSize;
    }

    //{secret}
    static void SetSendBufferSize(qUnsignedInt32 uiSize)
    {
        s_uiSendBufferSize = uiSize;
    }

    //{secret}
    static qUnsignedInt32 GetSendBufferSize()
    {
        return s_uiSendBufferSize;
    }

    //{secret}
    static qUnsignedInt8 GetDefaultPacketTTL()
    {
        return DEFAULT_PACKET_TTL;
    }

    //group=Transport Parameters

    /*
    Description: Returns the number of packets queued in the transport layer.
    Returns: The number of packets queued in the transport layer.
    Remarks: This method can be used with a selection of heuristics to adapt the game to the
    current network condition. For example, when the packet queue grows, a game could decide to decrease
    the quality of the game, or the number of updates per frame or simply stop the game.
    See Also: GetEstimatedPacketQueueMemoryUsage
    */
    virtual qUnsignedInt64 GetPacketQueueSize() const = 0;

    /*
    Description: Returns an estimation of the memory usage of the packets queued in the transport layer.
    Returns: An estimation of the memory usage of the packets queued in the transport layer.
    Remarks: This method is only an estimation of the memory usage. For a more accurate value, you can
    create your own memory allocation functions with counters to track the memory you allocate and deallocate
    and then register them with the MemoryManager to override the default Net-Z memory allocation functions.
    See Also: GetPacketQueueSize, MemoryManager
    */
    virtual qUnsignedInt64 GetEstimatedPacketQueueMemoryUsage() const = 0;

    //{secret}
    virtual qResult Send(TransportProtocol::Type transportProtocolType, StationURL *pStationURL, Buffer *pBuffer)
    {
        DUMMY_PARAM(transportProtocolType);
        DUMMY_PARAM(pStationURL);
        DUMMY_PARAM(pBuffer);
        return QRESULT_ERROR(Core, NotImplemented);
    }

    //{secret}
    static qUnsignedInt32 GetIOWaitTime()
    {
        return s_uiIOWaitTime;
    }
    //{secret}
    static void SetIOWaitTime(qUnsignedInt32 uiIOWaitTime)
    {
        s_uiIOWaitTime = uiIOWaitTime;
    }
    //{secret}
    static qUnsignedInt32 GetIdleSleepTime()
    {
        return s_uiIdleSleepTime;
    }
    //{secret}
    static void SetIdleSleepTime(qUnsignedInt32 uiIdleSleepTime)
    {
        s_uiIdleSleepTime = uiIdleSleepTime;
    }
    //{secret}
    qBool IsUsingUserThreadContext()
    {
        return m_bUseUserThreadContext;
    }
    //{secret}
    void UseUserThreadContext(qBool bUse)
    {
        m_bUseUserThreadContext = bUse;
    }

    //group=Network Interface

    //n1589:todo2:remove GetDefaultConnectionTimeout
    static qUnsignedInt32 GetDefaultConnectionTimeout()
    {
        return GetConnectEndPointTimeout();
    }

    /*
    Description: Sets the default connection timeout.
    Parameters:
    uiDefaultConnectionTimeout: The new default connection timeout in milliseconds. The default value is 30 000 ms (30 s).
    Remarks: The default connection timeout is the time that a Station will keep trying to connect to another station
    before giving up. This timeout is enforced any time a connection via a StationURL is attempted.
    */
    //n1589:todo2:remove SetDefaultConnectionTimeout
    static void SetDefaultConnectionTimeout(qUnsignedInt32 uiDefaultConnectionTimeout)
    {
        SetConnectEndPointTimeout(uiDefaultConnectionTimeout);
    }

    /*
    $ifdef RENDEZVOUS$
    {secret}
    $else$
    Description: Registers a PacketRecognizer to this RootTransport.
    Parameters:
    pPacketRecognizer: A pointer to the PacketRecognizer that will be registered to this RootTransport.
    Returns: true if the specified PacketRecognizer was successfully registered to this RootTransport,
    false if not or an error occurred.
    Remarks: For non-$PRODUCTNAME$ packets to be accepted by the system rather than automatically being dropped
    they must pass via a registered PacketRecognizer. Refer to the PacketRecognizer class for further details.
    See Also: UnregisterPacketRecognizer, PacketRecognizer
    $endif$
    */
    qBool RegisterPacketRecognizer(PacketRecognizer *pPacketRecognizer);

    /*
    $ifdef RENDEZVOUS$
    {secret}
    $else$
    Description: Unregisters a PacketRecognizer from this RootTransport.
    Parameters:
    pPacketRecognizer: A pointer to the PacketRecognizer that will be unregistered from this RootTransport.
    Returns: true if the specified PacketRecognizer was successfully unregistered from this RootTransport,
    false if not or an error occurred. This method will return false if the specified PacketRecognizer is
    not currently registered to this RootTransport.
    See Also: RegisterPacketRecognizer, PacketRecognizer
    $endif$
    */
    qBool UnregisterPacketRecognizer(PacketRecognizer *pPacketRecognizer);

    //{secret}
    qList<PacketRecognizer*> *GetPacketRecognizerList()
    {
        return &m_lstPacketRecognizers;
    }
    //{secret}
    qBool                     RejectPacket(Buffer *pBuffer, const InetAddress * const pInetAddress);

    //group=NAT Traversal

    /*!
     * @internal
     * @brief NAT種別判定サーバーのアドレスを設定します。
     * 別スレッドで呼び出すようになり問題が出なくなったため、ダミーかつassert落ちにしました。
     *
     * @see ConnectivityManager::StartNATSession()
     */
    qBool SetUpDefaultNATCheckServer() { SYSTEMCHECK(false); return true;}


    /*
    $ifdef RENDEZVOUS$
    Description: Sets the primary NATCheckServer address and ports.
    Parameters:
    pszNATCheckServer: The IP address of the primary NATCheckServer.
    ui16PrimaryPort: First port of the NATCheckServer to contact. The secondary NATCheckServer must use the same primary
    port.
    ui16SecondaryPort: Second port of the NATCheckServer to contact. The secondary NATCheckServer must use the same
    secondary port.
    Returns: True if the NATCheckServer address and ports have been set correctly; false otherwise.
    Remarks: Set the primary NATCheckServer address and ports before logging in to Rendez-Vous.
    NAT detection requires two NATCheck servers which are configured in parallel as they are both
    contacted during the detection of the station's NAT device Properties.
    See Also: SetSecondaryNATCheckServer
    $else$
    {secret}
    $endif$
    */
    qBool SetPrimaryNATCheckServer(const qChar *pszNATCheckServer,
                                   qUnsignedInt16 ui16PrimaryPort,
                                   qUnsignedInt16 ui16SecondaryPort);

    /*
    $ifdef RENDEZVOUS$
    Description: Sets the secondary NATCheckServer address and ports.
    Parameters:
    pszNATCheckServerAddress: The IP address of the secondary NATCheckServer.
    ui16PrimaryPort: First contact port for this secondary NATCheckServer. The primary NATCheckServer must use the same
    primary port.
    ui16SecondaryPort: Second contact port for this secondary NATCheckServer. The primary NATCheckServer must use the same
    secondary port.
    Returns: True if the NATCheckServer address and ports have been set correctly; false otherwise.
    Remarks: Set the secondary NATCheckServer address and ports before logging in to Rendez-Vous.
    NAT detection requires two NATCheck servers which are configured in parallel as they are both
    contacted during the detection of the station's NAT device Properties.
    See Also: SetPrimaryNATCheckServer
    $else$
    {secret}
    $endif$
    */
    qBool SetSecondaryNATCheckServer(const qChar *pszNATCheckServerAddress,
                                     qUnsignedInt16 ui16PrimaryPort,
                                     qUnsignedInt16 ui16SecondaryPort);



    //{secret}
    const qVector<StringStream *> &GetNATCheckServerAddresses() const;

    //{secret}
    qBool IsNATCheckServerAddressesSet() const
    {
        return m_isNATCheckServerAddressesSet;
    }

    qBool IsRelayServerAddressesSet() const
    {
        return m_isRelayServerAddressesSet;
    }

    //{secret}
    void SetNATProperties(const NATProperties &rNATProperties);

    /*
    $ifdef RENDEZVOUS$
    Description: Retrieves the NAT properties of the router between the station and the Internet.
    Returns: A NATProperties object containing the NAT properties (based on those outlined in RFC 4787) of the station's router.
    Remarks: To retrieve a valid reading of the station's NAT Properties, it is necessary to call this method
    after logging into Rendez-Vous. Otherwise, the contents of the NATProperties object will be invalid.
    See Also: NATProperties
    $else$
    {secret}
    $endif$
    */
    NATProperties GetNATProperties() const;

    //JobNameResolveから別スレッドで利用。内部では、qNewを発生させない、お互いを同時起動させないように実装
    qBool SetUpDefaultNATCheckServerInternal();
    qBool SetUpRelayServer();

    //{secret}
    void SetUPnPProperties(const UPnPProperties &rUPnPProperties);

    /*
    $ifdef RENDEZVOUS$
    Description: Retrieves the UPnP properties of the router between the station and the Internet.
    Returns: A UPnPProperties object.
    Remarks: To get a valid reading of the router's UPnP properties, it is necessary to call this method
    after logging into Rendez-Vous. Otherwise, the contents of the UPnPProperties object will be invalid.
    See Also: EnableUPnPSupport, UPnPProperties
    $else$
    {secret}
    $endif$
    */
    UPnPProperties GetUPnPProperties() const;

    //{secret}
    void EnableNATCheckSupport(qBool bEnableNATCheck);

    //{secret}
    qBool IsNATCheckEnabled() const;

    /*
    $ifdef RENDEZVOUS$
    Description: Enables or disables UPnP
    Remarks: UPnP allows an application behind a NAT router to configure the router
    to allow peers outside the private network to contact it.
    Note, enabling UPnP results in a longer login time.
    Parameters:
    bEnableUPnP: True to enable UPnP; false otherwise.
    See Also: GetUPnPProperties, UPnPProperties
    $else$
    {secret}
    $endif$
    */
    void EnableUPnPSupport(qBool bEnableUPnP);

    //{secret}
    qBool IsUPnPSupported() const;

    /*
    $ifdef RENDEZVOUS$
    Description: Enables or disables NAT-PMP.
    Remarks: The NAT Port Mapping Protocol allows an application behind a NAT router to
    configure the router to allow peers outside the private network to contact it.
    Note, enabling NAT-PMP results in a longer login time.
    See Also: IsNATPMPPortOpen
    $else$
    {secret}
    $endif$
    */
    void EnableNATPMPSupport(qBool bEnableNatPMP);

    //{secret}
    qBool IsNATPMPSupported() const;

    //{secret}
    void SetNATPMPPortOpen(qBool bFlag = true);

    /*
    $ifdef RENDEZVOUS$
    Description: Returns whether a port mapping has been added using NAT-PMP.
    Returns: True if a port has been opened in this way; false otherwise.
    See Also: EnableNATPMPSupport
    $else$
    {secret}
    $endif$
    */
    qBool IsNATPMPPortOpen() const;


    static void SetBlockedAddresses(const qList<InetAddress>& dropList);

    //n1589
#ifdef NEX_TRACE_ENABLED
    void Trace(qUnsignedInt64 uiTraceFlags);
#endif

    size_t GetVsocketCount()
    {
        return m_oTransportStreamManager.GetNBStreamByStreamType(Stream::Game);
    }
#ifdef NEX_CLIENT_PBPOOL
    virtual void ClearPacketInPacketBufferPool() {}
#endif // NEX_CLIENT_PBPOOL

    const InetAddress & GetRelayAddress() { return m_relayAddress;}
    void SetRelayNameAddress(String &str);

protected:
    mutable CriticalSection m_csPacketRecognizerList;
    mutable CriticalSection m_csNATDevice;

    qBool GetRelayServerAddress(const qChar *address);

    virtual void StopTransportThreadImpl() = 0;

    void SetIsNATCheckServerAddressesSet();

    qUnsignedInt32          m_uiMTU;
    qBool                   m_bUseUserThreadContext;

#ifdef NEX_PERFCOUNTER_ENABLED
    TransportPerfCounters       m_oPerfCounters;
#endif // NEX_PERFCOUNTER_ENABLED
    OutputEmulationDevice       m_oOutputEmulationDevice;
    InputEmulationDevice        m_oInputEmulationDevice;
    TransportStreamManager      m_oTransportStreamManager;
    VirtualNATDevice           *m_pVNATDevice;

    qUnsignedInt16              m_uiPreferredPortNumber;

    static qUnsignedInt32       s_uiMaxNbConnections;
    static qUnsignedInt32       s_uiIdleSleepTime;
    static qUnsignedInt32       s_uiIOWaitTime;
    static qUnsignedInt32       s_uiRecvBufferSize;
    static qUnsignedInt32       s_uiSendBufferSize;
    static qBool                s_bActiveFaultDetection;

    //2342  トランスポートバッファスレッドでnewしたバッファを捨てずに再利用するフラグ
    static qBool                s_bEnableTransportBufferThreadBufferPool;
    static size_t               s_TransportBufferPoolNum;
    static qUnsignedInt         s_TransportBufferThreadPriority;
    static qUnsignedInt         s_TransportBufferThreadSendPriority;
    static qInt32               s_TransportBufferThreadCoreNo;


    static qUnsignedInt         s_NameResolveThreadPriority;
    static qInt32               s_NameResolveThreadCoreNo;


#if NEX_ENABLE_HYBRID_DETECT_LIMITATION
    static qUnsignedInt32 s_uiRoutingHistoryUpdateSpan;
    static qUnsignedInt32 s_uiRoutingHistoryTTL;
    static qUnsignedInt32 s_uiRoutingLimit;
#endif

    qList<PacketRecognizer*>    m_lstPacketRecognizers;
    static qVector<qUnsignedInt32>  s_dropIPList;

    NATProperties m_oNATProperties;
    UPnPProperties m_oUPnPProperties;
    qBool m_bNATCheckSupport;
    qBool m_bUPnPsupport;
    qBool m_bNATPMPSupport;
    qBool m_bNATPMPPortOpen;
    volatile qBool m_isNATCheckServerAddressesSet;
    volatile qBool m_isRelayServerAddressesSet;

#if NEX_ENABLE_VIRTUAL_FILTER_DEVICE
    VirtualFilterDevice        *m_pFilterDevice;
#endif

    qVector< StringStream* > m_vecNATCheckAddresses;

    InetAddress m_relayAddress;
    String m_relayNameAddress;
};

END_NAMESPACE_NEX

