﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <OnlineCore/src/Transport/Interface/StationProbe.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief StationProbe のリストを内包するクラスです。

    VSocket::ProbeStations() で NAT トラバーサルを行う際に使用します。
    NAT トラバーサルの対象とする端末情報 (StationProbe) を保持します。
    StationProbe には NAT トラバーサルの結果となる詳細情報が含まれ、これは VSocket::ProbeStations() の完了後に
    StationProbeList::begin() などのイテレータを使用することにより参照します。
 */
class StationProbeList: public qList<StationProbe>
{
public:
    /*!
        @brief 各端末との間の PingTime (RTT) の統計情報です。

        VSocket::ProbeStations() を呼び出すと StationProbeList 内部で保持している本クラスの情報が更新されます。
     */
    class PingStats
    {
        friend class StationProbeList;
    public:
        /*!
            @brief コンストラクタです。
         */
        PingStats()
        {
            m_uiMin = Infinite;
            m_uiMax = Infinite;
            m_uiAverage = Infinite;
            m_uiMedian = Infinite;
        }

        /*!
            @brief 各端末との PingTime (RTT) のうち、最小のものを返します。

            デフォルトの最小の PingTime は Infinite です。
            Infinite が返された場合はいずれの端末とも疎通が取れていないことを示します。
            @return 各端末との PingTime のうち、最小のもの。単位はミリ秒です。
         */
        qUnsignedInt32 GetMin() const
        {
            return m_uiMin;
        }

        /*!
            @brief 各端末との PingTime (RTT) のうち、最大のものを返します。

            デフォルトの最大の PingTime は Infinite です。
            Infinite が返された場合は少なくとも一つの端末と疎通が取れていないことを示します。
            @return 各端末との PingTime のうち、最大のもの。単位はミリ秒です。
         */
        qUnsignedInt32 GetMax() const
        {
            return m_uiMax;
        }

        /*!
            @brief 各端末との PingTime (RTT) を平均したものを返します。

            デフォルトの平均 PingTime は Infinite です。
            Infinite が返された場合は少なくとも一つの端末と疎通が取れていないことを示します。
            @return 各端末との PingTime (RTT) を平均したもの。単位はミリ秒です。
         */
        qUnsignedInt32 GetAverage() const
        {
            return m_uiAverage;
        }

        /*!
            @brief 各端末との PingTime (RTT) の中央値を返します。

            デフォルトの PingTime の中央値は Infinite です。
            Infinite が返された場合は少なくとも半数の端末と疎通が取れていないことを示します。
            @return 各端末との PingTime (RTT) の中央値。単位はミリ秒です。
         */
        qUnsignedInt32 GetMedian() const
        {
            return m_uiMedian;
        }

    private:
        qUnsignedInt32 m_uiMin;
        qUnsignedInt32 m_uiMax;
        qUnsignedInt32 m_uiAverage;
        qUnsignedInt32 m_uiMedian;
    };

    /*!
        @brief コンストラクタです。空の StationProbeList を生成します。
     */
    StationProbeList();

    /*!
        @brief コンストラクタです。

        引数で与える StationURL のリストは、 MatchMakingClient::GetSessionURLs(),
        MatchMakingClient::GetParticipantsURLs(), MatchmakeExtensionClient::BrowseMatchmakeSession() などで、
        ゲームサーバーから取得したものである必要があります。
        これらの API で取得した StationURL は、通常一つの端末につき複数個存在します。
        指定した StationURL のリストは本コンストラクタ内部で自動的に端末ごとに分類され、StationProbe にまとめられます。
        生成したオブジェクトに対して StationProbeList::size() を呼び出すと端末の数と一致します。

        @param[in]    reflstStationURL 内部で保持する NAT トラバーサル対象の StationURL のリスト
     */
    StationProbeList(const qList<StationURL>& reflstStationURL);

    /*!
        @brief デストラクタです
     */
    virtual ~StationProbeList();

    /*!
        @brief StationURL を NAT トラバーサル対象として追加します

        同じ端末に対する StationProbe がすでに存在する場合は、その StationProbe に StationURL が追加されます。

        @param[in]  refStationURL  追加する URL。
        @return  StationProbe のポインタ
     */
    StationProbe* AddStationURL(const StationURL& refStationURL);


    //{secret}
    StationProbe *GetProbeByCID(ConnectionID idConnection);
    //{secret}
    void Update();

    /*!
        @brief 各端末との間の PingTime (RTT) の統計情報を取得します。

        VSocket::ProbeStations() の完了後に利用可能になります。
        個別の NAT トラバーサル結果を取得するには begin() などのイテレータを使用して StationProbe オブジェクトを参照してください。

        @return  各端末との間の PingTime (RTT) の統計情報。
     */
    PingStats       GetPingStats() const;

    /*!
        @brief 全ての端末との NAT トラバーサルが成功して疎通が取れたかどうかを取得します。

        VSocket::ProbeStations() の完了後に利用可能になります。
        個別の NAT トラバーサル結果を取得するには begin() などのイテレータを使用して StationProbe オブジェクトを参照してください。

        @return  全ての端末との NAT トラバーサルが成功して疎通が取れたかどうか。
     */
    qBool           EveryoneReached() const;

    /*!
        @brief いずれかの端末との NAT トラバーサルが失敗したかどうかを取得します。

        VSocket::ProbeStations() の完了後に利用可能になります。
        個別の NAT トラバーサル結果を取得するには begin() などのイテレータを使用して StationProbe オブジェクトを参照してください。

        @return  いずれかの端末との NAT トラバーサルが失敗したかどうか。
     */
    qBool           AnyoneFailed() const;

    /*!
        @brief      本オブジェクトの内容をログ出力します。
        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags);

    /*!
        @brief      内部で保持している NAT トラバーサルの結果をクリアします。

        内部で保持している StationProbe の PingTime や 接続情報、統計情報をクリアします。
        VSocket::ProbeStations() 開始時に自動的に呼び出されます。
    */
    void Reset();

};

END_NAMESPACE_NEX

