﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Transport/Interface/NATProperties.h>
#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>

#ifdef NON_SUPPORT_C_TARGET
#endif

#ifdef WII_TARGET
//  {secret}
typedef unsigned char DWC_AID;
// {secret}
static const DWC_AID aidINVALID = static_cast<DWC_AID>(-1);
#endif

//n1589
#ifdef NEX_CLIENT_OPTIMIZATION
//typedef qUnsignedInt16 NODE_ID_TYPE;
typedef unsigned short NODE_ID_TYPE;
static const NODE_ID_TYPE INVALID_NODE_ID = 0;
#endif

#define NEX_URL_TYPE_DELIMITER          NEX_T(":/")
#define NEX_URL_PARAM_DELIMITER         NEX_T(";")
#define NEX_URL_USER_PARAM_DELIMITER    NEX_T("#")
#define NEX_URL_TAG_ASSIGN              NEX_T("=")

BEGIN_NAMESPACE_NEX

class InetAddress;


/*
Description: Constructs a station URL.

Remarks: Use this class to construct a Rendez-Vous URL for any given Station. A station URL should contain
at least one URL and a port and, for NAT Traversal, the Rendez-Vous Connection ID.



$ifdef WII_DWC$
Although this class is part of the standard $PRODUCTNAME$ distribution, in this
distribution, several additional methods are included to enable
you to set the Wii AID, which uniquely identifies a specific Wii Console.
$endif$

*/

/*!
@brief    Station の URL を構築します。

このクラスは、特定のノードに対して URLを構築します。
StationURLには、最低1つのURLとポート、そして NAT トラバーサル用には ConnectionID が必要です。
*/
class StationURL: public RootObject
{

    //{secret}
    class Param;

    //{secret}
    friend class Param;

public:
    //group=Type Definitions

    // Description: Enumerates the various transport types that a StationURL may specify.
    // See Also: URLType
    /*!
    @brief     @ref StationURL の種類を列挙します。
    */
    enum URLType
    {
        // The transport type is unknown.
        unknown,  //!< 無効。
        // The transport type is PRUDP, Quazal's proprietary, reliable UDP transport protocol.
        prudp,    //!< PRUDP。信頼性のある UDP 通信プロトコルです。
        // The transport type is PRUDPS, Quazal's proprietary, reliable, secure UDP transport protocol
        // that is used by Rendez-Vous, Quazal's lobby solution.
        prudps,   //!< PRUDPS。信頼性のある セキュアな UDP 通信プロトコルです。ゲームサーバーとの通信に使用されます。
        // The transport type is UDP.  This represents a raw UDP protocol URL.  This can be used by developers
        // for feeding the game socket URL to Rendez-Vous.
        udp       //!< UDP。
    };

    // Description: Enumerates the various Flags that may be set for a StationURL.
    // See Also: Flags
    /*!
    @brief     @ref StationURL に設定可能な各種Flagsを列挙します。

    @see     Flags
    */
    enum Flags
    {
        // The Station that this StationURL pertains to is behind a Network Address Translator (NAT).
        BehindNAT           = (1 << 0),   //!< StationURLに対応するステーションは、NATデバイスの配下にある
        // The StationURL refers to a public IP address and port number.
        Public              = (1 << 1),   //!< StationURL は、パブリックなIPアドレスおよびポート番号を参照している
        DetectedByNatCheck  = (1 << 2),   //!< StationURL は、NAT チェックサーバーで検出されたグローバルアドレスを参照している。必ず同時に Public フラグも立つ。
        DetectedByNgs       = (1 << 3)    //!< StationURL は、NGS で検出されたグローバルアドレスを参照している。必ず同時に Public フラグも立つ。
    };

    enum PlatformType
    {
        PLATFORM_TYPE_UNKNOWN = 0,
        PLATFORM_TYPE_CTR = 1,
        PLATFORM_TYPE_WUP = 2,
        PLATFORM_TYPE_NX = 3,

        PLATFORM_TYPE_WINDOWS = 254,
        PLATFORM_TYPE_LINUX = 255 //Max Number
    };

public:

    //Group=Methods

    /*
    Parameters:
    strURL: The StationURL as a string.
    szURL: The StationURL as a qChar. This value must be null-terminated.
    Remarks: A StationURL that uses the prudp URLType is expressed in the form:
    "prudp:/address=xx;port=xxxx", where the address may be specified in either text or numeric
    form, /e.g./ "www.quazal.com" or "192.168.10.1". For example:

            StationURL oStationURL(NEX_T("prudp:/address=www.quazal.com;port=9103"));
            StationURL oStationURL(NEX_T("prudp:/address=192.168.10.1;port=9103"));

    On the other hand a StationURL that uses the prudps URLType is expressed in the form:
    "prudps:/PID=xx". For example:

            StationURL oStationURL(NEX_T("prudps:/PID=1005"));

    */

    /*!
     @brief    コンストラクタです
     */
    StationURL();
    /*! @internal
     @brief    コンストラクタです

     @param[in]    strURL    URL の文字列表現
     */
    StationURL(const String &strURL);
    /*! @internal
     @brief    コンストラクタです

     @param[in]    szURL    URL の文字列表現
     */
    StationURL(const qChar* szURL);
    /*!
     @brief    コピーコンストラクタです

     @param[in]    oStationURL    コピー元の StationURL
     */
    StationURL(const StationURL &oStationURL);
    /*!
     @brief    デストラクタです
     */
    virtual ~StationURL();

    /*
    Description: Sets the URLType that this StationURL will use.
    Parameters:
    eType: The URLType that this StationURL will use.
    See Also: GetURLType, URLType
    */

    /*!
    @brief    この StationURL が使用する URLType を設定します。

    @param[in]    eType    この StationURL が使用する URLType 。
    */
    void SetURLType(const URLType eType);

    /*
    Description: Returns the transport type specified by this StationURL.
    Returns: The URLType specified by this StationURL.
    See Also: SetURLType, URLType
    */

    /*!
    @brief    この StationURL が使用する URLType を取得します。

    @return    この StationURL が使用する URLType 。
    */
    URLType GetURLType() const;

    /*
    Description: Sets the address that this StationURL will use.
    Parameters:
    strAddress: The address that this StationURL will use, which may be specified in either text
    or numeric form, /e.g./ "www.quazal.com" or "192.168.10.1".
    Remarks: If you do not specify the address and port number of the StationURL in the constructor
    you can set it afterwards as follows:

                {
                    $PRODUCTNAME$ *g_p$PRODUCTNAME$ = qNew $PRODUCTNAME$;
                    StationURL urlSession;
                    urlSession.SetAddress(NEX_T("192.168.10.1"));
                    // Or you could use an address in text form
                    // StationURL::SetAddress(NEX_T("www.quazal.com"));
                    urlSession.SetPortNumber(9103);
                }

    Note: Addresses are only valid if the URLType is PRUDP.

    See Also: GetAddress, URLType
    */

    /*!
    @brief    この @ref StationURL  が使用するアドレスを設定します。

    アドレスとのポート番号をコンストラクタで指定しない場合は、本関数を使用して後で設定することができます。

    @param[in]    strAddress    この @ref StationURL が使用するアドレス。これはテキストまたは数字形式で指定できます（例"www.quazal.com" または "192.168.10.1"）。
    */
    qBool SetAddress(const qChar *strAddress);
    //{secret}
    qBool SetAddress(const String &strAddress)
    {
        return SetAddress(strAddress.CStr());
    }

    /*
    Description: Returns the address specified by this StationURL.
    Returns: The address specified by this StationURL.

    Note: Addresses are only valid if the URLType is PRUDP.

    $ifdef NON_SUPPORT_C$
    Note: (NON_SUPPORT_C Specific) If you are using $PRODUCTNAME$ for NON_SUPPORT_C with the NPTransportAdapter,
     do not use this method. Use SetNpId instead.
    $endif$

    See Also: SetAddress
    */

    /*!
     @brief    Station のアドレスを取得します

     @return Station のアドレス
     */
    String GetAddress() const;

    /*
    Description: Sets the port number that this StationURL will use.
    Parameters:
    ui16Port: The port number that this StationURL will use.
    Remarks: Refer to SetAddress for an example of this method in use.

    $ifdef NON_SUPPORT_C$
    Note: (NON_SUPPORT_C Specific) If you are using $PRODUCTNAME$ for the NON_SUPPORT_C with the NPTransportAdapter,
     do not use this method. Use SetNpIdGetConnectionID instead.
    $endif$

    Note: Port numbers are only valid if the URLType is PRUDP.
    See Also: GetPortNumber, URLType
    */

    /*!
    @brief    ポート番号をセットします。

    @param[in]    ui16Port     StationURL によって指定されるポート番号。
    */
    void SetPortNumber(const qUnsignedInt16 ui16Port);

    /*
    Description: Returns the port number specified by this StationURL.
    Returns: The port number specified by this StationURL.

    Note: Port numbers are only valid if the URLType is PRUDP.
    See Also: SetPortNumber
    */

    /*!
    @brief    この StationURL によって指定されたポート番号を返します。

    @return    この StationURL によって指定されたポート番号。

    @note    ポート番号は、 @ref URLType がPRUDPの場合のみ、有効です。
    */
    qUnsignedInt16 GetPortNumber() const;

    // {secret}
    void SetStreamID(const Stream::ID idStream);
    // {secret}
    Stream::ID GetStreamID() const;

    // {secret}
    void SetStreamType(const Stream::Type oStreamType);
    // {secret}
    Stream::Type GetStreamType() const;

#ifdef NON_SUPPORT_C_TARGET
#error "Not supported platform."
#endif

#ifdef WII_TARGET

    /*

    {group: Wii-DWC Specific Methods}

    $ifdef WII_DWC$

    Description: Sets this StationURL to the Wii ID (AID) which identifies a specific Wii console
    to the Nintendo matchmaking server.
    Parameters:
    ucAID: The AID that this StationURL will use.
    Remarks: In the $PRODUCTNAME$ distribution only, this method
    enables you to set the StationURL to the Nintendo matchmaking ID (AID) which
    identifies a specific Wii console.

    See Also: URLType
    $else$
    {secret}
    $endif$
    */
    void SetAID(DWC_AID ucAID);

    /*
    {group: Wii-DWC Specific Methods}

    $ifdef WII_DWC$
    Description: Retrieves the Wii ID (AID) contained in the StationURL.
    Parameters:
    ucAID: The AID that this StationURL uses.
    Remarks: In the $PRODUCTNAME$ distribution only, this method
    enables you to get the Nintendo matchmaking information (AID) contained
    in the StationURL.

    See Also: URLType
    $else$
    {secret}
    $endif$
    */
    DWC_AID GetAID() const;
#endif

#ifdef NEX_CLIENT_OPTIMIZATION
    void SetNodeId(qUnsignedInt16 nodeId);
    qUnsignedInt16 GetNodeId() const;
#endif

    /*
    {group: Rendez-Vous Specific Methods}

    Description: Sets the PrincipalID that this StationURL will use.
    Parameters:
    pid: The PrincipalID that this StationURL will use.
    Remarks: This method is only valid when $PRODUCTNAME$ is used in conjunction with
    Rendez-Vous, Quazal's lobby service.

    Note: PrincipalIDs are only valid if the URLType is PRUDPS.
    See Also: GetPrincipalID, URLType
    */

    /*!
    @brief    この StationURL によって指定されるPrincipalID を設定します。

    @param[in]    pid    この StationURL によって指定されるPrincipalID 。
    */
    void SetPrincipalID(const PrincipalID pid);

    /*
    {group: Rendez-Vous Specific Methods}

    Description: Returns the PrincipalID specified by this StationURL.
    Returns: The PrincipalID specified by this StationURL.
    Remarks: This method is only valid when Net-Z is used in conjunction with
    Rendez-Vous, Quazal's lobby service.

    Note: PrincipalIDs are only valid if the URLType is PRUDPS.
    See Also: SetPrincipalID
    */

    /*!
    @brief    この StationURL によって指定されたPrincipalID を取得します。

    この StationURL が MatchMakingClient::GetSessionURLs などによってゲームサーバーから取得したものでない場合は、
    PrincipalID がセットされていないことがあります。

    @return    この StationURL によって指定されたPrincipalID 。

    @note    PrincipalIDは、 @ref URLType がPRUDPSの場合のみ、有効になります。
    */
    PrincipalID GetPrincipalID() const;

    /*
    {group: Rendez-Vous Specific Methods}

    Description: Sets the ConnectionID that this StationURL will use.
    Parameters:
    cid: The ConnectionID that this StationURL will use.
    Remarks: This method is only valid when $PRODUCTNAME$ is used in conjunction with
    Rendez-Vous, Quazal's lobby service.

    Note: ConnectionIDs are only valid if the URLType is PRUDPS.
    See Also: GetConnectionID, URLType
    */
    void SetConnectionID(const ConnectionID cid);

    /*
    {group: Rendez-Vous Specific Methods}

    Description: Returns the ConnectionID specified by this StationURL.
    Returns: The ConnectionID specified by this StationURL.
    Remarks: This method is only valid when $PRODUCTNAME$ is used in conjunction with
    Rendez-Vous, Quazal's lobby service.

    Note: ConnectionIDs are only valid if the URLType is PRUDPS.
    See Also: SetConnectionID
    */
    ConnectionID GetConnectionID() const;

    /*
    {group: Rendez-Vous Specific Methods}

    Description:  Appends the RVConnectionID to this StationURL.

    Remarks: This must be the unique ID assigned to each client
    when it connects to a Rendez-Vous lobby. The MatchMakingClient
    automatically adds this value to Station URLS.
    */

    /*
    @brief    ゲームサーバーに対する ConnectionID をセットします。

    Station ごとにユニークなゲームサーバー用の ConnectionID をセットします。
    ゲームサーバーに接続したときにユニークな ID が割り当てられます。

    @param[in] cid   Station の ConnectionID
    */
    void SetRVConnectionID(const ConnectionID cid);

    /*
    {group: Rendez-Vous Specific Methods}

    Description:  Gets the unique RVConnectionID of this StationURL.

    Remarks: This is the unique ID assigned to each client when it connects to a Rendez-Vous
    lobby. The MatchMakingClient automatically adds this value to Station URLS.
    */
    /*
    @brief    ゲームサーバーに対する ConnectionID を取得します。

    @return    Station の ConnectionID
    */
    ConnectionID GetRVConnectionID() const;

    // {secret}
    void SetProbeRequestID(const qUnsignedInt32 prid);
    // {secret}
    qUnsignedInt32 GetProbeRequestID() const;
    // {secret}
    void SetFastProbeResponse(const qBool bFastProbeResponse);
    // {secret}
    qBool GetFastProbeResponse() const;

    // {secret}
    void SetNATMapping(NATProperties::MappingProperties eMappingType);
    // {secret}
    NATProperties::MappingProperties GetNATMapping() const;

    // {secret}
    void SetNATFiltering(NATProperties::FilteringProperties eFilteringType);
    // {secret}
    NATProperties::FilteringProperties GetNATFiltering() const;

    // {secret}
    void SetProbeRequestInitiation(qBool bProbeRequestInitiation);
    // {secret}
    qBool GetProbeRequestInitiation() const;

    // {secret}
    void SetUPnPSupport(qBool bSupportUPnP);
    // {secret}
    qBool GetUPnPSupport() const;

    // {secret}
    void SetNatPMPSupport(qBool bSupportNatPMP);
    // {secret}
    qBool GetNatPMPSupport() const;

    // {secret}
    void SetType(const qUnsignedInt32 uiFlags);
    // {secret}
    qUnsignedInt32 GetType() const;

    // {secret}
    void SetRelayServerAddress(const String& relayServerAddress);
    // {secret}
    String GetRelayServerAddress() const;

    // {secret}
    void SetOptionalUri(const String& uri);
    // {secret}
    String GetOptionalUri() const;

    // {secret}
    void SetRelayServerPort(qUnsignedInt16 relayServerPort);
    // {secret}
    qUnsignedInt16 GetRelayServerPort() const;

    void SetRelayAddress(const String& relayServerAddress);
    // {secret}
    String GetRelayAddress() const;

    // {secret}
    void SetRelayPort(qUnsignedInt16 relayServerPort);
    // {secret}
    qUnsignedInt16 GetRelayPort() const;

    // {secret}
    void SetUseRelayServer(qBool bUseRelayServer);
    // {secret}
    qBool GetUseRelayServer() const;

    void SetNatTraversalRequesterPrivateAddress(const String& natTraversalRequesterPrivateAddress);
    String GetNatTraversalRequesterPrivateAddress() const;

    void SetNatTraversalRequesterPrivatePort(qUnsignedInt16 natTraversalRequesterPrivatePort);
    qUnsignedInt16 GetNatTraversalRequesterPrivatePort() const;

    /*
     * プラットフォーム識別子を返します
     */
    qUnsignedInt32 GetPlatformType() const;

    void SetPlatformType();



    /*
    Description: Returns whether or not the Station that this StationURL pertains to is behind a NAT.
    Returns: true if the Station that this StationURL pertains to is behind a Network Address Translator (NAT),
    false if not or an error occurred.
    See Also: Flags
    */

    /*!
    @brief    この @ref StationURL が関連する Station が NAT デバイスの配下にあるものであるかを返します。

    @retval    true     StationURL が関連する Station が NAT デバイスの配下にある
    @retval    false    NAT デバイスの配下にない、またはエラーが発生

    @see     Flags
    */
    qBool IsBehindNAT() const
    {
        return (GetType() & BehindNAT)==BehindNAT;
    }

    // ループバックアドレスかどうかを返す
    qBool IsLoopbackAddress() const;

    /*
    Description: Returns whether or not this StationURL refers to a public IP address and port number.
    Returns: true if this StationURL refers to a public IP address and port number, false if not or an
    error occurred.
    See Also: Flags
    */

    /*!
    @brief    この StationURL がパブリックのIPアドレスおよびポート番号を参照しているかどうかを返します。

    @retval    true    この @ref StationURL がパブリックのIPアドレスおよびポート番号を参照している
    @retval    false    上記以外、またはエラーが発生した場合は偽。
    */
    qBool IsPublic() const
    {
        return (GetType() & Public) == Public;
    }

    /*
    @brief    この StationURL が NAT チェックサーバーで検出されたグローバルアドレスを参照しているかどうかを返します。
              ConnectivityManager::StartNATSession() が成功後にこの URL がローカル URL に登録されます。

    @retval    true    この @ref StationURL が NAT チェックサーバーで検出されたグローバルアドレスを参照している
    @retval    false    上記以外、またはエラーが発生した場合は偽。
    */
    qBool IsDetectedByNatCheck() const
    {
        return (GetType() & DetectedByNatCheck) == DetectedByNatCheck;
    }

    /*
    @brief    この StationURL が NGS で検出されたグローバルアドレスを参照しているかどうかを返します。
              ConnectivityManager::StartNATSession() が成功後にこの URL がローカル URL に登録されます。

    @retval    true    この @ref StationURL が NGS で検出されたグローバルアドレスを参照している
    @retval    false    上記以外、またはエラーが発生した場合は偽。
    */
    qBool IsDetectedByNgs() const
    {
        return (GetType() & DetectedByNgs) == DetectedByNgs;
    }

    //{secret}
    qBool SetInetAddress(const InetAddress *pInetAddress);
    //{secret}
    const InetAddress &GetInetAddress() const;

    /*!
     @refmanoverload{name1}

     @brief    代入演算子です。
     */
    StationURL &operator=(const StationURL &oStationURL);
    /*!
     @refmanoverload{name2}

     @brief    代入演算子です。
     */
    StationURL &operator=(const String &strURL);
    /*!
     @refmanoverload{name3}

     @brief    代入演算子です。
     */
    StationURL &operator=(const qChar* szURL);
    /*!
     @brief    比較演算子です。
     */
    qBool operator==(const StationURL &oStationURL) const;
    /*!
     @brief    比較演算子です。
     */
    qBool operator!=(const StationURL &oStationURL) const;

    /*
    Description: Returns the StationURL as a string.
    Returns: The StationURL as a string.
    */

    /*!
    @brief     StationURL を文字列で返します。

    @return    StationURL の文字列表現。
    */
    String GetStringRepresentation() const
    {
        return GetURL();
    }

#ifdef FOR_DOC_PURPOSE
    /*
    Description: Sets user-defined attributes in the StationURL.
    Parameters:
    strKey: The Key variable, as a String, of the attribute to set.
    strValue: The Value variable, as a String, of the attribute to set.
    Remarks: An attribute is a user-defined Key-Value pair of String variables. Attributes
    are used to add additional information to the StationURL. Key and Value strings
    cannot contain the following characters since they are reserved for delimiters:
    '=',':','/','#',';'.

    See Also: Get, Remove
    */

    /*!
    @brief        StationURL にユーザー定義属性を設定します。

    @return       設定できたら真、そうでない場合は偽。

    @param[in]    strKey       設定する値のキー 。
    @param[out]   strValue     設定する値。

    @see     Get, Remove
    */
    qBool Set(const String &strKey, const String &strValue);
#else
    // {secret}
    qBool Set(const String &strKey, const String &strValue, qBool bUserParam = false);
#endif

#ifdef FOR_DOC_PURPOSE
    /*
    Description: Returns the value associated with the specified key.
    Parameters:
    strKey: The Key, as a String, of the value to get.
    pstrValue: A pointer to store the Value for the given Key.
    Returns: true if the given attribute exists, false otherwise.
    See Also: Set, Remove
    */

    /*!
    @brief    指定されたキーに関連付けられている値を返します。

    @return    特定の属性が存在する場合は真、そうでない場合は偽。

    @param[in]    strKey    取得する値のキー 。
    @param[out]   pstrValue    指定のキーの値を保存するポインタ。

    @see     Set, Remove
    */
    qBool Get(const String &strKey, String *pstrValue) const;
#else
    // {secret}
    qBool Get(const String &strKey, String *pstrValue, qBool bUserParam = false) const;
#endif

#ifdef FOR_DOC_PURPOSE
    /*
    Description: Removes user-defined attributes from the StationURL.
    Parameters:
    strKey: The Key, as a String, of the attribute to remove.
    Returns: true if the given attribute was removed, false is the attribute was not found.
    szKey: The Key variable, as a string, of the attribute to remove.
    See Also: Set, GetAttribute, FindAttribute, FindFirstAttribute, FindNextAttribute
    */

    /*!
    @brief        StationURL からユーザー定義属性を削除します。

    @return       削除できたら真、そうでない場合は偽。

    @param[in]    strKey       削除する値のキー 。

    @see     Get, Set
    */
    qBool Remove(const String &strKey);
#else
    // {secret}
    qBool Remove(const String &strKey, qBool bUserParam = false);
#endif

    // Description: Traces the content of this StationURL object.
    // Remarks: This method traces the URL of this StationURL.
    /*!
    @brief    この StationURL の内容をトレースします。

    @param[in]    uiTraceFlags    TRACE フラグ
    */
    void Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS) const;

    /*
    Description: Returns the StationURL as a const C-style version of the string.
    Returns: The StationURL as a const C-style version of the string.
    */

    /*!
    @brief    Station の URL を文字列を返します。

    @return    Station の URL
    */
    const qChar *CStr() const
    {
        return GetURL();
    }

    //{secret}
    qResult  SetParamValue(const qChar* szKey, const qByte *pBuffer,
                           const size_t uiBufferSize);
    //{secret}
    qResult  SetParamValue(const qChar* szKey, const qUnsignedInt64 uiValue);

    //{secret}
    qUnsignedInt64 GetParamValue(const qChar* szKey, qUnsignedInt64 uiNotFoundValue=0) const;

    //{secret}
    qBool  GetParamValue(const qChar* szKey, qByte *pBuffer, const size_t uiBufferSize) const;

    //{secret}
    qResult GetURLType(qChar *szURLType, size_t uiSize) const;

    //{secret}
    size_t GetNbParam(qBool bUserParam = false);

    qBool IsMatchingExceptPort(const StationURL& urlStation) const;

    // {secret}
    qBool IsMatching(const StationURL& urlStation) const;

    // {secret}
    qBool IsAddressMatching(const StationURL &rurlStation) const;

    //{secret}
    qBool  IsValid() const;

    //{secret}
    const qChar *GetURL() const;
    //{secret}
    void SetURL(const qChar* szURL);

    void SetTransportProtocolType(TransportProtocol::Type tpt);
    TransportProtocol::Type GetTransportProtocolType() const;

#ifdef NEX_TRACE_ENABLED
    static void TraceList(qUnsignedInt64 uiTraceFlags, const qList<StationURL>& oURLList, const qChar* szContext=NULL);
#endif

private:

    typedef qMap<String,String> URLStrMap;
    typedef qMap<String,qUnsignedInt64> URLIntMap;

    static qResult CopyInSuppliedBuffer(qChar *szDest, size_t uiSize, const qChar *szSource,
                                        size_t uiNeededSize);

    void Format();

    // Parsing functions
    void Parse();
    URLType ParseURLType() const;
    qBool ParseParams(qChar *ptr, qBool bUserParam );
    qBool ParseParam(qChar *ptr, qBool bUserParam );

    void Copy(const StationURL &urlStation);
    void ValidateData() const;
    void ValidateStr() const;

    // Internal version of Get functions that do not copy strings
    void GetIterator( const String &strKey, URLStrMap::iterator &it )
    {
        ValidateData();
        it = m_StrMap.find(strKey);
    }

private:
    // Special cases
    InetAddress *m_pAddr;
    URLType     m_urlType;

    // Parsed data
    URLIntMap  m_IntMap;
    URLStrMap  m_StrMap;
    URLStrMap  m_UserMap;

    qChar   *m_szURL;

    // State flags
    qBool m_bDataValid;
    qBool m_bStrValid;
    qBool m_bURLValid;
};


// String streaming.
// {secret}
extern StringStream& operator<<(StringStream& oStream, const StationURL& strValue);

END_NAMESPACE_NEX

