﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Stack/Core/TransportProtocol.h>
#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/StreamSettings.h>
#include <OnlineCore/src/Core/PseudoGlobalVariable.h>

BEGIN_NAMESPACE_NEX
class Packet;

//group=Transport Protocol

/*
    Description: This class provides access to the settings associated with the specified transport stream Type.
    See Also: StreamSettings
*/

/*!
    @brief ストリームのクラスです。

    このクラスでは、複製オブジェクトのストリームの設定への参照を提供します。

    @see StreamSettings
 */
class Stream: public RootObject
{
public:
    /*
    n1589:memo
    DO
    RVAuthentication
    RVSecure
    SandBoxMgmt
    NAT
    SessionDiscovery
    NATEcho
    Routing
    Game
    */

#ifndef FOR_DOC_PURPOSE
    /*!
     * @internal
     @brief    ストリームの種別を表す列挙型です。アプリケーションからはDOのみを使用可能です。
     */
    typedef enum
    {
        //   Description: do:/address=;vport=0;
        DO                  = 1,  //!<  複製オブジェクト。

        //   Description: auth:/;vport=0;
        RVAuthentication    = 2,  //!<

        //   Description: rv:/PID=x;CID=y;
        OldRVSecure         = 3,  //!<

        //   Description: rv:/PID=x;CID=y;
        SandBoxMgmt         = 4,  //!<

        //   Description: NAT:/
        NAT                 = 5,  //!<

        //   Description: LSD:/
        SessionDiscovery    = 6,  //!<

        //   Description: NATEcho:/
        NATEcho             = 7,  //!<

        //   Description: Routing:/
        Routing             = 8,  //!<

        //   Description: Game:/
        Game                = 9,  //!<

        //   Description: rv:/PID=x;CID=y; security update
        RVSecure            = 10, //!<

        Relay               = 11,
        
        //   Description: LSD:/
        LastStreamType      = 11 //!<   enum Typeの末尾を表現するシンボル。
    } Type; // Description: The $PRODUCTNAME$ transport streams.
#endif

    // {secret}
    typedef qByte   ID;

    static const ID MAX_STREAM_PRUDP_NB=16;
#ifdef NEX_PRUDP_LITE_SUPPORT
    static const ID MAX_STREAM_PRUDPLite_NB=32; // 最大 255まで
    static const ID MAX_STREAM_ENTRY=MAX_STREAM_PRUDPLite_NB;
#else
    static const ID MAX_STREAM_ENTRY=16;
#endif

public:
    /*
    Remarks: This object should never be directly instantiated. Instead, it is created
    by the system when $FACADENAME$ is created.
    */
    Stream(Type oStreamType);

    /*
    Remarks: This object should never be directly destroyed. It is deleted by
    $FACADENAME$.
    */
    virtual ~Stream();

    // {secret}
    virtual qBool ReceiveIncomingPacket(qUnsignedInt16 uiPort, ID idStream, Packet *pPacket) = 0;
    // {secret}
    // currentTimeは現在時刻。渡せれば渡した方が良い。NULLの場合、必要なら内部でGetTime()が行われる。
    virtual void DoWork(const Time* currentTime) = 0;

    virtual qBool IsDuplicateReorderingPacket(Packet * /*pPacket*/)
    {
        return false;
    }

    // {secret}
    Type GetType() const
    {
        return m_oStreamType;
    }

    // {secret}
    const qChar* GetTypeName() const
    {
        return GetTypeName(GetType());
    }

    // {secret}
    static const qChar* GetTypeName(Type strType);

    // {secret}
    StreamSettings& GetThisSettings()
    {
        SYSTEMCHECK(m_oStreamType <= LastStreamType);
        return s_oStreamSettings[m_oStreamType].GetValue();
    }

    static TransportProtocol::Type StreamTypeToTransportProtocolType(Type streamType);
    static void SetRvTransportProtocolType(TransportProtocol::Type protocolType)
    {
        s_rvTranportProtocolType = protocolType;
    }
    static TransportProtocol::Type GetRvTransportProtocolType()
    {
        return s_rvTranportProtocolType;
    }

#ifdef FOR_DOC_PURPOSE
    /* Description: Returns a data object encapsulating the transport settings associated with the specified stream Type.
        Returns: A data object of type StreamSettings.
        See Also: StreamSettings
    */

    /*!
    @brief 複製オブジェクトのストリーム設定を取得します。

    複製オブジェクトのトランスポート設定オブジェクトを得ます。

    @return 複製オブジェクトのStreamSettingsオブジェクトへの参照。
    */
    static StreamSettings& GetSettings();
#else
    static StreamSettings& GetSettings(Type oStreamType = Stream::DO)
    {
        SYSTEMCHECK(oStreamType <= LastStreamType);
        return s_oStreamSettings[oStreamType].GetValue();
    }

#endif
    
    static void ResetSettings();

    static const TransportProtocol::Type DEFAULT_RV_TRANSPORT_PROTOCOL_TYPE;
private:
    Type m_oStreamType;
    static PseudoGlobalVariable<StreamSettings> s_oStreamSettings[MAX_STREAM_ENTRY];
    static TransportProtocol::Type s_rvTranportProtocolType;
};

END_NAMESPACE_NEX

