﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <cstddef>
#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Transport/Interface/StreamBundling.h>
#include <OnlineCore/src/Core/LiteBuffer.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Core/Key.h>
#include <OnlineCore/src/Core/Checksum/Interface/SignatureBytes.h>


BEGIN_NAMESPACE_NEX

class EncryptionAlgorithm;
class KeyedChecksumAlgorithm;
class Key;


//group=Transport Protocol

/*
    Description: Encapsulates the transport settings for a Stream::Type.
    Remarks: Instantiated in a call to Stream::GetSettings which returns a reference
    to an object of this class for a given transport stream Type. You can then use this object
    to access the settings for the stream.
    See Also: Stream
*/

/*!
    @brief ストリームのトランスポート設定をカプセル化したクラスです。

    @ref Stream::GetSettings() メソッド呼び出しによって、そのストリームに関連づけられた
    本クラスのオブジェクトへの参照が得られます。このオブジェクトを利用して、
    ストリーム設定にアクセスすることができます。

    @see Stream
 */
class StreamSettings: public RootObject
{
public:
#ifdef FOR_DOC_PURPOSE
    static const std::size_t MAX_ENCRYPT_KEY_LEN = 32; //!<  暗号キーの最大サイズです。
    static const std::size_t MIN_ENCRYPT_KEY_LEN = 16;//!<  暗号キーの最小サイズです。
#else
    enum
    {
        MAX_ENCRYPT_KEY_LEN = 32,
        MIN_ENCRYPT_KEY_LEN = 16
    };
#endif


    typedef LiteBuffer<MAX_ENCRYPT_KEY_LEN> EncryptKey;
    // {secret}
    StreamSettings();
    // {secret}
    virtual ~StreamSettings();
    /*
    Description: Sets the PRUDP window size. The default is 8.

    Parameters:
    uiWindowSize: The allowable number of outstanding unacknowledged packets.
    The default is 8.

    Remarks: Sets the number of packets that can be sent before waiting
    for acknowledgement of receipt from the receiving station.
    With the reliable update policy, only a limited number of
    packets are sent before waiting for acknowledgement of receipt.
    Use this method to override the default number of allowed outstanding
    unacknowledged packets. This method should be called before any
    $PRODUCTNAME$ objects are instantiated.

    See Also: GetWindowSize
    */
    qResult SetWindowSize(size_t uiWindowSize);

    /*
    Description: Returns the PRUDP window size, the default is 8.

    Remarks: The PRUDP window size is is the number of packets that can be sent
    before waiting for acknowledgement of receipt from the receiving station.
    When sending reliable packets, only a limited number are sent before waiting
    for acknowledgement of receipt.
    Use this method to get the default number of allowed outstanding
    unacknowledged packets. This is set with SetWindowSize and the default
    is 8.

    See Also: SetWindowSize
    */
    size_t GetWindowSize() const;

    /*
    Summary: Returns the current setting for the maximum retransmission.

    Remarks: Returns the value for the maximum number times that a
    single packet will be retransmitted to another station before it is
    assumed that the station is no longer connected.

    See also: SetMaxRetransmission
    */

    /*!
    @brief 現時点でのパケット再送回数の最大値を取得します。

    ピアステーションがもはや接続されていないと見なされるまでに、
    パケットの再送を最大で何度繰り返すかを取得します。

    @see SetMaxRetransmission()

    @return 再送回数の最大値。
     */
    size_t GetMaxRetransmission() const;

    /*
    Summary: Sets the value for the maximum retransmission.

    Parameters:
        uiMaxRetransmission: Desired value for Max Retransmission.

    Remarks: Sets the maximum number of times that a single packet will be
    retransmitted to another station before it is assumed that the station is no
    longer connected. Setting MaxSilenceTime and MaxRetrasmission to high values increases the
    tolerance to flaky network conditions. Setting them to low values will
    speed up fault detection.

    Default MaxRetransmission is 20.

    See also: GetMaxRetransmission, SetMaxSilenceTime
    */

    /*!
     @brief    再送回数の最大値を設定します。

     ステーションに向けて送られるパケットが再送される回数の最大値を設定します。
     再送回数が最大値を超えると、もはやステーションは接続されていないと見なされます。
     沈黙時間の最大値と、再送回数の最大値に大きな値を設定すれば、
     品質の良くないネットワーク環境での耐性を高めることができます。
     逆に小さな値を設定すれば、障害を検出するまで時間を短縮できます。

     @param[in] ui16MaxRetransmission 再送回数の最大値。

     @see      GetMaxRetransmission(), SetMaxSilenceTime()

     @return   常に QSUCCESS(Core, Success) が返ります
     */
    qResult SetMaxRetransmission(size_t ui16MaxRetransmission);

    /*
    Summary: Returns the value of the Keep Alive Timeout in milliseconds.

    See also: SetKeepAliveTimeout
    */

    /*!
    @brief  キープアライブタイムアウト値を取得します。

    @see    SetKeepAliveTimeout()

    デフォルト値は、1000ミリ秒です。

    @return キープアライブタイムアウト値（単位はミリ秒）
     */
    qUnsignedInt32 GetKeepAliveTimeout() const;

    /*
    Summary: Sets the time for the Keep Alive Timeout.

    Parameters:
    uiKeepAliveTimeout: Timeout in milliseconds

    Remarks: Keep Alive is used to send a packet to all other peer stations
    to let them know that your station is still operating. This is part of
    the fault detection mechanism. This method should be called before any
    $PRODUCTNAME$ objects are instantiated. Default is 1000ms for Stream::DO and
    10000ms for Stream::RendezVous

    See also: GetKeepAliveTimeout, SetMaxSilenceTime
    */

    /*!
    @brief キープアライブタイムアウト値を設定します。

    キープアライブは、自分以外のステーションにパケットを送り、自分の
    ステーションがまだ機能している（生存している）ことを知らせるために用いられます。
    これは障害検出メカニズムの一部です。　キープアライブタイムアウト値の間、パケットの送信していないステーションに
    キープアライブパケットを送ります。
    一方、  GetMaxSilenceTime()間連続して一切のパケット受信がない場合に、そのステーションが接続されていないと判断されます。
    このメソッドは、@ref NetZ または @ref NgsFacade のオブジェクトをインスタンス化する前に呼び出して下さい。

    デフォルト値は、1000ミリ秒です。

    @param[in] uiKeepAliveTimeout キープアライブタイムアウト値（単位はミリ秒）

    @see GetKeepAliveTimeout(), SetMaxSilenceTime(), GetMaxSilenceTime()

    @return 常に QSUCCESS(Core, Success) が返ります。
     */
    qResult SetKeepAliveTimeout(qUnsignedInt32 uiKeepAliveTimeout);

    /*! @internal
     *
     * @brief キープアライブパケットの送信抑制機能が有効となっているかを取得します。
     *
     * @retval true キープアライブパケットの送信抑制機能が有効です。(デフォルト値)
     * @retval false キープアライブパケットの送信抑制機能が無効です。
     *
     * 送信抑制機能が有効な場合、
     * キープアライブ以外のパケットによる送受信をステーションの生存確認に利用します。
     * キープアライブ以外のパケットによる送受信がキープアライブタイムアウトの時間まで無い場合のみ、
     * キープアライブパケットによるステーションの生存確認が行われるようになります。
     * デフォルト値はtrueです。
     *
     * @see SetIsKeepAliveReduced()
     */
    qBool IsKeepAliveReduced()
    {
        return m_bKeepAliveReduced;
    }
    /*! @internal
     *
     * @brief キープアライブパケットの送信抑制機能の有効・無効を設定します。
     *
     * @param flag true 有効 false 無効
     *
     * 送信抑制機能が有効な場合、
     * キープアライブ以外のパケットによる送受信をステーションの生存確認に利用します。
     * キープアライブ以外のパケットによる送受信がキープアライブタイムアウトの時間まで無い場合のみ、
     * キープアライブパケットによるステーションの生存確認が行われるようになります。
     * デフォルト値はtrueです。
     *
     * @see IsKeepAliveReduced()
     */
    void  SetIsKeepAliveReduced(qBool flag)
    {
        m_bKeepAliveReduced = flag;
    }

    /*
    Summary: Returns the value for the maximum silence time in milliseconds.

    See also: SetMaxSilenceTime, MAXSILENCETIME_DEFAULT
    */

    /*!
     @brief    沈黙時間の最大値を取得します（単位はミリ秒）。

     @see      SetMaxSilenceTime()

     デフォルト値は、10000ミリ秒です。

     @return   沈黙時間の最大値（単位はミリ秒）
     */
    qUnsignedInt32 GetMaxSilenceTime() const;

    /*
    Summary: Sets the value for the maximum silence time.

    Parameters:
    uiMaxSilenceTime: Maximum silence time in milliseconds.

    Remarks: If no packet from a peer station is received within the maximum
    silence time, then it is assumed that the station is no longer connected.
    Setting MaxSilenceTime and MaxRetrasmission to high values increases the
    tolerance to flaky network conditions. Setting them to low values will
    speed up fault detection.

    This method should be called before any $PRODUCTNAME$ objects are instantiated.
    Default is 5000ms for Stream::DO and 60000ms for Stream::RendezVous

    See also: GetMaxSilenceTime, SetMaxRetransmission
    */

    /*!
     @brief    沈黙時間の最大値を設定します。

     あるステーションから到着するはずのパケットが、最大沈黙時間を経過しても
     届かなかった場合、そのステーションはもはや接続されていないと見なします。
     正常状態であれば、最後にパケットを送信してから GetKeepAliveTimeout()間経つとキープアライブパケットを送られます。

     沈黙時間の最大値と、再送回数の最大値に大きな値を設定すれば、
     品質の良くないネットワーク環境での耐性を高めることができます。
     逆に小さな値を設定すれば、障害を検出するまで時間を短縮できます。

     デフォルト値は、10000ミリ秒です。

     このメソッドは、@ref NetZ または @ref NgsFacade のオブジェクトをインスタンス化する前に呼び出して下さい。

     @param[in] uiMaxSilenceTime 沈黙時間の最大値。単位はミリ秒です。

     @see      GetMaxSilenceTime() , SetMaxRetransmission() , GetKeepAliveTimeout()

     @return   常に QSUCCESS(Core, Success) が返ります
     */
    qResult SetMaxSilenceTime(qUnsignedInt32 uiMaxSilenceTime);

    // {secret}
    qBool IsBroadcastingEverywhere();
    // {secret}
    qResult BroadcastEverywhere(qBool bEverywhere=true);
    // {secret}
    qResult SetRetransmitTimeoutMultiplier(qReal rRetransmitTimeoutMultiplier);
    // {secret}
    qReal GetRetransmitTimeoutMultiplier()const;
    // {secret}
    qResult SetExtraRetransmitTimeoutMultiplier(qReal rExtraRetransmitTimeoutMultiplier);
    // {secret}
    qReal GetExtraRetransmitTimeoutMultiplier()const;
    // {secret}
    qResult SetExtraRetransmitTimeoutTrigger(qUnsignedInt uiExtraRetransmitTimeoutTrigger);
    // {secret}
    qUnsignedInt GetExtraRetransmitTimeoutTrigger()const;
    // {secret}
    qResult SetInitialRTT(qUnsignedInt uiInitialRTT);
    // {secret}
    qUnsignedInt GetInitialRTT()const;
    // {secret}
    qResult SetRTTRetransmit(size_t uiRTTRetransmit);
    // {secret}
    qUnsignedInt GetSynInitialRTT()const;
    // {secret}
    qResult SetSynInitialRTT(qUnsignedInt uiSynInitialRTT);
    // {secret}
    size_t GetRTTRetransmit()const;
    // {secret}
    void SetKey(const String& strKey);

    // {secret}
    void SetKey(qUnsignedInt32 uiKey);

    // {secret}
    qByte GetChecksumOffset() const
    {
        return m_byChecksumOffset;
    }
    qUnsignedInt32 GetChecksumOffset32() const
    {
        return m_uiChecksumOffset;
    }
#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @refmanoverload{name1}
     *
     * @brief 暗号化キーを設定します。
     *
     * 暗号化を利用する際には、
     * NetZで Session::JoinSession() もしくは、 Session::CreateSession() する前に、
     * Stream::GetSettings().SetIsEncryptionRequired(true);
     * として、 Stream::GetSettings().SetEncryptionKey() で鍵を設定します。
     * また、解除する場合には、 NetZ::Terminate() 処理完了後にしてください。
     * 正しい区間以外での操作は、アサートエラーが発生するか、設定に失敗します。
     *
     *
     * @param ptr 暗号化キーの先頭ポインタ
     * @param ptrLen 暗号化キーのサイズ
     *
     * キーは、@ref MIN_ENCRYPT_KEY_LEN バイトから @ref MAX_ENCRYPT_KEY_LEN バイトまで設定可能です。
     *
     * @see SetIsEncryptionRequired(), MAX_ENCRYPT_KEY_LEN, MIN_ENCRYPT_KEY_LEN
     * @see GetEncryptionKeyPaylod(), GetEncryptionKeyLength()
     */
#endif
    void SetEncryptionKey(const void *ptr, size_t ptrLen);

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @refmanoverload{name2}
     *
     * @brief 暗号化キーを設定します。
     *
     *
     * 暗号化を利用する際には、
     * NetZで Session::JoinSession() もしくは、 Session::CreateSession() する前に、
     * Stream::GetSettings().SetIsEncryptionRequired(true);
     * として、 Stream::GetSettings().SetEncryptionKey() で鍵を設定します。
     * また、解除する場合には、 NetZ::Terminate() 処理完了後にしてください。
     * 正しい区間以外での操作は、アサートエラーが発生するか、設定に失敗します。
     *
     *
     * @param key 暗号化キー
     *
     * キーは、@ref MIN_ENCRYPT_KEY_LEN バイトから @ref MAX_ENCRYPT_KEY_LEN バイトまで設定可能です。
     *
     *
     * @see SetIsEncryptionRequired(), MAX_ENCRYPT_KEY_LEN, MIN_ENCRYPT_KEY_LEN
     * @see GetEncryptionKeyPaylod(), GetEncryptionKeyLength()
     */
#endif
    void SetEncryptionKey(const qVector<qByte>& key);

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief 暗号化キーを取得します。
     *
     * @return 暗号化キーの先頭ポインタ。設定されていない場合は、NULLが返ります。
     *
     * @see SetEncryptionKey(), GetEncryptionKeyLength()
     */
#endif
    const void* GetEncryptionKeyPaylod();
    
#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief 暗号化キーのサイズを取得します。
     *
     * @return 暗号化キーのサイズ。設定されていない場合は、0が返ります。
     *
     * @see SetEncryptionKey(), GetEncryptionKeyPaylod()
     */
#endif
    size_t GetEncryptionKeyLength()
    {
        return m_encryptKey.GetContentSize();
    }

    const EncryptKey& GetEncryptionKey()
    {
        return m_encryptKey;
    }


    void SetEncryptionKey(const Key& key);

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     *
     * @brief 暗号化が有効か取得します
     *
     * @retval true 暗号化が有効です。
     * @retval false 暗号化が無効です。(デフォルト値)
     *
     * @see SetIsEncryptionRequired()
     */
#endif
    qBool IsEncryptionRequired()
    {
        return m_IsEncryptionRequired;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     *
     * @brief 暗号化が有効か無効か設定します。
     *
     * @param[in] flag true 有効 false 無効
     *
     * 暗号化を利用する際には、
     * NetZで Session::JoinSession() もしくは、 Session::CreateSession() する前に、
     * Stream::GetSettings().SetIsEncryptionRequired(true);
     * として、 Stream::GetSettings().SetEncryptionKey() で鍵を設定します。
     * また、解除する場合には、 NetZ::Terminate() 処理完了後にしてください。
     * 正しい区間以外での操作は、アサートエラーが発生するか、設定に失敗します。
     *
     * 鍵を設定せずに動作をさせることはできません。
     *
     * @see IsEncryptionRequired(), SetEncryptionKey()
     */
#endif
    void SetIsEncryptionRequired(qBool flag);

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief ユーザー用のリライアブルの最大サブストリーム数を取得します。
     *
     * システムとの共用が一本あり、それとは別にユーザーデータ用のリライアブルが何本あるかを取得します。
     *
     * @return 最大サブストリーム数
     *
     * @see SetMaxUserReliableSubStreams()
     */
#endif
    qByte GetMaxUserReliableSubStreams()
    {
        return m_byMaxUserReliableSubStreams;
    }

#ifdef NEX_NETZ_FOR_DOC_PURPOSE
    /*!
     * @brief ユーザー用のリライアブルの最大サブストリーム数を設定します。
     *
     * システムとの共用が一本あり、それとは別にユーザーデータ用のリライアブルを何本用意するか設定します。
     * 接続段階でネゴシエーションが行われて、小さい方の値に合わせられます。
     * デフォルトは0です。 255まで設定できますが、メモリ消費量が増えるので、そのあたりを考慮して設定してください。
     *
     * @param[in] subStreamID 最大サブストリーム数
     *
     * @see GetMaxUserReliableSubStreams(), SubStreamIDDefine
     */
#endif
    void SetMaxUserReliableSubStreams(qByte subStreamID)
    {
        m_byMaxUserReliableSubStreams = subStreamID;
    }

    /*
    $ifdef NETZ$
    Description: Returns the $PRODUCTNAME$ Session encryption algorithm.
    Returns: A pointer to the encryption algorithm selected via SetEncryptionAlgorithm
    or NULL if no encryption algorithm is currently loaded.
    Remarks: By default, no encryption algorithm is loaded. Use SetEncryptionAlgorithm to
    change the algorithm used.
    See Also: SetEncryptionAlgorithm
    $else$
    {secret}
    $endif$
    */
    EncryptionAlgorithm *GetEncryptionAlgorithm()
    {
        return m_pEncryptionAlgorithm;
    }

    /*
    $ifdef NETZ$
    Description: Selects the $PRODUCTNAME$ Session encryption algorithm.
    Parameters:
    pEncryptionAlgorithm: A pointer to the encryption algorithm that will be loaded. By default
    no encryption algorithm is loaded.
    Returns: true if the specified algorithm was sucessfully loaded, false if not or an error occurred.
    Remarks: The encryption algorithm used for a given Session must be set when the
    session is either created or joined and cannot be changed during the session. If you wish to disable
    encryption, pass NULL as the variable for this method. Note that as $PRODUCTNAME$ does not delete a
    given algorithm when it is either deselected or replaced, you are responsible for deleting the algorithm
    once it is no longer needed.

    For example, assuming you have an EnableEncryption flag in your create and your join methods, you could
    set the encryption algorithm as follows:

                m_ppEncryption[i] = NULL;
                if(bEnableEncryption){
                    m_ppEncryption[i] = qNew RC4Encryption();
                    Stream::GetSettings(oStreamType).SetEncryptionAlgorithm(m_ppEncryption);
                }

    See Also: GetEncryptionAlgorithm, EncryptionAlgorithm, Stream
    $else$
    {secret}
    $endif$
    */
    void SetEncryptionAlgorithm(EncryptionAlgorithm *pEncryptionAlgorithm)
    {
        m_pEncryptionAlgorithm = pEncryptionAlgorithm;
    }


    void SetMaxRetransmitTimeout(qUnsignedInt time)
    {
        m_uiMaxRetransmitTimeout = time;
    }
    qUnsignedInt GetMaxRetransmitTimeout() const
    {
        return m_uiMaxRetransmitTimeout;
    }

    //n2690
    KeyedChecksumAlgorithm* GetChecksumAlgorithm() const
    {
        return m_pChecksumAlgorithm;
    }

    void SetChecksumAlgorithm(KeyedChecksumAlgorithm* pAlgorithm)
    {
        m_pChecksumAlgorithm = pAlgorithm;
    }


    // {secret}
    qBool FaultDetectionIsActive();
    // {secret}
    void SetActiveFaultDetection(qBool bActiveFaultDetection);
    // {secret}
    void EnableTrace(qBool bEnable = true)
    {
        m_bTraceEnable = bEnable;
    }
    // {secret}
    qBool GetTraceEnable() const
    {
        return m_bTraceEnable;
    }

    /*!
     * @brief ストリームごとのパケットバンドリング設定を取得します。
     *
     * @return バンドリング設定
     */
    StreamBundling &GetBundling()
    {
        return m_oStreamBundling;
    }

    // n2370
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    void SetMaxWindowMapSize( qUnsignedInt32 size )
    {
        m_uiMaxWindowMapSize = size;
    }
    size_t GetMaxWindowMapSize() const
    {
        return m_uiMaxWindowMapSize;
    }
#endif

#ifdef NEX_TRACE_ENABLED
    void Trace(qUnsignedInt64 uiTraceFlags);
#endif

    // プロトコルとしてサポートしている StreamID 数
    qByte GetMaxStreamNumber() const;

    void SetStreamType(qUnsignedInt32 type)
    {
        m_streamType = type;
    }

    void SetCanChangeSettings(qBool flag)
    {
        m_bCanChangeSettings = flag;
    }

    enum FastSorryMode
    {
        FASTSORRY_DISABLED = 0,
        FASTSORRY_SERVER = 1,
        FASTSORRY_CLIENT = 2
    };

    FastSorryMode GetFastSorryMode() const
    {
        return m_fastSorryMode;
    }

    void SetFastSorryMode(FastSorryMode fastSorryMode)
    {
        m_fastSorryMode = fastSorryMode;
    }

    TransportProtocol::Type GetTransportProtocolType() const
    {
        return m_transportProtocolType;
    }

    void SetTransportProtocolType(TransportProtocol::Type transportProtocolType)
    {
        m_transportProtocolType = transportProtocolType;
    }

    const SignatureBytes& GetAccesssKeyHash() const { return m_accesssKeyHash; }

private:
    SignatureBytes          m_accesssKeyHash;
    qUnsignedInt32          m_streamType;
    qByte                   m_byChecksumOffset;
    qUnsignedInt32          m_uiChecksumOffset;
    EncryptionAlgorithm     *m_pEncryptionAlgorithm;
    StreamBundling          m_oStreamBundling;
    size_t                  m_uiMaxRetransmission;
    qUnsignedInt32          m_uiKeepAliveTimeout;
    qBool                   m_bKeepAliveReduced;
    qUnsignedInt32          m_uiMaxSilenceTime;
    size_t                  m_uiWindowSize;

    // n2370
#if NEX_ENABLE_RELIABLE_MAP_SIZE_LIMITATION
    size_t                  m_uiMaxWindowMapSize;
#endif

    qReal                   m_rRetransmitTimeoutMultiplier;
    qReal                   m_rExtraRetransmitTimeoutMultiplier;
    qUnsignedInt            m_uiExtraRetransmitTimeoutTrigger;
    qUnsignedInt            m_uiInitialRTT;
    qUnsignedInt            m_uiSynInitialRTT;
    size_t                  m_uiRTTRetransmit;
    qUnsignedInt            m_uiMaxRetransmitTimeout;
    qBool                   m_bBroadcastEverywhere;
    qBool                   m_bActiveFaultDetection;
    qBool                   m_bTraceEnable;
    qBool                   m_bCanChangeSettings;
    qByte                   m_byMaxUserReliableSubStreams;

    FastSorryMode           m_fastSorryMode;

    //n2690
    KeyedChecksumAlgorithm  *m_pChecksumAlgorithm;
    qBool m_IsEncryptionRequired;

    EncryptKey m_encryptKey;
    TransportProtocol::Type m_transportProtocolType;

    qBool IgnoreChangeSettings();
};

END_NAMESPACE_NEX

