﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/TransportDecl.h>
#include <OnlineCore/src/Platform/Core/ForcedCriticalSection.h>
#include <OnlineCore/src/Platform/Core/Condition.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Core/InstantiationContext.h>
#include <OnlineCore/src/Transport/Interface/TransportBufferThreadInterface.h>

BEGIN_NAMESPACE_NEX

template<class TargetClass, class ArgumentType> class ObjectThread;
class QueuingSocketTransportBuffer;

/*!
 * @brief Socketの送受信を行うクラスです。
 *
 * 送信と受信をそれぞれ1スレッドずつ用いて行います。
 * SocketTranportクラスから使用されます。
 *
 */
class TransportBufferMultiThread: public TransportBufferThreadInterface
{
public:
    /*!
     * @brief コンストラクタです。
     */
    TransportBufferMultiThread();

    /*!
     * @brief デストラクタです。
     */
    virtual ~TransportBufferMultiThread();

    /*!
     * @brief 指定されたポートに対応するソケットをmapに保存します。
     *
     * @param[in] port ポート番号
     * @param[in] pSocket ソケットのポインタ
     */
    virtual qBool InsertSocketPort(qUnsignedInt16 port, QueuingSocketTransportBuffer* pSocket);

    /*!
     * @brief 指定されたプロトコルとポートに対応するソケットをmapから削除します。
     *
     * @param[in] transportProtocolType プロトコル
     * @param[in] port ポート番号
     */
    virtual void EraseSocketPort(TransportProtocol::Type transportProtocolType, qUnsignedInt16 port);

    Condition* GetSendCond()
    {
        return &m_sendCond;
    }

private:
    /**
     * @brief 内部スレッドを開始します。
     */
    virtual void StartSendingThread();

    /*!
     * @brief 内部スレッドを停止させます。
     */
    virtual void StopSendingThread();

    /*!
     * @brief 送信のループです。
     */
    void TransportSendLoop(int);

    // トランスポートプロトコルごとにソケット・ポートを管理するクラス
    class TransportProtocolContext: public RootObject
    {
    public:
        TransportProtocolContext()
            : m_portAndSocketPairs()
            , m_pollInfos()
            , m_csObjectLock(LockMasks::Ignored)
            , m_csSocketLock(LockMasks::Ignored)
            , m_isPollInfoChanged(0)
            , m_isThreadStopping(0)
            , m_icContext(PseudoSingleton::GetCurrentContext())
            , m_pReceivingThread(NULL)
            , m_transportProtocolType(TransportProtocol::TYPE_UNKNOWN)
        {
            m_portAndSocketPairs.reserve(MAX_NUM_SOCKETS);
        }

        ~TransportProtocolContext()
        {
            // 全ての管理しているソケットを削除する。
            while (!m_portAndSocketPairs.empty())
            {
                Remove(m_portAndSocketPairs.begin()->first);
            }
        }

        // 自身のオブジェクトが受信を担当しているプロトコル
        void SetTransportProtocolType(TransportProtocol::Type transportProtocolType)
        {
            m_transportProtocolType = transportProtocolType;
        }

        void Send();
        qBool Receive();
        qBool Add(qUnsignedInt16 port, QueuingSocketTransportBuffer* pSocket);
        void TransportRecvLoop(int);
        void Remove(qUnsignedInt16 port);
        void StartReceivingThread();
        void StopReceivingThread();

    private:
        typedef std::pair<qUnsignedInt16, QueuingSocketTransportBuffer*> PortSocketPair;
        static const TimeInterval BLOCKING_RECV_TIMEOUT_MS = 100;   // 受信時のPollのタイムアウト時間(ms)
        static const size_t MAX_NUM_SOCKETS = 16;                   // トランスポートバッファスレッドで扱うソケット数(この分だけ並列にログインできたりする)
        qVector<PortSocketPair> m_portAndSocketPairs;               // ポートとソケットの紐付け情報
        qVector<SocketDriver::PollInfo> m_pollInfos;                // Pollに渡すための情報
        ForcedCriticalSection m_csObjectLock;                       // m_portAndSocketPairsとm_pollInfosを守るためのロック
        ForcedCriticalSection m_csSocketLock;                       // m_portAndSocketPairsから削除した後にPollやRecvしないためのロック
        AtomicValue m_isPollInfoChanged;                            // m_pollInfosが変更されたかどうかを示すフラグ。
        AtomicValue m_isThreadStopping;                             // 受信スレッドが停止処理中かどうかを示すフラグ。
        ICContext m_icContext;                                      // PseudoSingleton用コンテキスト
        ObjectThread<TransportProtocolContext, int>* m_pReceivingThread;  // 受信スレッド
        TransportProtocol::Type m_transportProtocolType;            // 自身のオブジェクトが受信を担当しているプロトコル
    };

    volatile qBool m_bShutdown;  ///< スレッドの停止フラグ

    // PseudoSingleton用コンテキスト
    ICContext m_icContext;

    // トランスポートプロトコルごとの情報
    TransportProtocolContext m_protocolContextes[TransportProtocol::TYPE_COUNT];

    // 送信用スレッド起床用Condition
    Condition m_sendCond;

    // 送信用スレッド起床用Condition Lock
    NonReentrantableForcedCriticalSection m_sendCondLock;

    // 送信スレッド
    ObjectThread<TransportBufferMultiThread, int>* m_pSendingThread;
};

END_NAMESPACE_NEX
