﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/CriticalSection.h>


BEGIN_NAMESPACE_NEX
class GameStream;
class CallContext;
class StationProbeList;
class ReadWriteLock;

/*!
    @brief VSocket は UDP によるソケット通信に似たインターフェースを提供する低レベルな通信ライブラリです。

    VSocket を使うと NEX のサーバー機能や NAT トラバーサル機能や暗号化・改ざん検知機能を利用しつつ
    クライアント同士の P2P 通信に任意のサードパーティー製のネットワークエンジンを利用できるようになります。
    サードパーティー製のネットワークエンジンからソケット API を呼び出している箇所を
    VSocket の API を呼び出すように変更して使用されることを想定しています。

    VSocket はソケットの UDP と同様にコネクションレスなプロトコルであり、信頼性や順序性を保証しません。
    一般に端末は NAT デバイスの配下に置かれている事が多いため、VSocket による通信を行う前に NAT トラバーサルを行う必要があります。
    通信経路上の NAT デバイスの NAT テーブルを維持するため、アプリケーション側でキープアライブを行う必要があります。
 */
class VSocket: public RootObject
{
    // New types
public:
    /*!
        @brief 結果を表す列挙子です。
    */
    enum Result
    {
        /// 呼び出しは成功で完了した。
        Success,
        /// 呼び出しはエラーで完了した。
        Error,
        /// 呼び出しはブロックするためまだ完了していない。
        WouldBlock,
        /// パケットバッファの不足により呼び出しは失敗した。PacketBufferManager を参照してください。
        PacketBufferFull
    };

    // (SocketConstants::DEFAULT_MTU(1364) - UDP_HEADER_SIZE(28) - PRUDPMessageV1::HEADER_SIZE(30)
    //     - RelayMessage::HEADER_SIZE(44) - MARGIN_SIZE(12) = 1250)
    static const size_t MAX_DATA_SIZE = 1250;   //!< 一度に送信可能な最大サイズ (byte)。
    static const size_t MARGIN_SIZE = 12;       // 内部プロトコルの変更があってもMAX_DATA_SIZEが変わらないようにマージンを取る
    static const size_t DEFAULT_MAX_RECV_PACKTES = 128; //!< デフォルトの受信パケットキューサイズです。

    // VSocket::ProbeStations() で使用されるタイムアウト時間 (ミリ秒) です。
    // 実際には内部で個別のタイムアウト処理がなされるため、これは理論上の最大時間です。
    static const qUnsignedInt32 TOTAL_NAT_TRAVERSAL_TIMEOUT_MS = 120 * 1000;

    /*!
        @brief IP アドレスとポート番号を保持するクラスです。
        このクラスは常にホストバイトオーダーで表現されます。
    */
    class InetAddress : public RootObject
    {
    public:
        /*!
            @brief コンストラクタ。無効な値で生成します。
        */
        InetAddress()
        {
            m_uiAddress = 0;
            m_uiPort = 0;
            m_uiVPort = 0;
        }
        /*!
            @brief デストラクタ。
        */
        ~InetAddress() {}

        /*!
            @brief コンストラクタ。

            引数はホストバイトオーダーで指定してください。

            @param[in] uiAddress IP アドレス。
            @param[in] uiPort ポート番号。
            @param[in] uiVPort 仮想ポート番号。
         */
        InetAddress(qUnsignedInt32 uiAddress, qUnsignedInt16 uiPort, qUnsignedInt8 uiVPort)
            :   m_uiAddress(uiAddress), m_uiPort(uiPort), m_uiVPort(uiVPort) {}

        /*!
            @brief IP アドレスを設定します。

            引数はホストバイトオーダーで指定してください。

            @param[in] uiAddress IP アドレス。
         */
        void SetAddress(const qUnsignedInt32 uiAddress)
        {
            m_uiAddress = uiAddress;
        }

        /*!
            @brief IP アドレスを取得します。

            返値はホストバイトオーダーです。

            @return IP アドレス。
         */
        qUnsignedInt32 GetAddress() const
        {
            return m_uiAddress;
        }

        /*!
            @brief ポート番号を設定します。

            引数はホストバイトオーダーで指定してください。

            @param[in] uiPort ポート番号。
         */
        void SetPortNumber(const qUnsignedInt16 uiPort)
        {
            m_uiPort = uiPort;
        }

        /*!
            @brief ポート番号を取得します。

            返値はホストバイトオーダーです。

            @return ポート番号。
         */
        qUnsignedInt16 GetPortNumber() const
        {
            return m_uiPort;
        }

        /*!
            @brief 仮想ポート番号を設定します。

            @param[in] uiVPort 仮想ポート番号。
         */
        void SetVPortNumber(const qUnsignedInt8 uiVPort)
        {
            m_uiVPort = uiVPort;
        }

        /*!
            @brief 仮想ポート番号を取得します。

            @return 仮想ポート番号。
         */
        qUnsignedInt8 GetVPortNumber() const
        {
            return m_uiVPort;
        }

        /*!
            @brief 比較演算子です
          */
        qBool operator==(const InetAddress& oOther) const
        {
            return m_uiAddress == oOther.m_uiAddress && m_uiPort == oOther.m_uiPort && m_uiVPort == oOther.m_uiVPort;
        }

    private:
        qUnsignedInt32 m_uiAddress;
        qUnsignedInt16 m_uiPort;
        qUnsignedInt8 m_uiVPort;
    };

    // Methods
public:
    /*!
       @brief コンストラクタです。
       
       maxRecvPackets は @ref RecvFrom() で受信するまでにライブラリ内部に貯めておけるパケットの数です。
       この値より多くのパケットを受信した場合は破棄されます。
       送信頻度を高くしたい場合はこの値を増やしてください。

       @param[in] maxRecvPackets 受信パケットキューサイズです。
     */
    VSocket(size_t maxRecvPackets = DEFAULT_MAX_RECV_PACKTES);

    /*!
       @brief デストラクタです。
     */
    virtual ~VSocket();

    /*!
        @brief VSocket をオープンします。

        事前に VSocket::StartNATSession() の呼び出しが必要です。
        マッチメイクし、新しい P2P セッションを構築する度に VSocket::StartNATSession() 及び VSocket::StopNATSession()
        を呼び出す必要があります。
        事前に NGS から取得した鍵を Network::SetP2PDataPacketSessionSignatureKey() でセットする必要があります。

        @sa MatchmakeSession::GetSessionKey() VSocket::Close() VSocket::StartNATSession()
       
        @retval true 成功。
        @retval false 事前に VSocket::StartNATSession() を呼び出していない。
                      もしくは事前に Network::SetP2PDataPacketSessionSignatureKey() を呼び出していない。
     */
    qBool Open();

    /*!
        @brief VSocket をクローズします。
        @sa VSocket::Open()
    */
    void Close();

    /*!
        @brief VSocket を指定した仮想ポートに紐づけます。

        複数の VSocket オブジェクトを生成し、それぞれ異なる仮想ポートを紐づけた場合、内部的には同一の UDP ソケットが使用されます。
        ただし、送受信データは仮想ポート番号に応じて多重化(マルチプレックス)、多重分離(デマルチプレックス)されます。
        一つのアプリケーションで最大 16 個の仮想ポートを使用することができます。

        @param[in] uiVPort 送受信に使用する仮想ポート。0 から 15 までの任意の数を指定してください。
     */
    qBool Bind(qUnsignedInt8 uiVPort);

    /*!
        @brief データを受信します。

        本関数はノンブロッキングで実行されます。

        @param[out] pBuffer 受信データを書き込むバッファ。
        @param[in] uiBufferSize pBuffer のサイズ。最大の受信サイズは VSocket::MAX_DATA_SIZE です。
        @param[out] pSourceAddress 送信元のアドレス情報。
        @param[out] puiNbBytesRecvd pBuffer に書き込まれたサイズ。
                    VSocket::SendTo() で送信されたサイズと同じサイズで受信します。一つの送信データが分割されて受信する事はありません。
                    NULL を指定した場合は書き込まれません。

        @retval VSocket::Success 受信に成功した。pSourceAddress と puiNbBytesReceived が更新されます。
        @retval VSocket::Error uiBufferSize が 0。 VSocket::Open() が呼び出されていない。VSocket::Bind() が呼び出されていない。
        @retval VSocket::WouldBlock 受信するデータが存在しない。
        @sa VSocket::SendTo()
     */
    Result RecvFrom(qByte* pBuffer,
                    size_t uiBufferSize,
                    VSocket::InetAddress* pSourceAddress,
                    size_t* puiNbBytesRecvd);

    /*!
        @brief データを送信します。

        本関数はノンブロッキングで実行されます。

        @param[in] pBuffer 送信データ。
        @param[in] uiBufferSize pBuffer のサイズ。1 以上 VSocket::MAX_DATA_SIZE 以下の数値を指定できます。
        @param[in] oDestinationAddress 宛先のアドレス情報。NAT トラバーサル済みの IP アドレスとポート番号、及び VSocket の仮想ポート番号を指定する必要があります。
        @param[out] puiNbBytesSent 送信されたサイズ。
                    VSocket::Success が返されたときは uiBufferSize と同じサイズ、それ以外の場合は 0 が書き込まれます。
                    中途半端なサイズで送信されることはありません。NULL を指定した場合は書き込まれません。

        @retval VSocket::Success 送信に成功した。puiNbBytesSent が更新されます。
        @retval VSocket::Error uiBufferSize が 0 もしくは VSocket::MAX_DATA_SIZE を超えている。
                               VSocket::Open() が呼び出されていない。VSocket::Bind() が呼び出されていない。
        @retval VSocket::PacketBufferFull 送受信バッファが不足していて送信できない。
        @sa VSocket::RecvFrom() PacketBufferManager
     */
    Result SendTo(const qByte* pBuffer,
                  size_t uiBufferSize,
                  const VSocket::InetAddress& oDestinationAddress,
                  size_t* puiNbBytesSent);


    /*!
        @brief NAT トラバーサルを行います。

        本関数を呼び出す前に ConnectivityManager::StartNATSession() の呼び出しが必要です。
        本関数呼び出し時に preflstStationProbe に保存されている前回試行した NAT トラバーサルの結果 (PingTime (RTT) 情報や接続情報) はクリアされます。
        本関数では、@ref CallContext::SetTimeout() や @ref CallContext::Cancel() で非同期処理をコントロールしないでください。
        タイムアウト時間は内部で自動的に設定されます。
        非同期処理に成功した場合は、指定した全ての相手と NAT トラバーサルが成功したことを示します。
        非同期処理の結果、以下の Result が返ることがあります。

        @li QERROR(Core, InvalidSequence) : 事前に ConnectivityManager::StartNatSession() を呼び出していない。
        @li QERROR(Transport, NatTraversalError) : 少なくとも一人の相手と NAT トラバーサルに失敗した。(タイムアウトによる失敗を含む)
                                                   bWaitForAllResult に true を指定した場合は preflstStationProbe を参照する事でどの端末と失敗したかを調べる事ができます。

        @param[in,out] pContext 呼び出し情報。
        @param[in,out] preflstStationProbe NAT トラバーサルを行う相手の情報を指定します。
                                           非同期処理が完了すると それぞれの相手ごとの NAT トラバーサルの成否、PingTime (RTT)、接続情報が書き込まれます。
        @param[in] ui32Timeout 指定した値は無視されます。0 を指定してください。タイムアウト時間は内部で自動的に設定されます。
        @param[in] bWaitForAllResult false を指定すると、NAT トラバーサル対象のうち一つでも NAT トラバーサルに失敗した時に即座に非同期処理を完了します。
                                     true を指定すると全ての NAT トラバーサル対象との成否が確定するまで非同期処理が完了しません。
                                     全ての相手と NAT トラバーサルをする必要があり、一つでも失敗してはいけないケースでは false を指定してください。
                                     これはフルメッシュ型もしくはサーバークライアント(スター)型のネットワークトポロジを構成する場合に該当します。
                                     NAT トラバーサルが成功した相手とのみ通信したいケースでは true を指定してください。
                                     これは NAT トラバーサルに失敗した相手とは他の端末を中継して通信するなどの特殊なネットワークトポロジを採用する場合に該当します。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    static qBool ProbeStations(CallContext *pContext, StationProbeList* preflstStationProbe, qUnsignedInt32 ui32Timeout = 0, qBool bWaitForAllResult = false);


    // Data members
private:
    GameStream* p_gameStream;
    size_t m_maxRecvPackets;
    ReadWriteLock* m_pReadWriteLock;
};

END_NAMESPACE_NEX

