﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Platform/Core/ProfilingUnit.h>
#include <OnlineCore/src/Core/ByteStream.h>
#include <OnlineCore/src/Core/LogicalClock.h>
#include <OnlineCore/src/Core/Checksum/Interface/SignatureBytes.h>
#include <OnlineCore/src/Transport/UDP/PacketDefines.h>
#include <OnlineCore/src/Transport/PRUDP/IORequestContext.h>
#include <OnlineCore/src/Transport/PRUDP/RTT.h>
#include <OnlineCore/src/Transport/Interface/EndPoint.h>
#include <OnlineCore/src/Transport/Interface/PacketEncDec.h>

//n1589:PRUDPの送受信状況をプロファイリング可能にする、アプリ開発のため製品ビルドでのプロファイリングが必要、処理負荷はごくわずか
#ifdef NEX_CLIENT_OPTIMIZATION
#define NEX_ENABLE_PRUDP_PROFILING 1
#endif
#include <OnlineCore/src/Transport/Interface/EndPointInfo.h>

//n1589:再送パケットタイムアウトをアプリで調節可能にする
#define NEX_ENABLE_CALC_RETRANSMISSION_RTO_CALLBACK 1

BEGIN_NAMESPACE_NEX

class PRUDPStream;
class PacketDispatchQueue;
class SlidingWindow;
class PacketOut;
class PacketIn;
class Packet;
class PRUDPMessageInterface;
class PRUDPEndPointTemplateInterface;

/*!
 * @brief   NEXのパケット通信機能を取りまとめて管理するクラスです。
 *
 * PRUDPEndPoint は、UDPパケットを使用してNEXの通信機能を実現・管理しているクラスです。
 * 主な機能として、
 *
 *  ・ Ackを管理してリライアブル通信を実現<br>
 *  ・ キープアライブの処理<br>
 *  ・ パケットのタイムアウト処理<br>
 *
 * などを行います。
 * @ref Station::GetPRUDPEndPoint で取得することができます。
 */
class PRUDPEndPoint: public EndPoint
{
    friend class PRUDPStream;
    friend class TimeoutManager;

public:


    static const qUnsignedInt32 SupportedFunctionsMinorVersionMask = 0xFF;
    static const qUnsignedInt32 SupportedFunctionsMinorVersionMaskShift = 0;
    static const qUnsignedInt32 SupportedFunctionsUsingOptionMask = 0xFFFFFF00;
    static const qUnsignedInt32 SupportedFunctionsUsingOptionMaskShift = 8;

    static const qUnsignedInt32 INITIAL_DISCONNECT_PACKET_RTO;  // TYPE_DISCONNECTパケットの初期RTO
    static const qUnsignedInt32 MAX_DISCONNECT_PACKET_RTO;      // TYPE_DISCONNECTパケットの最大RTO

    enum SupportedFunctionsFlags
    {
        // option mask (32bit)
        ENABLE_FAST_SORRY                                  = 0x000001ULL,
        // over 0xFFFFFF is created from minor vesion (64bit)
        SUPPORTED_FLAG_RVDDLCLASSREVISION               = 0x001000000ULL,  // PRUDPv1.3以降
        SUPPORTED_FLAG_NO_SESSION_ID                    = 0x002000000ULL,  // PRUDPLiteV1は、SeesionIDなし
        SUPPORTED_FLAG_CONNECTION_SIGNATURE             = 0x004000000ULL,  // PRUDPLiteV1 minor version 5 チャレンジレスポンスを行う設定
        SUPPORTED_FLAG_ADD_DISCONNECT_REASON            = 0x008000000ULL   // PRUDPv1 minor version 5 PRUDPLiteV1 minor version 6 Disconnect時に切断理由を付加する
    };


#if NEX_ENABLE_CALC_RETRANSMISSION_RTO_CALLBACK
    typedef qUnsignedInt32 (*PRUDPCalcRetransmissionTimeoutCallback)(qUnsignedInt32 uiRto, qUnsignedInt16 uiNbTransmission);
    static void SetCalcRetransmissionTimeoutCallback(PRUDPCalcRetransmissionTimeoutCallback callback);
#endif

    PRUDPEndPoint(ConnectionOrientedStream *pStream, const StationURL *pStationURL,
                  LogicalClock16 *p_oNextUnreliableBroadcastSeqIdToSend, qByte version);
    virtual ~PRUDPEndPoint();

    virtual qResult _Connect(Buffer *pOutBuffer, Buffer *pInBuffer,
                             const EndPoint::Callback pCallback, const UserContext &oCallbackContext,
                             qUnsignedInt32 uiTimeout);

    virtual qResult _Disconnect(const EndPoint::Callback pCallback, const UserContext &oCallbackContext,
                                qUnsignedInt32 uiTimeout);

    virtual qResult         _Send(Buffer *pBuffer, qUnsignedInt32 uiFlags, qBool qEncrypt=false, qByte substream=0, qUnsignedInt32 uiCallID=0);

    /*!
     * @internal
     * @brief   最新のRTTを取得します。
     *
     *          最後に通信して取得することができたRTTを返します。
     *
     *          RTTとは "Rount Trip Time" の略で、往復遅延時間のことで「パケットを送信して、相手から返ってくるまでの往復にかかった時間」のことです。
     *
     * @retval  qUnsignedInt32  最新のRTT。単位はmsec。
     */
    virtual qUnsignedInt32  GetRTT()  const
    {
        return m_oRTT.GetLastRTT();
    }

    /*!
     * @internal
     * @brief RTTの平均を取得します。
     *
     *          通信して取得することができたRTTの平均を返します。
     *
     *          RTTとは "Rount Trip Time" の略で、往復遅延時間のことで「パケットを送信して、相手から返ってくるまでの往復にかかった時間」のことです。
     *
     * @retval  qUnsignedInt32  RTTの平均値。単位はmsec。
     */
    virtual qUnsignedInt32  GetRTTAverage() const
    {
        return m_oRTT.GetRTTSmoothedAvg();
    }

    size_t  GetNbReliablePending(qByte substream);

    RTT             *GetRTTPtr()
    {
        return &m_oRTT;
    }

    virtual qBool IsNotConnected()
    {
        return (m_eConnectionState == NotConnected);
    }
    virtual qBool IsConnecting()
    {
        return (m_eConnectionState == Connecting);
    }
    virtual qBool IsDisconnecting()
    {
        return (m_eConnectionState == Disconnecting);
    }
    virtual qBool IsFaulty()
    {
        return (m_eConnectionState == Faulty);
    }
    virtual qBool IsConnected()
    {
        return (m_eConnectionState == Connected);
    }
    virtual ConnectionState GetConnectionState() const
    {
        return m_eConnectionState;
    }
    virtual qBool PeerIsConnected()
    {
        return m_bPeerConnected;
    }
    virtual qBool PeerIsDisconnected()
    {
        return !m_bPeerConnected;
    }

    virtual void            SetKeepAliveTimeout(qUnsignedInt32 uiKeepAliveTimeout);
    virtual qUnsignedInt32  GetKeepAliveTimeout()
    {
        return m_uiKeepAliveTimeout;
    }

    virtual qUnsignedInt32 GetMaxSilenceTime()
    {
        return m_uiMaxSilenceTime;
    }

    virtual void SetMaxSilenceTime(qUnsignedInt32 uiMaxSilenceTime)
    {
        m_uiMaxSilenceTime = uiMaxSilenceTime;
    }

    //  This is called when the endpoint receives an ack on a CONNECT request from the peer.
    virtual void SetPeerConnected()
    {
        m_bPeerConnected = true;
#ifdef NEX_TRACE_ENABLED    // n2370
        Trace();
#endif
    }

    virtual void SetPeerDisconnected()
    {
        m_bPeerConnected = false;
#ifdef NEX_TRACE_ENABLED    // n2370
        Trace();
#endif
    }

    // 接続相手のアドレスを取得する。
    const InetAddress *GetInetAddress()
    {
        return &m_oStationURL.GetInetAddress();
    }

    //n2690
    void SetIsOldRVSecure(qBool flag)
    {
        m_IsOldRVSecure = flag;
    }
    qBool IsOldRVSecure() const
    {
        return m_IsOldRVSecure;
    }

    qBool IsDuplicateReorderingPacket(PacketIn *pPacket);

    virtual qBool IsAnyPacketProcessed();

    //n1589:moved to public
    Stream::ID GetStreamID() const
    {
        return m_oStationURL.GetStreamID();
    }

    qByte GetPeerSessionID() const
    {
        return m_byPeerSessionID;
    }

    qByte GetSessionID() const
    {
        return m_bySessionID;
    }

    qByte           GetVersion() const
    {
        return m_version;
    }

    qUnsignedInt32  GetSupportedFunctions() const
    {
        return  m_uiSupportedFunctions;
    }

#if NEX_ENABLE_USE_SECURE_KEY
    virtual void SetEncryptionKey(const Key &oKey)
    {
        m_pPacketEncDec->SetEncryptionKey(oKey);
    }
    virtual Key *GetEncryptionKey()
    {
        return m_pPacketEncDec->GetEncryptionKey();
    }
#endif

#if NEX_ENABLE_HYBRID
    static void SetHybridPartialFaultProcessingMode(qBool bRecover);
    static qBool s_bRecoverPartialFault;
#endif

    static PseudoGlobalVariable<qByte> s_sessionID;

    /*
     * @brief       EndPointInfo のパラメータをリセットします。
     *
     * @see         EndPointInfo , LogPRUDPAnalyzeInfo() , GetEndPointInfo()
     */
    virtual void ResetEndPointInfo();

    /*
     * @brief       EndPointInfo のパラメータを取得します。
     *
     * @param[out]   info コピー先のEndPointInfo構造体を設定します
     * @retval true 取得が成功した場合
     * @retval false 取得が失敗した場合 (infoがnull)
     * @see         EndPointInfo , ResetEndPointInfo() , GetEndPointInfo()
     */
    virtual qBool GetEndPointInfo(EndPointInfo* info) const;


    /*!
     * @brief EndPointのシステムと共用のreliable送信バッファの現在のパケット数を取得します。
     *
     * reliable送信バッファは通信経路の混雑時などに徐々に増加していきます。
     * reliable送信バッファのパケット数がStream::DOのMaxWindowMapSize(SetMaxWindowMapSizeで指定したサイズ)を超えた場合、送信バッファあふれによるエラーとなります。
     *
     * @return reliable送信バッファのパケット数。
     */
    virtual size_t GetReliableBufferPacketCount() const;

    size_t GetReliableBufferPacketCount(qByte substream) const;

    /*!
     * @brief EndPointのすべてのreliable送信バッファの現在のパケット数を取得します。
     *
     * reliable送信バッファは通信経路の混雑時などに徐々に増加していきます。
     * reliable送信バッファのパケット数がStream::DOのMaxWindowMapSize(SetMaxWindowMapSizeで指定したサイズ)を超えた場合、送信バッファあふれによるエラーとなります。
     *
     * @param[out] counts パケット数を返すqVectorを指定します。添え字0は、システムと共用で、ユーザー用は添え字1から続きます。
     */
    virtual void GetReliableBufferPacketCounts(qVector<size_t>& counts) const;

    /*!
     * @brief 最大SubStreamIDを取得します。
     *
     * 通信相手と StreamSettings::SetMaxUserReliableSubStreams()の値が異なった場合には、ネゴシエーションにより小さい方に最大サブストリームIDが決定されます。
     *
     * @return 最大SubStreamID
     */
    virtual qByte GetMaxSubStreamID() const
    {
        return static_cast<qByte>(m_uiMaxUserReliableSubStreams);
    }

    void AnalyzeRttRto(qUnsignedInt32 rto);

    EndPointInfo* GetAnalyzeInfo()
    {
        return &m_AnalyzeInfo;
    }

    /*
    n1589:これらのパケロス率計算APIは、他のステーション宛ブロードキャストパケットを受信するステーションでは
    シーケンス番号がスキップするため適切な値にはなりません
    */
    //n2788:パケロス率計算API
    //      EndPointの実体作成時点もしくはResetAnalyzeInfo最終呼出時点から
    //      APIを呼び出した瞬間までのパケロス率を返す
    qFloat PacketLossRateInReceiveUnreliable();
    //n2788:パケロス率計算API 別仕様
    //      EndPointの実体作成時点もしくはResetAnalyzeInfo最終呼出時点から
    //      APIを呼び出した瞬間までの、「パケロス率」と「受信すべきパケットの総数」を返す
    void PacketLossRateInReceiveUnreliable(qFloat *packetLossRate, size_t *totalPackets);
    //n2788:パケロス率計算補助API
    //      EndPointの実体作成時点もしくはResetAnalyzeInfo最終呼出時点から
    //      受信すべきパケットの総数が指定値を超えていたらtrue、そうでなければfalseを返す
    qBool TotalPacketsInReceiveUnreliableIsOver(const size_t thresholdPackets);

    // 1パケットの最大サイズの取得。
    size_t GetMaxPacketSize();

    static inline qUnsignedInt32 SetSupportedFunctionsMinorVersion(qUnsignedInt32 supportedFunctions, qByte version)
    {
        qUnsignedInt32 v = (static_cast<qUnsignedInt32>(version) << SupportedFunctionsMinorVersionMaskShift)
                           & SupportedFunctionsMinorVersionMask;
        return (supportedFunctions & (~SupportedFunctionsMinorVersionMask)) | v;
    }
    static inline qByte GetSupportedFunctionsMinorVersion(qUnsignedInt32 supportedFunctions)
    {
        return static_cast<qByte>((supportedFunctions & SupportedFunctionsMinorVersionMask) >>  SupportedFunctionsMinorVersionMaskShift);
    }

    static inline qUnsignedInt32 SetSupportedFunctionsOptionMask(qUnsignedInt32 supportedFunctions, qUnsignedInt32 mask)
    {
        qUnsignedInt32 m = (mask << SupportedFunctionsUsingOptionMaskShift)
                           & SupportedFunctionsUsingOptionMask;
        return (supportedFunctions & (~SupportedFunctionsUsingOptionMask)) | m;
    }


    static inline qUnsignedInt32 GetSupportedFunctionsOptionMask(qUnsignedInt32 supportedFunctions)
    {
        return (supportedFunctions & SupportedFunctionsUsingOptionMask) >>  SupportedFunctionsUsingOptionMaskShift;
    }

    static inline qUnsignedInt32 SetSupportedFunctionsOptionFlag(qUnsignedInt32 supportedFunctions, qUnsignedInt32 flag)
    {
        return SetSupportedFunctionsOptionMask(supportedFunctions, GetSupportedFunctionsOptionMask(supportedFunctions) | flag);
    }

    static inline qUnsignedInt32 ClearSupportedFunctionsOptionFlag(qUnsignedInt32 supportedFunctions, qUnsignedInt32 flag)
    {
        return SetSupportedFunctionsOptionMask(supportedFunctions, GetSupportedFunctionsOptionMask(supportedFunctions) & (~flag));
    }

    static inline qBool DoesSetSupportedFunctionsOptionFlag(qUnsignedInt32 supportedFunctions, qUnsignedInt32 flag)
    {
        return (GetSupportedFunctionsOptionMask(supportedFunctions) & flag) != 0;
    }

    static qBool IsNegotiationConnectionSignatureVersion(const Packet *pPacket);

    virtual qUnsignedInt64 GetSupportedFunctionsFlags() const
    {
        return m_uiSupportedFunctionsFlags;
    }

    void SetSignaledFaultError(NetFaultReason uiError)
    {
        if (m_uiSignaledFaultError != NET_FAULT_INVALID)
        {
            // すでにエラーがセットされていた場合は変更しない。
            return;
        }

        // 初回にエラーをセットするときだけログ表示
        QLOG(
            EventLog::Info,
            NEX_T("PRUDPEndPoint::ProcessFaultEvent:ConnectionID:") << hex << GetConnectionID()
            << NEX_T(", IsConnected:") << IsConnected()
            << NEX_T(", PeerIsConnected:") << PeerIsConnected()
            << NEX_T(", Error:") << uiError << NEX_T(" -> ") << GetNetFaultString(uiError)
        );
        m_uiSignaledFaultError = uiError;
    }

    virtual NetFaultReason GetSignaledFaultError() const
    {
        return m_uiSignaledFaultError;
    }

    // EndPointを削除するまでの猶予時間
    static const TimeInterval EXTENSION_INTERVAL_TO_DELETE_MS = 5 * 1000;

protected:
#if NEX_ENABLE_CALC_RETRANSMISSION_RTO_CALLBACK
    static PRUDPCalcRetransmissionTimeoutCallback s_pfCalcRetransmissionTimeoutCallback;
#endif

    qResult Frag(
        Buffer *pBuffer,
        qUnsignedInt32 uiMTU,
        size_t uiMaxPacketSize,
        PacketType uiPacketType,
        qUnsignedInt16 uiPacketFlags,
        qBool bExpress,
        qUnsignedInt32 uiCallID,
        qByte subStreamID
    );

    qBool           Defrag(PacketIn *pDispatchPacket);
    PacketOut       *GetNextSendPacket();
    PacketIn        *GetNextDispatchPacket(qByte substream);
    void            Dispatch(PacketIn *pPacket, Time oIOTime);
    //n1589
    void HandlerDispatch(Buffer* pBuffer, PacketIn* pPacket);

    void            ServiceIncomingPacket(PacketIn *pPacket);

    void            ProcessConnect(Packet *pPacket);
    void            ServiceTimeout(PacketOut *pPacket, const Time& currentTime);
    virtual void    SignalFaultEvent(NetFaultReason uiError, qBool bProcessNow);
    void            ProcessFaultEvent(NetFaultReason uiError);

    qBool           EarlyConnectAllowed();
    void            AllowEarlyConnect();

    void            ConnectCompleted();

    void            PacketAggregateAcknoledged(Packet *pPacket);
    void            PacketAcknowledged(const Time &recvIoTime, const LogicalClock16& ackSeq, qByte subStreamID);
    qBool           CancelPacketTimeout(PacketOut *pPacket);
    void            StartPing();
    void            StopPing();
    void            TimeToPing();

    qBool           SendAggregateAck();

    qBool           ReadyToDisconnect();

    virtual qBool   SetConnectionState(ConnectionState eNewState);
    qBool           CompleteIO(qResult result);

    void            AdjustRTT(const Packet *pPacket, const PacketOut *pAckedPacket);
    void            AdjustRTT(const Time &ioTime, const PacketOut *pAckedPacket);
    qUnsignedInt    ComputeRetransmitTimeout(const PacketOut* pPacket);
    qReal           GetTimeoutMultiplier(qUnsignedInt16 ui16NbTransmission, StreamSettings &rSettings)const;

    void            SendNextReliable(qByte subStreamID);
    void            SendReliable(PacketOut *pPacket);
    qResult         SendPacket(PacketOut *pPacket);
    qResult         HandleSlidingWindowPushError();

    // 送信時に使用する自分のローカルポートを設定する。
    void            SetPreferredPort(qUnsignedInt16 uiPort)
    {
        m_uiPort = uiPort;
    }
    // 送信時に使用する自分のローカルポートを取得する。
    qUnsignedInt16  GetPreferredPort()
    {
        return m_uiPort;
    }

    qBool           IsIdle(qByte substream);
    qBool           IsFlooded();

    // AggregateACKをサポートしているか。
    qBool IsSendAggregateAckSupported()
    {
        return m_version != 0;
    }

    PRUDPStream *GetStream()
    {
        return (PRUDPStream*)m_pStream;
    }

    void SetDeadlineToDelete()
    {
        m_deadlineToDelete = Time::GetTime() + EXTENSION_INTERVAL_TO_DELETE_MS;
    }

    qBool IsExpiredToDelete(const Time& currentTime) const
    {
        return currentTime >= m_deadlineToDelete;
    }

    static const qChar* GetNetFaultString(qUnsignedInt32 uiError);

    qUnsignedInt16              m_uiPort;

    qVector<PacketDispatchQueue *> m_dispatchQueue;
    qVector<SlidingWindow *>    m_slidingWindow;

    PacketOut                   *m_pPingPacket;
    PacketOut                   *m_pSynPacket;
    qBool                       m_bSynPhaseCompleted;
    qUnsignedInt32              m_uiMaxSilenceTime;
    qUnsignedInt32              m_uiKeepAliveTimeout;
    ConnectionState             m_eConnectionState;
    qBool                       m_bPeerConnected;
    NetFaultReason              m_uiSignaledFaultError;

    RTT                         m_oRTT;

    Time                        m_oLastPacketSentTime;
    Time                        m_oLastPacketRecvTime;

    LogicalClock16              m_lcNextUnreliableSeqIdToSend;
    LogicalClock16              m_lcNextUnreliableSeqIdToRecv;
    LogicalClock16              *m_pNextUnreliableBroadcastSeqIdToSend; //n2967:PRUDPStreamよりPRUDPEndPoint間全体で共有される
    LogicalClock16              m_lcNextUnreliableBroadcastSeqIdToRecv;

    qUnsignedInt16              m_uiMaxUserReliableSubStreams;

    SignatureBytes         m_oSignature;
    SignatureBytes         m_oSendConnectionSignature;

    SignatureBytes* GetConnectionSignature()
    {
        return &m_oSignature;
    }
    SignatureBytes* GetSendConnectionSignature()
    {
        return &m_oSendConnectionSignature;
    }

    qVector<Buffer *>           m_currentPartialBuffer;
    qVector<qByte>              m_nextPartToRecv;

    //n1589:memo:this is not joining session id but endpoint session id
    qByte                       m_byPeerSessionID;
    qByte                       m_bySessionID;

    //n2697 Broadcastパケットは、代表となる一番上のStationIDから送信されるため通常の場合とSessionIDが異なる場合がある
    qByte                       m_byBroadcastPeerSessionID;

    qByte                       m_version;

    qUnsignedInt32              m_uiSupportedFunctions;

    qUnsignedInt64              m_uiSupportedFunctionsFlags;

    //n2690 旧RVSecureを使用しているクライアント
    qBool m_IsOldRVSecure;
    qVector<qBool> m_needAggregateACK; //注意 qBool


    ProfilingUnit               m_puSend;

    IORequestContext            m_oCurrentIORequestContext;

    PRUDPMessageInterface*       m_pPRUDPMessageInterface;
    PRUDPEndPointTemplateInterface*   m_pPRUDPEndPointTemplateInf;


#if NEX_ENABLE_USE_SECURE_KEY
    PacketEncDecInf* m_pPacketEncDec;
#endif

#if NEX_ENABLE_HYBRID
    //n2781 接続処理中に相手からの接続を受け取った場合Faultしない
    qBool m_bAvoidFaultWhileConnecting;
#endif

    EndPointInfo m_AnalyzeInfo;
    Time m_deadlineToDelete;        // EndPointを削除する時刻

    // CallContextに対するAck未受信数
    qMap<qUnsignedInt32, qUnsignedInt32> m_unackedCount;

private:
    void InitUnreliableSeq(PacketOut *packet);
    void SendACK(PacketIn *pPacket);
    void SendAggregateAck(Buffer* pBuffer, qUnsignedInt16 highestSeriesSeq, qByte subStreamID);
    void SetSupportedFunctionsFlags();
    void DecideMaxUserReliableSubStreams(qUnsignedInt16 maxUserReliableSubStreams);
    void SendResetDisconnectPacket();
};


END_NAMESPACE_NEX

