﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/


#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/Interface/Stream.h>

BEGIN_NAMESPACE_NEX
class PacketOut;
class PacketIn;
class TransportSignatureGenerator;
class ByteStream;
class SignatureBytes;


/**
 * @internal
 *
 * @brief PRUDPメッセージの生成・解読、署名作成を行うルーチンのInterface
 */
class PRUDPMessageInterface
{
public:
    /**
     * @internal
     *
     * @brief 署名ルーチン名
     */


    enum SignatureMethod
    {
        SignatureNone,
        SignatureAddress,
        SignatureData,
        SignatureDataWithHeader,
        SignatureAllPRUDP,
        SignatureAllPRUDPWithKey,
        SignatureHeaderOnly,
        SignatureRandom,
        SignatureAccesssKey,
        SignatureIgnored
    };

    /**
     * @internal
     *
     * @brief 署名チェック時の判定結果
     */
    enum SignatureMatchPattern
    {
        SignatureContinueProcessing,
        SignatureMatch,
        SignatureUnmatch,
        SignatureDrop
    };

    //ARMCCにて、pure virtual class内で、static constだとリンク時にシンボルエラーになるので、enumに修正
    enum
    {
        MAX_CALC_SIGNATURE_SIZE = 16, /*! すべてのバージョンを含めた最大署名サイズ */
        MIN_HEADER_SIZE = 10, /*! すべてのバージョンを含めた最少ヘッダサイズ */
        HEADER_MAGIC_NUMBER1 = 0xEA, /*! PRUDPV1以降のヘッダマジックナンバー ProtocolIDの1バイト目 */
        HEADER_MAGIC_NUMBER2 = 0xD0, /*! PRUDPV1以降のヘッダマジックナンバー ProtocolIDの2バイト目 */
        RELAY_HEADER_MAGIC_NUMBER = 0xF5, /*! Relay ProtocolのHeader Magic Number */
        RELAY_VERSION_BASE_NUMBER = 0xD0 /*! Relay Protocolの初期バージョン */
    };

    static const size_t SIGNATURE_SIZE = 16; /*! 署名サイズ */

    virtual ~PRUDPMessageInterface() {} ;


    /**
     * @internal
     *
     * @brief 署名演算
     *
     * @param [in] pPacket 演算対象パケット
     * @param [in] oMethod 利用署名ルーチン名 DecideSignatureMethodの帰り値
     * @param [in] pKey 付加キー情報
     * @param [in] oType Stream::Type
     * @param [in] pSignatureGenerator 署名生成クラス(一部使用)
     * @param [out] calcSignature 署名演算結果
     * @return false 内部処理に失敗
     */
    virtual qBool CalcSignature(PacketOut *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                                TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature) = 0;

    virtual void CalcSignature(Packet *pPacket, SignatureMethod oMethod, const Key *pKey, Stream::Type oType,
                               TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &calcSignature)= 0;

    /**
     * @internal
     *
     * @brief パケットに対する署名ルーチンを判別
     *
     * @param [in] pPacket 演算対象パケット
     * @param [in] oType Stream::Type
     * @param [in] hasEndPoint EndPointがあるか否か
     * @return 署名ルーチン名
     */
    virtual SignatureMethod DecideSignatureMethod(const Packet *pPacket, Stream::Type type, qBool hasEndPoint) = 0;

    /**
     * @internal
     *
     * @brief 受信パケットから期待される署名を演算
     *
     * @param [in] pPacket 演算対象パケット
     * @param [in] pKey 付加キー情報
     * @param [in] pSignatureGenerator 署名生成クラス(一部使用)
     * @param [out] calcSignature 署名演算結果
     */
    virtual SignatureMatchPattern CalcExpectedSignature(const Packet *pPacket, SignatureMethod method, const Key *pKey,
            TransportSignatureGenerator* pSignatureGenerator, const SignatureBytes *pConnectionSignature, SignatureBytes &expectedSignature) = 0;

    /**
     * @internal
     *
     * @brief PRUDPの解読
     *
     * @param [in,out] pByteStream 解読対象データ。解読できたサイズ分だけずれる
     * @param [out] pPacketIn 解読されたパケット
     * @param [out] pSizeUnpacked 解読サイズ
     * @return 解読できたか否か
     */
    virtual qBool Unpack(ByteStream *pByteStream, PacketIn *pPacketIn, size_t *pSizeUnpacked = NULL) = 0;

    /**
     * @internal
     *
     * @brief PRUDPの限定解読
     *
     * 過去履歴バンドリング用にm_oSourceVPort, m_uiPacketTypeFlags, m_lcSeqldとサイズのみチェック
     *
     * @param [in,out] pByteStream 解読対象データ。解読できたサイズ分だけずれる
     * @param [out] pPacketIn 解読されたパケット
     * @param [out] pSizeUnpacked 解読サイズ
     * @return 解読できたか否か
     */
    virtual qBool FastUnpack(ByteStream *pByteStream, PacketIn *pPacketIn, size_t *pPayloadSize = NULL) = 0;


    /**
     * @internal
     *
     * @brief PRUDPの生成
     *
     * @param [in,out] pPacketOut 出力パケット。生成されるとPackedSizeに生成されたデータサイズが設定される
     * @param [in,out] pByteStream 生成対象データ。解読できたサイズ分だけずれる
     * @return 生成できたか否か
     */
    virtual qBool Pack(PacketOut *pPacketOut, ByteStream *pByteStream) = 0;

};
END_NAMESPACE_NEX
