﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/PRUDP/PRUDPDecl.h>
#include <OnlineCore/src/Transport/Interface/ConnectionOrientedStream.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/TransportSignatureGenerator.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPointMap.h>
#include <OnlineCore/src/Transport/PRUDP/TimeoutManager.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPoint.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPMessageInterface.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDP.h>
#include <OnlineCore/src/Platform/Core/Time.h>
#include <OnlineCore/src/Platform/Core/qSet.h>

BEGIN_NAMESPACE_NEX

class Buffer;
class PacketOut;
class PacketIn;
class Packet;
class PRUDPMessageInterface;
class PRUDPStreamTemplateInterface;

//n1589:SilenceTimeを動的に反映する、KeepAliveTimeoutも動的変更にする予定
#define NEX_ENABLE_DYNAMIC_MAX_SILENCE_TIME 1

//n1589
#define NEX_ENABLE_PRUDP_STREAM_END_POINT_LOCK 1

#define PRUDP_ADDRESS_KEY           NEX_T("address")
#define PRUDP_PORT_KEY              NEX_T("port")

class PRUDPStream: public ConnectionOrientedStream
{
    friend class PRUDPEndPoint;
    friend class JobTerminateFacade; // TimeoutManagerを見たいため
    friend class JobTerminateDOCore; // TimeoutManagerを見たいため
public:
    PRUDPStream(Stream::Type oType, RootTransport *pTransport = NULL);
    virtual ~PRUDPStream();

    //{secret}
    virtual qResult         Initialize();
    //{secret}
    virtual qResult         Teardown();

    virtual qBool           ResponsibleForURL(const StationURL *pStationURL);
    virtual const qChar*    GetURLType();

    virtual EndPoint        *OpenEndPoint(const StationURL *pURL);
    virtual EndPoint        *OpenEndPoint(const ConnectionID cidStation);
    virtual qBool           OpenEndPoint(EndPoint *pEndPoint);
    virtual void            CloseEndPoint(EndPoint *pEndPoint);
    virtual void            ForceCompleteIO();

    virtual qBool           IsCapable(qUnsignedInt32 uiCapability);

    //{secret}
    virtual EndPoint        *FindEndPointByPID(const PrincipalID pidStation);
    //{secret}
    virtual qUnsignedInt32  GetEndPointNumber();
    //{secret}
    virtual EndPoint        *FindEndPointByCID(const ConnectionID cidStation);
    //{secret}
    virtual const qVector<ConnectionID> GetAllConnectionIDs() const;

    //{secret}
    virtual qResult         SendBroadcast(StationURL *pStationURL, Buffer *pBuffer);
    virtual qResult         Send(StationURL *pStationURL, Buffer *pBuffer);

    virtual qBool           StopListen();

#if NEX_ENABLE_DYNAMIC_MAX_SILENCE_TIME
    virtual void SetMaxSilenceTimeAll(qUnsignedInt32 uiMaxSilenceTime);
#endif

    //n1769 UDPTransport::Send()のキューイングを実行するかどうか
    static void  EnableQueuing   (void)
    {
        s_bQueuing = true;
    }
    static void  DisableQueuing  (void)
    {
        s_bQueuing = false;
    }
    static qBool IsEnabledQueuing(void)
    {
        return s_bQueuing;
    }

#if NEX_ENABLE_PRUDP_STREAM_END_POINT_LOCK
    virtual void LockEndPointMap();
    virtual void UnlockEndPointMap();
#endif

    void OnSetTerminateImmediately();

protected:
    // 以下の関数はフレンドクラスであるPRUDPEndPointから参照される。
    //{secret}
    virtual void            AddPIDEndPointAssociation(const PrincipalID pidStation, EndPoint *pEndPoint);
    //{secret}
    virtual qBool           RemovePIDEndPointAssociation(const PrincipalID pidStation, EndPoint *pEndPoint);
    //{secret}
    virtual void            AddCIDEndPointAssociation(const ConnectionID cidStation, EndPoint *pEndPoint);
    //{secret}
    virtual qBool           RemoveCIDEndPointAssociation(const ConnectionID cidStation, EndPoint *pEndPoint);

    void                    NotifySendAggregateAckNeeded(PRUDPEndPoint* pEndPoint)
    {
        SYSTEMCHECK(Scheduler::GetInstance()->IsLockedByCurrentThread());
        m_setAggregateAckNeededEndPoints.insert(pEndPoint);
    }
private:
    PRUDPEndPoint*  CreateEndPoint(const StationURL *pStationURL,
                                   qUnsignedInt16 uiOutgoingPort,
                                   qBool bOutgoing);
    inline void     CloseEndPointImpl(EndPoint *pEndPoint);
    void            ReleaseEndPoint(PRUDPEndPoint *pEndPoint);

    PRUDPEndPoint*  ServiceConnectionRequest(InetAddress *pInetAddress, Buffer *pBuffer,
            qUnsignedInt16 uiPort, Stream::ID idSource);
    void            ServiceUserMsg(InetAddress *pInetAddress, Buffer *pBuffer);

#if NEX_ENABLE_HMAC_USE_SECURE_KEY
    qBool           Send(qUnsignedInt16 uiPort, Stream::ID idDestination, PacketOut *pPacket, Key *pKey = 0);
#else
    qBool           Send(qUnsignedInt16 uiPort, Stream::ID idDestination, PacketOut *pPacket);
#endif

    //  void            DemuxIncomingICMP(const InetAddress *pInetAddress, VirtualPort oVPort);
    void            DemuxIncomingPacket(qUnsignedInt32 uiSocket, PacketIn *pPacket);

    TimeoutManager  *GetTimeoutManager()
    {
        return &m_oTimeoutManager;
    }
    
    void            CheckSocketStatus(const Time & currentTime);
    void            ServiceTimeouts(const Time& currentTime);
    void            PurgeDeletedEndPoints(const Time& currentTime, qBool force);

    virtual void            DoWork(const Time* currentTime);
    virtual qBool   ReceiveIncomingPacket(qUnsignedInt16 uiPort, Stream::ID idSource, Packet *pPacket);

    RootTransport*  GetTransport()
    {
        return m_pUDPTransport;
    }

    virtual qBool IsDuplicateReorderingPacket(Packet *pPacket);

#ifdef NEX_TRACE_ENABLED    // n2370
    void            Trace(qUnsignedInt64 uiTraceFlags = TRACE_ALWAYS);
#endif  // NEX_TRACE_ENABLED

    TransportSignatureGenerator*    GetSignatureGenerator()
    {
        return &m_oSignatureGenerator;
    }

    PRUDPMessageInterface::SignatureMatchPattern CheckSignature(InetAddress* pInetAddress, qUnsignedInt16 uiPort,
            Stream::ID idSource, Packet *pPacket);

private:
    // Illegal operations.
    PRUDPStream(const PRUDPStream&);
    PRUDPStream& operator=(const PRUDPStream&);

    void ReleaseEndPointImpl(PRUDPEndPoint *pEndPoint);

    qBool VersionNegotiation(Packet *pPacket, Packet *pAckPacket);
private:
    TimeoutManager              m_oTimeoutManager;
    PRUDPEndPointMap            m_oEndPointMap;
    qList<PRUDPEndPoint*>       m_oEndPointsToDelete;

    Time                        m_tiLastCheckSocketStatusTime;      // ServiceTimeouts()を最後に実行した時刻
    Time                        m_tiLastTimeoutServicingTime;       // ServiceTimeouts()を最後に実行した時刻
    Time                        m_tiLastPurgeDeletedEndPointsTime;  // PurgeDeletedEndPoints()を最後に実行した時刻
    qBool                       m_bShutdown;
    ICContext                   m_icPseudoSingletonContext;
#ifdef NEX_WATERMARK_ENABLED
    WaterMark                   m_wmConnections;
#endif

    TransportSignatureGenerator         m_oSignatureGenerator;

    LogicalClock16              m_lcNextUnreliableBroadcastSeqIdToSend; //n2967:PRUDPEndPoint間共通Broadcast用シーケンス番号

    typedef qMultiMap<PrincipalID,EndPoint*>    MMapPID2EndPoint;
    MMapPID2EndPoint                    m_mmapPIDEndPoint;

    // TODO: This should be a map, bug check out bug 2463
    typedef qMultiMap<ConnectionID,EndPoint*>    MMapCID2EndPoint;
    MMapCID2EndPoint                    m_mmapCIDEndPoint;
    qVector<ConnectionID>               m_vecCachedCID;
    qSet<PRUDPEndPoint*>                m_setAggregateAckNeededEndPoints;

    //n1769
    static qBool                        s_bQueuing;

    // CheckSocketStatus()を実行する最小間隔(ms)
    static const TimeInterval CHECK_SOCKET_STATUS_INTERVAL_MS;

    // ServiceTimeouts()を実行する最小間隔(ms)
    static const TimeInterval SERVICE_TIMEOUT_INTERVAL_MS;

    // PurgeDeletedEndPoints()を実行する最小間隔(ms)
    static const TimeInterval PURGE_DELETED_ENDPOINTS_INTERVAL_MS;

    PRUDPMessageInterface* m_pPRUDPMessageInterface;
    PRUDPStreamTemplateInterface* m_pPRUDPTemplateIntarface;
    qByte m_bInitialVersion;

#if NEX_ENABLE_PRUDP_STREAM_END_POINT_LOCK
    mutable CriticalSection m_csEndPointLock;
#endif

    qUnsignedInt16 m_responsibleRemotePort; // このStreamが受け持つ宛先ポート。0なら任意の接続先を受け持つ。

};

END_NAMESPACE_NEX

