﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Core/src/Client/AccountManagement/AccountManagementProtocolDDL.h>
#include <RendezVous/Core/src/Client/AccountManagement/BasicAccountInfoDDL.h>
#include <RendezVous/Core/src/Client/AccountManagement/AccountDataDDL.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>
#include <RendezVous/Core/src/Common/DataHolder.h>
#include <RendezVous/Core/src/Client/AccountManagement/PublicDataDDL.h>
#include <RendezVous/Core/src/Client/AccountManagement/PrivateDataDDL.h>

BEGIN_NAMESPACE_NEX

class StreamManager;
class Gathering;

typedef CustomDataHolder<PublicData> PublicDataHolder;
typedef CustomDataHolder<PrivateData> PrivateDataHolder;


/**
\ingroup group_rdv_core_services_accountmanagement_client
\brief This class enables Players to create and modify their lobby account.

\remarks Use this class to allow Players to create lobby account(s) that are uniquely identified by
a (case insensitive) user name and (case sensitive) password pair. Players can then login to the lobby using their own user
name / password pair defined when they created or modified the account. If you wish to allow the Player to add user-defined
data to an account, you can do so by implementing a custom Data type extension, as detailed in the
Custom Data Classes section.

\productname diffentiates between public and private account data in that a given AccountManagementClient can
access its own Public and Private data, whereas it can access only the Public data of others. Data is defined as public or private
according to the parameter through which it is passed in the UpdateCustomData  or CreateAccount methods.
It is recommended that you save personal details such as mailing addresses and credit card numbers as private data.
\sa Custom Data Classes
*/
class OSDK_API AccountManagementClient: public ServiceClient
{
public:

    /** Default contructor. */
    AccountManagementClient();
    /** Destructor */
    virtual ~AccountManagementClient();

    /**
    \brief Creates a lobby account.

    \param szGuestPassword The password for the /Guest/ BuiltInGroups, which has the right to
    create new accounts. This method logs into the RendezVous back-end as Guest to create the account.
    \param strAddress The address of the Rendez-Vous sandbox where the account will be created.
    \param uiBasePort The port used by the Rendez-Vous sandbox where the account will be created.
    \param strUserName The case insensitive username of the account to be created.
    \param szPassword The case sensitive password for the account to be created.
    \param uiGroups The groups to which the new Principal account will belong.
    \param strEmail The e-mail address of the user of the account to be created.
    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param pqResult A pointer to a qResult which contains the error or success code result for this call.
    The signature that contains this parameter is now obsolete. The result of the call should be obtained
    using CallContext::GetOutcome, which returns the appropriate qResult.
    \param strName The  case insensitive user name of the account to be created.
    \param strPassword The  case sensitive password for the account to be created.

    \returns true if the call was successfully initiated, false if not or an error occurred.
    A call to this method will fail even before the call is issued to the back end if you do
    not first instantiate the RendezVous object prior to making the call. This method
    will fail with the RendezVousErrorCode /UsernameAlreadyExists/ if the supplied user name
    already exists. The non-static version of this method will fail if this
    AccountManagementClient does not have the /CreatePlayer/ Capability.

    \remarks Before players can participate in lobby activities, they must
    first instantiate the backend services object. Then they must create a
    user account by means of the Account Management service. Every account
    created with this service is uniquely identified by its user name /
    password pair. The password may be changed using ChangePassword.

    By default, only Principals with the /CreatePlayer/ Capabilities may
    call the non-static version of this method.

    \note The static version of this method is the preferred way to create accounts.

    \code
    // Create the RendezVous object. Before creating your first account, you must create this singleton.
    RendezVous oS1;

    ProtocolCallcontext oContext;
    // Create a new account
    AccountManagementClient::CreateAccount(&oContext, GUEST_ACCOUNT_PASSWORD, strHost, iBasePort, NEX_T("Player101"),
                                        "my password", BuiltInGroups::Players, NEX_T("qwerty@keyboard.com"));
    oContext.Wait();
    assert(oContext.GetState()==CallContext::CallSuccess);
    // Check the outcome of the call in the call context, in case its state is CallContext::CallFailure.
    assert(oContext.GetOutcome());
    \endcode

    \sa RendezVous::Login
    */
    static qBool CreateAccount(ProtocolCallContext *pCallContext,
                               const qChar8* szGuestPassword, const String & strAddress, qUnsignedInt16 uiBasePort,
                               const String & strUserName, const qChar8* szPassword,
                               qUnsignedInt32 uiGroups, const String & strEmail);

    // IPアドレス/ポートではなくhttpから始まるURIで接続先を指定するAPI
    static qBool CreateAccount(ProtocolCallContext *pCallContext,
                               const qChar8* szGuestPassword, const String & strConnectUri,
                               const String & strUserName, const qChar8* szPassword,
                               qUnsignedInt32 uiGroups, const String & strEmail);

    qBool CreateAccount(ProtocolCallContext *pCallContext, const String& strName, const String& strPassword,
                        qUnsignedInt32 uiGroups, const String& strEmail);


#ifdef OBSOLETE_API_SUPPORT
    qBool CreateAccount(ProtocolCallContext *pCallContext, qResult *pqResult, const String& strName,
                        const String& strPassword, qUnsignedInt32 uiGroups, const String& strEmail);
#endif


    /**
    \param hPrivateData The Private data to be assigned to account. The signature with
    this parameter and hPublicData sets private and public data at the same time.
    \param hPublicData The Public data to be assigned to account. The signature with
    this parameter and hPrivateData sets private and public data at the same time
    */
    static qBool CreateAccount(ProtocolCallContext *pCallContext,
                               const qChar8* szGuestPassword, const String & strAddress, qUnsignedInt16 uiBasePort,
                               const String & strUserName, const qChar8* szPassword,
                               qUnsignedInt32 uiGroups, const String & strEmail,
                               const PublicDataHolder& hPublicData, const PrivateDataHolder& hPrivateData);

    // IPアドレス/ポートではなくhttpから始まるURIで接続先を指定するAPI
    static qBool CreateAccount(ProtocolCallContext *pCallContext,
                               const qChar8* szGuestPassword, const String & strConnectUri,
                               const String & strUserName, const qChar8* szPassword,
                               qUnsignedInt32 uiGroups, const String & strEmail,
                               const PublicDataHolder& hPublicData, const PrivateDataHolder& hPrivateData);

    /** \internal */
    qBool CreateAccount(ProtocolCallContext *pCallContext,
                        const String& strName, const String& strPassword, qUnsignedInt32 uiGroups, const String& strEmail,
                        const DataHolder& hPublicData, const DataHolder& hPrivateData);

    /**
    For regular AccountManagement service, this method is a no-op on backend side: it must be overridden with a
    post-hook on a package that require to match Principal data with service-specific info
    \internal */
    static qBool CustomCreateAccount(ProtocolCallContext *pCallContext,
                                     const qChar8* szGuestPassword, const String & strAddress, qUnsignedInt16 uiBasePort,
                                     const String& strName, const qChar8* szPassword, qUnsignedInt32 uiGroups, const String& strEmail,
                                     const DataHolder& hCustomAuthData, PrincipalID* pPid);

    // IPアドレス/ポートではなくhttpから始まるURIで接続先を指定するAPI
    static qBool CustomCreateAccount(ProtocolCallContext *pCallContext,
                                     const qChar8* szGuestPassword, const String & strConnectUri,
                                     const String& strName, const qChar8* szPassword, qUnsignedInt32 uiGroups, const String& strEmail,
                                     const DataHolder& hCustomAuthData, PrincipalID* pPid);

    /** \internal */
    qBool CustomCreateAccount(ProtocolCallContext *pCallContext,
                              const String& strName, const String& strPassword, qUnsignedInt32 uiGroups, const String& strEmail,
                              const DataHolder& hCustomAuthData, PrincipalID* pPid);

    static qBool NintendoCreateAccount(ProtocolCallContext *pCallContext,
                                       const qChar8* szGuestPassword, const String & strAddress, qUnsignedInt16 uiBasePort,
                                       const String& strName, const qChar8* szPassword, qUnsignedInt32 uiGroups, const String& strEmail,
                                       const DataHolder& hCustomAuthData, PrincipalID* pPid, String* pPidHMAC
                                      );

    // IPアドレス/ポートではなくhttpから始まるURIで接続先を指定するAPI
    static qBool NintendoCreateAccount(ProtocolCallContext *pCallContext,
                                       const qChar8* szGuestPassword, const String & strConnectUri,
                                       const String& strName, const qChar8* szPassword, qUnsignedInt32 uiGroups, const String& strEmail,
                                       const DataHolder& hCustomAuthData, PrincipalID* pPid, String* pPidHMAC
                                      );

    // {secret}
    qBool NintendoCreateAccount(ProtocolCallContext *pCallContext,
                                const String& strName, const String& strPassword, qUnsignedInt32 uiGroups, const String& strEmail,
                                const DataHolder& hCustomAuthData, PrincipalID* pPid, String* pPidHMAC
                               );

    /** If strName is empty, it only looks up the account
    \internal */
    qBool LookupOrCreateAccount(ProtocolCallContext *pCallContext,
                                const String& strName, const String& strPassword, qUnsignedInt32 uiGroups, const String& strEmail,
                                const DataHolder& hCustomAuthData, PrincipalID* pPid);


    /**
    \brief Disables the account of the specified Principal until the specified time.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param pqResult     A pointer to the qResult that contains the error or success code result of this call. The signature that
    contains this parameter is now obsolete. The result of the call should be obtained using CallContext::GetOutcome,
    which returns the appropriate qResult.
    \param idPrincipal  The PrincipalID of the Principal whose account will be disabled.
    \param dtUntil      The date and time (DateTime) until which the account will be disabled.
    \param strMessage   The message that will be returned upon login if the account is currently disabled.
    This parameter is optional.

    \returns true if the call was successfully initiated, false if not or an error occurred. If this method returns
    true then pqResult will point to the error or success code for the call. This method will return false if
    this AccountManagementClient does not have the /DisableAccount/ Capability.

    \remarks Typically, a Principal's account could be disabled for a certain period of time if the Principal has been
    temporarily banned from the lobby or has not paid the necessary subscription fees. This method is effectively the
    same as UpdateAccountExpiryDate as either may be used to disable an account.

    \note By default, only clientss with /DisableAccount/ Capabilities may call this method.
    \sa qResult, DateTime, RendezVous::Login, UpdateAccountExpiryDate
    */
    qBool DisableAccount(ProtocolCallContext *pCallContext, PrincipalID idPrincipal,
                         const DateTime& dtUntil, const String& strMessage);

#ifdef OBSOLETE_API_SUPPORT
    qBool DisableAccount(ProtocolCallContext *pCallContext, qResult *pqResult, PrincipalID idPrincipal,
                         const DateTime& dtUntil, const String& strMessage);
#endif


    /**
    \brief Deletes the account of the specified PrincipalID.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    idPrincipal: The PrincipalID of the Principal whose account will be deleted.

    \returns true if the call was successfully initiated, false if not or an error occurred.

    \note By default, only Principals with the /CreatePlayer/ Capabilities may call this method.
    The result of the call can be obtained using CallContext::GetOutcome, which will return the
    appropriate qResult.

    \note If the Principal to be deleted is still connected to the server, its account won't be removed.
    It is simply marked for deletion. When the Principal does log out, the AccountManagement service
    deletes its entry in the Principals table along with entries in other tables having a reference to it.
    It is possible to instruct the service about the tables to clean up upon account deletion.
    If you have, for example, a custom table with a field referring to Principals table, e.g. 'pid',
    you can add a call to AccountManagement.AddDeleteTrigger in your script; see MyMatchMaking.py
    for a sample.
    You can set a callback as well. This server code would be executed upon the same event.
    See a sample of AccountManagement.AddDeleteCallback in MyAccountManagement.py.

    \note DeleteAccount fails on any attempt to delete a BuiltInPrincipals.
    \sa Capabilities, BuiltInPrincipals
    */
    qBool DeleteAccount(ProtocolCallContext *pCallContext, PrincipalID idPrincipal);


    /**
    \brief Disconnect the user of the specified Principal.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param idPrincipal  The PrincipalID of the Principal whose account will be disabled.

    \returns true if the call was successfully initiated, false if not or an error occurred. If this method returns
    true then pqResult will point to the error or success code for the call. This method will return false if
    this AccountManagementClient does not have the /DisconnectPrincipal/ Capability.
    */
    qBool DisconnectPrincipal(ProtocolCallContext *pCallContext, PrincipalID idPrincipal);


    /**
    \brief Disconnect all principals connected (excepted caller).

    \param pCallContext A pointer to the ProtocolCallContext for this call.

    \returns true if the call was successfully initiated, false if not or an error occurred. If this method returns
    true then pqResult will point to the error or success code for the call. This method will return false if
    this AccountManagementClient does not have the /DisconnectAllPrincipals/ Capability.
    */
    qBool DisconnectAllPrincipals(ProtocolCallContext *pCallContext);


    /**
    \brief Changes the password of this AccountManagementClient.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param pbResult A pointer to a Boolean that returns true if the call succeeded, false if not. The signature that contains
    this parameter is now obsolete. The result of the call can be obtained using CallContext::GetOutcome, which returns the
    appropriate qResult.
    \param strNewPassword The new password for this account.

    \returns true if the call was successfully initiated, false if not or an error occurred. This method
    returns false if this AccountManagementClient does not have the /ChangePassword/ Capability.

    \note By default, only Principals with the /ChangePassword/ Capabilities may call this method.
    \sa Capabilities, UpdateAccount, Account Notification Package
    */
    qBool ChangePassword(ProtocolCallContext *pCallContext, const String& strNewPassword);

#ifdef OBSOLETE_API_SUPPORT
    qBool ChangePassword(ProtocolCallContext *pCallContext, qBool *pbResult, const String& strNewPassword);
#endif

    /** \internal  ChangePasswordByGuest */
    qBool ChangePassword(ProtocolCallContext *pCallContext, const String& strName, const String& strEmail, const String& strNewPassword);

    /** \internal */
    static qBool ChangePassword(ProtocolCallContext *pCallContext, const qChar8* szGuestPassword, const String& strAddress,
                                qUnsignedInt16 uiBasePort, const String& strName, const String& strEmail, const qChar8* szNewPassword);

    /**
    \brief Determines whether or not the caller has the specified Capability.

    \param pCallContext     A pointer to the ProtocolCallContext for this call.
    \param pbResult         A pointer to a Boolean that returns true if the call succeeded, false if not. The signature that
    contains this parameter is now obsolete. The result of the call can be obtained using CallContext::GetOutcome,
    which points to a Boolean that determines whether or not the caller has the specified
    capability uiCapability.
    \param uiCapability     The Capability to be tested.

    \returns true if the call was successfully initiated, false if not or an error occurred.
    \sa Capabilities
    */
    qBool TestCapability(ProtocolCallContext *pCallContext, qUnsignedInt32 uiCapability);


#ifdef OBSOLETE_API_SUPPORT
    qBool TestCapability(ProtocolCallContext *pCallContext, qBool *pbResult, qUnsignedInt32 uiCapability);
#endif

    /**
    \brief Updates the user name of the account associated with this AccountManagementClient.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param pqrResult    A pointer to the qResult that contains the error or success code result of this call. The signature that contains
    this parameter is now obsolete. The result of the call can be obtained using CallContext::GetOutcome, which returns the
    appropriate qResult.
    \param strName      The new case insensitive user name of this AccountManagementClient.

    \returns true if the call was successfully initiated, false if not or an error occurred. This method returns
    false if this AccountManagementClient does not have the /UpdateAccount/ Capability or the supplied user name
    already exists.

    \remarks The user name for a given account is set when the account is created via CreateAccount, after which
    the user name may be changed by means of this method.

    \note By default, only clients with the /UpdateAccount/ Capabilities may call this method.
    \sa UpdateAccountEmail, UpdateAccount
    */
    qBool UpdateAccountName(ProtocolCallContext *pCallContext, const String& strName);

#ifdef OBSOLETE_API_SUPPORT
    qBool UpdateAccountName(ProtocolCallContext *pCallContext, qResult *pqrResult, const String& strName);
#endif


    /**
    \brief Updates the e-mail address of the account associated with this AccountManagementClient.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param pqrResult    A pointer to the qResult that contains the error or success code result of this call. The signature that contains
    this parameter is now obsolete. The result of the call can be obtained via CallContext::GetOutcome, which returns the
    appropriate qResult.
    \param strEmail     The new e-mail address of this AccountManagementClient.

    \returns true if the call was successfully initiated, false if not or an error occurred. This method will return
    false if this AccountManagementClient does not have the /UpdateAccount/ Capability.

    \remarks The e-mail address for a given account is set when the account is created using CreateAccount,
    after which you can change the address using this method.

    \note By default, only clients with the /UpdateAccount/ Capabilities may call this method.
    \sa UpdateAccountName, UpdateCustomData, UpdateAccount
    */
    qBool UpdateAccountEmail(ProtocolCallContext *pCallContext, const String& strEmail);

#ifdef OBSOLETE_API_SUPPORT
    qBool UpdateAccountEmail(ProtocolCallContext *pCallContext, qResult *pqrResult, const String& strEmail);
#endif

    /**
    \brief Updates the public and /or private data of this AccountManagementClient.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param pqrResult    A pointer to the qResult that contains the error or success code result of this call. The signature that contains
    this parameter is now obsolete. The result of the call can be obtained using CallContext::GetOutcome, which returns the
    appropriate qResult.
    \param hPublicData  The Public data to be updated.
    \param hPrivateData The Private data to be updated.

    \returns true if the call was successfully initiated, false if not or an error occurred. This method will return
    false if this AccountManagementClient does not have the /UpdateAccount/ Capability.

    \remarks The data passed in the hPublicData and hPrivateData variables may be retrieved using the
    GetPublicData and GetPrivateData methods respectively.
    You may give any significance you wish to the Public data and Private data structures. The
    only difference as far as \productname is concerned is that a given AccountManagementClient can
    access both is Public and Private data, whereas it can only access other clients' Public data.
    Private and public data are typically updated in a manner similar to the following:

    \code
        qBool bCallResult;
        ProtocolCallContext oC;
        // Register custom data classes for the private and public data
        AnyDataAdapter::GetInstance()->RegisterPrototype(qNew MyPrivateData);
        AnyDataAdapter::GetInstance()->RegisterPrototype(qNew MyPublicData);

        // Specify the data that will be updated
        MyPrivateData* pPrivate=qNew MyPrivateData;
        pPrivate->m_strSpecifics = NEX_T("Some highly sensitive data...");
        PrivateDataHolder hPrivate = pPrivate;

        MyPublicData* pPublic = qNew MyPublicData;
        pPublic->m_strNickname = NEX_T("Foo");
        PublicDataHolder hPublic = pPublic;

        // Update the account
        pC1->GetAccountManagementClient()->UpdateCustomData(&oC, hPublic, hPrivate);
        // Wait for the result
        oC.Wait();
        oC.GetState()==CallContext::CallSuccess;
    \endcode

    \note By default, only clients with the /UpdateAccount/ Capabilities may call this method.
    \sa GetPrivateData, GetPublicData, UpdateAccountName, UpdateAccountEmail, UpdateAccount
    */
    qBool UpdateCustomData(ProtocolCallContext *pCallContext, const PublicDataHolder& hPublicData, const PrivateDataHolder& hPrivateData);

#ifdef OBSOLETE_API_SUPPORT
    qBool UpdateCustomData(ProtocolCallContext *pCallContext, qResult *pqrResult, const DataHolder& hPublicData, const DataHolder& hPrivateData);
#endif

    /**
    \brief Updates the password, e-mail address and public and/or private data of the account associated with this AccountManagementClient
    This combines other Update calls such as UpdateAccountEmail and UpdateCustomData

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param strPassword  The new case sensitive password of this AccountManagementClient.
    \param strEmail     The new e-mail address of this AccountManagementClient.
    \param hPublicData  The Public data to be updated.
    \param hPrivateData  The Private data to be updated.

    \returns True if the call was successfully initiated, false if not or an error occurred. This method returns
    false if this AccountManagementClient does not have the /UpdateAccount/ Capability.

    \sa GetPrivateData, GetPublicData, Change Password, UpdateAccountEmail, UpdateCustomData
    */
    qBool UpdateAccount(ProtocolCallContext *pCallContext, const String& strPassword, const String& strEmail,
                        const PublicDataHolder& hPublicData, const PrivateDataHolder& hPrivateData);

    /**
    \brief Returns the user name of the specified Principal.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param idPrincipal  The PrincipalID of the Principal whose user name we wish to know.
    \param pstrName     A pointer to the user name, as a string, of the Principal specified by idPrincipal.

    \returns true if the call was successfully initiated, false if not or an error occurred.
    If this method returns true then pstrName will point to the user name of the Principal
    with the specified PrincipalID.

    \sa AuthenticationClient::GetPID
    */
    qBool GetName(ProtocolCallContext *pCallContext, PrincipalID idPrincipal, String* pstrName);

    /**
    \brief Returns the AccountData of this AccountManagementClient.

    \param pCallContext     A pointer to the ProtocolCallContext for this call.
    \param pqrResult        A pointer to the qResult that contains the error or success code result of this call. The signature that contains
    this parameter is now obsolete. The result of the call can be obtained using CallContext::GetOutcome, which returns the
    appropriate qResult.
    \param pAccountData     If this method returns true, then pAccountData points to the Principal's AccountData.

    \returns true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pAccountData points to this client's AccountData.

    \remarks The AccountData class is defined in the Account Management DDL and includes the PrincipalID,
    user name, e-mail address, creation date, effective date, and expiration date of the account and the
    groups to which the client belongs.

    \sa Account Management DDL, RetrieveAccount
    */
    qBool GetAccountData(ProtocolCallContext *pCallContext, AccountData* pAccountData);

#ifdef OBSOLETE_API_SUPPORT
    qBool GetAccountData(ProtocolCallContext *pCallContext, qResult *pqrResult, AccountData* pAccountData);
#endif


    /**
    \brief Returns the Public data of one or many specified Principals.

    \param pCallContext     A pointer to the ProtocolCallContext for this call.
    \param pbResult         A pointer to a Boolean that returns true if the call succeeded, false if not. The signatures that
    contain this parameter are now obsolete. The result of the call can be obtained using
    CallContext::GetOutcome, which returns the appropriate qResult.

    \param idPrincipal      The PrincipalID of the Principal for whom the Public data will be returned.
    \param pPublicData      If this method returns  true, points to the Public data of the Principal specified by idPrincipal.
    \param lstPrincipals    A list of  of the Principal IDs for which the Public data will be returned.
    \param plstPublicData   If this method returns true, points to a list of the Public data of the Principals specified by lstPrincipals.

    \returns true if the call was successfully initiated, false if not or an error occurred. If this method returns
    true, then pPublicData or plstPublicData points to the Public data of the specified PrincipalID or PrincipalIDs
    respectively.

    \remarks Public data is defined as the data passed in the /hPublicData/ parameter of the UpdateCustomData method.
    You may give any significance you wish to the Public data and Private data structures. The
    only difference as far as \productname is concerned is that a given AccountManagementClient can
    access both its public and private data, whereas it can only access the public data of others.

    \sa GetPrivateData, UpdateCustomData, RetrieveAccount
    */
    qBool GetPublicData(ProtocolCallContext *pCallContext, PrincipalID idPrincipal, PublicDataHolder* pPublicData);
    qBool GetPublicData(ProtocolCallContext *pCallContext, qList<PrincipalID> lstPrincipals, qList<DataHolder>* plstPublicData);

#ifdef OBSOLETE_API_SUPPORT
    qBool GetPublicData(ProtocolCallContext *pCallContext, qBool *pbResult, PrincipalID idPrincipal, DataHolder* pPublicData);
    qBool GetPublicData(ProtocolCallContext *pCallContext, qBool *pbResult, qList<PrincipalID> lstPrincipals, qList<DataHolder>* plstPublicData);
#endif

    /**
    \brief Returns the Private data of this AccountManagementClient.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param pbResult     A pointer to a Boolean that returns true if the call succeeded, false if not. The signature that contains
    this parameter is now obsolete. The result of the call can be obtained using CallContext::GetOutcome, which returns
    the appropriate qResult.
    \param pPrivateData If this method returns true, points to the Private data of this AccountManagementClient.

    \returns true if the call was successfully initiated, false if not or an error occurred. If this method
    returns true then pPrivateData points to the Private data of this AccountManagementClient.

    \remarks Private data is defined as the data passed in the /hPrivateData/ parameter of the UpdateCustomData method.
    You may give any significance you wish to the Public data and Private data structures. The
    only difference as far as \productname is concerned is that a given AccountManagementClient can access its own public
    and private data, whereas it can only access the public data of others.

    \sa GetPublicData, UpdateCustomData, RetrieveAccount
    */
#ifdef OBSOLETE_API_SUPPORT
    qBool GetPrivateData(ProtocolCallContext *pCallContext, PrivateDataHolder* pPrivateData);

    qBool GetPrivateData(ProtocolCallContext *pCallContext, qBool *pbResult, DataHolder* pPrivateData);
#endif

    /**
    \brief Returns the Public, Private and Account Data of this AccountManagementClient.
    This combines other "get" calls for account data such as GetPrivateData, GetPublicData and GetAccountData

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param pAccountData If this method returns true, points to the AccountData of this AccountManagementClient.
    \param pPrivateData If this method returns true, points to the Private data of this AccountManagementClient.
    \param pPublicData  If this method returns true, points to the Public data of the Principal specified by idPrincipal.

    \returns True if the call was successfully initiated, false if not or an error occurred.
    If this method returns true, then pAccountData, pPrivateData and pPublicData point to the client's data.

    \remarks The AccountData class is defined in the Account Management DDL and includes the PrincipalID,
    user name, e-mail address, creation date, effective date, and expiration date of the account and the
    groups to which the Principal belongs.

    \note RetrieveAccount will fail if the specified account has missing private or public data.
    \sa Account Management DDL, GetPublicData, GetPrivateData
    */
    qBool RetrieveAccount(ProtocolCallContext *pCallContext, AccountData* pAccountData, DataHolder* pPublicData, DataHolder* pPrivateData);

    /**
    \brief Searches for accounts according to a regular expression that matches their user name.
    This method has been deprecated and replaced by the FindByNameLike method

    \param pCallContext     A pointer to the ProtocolCallContext for this call.
    \param uiGroups         The BuiltInGroups to which the account owner belongs. This will usually be set to /Players/.
    \param strRegex         The user name to find as a regular expression. This user name is case insensitive
    \param resultRange      The ResultRange of the returned list.
    \param plstAccountInfo  If this method returns true, points to a list of account data. The BasicAccountInfo class is defined
    in the Account Management DDL and contains the PrincipalID and user name of the account.

    \returns true if the call was successfully initiated, false if not or an error occurred. If this method returns true then
    plstAccountInfo will point to a list of accounts that match the provided group and user name.

    \remarks This method performs a search for accounts where the account owner belongs to
    the specified uiGroups and whose user name also matches in some way the regular expression(s) of the given strRegex.

    The syntax that may be used for the regular expression passed by the strRegex parameter is that
    defined by MySQL. Full details of the valid special characters and constructs, as well as examples
    of how to use the syntax are given in the MySQL manual available at
    {hyperlink:http://dev.mysql.com/doc/mysql/en/Regexp.html|http://dev.mysql.com/doc/mysql/en/Regexp.html}.
    Even so, the valid special characters (separated by commas) are as follows: ^, $, ., *, +, ?, |, and \\.
    In addition, -, =, :, <, and > can be used within one or more of the brackets (), [], and {}.

    \sa FindByNameLike, Account Management DDL
    */
    qBool FindByNameRegex(ProtocolCallContext *pCallContext, qUnsignedInt32 uiGroups, const String& strRegex, const ResultRange& resultRange, qList<BasicAccountInfo>* plstAccountInfo);

    /**
    \brief Searches for accounts according to a string that can occur anywhere in
    their description.

    \param pCallContext     A pointer to the ProtocolCallContext for this call.
    \param uiGroups         The BuiltInGroups to which the account owner belongs. This will usually be set to /Players/.
    \param strLike          The user name or portion of a user name (a regular expression) to search for. User names are case insensitive.
    \param resultRange      The ResultRange of the returned list.
    \param plstAccountInfo  If this method returns true, points to a list of accounts with names that match the string. The BasicAccountInfo class is
    defined in the Account Management DDL and contains the PrincipalID and user name of the account.

    \returns True if the call was successfully initiated, False if not or an error occurred. If this method returns true then
    plstAccountInfo points to a list of accounts that match the specified group name and with user names containing the specified string.

    \remarks This method performs a case insensitive search for accounts where the account owner belongs to
    the specified uiGroups and whose user name contains the given string.
    \sa Account Management DDL
    */
    qBool FindByNameLike(ProtocolCallContext *pCallContext, qUnsignedInt32 uiGroups, const String& strLike, const ResultRange& resultRange, qList<BasicAccountInfo>* plstAccountInfo);


    /**
    \brief Sets the date and time that the specified Principal's account will expire.

    \param pCallContext         A pointer to the ProtocolCallContext for this call.
    \param idPrincipal          The PrincipalID of the Principal whose account expiry date will be updated.
    \param dtExpiry             The date and time (DateTime) at which the account will expire.
    \param strExpiredMessage    The message that will be returned upon login if the account is currently expired.
    This parameter is optional.

    \returns true if the expiry date of the specified Principal's account was successfully updated, false if not
    or an error occurred. This method will return false if this AccountManagementClient does not have the
    /DisableAccount/ Capability.
    \remarks This method is effectively the same as DisableAccount as either may be used to disable an account.

    \note By default, only Principals with the /DisableAccount/ Capability may call this method.
    \sa UpdateAccountEffectiveDate, DateTime, RendezVous::Login, DisableAccount
    */
    qBool UpdateAccountExpiryDate(ProtocolCallContext *pCallContext, PrincipalID idPrincipal,
                                  const DateTime& dtExpiry, const String& strExpiredMessage);

    /**
    \brief Sets the date and time from which the specified Principal's account will be effective.

    \param pCallContext     A pointer to the ProtocolCallContext for this call.
    \param idPrincipal      The PrincipalID of the Principal whose account effective date will be updated.
    \param dtEffectiveFrom  The date and time (DateTime) from which the account will be effective, after which the Principal
    may log in to the lobby.
    \param strNotEffectiveMessage  The message that will be returned upon login if the account is currently not effective.
    This parameter is optional.

    \returns true if the effective date of the specified Principal's account was successfully updated, false if not
    or an error occurred. This method will return false if this AccountManagementClient does not have the
    /DisableAccount/ Capability.

    \remarks By default, only Principals with the /DisableAccount/ Capability may call this method.
    \sa UpdateAccountExpiryDate, DateTime, RendezVous::Login
    */
    qBool UpdateAccountEffectiveDate(ProtocolCallContext *pCallContext, PrincipalID idPrincipal,
                                     const DateTime& dtEffectiveFrom, const String& strNotEffectiveMessage);

    /**
    \brief If the PrincipalStatus Package is installed, updates the Player status of this AccountManagementClient. Does nothing otherwise.

    \param pCallContext  A pointer to the ProtocolCallContext for this call.
    \param strNewStatus  The (user defined) Player status to which this AccountManagementClient will be updated.

    \returns true if the call was successfully initiated, false if not or an error occurred.

    \remarks A Principal's status is /Offline/ until they connect to the Account Management Service when,
    if the PrincipalStatus Package is installed, it is automatically updated to /Online/. When they disconnect,
    it becomes Offline again. If a player, however, has multiple login sessions, their status remains
    Online until the last logout - only after which will it change to Offline.
    Online or offline are the only status defined by \productname, all other status, such as "playing a certain game", must
    be defined and implemented by you. This method allows you to do this. For example, when a player joins a game session,
    you could update the player's status as follows:

    \code
    {
        ProtocolCallContext oContext;
        pC2->GetFriendsClient()->GetCredentials()->GetPrincipalID()==1002;
        pC2->GetAccountManagementClient()->UpdateStatus(&oContext, NEX_T("Playing"));
        oContext.Wait();
    }
    \endcode

    \note This method can only update the PrincipalStatus if the PrincipalStatus Package is installed.
    \sa GetStatus
    */
    qBool UpdateStatus(ProtocolCallContext *pCallContext, const String& strNewStatus);

    /**
    \brief If the PrincipalStatus Package is installed, returns the Player status of the specified client.
    Does nothing otherwise.

    \param pCallContext A pointer to the ProtocolCallContext for this call.
    \param idPrincipal  The PrincipalID of the Principal for which the Player status will be returned.
    \param pstrStatus   If the call succeeds, points to the Player status of the Principal specified by idPrincipal.

    \returns true if the call was successfully initiated, false if not or an error occurred. If the
    call succeeds, then pstrStatus points to the current Player status of the specified Principal.
    Note that if a player has multiple login sessions, their status will be Online until the last logout - only
    after which will it change to Offline.

    \note This method can only return the PrincipalStatus if the PrincipalStatus Package is installed.
    \sa UpdateStatus
    */
    qBool GetStatus(ProtocolCallContext *pCallContext, PrincipalID idPrincipal, String* pstrStatus);

    /**
    \brief Returns the login and logout date and times of the Principal's last session and the current session login
    date and time.

    \param  pCallContext            A pointer to the ProtocolCallContext for this call.
    \param  idPrincipal             The PrincipalID of the client for whom the log in and out data will be returned.
    \param  pdtLastSessionLogin If this method returns true, points to a DateTime data type object of the Principal's previous login.
    If the ConnectionInfo Package is not installed, points to a NULL DateTime object.
    \param  pdtLastSessionLogout    If this method returns true, points to a DateTime data type object of the Principal's previous logout.
    If the ConnectionInfo Package is not installed, points to a NULL DateTime object.
    \param  pdtCurrentSessionLogin  If this method returns true, points to a DateTime data type object for the Principal's current log in.
    If the Principal is not currently logged in to the lobby, then this is a NULL pointer. If the ConnectionInfo
    Package is not installed, the DateTime object contains a NULL value.

    \returns True if the call was successfully initiated or an error occurred. If this method returns true and the ConnectionInfo
    Package is installed, then pdtLastSessionLogin, pdtLastSessionLogout, and pdtCurrentSessionLogin point to the date and time
    (DateTime data type objects) of the Principal's previous log in, previous log out, and current log in respectively.
    (Note that a player can have multiple login sessions.) If the ConnectionInfo Package is not installed, the DateTime objects
    contain NULL values.

    \remarks Typically, this method is used to return a player's previous log-in details upon log in.
    \sa DateTime, ConnectionInfo Package.
    */
    qBool GetLastConnectionStats(ProtocolCallContext *pCallContext, PrincipalID idPrincipal, DateTime *pdtLastSessionLogin,
                                 DateTime *pdtLastSessionLogout, DateTime *pdtCurrentSessionLogin);


    /**
     \note The Backend part of ResetPassword in AccountManagement.py does nothing. It is there to be
     hooked by the AccountNotification package which implements its own specific functionality
     (resets the password of the current loggedin pid with a random pwd and sends an email
     notification of what it has done). For this reason, this method has been removed from the
     documentation (for now). See Bug 4098.
     \internal
    */
    qBool ResetPassword(ProtocolCallContext *pCallContext);

protected:
    /** \internal */
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

private:
    OSDK_DISABLE_EXPORT_WARNING(AccountManagementProtocolClient m_oProtocol);
};

END_NAMESPACE_NEX

