﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Transport/Interface/NetFaults.h>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/RefCountedObject.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/ConnectionID.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Common/Protocol.h>

BEGIN_NAMESPACE_NEX

class StreamManager;
class EndPoint;
template class OSDK_API MemAllocator<ProtocolID >;
template class OSDK_API qList<ProtocolID>;

/*
\ingroup group_rdv_core_client
\brief This class assigns a unique identifier to each Principal.
\remarks The Credentials of a given Principal are unique to that Principal and encompass a
variety of information used internally for verification and security purposes.
*/

/*!
 @brief    クライアントの証明書情報を格納するクラスです。

 ログイン完了後、@ref NgsFacade::GetCredentials() で有効な Credentials オブジェクトが取得できます。
 取得した Credentials オブジェクトを直接操作しないでください。
 */
class OSDK_API Credentials: public RefCountedObject
{
public:
    /*
        \brief Constructor
        \param[in] id The PrincipalID of the Principal to which these Credentials belong.
        \param[in] pTransport A pointer to the service transport that the specified Principal uses.
        \note
        A Credentials object should never be directly instantiated. Instead, it is created
        by the lobby when a Principal logs in to the system.
    */
    Credentials(PrincipalID id, StreamManager *pTransport);

    Credentials();

    /*
        \note
        A Credentials object should never be directly destroyed. It is destroyed by the lobby
        when its associated Principal logs out of the system. */
    virtual ~Credentials();

    /*
    \brief Returns the PrincipalID of the Principal associated with this Credentials object.

    A PrincipalID is a unique positive integer used to identify each Principal in the lobby.
    PrincipalID s ranging from 1 to 99 are reserved for Quazal processes and cannot be
    assigned by the developer. IDs above and including 100 can be assigned by the developer however,
    IDs from 100 to 999 are reserved for server processes.

    \returns The ID of the Principal associated with this Credentials object.
    */

    /*!
     @brief    PrincipalID を取得します。

     @return   Credentials にセットされた PrincipalID。自分の PrincipalID になります。
     */
    PrincipalID GetPrincipalID() const
    {
        return m_idPrincipal;
    }

    /** \internal */
    void SetPrincipalID(PrincipalID pid)
    {
        m_idPrincipal = pid;
    }

    /** \internal */
    String GetPassword() const
    {
        return m_strPassword;
    }

    /** \internal */
    void SetPassword(String password)
    {
        m_strPassword = password;
    }

    /*
        \brief Returns the ConnectionID associated with this Credentials object.

        A ConnectionID is a unique positive integer used to identify each secure connection.

        \returns The ID of the connection associated with this Credentials object.
    */
    ConnectionID GetConnectionID() const
    {
        return m_idConnection;
    }

    /** \internal */
    void SetConnectionID(ConnectionID id)
    {
        m_idConnection = id;
    }

    /** \internal */
    ConnectionID* GetPointerToConnectionID()
    {
        return &m_idConnection;
    }

    /** \internal */
    // Auth、Secure共にStreamManagerを無効化する。
    void MarkAllInvalid(StreamManager** ppStreamManagerForAuthentication, StreamManager** ppStreamManagerForSecure);
    // AuthのStreamManagerを無効化し、代わりに今後はSecure用のStreamManagerをAuth用として使用するようにする。
    void MarkAuthInvalidAndUseSecureAsAuth(StreamManager** ppStreamManagerForAuthentication);

    // Auth用のStreamManagerを取得する
    StreamManager* GetStreamManagerForAuthentication() const
    {
        return m_pStreamManagerForAuthentication;
    }

    // Secure用のStreamManagerを取得する
    StreamManager* GetStreamManagerForSecure() const
    {
        return m_pStreamManagerForSecure;
    }

    /** \internal */
    // Secure用のStreamManagerを取得する。GetStreamManagerForSecure()と同じ。互換性維持用。
    StreamManager* GetStreamManager() const
    {
        return m_pStreamManagerForSecure;
    }

    /** \internal */
    void SetStreamManagerForAuthentication(StreamManager* pStreamManager);
    void SetStreamManagerForSecure(StreamManager* pStreamManager);

    /** \internal */
    EndPoint* GetConnection(ProtocolID uiProtocolID) const;

    /** \internal */
    EndPoint* GetAuthenticationConnection() const
    {
        return m_pAuthenticationConnection;
    }

    /** \internal */
    EndPoint* GetSecureConnection() const
    {
        return m_pRegularSecureConnection;
    }

    /** \internal */
    void SetAuthenticationConnection(EndPoint* pConnection);

    /** \internal */
    void SetSecureConnection(EndPoint* pConnection);

    /** \internal */
    void Logout();

    /*
        \brief Gets the locally logged-on player's name.

        This function sets the locally logged on player's name. Of course, you can only call this
        function after a successful login (while the Credentials object is valid).
    */
    const qChar* GetUserName() const
    {
        return m_strUserName.CStr();
    }

    /** \internal */
    void SetUserName(const String &sUserName)
    {
        m_strUserName = sUserName;
    }
    
    void SetNetFaultReason(NetFaultReason reason)
    {
    	m_netFaultReason = reason;
    }
    
    NetFaultReason GetNetFaultReason() const
    {
    	return m_netFaultReason;
    }

private:
    Credentials(const Credentials&);
    Credentials& operator=(const Credentials&);

private:
    PrincipalID m_idPrincipal;
    ConnectionID m_idConnection;
    String m_strUserName;
    String m_strPassword;

    // TODO - to clean all this up, we could probably simply keep
    // both EndPoints in the credentials, rather than within the
    // transport, that has to be accessed via a function callback.

    StreamManager* m_pStreamManagerForAuthentication;
    StreamManager* m_pStreamManagerForSecure;

    EndPoint* m_pAuthenticationConnection;
    EndPoint* m_pRegularSecureConnection;
    
    NetFaultReason m_netFaultReason;
};

END_NAMESPACE_NEX



