﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/String.h>

BEGIN_NAMESPACE_NEX

//==============================================================================
// Sandbox Connection Info (contains config for logging into the backend lobby)
//==============================================================================

/**
\ingroup group_rdv_core_client
*/
class OSDK_API SandboxConnectionInfo : public RootObject
{
public:
    //==========================================================================
    // Hosting Sandbox Credentials
    //==========================================================================

    /** \brief The HostCredentials data object. */
    struct OSDK_API HostCredentials
    {
        /**
        \brief Creates a HostCredentials structure containing
        connection information such as the host address, port, access key and
        Guest account password (for first time logins) supplied by Quazal.

        \param strAddress   The address of the sandbox to access.
        \param ui16Port     The port used by the specified sandbox.
        \param strAccessKey The access key of the sandbox.
        \param strGuestAccountPassword The guest account password.
        */
        HostCredentials();
        HostCredentials(
            const String& strAddress, qUnsignedInt16 ui16Port,
            const String& strAccessKey, const String& strGuestAccountPassword);

        /** \brief Returns true if the credentials are assigned. */
        qBool IsValid() const;

        /** \internal */
        void Trace(qUnsignedInt64 uiTraceFlags) const;

        /** \brief The address of the sandbox. */
        String m_strAddress;
        /** \brief The port used by the sandbox. */
        qUnsignedInt16 m_ui16Port;
        /** \brief The access key of the sandbox. */
        String m_strAccessKey;
        /** \brief The password for the /Guest/ account which has the rights to create new accounts. */
        String m_strGuestAccountPassword;
    };

public:

    /**
    \brief Constructor.
    */
    SandboxConnectionInfo();

    /**
    \brief Constructor.
    \param strAddress   The address of the sandbox that the client will attempt to connect to.
    \param ui16Port     The port used by the specified sandbox.
    \param strAccessKey Your access key for the specified sandbox.
    \param strGuestAccountPassword The password for the /Guest/ account which has the rights to create new accounts.
    \internal */
    SandboxConnectionInfo(
        const String& strAddress, qUnsignedInt16 ui16Port,
        const String& strAccessKey, const String& strGuestAccountPassword);

    /**
    \brief Constructor.
    \param hc   A HostCredentials data object containing the sandbox login information.
    */
    SandboxConnectionInfo(
        const HostCredentials& hc);

    /**
    \brief Constructor.
    \param hcDev    The development sandbox connection information.
    \param hcCert   The certification sandbox connection information.
    \param hcProd   The production sandbox connection information.
    */
    SandboxConnectionInfo(
        const HostCredentials& hcDev,
        const HostCredentials& hcCert,
        const HostCredentials& hcProd);

    virtual ~SandboxConnectionInfo();

    /**
    \brief Returns the current \productname development login connection information for the game console.
    */
    const HostCredentials& GetCredentialsDev() const;

    /**
    \brief Specifies the data object containing the \productname
    development login connection information for the game console
    (this method must be called prior to instantiating the \facadename object).

    \param hc   The connection information pointing to the development sandbox.

    \remarks The \facadename object uses this information to log in to the correct
    \productname development server.
    */
    void SetCredentialsDev(const HostCredentials& hc);

    /**
    \brief Returns the current \productname certification login connection information for the game console.
    */
    const HostCredentials& GetCredentialsCert() const;

    /**
    \brief Specifies the data object containing the \productname
    certification login connection information for the game console
    (to use prior to instantiating the \facadename object).

    \param hc   The connection information pointing to the certification sandbox.
    */
    void SetCredentialsCert(const HostCredentials& hc);

    /**
    \brief Returns the current \productname production login connection information for the game console.
    */
    const HostCredentials& GetCredentialsProd() const;

    /**
    \brief Specifies the data object containing the \productname
    production login connection information for the game console
    (this method must be called prior to instantiating the \facadename object).

    \param hc   The connection information pointing to the production (live) sandbox.

    \remarks The Login method uses this information to log in to the
    correct \productname server for the environment.


    */
    void SetCredentialsProd(const HostCredentials& hc);

    /**
    \param hcCert The connection information pointing to the certification
    sandbox.

    \param hcProd The connection information pointing to the production (live)
    sandbox.
    */
    void SetCredentialsProd(
        const HostCredentials& hcCert, const HostCredentials& hcProd);

#if defined(DEBUG)
    /**
    \brief Forces the issuer ID to a specific value
    (bypassing the real value). This is only intended for testing purposes.
    Setting it to zero disables this feature (default)
    \internal */
    static void ForceIssuerID(qUnsignedInt32 uiIssuerID);
#endif
    /** \internal */
    static qUnsignedInt32 GetIssuerID();

    // Prints out the host credentials
    void Trace(qUnsignedInt64 uiTraceFlags) const;

private:
    static void CheckSettings(
        const HostCredentials& hc, const String& strContext);

private:
    /** \internal */
    static qUnsignedInt32 s_uiIssuerID; // For testing purpose

    /** \internal */
    HostCredentials m_hcDev;    // (Development)

    /** \internal */
    HostCredentials m_hcCert;   // (Certification)

    /** \internal */
    HostCredentials m_hcProd;   // (Production) (DEFAULT)
};

END_NAMESPACE_NEX

