﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Core/src/Common/SecureConnectionProtocolDDL.h>
#include <RendezVous/Core/src/Common/ConnectionDataDDL.h>
#include <RendezVous/Core/src/Common/DataHolder.h>

BEGIN_NAMESPACE_NEX

class StreamManager;

/**
    \ingroup group_rdv_core_services_secureconnection_client
    \brief This service establishes a secure connection between two stations, either directly or routed
    via a secure station,

    Access to this service is achieved through the back-end services GetSecureConnectionClient method,
    which is only valid once a Principal has logged in to the lobby via the back-end services Login method.
    A client should only access this service directly in order to retrieve valid StationURL s that may be
    used to connect to another client.
*/
class OSDK_API SecureConnectionClient: public ServiceClient
{
public:
    /** Constructor */
    SecureConnectionClient();
    /** Destructor */
    virtual ~SecureConnectionClient();

    /** \internal */
    qBool RegisterURLs(ProtocolCallContext *pContext, qResult* prResult, StationURL* purlPublic);

    /** \internal */
    qBool RegisterURLsEx(ProtocolCallContext *pContext, qResult* prResult, StationURL* purlPublic,
                         const DataHolder& hCustomMessage);

    /** \internal */
    qBool RequestConnectionData(ProtocolCallContext *pContext,
                                ConnectionID cidTarget, PrincipalID pidTarget,
                                qList<ConnectionData>* plstConnectionsData);

    /**
        \brief Returns a list of valid URL s that this client may use to connect to the specified Principal.


        When a Principal logs in to the lobby, the Secure Connection Service retrieves a list of
        valid URLs for that station. If the station is located behind a NAT device, both the public and
        private URLs are retrieved.

        A list of valid StationURL s that one player may use to connect to a game hosted by a remote player
        is typically retrieved as follows:

        \include SimpleMatch.cpp

        \param[in] pContext A pointer to the ProtocolCallContext for this call.
        \param[in] cidTarget The ConnectionID of the pidTarget Principal.
        \param[in] pidTarget The PrincipalID of the Principal for which we wish to obtain valid StationURL s.
        \param[in, out] plstURLs A pointer to a list of valid StationURL s for the specified pidTarget Principal.
        \returns true if the call was successfully initiated; The plstURLs will point to a list of valid StationURL s via which this SecureConnectionClient
        \returns false if an error occurred
        \sa StationURL
    */
    qBool RequestURLs(ProtocolCallContext *pContext,
                      ConnectionID cidTarget, PrincipalID pidTarget,
                      qList<StationURL>* plstURLs);

    /**
        \brief Returns a list of valid URL s that this client may use to connect to the specified Principal.

        When a Principal logs in to the lobby, the Secure Connection Service retrieves a list of
        valid URLs for that station. If the station is located behind a NAT device, both the public and
        private URLs are retrieved.

        A list of valid StationURL s that one player may use to connect to a game hosted by a remote player
        is typically retrieved as follows:
        \include SimpleMatch.cpp

        \param[in] pContext A pointer to the ProtocolCallContext for this call.
        \param[in] pidTarget The PrincipalID of the Principal for which we wish to obtain valid StationURL s.
        \param[in, out] plstURLs A pointer to a list of valid StationURL s for the specified pidTarget Principal.
        \returns true if the call was successfully initiated; The plstURLs will point to a list of valid StationURL s via which this SecureConnectionClient
        \returns false if an error occurred
        may connect to the pidTarget Principal.
        \sa StationURL
    */
    qBool RequestURLs(ProtocolCallContext *pContext, PrincipalID pidTarget,
                      qList<StationURL>* plstURLs);

    qBool TestConnectivity(ProtocolCallContext *pContext);

    qBool UpdateURLs(ProtocolCallContext *pContext);


    /*
     * @internal
     * アドレスとポートが一致したローカルステーションのURL情報をサーバーも含めて更新します。
     *
     * Network::ReplaceLocalURL()で、targetUrlと同じアドレスとポートのローカルステーションの URL 更新が成功したら、
     * サーバーに登録されている　GatheringURLs() で取得可能なホストの URL を更新します。
     * ポート番号がないURLの更新はできません。更新時には、サーバ側で、RVCID と PID を付加します。
     * 自身の RVCID と一致するものか、RVCID が付加されていない URL もののみ更新できます。
     : 書き換え後の URL は書き換え前の URL とアドレス、RVCID、トランスポートプロトコルが一致する必要があります。
     *
     * \param[in, out] pContext この呼び出しのProtocolCallContextのポインタ
     * \param[in] targetUrl 書き換え対象のURL
     * \param[in] url 書き換え後のURL
     * \retval true 呼び出し成功(書き換え成功ではない)
     * \retval false 呼び出し失敗、もしくは、
                     ローカルステーションのURLの書き換え失敗。(同じアドレスのURLが存在しなかった)
                     targetUrl が自身の RVCID と一致しない。
                     targetUrl と url のアドレス、RVCID、トランスポートプロトコルが一致しない。
     */
    qBool ReplaceURL(ProtocolCallContext *pContext, const StationURL& targetUrl, const StationURL& url);

    /*
     * @internal
     * バイナリレポートをサーバに送信します。
     *
     * 送信の成否はチェックしません。
     *
     * \param[in] reportId 送信するレポートの種類
     * \param[in] reportData 送信レポートのバイナリデータのアドレス
     * \param[in] reportDataSize 送信レポートのバイナリデータサイズ
     */
    void SendReport(qUnsignedInt32 reportId, const void* reportData, qUnsignedInt32 reportDataSize);


protected:
    /** \internal */
    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

private:
    void PrepareURLs(qList<StationURL>& lstURLs);

private:
    OSDK_DISABLE_EXPORT_WARNING(SecureConnectionProtocolClient m_oProtocol);
};

END_NAMESPACE_NEX

