﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/String.h>
#include <OnlineCore/src/Platform/Core/PrincipalID.h>
#include <RendezVous/Core/src/Common/NotificationEventDDL.h>

BEGIN_NAMESPACE_NEX

class OSDK_API DDLCLASS(NotificationEvent);

/* \ingroup group_rdv_core_common
    \brief Use this class to construct and retrieve information about NotificationEvents.

    This class is used to construct and retrieve information about NotificationEvents, which are
    messages sent from a back-end service to another Process whenever an event occurs. NotificationEvents
    are sent via the SendNotificationEvent method, as described in the Script Module, and processed via the
    NotificationEventHandler::ProcessNotificationEvent method. Several notification events are pre-defined
    in Rendez-Vous as given in NotificationEvents however, you may also define your own custom notification
    events.

    The source ID, type, subtype (if any), and parameters of a NotificationEvent are defined when
    the NotificationEvent is constructed.

    \sa NotificationEventHandler, NotificationEvents
*/

/*!
 @brief    通知イベントに関するデータを格納するクラスです。

 通知イベントのタイプやイベントの発生元などのデータを格納するクラスです。
 通知イベントを受信するためには、@ref NgsFacade::RegisterNotificationEventHandler() で登録した
 @ref NotificationEventHandler::ProcessNotificationEvent() コールバックの引数として与えられます。

 @see      NotificationEventHandler, NotificationEvents
 */
class OSDK_API NotificationEvent: public DDLCLASS(NotificationEvent)
{
public:
    /**
    \brief Default Constructor
    */
    NotificationEvent()
    {
        m_pidSource = INVALID_PRINCIPALID;
        m_uiType = 0;
        m_uiParam1 = 0;
        m_uiParam2 = 0;
        m_uiParam3 = 0;
    }

    /*
        \brief Constructor

        The NotificationEvent class is used to create notification messages that will be sent
        from a back-end service to another Process when a given event occurs. The conditions under which
        a Notification event will be sent are defined via the BaseClass.SendNotificationEvent method
        of the Script module. Several NotificationEvents are defined in Rendez-Vous however, you may
        also define your own custom notification events.

        A NotificationEvent may specify a subtype event, which is defined via the uiMinorType parameter when
        an event is constructed, only if the given NotificationEvents has subtype events. The pre-defined subtype
        events are listed in the table below.

        \verbatim
        uiMinorType             uiMajorType
        ParticipationEvents     NotificationEvents::ParticipationEvent
        \endverbatim

        \param pidSource The PrincipalID of the back-end service that generates this NotificationEvent.
        \param uiType The aggregate type of this NotificationEvent. The default value is zero.
        \param uiParam1 The first parameter of this NotificationEvent. The default value is zero.
        \param uiParam2 The second parameter of this NotificationEvent. The default value is zero.
        \param strParam The string parameter of this NotificationEvent.
        \param uiParam3 The third parameter of this NotificationEvent. The default value is zero.

        \sa GetSource, GetType, GetParam1, GetParam2, GetStringParam, GetParam3
    */
    NotificationEvent(PrincipalID pidSource, qUnsignedInt32 uiType, qUnsignedInt64 uiParam1, qUnsignedInt64 uiParam2, const String& strParam, qUnsignedInt64 uiParam3=0)
    {
        m_pidSource = pidSource;
        m_uiType = uiType;
        m_uiParam1 = uiParam1;
        m_uiParam2 = uiParam2;
        m_uiParam3  = uiParam3;
        m_strParam = strParam;
    }

    /*
        \brief Constructor

        The NotificationEvent class is used to create notification messages that will be sent
        from a back-end service to another Process when a given event occurs. The conditions under which
        a Notification event will be sent are defined via the BaseClass.SendNotificationEvent method
        of the Script module. Several NotificationEvents are defined in Rendez-Vous however, you may
        also define your own custom notification events.

        A NotificationEvent may specify a subtype event, which is defined via the uiMinorType parameter when
        an event is constructed, only if the given NotificationEvents has subtype events. The pre-defined subtype
        events are listed in the table below.

        \verbatim
        uiMinorType             uiMajorType
        ParticipationEvents     NotificationEvents::ParticipationEvent
        \endverbatim

        \param pidSource The PrincipalID of the back-end service that generates this NotificationEvent.
        \param uiMajorType The major type of this NotificationEvent, which is defined in NotificationEvents.
        \param uiMinorType The minor type of this NotificationEvent. This value must be between 0 and 999.
                           The default value is zero. This parameter may only be specified for those NotificationEvents that
                           have subtype events.
        \param uiParam1 The first parameter of this NotificationEvent.
        \param uiParam2 The second parameter of this NotificationEvent.
        \param strParam The string parameter of this NotificationEvent.
        \param uiParam3 The third parameter of this NotificationEvent. The default value is zero.

        \sa GetSource, GetType, GetParam1, GetParam2, GetStringParam, GetParam3
    */
    NotificationEvent(PrincipalID pidSource, qUnsignedInt32 uiMajorType, qUnsignedInt32 uiMinorType, qUnsignedInt64 uiParam1, qUnsignedInt64 uiParam2, const String& strParam, qUnsignedInt64 uiParam3=0)
    {
        m_pidSource = pidSource;
        m_uiType = uiMajorType * 1000 + uiMinorType;
        m_uiParam1 = uiParam1;
        m_uiParam2 = uiParam2;
        m_uiParam3  = uiParam3;
        m_strParam = strParam;
    }

    NotificationEvent(PrincipalID pidSource, qUnsignedInt32 uiMajorType, qUnsignedInt32 uiMinorType, qUnsignedInt64 uiParam1, qUnsignedInt64 uiParam2, const String& strParam, qUnsignedInt64 uiParam3, const qMap<String, Variant>& mapParam)
    {
        m_pidSource = pidSource;
        m_uiType = uiMajorType * 1000 + uiMinorType;
        m_uiParam1 = uiParam1;
        m_uiParam2 = uiParam2;
        m_uiParam3  = uiParam3;
        m_strParam = strParam;
        m_mapParam = mapParam;
    }

    /*
        \brief Returns the PrincipalID of the back-end Service that generated this NotificationEvent.
        \returns The PrincipalID of the back-end Service that generated this NotificationEvent.
    */

    /*!
     @brief    イベントの発生元の PrincipalID を取得します。

     @return    イベントの発生元の PrincipalID
     */
    PrincipalID GetSource() const
    {
        return m_pidSource;
    }

    /*
        \brief Returns the type of this NotificationEvent.

        If the type of NotificationEvents is a ParticipationEvent, then GetSubType will return the type
        of ParticipationEvents that occurred. Currently, ParticipationEvents are the only NotificationEvents
        that specify a subtype.

        \returns The type of the NotificationEvent as a qUnsignedInt32.
        \sa GetSubType
    */

    /*!
     @brief    イベント種別を取得します。

     @ref NotificationEvents で定義されるイベント種別を取得します。

     @return    イベント種別
     */
    qUnsignedInt32 GetType() const
    {
        return m_uiType / 1000;
    }

    /*
        \brief Returns the subtype of this NotificationEvent.

        If the type of NotificationEvents is a /ParticipationEvent/, then GetSubType will return the type
        of ParticipationEvents that occurred. Currently, ParticipationEvents are the only NotificationEvents
        that specify a subtype.

        \returns The subtype of the NotificationEvent as a qUnsignedInt32.
        \sa GetType
    */

    /*!
     @brief    イベントのサブタイプを取得します。

     イベントのサブタイプを取得します。サブタイプは @ref NotificationEvents 毎に定義されています。

     @return    イベントのサブタイプ
     */
    qUnsignedInt32 GetSubType() const
    {
        return m_uiType % 1000;
    }

    /*
        \brief Returns the first parameter of this NotificationEvent.
        \returns The value of the first parameter of the NotificationEvent as a qUnsignedInt32.
        \sa GetParam2, GetStringParam
    */

    /*!
     @brief    イベントの種別毎に定義されるパラメータを取得します

     イベントの種別毎に定義されるパラメータを取得します。
     取得できる値の意味はイベントの種別（またはサブタイプ）によって定義されています。

     @return    イベントの種別毎に定義されるパラメータ
     */
    qUnsignedInt64 GetParam1() const
    {
        return m_uiParam1;
    }

    /*
        \brief Returns the second parameter of this NotificationEvent.
        \returns The value of the second parameter of the NotificationEvent as a qUnsignedInt32.
        \sa GetParam1, GetStringParam
    */

    /*!
     @brief    イベントの種別毎に定義されるパラメータを取得します

     イベントの種別毎に定義されるパラメータを取得します。
     取得できる値の意味はイベントの種別（またはサブタイプ）によって定義されています。

     @return    イベントの種別毎に定義されるパラメータ
     */
    qUnsignedInt64 GetParam2() const
    {
        return m_uiParam2;
    }

    /*
        \brief Returns the third parameter of this NotificationEvent.
        \returns The value of the second parameter of the NotificationEvent as a qUnsignedInt32.
        \sa GetParam1, GetStringParam
    */

    /*! @internal
     @brief    イベントの種別毎に定義されるパラメータを取得します

     イベントの種別毎に定義されるパラメータを取得します。
     取得できる値の意味はイベントの種別（またはサブタイプ）によって定義されています。

     @return    イベントの種別毎に定義されるパラメータ
     */
    qUnsignedInt64 GetParam3() const
    {
        return m_uiParam3;
    }

    /*
        \brief Returns the second parameter of this NotificationEvent.
        \returns The value of the second parameter of the NotificationEvent as a qUnsignedInt64.
        \sa GetParam1, GetParam2, GetStringParam
    */

    /*!
     @brief    イベントの種別毎に定義されるパラメータを取得します

     NotificationEvent::GetParam1() NotificationEvent::GetParam2() NotificationEvent::GetParam3()
     で取得できる型を qUnsignedInt32 から qUnsignedInt64 に変更したことに伴い、廃止予定です。
     NotificationEvent::GetParam1() で取得できる値と同じとなります。

     イベントの種別毎に定義されるパラメータを取得します。
     取得できる値の意味はイベントの種別（またはサブタイプ）によって定義されています。

     @return    イベントの種別毎に定義されるパラメータ
     */
    qUnsignedInt64 GetUInt64Param1() const
    {
        return m_uiParam1;
    }

    /*
        \brief Returns the string parameter of this NotificationEvent.
        \returns The string parameter of the NotificationEvent.
        \sa GetParam1, GetParam2
    */

    /*!
     @brief    イベントの種別毎に定義されるパラメータを取得します

     イベントの種別毎に定義されるパラメータを取得します。
     取得できる値の意味はイベントの種別（またはサブタイプ）によって定義されています。

     @return    イベントの種別毎に定義されるパラメータ
     */
    const String& GetStringParam() const
    {
        return m_strParam;
    }

    qBool GetParam(const String& key, Variant* value) const
    {
        qMap<String, Variant>::const_iterator v = m_mapParam.find(key);
        if (v == m_mapParam.end())
        {
            return false;
        }

        *value = v->second;
        return true;
    }
};

/*!
@brief 文字列ストリームにNotificationEventオブジェクトの情報を追加します。デバッグ用途です。
@param[in] oStream 情報を追加する文字列ストリームを指定します。
@param[in] oEvent NotificationEventオブジェクトを指定します。
@return 情報が追加された文字列ストリームが返ります。
 */
extern StringStream& operator<<(StringStream& oStream, const NotificationEvent& oEvent);

END_NAMESPACE_NEX

