﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Platform/Core/SpinTest.h>

BEGIN_NAMESPACE_NEX

class Credentials;

/*
\ingroup group_rdv_core_common
\brief This class defines the context of a call to a lobby service.
\remarks This class is used to define the call context of calls made to the lobby service. The basic
pattern you should follow when using a ProtocolCallContext with a \productname call is as follows,
where TEST() should be replaced by your own code that tests that the specified expressions are true.

\code
        ProtocolCallContext oContext;
        // ALWAYS initialize out parameters!
        qBool bCallResult = false;
        Parameter oSomeInputParameter = oSomeImportantValue;
        Parameter oSomeOutputParameter = oSomeInitialValue;
        qBool bCallIssued = GetSomeClient()->CallSomeRMC(&oContext, &bCallResult, oSomeInputParameter, &oSomeOutputParameter);
        // 1. Check that the RMC was issued successfully. The qBool return value
        // will be false if the RMC failed to be issued from the client.
        TEST(bCallIssued == true);
        // 2. Check that Wait returns true if a blocking call is made. This will return
        // false if the RMC did not get a return message (e.g. timeout)
        TEST(oContext.Wait());
        // 3. Check that GetState==CallSuccess. It will be false if the RMC execution failed
        // on the server (e.g. an exception was raised).
        if (oContext.GetState==CallContext::CallSuccess) {
            // 4a. GetOutcome normally returns true in this case. If not, you should check the
            // Error Codes for details.
            TEST(QSUCCEEDED(oContext.GetOutcome().GetReturnCode()));
            // 4b. This will be false iff the remote method returns false.
            TEST(bCallResult == true);
        } else {
            // 5a. GetOutcome returns false in this case. You should check the Error Codes for details.
            TEST(QFAILED(oContext.GetOutcome().GetReturnCode()));
            // 5b. The remote method's return value was likely not set since the call failed.
            TEST(bCallResult == false);
        }
\endcode

Refer to AccountManagementClient::UpdateCustomData for an example of a ProtocolCallContext in use.
*/

/*!
 @brief    ゲームサーバーとの通信処理に対する CallContext クラスです。

 ゲームサーバーとの通信処理に対する CallContext クラスです。NEX でゲームサーバーと通信するような
 非同期処理関数には、有効な ProtocolCallContext クラスオブジェクトを渡す必要があります。

 タイムアウト時間を設定する場合 30 秒よりも短い時間を指定しないようにしてください。
 */
class OSDK_API ProtocolCallContext : public CallContext
{
public:
    ProtocolCallContext(Credentials *pCredentials=NULL);
    virtual ~ProtocolCallContext();

#ifdef NEX_MOVE_SEMANTICS_SUPPORT

    /*!
    @brief    ムーブコンストラクタです。

    @param[in]    rhs    ProtocolCallContext オブジェクト。
    */
    ProtocolCallContext(ProtocolCallContext&& rhs) NEX_NOEXCEPT
        : CallContext()
        , m_vReturnValuePtr()
        , m_pbResult(0)
        , m_pqResult(0)
        , m_pCredentials(0)
#if defined(WIN32_TARGET) || defined(LINUX_TARGET) || defined(NEX_DEBUG_VERSION)
        , m_RMCName(NULL)
#endif
    {
        *this = std::move(rhs);
    }

    /*!
    @brief    ムーブ代入演算子です。

    @param[in]    rhs    ProtocolCallContext オブジェクト。
    */
    ProtocolCallContext& operator=(ProtocolCallContext&& rhs) NEX_NOEXCEPT
    {
        if (this == &rhs)
        {
            return *this;
        }
        this->Swap(rhs);
        return *this;
    }
#endif

    /**
    \internal
    */
    void AddReturnValuePtr(void* pReturnValue);
    /**
    \internal
    */
    void* GetReturnValuePtr(qUnsignedInt iIndex);
    /**
    \internal
    */
    void ClearReturnValuePtr();
    /**
    \internal
    */
    void PatchReturnCodePtr(qBool *pbResult)
    {
        m_pbResult = pbResult;
    }
    /**
    \internal
    */
    void PatchReturnCodePtr(qResult *pqResult)
    {
        m_pqResult = pqResult;
    }

    /**
    \internal
    */
    void SetCredentials(Credentials *pCredentials);
    /**
    \internal
    */
    Credentials* GetCredentials() const
    {
        return(m_pCredentials);
    }

#if defined(WIN32_TARGET) || defined(LINUX_TARGET) || defined(NEX_DEBUG_VERSION)
    void SetRMCName(const qChar* pString)
    {
        m_RMCName = pString;
    }
    const qChar* GetRMCName() const
    {
        return m_RMCName;
    }
#endif

    void Swap(ProtocolCallContext& rhs) NEX_NOEXCEPT
    {
        CallContext::Swap(rhs);
        std::swap(m_vReturnValuePtr, rhs.m_vReturnValuePtr);
        std::swap(m_pbResult, rhs.m_pbResult);
        std::swap(m_pqResult, rhs.m_pqResult);
        std::swap(m_pCredentials, rhs.m_pCredentials);
#if defined(WIN32_TARGET) || defined(LINUX_TARGET) || defined(NEX_DEBUG_VERSION)
        std::swap(m_RMCName, rhs.m_RMCName);
#endif
    }

    /*!
    @brief    この @ref ProtocolCallContext の完了時に実行するコールバックを設定します。

    設定したコールバックはこの @ref ProtocolCallContext が完了状態に遷移したときに呼び出されます。
    このコールバックを使用することで、 @ref ProtocolCallContext が完了したかどうかを確認するためにポーリングする手間を省くことができます。

    callback には @ref ProtocolCallContext* を引数に取る関数ポインタ、関数オブジェクト、ラムダ式を指定できます。
    AddCallContextArgumentAdapter() を併用すると引数を一つも取らない関数を渡すこともできます。

    関数オブジェクトを渡す場合は値もしくは参照型で渡す方法とポインタ型で渡す方法の二種類があり、それぞれ挙動が異なります。
    値もしくは参照型で渡した場合は内部で関数オブジェクトがコピーもしくはムーブされ、コールバックはそのコピーもしくはムーブされたオブジェクトに対して行われます。
    そのため、本関数呼び出し後は指定したオブジェクトを即座に破棄する事ができます。
    ポインタ型で渡した場合は、コールバックはそのポインタが指すオブジェクトに対して行われます。
    そのため、コールバックが呼び出されるまでその関数オブジェクトを呼び出し側で保持しておく必要があります。

    本関数を複数回呼び出すことで複数のコールバックを登録することが出来ます。
    複数のコールバックを登録した場合はデフォルトでは登録順にコールバックの呼び出しが行われ、最初に登録されたコールバックは最初に呼び出されます。
    この順序を変更したい場合は、コールバックを登録する際に bAddToEnd パラメータを設定する必要があります。

    callback に関数オブジェクトをポインタで渡した場合は deleteCallbackAfterCall を true にする事で
    callback の呼び出し後に自動的に callback を delete することができます。
    もし callback を delete するのではなくアプリケーション独自の解放処理を行いたい場合は、
    再度本関数を呼び出しそのための関数オブジェクトを追加する事で実装が可能です。

    コールバック関数内では NEX のブロック関数を呼び出さないようにしてください。

    @param[in]  callback コールバックする関数ポインタもしくは関数オブジェクトもしくはラムダ式。
    @param[in]  deleteCallbackAfterCall callback の実行が完了した後自動的に callback を delete するかどうか。
    @param[in]  bAddToEnd true（デフォルト）であれば、コールバックはリストの最後に追加されます。falseであればリストの先頭に追加されます。
    */
#ifdef NEX_MOVE_SEMANTICS_SUPPORT
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(TCallback&& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<ProtocolCallContext>(std::forward<TCallback>(callback), deleteCallbackAfterCall, bAddToEnd);
    }
#else
    template<typename TCallback>
    void RegisterCompletionCallbackGeneric(const TCallback& callback, qBool deleteCallbackAfterCall=false, qBool bAddToEnd=true)
    {
        RegisterCompletionCallbackGenericImpl<ProtocolCallContext>(callback, deleteCallbackAfterCall, bAddToEnd);
    }
#endif

private:
    virtual void BeginTransition(State eNewState, qResult rOutcome, qBool bSetOutcome);

private:
    OSDK_DISABLE_EXPORT_WARNING(qVector<void*> m_vReturnValuePtr);
    qBool *         m_pbResult;
    qResult *       m_pqResult;
    Credentials *   m_pCredentials;
#if defined(WIN32_TARGET) || defined(LINUX_TARGET) || defined(NEX_DEBUG_VERSION)
    const qChar* m_RMCName;
#endif
};

END_NAMESPACE_NEX

