﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Common/ResultRangeDefs.h>
#include <RendezVous/Core/src/Common/ResultRangeDDL.h>

BEGIN_NAMESPACE_NEX

/*
    \brief This class determines the size and offset of a returned list.
    \remarks The ResultRange sets the maximum number of entries returned and their offset
    for an associated list. The offset is the number of entries from the first entry from
    where the list will start. The default size and offset are equal to 20 and zero entries
    respectively.

    When the ResultRange is used to return a list, so that each Principal is not returned the same list
    you should use a random or semi-random offset. This you may define yourself or simply specify
    RESULTRANGE_ANY_OFFSET as the offset. Returning random lists is particularly important when returning
    game session lists as it avoids the same gathering participants being pinged by a multitude of Principal s.
    Refer to the Rendez-Vous /Best Practices/ document for further details.
*/

/*!
 @brief    検索時の結果リストのサイズとオフセットを保持するクラスです。

 サイズは検索条件に合致するエントリーの中から取得するエントリーの最大数、
 オフセットは取得するエントリーの開始位置を示します。

 @ref RESULTRANGE_ANY_OFFSET をオフセットに指定することで、ゲームサーバー上で
 検索結果全体を無作為に並び替えた上で、サイズ分の結果をクライアントに返します。
 */
class OSDK_API ResultRange: public DDLCLASS(ResultRange)
{
public:
    /*
    \brief Constructor
    \param[in] uiOffset The offset of this ResultRange. The default value is zero.
    \param[in] uiSize The size of this ResultRange. The default value is 20 entries.
    \remarks When RESULTRANGE_ANY_OFFSET is set as the offset, \PRODUCTNAME uses a pseudo random
    offset when returning the ResultRange.
    */

    /*!
     @brief    コンストラクタです。

     @param[in]    uiOffset    取得するエントリーのオフセット値（開始位置）。デフォルトは 0 です。
     @param[in]    uiSize    取得するエントリーの最大サイズ。デフォルトは RESULTRANGE_DEFAULT_SIZE です。
     */
    ResultRange(qUnsignedInt32 uiOffset = 0, qUnsignedInt32 uiSize = RESULTRANGE_DEFAULT_SIZE)
    {
        m_uiOffset = uiOffset;
        m_uiSize = uiSize;
    }
    virtual ~ResultRange() {}

    /*
    \brief Sets the offset for this ResultRange.
    \param[in] uiOffset The offset, in number of entries, that will be set for this ResultRange.
    \remarks The offset of a ResultRange is equal to the number of entries from the first entry from
    where the associated list result will start. When RESULTRANGE_ANY_OFFSET is set as the offset,
    \PRODUCTNAME uses a pseudo random offset when returning the ResultRange.
    \sa GetOffset, RESULTRANGE_ANY_OFFSET
    */

    /*!
     @brief    オフセット値をセットします。

     @param[in]    uiOffset    取得するエントリーのオフセット値（開始位置）。デフォルトは 0 です。
    */
    void SetOffset(qUnsignedInt32 uiOffset = 0)
    {
        m_uiOffset=uiOffset;
    }

    /*
    \brief Returns the offset of this ResultRange.
    \returns The offset, in number of entries, of this ResultRange.
    \sa SetOffset
    */

    /*!
     @brief    オフセット値を取得します。

     @return    取得するエントリーのオフセット値（開始位置）。
    */
    virtual qUnsignedInt32 GetOffset() const
    {
        return m_uiOffset;
    }

    /*
    \brief Sets the size of this ResultRange.
    \param[in] uiSize The size, in number of entries, that this ResultRange will be set to.
    \remarks The size of a ResultRange is equal to the maximum number of entries that
    the associated list result will return.
    \sa GetSize, RESULTRANGE_DEFAULT_SIZE
    */

    /*!
     @brief    取得するエントリーの最大サイズをセットします。

     @param[in]    uiSize    取得するエントリーの最大サイズ。デフォルトは RESULTRANGE_DEFAULT_SIZE です。
    */
    void SetSize(qUnsignedInt32 uiSize = RESULTRANGE_DEFAULT_SIZE)
    {
        m_uiSize=uiSize;
    }

    /*
    \brief Returns the size of this ResultRange.
    \returns The size, in number of entries, of this ResultRange.
    \sa SetSize
    */

    /*!
     @brief    取得するエントリーの最大サイズを取得します。

     @return    取得するエントリーの最大サイズ。
    */
    virtual qUnsignedInt32 GetSize() const
    {
        return m_uiSize;
    }

    /*!
     @refmanoverload{name1}

     @brief    取得するエントリーの最大サイズをインクリメントします。
     */
    ResultRange operator++(int)
    {
        ResultRange temp = *this;
        m_uiOffset += m_uiSize;
        return temp;
    } // post-increment ++

    /*!
    @refmanoverload{name1}

     @brief    取得するエントリーの最大サイズをインクリメントします。
     */
    ResultRange& operator++( )
    {
        m_uiOffset += m_uiSize;
        return *this;
    } // pre-increment ++
};


/*
\internal
*/
inline StringStream& operator<<(StringStream& oStr, const ResultRange& oRange)
{
    oStr << NEX_T("ResultRange: ( m_uiOffset=") << oRange.m_uiOffset
         << NEX_T(" m_uiSize=") << oRange.m_uiSize << NEX_T(") ");

    return oStr;
}


END_NAMESPACE_NEX

/** @} */


