﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/DataStore/src/Common/DataStoreConstants.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief      データの分割ダウンロードに使用するイベントリスナーの抽象クラスです。

        本クラスは RootObject から継承されません。本クラスを継承したクラスで必要に応じて RootObject を継承してください。
*/

#ifdef FOR_DOC_PURPOSE
class DataStoreGetObjectEventListener
#else
template<class TDataId>
class DataStoreGetObjectEventListenerTemplate
#endif  // FOR_DOC_PURPOSE
{
public:
#ifdef FOR_DOC_PURPOSE
    /*!
        @brief      コンストラクタです。
    */
    DataStoreGetObjectEventListener() {}
#else
    DataStoreGetObjectEventListenerTemplate() {}
#endif  // FOR_DOC_PURPOSE

#ifdef FOR_DOC_PURPOSE
    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreGetObjectEventListener() {}
#else
    virtual ~DataStoreGetObjectEventListenerTemplate() {}
#endif  // FOR_DOC_PURPOSE

    /*!
        @internal
        @brief      ダウンロードが開始された事を通知します。

        DataStoreClient::GetObjectOrMetaBinary() を呼び出した際、ダウンロード開始時に一度呼ばれます。
        本関数はBegin()の前に呼び出されます。

        @param[in]  dataId ダウンロードを開始したデータのデータID。
        @param[in]  size ダウンロードを開始したデータのサイズ。ストレージサーバーのファイルサイズもしくはメタバイナリのサイズです。
        @param[in]  ownerId ダウンロードを開始したデータのオーナー。
        @param[in]  dataType ダウンロードを開始したデータのデータタイプ。
        @param[in]  version ダウンロードを開始したデータのバージョン。
        @param[in]  isStorageServer ストレージサーバーからダウンロードする場合はTrue、メタバイナリの場合はFalse
    */
    virtual void BeginInternal(TDataId dataId, size_t size, PrincipalID ownerId, qUnsignedInt16 dataType, qUnsignedInt16 version, qBool isStorageServer)
    {
        (void)dataId;
        (void)size;
        (void)ownerId;
        (void)dataType;
        (void)version;
        (void)isStorageServer;
    }

#ifdef FOR_DOC_PURPOSE
    /*!
        @brief      ダウンロードが開始された事を通知します。

        ダウンロード開始時に一度呼ばれます。

        @param[in]  dataId ダウンロードを開始したデータのデータID。
        @param[in]  size ダウンロードを開始したデータのサイズ。
    */
    virtual void Begin(qUnsignedInt64 dataId, size_t size)
#else
    virtual void Begin(TDataId dataId, size_t size)
#endif  // FOR_DOC_PURPOSE
    {
        (void)dataId;
        (void)size;
    };

    /*!
        @brief      ダウンロードされたデータを分割して受け取ります。

        データがダウンロードされると、非同期処理が完了するまで本関数が複数回呼ばれます。
        ダウンロードの非同期処理が途中で失敗、キャンセルされた場合、受け取ったデータは完全ではありませんので注意してください。
        渡されるバッファの最大サイズは DataStoreClient::SetRelayBufferSize() で設定できます。

        @param[in]  buffer ダウンロードされたデータ。
        @param[in]  bufferSize buffer のサイズ。1以上かつ DataStoreClient::SetRelayBufferSize() で設定した値以下のサイズが渡されます。

        @return データの受信処理が正常に完了したときは true を、完了できなかったときは false を返してください。
                true を返した場合はダウンロード処理が継続されます。false を返した場合は次回の本関数呼び出しで再度同じデータが渡されます。
                false を返すことでフロー制御を行う事が出来ますが、10 秒以上 false を返し続けた場合、ダウンロードに失敗する可能性があります。
    */
    virtual qBool ProcessResponse(const void* buffer, size_t bufferSize) = 0;
};


typedef DataStoreGetObjectEventListenerTemplate<qUnsignedInt32> DataStoreGetObjectEventListenerV1;
typedef DataStoreGetObjectEventListenerTemplate<qUnsignedInt64> DataStoreGetObjectEventListener;

/*!
    @brief      データの分割アップロードに使用するイベントリスナーの抽象クラスです。

        本クラスは RootObject から継承されません。本クラスを継承したクラスで必要に応じて RootObject を継承してください。
*/
class DataStorePostObjectEventListener
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStorePostObjectEventListener() {}

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStorePostObjectEventListener() {}

    /*!
        @brief      アップロードするデータを分割して登録します。

        データのアップロードを開始すると、非同期処理が完了するまで本関数が複数回呼ばれます。
        アップロードするデータを順次 buffer に書き込みます。
        completed を true にしたときに書き込んだ合計サイズがアップロードする際に指定したサイズと一致している必要があります。
        一致していない場合はキャンセルされたときと同じ処理が行われます。
        また、completed を true にしていない場合でも、アップロードする際に指定したサイズより書き込んだサイズが大きくなったときも同様にキャンセルされます。
        渡されるバッファの最大サイズは DataStoreClient::SetRelayBufferSize() で設定できます。

        @param[out] buffer アップロードするデータをここに書き込みます。
        @param[in]  bufferSize buffer に書き込める最大サイズ。1以上かつ DataStoreClient::SetRelayBufferSize() で設定した値以下のサイズが渡されます。
        @param[out] completed アップロードが完了したら true を代入、完了していなければfalseを代入もしくは未設定にしてください。true にすると、以後本関数は呼び出されません。

        @return buffer に書き込んだサイズ。0以上 bufferSize 以下の値になるようにしてください。0を返し続けることによりアップロード処理を一時中断できます。
    */
    virtual size_t PostChunkedBuffer(void* buffer, size_t bufferSize, qBool* completed) = 0;
};

/*!
    @internal
    @brief      新着通知に利用されるトークンに関するイベントリスナーの抽象クラスです。

        本クラスは RootObject から継承されません。本クラスを継承したクラスで必要に応じて RootObject を継承してください。
*/
#ifdef FOR_DOC_PURPOSE
class DataStoreReceiveNewArrivedObjectsEventListener
#else
template<class TDataId>
class DataStoreReceiveNewArrivedObjectsEventListenerTemplate
#endif  // FOR_DOC_PURPOSE
{
public:
#ifdef FOR_DOC_PURPOSE
    /*!
        @brief      コンストラクタです。
    */
    DataStoreReceiveNewArrivedObjectsEventListener() {}
#else
    DataStoreReceiveNewArrivedObjectsEventListenerTemplate() {}
#endif  // FOR_DOC_PURPOSE

#ifdef FOR_DOC_PURPOSE
    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreReceiveNewArrivedObjectsEventListener() {}
#else
    virtual ~DataStoreReceiveNewArrivedObjectsEventListenerTemplate() {}
#endif  // FOR_DOC_PURPOSE

#ifdef FOR_DOC_PURPOSE
    /*!
        @brief      次にダウンロードするデータのメタデータを通知します。

        この通知があっても次にダウンロード処理が行われない場合があります。その場合は DataStoreGetObjectEventListener のコールバックは呼ばれません。

        @param[in] dataId 次にダウンロードする新着データのデータID。
        @param[in] size 次にダウンロードする新着データのサイズ。
        @param[in] ownerId 次にダウンロードする新着データのオーナーのプリンシパル ID 。
        @param[in] dataType 今回完了した新着データのデータタイプ。
        @param[in] version 次にダウンロードする新着データのバージョン。この値はデータ一つ一つに付加されるもので、
                           データを DataStoreClient::PostObject() でアップロードした直後は 1 となります。その後データを UpdateObject() で更新するに従い増加していきます。
                           値は連続したものではなく、飛び飛びの値になります。
        @param[in] notificationId 次にダウンロードする新着処理に対応するID。
                                  このIDは新着通知ごとに一意に割り当てられ、新しい更新になるほど値が増加していきます。
                                  値は連続したものではなく、飛び飛びの値になります。ゲーム内でユニークなIDです。
    */
    virtual void PrepareNextDownload(qUnsignedInt64 dataId,
                                     size_t size,
                                     PrincipalID ownerId,
                                     qUnsignedInt16 dataType,
                                     qUnsignedInt32 version,
                                     qUnsignedInt64 notificationId)
#else
    virtual void PrepareNextDownload(TDataId dataId,
                                     size_t size,
                                     PrincipalID ownerId,
                                     qUnsignedInt16 dataType,
                                     qUnsignedInt32 version,
                                     qUnsignedInt64 notificationId)
#endif  // FOR_DOC_PURPOSE
    {
        (void)dataId;
        (void)size;
        (void)ownerId;
        (void)dataType;
        (void)version;
        (void)notificationId;
    }

#ifdef FOR_DOC_PURPOSE
    /*!
        @brief      チェックトークンが更新されたことを通知します。

        最初のダウンロードが開始される前と、各データのダウンロード後に呼び出されます。ダウンロードとダウンロードの合間に複数回呼ばれることがあります。
        データダウンロード後の呼び出しではダウンロードされたデータに関連する情報が getObjectEventListener に含まれます。
        最初の呼び出しではトークンの更新のみが通知されるため、 getObjectEventListener は NULL となります。
        チェックトークンの更新が必要なかった場合、最初の呼び出しは行われません。
        もし newArrivalCheckToken を保存しないまま処理を終了した場合、次回の新着チェックで同じ同じデータがダウンロードされます。
        逆に正しく newArrivalCheckToken が保存され、次回そのトークンが DataStoreClient::ReceiveNewArrivedObjects() に与えられた場合、
        同じデータをダウンロードすることはありません。

        @param[in] newArrivalCheckToken これまでダウンロードされたデータが全て保存されたときに保存すべきチェックトークン。
        @param[in] getObjectEventListener 完了したダウンロードデータに対応するイベントリスナー。
                                         ダウンロードには対応せず、トークンのみ更新される場合があります。そのとき本パラメータは NULL となります。
        @param[in] notificationId 最後にダウンロードされた新着処理に対応するID。
    */
    virtual void TokenUpdated(const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
                              DataStoreGetObjectEventListener* getObjectEventListener,
                              qUnsignedInt64 notificationId) = 0;
#else
    virtual void TokenUpdated(const qByte newArrivalCheckToken[DataStoreConstants::NEW_ARRIVAL_CHECK_TOKEN_SIZE],
                              DataStoreGetObjectEventListenerTemplate<TDataId>* getObjectEventListener,
                              qUnsignedInt64 notificationId) = 0;
#endif  // FOR_DOC_PURPOSE
};

typedef DataStoreReceiveNewArrivedObjectsEventListenerTemplate<qUnsignedInt32> DataStoreReceiveNewArrivedObjectsEventListenerV1;
typedef DataStoreReceiveNewArrivedObjectsEventListenerTemplate<qUnsignedInt64> DataStoreReceiveNewArrivedObjectsEventListener;

END_NAMESPACE_NEX

