﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreGetMetaParamDDL.h"
#include "DataStorePersistenceTarget.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      メタ情報の取得に関するパラメータを表すクラスです。
*/
class DataStoreGetMetaParam: public DDLCLASS(DataStoreGetMetaParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreGetMetaParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreGetMetaParam() {}

    /*!
        @brief      データIDをセットします。

        取得するデータのデータIDを指定します。
        本パラメータを設定する場合は DataStoreGetMetaParam::SetPersistenceTarget() で永続化対象(プリンシパルIDと永続化スロットID)を設定しないでください。

        @param[in]  dataId_ 取得するデータのデータID。
    */
    void SetDataId(qUnsignedInt64 dataId_)
    {
        SYSTEMCHECK(persistenceTarget == DataStorePersistenceTarget() && "PersistenceTarget is already set. Please specify either DataID or PersistenceTarget");
        this->dataId = dataId_;
    }

    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      取得するデータを永続化対象(プリンシパルIDと永続化スロットID)で指定します。

        取得するデータをプリンシパルIDと永続化スロットIDで指定します。
        本パラメータを設定する場合は DataStoreGetMetaParam::SetDataId() でデータIDを設定しないでください。

        @param[in]  persistenceTarget_ 取得するデータの永続化対象。
    */
    void SetPersistenceTarget(DataStorePersistenceTarget persistenceTarget_)
    {
        SYSTEMCHECK(dataId == DataStoreConstants::INVALID_DATAID && "DataID is already set. Please specify either DataID or PersistenceTarget");
        this->persistenceTarget = persistenceTarget_;
    }

    /*!
        @brief      取得するデータの永続化対象を取得します。

        @return     取得するデータの永続化対象。
    */
    const DataStorePersistenceTarget& GetPersistenceTarget() const
    {
        return this->persistenceTarget;
    }

    /*!
        @brief      メタ情報取得の戻り値オプションをセットします。

        DataStoreConstants::ResultFlag のビット単位の論理和を用いて複数指定することができます。
        デフォルトは 0 です。

        @param[in] resultOption_ メタ情報取得の戻り値オプション値。
    */
    void SetResultOption(qUnsignedInt32 resultOption_)
    {
        this->resultOption = static_cast<qByte>(resultOption_);
    }

    /*!
        @brief      メタ情報取得の戻り値オプションを取得します。

        @return     メタ情報取得の戻り値オプション値。
    */
    qByte GetResultOption() const
    {
        return this->resultOption;
    }

    /*!
        @brief      参照パスワードを設定します。

        参照権限を持たないデータを参照する場合に参照パスワードを指定します。
        データのオーナーから受け取った参照パスワードを指定してください。
        参照権限を持っている場合は参照パスワードは無視され、常に参照可能です。
        DataStoreConstants::INVALID_PASSWORD を指定した場合は参照パスワードでの認証を行いません。
        本関数で指定しない場合は DataStoreConstants::INVALID_PASSWORD が使用されます。

        @param[in]  accessPassword_ 参照パスワード
    */
    void SetAccessPassword(qUnsignedInt64 accessPassword_)
    {
        this->accessPassword = accessPassword_;
    }

    /*!
        @brief      参照パスワードを取得します。

        @return     参照パスワード。
    */
    qUnsignedInt64 GetAccessPassword() const
    {
        return this->accessPassword;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        dataId = DataStoreConstants::INVALID_DATAID;
        persistenceTarget = DataStorePersistenceTarget();
        SetResultOption(0);
        SetAccessPassword(DataStoreConstants::INVALID_PASSWORD);
    }

    /*!
        @brief      DataStoreGetMetaParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("DataStoreGetMetaParam: (")
               << NEX_T("dataId=") << dataId
               << NEX_T(", resultOption=") << resultOption
               << NEX_T(", persistenceTarget=") << persistenceTarget
               << NEX_T(", accessPassword=") << accessPassword
               << NEX_T(")")
              );
    }
};

END_NAMESPACE_NEX

