﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreMetaInfoDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      メタ情報を表すクラスです。
*/
class DataStoreMetaInfo: public DDLCLASS(DataStoreMetaInfo)
{
public:
    /*!
        @brief      コンストラクタです。無効なメタ情報を作成します。
    */
    DataStoreMetaInfo()
    {
        // ユーザーがこのクラスを生成したときにIsValid()がfalseを返すようにする
        dataId = DataStoreConstants::INVALID_DATAID;
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreMetaInfo() {}

    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      データをアップロードした人のプリンシパル ID を取得します。

        @return     データをアップロードした人のプリンシパル ID 。
    */
    PrincipalID GetOwnerId() const
    {
        return this->ownerId;
    }

    /*!
        @brief      サイズを取得します。

        @return     サイズ。
    */
    qUnsignedInt32 GetSize() const
    {
        return this->size;
    }

    /*!
        @brief      タイトルを取得します。

        @return     タイトル。
    */
    String GetName() const
    {
        return this->name;
    }

    /*!
        @brief      データタイプを取得します。

        @return     データタイプ。
    */
    qUnsignedInt16 GetDataType() const
    {
        return this->dataType;
    }

    /*!
        @brief      参照権限を取得します。

        @return     参照権限。
    */
    const DataStorePermission& GetAccessPermission() const
    {
        return this->permission;
    }

    /*!
        @brief      更新権限を取得します。推奨されません。

        これは互換性のために残されている古い関数です。 DataStoreMetaInfo::GetUpdatePermission() を使用して下さい。

       @return     更新権限。
    */
    const DataStorePermission& GetDeletePermission() const
    {
        QLOG(EventLog::Warning, NEX_T("DataStoreMetaInfo::GetDeletePermission() is deprecated. Use DataStoreMetaInfo::GetUpdatePermission()"));
        return this->delPermission;
    }

    /*!
        @brief      更新権限を取得します。

        @return     更新権限。
    */
    const DataStorePermission& GetUpdatePermission() const
    {
        return this->delPermission;
    }

    /*!
        @brief      アップロード日時を取得します。

        @return     アップロード日時。
    */
    const DateTime& GetCreatedTime() const
    {
        return this->createdTime;
    }

    /*!
        @brief      更新日時を取得します。

        @return     更新日時。
    */
    const DateTime& GetUpdatedTime() const
    {
        return this->updatedTime;
    }

    /*!
        @brief      有効日数を取得します。

        @return     有効日数。
    */
    qUnsignedInt16 GetPeriod() const
    {
        return this->period;
    }

    /*!
        @brief      データのステータスを取得します。

        @return     ステータス。
    */
    DataStoreConstants::DataStatus GetDataStatus() const
    {
        return static_cast<DataStoreConstants::DataStatus>(this->status);
    }

    /*!
        @internal
        @brief      ダウンロードされた回数を取得します。廃止されるため使用できません。

        @return     ダウンロードされた回数。
    */
    qUnsignedInt32 GetReferredCount() const
    {
        return this->referredCnt;
    }

    /*!
        @brief      関連づけられたデータIDを取得します。

        @return     関連づけられたデータID。
    */
    qUnsignedInt32 GetReferDataId() const
    {
        return this->referDataId;
    }

    /*!
        @brief      アップロードについての設定フラグを取得します。

        @return     アップロードについての設定フラグ。
    */
    DataStoreConstants::DataFlag GetDataFlag() const
    {
        return static_cast<DataStoreConstants::DataFlag>(this->flag);
    }

    /*!
        @internal
        @brief      参照日時を取得します。廃止されるため使用できません。

        @return     参照日時。
    */
    const DateTime& GetReferredTime() const
    {
        return this->referredTime;
    }

    /*!
        @brief      有効期限を取得します。

        @return     有効期限。
    */
    const DateTime& GetExpireTime() const
    {
        return this->expireTime;
    }

    /*!
        @brief      タグを取得します。

        @return     タグ。
    */
    const qVector<String>& GetTags() const
    {
        return this->tags;
    }

    /*!
        @brief      評価情報を取得します。

        @return     評価情報。
    */
    qMap<qInt8, DataStoreRatingInfo> GetRating() const
    {
        qMap<qInt8, DataStoreRatingInfo> ratingMap;

        qVector<DataStoreRatingInfoWithSlot>::const_iterator it;
        for (it = this->ratings.begin(); it != this->ratings.end(); ++it)
        {
            ratingMap.insert(qMap<qInt8, DataStoreRatingInfo>::value_type(it->slot, it->rating));
        }
        return ratingMap;
    }

    /*!
        @brief      評価情報を取得します。

        こちらのオーバーロード関数の方が効率的です。

        @param[out]  ratingMap 評価情報。
    */
    void GetRating(qMap<qInt8, DataStoreRatingInfo>& ratingMap) const
    {
        ratingMap.clear();
        qVector<DataStoreRatingInfoWithSlot>::const_iterator it;
        for (it = this->ratings.begin(); it != this->ratings.end(); ++it)
        {
            ratingMap.insert(qMap<qInt8, DataStoreRatingInfo>::value_type(it->slot, it->rating));
        }
    }

    /*!
        @refmanoverload{name1}

        @brief      メタ情報内バイナリデータを取得します。

        @return     メタ情報内バイナリデータ
    */
    qVector<qByte> GetMetaBinary() const
    {
        // qBuffer -> qVector<qByte>への変換
        qVector<qByte> vb;
        vb.assign(this->metaBinary.begin(), this->metaBinary.end());
        return vb;
    }

    /*!
        @refmanoverload{name2}

        @brief      メタ情報内バイナリデータを取得します。

        こちらのオーバーロード関数の方が効率的です。

        @param[out]  metaBinary_ メタ情報内バイナリデータ。
    */
    void GetMetaBinary(qVector<qByte>& metaBinary_) const
    {
        // qBuffer -> qVector<qByte>への変換
        metaBinary_.assign(this->metaBinary.begin(), this->metaBinary.end());
    }

    /*!
        @brief      このオブジェクトが有効かどうかを調べます。

        返値が false だった場合は内部パラメータは無効ですので値を取得しないでください。

        @return     このオブジェクトが有効なら true 、無効なら false 。
    */
    qBool IsValid() const
    {
        return dataId != DataStoreConstants::INVALID_DATAID;
    }

    /*!
        @brief      DataStoreMetaInfoオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << NEX_T("DataStoreMetaInfo: (")
           << NEX_T("dataId=") << dataId
           << NEX_T(", ownerId=") << ownerId
           << NEX_T(", size=") << size
           << NEX_T(", name=") << name
           << NEX_T(", dataType=") << dataType
           << NEX_T(", accessPermission=") << permission
           << NEX_T(", updatePermission=") << delPermission
           << NEX_T(", createdTime=") << createdTime
           << NEX_T(", updatedTime=") << updatedTime
           << NEX_T(", period=") << period
           << NEX_T(", status=") << status
           << NEX_T(", referDataId=") << referDataId
           << NEX_T(", dataFlag=") << hex << flag << dec
           << NEX_T(", expireTime=") << expireTime;

        ss << NEX_T(", tags=(");
        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }
        ss << NEX_T(")");

        ss << NEX_T(", ratings=(");
        for (qVector<DataStoreRatingInfoWithSlot>::const_iterator it = ratings.begin(); it != ratings.end(); ++it)
        {
            ss << NEX_T("slot=") << it->slot << NEX_T(", ratingInfo=") << it->rating << NEX_T(", ");
        }
        ss << NEX_T(")");

        ss << NEX_T(", metaBinary=");
        for (qBuffer::const_iterator it = metaBinary.begin(); it != metaBinary.end(); ++it)
        {
            ss << hex << *it;
        }

        ss << NEX_T(")");

        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

