﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePermissionDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      権限を表すクラスです。
*/
class DataStorePermission: public DDLCLASS(DataStorePermission)
{

public:
    /*!
        @brief      コンストラクタです。

        @param[in]  permission 権限の種類。
        @param[in]  recipientIds 権限を保有する人の PrincipalID。
    */
    explicit DataStorePermission(DataStoreConstants::Permission permission = DataStoreConstants::PERMISSION_PRIVATE, const qVector<PrincipalID>& recipientIds = qVector<PrincipalID>())
    {
        SetPermission(permission);
        SetRecipientIds(recipientIds);
        // 順番が変わるとテストで比較できなくなるが、テスト以外では困らないので、ソートしない
        // recipientIds.sort();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStorePermission() {}

    /*!
        @brief      権限の種類をセットします。

        @param[in]  permission_ 権限の種類。
    */
    void SetPermission(DataStoreConstants::Permission permission_)
    {
        this->permission = static_cast<qByte>(permission_);
    }

    /*!
        @brief      権限の種類を取得します。

        @return     権限の種類。
    */
    DataStoreConstants::Permission GetPermission() const
    {
        return static_cast<DataStoreConstants::Permission>(this->permission);
    }

    /*!
        @brief      権限保有者をセットします。

        権限を保有する人の プリンシパル ID  を指定します。
        指定した値は DataStorePermission::SetPermission() で DataStoreConstants::PERMISSION_SPECIFIED もしくは DataStoreConstants::PERMISSION_SPECIFIED_FRIEND を指定したときのみ利用できます。
        DataStoreConstants::PERMISSION_SPECIFIED を指定した場合は本関数で指定した プリンシパル ID  の人のみ権限を保有します。
        DataStoreConstants::PERMISSION_SPECIFIED_FRIEND を指定した場合は本関数で指定した プリンシパル ID  の内、実際に参照した時点でフレンドの人のみ権限を保有します。

        @param[in]  recipientIds_ 権限を保有する人の プリンシパル ID  のリスト。
    */
    void SetRecipientIds(const qVector<PrincipalID>& recipientIds_)
    {
        this->recipientIds = recipientIds_;
    }

    /*!
        @brief      権限保有者を取得します。

        @return     権限を保有する人の プリンシパル ID  のリスト。

        DataStoreClient::SearchObject() や DataStoreClient::SearchObjectLight() で検索した場合、
        検索結果レコードの参照権限(DataStoreMetaInfo::GetAccessPermission)、更新権限(DataStoreMetaInfo::GetUpdatePermission)の
        権限保有者の プリンシパルID は返却されず、常に空となっています。
        DataStoreClient::GetMeta() で取得した場合は有効な値が格納されます。
    */
    const qVector<PrincipalID>& GetRecipientIds() const
    {
        return this->recipientIds;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetPermission(DataStoreConstants::PERMISSION_PRIVATE);
        SetRecipientIds(qVector<PrincipalID>());
    }

    /*!
        @brief      DataStorePermissionオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド定義です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const DataStorePermission& oPermission);
};

inline StringStream& operator<<(StringStream& oStr, const DataStorePermission& oPermission)
{
    qVector<PrincipalID>::const_iterator itr = oPermission.recipientIds.begin();

    oStr << NEX_T("DataStorePermission: (permission=") << oPermission.permission
         << NEX_T(", targetIds=(");

    for (int i = 0; itr != oPermission.recipientIds.end(); i++, itr++)
    {
        if (i)
        {
            oStr << NEX_T(", ");
        }
        oStr << *itr;
    }

    oStr << NEX_T("))");

    return oStr;
}

END_NAMESPACE_NEX

