﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePersistenceTargetDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      永続化対象を表すクラスです。
*/
class DataStorePersistenceTarget: public DDLCLASS(DataStorePersistenceTarget)
{
public:
    /*!
        @brief      コンストラクタです。

        永続化対象を指定します。

        @param[in]  ownerId オーナーのプリンシパルID
        @param[in]  persistenceSlotId 永続化スロットID
    */
    explicit DataStorePersistenceTarget(PrincipalID ownerId=INVALID_PRINCIPALID, qUnsignedInt16 persistenceSlotId=DataStoreConstants::INVALID_PERSISTENCE_SLOT_ID)
    {
        SetOwnerId(ownerId);
        SetPersistenceSlotId(persistenceSlotId);
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStorePersistenceTarget() {}

    /*!
        @brief      オーナーのプリンシパルIDをセットします。

        @param[in]  ownerId_ オーナーのプリンシパルID
    */
    void SetOwnerId(PrincipalID ownerId_)
    {
        this->ownerId = ownerId_;
    }

    /*!
        @brief      オーナーのプリンシパルIDを取得します。

        @return     オーナーのプリンシパルID
    */
    PrincipalID GetOwnerId() const
    {
        return this->ownerId;
    }

    /*!
        @brief      永続化スロットIDをセットします。

        @param[in]  persistenceSlotId_ 永続化スロットID
    */
    void SetPersistenceSlotId(qUnsignedInt16 persistenceSlotId_)
    {
        this->persistenceSlotId = persistenceSlotId_;
    }

    /*!
        @brief      永続化スロットIDを取得します。

        @return     永続化スロットID
    */
    qUnsignedInt16 GetPersistenceSlotId() const
    {
        return this->persistenceSlotId;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetOwnerId(INVALID_PRINCIPALID);
        SetPersistenceSlotId(DataStoreConstants::INVALID_PERSISTENCE_SLOT_ID);
    }

    /*!
        @brief      DataStorePersistenceTargetオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド定義です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const DataStorePersistenceTarget& oPersistenceTarget);
};

inline StringStream& operator<<(StringStream& oStr, const DataStorePersistenceTarget& oPersistenceTarget)
{
    oStr <<NEX_T("DataStorePersistenceTarget: (")
         << NEX_T("ownerId=") << oPersistenceTarget.ownerId
         << NEX_T(", persistenceSlotId=") << oPersistenceTarget.persistenceSlotId
         << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

