﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStorePreparePostParamDDL.h"
#include "DataStoreConstants.h"
#include <algorithm>

BEGIN_NAMESPACE_NEX

/*!
    @brief      データのアップロードに関するパラメータを表すクラスです。

    このクラスは、 DataStorePostParam クラスと同一です。

*/
class DataStorePreparePostParam: public DDLCLASS(DataStorePreparePostParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStorePreparePostParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStorePreparePostParam() {}

    /*!
        @brief      サイズをセットします

        アップロードするデータのサイズを指定します。本パラメータは必ず指定する必要があります。
        実際にアップロードするデータのサイズと一致している必要があります。
        指定できる最大サイズは 10485760 バイトです。
        メタ情報のみアップロードす場合は 0 を指定します。

        @param[in]  size_ アップロードするデータのサイズ。
    */
    void SetSize(qUnsignedInt32 size_)
    {
        this->size = size_;
    }

    /*!
        @brief      サイズを取得します。

        @return     サイズ。
    */
    qUnsignedInt32 GetSize() const
    {
        return this->size;
    }

    /*!
        @brief      タイトルをセットします

        アップロードするデータにメタ情報として付加するタイトル(要約情報)を指定します。
        Unicodeの任意の文字列(UTF-8の文字数換算で最大64文字)を指定することができます。本パラメータの指定は必須ではありません。デフォルトは空文字列です。

        @param[in]  name_ アップロードするデータにメタ情報として付加するタイトル(要約情報)。
    */
    void SetName(const String& name_)
    {
#ifdef NEX_LOG_ENABLED
#if defined(NEX_UNICODE_SUPPORT)
        qInt32 length = name_.GetWideCharLength();
#else
        size_t length = name_.GetLength();
#endif
        if (length > DataStoreConstants::MAX_NAME_LENGTH)
        {
            QLOG(EventLog::Warning, NEX_T("DataStorePostParam::SetName(): name length is over the limit. ") << NEX_T("name=") << name_ << NEX_T(", length=") << length);
        }
#endif
        this->name = name_;
    }

    /*!
        @brief      タイトルを取得します。

        @return     タイトル。
    */
    String GetName() const
    {
        return this->name;
    }

    /*!
        @brief      データタイプをセットします。

        アップロードするデータにメタ情報として付加するデータタイプを指定します。
        ここでセットした値は DataStoreClient::SearchObject() や DataStoreClient::SearchObjectLight() から検索できます。
        デフォルトは0です。DataStoreConstants::INVALID_DATA_TYPE は使用できません。

        @param[in]  dataType_ アップロードするデータにメタ情報として付加するデータタイプ。
    */
    void SetDataType(const qUnsignedInt16 dataType_)
    {
        this->dataType = dataType_;
    }

    /*!
        @brief      データタイプを取得します。

        @return     データタイプ。
    */
    qUnsignedInt16 GetDataType() const
    {
        return this->dataType;
    }

    /*!
        @brief      参照権限をセットします。

        アップロードするデータを検索、参照できる範囲を表す権限を指定します。
        デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  permission_ アップロードするデータを検索、参照できる範囲を表す権限。
    */
    void SetAccessPermission(const DataStorePermission& permission_)
    {
        this->permission = permission_;
    }

    /*!
        @brief      参照権限を取得します。

        @return     参照権限。
    */
    const DataStorePermission& GetAccessPermission() const
    {
        return this->permission;
    }

    /*!
        @brief      更新権限をセットします。推奨されません。

        これは互換性のために残されている古い関数です。 DataStorePostParam::SetUpdatePermission() を使用して下さい。
        新しい更新権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  delPermission_ 新しい更新権限。
    */
    void SetDeletePermission(const DataStorePermission& delPermission_)
    {
        QLOG(EventLog::Warning, NEX_T("DataStorePostParam::SetDeletePermission() is deprecated. Use DataStorePostParam::SetUpdatePermission()"));
        this->delPermission = delPermission_;
    }

    /*!
        @brief      更新権限を取得します。推奨されません。

        これは互換性のために残されている古い関数です。 DataStorePostParam::GetUpdatePermission() を使用して下さい。

       @return     更新権限。
    */
    const DataStorePermission& GetDeletePermission() const
    {
        QLOG(EventLog::Warning, NEX_T("DataStorePostParam::GetDeletePermission() is deprecated. Use DataStorePostParam::GetUpdatePermission()"));
        return this->delPermission;
    }

    /*!
        @brief      更新権限をセットします。

        新しい更新権限を指定します。本パラメータの指定は必須ではありません。デフォルトは DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE) です。

        @param[in]  updatePermission 新しい更新権限。
    */
    void SetUpdatePermission(const DataStorePermission& updatePermission)
    {
        this->delPermission = updatePermission;
    }

    /*!
        @brief      更新権限を取得します。

        @return     更新権限。
    */
    const DataStorePermission& GetUpdatePermission() const
    {
        return this->delPermission;
    }

    /*!
        @brief      設定フラグをセットします。

        アップロードについての設定フラグを指定します。 DataStoreConstants::DataFlag に対してビット単位の論理和を用いて複数指定することができます。
        デフォルトは DataStoreConstants::DATA_FLAG_NONE です。

        @param[in]  flag_ アップロードについての設定のビットフラグ。 DataStoreConstants::DataFlag に対するビット単位の論理和です。
    */
    void SetDataFlag(qUnsignedInt32 flag_)
    {
        this->flag = flag_;
    }

    /*!
        @brief      設定フラグを取得します。

        @return     アップロードについての設定のビットフラグ。 DataStoreConstants::DataFlag に対するビット単位の論理和です。
    */
    qUnsignedInt32 GetDataFlag() const
    {
        return this->flag;
    }

    /*!
        @brief      有効日数をセットします。

        アップロードするデータを参照できる期間を日数で指定します。
        SetFlag() の引数に DataStoreConstants::DATA_FLAG_PRIODFROMLASTREFERED を指定した場合は最後に DataStoreClient::GetObject 関数により参照された日からカウントされます。
        DataStoreConstants::DATA_FLAG_PRIODFROMLASTREFERED を指定しなかった場合はアップロードした時点からカウントされます。
        UpdateObjectを使用してデータを更新した場合、その時点から再カウントされます。
        デフォルトは DataStoreConstants::DEFAULT_PERIOD です。
        指定できる最大有効日数は365日です。それ以上の値を指定した場合の結果は、 QERROR(DataStore, InvalidArgument) になります。

        @param[in]  period_ アップロードするデータを参照できる期間(日数)。
    */
    void SetPeriod(qUnsignedInt16 period_)
    {
        this->period = period_;
    }

    /*!
        @brief      有効日数を取得します。

        @return     有効日数。
    */
    qUnsignedInt16 GetPeriod() const
    {
        return this->period;
    }

    /*!
        @internal
        @brief      関連づけるデータIDをセットします。

        アップロードするデータに関連づけるデータIDを指定します。
        本パラメータの指定は必須ではありません。デフォルトは DataStoreConstants::INVALID_DATAID です。

        @param[in]  referDataId_ アップロードするデータに関連づけるデータID。
    */
    void SetReferDataId(qUnsignedInt32 referDataId_)
    {
        this->referDataId = referDataId_;
    }

    /*!
        @internal
        @brief      関連づけるデータIDを取得します。

        @return     関連づけるデータID。
    */
    qUnsignedInt32 GetReferDataId() const
    {
        return this->referDataId;
    }

    /*!
        @brief      タグをセットします

        アップロードするデータにメタ情報として付加するタグを指定します。最大 DataStoreConstants::NUM_TAG_SLOT 個のタグを指定できます。
        タグとして指定できる文字と文字数についてはプログラミングマニュアルを参照してください。
        本パラメータの指定は必須ではありません。デフォルトは空です。


        @param[in]  tags_ アップロードするデータにメタ情報として付加するタグ。
    */
    void SetTags(const qVector<String>& tags_)
    {
#ifdef NEX_LOG_ENABLED
        if (tags_.size() > DataStoreConstants::NUM_TAG_SLOT)
        {
            QLOG(EventLog::Warning, NEX_T("DataStorePostParam::SetTags(): slot size is over the limit. size=") << tags_.size());
        }
#endif

#ifdef NEX_SYSTEMCHECK_ENABLED
        qVector<String> tempTags(tags_);
        std::sort(tempTags.begin(), tempTags.end());
        qVector<String> uniqueTags;
        std::unique_copy(tempTags.begin(), tempTags.end(), std::back_inserter(uniqueTags));
        SYSTEMCHECK(tempTags.size() == uniqueTags.size() && NEX_T("Duplicate tags."));
#endif // NEX_SYSTEMCHECK_ENABLED

        this->tags = tags_;
    }

    /*!
        @brief      タグを取得します。

        @return     タグ。
    */
    const qVector<String>& GetTags() const
    {
        return this->tags;
    }

    /*!
        @brief      評価スロットの初期化設定を行います

        本パラメータの指定は必須ではありません。デフォルトは未指定ですが、評価機能を利用する場合は
        必ず初期化が必要になります。
        mapのキーは、0から15までのスロット番号の値を、値は、各スロットの初期化パラメータを設定します。

        @param[in]  ratingInitParam 評価スロットの初期化パラメータ
    */
    void SetRatingSetting(const qMap<qInt8, DataStoreRatingInitParam>& ratingInitParam)
    {
        this->ratingInitParams.clear();
        qMap<qInt8, DataStoreRatingInitParam>::const_iterator it = ratingInitParam.begin();
        for (; it != ratingInitParam.end(); ++it)
        {
            DataStoreRatingInitParamWithSlot paramSlot;
            paramSlot.slot = it->first;
            paramSlot.param = it->second;
            this->ratingInitParams.push_back(paramSlot);
        }
    }

    /*!
     *  @brief 評価スロットを未指定にします。
     */
    void ClearRatingSetting()
    {
        this->ratingInitParams.clear();
    }

    /*!
     *  @brief 評価スロットを一つ追加します。
     *
     *  本パラメータの指定は必須ではありません。デフォルトは未指定ですが、評価機能を利用する場合は
     *  必ず初期化が必要になります。
     *
     *  すでにある評価スロットの設定は残されて、指定された値を追加されます。
     *
     *  @retval true 追加成功
     *  @retval false 同じ　slot 番号の要素があって追加に失敗した
     */
    qBool AddRatingSetting(qInt8 slot, const DataStoreRatingInitParam& ratingInitParam)
    {
        qVector<DataStoreRatingInitParamWithSlot>::const_iterator it = this->ratingInitParams.begin();
        for (; it != this->ratingInitParams.end(); ++it)
        {
            if(it->slot == slot)
            {
                return false;
            }
        }

        DataStoreRatingInitParamWithSlot paramSlot;
        paramSlot.slot = slot;
        paramSlot.param = ratingInitParam;
        this->ratingInitParams.push_back(paramSlot);

        return true;
    }




    /*!
        @brief      評価スロットの初期化設定を取得します。

        @param[out]  ratingInitParam 評価スロットの初期化パラメータ
    */
    void GetRatingSetting(qMap<qInt8, DataStoreRatingInitParam>& ratingInitParam) const
    {
        ratingInitParam.clear();
        qVector<DataStoreRatingInitParamWithSlot>::const_iterator it = this->ratingInitParams.begin();
        for (; it != this->ratingInitParams.end(); ++it)
        {
            ratingInitParam.insert(std::make_pair(it->slot, it->param));
        }
    }

    /*!
        @brief      メタ情報内バイナリデータをセットします。

        アップロードするデータにメタ情報として付加するメタ情報内バイナリデータを指定します。
        0-1024 バイトまで設定できます。本パラメータの指定は必須ではありません。デフォルトは空です。

        @param[in]  p アップロードするデータにメタ情報として付加するバイナリデータのポインタ。NULLを指定すると空になります。
        @param[in]  size_ アップロードするバイナリデータのデータサイズ(0-1024)。
    */
    void SetMetaBinary(const void* p, size_t size_)
    {
        this->metaBinary.clear();
        if (p != NULL)
        {
            const qByte* pb = static_cast<const qByte*>(p);
            this->metaBinary.push_back(pb, size_);
        }
    }

    /*!
        @refmanoverload{name1}

        @brief      メタ情報内バイナリデータを取得します。

        @return     メタ情報内バイナリデータ
    */
    qVector<qByte> GetMetaBinary() const
    {
        // qBuffer -> qVector<qByte>への変換
        qVector<qByte> vb;
        vb.assign(this->metaBinary.begin(), this->metaBinary.end());
        return vb;
    }

    /*!
        @refmanoverload{name2}

        @brief      メタ情報内バイナリデータを取得します。

        こちらのオーバーロード関数の方が効率的です。

        @param[out]  metaBinary_ メタ情報内バイナリデータ。
    */
    void GetMetaBinary(qVector<qByte>& metaBinary_) const
    {
        // qBuffer -> qVector<qByte>への変換
        metaBinary_.assign(this->metaBinary.begin(), this->metaBinary.end());
    }

    /*!
        @brief      永続化情報をセットします。

        @param[in]  persistenceInitParam_ 永続化情報

        データを永続化する場合は、本関数を使用して永続化情報も設定してください。
        永続化したデータは有効期限により削除されることはありません。

    */
    void SetPersistenceInitParam(const DataStorePersistenceInitParam& persistenceInitParam_)
    {
        this->persistenceInitParam = persistenceInitParam_;
    }

    /*!
        @brief      永続化情報を取得します。

        @return     永続化情報
    */
    const DataStorePersistenceInitParam& GetPersistenceInitParam() const
    {
        return this->persistenceInitParam;
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetSize(0);
        SetName(NEX_T(""));
        SetDataType(0);
        SetAccessPermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetUpdatePermission(DataStorePermission(DataStoreConstants::PERMISSION_PRIVATE));
        SetDataFlag(DataStoreConstants::DATA_FLAG_NONE);
        SetPeriod(DataStoreConstants::DEFAULT_PERIOD);
        SetReferDataId(DataStoreConstants::INVALID_DATAID);
        SetTags(qVector<String>());
        SetMetaBinary(NULL, 0);
        SetPersistenceInitParam(DataStorePersistenceInitParam());
        SetRatingSetting(qMap<qInt8, DataStoreRatingInitParam>());
        this->extraData.clear();
    }

    /*!
        @brief      DataStorePreparePostParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        StringStream ss;
        ss << NEX_T("DataStorePreparePostParam: (")
           << NEX_T("size=") << size
           << NEX_T(", name=") << name
           << NEX_T(", dataType=") << dataType
           << NEX_T(", accessPermission=") << permission
           << NEX_T(", updatePermission=") << delPermission
           << NEX_T(", dataFlag=") << hex << flag << dec
           << NEX_T(", period=") << period
           << NEX_T(", referDataId=") << referDataId
           << NEX_T(", persistenceInitParam=") << persistenceInitParam;

        ss << NEX_T(", ratingInitParams=(");
        for (qVector<DataStoreRatingInitParamWithSlot>::const_iterator it = ratingInitParams.begin(); it != ratingInitParams.end(); ++it)
        {
            ss << NEX_T("slot=") << it->slot << NEX_T(", ratingInitParam=") << it->param << NEX_T(", ");
        }
        ss << NEX_T(")");

        ss << NEX_T(", tags=(");
        for (qVector<String>::const_iterator it = tags.begin(); it != tags.end(); ++it)
        {
            ss << *it << NEX_T(", ");
        }
        ss << NEX_T(")");

        ss << NEX_T(", metaBinary=");
        for (qBuffer::const_iterator it = metaBinary.begin(); it != metaBinary.end(); ++it)
        {
            ss << hex << *it;
        }

        ss << NEX_T(")");

        QTRACE(uiTraceFlags, ss.CStr());
    }

};

END_NAMESPACE_NEX

