﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreRatingInitParamDDL.h"
#include "DataStoreRatingLockInitParam.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      評価の初期化パラメータを表すクラスです。
*/
class DataStoreRatingInitParam: public DDLCLASS(DataStoreRatingInitParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreRatingInitParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreRatingInitParam() {}


    /*!
        @brief      評価合計値の初期値をセットします。
        @param[in]  initialValue_ 評価合計値の初期値。
    */
    void SetInitialValue(qInt64 initialValue_)
    {
        this->initialValue = initialValue_;
    }

    /*!
        @brief      一度に評価可能な評価値の最低値を設定します。
        @param[in]  min_ 一度に評価可能な評価値の最低値。
    */
    void SetRangeMin(qInt32 min_)
    {
        this->rangeMin = min_;
        this->internalFlag |= DataStoreConstants::RATING_INTERNAL_FLAG_USE_RANGE_MIN;
    }

    /*!
        @brief      一度に評価可能な評価値の最大値を設定します。
        @param[in]  max 一度に評価可能な評価値の最大値。
    */
    void SetRangeMax(qInt32 max)
    {
        this->rangeMax = max;
        this->internalFlag |= DataStoreConstants::RATING_INTERNAL_FLAG_USE_RANGE_MAX;
    }

    /*!
        @refmanoverload{name1}
        @brief      重複ロックを設定します。非推奨です。DataStoreRatingLockInitParam を引数に取るオーバーロード関数を使用して下さい。

        日時は UTC で評価されます。

        @param[in]  lockType_ ロックのタイプ
        @param[in]  periodDuration_ ロックの期間。
        lockType が DataStoreConstants::RATING_LOCK_INTERVAL ならロックする秒数、
        lockType が DataStoreConstants::RATING_LOCK_PERIOD なら DataStoreConstants::RatingLockPeriod もしくは 0 以上の整数。
        0 以上の整数を与えた場合は日数となります。
        lockType が DataStoreConstants::RATING_LOCK_PERMANENT もしくは DataStoreConstants::RATING_LOCK_NONE の場合は無視されます。
        @param[in]  periodHour_ ロックを解除する時刻 (lockType が DataStoreConstants::RATING_LOCK_PERIOD のときのみ有効)
    */
    void SetLock(DataStoreConstants::RatingLockType lockType_, qInt16 periodDuration_ = 0, qInt8 periodHour_ = 0)
    {
        this->lockType = static_cast<qByte>(lockType_);
        this->periodDuration = periodDuration_;
        this->periodHour = periodHour_;
    }

    /*!
        @refmanoverload{name2}
        @brief      重複ロックを設定します。

        @param[in]  ratingLockInitParam 重複ロック設定。
    */
    void SetLock(const DataStoreRatingLockInitParam& ratingLockInitParam)
    {
        this->lockType = static_cast<qByte>(ratingLockInitParam.GetLockType());
        this->periodDuration = ratingLockInitParam.GetPeriodDuration();
        this->periodHour = ratingLockInitParam.GetPeriodHour();
    }
    /*!
        @brief      オプションフラグを設定します

        DataStoreConstants::RatingFlag に対してビット単位の論理和を用いて複数指定することができます。

        @param[in]  flag_ フラグ
    */
    void SetFlag(qUnsignedInt32 flag_)
    {
        this->flag = static_cast<qByte>(flag_);
    }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        this->flag = 0;
        this->internalFlag = 0;
        this->lockType = 0;
        this->initialValue = 0;
        this->rangeMin = 0;
        this->rangeMax = 0;
        this->periodHour = 0;
        this->periodDuration = 0;
    }

    /*!
        @brief      DataStoreRatingInitParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド定義です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const DataStoreRatingInitParam& oRatingInitParam);
};

inline StringStream& operator<<(StringStream& oStr, const DataStoreRatingInitParam& oRatingInitParam)
{
    oStr << NEX_T("DataStoreRatingInitParam: (")
         << NEX_T("flag=") << oRatingInitParam.flag
         << NEX_T(", internalFlag=") << oRatingInitParam.internalFlag
         << NEX_T(", lockType=") << oRatingInitParam.lockType
         << NEX_T(", initialValue=") << oRatingInitParam.initialValue
         << NEX_T(", rangeMin=") << oRatingInitParam.rangeMin
         << NEX_T(", rangeMax=") << oRatingInitParam.rangeMax
         << NEX_T(", periodHour=") << oRatingInitParam.periodHour
         << NEX_T(", periodDuration=") << oRatingInitParam.periodDuration
         << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

