﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/



#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      評価の重複ロック設定を表すクラスです。
*/
class DataStoreRatingLockInitParam: public RootObject
{
public:
    /*!
        @brief      コンストラクタです。重複ロックをしない設定を生成します。一度評価した後でもすぐ再度評価できます。
    */
    DataStoreRatingLockInitParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreRatingLockInitParam() {}

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。重複ロックをしない設定に戻ります。
    */
    void Reset()
    {
        lockType = DataStoreConstants::RATING_LOCK_NONE;
        periodDuration = 0;
        periodHour = 0;
    }

    /*!
        @brief      評価してから一定の秒数は再度評価ができない設定にします。

        @param[in] intervalSec 評価したあとに再度評価できないようにする秒数。
    */
    void SetIntervalLock(qInt16 intervalSec)
    {
        SYSTEMCHECK(intervalSec >= 0);
        lockType = DataStoreConstants::RATING_LOCK_INTERVAL;
        periodDuration = intervalSec;
        periodHour = 0;
    }

    /*!
        @brief      特定の曜日や月初めになるまで再度評価できない設定にします。

        本関数を使用すると毎週一回もしくは月一回だけ評価が可能な設定になります。
        日時はすべて UTC で評価されます。

        @param[in] period 再度評価できるようにする曜日もしくは月初めを指定してください。
        @param[in] hour 指定した period の何時にロックを解除するか。
                   -23 以上 23 以下の値を指定できます。マイナスの値を指定すると前日を指定できます。
                   例えば period に DataStoreConstants::RATING_LOCK_PERIOD_DAY1 、hour に -1 を指定した場合は前月末の日の 23 時となります。
    */
    void SetPeriodicLock(DataStoreConstants::RatingLockPeriod period, qInt8 hour)
    {
        SYSTEMCHECK(period < 0);
        SYSTEMCHECK(hour >= -23 && hour <= 23);
        lockType = DataStoreConstants::RATING_LOCK_PERIOD;
        periodDuration = static_cast<qInt16>(period);
        periodHour = hour;
    }

    /*!
        @brief      指定した日数が経過した後の指定した時間まで再度評価できない設定にします。

        days の日数分経過した後最初に hour で指定した時刻になるまで再度評価できない設定になります。
        日時はすべて UTC で評価されます。
        例えば days に 0 を指定し、 hour に 2 を指定したものに対し、現在時刻が 1 時のときに評価したとすると同じ日の 2 時に再度評価できるようになります。
        現在時刻が 3 時のときに評価したとすると翌日の 2 時に再度評価できるようになります。
        つまり毎日 2 時を過ぎるとすべてのユーザーは再度評価できることになります。
        days を増加させるとその日数分ロックされる期間が長くなります。

        @param[in] days 再度評価できるようにする日数。
        @param[in] hour 指定した days 経過後の何時にロックを解除するか。
                   0 以上 23 以下の値を指定できます。
    */
    void SetDaysAfterLock(qInt16 days, qInt8 hour)
    {
        SYSTEMCHECK(days >= 0);
        SYSTEMCHECK(hour >= -23 && hour <= 23);
        lockType = DataStoreConstants::RATING_LOCK_PERIOD;
        periodDuration = days;
        periodHour = hour;
    }

    /*!
        @brief      一度評価すると再度評価できない設定にします。

    */
    void SetPermanentLock()
    {
        lockType = DataStoreConstants::RATING_LOCK_PERMANENT;
        periodDuration = 0;
        periodHour = 0;
    }

    // secret
    DataStoreConstants::RatingLockType GetLockType() const
    {
        return lockType;
    }

    // secret
    qInt8 GetPeriodHour() const
    {
        return periodHour;
    }

    // secret
    qInt16 GetPeriodDuration() const
    {
        return periodDuration;
    }
    /*!
        @brief      DataStoreRatingLockInitParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    inline friend StringStream& operator<<(StringStream& oStr, const DataStoreRatingLockInitParam& oRatingLockInitParam);

    qBool operator==(const DataStoreRatingLockInitParam& rhs) const
    {
        return lockType == rhs.lockType
               && periodHour == rhs.periodHour
               && periodDuration == rhs.periodDuration;
    }
    qBool operator!=(const DataStoreRatingLockInitParam& rhs) const
    {
        return !(*this == rhs);
    }
private:
    DataStoreConstants::RatingLockType lockType;
    qInt8 periodHour;
    qInt16 periodDuration;
};

inline StringStream& operator<<(StringStream& oStr, const DataStoreRatingLockInitParam& oRatingLockInitParam)
{
    oStr <<NEX_T("DataStoreRatingLockInitParam: (")
         << NEX_T("lockType=") << oRatingLockInitParam.lockType
         << NEX_T(", periodHour=") << oRatingLockInitParam.periodHour
         << NEX_T(", periodDuration=") << oRatingLockInitParam.periodDuration
         << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

