﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "DataStoreTouchObjectParamDDL.h"
#include "DataStoreConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      指定したデータのダウンロードをしたフリするためのパラメータを表すクラスです。
*/
class DataStoreTouchObjectParam: public DDLCLASS(DataStoreTouchObjectParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    DataStoreTouchObjectParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~DataStoreTouchObjectParam() {}

    /*!
        @brief      データIDをセットします。

        データのデータIDを指定します。本パラメータは必ず指定する必要があります。

        @param[in]  dataId_ 取得するデータのデータID。
    */
    void SetDataId(qUnsignedInt64 dataId_)
    {
        this->dataId = dataId_;
    }

    /*!
        @brief      データIDを取得します。

        @return     データID。
    */
    qUnsignedInt64 GetDataId() const
    {
        return this->dataId;
    }

    /*!
        @brief      参照パスワードを設定します。

        参照権限を持たないデータを参照する場合に参照パスワードを指定します。
        データのオーナーから受け取った参照パスワードを指定してください。
        参照権限を持っている場合は参照パスワードは無視され、常に参照可能です。
        DataStoreConstants::INVALID_PASSWORD を指定した場合は参照パスワードでの認証を行いません。
        本関数で指定しない場合は DataStoreConstants::INVALID_PASSWORD が使用されます。

        @param[in]  accessPassword_ 参照パスワード
    */
    void SetAccessPassword(qUnsignedInt64 accessPassword_)
    {
        this->accessPassword = accessPassword_;
    }

    /*!
        @brief      参照パスワードを取得します。

        @return     参照パスワード。
    */
    qUnsignedInt64 GetAccessPassword() const
    {
        return this->accessPassword;
    }

    /*
        @internal
        @brief      ロックIDをセットします。

        ダウンロードするデータに対するロックIDを指定します。
        データをアップロードするときに DataStoreConstants::DATA_FLAG_USE_READ_LOCK を指定した場合、
        初めて DataStoreClient::GetObject() で参照されたときにロックIDがサーバーで保存されます。以後はそのロックIDと一致する場合のみダウンロード可能となります。
        例えば誰か一人にデータを渡したいときに利用します。
        本パラメータの指定は必須ではありません。デフォルトは0です。

        @param[in]  lockId データにひも付けるロックID。
    */
    //    void SetLockId(qUnsignedInt32 lockId) { this->lockId = lockId; }

    /*
        @internal
        @brief      ロックIDを取得します。

        @return     ロックID。
    */
    //    qUnsignedInt32 GetLockId() const { return this->lockId; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetDataId(DataStoreConstants::INVALID_DATAID);
        SetAccessPassword(DataStoreConstants::INVALID_PASSWORD);
        // SetLockId(0);
        // 変な値が入るのは困るので0を入れておく
        lockId = 0;
    }

    /*!
        @brief      DataStoreTouchObjectParamオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("DataStoreTouchObjectParam: (")
               << NEX_T("dataId=") << dataId
               << NEX_T(", accessPassword=") << accessPassword
               << NEX_T(")")
              );
    }
};

END_NAMESPACE_NEX

