﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Core/src/Client/RendezVous.h>

#ifdef NEX_SIGLO_FAMILY
namespace nn { namespace account {
    struct UserHandle;
    struct NetworkServiceAccountId;
}}
#include <nn/nn_Result.h>
#include <nn/err/err_Types.h>
#endif

BEGIN_NAMESPACE_NEX

#if !defined(CTR_TARGET) && !defined(CAFE_TARGET) && !defined(NEX_SIGLO_FAMILY)
class WaitableLobbyApi;
#endif

class JobLogin;
class JobTerminate;
class AuthenticationInfo;
class NgsBridgeInterface;

/*!
    @brief ログイン処理を行うクラスです。

    内部で nn::os::Event オブジェクトを一つ使用します。

    NgsFacade オブジェクトが生存している間は定期的に
    @ref Scheduler::Dispatch() を呼び出す必要があります。
*/
class NgsFacade : public RendezVous
{
public:
    /*!
        @brief         コンストラクタです。
    */
    NgsFacade();

    /*!
        @brief         デストラクタです。

        ログイン処理中・ログアウト処理中・ログイン中のいずれかの状態のとき、デストラクタ内で @ref NgsFacade::Terminate() が呼ばれ、
        オフライン状態に遷移するまでブロックします。デストラクタが呼ばれる前に @ref NgsFacade::Terminate() を呼ぶことを推奨します。
    */
    virtual ~NgsFacade();

#ifdef NEX_SIGLO_FAMILY
    /*!
        @refmanoverload{name3}

        @brief         ログイン処理を行います。推奨されません。

        NGS (Nintendo Game Server) へのログイン処理を行います。

        @deprecated nn::account::NetworkServiceAccountId と ネットワークサービスアカウントの ID トークン文字列を引数として指定するオーバーロード関数の使用を推奨します。

        NGS へログインするには事前にネットワークサービスアカウントを作成しておく必要があります。

        本関数を呼び出す前に、 nn::account, nn::nifm, nn::socket ライブラリの初期化処理を行う必要があります。

        本関数は、デバイス認証サーバー、アプリケーション認証サーバー、アカウントサーバーでゲームの認証処理を行った後、NGS へログインします。

        本関数では、@ref ProtocolCallContext::SetTimeout() や @ref ProtocolCallContext::Cancel() で非同期処理をコントロールしないでください。
        タイムアウト時間に関しては引数で指定し、ログイン処理を中断したい場合は @ref NgsFacade::Terminate() を使用してください。

        非同期処理の結果は @ref CallContext::GetState() で成功・失敗の判断を行い、失敗の場合は
        @ref GetLastLoginNnResult() で nn::Result を取得するか、
        @ref GetLastLoginErrorCodeStruct() でエラーコードを取得してください。
        GetLastLoginNnResult(), GetLastLoginErrorCodeStruct() にはアカウントのエラーコードが含まれる可能性があります。
        CallContext::GetOutcome() で結果を取得しないでください。

        ログイン処理が成功すると、NEX ライブラリ内部でゲームサーバーにキープアライブを送信します。
        ゲームサーバーから 30 秒間キープアライブの応答がない場合、自動的にライブラリの内部状態がオフライン状態に遷移します。
        このとき、アプリケーションは終了処理を行い、 @ref NgsFacade::Logout() または @ref NgsFacade::Terminate() を実行してください。
        ライブラリの内部状態は @ref IsConnected() で取得できます。

        ゲームサーバー側でも、30秒間クライアントのキープアライブパケットが受け取れなかった場合にログアウトしたものとして扱います。

        @param[in,out]     pContext 呼び出し情報。
                                このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     gameServerId ゲームサーバ ID。
        @param[in]     pAccessKey ゲームサーバーのアクセスキー。
        @param[in]     userHandle nn::account::OpenUser で取得した nn::account::UserHandle 。
        @param[in]     uiTimeout タイムアウト時間（ミリ秒）。デフォルトは30秒です。

        @retval        "QSUCCESS(Core,Success)"               ログイン処理の起動に成功。
        @retval        "QERROR(Core,InvalidArgument)"         引数が不正。
        @retval        "QERROR(Core,OperationAborted)"        ログイン/ログアウト/ターミネート処理中、またはオンライン状態
    */
    qResult Login(ProtocolCallContext* pContext,
                  qUnsignedInt32 gameServerId,
                  const qChar* pAccessKey,
                  const nn::account::UserHandle& userHandle,
                  TimeInterval uiTimeout = 30000);

    /*!
        @refmanoverload{name4}

        @brief         ログイン処理を行います。

        NGS (Nintendo Game Server) へのログイン処理を行います。

        NGS へログインするには事前にネットワークサービスアカウントを作成しておく必要があります。

        本関数を呼び出す前に、 nn::account, nn::nifm, nn::socket ライブラリの初期化処理を行う必要があります。

        本関数は、指定したネットワークサービスアカウントの ID とネットワークサービスアカウントの ID トークンを使って NGS へログインします。
        ネットワークサービスアカウントの ID や ID トークンの取得方法はサンプルコードや NintendoSDK のドキュメントを参照してください。

        本関数では、@ref ProtocolCallContext::SetTimeout() や @ref ProtocolCallContext::Cancel() で非同期処理をコントロールしないでください。
        タイムアウト時間に関しては引数で指定し、ログイン処理を中断したい場合は @ref NgsFacade::Terminate() を使用してください。

        非同期処理の結果は @ref CallContext::GetState() で成功・失敗の判断を行い、失敗の場合は
        @ref GetLastLoginNnResult() で nn::Result を取得するか、
        @ref GetLastLoginErrorCodeStruct() でエラーコードを取得してください。
        CallContext::GetOutcome() で結果を取得しないでください。

        ログイン処理が成功すると、NEX ライブラリ内部でゲームサーバーにキープアライブを送信します。
        ゲームサーバーから 30 秒間キープアライブの応答がない場合、自動的にライブラリの内部状態がオフライン状態に遷移します。
        このとき、アプリケーションは終了処理を行い、 @ref NgsFacade::Logout() または @ref NgsFacade::Terminate() を実行してください。
        ライブラリの内部状態は @ref IsConnected() で取得できます。

        ゲームサーバー側でも、30秒間クライアントのキープアライブパケットが受け取れなかった場合にログアウトしたものとして扱います。

        @param[in,out]     pContext 呼び出し情報。
                                このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     gameServerId ゲームサーバ ID。
        @param[in]     pAccessKey ゲームサーバーのアクセスキー。
        @param[in]     nsaId ネットワークサービスアカウントの ID
        @param[in]     pNsaIdToken NULL 終端されたネットワークサービスアカウントの ID トークン
        @param[in]     uiTimeout タイムアウト時間（ミリ秒）。デフォルトは30秒です。

        @retval        "QSUCCESS(Core,Success)"               ログイン処理の起動に成功。
        @retval        "QERROR(Core,InvalidArgument)"         引数が不正。
        @retval        "QERROR(Core,OperationAborted)"        ログイン/ログアウト/ターミネート処理中、またはオンライン状態
    */
    qResult Login(ProtocolCallContext* pContext,
                  qUnsignedInt32 gameServerId,
                  const qChar* pAccessKey,
                  const nn::account::NetworkServiceAccountId& nsaId,
                  const qChar8* pNsaIdToken,
                  TimeInterval uiTimeout = 30000);
#endif

#if defined(CTR_TARGET)
    /*!
        @refmanoverload{name1}

        @brief         ログイン処理を行います。

        NGS (Nintendo Game Server) へのログイン処理を行います。

        NGS へログインするには事前に NFS (Nintendo Friend Server) へのログイン処理が成功している必要があります。
        NFS へのログインについては、 nn::friends のマニュアルを参照してください。

        本関数は、認証サーバーでゲームの認証処理を行った後、NGS へログインします。

        本関数では、@ref ProtocolCallContext::SetTimeout() や @ref ProtocolCallContext::Cancel() で非同期処理をコントロールしないでください。
        タイムアウト時間に関しては引数で指定し、ログイン処理を中断したい場合は @ref NgsFacade::Terminate() を使用してください。

        非同期処理の結果は @ref CallContext::GetState() で成功・失敗の判断を行い、失敗の場合は
        @ref GetLastLoginErrorCode() でネットワークエラーコードを取得してください。
        CallContext::GetOutcome() で結果を取得しないでください。

        ログイン処理が成功すると、NEX ライブラリ内部でゲームサーバーにキープアライブを送信します。
        ゲームサーバーから 30 秒間キープアライブの応答がない場合、自動的にライブラリの内部状態がオフライン状態に遷移します。
        このとき、アプリケーションは終了処理を行い、 @ref NgsFacade::Logout() または @ref NgsFacade::Terminate() を実行してください。
        ライブラリの内部状態は @ref IsConnected() で取得できます。

        ゲームサーバー側でも、30秒間クライアントのキープアライブパケットが受け取れなかった場合にログアウトしたものとして扱います。

        @param[in,out]     pContext    呼び出し情報。
                                   このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     gameServerId ゲームサーバ ID。
        @param[in]     pAccessKey  ゲームサーバーのアクセスキー。
        @param[in]     uiTimeout   タイムアウト時間（ミリ秒）。デフォルトは30秒です。
        @param[in]     principalId INVALID_PRINCIPALID（デフォルト） または、本体の プリンシパルID を指定してください。

        @retval        "QSUCCESS(Core,Success)"               ログイン処理の起動に成功。
        @retval        "QERROR(Core,InvalidArgument)"         引数が不正。
        @retval        "QERROR(Core,OperationAborted)"        ログイン/ログアウト/ターミネート処理中、またはオンライン状態
        @retval        "QERROR(FPD,NotInitialized)"           フレンドライブラリが初期化されていない。
        @retval        "QERROR(FPD,NotConnected)"             NFS にログインしていない
        @retval        "QERROR(FPD,InvalidPrincipalID)"       引数で与えた principalId が INVALID_PRINCIPALID とも本体の プリンシパルID とも一致しない。
    */
#elif defined(CAFE_TARGET)
    /*!
        @refmanoverload{name1}

        @brief         ログイン処理を行います。

        NGS (Nintendo Game Server) へのログイン処理を行います。

        NGS へログインするには事前にネットワークアカウントを作成しておく必要があります。
        ネットワークアカウントについては「Wii U アカウントシステム デベロッパーズガイド」を参照してください。<br>

        本関数を呼び出す前に、 nn::act, nn::fp, nn::ac, SO ライブラリの初期化処理を行う必要があります。

        本関数は、アカウントサーバーでゲームの認証処理を行った後、NGS へログインします。認証処理用のスレッドが１本起動します(認証処理が完了後にスレッドは破棄されます)。

        本関数では、@ref ProtocolCallContext::SetTimeout() や @ref ProtocolCallContext::Cancel() で非同期処理をコントロールしないでください。
        タイムアウト時間に関しては引数で指定し、ログイン処理を中断したい場合は @ref NgsFacade::Terminate() を使用してください。

        ただし、認証処理用のスレッドは、タイムアウト時間を超えても認証処理が完了するまで動作し続けます。 NgsFacade のデストラクタは
        認証処理用のスレッドが終了するまでブロックしますので、タイムアウト発生時やシャットダウン時には nn::act::Cancel() を呼び出し、
        認証処理を明示的にキャンセルすることを推奨します。認証処理中にタイムアウトが発生したかどうかは、
        非同期処理の結果が QERROR(Core,Timeout) かどうかで判断を行ってください。

        非同期処理の結果は @ref CallContext::GetState() で成功・失敗の判断を行い、失敗の場合は
        @ref GetLastLoginErrorCode() でネットワークエラーコードを取得してください。
        CallContext::GetOutcome() で結果を取得しないでください。

        ログイン処理が成功すると、NEX ライブラリ内部でゲームサーバーにキープアライブを送信します。
        ゲームサーバーから 30 秒間キープアライブの応答がない場合、自動的にライブラリの内部状態がオフライン状態に遷移します。
        このとき、アプリケーションは終了処理を行い、 @ref NgsFacade::Logout() または @ref NgsFacade::Terminate() を実行してください。
        ライブラリの内部状態は @ref IsConnected() で取得できます。

        ゲームサーバー側でも、30秒間クライアントのキープアライブパケットが受け取れなかった場合にログアウトしたものとして扱います。

        アプリケーションの Unique ID が適切に設定されていない場合、認証処理でエラーが発生してゲームサーバーにログインできません。
        「Wii U アプリケーション設定ツール」を使用して Unique ID を設定してください。サンプルデモ用サーバーを使用する場合は、
        SDK に付属するデフォルトの app.xml をそのまま使用して下さい。

        @param[in,out]     pContext    呼び出し情報。
                                   このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     gameServerId ゲームサーバ ID。
        @param[in]     pAccessKey  ゲームサーバーのアクセスキー。
        @param[in]     uiTimeout   タイムアウト時間（ミリ秒）。デフォルトは30秒です。
        @param[in]     principalId INVALID_PRINCIPALID（デフォルト） または、本体の プリンシパルID を指定してください。

        @retval        "QSUCCESS(Core,Success)"               ログイン処理の起動に成功。
        @retval        "QERROR(Core,InvalidArgument)"         引数が不正。
        @retval        "QERROR(Core,OperationAborted)"        ログイン/ログアウト/ターミネート処理中、またはオンライン状態
        @retval        "QERROR(FPD,InvalidPrincipalID)"       引数で与えた principalId が INVALID_PRINCIPALID とも本体の プリンシパルID とも一致しない。
        @retval        "QERROR(FPD,NotInitialized)"           アカウントライブラリ・フレンドライブラリが初期化されていない。
    */
#endif
    qResult Login(ProtocolCallContext* pContext,
                  qUnsignedInt32 gameServerId,
                  const qChar* pAccessKey,
                  TimeInterval uiTimeout = 30000,
                  PrincipalID principalId = INVALID_PRINCIPALID);

#if defined(CTR_TARGET)
    /*!
        @brief         ログイン処理と独自サーバーの認証トークンの要求を行います。

        本関数は、 @ref Login() と、 @ref IndependentServer::RequestAuthenticationToken() の処理をひとまとめにして行う関数です。
        詳しくは、各関数のリファレンスを参照して下さい。

        非同期処理の結果は @ref CallContext::GetState() で成功・失敗の判断を行い、失敗の場合は
        @ref GetLastLoginErrorCode() でネットワークエラーコードを取得してください。
        CallContext::GetOutcome() で結果を取得しないでください。

        @param[in,out]     pContext     呼び出し情報。
                                    このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     gameServerId ゲームサーバ ID。
        @param[in]     pAccessKey   ゲームサーバーのアクセスキー。
        @param[in]     pKeyHash     キーハッシュ値。
        @param[out]    pAuthToken   認証トークンを受け取るバッファ。@n
                                    バッファサイズは @ref IndependentServer::AUTH_TOKEN_SIZE 以上にして下さい。
        @param[in]     uiTimeout    タイムアウト時間（ミリ秒）。デフォルトは30秒です。
        @param[in]     principalId  INVALID_PRINCIPALID（デフォルト） または、本体の プリンシパルID を指定してください。

        @retval        "QSUCCESS(Core,Success)"               ログイン処理の起動に成功。
        @retval        "QERROR(Core,InvalidArgument)"         引数が不正。
        @retval        "QERROR(Core,OperationAborted)"        ログイン/ログアウト/ターミネート処理中、またはオンライン状態
        @retval        "QERROR(FPD,NotInitialized)"           フレンドライブラリが初期化されていない。
        @retval        "QERROR(FPD,NotConnected)"             NFS にログインしていない
        @retval        "QERROR(FPD,InvalidPrincipalID)"       引数で与えた principalId が INVALID_PRINCIPALID とも本体の プリンシパルID とも一致しない。
    */
#endif
    qResult LoginAndRequestAuthenticationToken(ProtocolCallContext* pContext,
            qUnsignedInt32 gameServerId,
            const qChar* pAccessKey,
            const qChar8* pKeyHash,
            qChar8* pAuthToken,
            TimeInterval uiTimeout = 30000,
            PrincipalID principalId = INVALID_PRINCIPALID);

#if defined(CTR_TARGET) && defined(NEX_ENABLED_CTR_NNID)
    /*!
        @refmanoverload{name2}

        @brief         NEX 向け認証情報を用いてログイン処理を行います。

        nn::act::AcquireNexAuthenticationData() で取得したNEX 向け認証情報を使用して NGS (Nintendo Game Server) へのログイン処理を行います。
        nn::act::AcquireNexAuthenticationData() については、nn::act のマニュアルを参照してください。
        本 API を利用する時には、libJugemLoginClient　の代わりに libJugemLoginClientNNID をリンクするようにしてください。

        非同期処理の結果として QERROR(Authentication, TokenExpired) が返される事があります。
        この場合はトークンの有効期限が切れているため、 nn::act::applet::AcquireNexAuthenticationData() からやり直してください。

        注意事項やログイン後の動作については、同名関数の説明を参照してください。

        @param[in,out]     pContext    呼び出し情報。
                                   このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     pNexAuthData NEXサービストークンを含む認証結果が入ったバッファ(nn::act::NexAuthenticationData* 型)
        @param[in]     pAccessKey  ゲームサーバーのアクセスキー。
        @param[in]     uiTimeout   タイムアウト時間（ミリ秒）。デフォルトは30秒です。
        @param[in]     principalId INVALID_PRINCIPALID（デフォルト） または、本体の プリンシパルID を指定してください。

        @retval        "QSUCCESS(Core,Success)"               ログイン処理の起動に成功。
        @retval        "QERROR(Core,InvalidArgument)"         引数が不正。
        @retval        "QERROR(Core,OperationAborted)"        ログイン/ログアウト/ターミネート処理中、またはオンライン状態
        @retval        "QERROR(FPD,InvalidPrincipalID)"       引数で与えた principalId が INVALID_PRINCIPALID とも本体の プリンシパルID とも一致しない。
        @retval        "QERROR(FPD,NotInitialized)"           アカウントライブラリが初期化されていない。
        @retval        "QERROR(Core, NotImplemented)"         NNID 対応ライブラリ( libJugemLoginClientNNID )をリンクしていない。

    */
#endif
    qResult Login(ProtocolCallContext* pContext,
                  const void* pNexAuthData,
                  const qChar* pAccessKey,
                  TimeInterval uiTimeout = 30000,
                  PrincipalID principalId = INVALID_PRINCIPALID);

private:
    /*
        @brief         ログイン処理の実体です。 各 Login 処理が呼び出します。

                       独自サーバーでは、pKeyHash, pAuthToken を指定します。
                       NNIDトークンで NGS へのログインのみする場合は、pNnidToken を指定します(gameId は無視されます)。
                       NSA-IDトークンで NGS へのログインをする場合は、pNsaIdToken を指定します
                       使用しないパラメタは、 NULL もしくは 0 を指定します。
    */
    qResult LoginAndRequestAuthenticationTokenImpl(ProtocolCallContext* pContext, qUnsignedInt32 gameId, const qChar* pAccessKey,
        const qChar8* pKeyHash, qChar8* pAuthToken, const void* pNnidToken, const void* pNsaIdToken,
        TimeInterval uiTimeout, PrincipalID principalId, const void* userHandle);

public:

    /*!
        @internal
        @brief         アカウント作成を行います。

                       接続先を指定してアカウントの作成処理を行います。
                       本関数は開発用の関数です。製品版では使用しないでください。

        @param[in,out]     pContext   呼び出し情報。
                                  このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     pHostName  サーバーのホスト名。
        @param[in]     port       サーバーのポート番号。
        @param[in]     pUserName  アカウントのユーザー名。
        @param[in]     pPassword  アカウントのパスワード。
        @param[in]     pAccessKey サーバーのアクセスキー。
        @param[in]     uiTimeout  タイムアウト時間。
                                  デフォルトは30秒です。

        @return        処理の結果が返ります。
    */
    qResult DebugCreateAccount(ProtocolCallContext* pContext, const qChar* pHostName, qUnsignedInt16 port, const qChar* pUserName,
                               const qChar8* pPassword, const qChar* pAccessKey, TimeInterval uiTimeout = 30000);

    /*!
        @internal
        @brief         アカウント作成を行います。

                       接続先を指定してアカウントの作成処理を行います。
                       本関数は開発用の関数です。製品版では使用しないでください。

        @param[in,out]     pContext   呼び出し情報。
                                  このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     pUri       接続先の URI
        @param[in]     pUserName  アカウントのユーザー名。
        @param[in]     pPassword  アカウントのパスワード。
        @param[in]     pAccessKey サーバーのアクセスキー。
        @param[in]     uiTimeout  タイムアウト時間。
                                  デフォルトは30秒です。

        @return        処理の結果が返ります。
    */
    qResult DebugCreateAccount(ProtocolCallContext* pContext, const qChar* pUri, const qChar* pUserName,
                               const qChar8* pPassword, const qChar* pAccessKey, TimeInterval uiTimeout = 30000);

    /*!
        @internal
        @brief         ログイン処理を行います。

                       接続先を指定してログイン処理を行います。
                       本関数は開発用の関数です。製品版では使用しないでください。

        @param[in,out]     pContext   呼び出し情報。
                                  このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     pHostName  サーバーのホスト名。
        @param[in]     port       サーバーのポート番号。
        @param[in]     pUserName  ユーザー名。
        @param[in]     pPassword  パスワード。
        @param[in]     pAccessKey サーバーのアクセスキー。
        @param[in]     uiTimeout  タイムアウト時間。<br>
                                  デフォルトは30秒です。

        @retval "QSUCCESS(Core,Success)" 非同期処理の開始に成功
        @retval "QERROR(Core,CallInitiationFailure)"  pContext が NULL もしくはすでに進行中状態のものだったため非同期処理の開始に失敗。
    */
    qResult DebugLoginDirect(ProtocolCallContext* pContext, const qChar* pHostName, qUnsignedInt16 port, const qChar* pUserName,
                             const qChar8* pPassword, const qChar* pAccessKey, TimeInterval uiTimeout = 30000);

#ifdef NEX_SIGLO_FAMILY
    /*!
        @brief         ログイン処理を行います。

                       接続先を指定してログイン処理を行います。
                       本関数は開発用の関数です。製品版では使用しないでください。
                       非同期処理の結果は
                       @ref GetLastLoginNnResult() で nn::Result を取得するか、
                       @ref GetLastLoginErrorCodeStruct() でエラーコードを取得してください。
                       NgsFacade::Login() と異なり、CallContext::GetOutcome() でも結果を取得可能です。

        @param[in,out]     pContext   呼び出し情報。
                                  このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。
        @param[in]     pUri       接続先の URI
        @param[in]     pUserName  ユーザー名。
        @param[in]     pPassword  パスワード。
        @param[in]     pAccessKey サーバーのアクセスキー。
        @param[in]     uiTimeout  タイムアウト時間。
                                  デフォルトは30秒です。

        @retval "QSUCCESS(Core,Success)" 非同期処理の開始に成功
        @retval "QERROR(Core,CallInitiationFailure)"  pContext が NULL もしくはすでに進行中状態のものだったため非同期処理の開始に失敗。
    */
#endif
    qResult DebugLoginDirect(ProtocolCallContext* pContext, const qChar* pUri, const qChar* pUserName,
                             const qChar8* pPassword, const qChar* pAccessKey, TimeInterval uiTimeout = 30000);

    /*!
        @brief         ログアウト処理を行います。

        本関数を呼ぶ前に @ref ServiceClient 継承クラス（ @ref MatchmakeExtensionClient , @ref NATTraversalClient など）の
        Unbind() および delete を実行してください。

        @param[in,out]     pContext 呼び出し情報。
                                このオブジェクトでタイムアウト時間の設定やキャンセルを行わないで下さい。

        @return        処理の結果が返ります。
        @retval        "QSUCCESS(Core,Success)"   ログアウト処理の起動に成功。
        @retval        "QERROR(Core,OperationAborted)"  ログイン処理が実行中
        @retval        "QERROR(Core,CallInitiationFailure)"  上記以外の失敗（別のログアウト処理が実行中など）
    */
    qResult Logout(ProtocolCallContext* pContext);

    /*!
        @brief        インターネットとの接続性テストを行います。

        ゲームサーバーと通信することによりインターネットとの接続性テストを行います。
        正常に稼動しているゲームサーバーと通信できる場合は、本関数の非同期処理は必ず成功を返します。
        本関数の非同期処理の完了を待つ必要はなく、アプリケーションで指定した時間（2, 3秒程度）で
        非同期処理が完了しなかった場合は、接続性テストに失敗したと判定して構いません。

        本関数を定期的に呼び出すことは禁止されています。クライアントが P2P 通信時に相手との通信が
        出来なくなった場合に、切断の理由が相手クライアント側にあるのか、自分にあるのかを確認するために使用してください。
        サーバーとの通信状態が維持されているかを確認する場合は @ref NgsFacade::IsConnected() を使用し、
        アクセスポイントとの接続が維持されているかを確認する場合は  ac&#58&#58IsConnected()(または切断イベントのハンドリング)
         を使用してください。

        ゲームサーバーにログインしていない場合は使用できません。

        @param[in,out]     pContext 呼び出し情報。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool TestConnectivity(ProtocolCallContext* pContext);

    /*!
        @internal
        @brief         ログイン処理中かどうかを取得します。

        @return        ログイン処理中ならtrue。
    */
    qBool LoginJobIsInProgress() const;

    /*!
        @internal
        @brief         Terminate中かどうかを取得します。

        @return        Terminate 処理中ならtrue。
    */
    qBool TerminateJobIsInProgress() const;

    /*!
        @brief         前回のログインでエラーが発生したかどうかを取得します。

        本関数はログイン処理が完了後もしくは試行前に呼び出すことができます。ログイン処理中は呼び出すことができません。

        @retval true    前回のログインでエラーが発生した。
        @retval false   前回のログインでエラーが発生していない。もしくはログインを試行していない。
        @see  ResetLastLoginError()
    */
    qBool HasErrorOccuredLastLogin() const
    {
        return !static_cast<qBool>(m_lastLoginResult);
    }

    /*!
        @brief         前回のログインでエラーが発生したかどうかの情報をクリアします。

        本関数を実行後は @ref HasErrorOccuredLastLogin() が false を返します。
        NgsFacade::Login() を呼び出すと自動的に本関数が呼び出されます。

        @see  HasErrorOccuredLastLogin()
    */
    void ResetLastLoginError()
    {
        m_lastLoginResult = QRESULT_SUCCESS(Core, Success);
    }
#ifdef NEX_SIGLO_FAMILY
    /*!
        @brief         ログイン処理で発生したエラーコードを取得します。
                       NgsFacade::Login() の非同期処理に失敗した場合にのみ有効な値がセットされます。
                       本関数を呼び出せるかどうかは @ref HasErrorOccuredLastLogin() で取得できます。

        @return        ログイン時のエラーコード。
        
        @see  GetLastLoginNnResult(), GetLastLoginErrorCode()
    */
    nn::err::ErrorCode GetLastLoginErrorCodeStruct() const;

    /*!
        @brief         ログイン処理で発生した nn::Result を取得します。
                       NgsFacade::Login() の非同期処理に失敗した場合にのみ有効な値がセットされます
                       本関数を呼び出せるかどうかは @ref HasErrorOccuredLastLogin() で取得できます。

        @return        ログイン時の nn::Result 。

        @see  GetLastLoginErrorCodeStruct(), GetLastLoginErrorCode()
    */
    nn::Result GetLastLoginNnResult() const;


    /*!
        @brief         ログイン処理で発生したエラーコードを取得します(デバッグ用)。
                       NgsFacade::Login() の非同期処理に失敗した場合にのみ有効な値がセットされます。
                       本関数を呼び出せるかどうかは @ref HasErrorOccuredLastLogin() で取得できます。
                       エラービューワーに渡すエラーコードは、 NgsFacade::GetLastLoginErrorCodeStruct() もしくは、 NgsFacade::GetLastLoginNnResult() を利用してください。

        @return        ログイン時のエラーコード。

        @see GetLastLoginErrorCodeStruct(), GetLastLoginNnResult()
    */
#else
    /*!
        @brief         ログイン処理で発生したネットワークエラーコードを取得します。
                       NgsFacade::Login() の非同期処理に失敗した場合にのみ有効な値がセットされます
                       本関数を呼び出せるかどうかは @ref HasErrorOccuredLastLogin() で取得できます。

        @return        ログイン時のネットワークエラーコード。
    */
#endif
    qInt32 GetLastLoginErrorCode() const { return GetLastLoginErrorCodeInner(); }

    /*!
        @brief         Pia と連携するための Interface を取得します。

        @return        NgsBridgeInterface のポインタ

        @see           NgsBridgeInterface
    */
    NgsBridgeInterface* GetNgsBridge() const
    {
        return m_pNgsBridgeInterface;
    }

    /*!
        @internal
        @brief         NASエラーコードを取得します。

        @return        NASエラーコード。
    */
    qInt32 GetNasErrorCode() const;

    /*!
        @internal
        @brief         HTTPステータスコードを取得します。

        @return        HTTPステータスコード。
    */
    qInt32 GetHttpStatusCode() const;

    /*!
        @internal
        @brief         サーバーのバージョンが古くても接続を許可するように設定します。

        @param[in]  ignoreServerVersion   trueならサーバーのバージョンが古い場合でも接続します。
                                          falseなら接続を拒否します。デフォルトでは接続を拒否します。
    */
    void IgnoreServerVersion(qBool ignoreServerVersion)
    {
        m_ignoreServerVersion = ignoreServerVersion;
    }

    /*!
        @internal
        @brief         サーバーのバージョンが古くても接続を許可するかどうかの設定を取得します。

        @return        サーバーのバージョンが古くても接続を許可するかどうか
    */
    qBool IsIgnoreServerVersion() const
    {
        return m_ignoreServerVersion;
    }

#if !defined(CTR_TARGET) && !defined(CAFE_TARGET) && !defined(NEX_SIGLO_FAMILY)
    void SetWaitableLobbyApi(WaitableLobbyApi* pApi)
    {
        m_pWaitableLobbyApi = pApi;
    }
    WaitableLobbyApi* GetWaitableLobbyApi()
    {
        return  m_pWaitableLobbyApi;
    }
#endif

    void SetNasErrorCode(qInt32 code)
    {
        m_nasReturnCode = code;
    }
    void SetHttpStatusCode(qInt32 code)
    {
        m_httpStatusCode = code;
    }
    void SetLastLoginErrorCode(qInt32 code)
    {
        m_lastLoginErrorCode = code;
    }
    void SetLastLoginResult(qResult lastLoginResult)
    {
        m_lastLoginResult = lastLoginResult;
    }

    void SetIgnoreNFSLoginCheck(qBool flag)
    {
        m_ignoreNFSLoginCheck = flag;
    }
    qBool GetIgnoreNFSLoginCheck()
    {
        return m_ignoreNFSLoginCheck;
    }

    void SetIgnorePidCheck(qBool flag)
    {
        m_ignorePidCheck = flag;
    }
#ifdef NEX_SIGLO_FAMILY
    void SetLastLoginNnResult(const nn::Result& result)
    {
        m_lastLoginNnResult = result;
    }
#endif

    void SetHost(const String& host)
    {
        m_host = host;
    }

    const String& GetHost() const
    {
        return m_host;
    }

    void SetPort(qUnsignedInt16 port)
    {
        m_port = port;
    }

    qUnsignedInt16 GetPort() const
    {
        return m_port;
    }

    void SetUri(const String& uri)
    {
        m_uri = uri;
    }

    const String& GetUri() const
    {
        return m_uri;
    }

// サンプルデモサーバーのゲームサーバーID
// 各プラットフォームのNexUtil.hの定義と一致している必要があります。
#if defined(NEX_SIGLO_FAMILY)
    static const qUnsignedInt32 SAMPLE_DEMO_GAME_SERVER_ID = 0x27851C01;
#elif defined(CTR_TARGET)
    static const qUnsignedInt32 SAMPLE_DEMO_GAME_SERVER_ID = 0x25E5BE00;
#elif defined(CAFE_TARGET)
    static const qUnsignedInt32 SAMPLE_DEMO_GAME_SERVER_ID = 0x1F940003;
#else
    static const qUnsignedInt32 SAMPLE_DEMO_GAME_SERVER_ID = 0x1F940003;
#endif

protected:
    friend class JobLogin;
    friend class JobTerminate;

protected:
    // Credentialsオブジェクト
    Credentials* m_pCredentials;
    // Jobオブジェクト
    JobLogin* m_pJobNgsLogin;
    JobTerminate* m_pJobNgsTerminate;

    NgsBridgeInterface* m_pNgsBridgeInterface;

    // Terminateコールバック関数
    static void OnTerminate(CallContext* pContext, const UserContext* pUserContext);
    // NgsFacade独自のTerminate関数
    void TerminateImpl(CallContext* pContext);

    // ログイン処理用のパラメータを設定する
    void SetLoginParameters(const qChar* pAccessKey, TimeInterval uiTimeout);
    // ログイン処理時のエラーコードを取得する内部関数
    qInt32 GetLastLoginErrorCodeInner() const;

    // NASエラーコード
    qInt32 m_nasReturnCode;
    // HTTPステータスコード
    qInt32 m_httpStatusCode;
    // JobLogin時の結果
    qResult m_lastLoginResult;
    // エラーコード
    qInt32 m_lastLoginErrorCode;
    // サーバーのバージョン確認を無視するか
    qBool m_ignoreServerVersion;
    // FPDへのログインを不要とするか
    qBool m_ignoreNFSLoginCheck;

    // NNIDテスト用
    qBool m_ignorePidCheck;

#ifdef NEX_SIGLO_FAMILY
    // nn::Resultの結果
    nn::Result m_lastLoginNnResult;
#endif
    // 特定のサーバに接続したい場合に使用する設定。テスト、デバッグ向け
    String m_host;
    qUnsignedInt16 m_port;
    String m_uri;

    CustomDataHolder<AuthenticationInfo> m_pTokenHolder;

#if !defined(CTR_TARGET) && !defined(CAFE_TARGET) && !defined(NEX_SIGLO_FAMILY)
    WaitableLobbyApi* m_pWaitableLobbyApi;
#endif
};

END_NAMESPACE_NEX

