﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Core/src/Common/Constants/ServiceID.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeExtensionProtocolDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeSession.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeSessionSearchCriteria.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/AutoMatchmakeParam.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/CreateMatchmakeSessionParam.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/JoinMatchmakeSessionParam.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/Community.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/FindMatchmakeSessionByParticipantParam.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/FindMatchmakeSessionByParticipantResult.h>
#include <RendezVous/Services/MatchMaking/src/Client/MatchMakingClient.h>

BEGIN_NAMESPACE_NEX

class KeyedChecksumAlgorithm;

/*!
@internal
@brief      MatchmakeSessionをサーバーとやり取りするためのHolderクラスです
*/
typedef CustomGatheringHolder<MatchmakeSession> MatchmakeSessionHolder;

/*!
    @brief マッチメイク機能を利用するためのクラス。
*/
class MatchmakeExtensionClient : public MatchMakingClient
{
public:

    /*!
    @brief      コンストラクタです。

    コンストラクタです。
    */
    MatchmakeExtensionClient();

    /*!
    @brief      デストラクタです。

    デストラクタです。
    */
    virtual ~MatchmakeExtensionClient();

    /*!
        @brief    サービスを利用するための証明書を登録します。

        サービスを利用するための証明書をセットします。
        機能を利用する前にセットする必要があります。@ref NgsFacade::Login() の非同期処理に成功後、
        @ref NgsFacade::GetCredentials() で取得した証明書を引数に指定してください。
        @ref Bind() で証明書をセットした後もう一度 @ref Bind() を呼び出すと証明書を差し替える事ができます。

        @param[in]     pCredentials    サービスにアクセスするための証明書。

        @retval     true 成功。pCredentials が NULL でない場合は必ず成功します。
        @retval     false pCredentials に NULL が指定された。
    */
    virtual qBool Bind(Credentials* pCredentials);

    /*!
        @brief     証明書の登録を解除します。

        @ref Bind() でセットされた証明書を解除します。@ref Bind() が呼び出されていなくても本関数を呼び出すことができます。
        本オブジェクトを破棄する際に自動的に @ref Unbind() が呼び出されるため、明示的に呼び出す必要はありません。

        @retval      true 証明書の解除を行った。
        @retval      false Bind() が行われていないため何も行わなかった。
    */
    virtual qBool Unbind();

    /*!
    @refmanoverload{AutoMatchmakeWithParam}

    @brief      自動的にマッチメイクセッションを検索し、参加します。

    指定した AutoMatchmakeParam の内容に応じてマッチメイクセッション(ギャザリング)を検索し、一致するものが見つかればそこに参加します。
    見つからなかった場合は新たにマッチメイクセッションを作り、そこに参加、さらに自分がホストになります。

    %AutoMatchmake() のオーバーロード関数の内本関数のみ複数人のマッチメイクセッションへの参加が行えます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(RendezVous, NotParticipatedGathering) : 追加で参加させようとしたユーザーが AutoMatchmakeParam::SetGatheringIdForParticipationCheck() で指定したギャザリングに参加していない。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。
                                           マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                           0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。
    @li    QERROR(RendezVous, OutOfRatingRange) : MatchmakeSessionSearchCriteria::SetSelectionMethod () で MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE もしくは MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE_WITH_PROGRESS_SCORE を指定したとき、 インデックス MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX の属性値がレンジ外。
    @li    QERROR(RendezVous, LimitExceeded) : 開発環境で、ギャザリング作成時に自分がオーナーのマッチメイクセッションが 20 個すでに存在していた場合に発生します。
                                                このエラーはあくまでクライアントのバグを検出したりシステムを保護したりするのを目的に、開発環境でのみ発生するものです。
                                                このエラーが返ってくることを前提として作成するマッチメイクセッションの個数を制御したりしないでください。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     autoMatchmakeParam    オートマッチメイク処理に関するパラメータ
    @param[out]    joinedMatchmakingSession    参加したマッチメイクセッション。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AutoMatchmake(ProtocolCallContext *pContext,
                        const AutoMatchmakeParam& autoMatchmakeParam,
                        MatchmakeSession* joinedMatchmakingSession);



    /*! @internal
    @refmanoverload{AutoMatchmake}

    @brief      自動的にマッチメイクセッションを検索し、参加します。推奨されません。

    @deprecated AutoMatchmakeParam を引数に取るオーバーロード関数の使用を推奨します。

    指定した マッチメイクセッション情報からマッチメイクセッション(ギャザリング)を検索し、一致するものが見つかればそこに参加します。
    見つからなかった場合は新たにマッチメイクセッションを作り、そこに参加、さらに自分がホストになります。

    @li ゲームモード
    @li 最小・最大参加人数
    @li マッチメイクシステムタイプ
    @li 検索属性

    の全てが一致するものでかつ、参加可能なものを検索します。

    但し、NAT トラバーサルができない等の理由で P2P 接続が確立できなかったにもかかわらず、オートマッチの結果同じマッチメイクセッションに
    何度も参加してしまう問題を避けるため、過去3回にわたって参加したマッチメイクセッションに関して、サーバー側で参加対象から外しています。
    サーバー側の参加記録は約1時間で無効になります。自分がマッチメイクセッションの作成者の場合は参加記録に含まれません。

    また P2P 通信のホストが決定していないものも参加対象から除外されます。
    P2P 通信は行わない場合など、ホストが未決定なマッチメイクセッションを参加対象に含めたい場合は MatchmakeSessionSearchCriteria を引数に取るオーバーロード関数を利用し、
    MatchmakeSessionSearchCriteria::SetExcludeNonHostPid(false) を呼び出してください。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。
                                           マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                           0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。
    @li    QERROR(RendezVous, LimitExceeded) : 開発環境で、ギャザリング作成時に自分がオーナーのマッチメイクセッションが 20 個すでに存在していた場合に発生します。
                                               このエラーはあくまでクライアントのバグを検出したりシステムを保護したりするのを目的に、開発環境でのみ発生するものです。
                                               このエラーが返ってくることを前提として作成するマッチメイクセッションの個数を制御したりしないでください。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     matchmakingSession    検索、作成するマッチメイクセッション。この情報の内、GameMode, Attribute, MatchmakeSystemTypeが一致するものを検索します。
                                         一致するものが見つからなかった場合、この情報のギャザリングが作成されます。
    @param[out]    joinedMatchmakingSession    参加したマッチメイクセッション。
    @param[in]     strMessage   ギャザリングのオーナーもしくは参加者に通知されるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AutoMatchmake(ProtocolCallContext *pContext,
                        const GatheringHolder& matchmakingSession,
                        GatheringHolder* joinedMatchmakingSession,
                        const String strMessage = NEX_T(""));

    /*! @internal
    @refmanoverload{AutoMatchmakeWithSearchCriteria}

    @brief      自動的にマッチメイクセッションを検索し、参加します。推奨されません。

    @deprecated AutoMatchmakeParam を引数に取るオーバーロード関数の使用を推奨します。

    指定した マッチメイクセッション検索条件からマッチメイクセッション(ギャザリング)を検索し、一致するものが見つかればそこに参加します。
    検索条件に合致するものが存在しない場合、指定したマッチメイクセッション情報から新たにマッチメイクセッションを作り、そこに参加、
    さらに自分がホストになります。

    但し、NAT トラバーサルができない等の理由で P2P 接続が確立できなかったにもかかわらず、オートマッチの結果同じマッチメイクセッションに
    何度も参加してしまう問題を避けるため、過去3回にわたって参加したマッチメイクセッションに関して、サーバー側で参加対象から外しています。
    サーバー側の参加記録は約1時間で無効になります。自分がマッチメイクセッションの作成者の場合は参加記録に含まれません。

    検索条件はリストとして複数渡すことができ、ゲームサーバーはリストの最初から順番に検索を行います（参加できた段階で検索は打ち切られます）。
    したがって条件を徐々に緩めながらマッチメイクセッションを検索していくことが可能です。

    検索条件のうち、参加がロックされているもの、すでに満員のものは自動的に検索対象外になります。
    検索条件は最大 2 つまで指定できます。3 つ以上指定した場合は QERROR(Core, InvalidArgument) となります。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。
                                           マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                           0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。
    @li    QERROR(RendezVous, OutOfRatingRange) : MatchmakeSessionSearchCriteria::SetSelectionMethod () で MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE もしくは MATCHMAKE_SELECTION_METHOD_BROADEN_RANGE_WITH_PROGRESS_SCORE を指定したとき、 インデックス MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX の属性値がレンジ外。
    @li    QERROR(RendezVous, LimitExceeded) : 開発環境で、ギャザリング作成時に自分がオーナーのマッチメイクセッションが 20 個すでに存在していた場合に発生します。
                                               このエラーはあくまでクライアントのバグを検出したりシステムを保護したりするのを目的に、開発環境でのみ発生するものです。
                                               このエラーが返ってくることを前提として作成するマッチメイクセッションの個数を制御したりしないでください。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     searchCriteriaList    検索条件。リストの最初から順番に検索します。
    @param[in]     matchmakingSession    検索条件に合致するものがなかったときに、作成するマッチメイクセッション。
    @param[out]    joinedMatchmakingSession    参加したマッチメイクセッション。
    @param[in]     strMessage   ギャザリングのオーナーもしくは参加者に通知されるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AutoMatchmake(ProtocolCallContext *pContext,
                        const qList<MatchmakeSessionSearchCriteria>& searchCriteriaList,
                        const GatheringHolder& matchmakingSession,
                        GatheringHolder* joinedMatchmakingSession,
                        const String strMessage = NEX_T(""));

    /*! @internal
    @refmanoverload{AutoMatchmakeWithGatheringId}

    @brief      指定されたギャザリングに参加します。推奨されません。

    @deprecated AutoMatchmakeParam を引数に取るオーバーロード関数の使用を推奨します。

    ギャザリングIDを指定してマッチメイクセッション(ギャザリング)に参加します。
    ギャザリングIDは複数指定でき、与えられたギャザリングIDのリストの先頭から順に参加を試み、
    最初に参加できたマッチメイクセッションを返します。

    指定したマッチメイクセッションに参加できなかった場合には、合わせて指定されたマッチメイクセッション情報に従い
    新たにマッチメイクセッションを作り、そこに参加、さらに自分がホストになります。

    他の AutoMatchmake() と異なり、過去3回にわたって参加したマッチメイクセッションを参加の対象から取り除く処理は含まれていません。

    アプリケーション側で MatchmakeExtensionClient::BrowseMatchmakeSession() を用いて検索を行い、検索結果のフィルタリングを行った後で、本関数を用いてマッチメイクセッションに参加、
    もしくはを作成するような実装は許容されています。しかし多くの場合、 検索条件を指定できる AutoMatchmake() を使用しても、 MatchmakeSessionSearchCriteria
    を適切に指定することで同様のことは実現できます。まずは検索条件を指定できる AutoMatchmake() の利用をご検討ください。

    その上で本関数を使用される場合、指定したマッチメイクセッションに参加できないなどの理由で不必要にマッチメイクセッションが作成されないよう、
    ギャザリングIDは複数指定し、リストの並び順はなるべくクライアント毎に異なるようにするような実装を強く推奨します。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。
                                           マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                           0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。
    @li    QERROR(RendezVous, LimitExceeded) : 開発環境で、ギャザリング作成時に自分がオーナーのマッチメイクセッションが 20 個すでに存在していた場合に発生します。
                                               このエラーはあくまでクライアントのバグを検出したりシステムを保護したりするのを目的に、開発環境でのみ発生するものです。
                                               このエラーが返ってくることを前提として作成するマッチメイクセッションの個数を制御したりしないでください。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gidList    参加したいギャザリングIDのリスト。リストの先頭から順に参加を試みます。
    @param[in]     matchmakingSession    作成するマッチメイクセッション。参加できるものが見つからなかった場合、この情報のギャザリングが作成されます。
    @param[out]    joinedMatchmakingSession    参加したマッチメイクセッション。
    @param[in]     strMessage   ギャザリングのオーナーもしくは参加者に通知されるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool AutoMatchmake(ProtocolCallContext *pContext,
                        const qList<GatheringID> gidList,
                        const GatheringHolder& matchmakingSession,
                        GatheringHolder* joinedMatchmakingSession,
                        const String strMessage = NEX_T(""));

    /*!
    @refmanoverload{BrowseMatchmakeSessionNoHolderNoResultRange}

    @brief      マッチメイクセッションを検索します。

    指定した条件でマッチメイクセッションを検索します。 検索範囲は、 引数 searchCriteria オブジェクトの MatchmakeSessionSearchCriteria::SetResultRange() で指定します。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 永続ギャザリングマッチ時に永続ギャザリングのギャザリングIDをセットしていない。 引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     searchCriteria    検索条件。
    @param[out]    lstMatchmakeSession    見つかったマッチメイクセッション。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see MatchmakeSessionSearchCriteria::SetResultRange()
    */
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 qList<MatchmakeSession>* lstMatchmakeSession);

    /*!
    @refmanoverload{BrowseMatchmakeSessionWithHostURLNoHolderNoResultRange}

    @brief      マッチメイクセッションを検索します。

    指定した条件でマッチメイクセッションを検索し、マッチメイクセッションのホストのURLも併せて取得します。 検索範囲は、 引数 searchCriteria オブジェクトの MatchmakeSessionSearchCriteria::SetResultRange() で指定します。
    ホストが決定していないマッチメイクセッションは検索対象から除外されます。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 永続ギャザリングマッチ時に永続ギャザリングのギャザリングIDをセットしていない。 引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     searchCriteria    検索条件。
    @param[out]    lstMatchmakeSession    見つかったマッチメイクセッション。
    @param[out]    lstGatheringURLs    マッチメイクセッションのホストのURL

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see MatchmakeSessionSearchCriteria::SetResultRange()
    */
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 qList<MatchmakeSession>* lstMatchmakeSession,
                                 qList<GatheringURLs>* lstGatheringURLs);

    /*! @internal
    @refmanoverload{BrowseMatchmakeSession}

    @brief      マッチメイクセッションを検索します。推奨されません。

    @deprecated MatchmakeSession クラスで結果を取得し、MatchmakeSessionSearchCriteria クラスで検索範囲を指定するオーバーロード関数の使用を推奨します。

    指定した条件でマッチメイクセッションを検索します。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                           0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。
                                           引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     searchCriteria    検索条件。
    @param[in]     resultRange    検索範囲。
    @param[out]    lstGathering    見つかったマッチメイクセッション。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 const ResultRange& resultRange,
                                 qList<GatheringHolder>* lstGathering);

    /*! @internal
    @refmanoverload{BrowseMatchmakeSessionWithHostURL}

    @brief      マッチメイクセッションを検索します。推奨されません。

    @deprecated MatchmakeSession クラスで結果を取得し、MatchmakeSessionSearchCriteria クラスで検索範囲を指定するオーバーロード関数の使用を推奨します。

    指定した条件でマッチメイクセッションを検索し、マッチメイクセッションのホストのURLも併せて取得します。
    ホストが決定していないマッチメイクセッションは検索対象から除外されます。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                           0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。
                                           引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     searchCriteria    検索条件。
    @param[in]     resultRange    検索範囲。
    @param[out]    lstGathering    見つかったマッチメイクセッション。
    @param[out]    lstGatheringURLs    マッチメイクセッションのホストのURL

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 const ResultRange& resultRange,
                                 qList<GatheringHolder>* lstGathering,
                                 qList<GatheringURLs>* lstGatheringURLs);

    /*!
    @refmanoverload{BrowseMatchmakeSessionNoHolder}

    @brief      マッチメイクセッションを検索します。推奨されません。

    @deprecated MatchmakeSession クラスで結果を取得し、MatchmakeSessionSearchCriteria クラスで検索範囲を指定するオーバーロード関数の使用を推奨します。

    指定した条件でマッチメイクセッションを検索します。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 永続ギャザリングマッチ時に永続ギャザリングのギャザリングIDをセットしていない。 引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     searchCriteria    検索条件。
    @param[in]     resultRange    検索範囲。
    @param[out]    lstMatchmakeSession    見つかったマッチメイクセッション。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 const ResultRange& resultRange,
                                 qList<MatchmakeSession>* lstMatchmakeSession);

    /*!
    @refmanoverload{BrowseMatchmakeSessionWithHostURLNoHolder}

    @brief      マッチメイクセッションを検索します。推奨されません。

    @deprecated MatchmakeSession クラスで結果を取得し、MatchmakeSessionSearchCriteria クラスで検索範囲を指定するオーバーロード関数の使用を推奨します。

    指定した条件でマッチメイクセッションを検索し、マッチメイクセッションのホストのURLも併せて取得します。
    ホストが決定していないマッチメイクセッションは検索対象から除外されます。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 永続ギャザリングマッチ時に永続ギャザリングのギャザリングIDをセットしていない。 引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     searchCriteria    検索条件。
    @param[in]     resultRange    検索範囲。
    @param[out]    lstMatchmakeSession    見つかったマッチメイクセッション。
    @param[out]    lstGatheringURLs    マッチメイクセッションのホストのURL

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool BrowseMatchmakeSession(ProtocolCallContext *pContext,
                                 const MatchmakeSessionSearchCriteria& searchCriteria,
                                 const ResultRange& resultRange,
                                 qList<MatchmakeSession>* lstMatchmakeSession,
                                 qList<GatheringURLs>* lstGatheringURLs);

    /*!
    @refmanoverload{CreateMatchmakeSessionWithParam}

    @brief      マッチメイクセッションを作成し、参加します。

    指定したマッチメイクセッションを作成し、参加、さらに自分がオーナー、ホストになります。
    非同期処理に成功すると、作成されたマッチメイクセッションの GatheringID と ゲームサーバーで生成された P2P 通信用の共通鍵がセットされます。
    共通鍵はマッチメイクセッション参加者に共有されます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, NotParticipatedGathering) : 追加で参加させようとしたユーザーが CreateMatchmakeSessionParam::SetGatheringIdForParticipationCheck() で指定したギャザリングに参加していない。
    @li QERROR(RendezVous, UserIsOffline) : 追加で参加させようとしたユーザーが NGS にログインしていない。

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。matchmakingSessionパラメータが不正。
                                        マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                        0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。
    @li QERROR(RendezVous, LimitExceeded) : 開発環境で、ギャザリング作成時に自分がオーナーのマッチメイクセッションが 20 個すでに存在していた場合に発生します。
                                            このエラーはあくまでクライアントのバグを検出したりシステムを保護したりするのを目的に、開発環境でのみ発生するものです。
                                            このエラーが返ってくることを前提として作成するマッチメイクセッションの個数を制御したりしないでください。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     createMatchmakeSessionParam マッチメイクセッション作成処理に関するパラメータ。
    @param[out]    joinedMatchmakingSession    参加したマッチメイクセッション。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see Network::SetP2PDataPacketSessionSignatureKey()

    */
    qBool CreateMatchmakeSession(ProtocolCallContext *pContext,
                                 const CreateMatchmakeSessionParam& createMatchmakeSessionParam,
                                 MatchmakeSession* joinedMatchmakingSession);
    
    /*! @internal
    @refmanoverload{CreateMatchmakeSession}

    @brief      マッチメイクセッションを作成し、参加します。推奨されません。

    @deprecated  CreateMatchmakeSessionParam を引数に取るオーバーロード関数の使用を推奨します。

    指定したマッチメイクセッションを作成し、参加、さらに自分がオーナー、ホストになります。
    非同期処理に成功すると、作成されたマッチメイクセッションの GatheringID と ゲームサーバーで生成された P2P 通信用の共通鍵がセットされます。
    共通鍵はマッチメイクセッション参加者に共有されます。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。matchmakingSessionパラメータが不正。
                                        マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                        0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。
    @li QERROR(RendezVous, LimitExceeded) : 開発環境で、ギャザリング作成時に自分がオーナーのマッチメイクセッションが 20 個すでに存在していた場合に発生します。
                                            このエラーはあくまでクライアントのバグを検出したりシステムを保護したりするのを目的に、開発環境でのみ発生するものです。
                                            このエラーが返ってくることを前提として作成するマッチメイクセッションの個数を制御したりしないでください。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     matchmakingSession    作成するマッチメイクセッション。@ref MatchmakeExtensionClient::AutoMatchmake() や @ref MatchmakeExtensionClient::BrowseMatchmakeSession()
                                         で取得したオブジェクトは指定できません。
    @param[out]    gid    作成されたマッチメイクセッションのID。
    @param[out]    pSessionKey    P2P 通信用の共通鍵。鍵長は @ref Network::MAX_P2P_SIGNATURE_KEY_LEN byte です。
    @param[in]     strMessage   設定によりギャザリングの参加者に通知されるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see Network::SetP2PDataPacketSessionSignatureKey()

    */
    qBool CreateMatchmakeSession(ProtocolCallContext *pContext,
                                 const GatheringHolder& matchmakingSession,
                                 GatheringID* gid,
                                 qVector<qByte>* pSessionKey,
                                 const String strMessage = NEX_T(""));

    /*! @internal
    @refmanoverload{CreateMatchmakeSessionWithParticipationCount}

    @brief      マッチメイクセッションを作成し、参加します。推奨されません。

    @deprecated  CreateMatchmakeSessionParam を引数に取るオーバーロード関数の使用を推奨します。

    指定したマッチメイクセッションを作成し、参加、さらに自分がオーナー、ホストになります。
    非同期処理に成功すると、作成されたマッチメイクセッションの GatheringID と ゲームサーバーで生成された P2P 通信用の共通鍵がセットされます。
    共通鍵はマッチメイクセッション参加者に共有されます。

    非同期処理の結果、以下の Result が返ることがあります。

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。matchmakingSessionパラメータが不正。
                                        マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                        0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。
                                        participationCountパラメータが不正。
    @li QERROR(RendezVous, LimitExceeded) : 開発環境で、ギャザリング作成時に自分がオーナーのマッチメイクセッションが 20 個すでに存在していた場合に発生します。
                                            このエラーはあくまでクライアントのバグを検出したりシステムを保護したりするのを目的に、開発環境でのみ発生するものです。
                                            このエラーが返ってくることを前提として作成するマッチメイクセッションの個数を制御したりしないでください。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     matchmakingSession    作成するマッチメイクセッション。@ref MatchmakeExtensionClient::AutoMatchmake() や @ref MatchmakeExtensionClient::BrowseMatchmakeSession()
                                         で取得したオブジェクトは指定できません。
    @param[out]    gid    作成されたマッチメイクセッションのID。
    @param[out]    pSessionKey    P2P 通信用の共通鍵。鍵長は @ref Network::MAX_P2P_SIGNATURE_KEY_LEN byte です。
    @param[in]     participationCount   このクライアントからの参加人数。1 以上、かつ作成するマッチメイクセッションの最大参加人数以下の値を設定してください。
    @param[in]     strMessage   設定によりギャザリングの参加者に通知されるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see Network::SetP2PDataPacketSessionSignatureKey()

    */
    qBool CreateMatchmakeSession(ProtocolCallContext *pContext,
                                 const GatheringHolder& matchmakingSession,
                                 GatheringID* gid,
                                 qVector<qByte>* pSessionKey,
                                 qUnsignedInt16 participationCount,
                                 const String strMessage = NEX_T(""));

    /*!
    @refmanoverload{JoinMatchmakeSession}

    @brief      マッチメイクセッションに参加します。推奨されません。

    @deprecated  JoinMatchmakeSessionParam を引数に取るオーバーロード関数の使用を推奨します。

    指定したIDのマッチメイクセッションに参加します。参加に成功するとマッチメイクセッションのオーナーにイベントが通知されます。
    非同期処理に成功すると、 P2P 通信用の共通鍵がセットされます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, SessionFull) : すでに最大参加人数に達している。
    @li QERROR(RendezVous, SessionClosed) : マッチメイクセッションが締め切られている。
    @li QERROR(RendezVous, NotFriend) :  参加しようとしたマッチメイクセッションのオーナーとフレンド関係がない。 (MATCHMAKE_SYSTEM_TYPE_FRIEND を使用した場合のみ発生)
    @li QERROR(RendezVous, DeniedByParticipants) : マッチメイクセッションの参加者が自分をブロックリストに登録している
    @li QERROR(RendezVous, ParticipantInBlockList) : 自分のブロックリストの中にマッチメイクセッションの参加者が登録されている

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : MATCHMAKE_STRING_MAX_LENGTH を超える文字列を送信しようとした。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    参加したいマッチメイクセッションのID。
    @param[out]    pSessionKey    P2P セッション用の通信鍵。鍵長は @ref Network::MAX_P2P_SIGNATURE_KEY_LEN byte です。
    @param[in]     strMessage   ギャザリングのオーナーもしくは参加者に通知されるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    qBool JoinMatchmakeSession(ProtocolCallContext *pContext,
                               GatheringID gid,
                               qVector<qByte>* pSessionKey,
                               const String strMessage=NEX_T(""));

    /*! @internal

    @brief      マッチメイクセッションに参加します。推奨されません。

    @deprecated JoinMatchmakeSessionParam を引数に取るオーバーロード関数の使用を推奨します。

    指定したIDのマッチメイクセッションに参加します。参加に成功するとマッチメイクセッションのオーナーにイベントが通知されます。
    非同期処理に成功すると、 P2P 通信用の共通鍵がセットされます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, SessionFull) : すでに最大参加人数に達している。
    @li QERROR(RendezVous, SessionClosed) : マッチメイクセッションが締め切られている。
    @li QERROR(RendezVous, NotFriend) :  参加しようとしたマッチメイクセッションのオーナーとフレンド関係がない。 (MATCHMAKE_SYSTEM_TYPE_FRIEND を使用した場合のみ発生)
    @li QERROR(RendezVous, DeniedByParticipants) : マッチメイクセッションの参加者が自分をブロックリストに登録している
    @li QERROR(RendezVous, ParticipantInBlockList) : 自分のブロックリストの中にマッチメイクセッションの参加者が登録されている

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : MATCHMAKE_STRING_MAX_LENGTH を超える文字列を送信しようとした。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    参加したいマッチメイクセッションのID。
    @param[out]    pSessionKey    P2P セッション用の通信鍵。鍵長は @ref Network::MAX_P2P_SIGNATURE_KEY_LEN byte です。
    @param[in]     strMessage   ギャザリングのオーナーもしくは参加者に通知されるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。
    @param[in]     dontCareMyBlockList    true の場合、参加者に自分がブロックリストに登録しているユーザーが存在していても強制的に参加します。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    qBool JoinMatchmakeSessionWithDontCareMyBlockList(ProtocolCallContext *pContext,
                               GatheringID gid,
                               qVector<qByte>* pSessionKey,
                               const String strMessage=NEX_T(""),
                               qBool dontCareMyBlockList = false);

    /*! @internal
    @refmanoverload{JoinMatchmakeSessionWithParticipationCount}

    @brief      マッチメイクセッションに参加します。推奨されません。

    @deprecated JoinMatchmakeSessionParam を引数に取るオーバーロード関数の使用を推奨します。

    指定したIDのマッチメイクセッションに参加します。参加に成功するとマッチメイクセッションのオーナーにイベントが通知されます。
    非同期処理に成功すると、 P2P 通信用の共通鍵がセットされます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, SessionFull) : すでに最大参加人数に達している。
    @li QERROR(RendezVous, SessionClosed) : マッチメイクセッションが締め切られている。
    @li QERROR(RendezVous, NotFriend) :  参加しようとしたマッチメイクセッションのオーナーとフレンド関係がない。 (MATCHMAKE_SYSTEM_TYPE_FRIEND を使用した場合のみ発生)。
    @li QERROR(RendezVous, DeniedByParticipants) : マッチメイクセッションの参加者が自分をブロックリストに登録している。
    @li QERROR(RendezVous, ParticipantInBlockList) : 自分のブロックリストの中にマッチメイクセッションの参加者が登録されている。

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : MATCHMAKE_STRING_MAX_LENGTH を超える文字列を送信しようとした。 participationCount パラメータが不正。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    参加したいマッチメイクセッションのID。
    @param[out]    pSessionKey    P2P セッション用の通信鍵。鍵長は @ref Network::MAX_P2P_SIGNATURE_KEY_LEN byte です。
    @param[in]     participationCount   このクライアントからの参加人数。1 以上の値を設定してください。
    @param[in]     strMessage   ギャザリングのオーナーもしくは参加者に通知されるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    qBool JoinMatchmakeSession(ProtocolCallContext *pContext,
                               GatheringID gid,
                               qVector<qByte>* pSessionKey,
                               qUnsignedInt16 participationCount,
                               const String strMessage=NEX_T(""));

    /*!
    @refmanoverload{JoinMatchmakeSessionWithParam}

    @brief      マッチメイクセッションに参加します。

    指定したIDのマッチメイクセッションに参加します。参加に成功するとマッチメイクセッションのオーナーにイベントが通知されます。
    非同期処理に成功すると、 P2P 通信用の共通鍵がセットされます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, SessionFull) : すでに最大参加人数に達している。
    @li QERROR(RendezVous, SessionClosed) : マッチメイクセッションが締め切られている。
    @li QERROR(RendezVous, NotFriend) :  参加しようとしたマッチメイクセッションのオーナーとフレンド関係がない。 (MATCHMAKE_SYSTEM_TYPE_FRIEND を使用した場合のみ発生)。
    @li QERROR(RendezVous, DeniedByParticipants) : マッチメイクセッションの参加者が自分をブロックリストに登録している。
    @li QERROR(RendezVous, ParticipantInBlockList) : 自分のブロックリストの中にマッチメイクセッションの参加者が登録されている。
    @li QERROR(RendezVous, MatchmakeSessionUserPasswordUnmatch) : マッチメイクセッションユーザーパスワードがセットされているマッチメイクセッションに参加しようとしたが
                                                                  マッチメイクセッションユーザーパスワードが一致しない。
    @li QERROR(RendezVous, MatchmakeSessionSystemPasswordUnmatch) : マッチメイクセッションシステムパスワードがセットされているマッチメイクセッションに参加しようとしたが
                                                                    マッチメイクセッションシステムパスワードが一致しない。
    @li QERROR(RendezVous, NotParticipatedGathering) : 追加で参加させようとしたユーザーが JoinMatchmakeSessionParam::SetGatheringIdForParticipationCheck() で指定したギャザリングに参加していない。
                                                       もしくは JoinMatchmakeSessionParam::SetJoinMatchmakeSessionBehavior() に JOIN_MATCHMAKE_SESSION_OPTION_IM_ALREADY_JOINED を指定して
                                                       他人のみを参加させようとしたときに自身が JoinMatchmakeSessionParam::SetGatheringId() で指定したギャザリングに参加していない。
    @li QERROR(RendezVous, AlreadyParticipatedGathering) : 追加で参加させようとしたユーザーがすでにそのギャザリングに参加している。
                                                           もしくは自身がすでに参加済みの状態で JoinMatchmakeSessionParam::SetJoinMatchmakeSessionBehavior() に
                                                          JOIN_MATCHMAKE_SESSION_OPTION_JOIN_MYSELF を指定して自身を再度参加させようとした。
    @li QERROR(RendezVous, UserIsOffline) : 追加で参加させようとしたユーザーが NGS にログインしていない。

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : MATCHMAKE_STRING_MAX_LENGTH を超える文字列を送信しようとした。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     joinMatchmakeSessionParam   マッチメイクセッション参加処理に関するパラメータ。
    @param[out]    joinedMatchmakingSession    参加したマッチメイクセッション。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    qBool JoinMatchmakeSession(ProtocolCallContext *pContext,
                               const JoinMatchmakeSessionParam& joinMatchmakeSessionParam,
                               MatchmakeSession* joinedMatchmakingSession);

    /*!
    @brief      マッチメイクセッションへの参加を締め切ります。

    指定したマッチメイクセッションへの参加を締め切ります。マッチメイクセッションのオーナーしか行えません。
    タイミングによっては本 RMC 呼び出し成功後も参加される可能性がありますのでご注意ください。

    非同期処理に成功すると、@ref MatchmakeExtensionClient::BrowseMatchmakeSession() などで取得できる @ref MatchmakeSession の募集状態フラグ
    (@ref MatchmakeSession::GetOpenParticipation() で取得可能)もfalseになります。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    参加を締め切るマッチメイクセッションのID。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool CloseParticipation(ProtocolCallContext *pContext, GatheringID gid);

    /*!
    @brief      マッチメイクセッションへの参加を再開します。

    指定したマッチメイクセッションへの参加を再開します。マッチメイクセッションのオーナーしか行えません。

    非同期処理に成功すると、@ref MatchmakeExtensionClient::BrowseMatchmakeSession() などで取得できる @ref MatchmakeSession の募集状態フラグ
    (@ref MatchmakeSession::GetOpenParticipation() で取得可能)もtrueになります。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    参加を再開させるマッチメイクセッションのID。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool OpenParticipation(ProtocolCallContext *pContext, GatheringID gid);

    /*!
    @brief      マッチメイクセッションの属性を変更します。

    指定したマッチメイクセッションの属性を変更します。マッチメイクセッションのオーナーしか行えません。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    属性を変更するマッチメイクセッションのID。
    @param[in]     attribIndex    変更する属性のインデックス[1-NUM_MATCHMAKE_SESSION_ATTRIBUTES]。
    @param[in]     newValue    変更後の属性値。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool ModifyMatchmakeSessionAttribute(ProtocolCallContext *pContext, GatheringID gid, qUnsignedInt32 attribIndex, qUnsignedInt newValue);

    /*!
    @brief      マッチメイクセッションの属性を変更します。

    指定したマッチメイクセッションの属性を全て変更します。マッチメイクセッションのオーナーしか行えません。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    属性を変更するマッチメイクセッションのID。
    @param[in]     attributes    変更後の属性値の配列。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UpdateMatchmakeSessionAttribute(ProtocolCallContext *pContext, GatheringID gid, qUnsignedInt32 attributes[nn::nex::NUM_MATCHMAKE_SESSION_ATTRIBUTES]);

    /*!
    @refmanoverload{UpdateProgressScore}

    @brief      マッチメイクセッションのゲーム進行度を更新します。

    マッチメイクセッションのゲーム進行度を更新します。マッチメイクセッションのオーナーしか行えません。

    この関数は、初回呼び出し時以降、 UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME 秒以上の間隔で呼んでください。
    UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME 秒より短い間隔で呼ぶと、失敗します。
    ただし、ゲーム進行度に 0 または MAX_PROGRESS_SCORE を指定した場合は、
    UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME 秒より短い間隔で呼んでも、ゲーム進行度を更新することができます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。
    @li QERROR(Core, InvalidArgument) : MAX_PROGRESS_SCORE より大きいゲーム進行度を指定した。gid に INVALID_GATHERINGID を指定した。
    @li QERROR(Core, InvalidSequence) : UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME 秒より短い間隔でこの関数を呼んだ。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    属性を変更するマッチメイクセッションのID。
    @param[in]     progressScore    ゲーム進行度。0 以上 MAX_PROGRESS_SCORE 以下の値を指定できます。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UpdateProgressScore(ProtocolCallContext *pContext, GatheringID gid, qUnsignedInt8 progressScore);

    /*!
    @refmanoverload{UpdateProgressScoreWithoutCallContext}

    @brief      マッチメイクセッションのゲーム進行度を更新します。

    マッチメイクセッションのゲーム進行度を更新します。マッチメイクセッションのオーナーしか行えません。

    この関数は、初回呼び出し時以降、 UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME 秒以上の間隔で呼んでください。
    UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME 秒より短い間隔で呼ぶと、失敗します。
    ただし、ゲーム進行度に 0 または MAX_PROGRESS_SCORE を指定した場合は、
    UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME 秒より短い間隔で呼んでも、ゲーム進行度を更新することができます。

    ProtocolCallContext は不要で、内部で処理されます。

    @param[in]     gid    属性を変更するマッチメイクセッションのID。
    @param[in]     progressScore    ゲーム進行度。0 以上 MAX_PROGRESS_SCORE 以下の値を指定できます。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UpdateProgressScore(GatheringID gid, qUnsignedInt8 progressScore);

    /*!
    @brief      マッチメイクセッションのアプリケーション定義データを変更します。

    指定したマッチメイクセッションのアプリケーション定義データを変更します。マッチメイクセッションのオーナーしか行えません。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : MATCHMAKE_BUFFER_MAX_LENGTH を超えるデータを送信しようとした。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    属性を変更するマッチメイクセッションのID。
    @param[in]     applicationBuffer    アプリケーション定義データ。最大 MATCHMAKE_BUFFER_MAX_LENGTH byte までセットできます。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UpdateApplicationBuffer(ProtocolCallContext *pContext, GatheringID gid, qVector<qUnsignedInt8> applicationBuffer);

    /*!
    @refmanoverload{UpdateMatchmakeSession}

    @brief      マッチメイクセッションを更新します。推奨されません。

    @deprecated UpdateMatchmakeSessionParam を引数に取るオーバーロード関数の使用を推奨します。

    マッチメイクセッションの ID、オーナー、 P2P 通信のホスト、ギャザリングフラグ, P2P セッション用の通信鍵,
    関連する永続ギャザリングの ID 以外の全てのデータを更新します。
    マッチメイクシステムタイプは MATCHMAKE_SYSTEM_TYPE_ANYBODY と MATCHMAKE_SYSTEM_TYPE_FRIEND との間のみ変更できます。
    マッチメイクセッションのオーナーしか行えません。
    本関数を呼び出す前に、更新対象のマッチメイクセッションにギャザリングID を指定する必要があります。指定は Gathering::SetID() で行います。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : UpdateMatchmakeSessionParam::SetMatchmakeSystemType() で MATCHMAKE_SYSTEM_TYPE_ANYBODY と MATCHMAKE_SYSTEM_TYPE_FRIEND
                                        との間以外の変更をした。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     matchmakingSession    更新するマッチメイクセッション。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UpdateMatchmakeSession(ProtocolCallContext *pContext, const GatheringHolder& matchmakingSession);

    /*!
    @refmanoverload{UpdateMatchmakeSessionPart}

    @brief      マッチメイクセッションを更新します。

    指定したデータのみを更新します。
    マッチメイクセッションのオーナーしか行えません。
    本関数を呼び出す前に、更新対象のマッチメイクセッションにギャザリングID を指定する必要があります。指定は Gathering::SetID() で行います。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。

    以下は
    @errorapplet

    @li QERROR(Core, InvalidArgument) : UpdateMatchmakeSessionParam::SetModificationFlag() に MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM と
                                        MATCHMAKE_SESSION_MODIFICATION_FLAG_MATCHMAKE_PARAM_OVERRIDE の両方を指定した。
                                        UpdateMatchmakeSessionParam::SetMatchmakeSystemType() で MATCHMAKE_SYSTEM_TYPE_ANYBODY と MATCHMAKE_SYSTEM_TYPE_FRIEND
                                        との間以外の変更をした。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     updateMatchmakeSessionParam    マッチメイクセッション更新処理に関するパラメータ。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UpdateMatchmakeSession(ProtocolCallContext *pContext, const UpdateMatchmakeSessionParam& updateMatchmakeSessionParam);

    /*!
    @brief      Notificationデータをサーバーに登録（存在する場合は更新）し、フレンドに通知します。推奨されません。

    @deprecated  これは互換性のために残されている古い関数です。 プラットフォームごとの SDK に含まれるフレンドプレゼンス API 利用することを推奨します。

    Notificationデータが登録してあると、ログアウトしたときフレンドに @ref NotificationEvents::GameNotificationLogout
    が通知されます。

    非同期処理の結果、以下の Result が返ることがあります。
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : MATCHMAKE_STRING_MAX_LENGTH を超える文字列を送信した。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     uiType      通知のタイプ
    @param[in]     uiParam1    通知データ1
    @param[in]     uiParam2    通知データ2
    @param[in]     strParam    通知データ(最大 MATCHMAKE_STRING_MAX_LENGTH 文字の文字列)

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UpdateNotificationData(ProtocolCallContext *pContext, qUnsignedInt32 uiType, qUnsignedInt64 uiParam1, qUnsignedInt64 uiParam2, String strParam);

    /*!
    @refmanoverload{name1}

    @brief      フレンドの通知データをサーバーから取得します。推奨されません。

    @deprecated これは互換性のために残されている古い関数です。 プラットフォームごとの SDK に含まれるフレンドプレゼンス API 利用することを推奨します。

    @param[in,out]     pContext    呼び出し情報。
    @param[out]    dataList    通知リスト
    @param[in]     uiType      要求する通知のタイプ (NotificationEvents::GameNotificationType1 ～ NotificationEvents::GameNotificationType8 の範囲内である必要があります)

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetFriendNotificationData(ProtocolCallContext *pContext, qList<NotificationEvent> *dataList, qUnsignedInt32 uiType);

    /*!
    @refmanoverload{name2}

    @brief      フレンドの通知データをサーバーから取得します。推奨されません。

    @deprecated これは互換性のために残されている古い関数です。 プラットフォームごとの SDK に含まれるフレンドプレゼンス API 利用することを推奨します。

    @param[in,out]     pContext    呼び出し情報。
    @param[out]    dataList    通知リスト
    @param[in]     lstTypes    要求する通知のタイプリスト (NotificationEvents::GameNotificationType1 ～ NotificationEvents::GameNotificationType8 の範囲内である必要があります)

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetFriendNotificationData(ProtocolCallContext *pContext, qList<NotificationEvent> *dataList, const qList<qUnsignedInt32>& lstTypes);

#if !defined(NEX_SIGLO_FAMILY)
    /*!
    @brief    指定したユーザーが参加しているマッチメイクセッションのリストを取得します。

    フレンド以外であっても参加中のセッション情報が取得可能ですが、検索対象ユーザーが自分をブロックリストに登録している場合は検索結果に含まれることはありません。

    指定した相手が複数のマッチメイクセッションに参加している場合は、複数のセッション情報が取得できます。最大 MAX_MATCHMAKE_SESSION_BY_PARTICIPANT 件返ります。

    非同期処理の結果、以下の Result が返ることがあります。
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 引数で与える プリンシパル ID  のリストのエントリー数が @ref MAX_PRINCIPALID_SIZE_TO_FIND_MATCHMAKE_SESSION より大きい

    @param[in,out]    pContext             呼び出し情報
    @param[in]    principalIdList      参加者のID
    @param[out]   pPlayingSessionList  参加していているマッチメイクセッションのリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

     @deprecated MachmakeExtensionClient::FindMatchmakeSessionByParticipant を使用してください
     */
#endif
    qBool GetPlayingSession(ProtocolCallContext *pContext,
                            const qList<PrincipalID>& principalIdList,
                            qList<PlayingSession>* pPlayingSessionList);

    /*!
    @brief    指定したユーザーが参加しているマッチメイクセッションのリストを取得します。

    param で指定された検索パラメータによってブロックリストの有無や、取得できるマッチメイクセッション情報が変わります

    指定した相手が複数のマッチメイクセッションに参加している場合は、複数のセッション情報が取得できます。 最大 MAX_MATCHMAKE_SESSION_BY_PARTICIPANT 件返ります。

    非同期処理の結果、以下の Result が返ることがあります。
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 引数で与える プリンシパル ID  のリストのエントリー数が @ref MAX_PRINCIPALID_SIZE_TO_FIND_MATCHMAKE_SESSION より大きい

    @param[in,out]    pContext             呼び出し情報
    @param[in]    param                検索パラメータ
    @param[out]   pFindResultList      参加していているマッチメイクセッションのリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool FindMatchmakeSessionByParticipant(ProtocolCallContext *pContext,
                                            const FindMatchmakeSessionByParticipantParam& param,
                                            qList<FindMatchmakeSessionByParticipantResult>* pFindResultList);

    /*!
    @internal
    @brief    参加しているマッチメイクセッション・参加している永続ギャザリングの公開設定を変更します

    マッチメイクセッションの参加状態・永続ギャザリングの参加状態の公開設定を変更します。

    onlineStatusをfalseに設定すると、@ref MatchmakeExtensionClient::GetPlayingSession() で検索対象となったときに空のリストが返るようになり、
    participationをfalseに設定すると、@ref MatchmakeExtensionClient::FindPersistentGatheringByParticipant() で検索対象となったときに空のリストが返ります。

    本関数を一度も呼んでいないユーザーの設定は公開状態となります。

    @param[in,out]    pContext        呼び出し情報
    @param[in]    onlineStatus    オンライン状態の公開設定(true : 公開, false : 非公開)
    @param[in]    participation   所属永続ギャザリングの公開設定(true : 公開, false : 非公開)

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool UpdatePrivacySetting(ProtocolCallContext *pContext, qBool onlineStatus, qBool participation);

    /*!
    @brief      マッチメイクセッションの詳細情報を取得します。

    指定したIDのマッチメイクセッションの詳細情報を取得します。情報には P2P 通信用の共通鍵が含まれます。
    指定したマッチメイクセッションに参加している必要があります。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : 指定したマッチメイクセッションに参加していない。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     gid    情報を取得したいマッチメイクセッションのID。
    @param[out]    matchmakingSession    取得したマッチメイクセッション情報。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see Network::SetP2PDataPacketSessionSignatureKey()
    */
    qBool FindMatchmakeSessionByGatheringIdDetail(ProtocolCallContext *pContext,
                               GatheringID gid,
                               MatchmakeSession* matchmakingSession);

    /*!
    @refmanoverload{FindMatchmakeSessionByGatheringId}

    @brief   マッチメイクセッションをIDで検索します

    指定したIDのマッチメイクセッションを検索します。
    参加していないマッチメイクセッションの情報も取得できますが、取得した情報には P2P 通信用の共通鍵が含まれません。
    P2P 通信用の共通鍵を含んだ情報を取得する場合は、 MatchmakeExtensionClient::FindMatchmakeSessionByGatheringIdDetail() を使用します。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(Core, InvalidArgument) : 引数で与える GatheringID のリストのエントリー数が @ref MAX_MATCHMAKE_BROWSE_SIZE より大きい

    @param[in,out]    pContext    呼び出し情報
    @param[in]    gatheringIdList    マッチメイクセッションのIDのリスト
    @param[out]   lstMatchmakeSession 見つかったマッチメイクセッションのリスト、存在しないマッチメイクセッションは含まれません、また取得されるマッチメイクセッションの順番は不定です

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see MatchmakeExtensionClient::FindMatchmakeSessionByGatheringIdDetail()
    */
    qBool FindMatchmakeSessionByGatheringId(ProtocolCallContext *pContext,
                                            const qList<GatheringID>& gatheringIdList,
                                            qList<MatchmakeSession>* lstMatchmakeSession);

    /*!
    @refmanoverload{FindMatchmakeSessionBySingleGatheringId}

    @brief   マッチメイクセッションをIDで検索します

    指定したIDのマッチメイクセッションを検索します。
    参加していないマッチメイクセッションの情報も取得できますが、取得した情報には P2P 通信用の共通鍵が含まれません。
    P2P 通信用の共通鍵を含んだ情報を取得する場合は、 MatchmakeExtensionClient::FindMatchmakeSessionByGatheringIdDetail() を使用します。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(RendezVous, SessionVoid) : 指定した ID のマッチメイクセッションが存在しない。

    @param[in,out]    pContext    呼び出し情報
    @param[in]    gid    マッチメイクセッションのID
    @param[out]   matchmakeSession 見つかったマッチメイクセッション

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。

    @see MatchmakeExtensionClient::FindMatchmakeSessionByGatheringIdDetail()
    */
    qBool FindMatchmakeSessionByGatheringId(ProtocolCallContext *pContext,
                                            GatheringID gid,
                                            MatchmakeSession* matchmakeSession);

    /*!
    @brief      マッチメイクセッションをオーナーの PrincipalId で検索します。

    指定した PrincipalId のユーザーがオーナーとなっている見つかったマッチメイクセッションを検索します。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(Core, InvalidArgument) : 引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     pidOwner    検索キーとなる PrincipalId
    @param[in]     resultRange    検索範囲
    @param[out]    lstMatchmakeSession    見つかったマッチメイクセッション

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool FindMatchmakeSessionByOwner(ProtocolCallContext *pContext,
                                      PrincipalID pidOwner,
                                      const ResultRange& resultRange,
                                      qList<MatchmakeSession>* lstMatchmakeSession);

    /*
    @brief      サーバサイドマッチメイクを申請します。

    マッチメイクリクエストを行います。

    AutoMatchmakeParam::SetSearchCriteria() により検索条件を一つ指定する必要があります。
    AutoMatchmakeParam::SetSourceMatchmakeSession() で指定したマッチメイクセッションに検索条件が一致したメンバーでグループを形成します。
    そのグループの中の一人が代表となり、 そのユーザーが AutoMatchmakeParam::SetSourceMatchmakeSession() で指定した条件のマッチメイクセッションが新規に作成されます。
    そしてそのユーザーがマッチメイクセッションのオーナーとなります。
    この際、AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザーはオーナーになりません。

    マッチメイクリクエストを取り下げるには MatchmakeExtensionClient::WithdrawMatchmaking() を使用してください。
    取り下げ処理はマッチメイクリクエストを行ったユーザー及び AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザーから行えます。
    マッチメイクリクエストを行ったユーザー及び AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザーが
    NGS からログアウトしたり切断したりした場合は自動的にマッチメイクリクエストが取り下げられます。
    マッチメイクリクエストを取り下げる目的で ProtocolCallContext::Cancel() を利用することはできません。
    ただし、その後 NGS からログアウトする前提の場合は上記の通り自動的に取り下げ処理が行われるため ProtocolCallContext::Cancel() を利用可能です。

    マッチメイクリクエストを行ったユーザー及び AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザー全てに対し以下の通知が行われます。

    @li NotificationEvents::ServerSideMatchmakingRequested : マッチメイクリクエストが開始された事を通知します。
    @li NotificationEvents::ParticipationEvent : 参加メンバー全員分のマッチメイクセッションへの参加を通知します。通知対象は GatheringFlags により変更可能です。
    @li NotificationEvents::ServerSideMatchmakingCompleted : サーバーサイドマッチメイクが完了したことを通知します。

    加えて、 AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザー全てに対し以下の通知が行われます。

    @li NotificationEvents::AddedToGathering : マッチメイクセッションに追加された事を通知します。NotificationEvents::ParticipationEvent 通知の前に送信されます。

    AutoMatchmakeParam::SetGatheringIdForParticipationCheck() を指定することによるチェックは本非同期処理を開始したときに行われます。
    マッチメイク完了時に該当するマッチメイクセッションへ参加しているかどうか保証されません。
    AutoMatchmakeParam::SetSearchCriteria() で指定した検索条件で AutoMatchmakeParam::SetSourceMatchmakeSession() に指定した自分自身のマッチメイクセッションを検索できる必要があります。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(RendezVous, NotParticipatedGathering) : 追加で参加させようとしたユーザーが AutoMatchmakeParam::SetGatheringIdForParticipationCheck() で指定したギャザリングに参加していない。
    @li    QERROR(RendezVous, MatchmakingWithdrawn) : MatchmakeExtensionClient::WithdrawMatchmaking() によりマッチメイクリクエストが取り下げられた。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。
                                           マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                           0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。<br>
                                           AutoMatchmakeParam::SetSearchCriteria() に指定した検索条件が 2 個以上もしくは 0 個。<br>
                                           AutoMatchmakeParam::SetSearchCriteria() で指定した検索条件で AutoMatchmakeParam::SetSourceMatchmakeSession() に
                                           指定した自分自身のマッチメイクセッションを検索できない。
    @li    QERROR(RendezVous, DuplicateEntry) : 同一のカテゴリへ二重にマッチメイクリクエストを行った。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     autoMatchmakeParam    オートマッチメイク処理に関するパラメータ。
    @param[out]    requestId   申請に割り当てられるID。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool RequestMatchmaking(ProtocolCallContext *pContext,
                             const AutoMatchmakeParam& autoMatchmakeParam,
                             qUnsignedInt64* requestId);

    /*
    @brief      マッチメイクリクエストを発行し、サーバーサイドマッチメイクを行います。

    マッチメイクリクエストを行い、指定した条件のメンバーがそろうまで待ちます。
    最大参加人数分のメンバーがそろいマッチメイクセッションへの参加が完了した段階で非同期処理が完了します。

    AutoMatchmakeParam::SetSearchCriteria() により検索条件を一つ指定する必要があります。
    AutoMatchmakeParam::SetSourceMatchmakeSession() で指定したマッチメイクセッションに検索条件が一致したメンバーでグループを形成します。
    そのグループの中の一人が代表となり、 そのユーザーが AutoMatchmakeParam::SetSourceMatchmakeSession() で指定した条件のマッチメイクセッションが新規に作成されます。
    そしてそのユーザーがマッチメイクセッションのオーナーとなります。
    この際、AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザーはオーナーになりません。

    マッチメイクリクエストを取り下げるには MatchmakeExtensionClient::WithdrawMatchmaking() を使用してください。
    取り下げ処理はマッチメイクリクエストを行ったユーザー及び AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザーから行えます。
    マッチメイクリクエストを行ったユーザー及び AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザーが
    NGS からログアウトしたり切断したりした場合は自動的にマッチメイクリクエストが取り下げられます。
    マッチメイクリクエストを取り下げる目的で ProtocolCallContext::Cancel() を利用することはできません。
    ただし、その後 NGS からログアウトする前提の場合は上記の通り自動的に取り下げ処理が行われるため ProtocolCallContext::Cancel() を利用可能です。

    マッチメイクリクエストを行ったユーザー及び AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザー全てに対し以下の通知が行われます。

    @li NotificationEvents::ServerSideMatchmakingRequested : マッチメイクリクエストが開始された事を通知します。
    @li NotificationEvents::ParticipationEvent : 参加メンバー全員分のマッチメイクセッションへの参加を通知します。通知対象は GatheringFlags により変更可能です。
    @li NotificationEvents::ServerSideMatchmakingCompleted : サーバーサイドマッチメイクが完了したことを通知します。

    加えて、 AutoMatchmakeParam::SetAdditionalParticipant() で指定したユーザー全てに対し以下の通知が行われます。

    @li NotificationEvents::AddedToGathering : マッチメイクセッションに追加された事を通知します。NotificationEvents::ParticipationEvent 通知の前に送信されます。

    AutoMatchmakeParam::SetGatheringIdForParticipationCheck() を指定することによるチェックは本非同期処理を開始したときに行われます。
    マッチメイク完了時に該当するマッチメイクセッションへ参加しているかどうか保証されません。
    AutoMatchmakeParam::SetSearchCriteria() で指定した検索条件で AutoMatchmakeParam::SetSourceMatchmakeSession() に指定した自分自身のマッチメイクセッションを検索できる必要があります。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(RendezVous, NotParticipatedGathering) : 追加で参加させようとしたユーザーが AutoMatchmakeParam::SetGatheringIdForParticipationCheck() で指定したギャザリングに参加していない。
    @li    QERROR(RendezVous, MatchmakingWithdrawn) : MatchmakeExtensionClient::WithdrawMatchmaking() によりマッチメイクリクエストが取り下げられた。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。
                                           マッチメイクシステムタイプに MATCHMAKE_SYSTEM_TYPE_PERSISTENT_GATHERING もしくは MATCHMAKE_SYSTEM_TYPE_OLV_COMMUNITY を指定したが、
                                           0 番目の属性もしくは関連付ける永続ギャザリングの ID に永続ギャザリングのギャザリングIDをセットしていない。<br>
                                           AutoMatchmakeParam::SetSearchCriteria() に指定した検索条件が 2 個以上もしくは 0 個。<br>
                                           AutoMatchmakeParam::SetSearchCriteria() で指定した検索条件で AutoMatchmakeParam::SetSourceMatchmakeSession() に
                                           指定した自分自身のマッチメイクセッションを検索できない。
    @li    QERROR(RendezVous, DuplicateEntry) : 同一のカテゴリへ二重にマッチメイクリクエストを行った。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     autoMatchmakeParam    オートマッチメイク処理に関するパラメータ。
    @param[out]    joinedMatchmakingSession    参加したマッチメイクセッション。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool DoMatchmaking(ProtocolCallContext *pContext,
                        const AutoMatchmakeParam& autoMatchmakeParam,
                        MatchmakeSession* joinedMatchmakeSession);

    /*
    @brief      マッチメイクリクエストを取り下げます。

    MatchmakeExtensionClient::DoMatchmaking() で発行したマッチメイクリクエストを取り下げます。
    自分自身がリクエストしたものと、他のユーザーから AutoMatchmakeParam::SetAdditionalParticipant() で追加されたリクエストを取り下げられます。
    本非同期処理の結果は取り下げのリクエストがサーバーに届けられたかどうかを意味します。
    実際に取り下げ処理が行われたかどうかは対象となる MatchmakeExtensionClient::DoMatchmaking() の結果を確認してください。
    取り下げ処理が行われた場合は MatchmakeExtensionClient::DoMatchmaking() の結果が QERROR(RendezVous, MatchmakingWithdrawn) となります。
    取り下げが間に合わずにマッチメイクが完了する場合もあります。
    マッチメイク完了後は本非同期処理を使ってマッチメイクセッションから退出することはできません。
    マッチメイクセッションから退出するには MatchmakeExtensionClient::EndParticipation() を使用して下さい。
    ただし、マッチメイク完了後に退出すると他の参加者の迷惑になる可能性があるため極力控えることを推奨します。
    指定したリクエスト ID が存在しない場合や、自分自身がリクエストしていないリクエスト ID を指定しても本非同期処理は成功します。
    ただし、その場合は何も行われません。

    @param[in,out]     pContext    呼び出し情報。
    @param[in]     requestId   MatchmakeExtensionClient::RequestMatchmaking() によって割り当てられたリクエスト ID。
                               NotificationEvents::ServerSideMatchmakingRequested 通知から取得可能です。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool WithdrawMatchmaking(ProtocolCallContext *pContext,
                              qUnsignedInt64 requestId);

    /*
    @brief      自分自身に関するマッチメイクリクエストを全て取り下げます。

    MatchmakeExtensionClient::DoMatchmaking() で発行されたマッチメイクリクエストを取り下げます。
    自分自身がリクエストしたものと、他のユーザーから AutoMatchmakeParam::SetAdditionalParticipant() で追加されたリクエストを全て取り下げます。
    本非同期処理の結果は取り下げのリクエストがサーバーに届けられたかどうかを意味します。
    実際に取り下げ処理が行われたかどうかは対象となる MatchmakeExtensionClient::DoMatchmaking() の結果を確認してください。
    取り下げ処理が行われた場合は MatchmakeExtensionClient::DoMatchmaking() の結果が QERROR(RendezVous, MatchmakingWithdrawn) となります。
    取り下げが間に合わずにマッチメイクが完了する場合もあります。
    マッチメイク完了後は本非同期処理を使ってマッチメイクセッションから退出することはできません。
    マッチメイクセッションから退出するには MatchmakeExtensionClient::EndParticipation() を使用して下さい。
    ただし、マッチメイク完了後に退出すると他の参加者の迷惑になる可能性があるため極力控えることを推奨します。

    @param[in,out]     pContext    呼び出し情報。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool WithdrawMatchmaking(ProtocolCallContext *pContext);

#ifdef NEX_BLOCKLIST_OWNED_BY_NGS
    /*!
    @brief    ブロックリストにユーザーを追加します。推奨されません。

    @deprecated これは互換性のために残されている古い関数です。MatchmakeExtensionClient::AddToBlockList() を使用して下さい。

    すでにフレンド関係が成立している場合は登録できません。
    すでにブロックリストに追加されているユーザーは無視されます。

    ブロックリスト登録後にフレンド関係が成立しても、自動的にブロックリストから削除されることはありません。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(Core, InvalidArgument) :  引数で与える principalIdList のリストのエントリー数が 101 件以上

    @param[in,out]    pContext        呼び出し情報
    @param[out]   principalIdList    追加するユーザーのリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool AddToBlackList(ProtocolCallContext *pContext,
                         const qList<PrincipalID>& principalIdList);

    /*!
    @brief    ブロックリストにユーザーを追加します。

    すでにフレンド関係が成立している場合は登録できません。
    すでにブロックリストに追加されているユーザーは無視されます。

    ブロックリスト登録後にフレンド関係が成立しても、自動的にブロックリストから削除されることはありません。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(Core, InvalidArgument) :  引数で与える principalIdList のリストのエントリー数が 101 件以上

    @param[in,out]    pContext        呼び出し情報
    @param[out]   principalIdList    追加するユーザーのリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool AddToBlockList(ProtocolCallContext *pContext,
                         const qList<PrincipalID>& principalIdList);

    /*!
    @brief    ブロックリストから特定のユーザーを削除します。推奨されません。

    @deprecated これは互換性のために残されている古い関数です。MatchmakeExtensionClient::RemoveFromBlockList() を使用して下さい。

    @param[in,out]    pContext        呼び出し情報
    @param[out]   principalIdList    PrincipalIdのリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool RemoveFromBlackList(ProtocolCallContext *pContext,
                              const qList<PrincipalID>& principalIdList);

    /*!
    @brief    ブロックリストから特定のユーザーを削除します。

    @param[in,out]    pContext        呼び出し情報
    @param[out]   principalIdList    PrincipalIdのリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool RemoveFromBlockList(ProtocolCallContext *pContext,
                              const qList<PrincipalID>& principalIdList);

    /*!
    @brief    自分がブロックリストに登録しているユーザーのリストを取得します。推奨されません。

    @deprecated これは互換性のために残されている古い関数です。MatchmakeExtensionClient::GetMyBlockList() を使用して下さい。

    @param[in,out]    pContext        呼び出し情報
    @param[out]   pPrincipalIdList    自分がブロックリストに登録しているユーザーのリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool GetMyBlackList(ProtocolCallContext *pContext,
                         qList<PrincipalID>* pPrincipalIdList);

    /*!
    @brief    自分がブロックリストに登録しているユーザーのリストを取得します。

    @param[in,out]    pContext        呼び出し情報
    @param[out]   pPrincipalIdList    自分がブロックリストに登録しているユーザーのリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool GetMyBlockList(ProtocolCallContext *pContext,
                         qList<PrincipalID>* pPrincipalIdList);

    /*!
    @brief    自分がサーバーに登録したブロックリストを全て削除します。推奨されません。

    @deprecated これは互換性のために残されている古い関数です。MatchmakeExtensionClient::ClearBlockList() を使用して下さい。

    @param[in,out]    pContext        呼び出し情報

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool ClearBlackList(ProtocolCallContext *pContext);

    /*!
    @brief    自分がサーバーに登録したブロックリストを全て削除します。

    @param[in,out]    pContext        呼び出し情報

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool ClearBlockList(ProtocolCallContext *pContext);

#endif
    /*!
    @internal
    @brief    違反ユーザーを通報します。

    violationCode に対応するスコア分、通報対象の通報スコアを加算して通報ログをゲームサーバーに保存します。
    管理ツール上から設定できる閾値よりも通報スコアが高いユーザーは不正ユーザーとして扱われ、不正ユーザー同士でのみマッチメイクを行うようになります。
    通報スコアは一定期間(検討中)通報が行われていない場合に 0 にクリアされます。

    同一ユーザーに対する通報は1回のみに制限されますが、非同期処理は2回目以降であっても成功を返します。
    また、ゲームサーバー上では通報スコアを本体固有の識別子ごとにカウントしています。
    したがって、本体を初期化してプリンシパルIDを再発行しても通報スコアはクリアされません。

    通報ログの閲覧や特定ユーザーに対する通報スコアの変更も、管理ツールから閲覧可能です。
    violationCode(通報理由コード) は管理ツール上からアプリケーションごとに設定可能です。
    管理ツール上で定義されていないコードに対するスコアは常に 1 となります。

    @param[in,out]    pContext        呼び出し情報
    @param[in]    principalId     通報対象ユーザー
    @param[in]    userName        通報対象ユーザーのアプリケーションでの表示名（最大32文字）
    @param[in]    violationCode   通報理由コード

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool ReportViolation(ProtocolCallContext *pContext,
                          PrincipalID principalId,
                          const String& userName,
                          qUnsignedInt8 violationCode = 0);

    /*!
    @internal
    @brief    自分が違反ユーザーとして判定されているかどうかを取得します。

    @param[in,out]    pContext         呼び出し情報
    @param[out]   pIsViolationUser 自分が違反ユーザーとして判定されているかどうか
    @param[out]   pScore           通報スコア

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool IsViolationUser(ProtocolCallContext *pContext,
                          qBool* pIsViolationUser,
                          qUnsignedInt32* pScore);

    /*!
    @internal
    @brief    指定したユーザーが参加しているマッチメイクセッションのリストを取得します。

    指定したユーザーが所属しているマッチメイクセッションのリストを取得します。

    フレンド以外であっても参加中のセッション情報が取得可能ですが、検索対象ユーザーが自分をブロックリストに登録している場合は検索結果に含まれることはありません。

    指定した相手が複数のマッチメイクセッションに参加している場合は、複数のセッション情報が取得できます。

    @param[in,out]    pContext             呼び出し情報
    @param[in]    principalIdList      参加者のIDのリスト
    @param[out]   pSimplePlayingSessionList  参加していているマッチメイクセッション（簡易版）のリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool GetSimplePlayingSession(ProtocolCallContext *pContext,
                                  const qList<PrincipalID>& principalIdList,
                                  qList<SimplePlayingSession>* pSimplePlayingSessionList);

    /*!
    @internal
    @brief    指定したギャザリングIDのコミュニティを取得します(簡易版)

    @param[in,out]    pContext             呼び出し情報
    @param[in]    gatheringIdList      コミュニティIDのリスト
    @param[out]   pSimpleCommunityList  コミュニティ（簡易版）のリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool GetSimpleCommunity(ProtocolCallContext *pContext,
                             const qList<GatheringID>& gatheringIdList,
                             qList<SimpleCommunity>* pSimpleCommunityList);

    /*!
    @brief    永続ギャザリングを登録します

    指定した永続ギャザリングを作成し、参加、さらに自分がオーナーになります。
    一人のユーザが作成できる永続ギャザリングの数は、PERSISTENT_GATHERING_CREATION_MAX です。
    また、作成した永続ギャザリングに自動的に参加するため、参加している永続ギャザリングの数が、
    PERSISTENT_GATHERING_PARTICIPATION_MAX 未満である必要があります。

    公式永続ギャザリングは、作成時にオーナーのプリンシパル ID が、特別なプリンシパル ID(101) となり、 作成したクライアント から、削除・更新ができなくなります。
    NMAS からの操作で、削除・更新を行ってください。

    永続ギャザリングは参加者が 0 になる、または永続ギャザリングのオーナーが明示的に @ref MatchMakingClient::UnregisterGathering()
    によって削除しない限り、永続的にゲームサーバに残り続けます。

    @param[in,out]     pContext     呼び出し情報。
    @param[in]     persistentGathering    登録する永続ギャザリング
    @param[out]    pGatheringId 登録成功時にサーバが発行する永続ギャザリングのID
    @param[in]     strMessage   参加者が固有に登録できるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(RendezVous, PersistentGatheringCreationMax) : 永続ギャザリングの登録上限を超過。
    @li    QERROR(RendezVous, PersistentGatheringParticipationMax) : 永続ギャザリングの参加上限を超過。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool CreatePersistentGathering(ProtocolCallContext *pContext,
                          const PersistentGathering& persistentGathering,
                          GatheringID* pGatheringId,
                          const String& strMessage = NEX_T(""));

    /*!
    @brief    永続ギャザリングを更新します。

    永続ギャザリングを更新します。永続ギャザリングの Id, Owner, PersistentGatheringType 以外の全てのデータを更新します。
    永続ギャザリングのオーナーしか行えません。 公式永続ギャザリングの場合は、更新に失敗します。

    非同期処理の結果、以下の Result が返ることがあります。
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : 送信しようとした文字列・バッファのサイズ超過。PersistentGatheringTypeを変更しようとした
    @li    QERROR(RendezVous, SessionVoid) : ギャザリングが存在しない。
    @li    QERROR(RendezVous, PermissionDenied) : オーナーではない。もしくは、公式永続ギャザリングを更新しようとした。

    @param[in,out]    pContext        呼び出し情報
    @param[in]    persistentGathering       変更する永続ギャザリング情報

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool UpdatePersistentGathering(ProtocolCallContext *pContext, const PersistentGathering& persistentGathering);

    /*!
    @brief    永続ギャザリングに参加します

    指定したIDの永続ギャザリングに参加します。
    一人のユーザーが参加できる永続ギャザリングの数は、PERSISTENT_GATHERING_PARTICIPATION_MAX です。
    @ref MatchMakingClient::EndParticipation() で明示的に退出しない限り永続ギャザリングへの参加状態は残り続けます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(RendezVous, PersistentGatheringParticipationMax) : 永続ギャザリングの参加上限を超過。
    @li    QERROR(RendezVous, SessionVoid) : 永続ギャザリングが存在しない。
    @li    QERROR(RendezVous, SessionFull) : すでに最大参加人数に達している。
    @li    QERROR(RendezVous, InvalidGatheringPassword) : パスワード付永続ギャザリングでパスワードが合致していない。
    @li    QERROR(RendezVous, AlreadyParticipatedGathering) : すでにそのギャザリングに参加している。
    @li    QERROR(RendezVous, WithoutParticipationPeriod) : 参加可能期間外。

    以下は
    @errorapplet

    @li    QERROR(Core, InvalidArgument) : MATCHMAKE_STRING_MAX_LENGTH を超える文字列を送信しようとした。

    @param[in,out]    pContext     呼び出し情報。
    @param[in]    gatheringId  参加する永続ギャザリングのID
    @param[in]    strMessage   参加者が固有に登録できるメッセージ。最大 MATCHMAKE_STRING_MAX_LENGTH 文字です。
    @param[in]    strPassword  永続ギャザリングのパスワード。永続ギャザリングがPasswordLockの場合のみ有効

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool JoinPersistentGathering(ProtocolCallContext *pContext,
                        GatheringID gatheringId,
                        const String& strMessage = NEX_T(""),
                        const String& strPassword = NEX_T(""));

    /*!
    @brief    永続ギャザリングをギャザリングIDで検索します

    非同期処理の結果、以下の Result が返ることがあります。
    @errorapplet

     @li    QERROR(Core, InvalidArgument) : 引数で与える GatheringID のリストのエントリー数が @ref MAX_MATCHMAKE_BROWSE_SIZE より大きい

    @param[in,out]    pContext        呼び出し情報
    @param[in]    gatheringIdList 永続ギャザリングのギャザリングIDのリスト
    @param[out]   pPersistentGatheringList  検索でみつかった永続ギャザリングのリスト

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool FindPersistentGatheringByGatheringId(ProtocolCallContext *pContext,
                                     const qList<GatheringID>& gatheringIdList,
                                     qList<PersistentGathering>* pPersistentGatheringList);

    /*!
    @brief    公式永続ギャザリングを検索します

    非同期処理の結果、以下の Result が返ることがあります。
    @errorapplet

    @li    QERROR(Core, InvalidArgument) :  引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    @param[in,out]    pContext        呼び出し情報
    @param[out]   pPersistentGatheringList  検索でみつかった永続ギャザリングのリスト
    @param[in]    isAvailableOnly 参加可能な永続ギャザリングのみ検索対象に含めるかどうか
    @param[in]    resultRange    検索範囲

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool FindOfficialPersistentGathering(ProtocolCallContext *pContext,
                                qBool isAvailableOnly,
                                const ResultRange& resultRange,
                                qList<PersistentGathering>* pPersistentGatheringList);

    /*!
    @brief    永続ギャザリングを参加者のPrincipalIdで検索します

    PrincipalIDで指定されるユーザが参加している永続ギャザリングのリストを取得します。

    非同期処理の結果、以下の Result が返ることがあります。
    @errorapplet

    @li    QERROR(Core, InvalidArgument) :  引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    フレンド以外であっても永続ギャザリングの参加状態が取得可能ですが、検索対象ユーザが自分をブロックリストに登録している場合は検索結果に含まれることはありません。

    @param[in,out]    pContext        呼び出し情報
    @param[out]   pPersistentGatheringList  検索でみつかった永続ギャザリングのリスト
    @param[in]    principalId     参加者のPrincipalId
    @param[in]    resultRange    検索範囲

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool FindPersistentGatheringByParticipant(ProtocolCallContext *pContext,
                                     PrincipalID principalId,
                                     const ResultRange& resultRange,
                                     qList<PersistentGathering>* pPersistentGatheringList);

    /*!
    @brief      永続ギャザリングをオーナーの PrincipalId で検索します。

    指定した PrincipalId のユーザーがオーナーとなっている見つかった永続ギャザリングを検索します。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li    QERROR(Core, InvalidArgument) : 引数で与える ResultRange の取得するエントリーの最大サイズに @ref MAX_MATCHMAKE_BROWSE_SIZE を超える件数を指定した。

    @param[in,out]     pContext    呼び出し情報。
    @param[out]    pPersistentGatheringList  検索でみつかった永続ギャザリングのリスト
    @param[in]     pidOwner    検索キーとなる PrincipalId
    @param[in]     resultRange    検索範囲

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool FindPersistentGatheringByOwner(ProtocolCallContext *pContext,
                                     PrincipalID pidOwner,
                                     const ResultRange& resultRange,
                                     qList<PersistentGathering>* pPersistentGatheringList);


    /*!
    @brief    マッチメイクセッションに、マッチメイクセッションシステムパスワードを設定します。

    サーバ側で SYSTEM_PASSWORD_LENGTH 文字のパスワードを生成し、引数で与えたマッチメイクセッションに設定します。
    設定されたパスワードはマッチメイクセッションの参加者全員に、 @ref NotificationEvents::MatchmakeSessionSystemPasswordSet で通知されます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。

    @param[in,out]    pContext        呼び出し情報
    @param[in]    gatheringId     生成対象のギャザリングID
    @param[out]   pSystemPassword  生成されたマッチメイクシステムパスワード

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool GenerateMatchmakeSessionSystemPassword(ProtocolCallContext *pContext, GatheringID gatheringId, String* pSystemPassword);

    /*!
    @brief    マッチメイクセッションに設定されたマッチメイクセッションシステムパスワードを解除します。

    マッチメイクセッションに設定されたマッチメイクセッションシステムパスワードを解除します。
    解除した事はマッチメイクセッションの参加者全員に、 @ref NotificationEvents::MatchmakeSessionSystemPasswordClear で通知されます。

    非同期処理の結果、以下の Result が返ることがあります。これらはアプリケーション側でハンドリングしてください。

    @li QERROR(RendezVous, SessionVoid) : マッチメイクセッションが存在しない。
    @li QERROR(RendezVous, PermissionDenied) : オーナーではない。

    @param[in,out]    pContext        呼び出し情報
    @param[in]    gatheringId     解除対象のギャザリングID

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool ClearMatchmakeSessionSystemPassword(ProtocolCallContext *pContext, GatheringID gatheringId);

    /*!
     @brief    ギャザリングID から表示用の永続ギャザリングコードを生成します

     永続ギャザリングコードはギャザリングIDとタイトル固有のHMACチェックサム値(14bit)から成り、
     10 進 14 桁の正の整数となります(最大 0x3FFFFFFFFFFF まで)。

     ギャザリングIDが INVALID_GATHERINGID、または @ref SetPersistentGatheringCodeKey() で鍵が設定されていない場合、INVALID_PERSISTENT_GATHERING_CODE を返します。

     本関数を使用する前に @ref SetPersistentGatheringCodeKey() にて永続ギャザリングコードのHMAC鍵を設定する必要があります。 @ref Bind() を行っていなくても実行可能です。

     @param[in]    gatheringId    ギャザリングID

     @return    永続ギャザリングコード
     */
    static qUnsignedInt64 GatheringIdToPersistentGatheringCode(GatheringID gatheringId);

    /*!
     @brief    永続ギャザリングコードからギャザリングIDを復元します。

     永続ギャザリングコードが INVALID_PERSISTENT_GATHERING_CODE、鍵が一致していない、または @ref SetPersistentGatheringCodeKey() で鍵が設定されていない場合、
     INVALID_GATHERINGID を返します。

     本関数を使用する前に @ref SetPersistentGatheringCodeKey() にて永続ギャザリングコードのHMAC鍵を設定する必要があります。 @ref Bind() を行っていなくても実行可能です。

     @param[in]    persistentGatheringCode    永続ギャザリングコード

     @return    ギャザリングID
     */
    static GatheringID PersistentGatheringCodeToGatheringId(qUnsignedInt64 persistentGatheringCode);

    /*!
     @brief    永続ギャザリングコードが妥当であるかを判定します。

     本関数を使用する前に @ref SetPersistentGatheringCodeKey() にて永続ギャザリングコードのHMAC鍵を設定する必要があります。 @ref Bind() を行っていなくても実行可能です。

     @param[in]    persistentGatheringCode    永続ギャザリングコード

     @return    妥当な永続ギャザリングコードであるかどうか
     */
    static qBool IsValidPersistentGatheringCode(qUnsignedInt64 persistentGatheringCode);

    /*!
     @brief    永続ギャザリングコードのHMAC鍵を設定します。

     @param[in]    strKey    永続ギャザリングコードのHMAC鍵。自タイトルのアクセスキーを指定してください。
     */
    static void SetPersistentGatheringCodeKey(const String& strKey);

    /*!
     @brief    永続ギャザリングコードのHMAC鍵をクリアします。

     @ref SetPersistentGatheringCodeKey() で永続ギャザリングコードのHMAC鍵を設定した場合、ライブラリ終了時には必ず本関数を呼び出してください。
     */
    static void ClearPersistentGatheringCodeKey();

    /*!
    @brief    指定PIDに任意のイベントを送ります。デバッグ用機能です。製品では使用しないでください。

    @param[in,out]  pContext 呼び出し情報
    @param[in]  pid 通知先PID
    @param[in]  type NotificationEvent::GetType()で受け取る値
    @param[in]  subType NotificationEvent::GetSubType()で受け取る値
    @param[in]  param1 NotificationEvent::GetParam1()で受け取る値
    @param[in]  param2 NotificationEvent::GetParam2()で受け取る値
    @param[in]  stringParam NotificationEvent::GetStringParam()で受け取る値

    @retval     true 非同期処理の開始に成功。
    @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                      これは他の非同期処理で使用中であることを示します。
                      プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
     */
    qBool DebugNotifyEvent(ProtocolCallContext *pContext, PrincipalID pid, qUnsignedInt32 type, qUnsignedInt32 subType, qUnsignedInt64 param1, qUnsignedInt64 param2, const String& stringParam);



private:
    MatchmakeExtensionProtocolClient m_oProtocol;
    static KeyedChecksumAlgorithm* s_pChecksumAlgorithm;
    static qBool s_isSetCommunityCodeKey;
    Time m_ProgressScoreLastUpdateTime;
};

#ifdef FOR_DOC_PURPOSE
/*!
 @brief    マッチメイクセッション参加者の詳細情報を格納するクラスです。
 */
class ParticipantDetails: public RootObject
{
public:
    /*!
     @brief    参加者の プリンシパル ID  です。
     */
    PrincipalID m_idParticipant;
    /*!
     @brief    参加者の名前です。参加者の プリンシパル ID  を文字列化したものとなります。
     */
    String m_strName;
    /*!
     @brief    参加者の詳細情報です。AutoMatchmakeSession(), CreateMatchmakeSession(),
               JoinMatchmakeSession(), EndParticipation() で指定した文字列データがセットされます。
     */
    String m_strMessage;
    /*! @internal
     @brief    参加人数です。
     */
    qUnsignedInt16 m_uiParticipants;
};
#endif

END_NAMESPACE_NEX

