﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once


#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/MatchMaking/src/Common/GatheringID.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/JoinMatchmakeSessionParamDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>


BEGIN_NAMESPACE_NEX

/*!
@brief      MatchmakeExtensionClient::JoinMatchmakeSession() に関するパラメータを格納するデータクラスです。
*/
class JoinMatchmakeSessionParam: public DDLCLASS(JoinMatchmakeSessionParam)
{
public:

    /*!
    @brief      コンストラクタです。

    コンストラクタです。最低限 JoinMatchmakeSessionParam::SetTargetGatheringId() で参加したいギャザリング ID を指定する必要があります。
    */
    JoinMatchmakeSessionParam()
    {
        Reset();
    };
    
    /*!
     @brief    デストラクタです。
     */
    virtual ~JoinMatchmakeSessionParam() {}

    /*!
    @brief      参加する対象のギャザリング ID を設定します。

    初期状態は INVALID_GATHERINGID です。

    @param[in] gid_ 参加する対象のギャザリング ID 
    */
    void SetTargetGatheringId(GatheringID gid_) { this->gid = gid_; }
    /*!
    @brief      参加する対象のギャザリング ID を取得します。

    @return 参加する対象のギャザリング ID 。
    */
    GatheringID GetTargetGatheringId() const { return gid; }

    /*!
    @brief      追加で参加させるユーザーを設定します。

    自分が参加すると同時に他人も同時にマッチメイクセッションに参加させたい場合、本関数でそのプリンシパル ID を指定します。
    例えば、フレンドとチームを組んで対戦したいといった場合に利用できます。
    対象のユーザーは NGS にログインしている必要があります。
    追加で参加したユーザに対しては NotificationEvents::AddedToGathering の通知が送信されます。
    追加で参加したユーザは MatchmakeExtensionClient::FindMatchmakeSessionByGatheringIdDetail() で P2P 通信用の共通鍵を含んだマッチメイクセッションの詳細情報を取得できます。
    MATCHMAKE_SYSTEM_TYPE_FRIEND を使用した場合、追加で参加するユーザーとマッチメイクセッションのオーナー間のフレンド関係はチェックされません。
    初期状態は空です。

    @param[in] additionalParticipants_ 追加で参加させるユーザー。

    @see JoinMatchmakeSessionParam::SetGatheringIdForParticipationCheck()
    */
    void SetAdditionalParticipant(const qList<PrincipalID>& additionalParticipants_) { this->additionalParticipants = additionalParticipants_; }
    /*!
    @brief      追加で参加させるユーザーを取得します。

    @return 追加で参加させるユーザー。
    */
    const qList<PrincipalID>& GetAdditionalParticipants() const { return additionalParticipants; }

    /*!
    @brief      自分と追加で参加させるがユーザーが指定したギャザリングに参加しているかを確認するためのギャザリング ID を設定します。

    自分を含む参加者全員が特定のギャザリングに参加している事を条件とする場合は本関数でそのギャザリング ID を指定します。
    初期状態は INVALID_GATHERINGID です。

    @param[in] gidForParticipationCheck_ 参加を条件とするギャザリングの ID 。INVALID_GATHERINGID を指定するとチェックは行われません。

    @see JoinMatchmakeSessionParam::SetAdditionalParticipant()
    */
    void SetGatheringIdForParticipationCheck(GatheringID gidForParticipationCheck_) { this->gidForParticipationCheck = gidForParticipationCheck_; }
    /*!
    @brief      自分と追加で参加させるがユーザーが指定したギャザリングに参加しているかを確認するためのギャザリング ID を取得します。

    @return 参加を条件とするギャザリングの ID 。
    */
    GatheringID GetGatheringIdForParticipationCheck() const { return gidForParticipationCheck; }

    /*!
    @brief      マッチメイクセッションユーザーパスワードを設定します。

    マッチメイクセッションにユーザーパスワードがセットされている場合に一致が確認されます。
    初期状態は空文字列です。

    @param[in] strUserPassword_ マッチメイクセッションユーザーパスワード。
    @see MatchmakeSession::SetUserPassword()
    */
    void SetUserPassword(const String& strUserPassword_) { this->strUserPassword = strUserPassword_; }
    /*!
    @brief      マッチメイクセッションユーザーパスワードを取得します。

    @return マッチメイクセッションユーザーパスワード。
    */
    const String& GetUserPassword() const { return strUserPassword; }

    /*!
    @brief      マッチメイクセッションシステムパスワードを設定します。

    マッチメイクセッションにシステムパスワードがセットされている場合に一致が確認されます。
    初期状態は空文字列です。

    @param[in] strSystemPassword_ マッチメイクセッションシステムパスワード。
    @see MatchmakeExtensionClient::GenerateMatchmakeSessionSystemPassword()
    */
    void SetSystemPassword(const String& strSystemPassword_) { this->strSystemPassword = strSystemPassword_; }
    /*!
    @brief      マッチメイクセッションシステムパスワードを取得します。

    @return マッチメイクセッションシステムパスワード。
    */
    const String& GetSystemPassword() const { return strSystemPassword; }

    /*!
    @internal
    @brief      JoinMatchmakeSessionOption で列挙されるオプションビットフラグを設定します。

    初期状態は JOIN_MATCHMAKE_SESSION_OPTION_NONE です。

    @param[in] joinMatchmakeSessionOption_ オプションビットフラグ。
    */
    void SetJoinMatchmakeSessionOption(qUnsignedInt32 joinMatchmakeSessionOption_) { this->joinMatchmakeSessionOption = joinMatchmakeSessionOption_; }
    /*!
    @internal
    @brief      JoinMatchmakeSessionOption で列挙されるオプションビットフラグを取得します。

    @return オプションビットフラグ。
    */
    qUnsignedInt32 GetJoinMatchmakeSessionOption() const { return joinMatchmakeSessionOption; }

    /*!
    @brief      JoinMatchmakeSessionBehavior で列挙される参加に関する振る舞いを設定します。

    初期状態は JOIN_MATCHMAKE_SESSION_BEHAVIOR_JOIN_MYSELF です。

    @param[in] joinMatchmakeSessionBehavior_ 参加に関する振る舞い。
    */
    void SetJoinMatchmakeSessionBehavior(JoinMatchmakeSessionBehavior joinMatchmakeSessionBehavior_) { this->joinMatchmakeSessionBehavior = static_cast<qByte>(joinMatchmakeSessionBehavior_); }
    /*!
    @brief      JoinMatchmakeSessionBehavior で列挙される参加に関する振る舞いを取得します。

    @return 参加に関する振る舞い。
    */
    JoinMatchmakeSessionBehavior GetJoinMatchmakeSessionBehavior() const { return static_cast<JoinMatchmakeSessionBehavior>(joinMatchmakeSessionBehavior); }

    /*!
    @brief      ギャザリングのオーナーもしくは参加者に通知されるメッセージを設定します。

    このメッセージはユーザーごとにサーバーに保存され、MatchmakeExtensionClient::GetDetailedParticipants() で取得できます。
    JoinMatchmakeSessionParam::SetAdditionalParticipant() で参加させたユーザーのメッセージも本関数で設定したものになります。
    初期状態は空文字列です。

    @param[in] joinMessage_ ギャザリングのオーナーもしくは参加者に通知されるメッセージ。
    @see MatchmakeExtensionClient::GetDetailedParticipants()
    */
    void SetJoinMessage(const String& joinMessage_) { this->joinMessage = joinMessage_; }
    /*!
    @brief      ギャザリングのオーナーもしくは参加者に通知されるメッセージを取得します。

    @return ギャザリングのオーナーもしくは参加者に通知されるメッセージ。
    */
    const String& GetJoinMessage() const { return joinMessage; }

    /*!
    @internal
    @brief      このクライアントからの参加人数を設定します。。

    自分自身を何人分としてカウントするかを指定します。
    1 以上、かつ作成するマッチメイクセッションの最大参加人数以下の値を設定してください。
    2 以上を指定し、かつ JoinMatchmakeSessionParam::SetAdditionalParticipant() で追加参加者を指定することはできません。
    初期状態は 1 です。

    @param[in] participationCount_ このクライアントからの参加人数。
    */
    void SetParticipationCount(const qUnsignedInt16 participationCount_) { this->participationCount = participationCount_; }
    /*!
    @internal
    @brief      このクライアントからの参加人数を取得します。

    @return このクライアントからの参加人数。
    */
    const qUnsignedInt16& GetParticipationCount() const { return participationCount; }

    /*!
    @internal
    @brief      追加枠人数を設定します。

    設定できる最大人数は MatchmakeConstants::MAX_EXTRA_PARTICIPANTS となります。デフォルトは 0 です。
    指定することで、ギャザリングの最大人数に加えて追加枠人数までギャザリングに参加可能となります。
    フレンドのいるギャザリングへの合流などで使用することを想定しています。

    @param[in] extraParticipants_ 追加枠人数。
    */
    void SetExtraParticipants(qUnsignedInt16 extraParticipants_) { this->extraParticipants = extraParticipants_; }

    /*!
    @internal
    @brief      追加枠人数を取得します。

    @return 追加枠人数。
    */
    qUnsignedInt16 GetExtraParticipants() const { return extraParticipants; }

    /*!
    @brief      ブロックリストの挙動を指定するパラメータを設定します。

    マッチメイクセッションへ参加する際にブロックリストのチェックを行うかどうかを指定するパラメータを設定します。
    blockListParam_ の初期状態は MatchmakeBlockListParam::SetUseMyBlockList(true) および MatchmakeBlockListParam::SetUseBlockListByOthers(true) された状態です。

    @param[in] blockListParam_ ブロックリストの挙動を指定するパラメータ。
    */
    void SetBlockListParam(const MatchmakeBlockListParam& blockListParam_) { this->blockListParam = blockListParam_; }

    /*!
    @brief      ブロックリストの挙動を指定するパラメータを取得します。

    @return ブロックリストの挙動を指定するパラメータ。
    */
    const MatchmakeBlockListParam& GetBlockListParam() const { return this->blockListParam; }

    /*!
    @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetTargetGatheringId(INVALID_GATHERINGID);
        SetAdditionalParticipant(qList<PrincipalID>());
        SetGatheringIdForParticipationCheck(INVALID_GATHERINGID);
        SetJoinMatchmakeSessionOption(JOIN_MATCHMAKE_SESSION_OPTION_NONE);
        SetJoinMatchmakeSessionBehavior(JOIN_MATCHMAKE_SESSION_BEHAVIOR_JOIN_MYSELF);
        SetUserPassword(NEX_T(""));
        SetSystemPassword(NEX_T(""));
        SetJoinMessage(String());
        SetParticipationCount(1);
        SetExtraParticipants(0);
        SetBlockListParam(MatchmakeBlockListParam());
    }

    /*!
    @brief      JoinMatchmakeSessionParam オブジェクトの内容をトレースします。

    @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
    @internal
    @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const JoinMatchmakeSessionParam& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const JoinMatchmakeSessionParam& rhs)
{
    oStr << NEX_T("JoinMatchmakeSessionParam: (")
               << NEX_T("sourceMatchmakeSession=(not implemented)")
               << NEX_T(", joinMessage=") << rhs.joinMessage;

    {
        oStr << NEX_T(", additionalParticipants=(");
        qList<PrincipalID>::const_iterator itr = rhs.additionalParticipants.begin();
        for (int i = 0; itr != rhs.additionalParticipants.end(); ++i, ++itr)
        {
            if (i != 0)
            {
                oStr << NEX_T(", ");
            }
            oStr << *itr;
        }
        oStr << NEX_T(")");
    }

    oStr << NEX_T(", participationCount=") << rhs.participationCount;
    oStr << NEX_T(", extraParticipants=") << rhs.extraParticipants;
    oStr << NEX_T(", blockListParam=") << rhs.blockListParam;
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

