﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/PersistentGatheringDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>

BEGIN_NAMESPACE_NEX

/*!
 @brief    永続ギャザリング種別
 */
namespace PersistentGatheringType
{
/*!
 @brief    永続ギャザリング種別
 */
typedef enum
{
    Open,              //!< 一般永続ギャザリング
    PasswordLocked,    //!< パスワード付永続ギャザリング
    Official           //!< 公式永続ギャザリング
} PersistentGatheringType;
}

// 互換性のための定義
class PersistentGathering;
typedef PersistentGathering Community;
namespace CommunityType
{
#ifndef FOR_DOC_PURPOSE
typedef enum
{
    Open           = PersistentGatheringType::Open,            // 一般コミュニティ
    PasswordLocked = PersistentGatheringType::PasswordLocked,  // パスワード付コミュニティ
    Official       = PersistentGatheringType::Official         // 公式コミュニティ
} CommunityType;
#endif
}

/*!
 @brief 永続ギャザリングの情報を格納するクラスです。
 */
#ifdef FOR_DOC_PURPOSE
class PersistentGathering: public Gathering
#else
class PersistentGathering: public DDLCLASS(PersistentGathering)
#endif
{
public:

    /*!
    @brief      コンストラクタです。

    初期状態は永続ギャザリングタイプはOpen、属性は全て 0 となります。
    */
    PersistentGathering();

    /*!
    @brief      デストラクタです。
    */
    virtual ~PersistentGathering() {};

    /*!
    @brief      セットしたデータを初期化します。

    永続ギャザリングタイプはOpen、属性は全て 0 となります。
    */
    void Reset();

    /*!
    @brief      永続ギャザリングタイプをセットします。

    @param[in] type 永続ギャザリングタイプ
    */
    void SetPersistentGatheringType(PersistentGatheringType::PersistentGatheringType type);
    void SetCommunityType(CommunityType::CommunityType type)
    {
        SetPersistentGatheringType(static_cast<PersistentGatheringType::PersistentGatheringType>(type));
    }

    /*!
     @brief    永続ギャザリングタイプを取得します。

     @return   永続ギャザリングタイプ
     */
    PersistentGatheringType::PersistentGatheringType GetPersistentGatheringType() const;
    CommunityType::CommunityType GetCommunityType() const
    {
        return static_cast<CommunityType::CommunityType>(GetPersistentGatheringType());
    }

    /*!
     @brief    永続ギャザリング参加時のパスワードをセットします。

     パスワードは永続ギャザリングタイプが PasswordLocked の場合にのみ有効です。

     @param[in] password 永続ギャザリング参加時のパスワード
     */
    void SetPassword(const String& password)
    {
        m_Password = password;
    }

    /*!
     @brief    永続ギャザリング参加時のパスワードを取得します。

     ゲームサーバから永続ギャザリングを取得したときにはパスワードはセットされません。

     @return 永続ギャザリング参加時のパスワード
     */
    const String& GetPassword() const
    {
        return m_Password;
    }

    /*!
    @brief      属性をセットします。

    このマッチメイクセッションの属性をセットします。最大6つ指定できます。

    @param[in]     uiIndex    セットする属性のインデックス。0以上NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1以下の値を指定できます。
    @param[in]     val    アプリケーション定義の属性値。
    */
    void SetAttribute(qUnsignedInt32 uiIndex, qUnsignedInt32 val);

    /*!
    @brief      属性を取得します。

    このマッチメイクセッションにセットされた属性を取得します。

    @param[in]     uiIndex    取得する属性のインデックス。0以上NUM_MATCHMAKE_SESSION_ATTRIBUTES - 1以下の値を指定できます。

    @return     指定されたインデックスの属性値。
    */
    qUnsignedInt32 GetAttribute(qUnsignedInt32 uiIndex) const;

    /*!
    @brief      アプリケーション定義のデータをセットします。

    このマッチメイクセッションにアプリケーション定義のデータをセットします。最大512byteです。

    @param[in]     buffer    アプリケーション定義のデータ。
    */
    void SetApplicationBuffer(qVector<qByte> buffer)
    {
        m_ApplicationBuffer = buffer;
    }

    /*!
    @brief      アプリケーション定義のデータを取得します。

    このマッチメイクセッションにセットされたアプリケーション定義のデータを取得します。

    @return    アプリケーション定義のデータの参照
    */
    const qVector<qByte>& GetApplicationBuffer() const
    {
        return m_ApplicationBuffer;
    }

    /*!
    @brief      この永続ギャザリングに参加可能となる日時を設定します。

    この日時より前に MatchmakeExtensionClient::JoinPersistentGathering() を実行しても参加することはできません。
    @ref DateTime::Never を設定すると、参加可能日時の条件は無視されます。
    日時は UTC 時刻で入力してください。

    @param[in]     date    永続ギャザリングに参加可能となる日時
    */
    void SetParticipationStartDate(const DateTime& date)
    {
        m_ParticipationStartDate = date;
    }

    /*!
    @brief      この永続ギャザリングに参加可能となる日時を取得します。

    @return    永続ギャザリングに参加可能となる日時
    */
    const DateTime& GetParticipationStartDate() const
    {
        return m_ParticipationStartDate;
    }

    /*!
    @brief      この永続ギャザリングへの参加締め切り日を設定します。

    この日時より後に MatchmakeExtensionClient::JoinPersistentGathering() を実行しても参加することはできません。
    @ref DateTime::Never を設定すると、参加締め切り日の条件は無視されます。
    日時は UTC 時刻で入力してください。

    @param[in]     date    永続ギャザリングへの参加締め切る日時
    */
    void SetParticipationEndDate(const DateTime& date)
    {
        m_ParticipationEndDate = date;
    }

    /*!
    @brief      この永続ギャザリングへの参加締め切り日を取得します。

    @return    永続ギャザリングへの参加を締め切る日時
    */
    const DateTime& GetParticipationEndDate() const
    {
        return m_ParticipationEndDate;
    }

    /*!
    @brief      この永続ギャザリングに関連する永続ギャザリングマッチの数を取得します。

    @return    この永続ギャザリングに関連する永続ギャザリングマッチの数
    @see MatchmakeSession::SetReferPersistentGatheringId()
    */
    qUnsignedInt32 GetMatchmakeSessionCount() const
    {
        return m_MatchmakeSessionCount;
    }

    /*!
    @brief      この永続ギャザリングへの参加人数を取得します

    @return    この永続ギャザリングへの参加人数
    */
    qUnsignedInt32 GetParticipationCount() const
    {
        return m_ParticipationCount;
    }
};

END_NAMESPACE_NEX

