﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <RendezVous/Services/Jugem/Matchmake/src/Common/UpdateMatchmakeSessionParamDDL.h>
#include <RendezVous/Services/Jugem/Matchmake/src/Common/MatchmakeConstants.h>


BEGIN_NAMESPACE_NEX

/*!
@brief      MatchmakeExtensionClient::UpdateMatchmakeSession() に関するパラメータを格納するデータクラスです。
*/
class UpdateMatchmakeSessionParam: public DDLCLASS(UpdateMatchmakeSessionParam)
{
public:

    /*!
    @brief      コンストラクタです。

    コンストラクタです。
    */
    UpdateMatchmakeSessionParam()
    {
        Reset();
    };
    
    /*!
     @brief    デストラクタです。
     */
    virtual ~UpdateMatchmakeSessionParam() {}

    /*!
    @brief      変更する対象のギャザリング ID を設定します。本パラメータは必ず指定する必要があります。

    初期状態は INVALID_GATHERINGID です。

    @param[in] gid_ 変更する対象のギャザリング ID。
    */
    void SetTargetGatheringId(GatheringID gid_) { this->gid = gid_; }
    /*!
    @brief      変更する対象のギャザリング ID を取得します。

    @return 変更する対象のギャザリング ID 。
    */
    GatheringID GetTargetGatheringId() const { return gid; }

    /*!
    @brief      変更する箇所を表すフラグを設定します。

    変更する箇所を表すフラグを指定します。本パラメータは必ず指定する必要があります。
    MatchmakeSessionModificationFlag に対してビット単位の論理和を用いて複数指定することができます。

    @param[in]  modificationFlag_ 変更する箇所を表すビットフラグ。 MatchmakeSessionModificationFlag に対するビット単位の論理和です。
    */
    void SetModificationFlag(qUnsignedInt32 modificationFlag_)
    {
        this->modificationFlag = modificationFlag_;
    }

    /*!
    @brief      変更フラグを取得します。

    @return     変更する箇所を表すビットフラグ。 MatchmakeSessionModificationFlag に対するビット単位の論理和です。
    */
    qUnsignedInt32 GetModificationFlag() const
    {
        return this->modificationFlag;
    }

    /*!
    @brief      属性を設定します。

    全ての属性を設定します。

    @param[in]  attributes_ 変更後の属性値の配列。
    */
    void SetAttribute(const qUnsignedInt32 attributes_[NUM_MATCHMAKE_SESSION_ATTRIBUTES])
    {
        this->attributes.assign(attributes_, attributes_ + NUM_MATCHMAKE_SESSION_ATTRIBUTES);
    }

    /*!
    @brief      属性を取得します。

    @param[out]  attributes_ 変更後の属性値の配列。
    */
    void GetAttribute(qUnsignedInt32 attributes_[NUM_MATCHMAKE_SESSION_ATTRIBUTES]) const
    {
#ifdef WIN_FAMILY
        // 以下の警告を回避する対応
        // warning C4996: 'std::_Copy_impl': Function call with parameters that may be unsafe
        std::copy(this->attributes.begin(), this->attributes.begin() + NUM_MATCHMAKE_SESSION_ATTRIBUTES,
            stdext::checked_array_iterator<qUnsignedInt32*>(attributes_, NUM_MATCHMAKE_SESSION_ATTRIBUTES));
#else
        std::copy(this->attributes.begin(), this->attributes.begin() + NUM_MATCHMAKE_SESSION_ATTRIBUTES, attributes_);
#endif
    }

    /*!
    @brief      マッチメイクセッションへ参加可能かどうかを設定します。

    このマッチメイクセッションへ参加可能であるかを設定します。

    @param[in]     openParticipation_    変更後のマッチメイクセッションへ参加可能かどうかの状態。
    */
    void SetOpenParticipation(qBool openParticipation_)
    {
        this->openParticipation = openParticipation_;
    }

    /*!
    @brief      マッチメイクセッションへ参加可能かどうかを取得します。

    @return     変更後のマッチメイクセッションへ参加可能かどうかの状態。
    */
    qBool GetOpenParticipation() const
    {
        return openParticipation;
    }

    /*!
    @brief      アプリケーション定義のデータを設定します。

    アプリケーション定義のデータを設定します。最大 MATCHMAKE_BUFFER_MAX_LENGTH byteです。

    @param[in]     applicationBuffer_    変更後のアプリケーション定義のデータ。
    */
    void SetApplicationBuffer(const qVector<qUnsignedInt8>& applicationBuffer_)
    {
        this->applicationBuffer = applicationBuffer_;
    }

    /*!
    @brief      アプリケーション定義のデータを取得します。

    @param[out]     pApplicationBuffer    変更後のアプリケーション定義のデータへのポインタ。
    */
    void GetApplicationBuffer(qVector<qUnsignedInt8>* pApplicationBuffer) const
    {
        SYSTEMCHECK(pApplicationBuffer != NULL);
        *pApplicationBuffer = this->applicationBuffer;
    };

    /*!
    @brief      ゲーム進行度をセットします。

    ゲーム進行度をセットします。

    @param[in]     progressScore_    変更後のアプリケーション定義のゲーム進行度。0 以上 MAX_PROGRESS_SCORE 以下の値を指定できます。
    @return     ゲーム進行度が MAX_PROGRESS_SCORE より大きい場合は、何もせずにfalseを返します。 それ以外の場合は、ゲーム進行度を設定しtrueを返します。
    */
    qBool SetProgressScore(qUnsignedInt8 progressScore_)
    {
        if (progressScore_ > MAX_PROGRESS_SCORE)
        {
            return false;
        }
        this->progressScore = progressScore_;
        return true;
    }

    /*!
    @brief      ゲーム進行度を取得します。

    @return     変更後のアプリケーション定義のゲーム進行度。
    */
    qUnsignedInt8 GetProgressScore() const
    {
        return progressScore;
    }

    /*!
    @brief      マッチメイクパラメータを設定します。

    マッチメイクパラメータを設定します。

    @param[in]     matchmakeParam_    変更後のマッチメイクパラメータ。
    */
    void SetMatchmakeParam(const MatchmakeParam& matchmakeParam_)
    {
        this->matchmakeParam = matchmakeParam_;
    }

    /*!
    @brief      マッチメイクパラメータを取得します。

    @return     変更後のマッチメイクパラメータ。
    */
    const MatchmakeParam& GetMatchmakeParam() const
    {
        return matchmakeParam;
    }

    /*!
    @brief      マッチメイクセッションユーザーパスワードを設定します。

    マッチメイクセッションユーザーパスワードを設定します。
    マッチメイクセッションユーザーパスワードは最大 MAX_MATCHMAKE_SESSION_USER_PASSWORD_LENGTH 文字までセットできます。

    @param[in]     userPassword_    変更後のマッチメイクセッションユーザーパスワード。
    */
    void SetUserPassword(const String& userPassword_)
    {
        this->userPassword = userPassword_;
    }

    /*!
    @brief      マッチメイクセッションユーザーパスワードを取得します。

    @return     変更後のマッチメイクセッションユーザーパスワード。
    */
    const String& GetUserPassword() const
    {
        return userPassword;
    }

    /*!
    @brief      マッチメイク開始日時をセットします。

    マッチメイクを開始した日時をセットします。
    現在時刻より未来の日時が設定された場合は、現在時刻がサーバーでセットされます。

    @param[in]     startedTime_    変更後のマッチメイク開始日時。デフォルトは9999/12/31 23:59:59。
    */
    void SetStartedTime(const DateTime& startedTime_ = DateTime::Future)
    {
        this->startedTime = startedTime_;
    };

    /*!
    @brief      マッチメイク開始日時を取得します。

    @return     変更後のマッチメイク開始日時。
    */
    const DateTime& GetStartedTime() const
    {
        return startedTime;
    }

    /*!
    @brief      ゲームモードをセットします。

    ゲームモードをセットします。

    @param[in]     gameMode_    変更後のゲームモード。
    */
    void SetGameMode(qUnsignedInt32 gameMode_)
    {
        this->gameMode = gameMode_;
    };

    /*!
    @brief      ゲームモードを取得します。

    @return     変更後のゲームモード。
    */
    qUnsignedInt32 GetGameMode() const
    {
        return gameMode;
    }

    /*!
    @brief 説明文字列をセットします。

    説明文字列をセットします。アプリケーションが任意の値を設定できます（最大 MATCHMAKE_STRING_MAX_LENGTH 文字）。

    @param[in]    description_    変更後の説明文字列。
    */
    void SetDescription(const String& description_)
    {
        this->description = description_;
    }

    /*!
    @brief 説明文字列を取得します。

    説明文字列を取得します。

    @return    変更後の説明文字列。
    */
    const String& GetDescription() const
    {
        return description;
    }

    /*!
    @brief 最小参加人数をセットします。

    最小参加人数をセットします。現在、本関数でセットした値はマッチメイクに影響を与えません。

    @param[in]    minParticipants_    変更後の最小参加人数。
    */
    void SetMinParticipants(qUnsignedInt16 minParticipants_)
    {
        this->minParticipants = minParticipants_;
    }

    /*!
    @brief 最小参加人数を取得します

    @return    変更後の最小参加人数。
    */
    qUnsignedInt16 GetMinParticipants() const
    {
        return minParticipants;
    }

    /*!
    @brief 最大参加人数をセットします。

    この人数を越えてギャザリングに参加することはできません。
    0 を指定した場合は最大参加人数が無制限になります。
    現在の参加人数未満の値を指定する事もできます。この場合は既存の参加者が退出させられることはありませんが、
    現在の参加人数が最大参加人数を下回るまでそのマッチメイクセッションに参加することはできません。

    @param[in]    maxParticipants_    変更後の最大参加人数。
     */
    void SetMaxParticipants(qUnsignedInt16 maxParticipants_)
    {
        this->maxParticipants = maxParticipants_;
    }

    /*!
    @brief 最大参加人数を取得します

    @return    変更後の最大参加人数
    */
    qUnsignedInt16 GetMaxParticipants() const
    {
        return maxParticipants;
    }

    /*!
    @brief マッチメイクシステムタイプをセットします。

    MATCHMAKE_SYSTEM_TYPE_ANYBODY と MATCHMAKE_SYSTEM_TYPE_FRIEND との間の変更のみ行えます。
    マッチメイクシステムタイプは変更せず、ポリシー引数のみを変更することもできます。
    MATCHMAKE_SYSTEM_TYPE_ANYBODY もしくは MATCHMAKE_SYSTEM_TYPE_FRIEND 以外への変更や、
    MATCHMAKE_SYSTEM_TYPE_ANYBODY もしくは MATCHMAKE_SYSTEM_TYPE_FRIEND 以外からの変更はエラーになります。

    @param[in]     matchmakeSystemType_    変更後のマッチメイクシステムタイプ。
    @param[in]     policyArgument_    変更後のポリシー引数（デフォルト値 0）
     */
    void SetMatchmakeSystemType(MatchmakeSystemType matchmakeSystemType_, qUnsignedInt32 policyArgument_=0)
    {
        this->matchmakeSystemType = static_cast<qUnsignedInt32>(matchmakeSystemType_);

        switch(this->matchmakeSystemType)
        {
        case MATCHMAKE_SYSTEM_TYPE_FRIEND:
            SetParticipationPolicy(BuiltInParticipationPolicies::FriendOnlyParticipationPolicy, policyArgument_);
            break;
        case MATCHMAKE_SYSTEM_TYPE_ANYBODY:
            SetParticipationPolicy(BuiltInParticipationPolicies::AnybodyParticipationPolicy, policyArgument_);
            break;
        default:
            // AnybodyとFriend以外の場合、同じマッチメイクシステムタイプへの変更は許可される。
            // 異なるマッチメイクシステムタイプに変更しようとした場合はサーバからエラーが返される。
            SetParticipationPolicy(BuiltInParticipationPolicies::MatchmakeSessionParticipationPolicy);
            break;
        }
    }

    /*!
    @brief マッチメイクシステムタイプを取得します。

    @return 変更後のマッチメイクシステムタイプ
    */
    MatchmakeSystemType GetMatchmakeSystemType() const
    {
        return static_cast<MatchmakeSystemType>(this->matchmakeSystemType);
    }

    /*! @internal
    @brief 参加ポリシー引数を取得します。

    @return 変更後の参加ポリシー引数
    */
    qUnsignedInt32 GetPolicyArgument() const
    {
        return this->policyArgument;
    }

    /*!
    @brief      あいことばを設定します。

    空文字列をセットすることにより、あいことばを利用しない設定にすることができます。

    @param[in]  codeword_ あいことばの文字列

    @retval        true  成功。あいことばとして使用できる文字列もしくは空文字列です
    @retval        false 失敗。あいことばとして使用できない文字が含まれています
    */
    qBool SetCodeword(const String& codeword_)
    {
        if (codeword_.Empty())
        {
            this->codeword = NEX_T("");
            return true;
        }
        if (!MatchmakingUtil::ValidateCodeword(codeword_))
        {
            return false;
        }
        this->codeword = codeword_;
        return true;
    }

    /*!
    @brief  あいことばを取得します。

    @return あいことばの文字列
    */
    const String& GetCodeword() const
    {
        return this->codeword;
    }

    /*!
    @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetTargetGatheringId(INVALID_GATHERINGID);
        SetModificationFlag(MATCHMAKE_SESSION_MODIFICATION_FLAG_NONE);
        qUnsignedInt32 attributes_[NUM_MATCHMAKE_SESSION_ATTRIBUTES];
        memset(attributes_, 0, sizeof(attributes_));
        SetAttribute(attributes_);
        SetOpenParticipation(true);
        SetApplicationBuffer(qVector<qUnsignedInt8>());
        SetProgressScore(MAX_PROGRESS_SCORE);
        SetMatchmakeParam(MatchmakeParam());
        SetUserPassword(String());
        SetStartedTime(DateTime::Never);
        SetGameMode(0);
        SetDescription(String());
        SetMaxParticipants(0);
        SetMinParticipants(0);
        SetCodeword(NEX_T(""));
    }

    /*!
    @brief      UpdateMatchmakeSessionParam オブジェクトの内容をトレースします。

    @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
    @internal
    @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const UpdateMatchmakeSessionParam& rhs);

private:
    void SetParticipationPolicy(qUnsignedInt32 participationPolicy_, qUnsignedInt32 policyArgument_=0)
    {
        this->participationPolicy = participationPolicy_;
        this->policyArgument = policyArgument_;
    }
};

inline StringStream& operator<<(StringStream& oStr, const UpdateMatchmakeSessionParam& rhs)
{
    oStr << NEX_T("UpdateMatchmakeSessionParam: (")
               << NEX_T("gid=") << rhs.gid
               << NEX_T(", modificationFlag=") << rhs.modificationFlag;

    {
        oStr << NEX_T(", attributes=(");
        qVector<qUnsignedInt32>::const_iterator itr = rhs.attributes.begin();
        for (int i = 0; itr != rhs.attributes.end(); ++i, ++itr)
        {
            if (i != 0)
            {
                oStr << NEX_T(", ");
            }
            oStr << *itr;
        }
        oStr << NEX_T(")");
    }

    oStr << NEX_T(", openParticipation=") << rhs.openParticipation
         << NEX_T(", applicationBuffer=(not implemented)")
         << NEX_T(", progressScore=") << rhs.progressScore
         << NEX_T(", matchmakeParam=") << rhs.matchmakeParam
         << NEX_T(", startedTime=") << rhs.startedTime
         << NEX_T(", userPassword=") << rhs.userPassword
         << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

