﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeConstants.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeDDF.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeProtocolDDL.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeErrors.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeSuccesses.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeStatsTarget.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereePersonalRoundResult.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeStats.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeStatsInitParam.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeRound.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeEndRoundParam.h>
#include <RendezVous/Services/Jugem/MatchmakeReferee/src/Common/MatchmakeRefereeStartRoundParam.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief MatchmakeReferee 機能を利用するためのクラス。
*/
class MatchmakeRefereeClient : public ServiceClient
{
public:

    /*!
    @brief      コンストラクタです。

       コンストラクタです。
    */
    MatchmakeRefereeClient();

    /*!
    @brief      デストラクタです。

       デストラクタです。
    */
    virtual ~MatchmakeRefereeClient();

    /*!
        @brief    サービスを利用するための証明書を登録します。

        サービスを利用するための証明書をセットします。
        機能を利用する前にセットする必要があります。@ref NgsFacade::Login() の非同期処理に成功後、
        @ref NgsFacade::GetCredentials() で取得した証明書を引数に指定してください。
        @ref Bind() で証明書をセットした後もう一度 @ref Bind() を呼び出すと証明書を差し替える事ができます。

        @param[in]     pCredentials    サービスにアクセスするための証明書。

        @retval     true 成功。pCredentials が NULL でない場合は必ず成功します。
        @retval     false pCredentials に NULL が指定された。
    */
    virtual qBool Bind(Credentials* pCredentials);

    /*!
        @brief     証明書の登録を解除します。

        @ref Bind() でセットされた証明書を解除します。@ref Bind() が呼び出されていなくても本関数を呼び出すことができます。
        本オブジェクトを破棄する際に自動的に @ref Unbind() が呼び出されるため、明示的に呼び出す必要はありません。

        @retval      true 証明書の解除を行った。
        @retval      false Bind() が行われていないため何も行わなかった。
    */
    virtual qBool Unbind();

    void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

    /*!
        @brief      ラウンドを開始します。

        MatchmakeRefereeStartRoundParam::SetPrincipalIds() で指定した人がラウンドの参加者になります。<br>
        MatchmakeRefereeStartRoundParam::SetGatheringId() で指定した Gatherig の参加者のみ MatchmakeRefereeStartRoundParam::SetPrincipalIds() に指定できます。<br>
        ラウンドの参加者は開始した後に EndRound() もしくは EndRoundWithoutReport() を呼ぶ必要があります。<br>
        StartRound() されたことは NotificationEvents::RoundStarted により、ラウンド参加者に通知されます。<br>

        MatchmakeRefereeStartRoundParam::SetPrincipalIds() に空もしくは重複したプリンシパル ID を指定すると、 QERROR(MatchmakeReferee, InvalidArgument) になります。<br>
        MatchmakeRefereeStartRoundParam::SetPrincipalIds() に指定した人のうち、MatchmakeRefereeStartRoundParam::SetGatheringId() で指定した Gathering の参加者に含まれない人がいた場合、
        QERROR(MatchmakeReferee, NotParticipatedGathering) になります。

        @param[in,out] pContext 呼び出し情報。
        @param[in] param 試合開始パラメータ。
        @param[out] roundId ラウンドID。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool StartRound(ProtocolCallContext* pContext, const MatchmakeRefereeStartRoundParam& param, qUnsignedInt64* roundId);

    /*!
        @brief      StartRound() で設定したラウンドのパラメータを取得します。

        取得対象のラウンドが存在しない場合、 QERROR(MatchmakeReferee, RoundNotFound) が返ります。

        @param[in,out] pContext 呼び出し情報。
        @param[in] roundId 取得対象のラウンドID。
        @param[out] param StartRound() で設定したパラメータ。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetStartRoundParam(ProtocolCallContext* pContext, qUnsignedInt64 roundId, MatchmakeRefereeStartRoundParam* param);

    /*!
        @brief      ラウンドを終了し、レポートを送信します。

        ラウンドの結果をサーバにレポートします。<br>
        ラウンドが終了したときに呼びます。<br>
        全員分のレポートが集まるか、切断されるか、最初にレポートが送信されてから1分で締め切り集計処理を行われます。
        StartRound() を呼び出してから本 API が最初に呼ばれたときはラウンド参加者に NotificationEvents::FirstRoundReportReceived 
        通知が届きます。<br>
        集計処理が完了すると、 NotificationEvents::RoundSummarized 通知がラウンド参加者に届きます。

        送信したレポートの集計処理について、詳しくはプログラミングマニュアルを参照してください。

        endRoundParam にセットする personalRoundResults のうち、buffer のサイズが MatchmakeRefereeConstants::STATS_BUFFER_MAX_SIZE 
        を超える場合、 QERROR(MatchmakeReferee, InvalidArgument) が返ります。<br>
        endRoundParam にセットする personalRoundResults のサイズがラウンド参加者数と異なる場合、 QERROR(MatchmakeReferee, InvalidArgument) が返ります。<br>
        レポート対象のラウンドが存在しない場合、 QERROR(MatchmakeReferee, RoundNotFound) が返ります。<br>
        レポート対象のラウンドが既にレポート済みの場合、 QERROR(MatchmakeReferee, AlreadyExists) が返ります。<br>
        レポート対象のラウンドが既に集計済みの場合、 QERROR(MatchmakeReferee, RoundArbitrated) が返ります。<br>
        レポート対象のラウンドに参加していない場合、 QERROR(MatchmakeReferee, NotParticipatedRound) が返ります。

        @param[in,out] pContext 呼び出し情報。
        @param[in] endRoundParam サーバに送信するレポート。 endRoundParam.personalRoundResults にラウンド参加者全員分の結果をセットします。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool EndRound(ProtocolCallContext* pContext, const MatchmakeRefereeEndRoundParam& endRoundParam);

    /*!
        @brief      レポートを送信せず Round を終了します。

        自らラウンドを抜けるときに呼びます。本 API で抜けた場合は他のラウンド参加者が「この人は切断した」と見なすため
        切断扱いになります。<br>
        負けた後に抜けた人に対しては残ったラウンド参加者が切断のレポートではなく負け扱いのレポートを送る必要があります。
        本 API では、集計処理のカウントダウンは開始されません。本 API を呼ばなくても最終的に同じ結果になりますが、
        集計時の 1 分のタイムアウトを待つ必要が無くなるので呼ぶようにして下さい。<br>
        集計処理が完了すると、 NotificationEvents::RoundSummarized 通知がラウンド参加者に届きます。

        レポート対象のラウンドが存在しない場合、 QERROR(MatchmakeReferee, RoundNotFound) が返ります。<br>
        レポート対象のラウンドが既にレポート済みの場合、 QERROR(MatchmakeReferee, AlreadyExists) が返ります。<br>
        レポート対象のラウンドが既に集計済みの場合、 QERROR(MatchmakeReferee, RoundArbitrated) が返ります。<br>
        レポート対象のラウンドに参加していない場合、 QERROR(MatchmakeReferee, NotParticipatedRound) が返ります。

        @param[in,out] pContext 呼び出し情報。
        @param[in] roundId レポート対象のラウンドID

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool EndRoundWithoutReport(ProtocolCallContext* pContext, qUnsignedInt64 roundId);

    /*!
        @brief Round参加者を取得します。

        @param[in,out] pContext 呼び出し情報。
        @param[in] roundId 対象のラウンド ID
        @param[out] pids ラウンド参加者のプリンシパル ID リスト

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetRoundParticipants(ProtocolCallContext* pContext, qUnsignedInt64 roundId, qVector<PrincipalID>* pids);

    /*!
        @brief 現在自分が参加していて、かつ未集計のラウンドを取得します。

        電源断等をしたあとに重複してラウンドに参加しないようにするためには、この関数を使用して確認して下さい。

        @param[in,out] pContext 呼び出し情報。
        @param[out] rounds 自分が参加中の未集計ラウンド。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetNotSummarizedRound(ProtocolCallContext* pContext, qVector<MatchmakeRefereeRound>* rounds);

    /*!
        @brief ラウンドを取得します。

        集計が完了していれば正規化されたレポートを取得できます。<br>

        対象のラウンドが存在しない場合、 QERROR(MatchmakeReferee, RoundNotFound) が返ります。

        @param[in,out] pContext 呼び出し情報。
        @param[in] roundId 対象のラウンドID。
        @param[out] round ラウンド情報。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetRound(ProtocolCallContext* pContext, qUnsignedInt64 roundId, MatchmakeRefereeRound* round);

    /*!
    @refmanoverload{GetStatsPid}

       @brief 対象プリンシパル ID の個人統計を取得します。

        個人統計が存在しない場合、 QERROR(MatchmakeReferee, StatsNotFound) が返ります。

        @param[in,out] pContext 呼び出し情報。
        @param[in] target 取得対象。
        @param[out] stats 個人統計。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetStats(ProtocolCallContext* pContext, const MatchmakeRefereeStatsTarget& target, MatchmakeRefereeStats* stats);

    /*!
    @refmanoverload{GetStatsPrimaryRelatedNexUniqueId}

       @brief 対象プリンシパル ID のプライマリ関連 NEX ユニーク IDの個人統計を取得します。

        @param[in,out] pContext 呼び出し情報。
        @param[in] targets 取得対象。
        @param[out] stats 個人統計 (取得対象にセットした順で格納されます)。
        @param[out] results 取得結果: QSUCCESS(MatchmakeReferee, Success) 又は 
                                      QERROR(MatchmakeReferee, StatsNotFound) (取得対象にセットした順で格納されます)。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetStats(ProtocolCallContext* pContext, const qVector<MatchmakeRefereeStatsTarget>& targets, qVector<MatchmakeRefereeStats>* stats, qVector<qResult>* results);

    /*!
    @refmanoverload{GetStatsAllRelatedNexUniqueId}

        @brief 対象プリンシパル ID の全ての関連 NEX ユニーク ID の個人統計を取得します。

        個人統計が見つからない場合、 QERROR(MatchmakeReferee, StatsNotFound) が返ります。

        @param[in,out] pContext 呼び出し情報。
        @param[in] target 取得対象。
        @param[out] stats 個人統計。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetStats(ProtocolCallContext* pContext, const MatchmakeRefereeStatsTarget& target, qVector<MatchmakeRefereeStats>* stats);

    /*!
        @brief プライマリ関連 NEX ユニーク ID に紐付く個人統計を新規作成します。

       個人統計が既に存在する場合、 QERROR(MatchmakeReferee, AlreadyExists) が返ります。

        @param[in,out] pContext 呼び出し情報。
        @param[in] param 作成パラメータ。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool CreateStats(ProtocolCallContext* pContext, const MatchmakeRefereeStatsInitParam& param);

    /*!
        @brief プライマリ関連 NEX ユニーク ID に紐付く個人統計があればそれを取得し、存在しなければ新規作成してそれを取得します。

        @param[in,out] pContext 呼び出し情報。
        @param[in] param 作成パラメータ。
        @param[out] stats 個人統計 新規作成した個人統計か、既存の個人統計が格納されます。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetOrCreateStats(ProtocolCallContext* pContext, const MatchmakeRefereeStatsInitParam& param, MatchmakeRefereeStats* stats);


private:
    MatchmakeRefereeProtocolClient m_oProtocol;
};

END_NAMESPACE_NEX

