﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Services/Jugem/Utility/src/Common/UniqueId.h>
#include "MatchmakeRefereeStatsDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief    個人統計を表すクラスです。
*/
class MatchmakeRefereeStats: public DDLCLASS(MatchmakeRefereeStats)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    MatchmakeRefereeStats()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~MatchmakeRefereeStats() {}

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        this->uniqueId = INVALID_UNIQUEID;
        this->category = 0;
        this->pid = 0;
        this->recentDisconnection = 0;
        this->recentViolation = 0;
        this->recentMismatch = 0;
        this->recentWin = 0;
        this->recentLoss = 0;
        this->recentDraw = 0;
        this->totalDisconnect = 0;
        this->totalViolation = 0;
        this->totalMismatch = 0;
        this->totalWin = 0;
        this->totalLoss = 0;
        this->totalDraw = 0;
        this->ratingValue = 0;
    }

    /*!
        @brief      このオブジェクトが有効かどうかを調べます。

        返値が false だった場合は内部パラメータは無効ですので値を取得しないでください。

        @return     このオブジェクトが有効なら true 、無効なら false 。
    */
    qBool IsValid() const
    {
        return this->uniqueId != INVALID_UNIQUEID;
    }

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt64 GetUniqueId() const
    {
        return this->uniqueId;
    }

    /*!
        @brief      カテゴリを取得します。

        @return     カテゴリ。
    */
    qUnsignedInt32 GetCategory() const
    {
        return this->category;
    }

    /*!
        @brief      MatchmakeReferee の個人統計を更新する際、最後に使われたプリンシパル ID を取得します。

        @return     プリンシパル ID。
    */
    PrincipalID GetPrincipalId() const
    {
        return this->pid;
    }

    /*!
        @brief      直近 100 ラウンド分の切断数を取得します。

        切断数は、レポートの正規化結果から切断と判断された場合、カウントアップされます。

        @return     切断数。
    */
    qUnsignedInt32 GetRecentDisconnectCount() const
    {
        return this->recentDisconnection;
    }

    /*!
        @brief      直近 100 ラウンド分の違反数を取得します。

        違反数は、レポートの正規化結果から違反と判断された場合、カウントアップされます。

        @return     違反数。
    */
    qUnsignedInt32 GetRecentViolationCount() const
    {
        return this->recentViolation;
    }

    /*!
        @brief      直近 100 ラウンド分のレポート不一致数を取得します。

        レポート不一致数は、レポートの正規化結果から、過半数となったレポートと異なるレポートを
        送信した場合に、カウントアップされます。

        @return     レポート不一致数。
    */
    qUnsignedInt32 GetRecentMismatchCount() const
    {
        return this->recentMismatch;
    }

    /*!
        @brief      直近 100 ラウンド分の勝ち数を取得します。

        勝ち数は、レポートの正規化結果から勝ちと判断された場合、カウントアップされます。

        @return     勝ち数。
    */
    qUnsignedInt32 GetRecentWinCount() const
    {
        return this->recentWin;
    }

    /*!
        @brief      直近 100 ラウンド分の負け数を取得します。

        負け数は、レポートの正規化結果から負けと判断された場合、カウントアップされます。

        @return     負け数。
    */
    qUnsignedInt32 GetRecentLossCount() const
    {
        return this->recentLoss;
    }

    /*!
        @brief      直近 100 ラウンド分の引き分け数を取得します。

        引き分け数は、レポートの正規化結果から引き分けと判断された場合、カウントアップされます。

        @return     引き分け数。
    */
    qUnsignedInt32 GetRecentDrawCount() const
    {
        return this->recentDraw;
    }

    /*!
        @brief      切断数の合計を取得します。

        切断数は、レポートの正規化結果から切断と判断された場合、カウントアップされます。

        @return     切断数。
    */
    qUnsignedInt32 GetTotalDisconnectCount() const
    {
        return this->totalDisconnect;
    }

    /*!
        @brief      違反数の合計を取得します。

        違反数は、レポートの正規化結果から違反と判断された場合、カウントアップされます。

        @return     違反数。
    */
    qUnsignedInt32 GetTotalViolationCount() const
    {
        return this->totalViolation;
    }

    /*!
        @brief      レポート不一致数の合計を取得します。

        レポート不一致数は、レポートの正規化結果から、過半数となったレポートと異なるレポートを
        送信した場合に、カウントアップされます。

        @return     レポート不一致数。
    */
    qUnsignedInt32 GetTotalMismatchCount() const
    {
        return this->totalMismatch;
    }

    /*!
        @brief      勝ち数の合計を取得します。

        勝ち数は、レポートの正規化結果から勝ちと判断された場合、カウントアップされます。

        @return     勝ち数。
    */
    qUnsignedInt32 GetTotalWinCount() const
    {
        return this->totalWin;
    }

    /*!
        @brief      負け数の合計を取得します。

        負け数は、レポートの正規化結果から負けと判断された場合、カウントアップされます。

        @return     負け数。
    */
    qUnsignedInt32 GetTotalLossCount() const
    {
        return this->totalLoss;
    }

    /*!
        @brief     引き分け数の合計を取得します。

        引き分け数は、レポートの正規化結果から引き分けと判断された場合、カウントアップされます。

        @return     引き分け数。
    */
    qUnsignedInt32 GetTotalDrawCount() const
    {
        return this->totalDraw;
    }

    /*!
        @brief     レート値を取得します。

        統計を計算した結果が qUnsignedInt32 で収まらない値の場合には結果が丸められます。

        @li ratingValue < 0 の場合には 0 が返ります。
        @li ratingValue > std::numeric_limits<qUnsignedInt32>::max() の場合には std::numeric_limits<qUnsignedInt32>::max() が返ります。

        @return    レート値。
    */
    qUnsignedInt32 GetRatingValue() const
    {
        return this->ratingValue;
    }

    /*!
        @brief      MatchmakeRefereeStatsオブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag()の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, NEX_T("MatchmakeRefereeStats: (")
               << NEX_T("uniqueId=") << hex << this->uniqueId
               << NEX_T(", category=") << dec << this->category
               << NEX_T(", pid=") << pid
               << NEX_T(", recentDisconnection=") << this->recentDisconnection
               << NEX_T(", recentViolation=") << this->recentViolation
               << NEX_T(", recentMismatch=") << this->recentMismatch
               << NEX_T(", recentWin=") << this->recentWin
               << NEX_T(", recentLoss=") << this->recentLoss
               << NEX_T(", recentDraw=") << this->recentDraw
               << NEX_T(", totalDisconnect=") << this->totalDisconnect
               << NEX_T(", totalViolation=") << this->totalViolation
               << NEX_T(", totalMismatch=") << this->totalMismatch
               << NEX_T(", totalWin=") << this->totalWin
               << NEX_T(", totalLoss=") << this->totalLoss
               << NEX_T(", totalDraw=") << this->totalDraw
               << NEX_T(", ratingValue=") << this->ratingValue
               << NEX_T(")"));
    }
};

END_NAMESPACE_NEX

