﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>

#if defined(CAFE_TARGET)
#include <nn/ffl/FFLStandard.h>
#elif defined(CTR_TARGET)
#include <nn/mii/mii_StoreData.h>
#endif

#include "Ranking2CommonDataDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      ユーザー共通データを表すクラスです。
*/
class Ranking2CommonData: public DDLCLASS(Ranking2CommonData)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    Ranking2CommonData()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~Ranking2CommonData() {}

    /*!
        @brief      ユーザー名を取得します。

        @return     ユーザー名。
    */
    const String& GetUserName() const
    {
        return userName;
    };

    /*!
        @brief      ユーザー名をセットします。

        指定できる文字数は、半角全角ともに最大で Ranking2Constants::MAX_USERNAME_LENGTH です。

        @param[in]  userName_  ユーザー名。
    */
    void SetUserName(const String& userName_)
    {
        this->userName = userName_;
    };

#if defined(NEX_GET_MII_SUPPORT)
    /*!
        @refmanoverload{name1}
        @internal

        @brief      Mii データを取得します。

        暗号化されたもの(CTRのもの)なら112byte, 復号されたもの(WiiUのもの)なら96byteのデータが格納される

        @param[out]  mii_ Mii。
        @return 取得できたかどうか
    */
    qBool GetMii(qVector<qByte>& mii_) const
    {
        if (this->mii.empty())
        {
            return false;
        }
        // qBuffer -> qVector<qByte>への変換
        mii_.assign(this->mii.begin(), this->mii.end());
        return true;
    }

#if defined(CAFE_TARGET)
    /*!
        @refmanoverload{cafe}

        @brief      Mii データを FFLStoreData 形式で取得します。

        Ranking2GetOptionFlags::MII_REQUIRED のフラグを有効にして取得したランキング情報からのみ
        Mii データを取得できます。
        取得した Mii データを利用するには別途「似顔絵ライブラリ」が必要です。

        @param[out]  pMiiData	Mii データを格納するバッファ。
        @return 取得できたかどうか
    */
    qBool GetMii(FFLStoreData* pMiiData) const
    {
        if (mii.size() != sizeof(FFLStoreData))
        {
            // 取得しなかった、もしくは暗号化されていて読めなかった
            return false;
        }
        // qBuffer -> FFLStoreData への変換
        this->mii.copy(mii.begin(), sizeof(FFLStoreData), static_cast<qByte*>(pMiiData->data));
        return true;
    }
#endif

#if defined(CTR_TARGET)
    /*!
        @refmanoverload{ctr}

        @brief      Mii データを nn::mii::StoreData 形式で取得します。

        Ranking2GetOptionFlags::MII_REQUIRED のフラグを有効にして取得したランキング情報からのみ
        Mii データを取得できます。
        取得した Mii データを利用するには別途「似顔絵ライブラリ」が必要です。

        @param[out]  pMiiData	Mii データを格納するバッファ。
        @return 取得できたかどうか
    */
    qBool GetMii(nn::mii::StoreData* pMiiData) const;
#endif

#endif // end of #if defined(NEX_GET_MII_SUPPORT)

    /*!
        @refmanoverload{name1}

        @brief      バイナリデータを取得します。

        @return     バイナリデータ。
    */
    qVector<qByte> GetBinaryData() const
    {
        // qBuffer -> qVector<qByte>への変換
        qVector<qByte> vb;
        vb.assign(this->binaryData.begin(), this->binaryData.end());
        return vb;
    };

    /*!
        @refmanoverload{name2}

        @brief      バイナリデータを取得します。

        こちらのオーバーロード関数の方が効率的です。

        @param[out]  binaryData_ バイナリデータ。
    */
    void GetBinaryData(qVector<qByte>& binaryData_) const
    {
        // qBuffer -> qVector<qByte>への変換
        binaryData_.assign(this->binaryData.begin(), this->binaryData.end());
    }

    /*!
        @brief      バイナリデータをセットします。

        指定できるデータは最大で Ranking2Constants::MAX_BINARY_DATA_LENGTH バイトです。

        @param[in]  binaryData_  バイナリデータ。
    */
    void SetBinaryData(qVector<qByte>& binaryData_)
    {
        this->binaryData.clear();
        for(qVector<qByte>::iterator it = binaryData_.begin(); it != binaryData_.end(); it++)
        {
            this->binaryData.push_back(*it);
        }
    };

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetUserName("");
        mii.clear();
        binaryData.clear();
    }

    /*!
        @brief      Ranking2CommonData オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const Ranking2CommonData& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const Ranking2CommonData& rhs)
{
    oStr << NEX_T("Ranking2CommonData: (")
               << NEX_T("userName=") << rhs.userName;

    oStr << NEX_T(", binaryData=");
    for (qBuffer::const_iterator it = rhs.binaryData.begin(); it != rhs.binaryData.end(); ++it)
    {
        oStr << hex << *it;
    }

    oStr << NEX_T(", mii=");
    for (qBuffer::const_iterator it = rhs.mii.begin(); it != rhs.mii.end(); ++it)
    {
        oStr << hex << *it;
    }

    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

