﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "Ranking2GetByListParamDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      ランキングの取得に関するパラメータを表すクラスです。
*/
class Ranking2GetByListParam: public DDLCLASS(Ranking2GetByListParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    Ranking2GetByListParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~Ranking2GetByListParam() {}

    /*!
        @brief      カテゴリを取得します。

        @return     カテゴリの値。
    */
    qUnsignedInt32 GetCategory() const
    {
        return category;
    };

    /*!
        @brief      カテゴリをセットします。

        @param[in]  category_  カテゴリの値。デフォルトは 0。
    */
    void SetCategory(qUnsignedInt32 category_)
    {
        this->category = category_;
    };

    /*!
        @brief      さかのぼるシーズンの数を取得します。

        @return     何回前のシーズンのランキングを取得するかを指定する値。
                    最新のランキングは 0 で、過去に向かって 1,2,3 …と増えます。デフォルト 0。
    */
    qByte GetNumSeasonsToGoBack() const
    {
        return numSeasonsToGoBack;
    };

    /*!
        @brief      さかのぼるシーズンの数をセットします。

        @param[in]  numSeasonsToGoBack_  何回前のシーズンのランキングを取得するかを指定する値。
                    最新のランキングは 0 で、過去に向かって 1,2,3 …と増えます。デフォルト 0。
                    0 ～ カテゴリ設定の過去シーズンの保存数の範囲内で指定します。
    */
    void SetNumSeasonsToGoBack(qByte numSeasonsToGoBack_)
    {
        this->numSeasonsToGoBack = numSeasonsToGoBack_;
    };

    /*!
        @brief      開始位置を取得します。

        @return     開始位置の値。
    */
    qUnsignedInt32 GetOffset() const
    {
        return offset;
    };

    /*!
        @brief      開始位置をセットします。

        @param[in]  offset_ 開始位置の値。デフォルトは 0。
                           有効な範囲は 0 ～ 最低順位 - 1 です。
    */
    void SetOffset(qUnsignedInt32 offset_)
    {
        this->offset = offset_;
    };

    /*!
        @brief      取得長を取得します。

        @return     取得長の値。
    */
    qUnsignedInt32 GetLength() const
    {
        return length;
    };

    /*!
        @brief      取得長をセットします。

        @param[in]  length_  取得長の値。デフォルトは 10。
                            有効な範囲は 1 ～ Ranking2Constants::MAX_RANKING_LENGTH です。
    */
    void SetLength(qUnsignedInt32 length_)
    {
        this->length = length_;
    };

    /*!
        @brief      ソートのオプション指定フラグを取得します。

        @return     ソートのオプション指定フラグの値。

        @see        Ranking2SortFlags
    */
    qUnsignedInt32 GetSortFlags() const
    {
        return sortFlags;
    };

    /*!
        @brief      ソートのオプション指定フラグをセットします。

        @param[in]  sortFlags_  ソートのオプション指定フラグの値。デフォルトは 0。

        @see        Ranking2SortFlags
    */
    void SetSortFlags(qUnsignedInt32 sortFlags_)
    {
        this->sortFlags = sortFlags_;
    };

    /*!
        @brief      オプション指定フラグを取得します。

        @return     オプション指定フラグ。 

        @see        Ranking2GetOptionFlags
    */
    qUnsignedInt32 GetOptionFlags() const
    {
        return optionFlags;
    };

    /*!
        @brief      オプション指定フラグをセットします。

        @param[in]  optionFlags_  オプション指定フラグ。

        @see        Ranking2GetOptionFlags
    */
    void SetOptionFlags(qUnsignedInt32 optionFlags_)
    {
        this->optionFlags = optionFlags_;
    };

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetCategory(0);
        SetNumSeasonsToGoBack(0);
        SetOffset(0);
        SetLength(10);
        SetSortFlags(0);
        SetOptionFlags(0);
    }

    /*!
        @brief      Ranking2GetByListParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const Ranking2GetByListParam& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const Ranking2GetByListParam& rhs)
{
    oStr << NEX_T("Ranking2GetByListParam: (")
               << NEX_T("category=") << rhs.category
               << NEX_T(", offset=") << rhs.offset
               << NEX_T(", length=") << rhs.length
               << NEX_T(", numSeasonsToGoBack=") << rhs.numSeasonsToGoBack
               << NEX_T(", sortFlags=") << rhs.sortFlags
               << NEX_T(", optionFlags=") << rhs.optionFlags;
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

