﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemDDF.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemProtocolDDL.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Common/ServiceItemErrors.h>
#include <RendezVous/Services/Jugem/ServiceItem/src/Client/ServiceItemInternalClientInterface.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief サービスアイテム機能のためのサーバー通信処理を行うクラスです。

    開発者が直接このインタフェースを使用することはありません。
    ServiceItemClient を使用してください。
*/
class ServiceItemInternalClient : public ServiceClient, public ServiceItemInternalClientInterface
{
public:

    /*!
    @brief      コンストラクタです。

    コンストラクタです。
    */
    ServiceItemInternalClient();

    /*!
    @brief      デストラクタです。

    デストラクタです。
    */
    virtual ~ServiceItemInternalClient();

    /*!
        @brief    サービスを利用するための証明書を登録します。

        サービスを利用するための証明書をセットします。
        機能を利用する前にセットする必要があります。@ref NgsFacade::Login() の非同期処理に成功後、
        @ref NgsFacade::GetCredentials() で取得した証明書を引数に指定してください。
        @ref Bind() で証明書をセットした後もう一度 @ref Bind() を呼び出すと証明書を差し替える事ができます。

        @param[in]     pCredentials    サービスにアクセスするための証明書。

        @retval     true 成功。pCredentials が NULL でない場合は必ず成功します。
        @retval     false pCredentials に NULL が指定された。
    */
    virtual qBool Bind(Credentials* pCredentials);

    /*!
        @brief     証明書の登録を解除します。

        @ref Bind() でセットされた証明書を解除します。@ref Bind() が呼び出されていなくても本関数を呼び出すことができます。
        本オブジェクトを破棄する際に自動的に @ref Unbind() が呼び出されるため、明示的に呼び出す必要はありません。

        @retval      true 証明書の解除を行った。
        @retval      false Bind() が行われていないため何も行わなかった。
    */
    virtual qBool Unbind();

    virtual Credentials* GetCredentials() const
    {
        return ServiceClient::GetCredentials();
    }

    void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

    /*!
        @internal
        @brief      NGS/ショップの環境情報を取得。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  uniqueId クライアントのユニークID。
        @param[out] str NGS/ショップの環境情報。NGSの環境名+ショップの接続先ドメイン。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetEnvironment(ProtocolCallContext *pContext, qUnsignedInt32 uniqueId, String* str);

    virtual qBool HttpGetRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemHttpGetParam& request);
    virtual qBool HttpGetResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemHttpGetResponse* response);

    virtual qBool PurchaseServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemPurchaseServiceItemParam& request);
    virtual qBool PurchaseServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemPurchaseServiceItemResponse* response);

    virtual qBool ListServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemListServiceItemParam& request);
    virtual qBool ListServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemListServiceItemResponse* response);

    virtual qBool GetBalanceRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetBalanceParam& request);
    virtual qBool GetBalanceResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetBalanceResponse* response);

    virtual qBool GetPrepurchaseInfoRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetPrepurchaseInfoParam& request);
    virtual qBool GetPrepurchaseInfoResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetPrepurchaseInfoResponse* response);

    virtual qBool GetServiceItemRightRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetServiceItemRightParam& request);
    virtual qBool GetServiceItemRightResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetServiceItemRightResponse* response);

    virtual qBool GetServiceItemRightStatementRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetServiceItemRightParam& request);
    virtual qBool GetServiceItemRightStatementResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetServiceItemRightResponse* response);

    virtual qBool GetPurchaseHistoryRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetPurchaseHistoryParam& request);
    virtual qBool GetPurchaseHistoryResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetPurchaseHistoryResponse* response);

    virtual qBool GetLawMessageRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetLawMessageParam& request);
    virtual qBool GetLawMessageResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetLawMessageResponse* response);

    virtual qBool ListAutoBillingServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemListAutoBillingServiceItemParam& request);
    virtual qBool ListAutoBillingServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemListAutoBillingServiceItemResponse* response);

    virtual qBool GetAutoBillingContractRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemGetAutoBillingContractParam& request);
    virtual qBool GetAutoBillingContractResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemGetAutoBillingContractResponse* response);

    virtual qBool CancelAutoBillingRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemCancelAutoBillingParam& request);
    virtual qBool CancelAutoBillingResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemCancelAutoBillingResponse* response);

    virtual qBool ListRedemptionServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemListRedemptionServiceItemParam& request);
    virtual qBool ListRedemptionServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemListRedemptionServiceItemResponse* response);

    virtual qBool RedeemServiceItemRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemRedeemServiceItemParam& request);
    virtual qBool RedeemServiceItemResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemRedeemServiceItemResponse* response);

    /*!
        @brief      アカウント単位で利用する権利用バイナリデータを登録します。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool PostRightBinaryByAccount(ProtocolCallContext *pContext, const ServiceItemPostRightBinaryByAccountParam& request, ServiceItemPostRightBinaryResponse* response);
    //virtual qBool PostRightBinaryByAccountRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemPostRightBinaryByAccountParam& request);
    //virtual qBool PostRightBinaryByAccountResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemPostRightBinaryResponse* response);

    virtual qBool UseServiceItemByAccountRequest(ProtocolCallContext *pContext, qUnsignedInt32* requestId, const ServiceItemUseServiceItemByAccountParam& request);
    virtual qBool UseServiceItemByAccountResponse(ProtocolCallContext *pContext, qUnsignedInt32 requestId, ServiceItemUseServiceItemResponse* response);

    /*!
        @brief      サービスアイテムを獲得すると同時にアカウント単位で利用する権利用バイナリデータを登録します。

        現時点では、獲得できるサービスアイテムの種類として消費型サービス利用券のみをサポートしています。
        本関数で取得できるサービスアイテムの最大数は9999です。これ以上の数のアイテムの購入が必要になる場合はアイテム自体を別にする等の対応が必要になります。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  request リクエストパラメータ。
        @param[out] response レスポンス。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool AcquireServiceItemByAccount(ProtocolCallContext *pContext, const ServiceItemAcquireServiceItemByAccountParam& request, ServiceItemAcquireServiceItemResponse* response);

    /*!
        @brief      ユーザーを一意に識別できるサポート ID を取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[out] supportId サポート ID を受け取る文字列オブジェクト。
        @param[in]  param 追加のユーザー情報。

        @retval     true RMC 呼び出しの初期化に成功。
        @retval     false 失敗。
    */
    virtual qBool GetSupportId(ProtocolCallContext *pContext, String* supportId, const ServiceItemGetSupportIdParam& param);

private:
    ServiceItemProtocolClient m_oProtocol;
};

END_NAMESPACE_NEX

