﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/qVector.h>
#include "ServiceItemAccountRightConsumptionDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @extends    ServiceItemAccountRight
    @brief      一つのプリンシパル ID に対する消費型サービス利用券の権利情報を表すクラスです。
*/
class ServiceItemAccountRightConsumption: public DDLCLASS(ServiceItemAccountRightConsumption)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAccountRightConsumption()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAccountRightConsumption() {}

    /*!
        @brief      累計の獲得個数を取得します。

        取得する値は、有料で購入した権利の個数と無料で獲得した権利の個数の和です。

        @return     累計の獲得個数。
    */
    qUnsignedInt32 GetAcquiredCount() const { return limitation.limitationValue; }

    /*!
        @brief      累計の消費個数を取得します。

        取得する値には、無料で獲得した権利の消費個数も含まれます。

        @return     累計の消費個数。
    */
    qUnsignedInt32 GetUsedCount() const { return usedCount; }

    /*!
        @brief      累計の失効済み個数を取得します。

        取得する値には、無料で獲得した権利が失効した個数も含まれます。

        @return     累計の失効済み個数。
    */
    qUnsignedInt32 GetExpiredCount() const { return expiredCount; }

    /*!
        @brief      失効する個数のリストを取得します。

        リストの要素数は 5 固定です。それぞれの要素に格納している個数の意味は以下のとおりです。

        @li 0 番目の要素は、今月末に失効する個数です。
        @li 1 番目の要素は、来月末に失効する個数です。
        @li 2 番目の要素は、 2 ヶ月後の月末に失効する個数です。
        @li 3 番目の要素は、 3 ヶ月後の月末に失効する個数です。
        @li 4 番目の要素は、 4 ヶ月後の月末に失効する個数です。

        取得する値には、無料で獲得した権利が失効する個数も含まれます。

        @return     失効する個数のリスト。
    */
    const qVector<qUnsignedInt32>& GetExpiryCounts() const { return expiryCounts; }

    /*!
        @brief      ServiceItemAccountRightConsumption オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        {
            StringStream ss;
            ss << title << NEX_T("ServiceItemAccountRightConsumption(");
            ss << NEX_T("PID:") << GetPid();
            ss << NEX_T(" AcquiredCount:") << GetAcquiredCount();
            ss << NEX_T(" UsedCount:") << GetUsedCount();
            ss << NEX_T(" ExpiredCount:") << GetExpiredCount();
            ss << NEX_T(" ExpiryCounts:");
            for (qVector<qUnsignedInt32>::const_iterator it = GetExpiryCounts().begin();
                    it != GetExpiryCounts().end(); ++it)
            {
                if (it != GetExpiryCounts().begin())
                {
                    ss << NEX_T(", ");
                }
                ss << *it;
            }
            ss << NEX_T(")");
            QTRACE(uiTraceFlags, ss.CStr());
        }

        GetLimitation().Trace(uiTraceFlags, title + NEX_T("    "));

        for (qVector<ServiceItemRightBinary>::const_iterator it = GetRightBinaries().begin();
                it != GetRightBinaries().end(); ++it)
        {
            it->Trace(uiTraceFlags, title + NEX_T("    "));
        }

    }
};

END_NAMESPACE_NEX

