﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemAcquireServiceItemResponseDDL.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      獲得後の権利情報を表すクラスです。

    GetLimitationType() で取得できる権利種別によって参照できる情報が異なります。
*/
class ServiceItemAcquireServiceItemResponse: public DDLCLASS(ServiceItemAcquireServiceItemResponse)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemAcquireServiceItemResponse()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemAcquireServiceItemResponse() {}

    /*!
        @brief      権利種別を取得します。

        @return     権利種別。
    */
    ServiceItemConstants::LimitationType GetLimitationType() const { return static_cast<ServiceItemConstants::LimitationType>(limitationType); }

    /*!
        @brief      累計の獲得個数を取得します。

        権利種別が ServiceItemConstants::LIMITATION_TYPE_PERMANENT または ServiceItemConstants::LIMITATION_TYPE_ADDITIONAL_TIME の場合は
        1 になります。

        権利種別が ServiceItemConstants::LIMITATION_TYPE_CONSUMPTION の場合は累計の獲得個数になります。

        取得する値は、有料で購入した権利の個数と無料で獲得した権利の個数の和です。

        @return     累計の獲得個数。
    */
    qUnsignedInt32 GetAcquiredCount() const { return acquiredCount; }

    /*!
        @brief      累計の消費個数を取得します。

        権利種別が ServiceItemConstants::LIMITATION_TYPE_CONSUMPTION 以外の場合は使用しないでください。

        取得する値には、無料で獲得した権利の消費個数も含まれます。

        @return     累計の消費個数。
    */
    qUnsignedInt32 GetUsedCount() const { return usedCount; }

    /*!
        @brief      有効期限 (1970 年 1 月 1 日 00:00:00 UTC からの経過秒数) を取得します。

        権利種別が ServiceItemConstants::LIMITATION_TYPE_ADDITIONAL_TIME 以外の場合は使用しないでください。

        @return     有効期限。
    */
    qUnsignedInt32 GetExpiryDate() const { return expiryDate; }

    /*!
        @brief      累計の失効済み個数を取得します。

        権利種別が ServiceItemConstants::LIMITATION_TYPE_CONSUMPTION 以外の場合は使用しないでください。

        取得する値には、無料で獲得した権利が失効した個数も含まれます。

        @return     累計の失効済み個数。
    */
    qUnsignedInt32 GetExpiredCount() const { return expiredCount; }

    /*!
        @brief      失効する個数のリストを取得します。

        権利種別が ServiceItemConstants::LIMITATION_TYPE_CONSUMPTION 以外の場合は使用しないでください。

        リストの要素数は 5 固定です。それぞれの要素に格納している個数の意味は以下のとおりです。

        @li 0 番目の要素は、今月末に失効する個数です。
        @li 1 番目の要素は、来月末に失効する個数です。
        @li 2 番目の要素は、 2 ヶ月後の月末に失効する個数です。
        @li 3 番目の要素は、 3 ヶ月後の月末に失効する個数です。
        @li 4 番目の要素は、 4 ヶ月後の月末に失効する個数です。

        取得する値には、無料で獲得した権利が失効する個数も含まれます。

        @return     失効する個数のリスト。
    */
    const qVector<qUnsignedInt32>& GetExpiryCounts() const { return expiryCounts; }

    /*!
        @brief      ServiceItemAcquireServiceItemResponse オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemAcquireServiceItemResponse(")
                    << NEX_T("limitationType=") << limitationType
                    << NEX_T(" acquiredCount=") << acquiredCount
                    << NEX_T(" usedCount=") << usedCount
                    << NEX_T(" expiryDate=") << expiryDate
                    << NEX_T(" expiredCount=") << expiredCount
                    << NEX_T(" expiryCounts=(");
        for (qVector<qUnsignedInt32>::const_iterator itr = expiryCounts.begin();
             itr != expiryCounts.end(); ++itr)
        {
            if (itr != expiryCounts.begin())
            {
                ss << NEX_T(", ");
            }
            ss << *itr;
        }
        ss << NEX_T("))");
        QTRACE(uiTraceFlags, ss.CStr());
    }
};

END_NAMESPACE_NEX

