﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemCancelAutoBillingParamDDL.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      継続課金解除をする為のパラメータを表すクラスです。
*/
class ServiceItemCancelAutoBillingParam: public DDLCLASS(ServiceItemCancelAutoBillingParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemCancelAutoBillingParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemCancelAutoBillingParam() {}

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId) { this->uniqueId = uniqueId; }

    /*!
        @brief      アイテムコードを取得します。

        @return     アイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      [必須パラメタ] アイテムコード をセットします。

        @param[in]  itemCode アイテムコード。
    */
    void SetItemCode(const String& itemCode) { this->itemCode = itemCode; }

    /*!
        @brief      EC サービストークンを取得します。

        @return     EC サービストークン。
    */
    const String& GetEcServiceToken() const { return ecServiceToken; }

    /*!
        @brief      [必須パラメタ] アカウントサーバーから取得した EC サービストークンをセットします。

        アカウントサーバーから取得したECサービストークン (Base64フォーマット)

        @param[in]  ecServiceToken EC サービストークン。
    */
    void SetEcServiceToken(const String& ecServiceToken) { this->ecServiceToken = ecServiceToken; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        SetItemCode(ServiceItemConstants::INVALID_ITEM_CODE);
        SetEcServiceToken(ServiceItemConstants::INVALID_EC_SERVICE_TOKEN);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
    }
    /*!
        @brief      ServiceItemCancelAutoBillingParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemCancelAutoBillingParam& rhs);

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemCancelAutoBillingParam::SetUniqueId()."));
            success = false;
        }
        if(GetItemCode() == ServiceItemConstants::INVALID_ITEM_CODE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid itemCode. Please set itemCode with ServiceItemCancelAutoBillingParam::SetItemCode()."));
            success = false;
        }
        if(GetEcServiceToken() == ServiceItemConstants::INVALID_EC_SERVICE_TOKEN)
        {
            QLOG(EventLog::Error, NEX_T("Invalid ecServiceToken. Please set ecServiceToken with ServiceItemCancelAutoBillingParam::SetEcServiceToken()."));
            success = false;
        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemCancelAutoBillingParam& rhs)
{
    oStr << NEX_T("ServiceItemCancelAutoBillingParam: (")
               << NEX_T("itemCode=") << rhs.itemCode
               << NEX_T(", ecServiceToken=") << rhs.ecServiceToken
               << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX
