﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemCatalogDDL.h"
#include "ServiceItemListItem.h"
#include "ServiceItemConstants.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      サービスアイテム情報の一覧を表すクラスです。
*/
class ServiceItemCatalog: public DDLCLASS(ServiceItemCatalog)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemCatalog()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemCatalog() {}

    /*!
        @brief      検索にマッチしたサービスアイテム情報の総数を取得します。

        @return     検索にマッチしたサービスアイテム情報の総数。
    */
    qUnsignedInt32 GetTotalSize() const { return totalSize; }

    /*!
        @brief      オフセットを取得します。

        検索にマッチしたすべてのサービスアイテムの中から、リストに取得したサービスアイテムのオフセット (開始位置) です。

        @return     オフセット。
    */
    qUnsignedInt32 GetOffset() const { return offset; }

    /*!
        @brief      サービスアイテム情報のリストを取得します。

        取得したリストに含まれるサービスアイテム情報数は ServiceItemListServiceItemParam::SetSize() に指定したサイズが上限となります。

        @return     サービスアイテム情報のリスト。
    */
    const qVector<ServiceItemListItem>& GetListItems() const { return listItems; }

    /*!
        @brief      残高情報があるかどうかを取得します。

        @retval true 残高情報がある。
        @retval false 残高情報がない。
    */
    qBool GetIsBalanceAvailable() const { return isBalanceAvailable; }

    /*!
        @brief      残高情報を取得します。

        GetIsBalanceAvailable() が true を返す場合のみ呼び出し可能です。

        @return     残高情報。
    */
    const ServiceItemAmount& GetBalance() const { SYSTEMCHECK(isBalanceAvailable && "Do not use GetBalance() when GetIsBalanceAvailable() returns false."); return balance; }

    /*!
        @brief      ServiceItemCatalog オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        {
            StringStream ss;
            ss << title << NEX_T("ServiceItemCatalog(");
            ss << NEX_T("Total Size:") << GetTotalSize();
            ss << NEX_T(" Offset:") << GetOffset();
            ss << NEX_T(" Balance Available:") << GetIsBalanceAvailable();
            ss << NEX_T(")");
            QTRACE(uiTraceFlags, ss.CStr());
        }

        if (GetIsBalanceAvailable())
        {
            GetBalance().Trace(uiTraceFlags, title + NEX_T("    "));
        }

        qVector<nn::nex::ServiceItemListItem>::const_iterator it = GetListItems().begin();
        for (qInt i = 0; it != GetListItems().end(); ++it, ++i)
        {
            QTRACE(uiTraceFlags, title + NEX_T("    ---- item[") << i << NEX_T("] ----"));
            it->Trace(uiTraceFlags, title + NEX_T("    "));
        }
    }
};

END_NAMESPACE_NEX

