﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemGetPrepurchaseInfoParamDDL.h"
#include "ServiceItemConstants.h"
#include "ServiceItemLimitation.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      購入前情報を取得するためのパラメータを表すクラスです。
*/
class ServiceItemGetPrepurchaseInfoParam: public DDLCLASS(ServiceItemGetPrepurchaseInfoParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemGetPrepurchaseInfoParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemGetPrepurchaseInfoParam() {}

    /*!
        @brief      アイテムコードを取得します。

        @return     アイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      [必須パラメタ] アイテムコードをセットします。

        購入したいサービスアイテムのアイテムコードをセットします。

        @param[in]  itemCode_ 購入したいサービスアイテムのアイテムコード。
    */
    void SetItemCode(const String& itemCode_) { this->itemCode = itemCode_; }

    /*!
        @brief      リファレンス ID を取得します。

        @return     リファレンス ID 。
    */
    const String& GetReferenceId() const { return referenceId; }

    /*!
        @brief      [必須パラメタ] リファレンス ID をセットします。

        購入したいサービスアイテムのリファレンス ID をセットします。

        @param[in]  referenceId_ 購入したいサービスアイテムのリファレンス ID 。
    */
    void SetReferenceId(const String& referenceId_) { this->referenceId = referenceId_; }

    /*!
        @brief      [必須パラメタ] 権利情報をセットします。

        ServiceItemClient::ListServiceItem() で取得した、購入したいサービスアイテムの権利情報をセットします。
        権利情報は、 ServiceItemListItem オブジェクトの ServiceItemListItem::GetLimitation() で取得できます。

        @param[in]  limitation_ 購入したいサービスアイテムの権利情報。
    */
    void SetLimitation(const ServiceItemLimitation& limitation_) { this->limitation = limitation_; }

    /*!
        @brief      権利情報を取得します。

        @return     権利情報。
    */
    const ServiceItemLimitation& GetLimitation() const { return limitation; }

    /*!
        @brief      言語を取得します。

        @return     言語。
    */
    const String& GetLanguage() const { return language; }

    /*!
        @brief      [必須パラメタ] 言語をセットします。

        言語をセットします。通貨表記のフォーマットを決定するために使用します。
        ISO 639 で規定されている 2 文字の言語コードを使用します。
        
        nn::cfg::CTR::GetLanguageCodeA2() 関数で取得した文字列を指定してください。
        

        @param[in]  language_ 言語。
    */
    void SetLanguage(const String& language_) { this->language = language_; }

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId_ ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId_) { this->uniqueId = uniqueId_; }

    /*!
        @brief      有効期限 ID を取得します。

        periodId は、消費型サービス利用券の有効期限を設定するための ID です。
        pid ごとに有効期限を変えたい場合に使用します。
        periodId に紐づいた有効期限の設定値をサーバに登録する必要があるため、
        利用する場合はサーバー担当者にご相談ください。

        @return     有効期限 ID。
    */
    qUnsignedInt16 GetPeriodId() const { return periodId; }

    /*!
        @brief      有効期限 ID をセットします。

        periodId は、消費型サービス利用券の有効期限を設定するための ID です。
        pid ごとに有効期限を変えたい場合に使用します。
        periodId に紐づいた有効期限の設定値をサーバに登録する必要があるため、
        利用する場合はサーバー担当者にご相談ください。

        @param[in]  periodId_ 有効期限 ID。
    */
    void SetPeriodId(qUnsignedInt16 periodId_) { this->periodId = periodId_; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetItemCode(ServiceItemConstants::INVALID_ITEM_CODE);
        SetReferenceId(ServiceItemConstants::INVALID_REFERENCE_ID);
        SetLimitation(ServiceItemLimitation(ServiceItemConstants::LIMITATION_TYPE_INVALID, 0));
        SetLanguage(ServiceItemConstants::INVALID_LANGUAGE);
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
        SetPeriodId(0);
    }

    /*!
        @brief      ServiceItemGetPrepurchaseInfoParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
        GetLimitation().Trace(uiTraceFlags, NEX_T("    "));
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemGetPrepurchaseInfoParam& rhs);

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if(GetItemCode().GetLength() == 0)
        {
            QLOG(EventLog::Error, NEX_T("Invalid itemCode. Please set itemCode with ServiceItemGetPrepurchaseInfoParam::SetItemCode()."));
            success = false;
        }
        if(GetReferenceId() == ServiceItemConstants::INVALID_REFERENCE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid referenceId. Please set referenceId with ServiceItemGetPrepurchaseInfoParam::SetReferenceId()."));
            success = false;
        }
        if(GetLimitation().GetLimitationType() == ServiceItemConstants::LIMITATION_TYPE_INVALID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid limitation. Please set limitation with ServiceItemGetPrepurchaseInfoParam::SetLimitation()."));
            success = false;
        }
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemGetPrepurchaseInfoParam::SetUniqueId()."));
            success = false;
        }
        if(GetLanguage() == ServiceItemConstants::INVALID_LANGUAGE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid language. Please set language with ServiceItemGetPrepurchaseInfoParam::SetLanguage()."));
            success = false;
        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemGetPrepurchaseInfoParam& rhs)
{
    oStr << NEX_T("ServiceItemGetPrepurchaseInfoParam: (")
               << NEX_T("itemCode=") << rhs.itemCode
               << NEX_T(", referenceId=") << rhs.referenceId
               << NEX_T(", language=") << rhs.language
               << NEX_T(", uniqueId=") << rhs.uniqueId
               << NEX_T(", periodId=") << rhs.periodId
               << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

