﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemGetServiceItemRightParamDDL.h"
#include "ServiceItemConstants.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      権利情報を取得するためのパラメータを表すクラスです。
*/
class ServiceItemGetServiceItemRightParam: public DDLCLASS(ServiceItemGetServiceItemRightParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemGetServiceItemRightParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemGetServiceItemRightParam() {}

    /*!
        @brief      リファレンス ID を取得します。

        @return     リファレンス ID。
    */
    const String& GetReferenceId() const { return referenceId; }

    /*!
        @brief      リファレンス ID をセットします。

        権利情報を取得したいサービスアイテムのリファレンス ID をセットします。
        省略もしくは ServiceItemConstants::INVALID_REFERENCE_ID がセットされた場合はすべての権利情報を取得します。

        @param[in]  referenceId_ 権利情報を取得したいサービスアイテムのリファレンス ID。
    */
    void SetReferenceId(const String& referenceId_) { this->referenceId = referenceId_; }

    /*!
        @brief      本体固有 ID を取得します。

        @return     本体固有 ID 。
    */
    const String& GetDeviceId() const { return deviceId; }

    /*!
        @brief      [必須パラメタ] 本体固有 ID をセットします。

        @param[in]  deviceId_ 本体固有 ID 。
    */
    void SetDeviceId(const String& deviceId_) { this->deviceId = deviceId_; }

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId_ ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId_) { this->uniqueId = uniqueId_; }

    /*!
        @internal
        @brief      アイテムグループを取得します。

        @return     アイテムグループ。
    */
    qByte GetItemGroup() const { return itemGroup; }

    /*!
        @internal
        @brief      アイテムグループをセットします。

        アイテムグループとは、リファレンス ID の先頭の 1 文字を指します。
        アイテムグループを設定した場合、GetServiceItemRight 関数は、
        該当するアイテムグループの権利のみを返します。
        リファレンス ID とアイテムグループを同時に設定することはできません。

        2015 年 7 月現在、アイテムグループで予約されている値は下記の通りです。
        0: アプリケーションで利用
        1: SandboxCheckerで利用
        アイテムグループに設定できる値は 0x00 ～ 0x0f の範囲です。
        それ以外の値やデフォルト値を指定した場合、アイテムグループの値は無視されます。

        なお、このパラメータは 2015 年 7 月現在、非公開です。

        @param[in]  itemGroup_ アイテムグループ。
    */
    void SetItemGroup(qByte itemGroup_) { this->itemGroup = itemGroup_; }

    /*!
        @brief      有効期限 ID を取得します。

        periodId は、消費型サービス利用券の有効期限を設定するための ID です。
        pid ごとに有効期限を変えたい場合に使用します。
        periodId に紐づいた有効期限の設定値をサーバに登録する必要があるため、
        利用する場合はサーバー担当者にご相談ください。

        @return     有効期限 ID。
    */
    qUnsignedInt16 GetPeriodId() const { return periodId; }

    /*!
        @brief      有効期限 ID をセットします。

        periodId は、消費型サービス利用券の有効期限を設定するための ID です。
        pid ごとに有効期限を変えたい場合に使用します。
        periodId に紐づいた有効期限の設定値をサーバに登録する必要があるため、
        利用する場合はサーバー担当者にご相談ください。

        @param[in]  periodId 有効期限 ID。
    */
    void SetPeriodId(qUnsignedInt16 periodId) { this->periodId = periodId; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetReferenceId(ServiceItemConstants::INVALID_REFERENCE_ID);
        SetDeviceId(ServiceItemConstants::INVALID_DEVICE_ID);
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        SetItemGroup(ServiceItemConstants::DEFAULT_ITEM_GROUP);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
        SetPeriodId(0);
    }

    /*!
        @brief      ServiceItemGetServiceItemRightParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemGetServiceItemRightParam& rhs);

    /*!
        @internal
        @brief      RMC のリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if(GetDeviceId() == ServiceItemConstants::INVALID_DEVICE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid deviceId. Please set devideId with ServiceItemGetServiceItemRightParam::SetDeviceId()."));
            success = false;
        }
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemGetServiceItemRightParam::SetUniqueId()."));
            success = false;
        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemGetServiceItemRightParam& rhs)
{
    oStr << NEX_T("ServiceItemGetServiceItemRightParam: (")
               << NEX_T("referenceId=") << rhs.referenceId
               << NEX_T(", deviceId=") << rhs.deviceId
               << NEX_T(", uniqueId=") << rhs.uniqueId
               << NEX_T(", itemGroup=") << rhs.itemGroup
               << NEX_T(", periodId=") << rhs.periodId
               << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

