﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemListServiceItemParamDDL.h"
#include "ServiceItemConstants.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      販売しているサービスアイテム情報のリストを取得するためのパラメータを表すクラスです。
*/
class ServiceItemListServiceItemParam: public DDLCLASS(ServiceItemListServiceItemParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemListServiceItemParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemListServiceItemParam() {}

    /*!
        @brief      言語を取得します。

        @return     言語。
    */
    const String& GetLanguage() const { return language; }

    /*!
        @brief      [必須パラメタ] 言語をセットします。

        言語をセットします。通貨表記のフォーマットを決定するために使用します。
        ISO 639 で規定されている 2 文字の言語コードを使用します。
        
        nn::cfg::CTR::GetLanguageCodeA2() 関数で取得した文字列を指定してください。
        

        @param[in]  language_ 言語。
    */
    void SetLanguage(const String& language_) { this->language = language_; }

    /*!
        @brief      オフセットを取得します。

        @return     オフセット。
    */
    qUnsignedInt32 GetOffset() const { return offset; }

    /*!
        @brief      オフセットをセットします。

        取得するサービスアイテム情報のオフセット (開始位置) をセットします。セットしない場合の初期値は 0 です。

        @param[in]  offset_ オフセット。
    */
    void SetOffset(qUnsignedInt32 offset_) { this->offset = offset_; }

    /*!
        @brief      取得するサービスアイテム情報の最大個数を取得します。

        @return     取得するサービスアイテム情報の最大個数。
    */
    qUnsignedInt32 GetSize() const { return size; }

    /*!
        @brief      取得するサービスアイテム情報の最大個数をセットします。

        取得するサービスアイテム情報の最大個数をセットします。セットしない場合の初期値は 20 です。

        @param[in]  size_ 取得する情報の最大個数。
    */
    void SetSize(qUnsignedInt32 size_) { this->size = size_; }

    /*!
        @brief      レスポンスに残高情報を含めるかどうかを取得します。

        @retval true レスポンスに残高情報を含める。
        @retval false レスポンスに残高情報を含めない。
    */
    qBool GetIsBalanceAvailable() const { return isBalanceAvailable; }

    /*!
        @brief      レスポンスに残高情報を含めるかどうかをセットします。

        セットしない場合の初期値は false です。

        @param[in]  isBalanceAvailable_ レスポンスに残高情報を含める場合は true 、含めない場合は false 。
    */
    void SetIsBalanceAvailable(qBool isBalanceAvailable_) { this->isBalanceAvailable = isBalanceAvailable_; }

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId_ ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId_) { this->uniqueId = uniqueId_; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetLanguage(ServiceItemConstants::INVALID_LANGUAGE);
        SetOffset(0);
        SetSize(20);
        SetIsBalanceAvailable(false);
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
    }

    /*!
        @brief      ServiceItemListServiceItemParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemListServiceItemParam& rhs);

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemListServiceItemParam.::SetUniqueId()."));
            success = false;
        }
        if(GetLanguage() == ServiceItemConstants::INVALID_LANGUAGE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid language. Please set language with ServiceItemListServiceItemParam::SetLanguage()."));
            success = false;
        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemListServiceItemParam& rhs)
{
    oStr << NEX_T("ServiceItemListServiceItemParam: (")
               << NEX_T("language=") << rhs.language
               << NEX_T(", offset=") << rhs.offset
               << NEX_T(", size=") << rhs.size
               << NEX_T(", isBalanceAvailable=") << rhs.isBalanceAvailable
               << NEX_T(", uniqueId=") << rhs.uniqueId
               << NEX_T(")");

    return oStr;
}

END_NAMESPACE_NEX

