﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemPostRightBinaryByAccountParamDDL.h"
#include "ServiceItemConstants.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      アカウント単位で利用する権利用バイナリデータを登録するためのパラメータを表すクラスです。

    @copydoc    DescriptionOfRightBinary
*/
class ServiceItemPostRightBinaryByAccountParam: public DDLCLASS(ServiceItemPostRightBinaryByAccountParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemPostRightBinaryByAccountParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemPostRightBinaryByAccountParam() {}

    /*!
        @brief      リファレンス ID を取得します。

        @return     リファレンス ID 。
    */
    const String& GetReferenceId() const { return referenceId; }

    /*!
        @brief      [必須パラメタ] リファレンス ID をセットします。

        消費型サービス利用券のリファレンス ID をセットします。

        @param[in]  referenceId_ 消費したい消費型サービス利用券のリファレンス ID 。
    */
    void SetReferenceId(const String& referenceId_) { this->referenceId = referenceId_; }

    /*!
        @brief      権利用バイナリデータの利用タイプを取得します。

        @return     権利用バイナリデータの利用タイプ。
    */
    qByte GetUseType() const { return useType; }

    /*!
        @brief      [必須パラメタ] 権利用バイナリデータの利用タイプをセットします。

        アプリケーションでの権利用バイナリデータの用途に応じて 1-15 までの値をセットできます。

        @param[in]  useType_ 権利用バイナリデータの利用タイプ。
    */
    void SetUseType(qByte useType_)
    {
        SYSTEMCHECK(useType_ >= 1 && useType_ <= 15 && "Invalid use type. Please set between 1 and 15.");
        this->useType = useType_;
    }

    /*!
        @brief      アカウント単位で利用する権利用バイナリデータを取得します。

        @param[out]  rightBinary_ アカウント単位で利用する権利用バイナリデータ。
    */
    void GetRightBinary(qVector<qByte>& rightBinary_) const
    {
        // qBuffer -> qVector<qByte>への変換
        rightBinary_.assign(this->rightBinary.begin(), this->rightBinary.end());
    }

    /*!
        @brief      アカウント単位で利用する権利用バイナリデータをセットします。

        アカウント単位で利用する権利で保護すべきバイナリデータを指定します。
        0 ～ ServiceItemConstants::RIGHT_BINARY_MAX_SIZE バイトまでセットできます。
        本パラメータの指定は必須ではありません。セットしない場合の初期値は空です。

        指定された権利用バイナリデータがそのままサーバに登録されます。
        空の場合に、削除されたり登録がスキップされることはありません。

        @param[in]  pRightBinary アカウント単位で利用する権利用バイナリデータのポインタ。 NULL を指定すると空になります。
        @param[in]  size アカウント単位で利用する権利用バイナリデータのデータサイズ (0 ～ ServiceItemConstants::RIGHT_BINARY_MAX_SIZE) 。
    */
    void SetRightBinary(const void* pRightBinary, qUnsignedInt32 size)
    {
        SYSTEMCHECK(size <= ServiceItemConstants::RIGHT_BINARY_MAX_SIZE && "Invalid pRightBinary. Please set ServiceItemConstants::RIGHT_BINARY_MAX_SIZE or less.");
        this->rightBinary.clear();
        if (pRightBinary != NULL)
        {
            const qByte* pb = static_cast<const qByte*>(pRightBinary);
            this->rightBinary.push_back(pb, size);
        }
    }

    /*!
        @brief      ログメッセージを取得します。

        @return     ログメッセージ。
    */
    const String& GetLogMessage() const { return logMessage; }

    /*!
        @brief      [必須パラメタ] ログメッセージをセットします。

        ユーザサポート向けの管理ツールに表示されるログメッセージをセットします。
        最大で ServiceItemConstants::SUPPORT_LOG_MESSAGE_MAX_SIZE バイトまでセットできます。

        @param[in]  logMessage_ ログメッセージ。
    */
    void SetLogMessage(const String& logMessage_)
    {
        SYSTEMCHECK(logMessage_.GetLength() > 0 && (sizeof(qChar) * logMessage_.GetLength()) <= ServiceItemConstants::SUPPORT_LOG_MESSAGE_MAX_SIZE && "Invalid logMessage. Please set between 1 and ServiceItemConstants::SUPPORT_LOG_MESSAGE_MAX_SIZE bytes.");
        this->logMessage = logMessage_;
    }

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId_ ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId_) { this->uniqueId = uniqueId_; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetReferenceId(ServiceItemConstants::INVALID_REFERENCE_ID);
        this->useType = ServiceItemConstants::INVALID_USE_TYPE;
        SetRightBinary(NULL, 0);
        this->logMessage = ServiceItemConstants::INVALID_LOG_MESSAGE;
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
    }

    /*!
        @brief      ServiceItemPostRightBinaryByAccountParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        {
            StringStream ss;
            ss << NEX_T("ServiceItemPostRightBinaryByAccountParam: (")
                        << NEX_T("referenceId=") << referenceId
                        << NEX_T(", useType=") << useType
                        << NEX_T(", logMessage=") << logMessage
                        << NEX_T(", uniqueId=") << uniqueId
                        << NEX_T(")");
            QTRACE(uiTraceFlags, ss.CStr());
        }

        {
            // rightBinaryは大きい可能性があるので別でログ出力する
            StringStream ss;
            ss << NEX_T("    rightBinary=<");
            if(!rightBinary.empty())
            {
                ss.BytesDump(&rightBinary[0], rightBinary.size());
            }
            ss << NEX_T(">");
            QTRACE(uiTraceFlags, ss.CStr())
        }
    }

    /*!
        @internal
        @brief      RMC のリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if(GetReferenceId() == ServiceItemConstants::INVALID_REFERENCE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid referenceId. Please set referenceId with ServiceItemPostRightBinaryByAccountParam::SetReferenceId()."));
            success = false;
        }
        if(GetUseType() == ServiceItemConstants::INVALID_USE_TYPE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid useType. Please set useType with ServiceItemPostRightBinaryByAccountParam::SetUseType()."));
            success = false;
        }
        if(GetLogMessage() == ServiceItemConstants::INVALID_LOG_MESSAGE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid logMessage. Please set logMessage with ServiceItemPostRightBinaryByAccountParam::SetLogMessage()."));
            success = false;
        }
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemPostRightBinaryByAccountParam::SetUniqueId()."));
            success = false;
        }
        // RightBinary は一応消せる仕様なのでチェックしない

        return success;
    }
};

END_NAMESPACE_NEX

