﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemPrepurchaseInfoDDL.h"
#include "ServiceItemAmount.h"
#include "ServiceItemConstants.h"
#include "ServiceItemPrepurchaseRightInfo.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      購入前情報を表すクラスです。
*/
class ServiceItemPrepurchaseInfo: public DDLCLASS(ServiceItemPrepurchaseInfo)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemPrepurchaseInfo()
    {
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemPrepurchaseInfo() {}

    /*!
        @brief      アイテムコードを取得します。

        @return     アイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      価格情報の識別 ID を取得します。

        @return     価格情報の識別 ID 。
    */
    const String& GetPriceId() const { return priceId; }

    /*!
        @brief      販売価格情報を取得します。

        @return     販売価格情報。
    */
    const ServiceItemAmount& GetRegularPrice() const { return regularPrice; }

    /*!
        @brief      税額があるかどうかを取得します。

        @retval true 税額がある。
        @retval false 税額がない。
    */
    qBool GetIsTaxAvailable() const { return isTaxAvailable; }

    /*!
        @brief      税額情報を取得します。

        GetIsTaxAvailable() が true を返す場合のみ呼び出し可能です。

        @return     税額情報。
    */
    const ServiceItemAmount& GetTaxAmount() const { SYSTEMCHECK(isTaxAvailable && "Do not use GetTaxAmount() when GetIsTaxAvailable() returns false."); return taxAmount; }

    /*!
        @brief      総額情報を取得します。

        @return     総額情報。
    */
    const ServiceItemAmount& GetTotalAmount() const { return totalAmount; }

    /*!
        @brief      現在の残高情報を取得します。

        @return     現在の残高情報。
    */
    const ServiceItemAmount& GetCurrentBalance() const { return currentBalance; }

    /*!
        @brief      購入後の残高情報を取得します。

        @return     購入後の残高情報。
    */
    const ServiceItemAmount& GetPostBalance() const { return postBalance; }

    /*!
        @brief      現在の権利情報を取得します。

        @return     現在の権利情報。
    */
    const ServiceItemPrepurchaseRightInfo& GetCurrentRightInfo() const { return currentRightInfo; }

    /*!
        @brief      購入後の権利情報を取得します。

        @return     購入後の権利情報。
    */
    const ServiceItemPrepurchaseRightInfo& GetPostRightInfo() const { return postRightInfo; }

    /*!
        @brief      ServiceItemPrepurchaseInfo オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
        @param[in]  title   ログの先頭に表示する文字列です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS, const String& title = NEX_T("")) const
    {
        StringStream ss;
        ss << title << NEX_T("ServiceItemPrepurchaseRightInfo(");
        ss << NEX_T("ItemCode:") << GetItemCode();
        ss << NEX_T(" PriceId:") << GetPriceId();
        ss << NEX_T(" TaxAvailable:") << GetIsTaxAvailable();
        ss << NEX_T(")");
        QTRACE(uiTraceFlags, ss.CStr());

        GetRegularPrice().Trace(uiTraceFlags, title + NEX_T("    RegularPrice "));
        if (GetIsTaxAvailable())
        {
            GetTaxAmount().Trace(uiTraceFlags, title + NEX_T("    TaxAmount "));
        }
        GetTotalAmount().Trace(uiTraceFlags, title + NEX_T("    TotalAmount "));
        GetCurrentBalance().Trace(uiTraceFlags, title + NEX_T("    CurrentBalance "));
        GetPostBalance().Trace(uiTraceFlags, title + NEX_T("    PostBalance "));
        GetCurrentRightInfo().Trace(uiTraceFlags, title + NEX_T("    CurrentRightInfo "));
        GetPostRightInfo().Trace(uiTraceFlags, title + NEX_T("    PostRightInfo "));
    }
};

END_NAMESPACE_NEX

