﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "ServiceItemPurchaseServiceItemParamDDL.h"
#include "ServiceItemConstants.h"
#include "ServiceItemConstantsInternal.h"

BEGIN_NAMESPACE_NEX

/*!
    @brief      サービスアイテムを購入するためのパラメータを表すクラスです。
*/
class ServiceItemPurchaseServiceItemParam: public DDLCLASS(ServiceItemPurchaseServiceItemParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    ServiceItemPurchaseServiceItemParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~ServiceItemPurchaseServiceItemParam() {}

    /*!
        @brief      アイテムコードを取得します。

        @return     アイテムコード。
    */
    const String& GetItemCode() const { return itemCode; }

    /*!
        @brief      [必須パラメタ] アイテムコードをセットします。

        購入したいサービスアイテムのアイテムコードをセットします。

        @param[in]  itemCode_ 購入したいサービスアイテムのアイテムコード。
    */
    void SetItemCode(const String& itemCode_) { this->itemCode = itemCode_; }

    /*!
        @brief      価格情報の識別 ID を取得します。

        @return     価格情報の識別 ID 。
    */
    const String& GetPriceId() const { return priceId; }

    /*!
        @brief      [必須パラメタ] 価格情報の識別 ID をセットします。

        ServiceItemClient::GetPrepurchaseInfo() で取得した、購入したいサービスアイテムの価格情報の識別 ID をセットします。

        @param[in]  priceId_ 購入したいサービスアイテムの価格情報の識別 ID。
    */
    void SetPriceId(const String& priceId_) { this->priceId = priceId_; }

    /*!
        @brief      リファレンス ID を取得します。

        @return     リファレンス ID。
    */
    const String& GetReferenceId() const { return referenceId; }

    /*!
        @brief      [必須パラメタ] リファレンス ID をセットします。

        購入したいサービスアイテムのリファレンス ID をセットします。

        @param[in]  referenceId_ 購入したいサービスアイテムのリファレンス ID。
    */
    void SetReferenceId(const String& referenceId_) { this->referenceId = referenceId_; }

    /*!
        @brief      アカウントの未フォーマットの残高を取得します。

        @return     アカウントの未フォーマットの残高。
    */
    const String& GetCurrentBalanceRawValue() const { return balance; }

    /*!
        @brief      [必須パラメタ] アカウントの未フォーマットの残高をセットします。

        ServiceItemClient::GetBalance() もしくは  ServiceItemClient::GetPrepurchaseInfo() で取得した、アカウントの未フォーマットの残高をセットします。
        未フォーマットの残高は、残高を表す ServiceItemAmount オブジェクトの ServiceItemAmount::GetRawValue() で取得できます。

        @param[in]  balance_ アカウントの未フォーマットの残高。
    */
    void SetCurrentBalanceRawValue(const String& balance_) { this->balance = balance_; }

    /*!
        @brief      アイテム名を取得します。

        @return     アイテム名。
    */
    const String& GetItemName() const { return itemName; }

    /*!
        @brief      [必須パラメタ] アイテム名をセットします。

        購入するサービスアイテムのアイテム名をセットします。購入履歴での表示用です。

        @param[in]  itemName_ 購入するサービスアイテムのアイテム名。
    */
    void SetItemName(const String& itemName_) { this->itemName = itemName_; }

    /*!
        @brief      EC サービストークンを取得します。

        @return     EC サービストークン。
    */
    const String& GetEcServiceToken() const { return ecServiceToken; }

    /*!
        @brief      EC サービストークンをセットします。

        NNAS から取得したNintendo eShop のサービストークン (Base64 フォーマット) をセットします。
        ServiceItemClient::AcquireEcServiceTokenAndPurchaseServiceItem() を使用する場合はライブラリ内部で自動的に EC サービストークンを取得するため、
        本関数でのセットは不要です。

        @param[in]  ecServiceToken_ EC サービストークン。
    */
    void SetEcServiceToken(const String& ecServiceToken_) { this->ecServiceToken = ecServiceToken_; }

    /*!
        @brief      言語を取得します。

        @return     言語。
    */
    const String& GetLanguage() const { return language; }

    /*!
        @brief      [必須パラメタ] 言語をセットします。

        言語をセットします。通貨表記のフォーマットを決定するために使用します。
        ISO 639 で規定されている 2 文字の言語コードを使用します。
        
        nn::cfg::CTR::GetLanguageCodeA2() 関数で取得した文字列を指定してください。
        

        @param[in]  language_ 言語。
    */
    void SetLanguage(const String& language_) { this->language = language_; }

    /*!
        @brief      ユニーク ID を取得します。

        @return     ユニーク ID。
    */
    qUnsignedInt32 GetUniqueId() const { return uniqueId; }

    /*!
        @brief      [必須パラメタ] ユニーク ID をセットします。

        NEX サーバーから接続する eShop サーバーを決定するためのパラメーターです。
        通常はタイトルのユニーク ID を指定します。

        @param[in]  uniqueId_ ユニーク ID。
    */
    void SetUniqueId(qUnsignedInt32 uniqueId_) { this->uniqueId = uniqueId_; }

    /*!
        @brief      有効期限 ID を取得します。

        periodId は、消費型サービス利用券の有効期限を設定するための ID です。
        pid ごとに有効期限を変えたい場合に使用します。
        periodId に紐づいた有効期限の設定値をサーバに登録する必要があるため、
        利用する場合はサーバー担当者にご相談ください。

        @return     有効期限 ID。
    */
    qUnsignedInt16 GetPeriodId() const { return periodId; }

    /*!
        @brief      有効期限 ID をセットします。

        periodId は、消費型サービス利用券の有効期限を設定するための ID です。
        pid ごとに有効期限を変えたい場合に使用します。
        periodId に紐づいた有効期限の設定値をサーバに登録する必要があるため、
        利用する場合はサーバー担当者にご相談ください。

        @param[in]  periodId_ 有効期限 ID。
    */
    void SetPeriodId(qUnsignedInt16 periodId_) { this->periodId = periodId_; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetItemCode(ServiceItemConstants::INVALID_ITEM_CODE);
        SetPriceId(NEX_T(""));
        SetReferenceId(ServiceItemConstants::INVALID_REFERENCE_ID);
        SetCurrentBalanceRawValue(NEX_T(""));
        SetItemName(NEX_T(""));
        SetEcServiceToken(ServiceItemConstants::INVALID_EC_SERVICE_TOKEN);
        SetLanguage(ServiceItemConstants::INVALID_LANGUAGE);
        SetUniqueId(ServiceItemConstants::INVALID_UNIQUE_ID);
        platform = NEX_SERVICEITEM_GET_PLATFORM;
        SetPeriodId(0);
    }

    /*!
        @brief      ServiceItemPurchaseServiceItemParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
        QTRACE(uiTraceFlags, NEX_T("    ") + GetEcServiceToken()); // ecServiceToken だけ非常に長いので別にログしておく
    }

    /*!
        @internal
        @brief      Trace()で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const ServiceItemPurchaseServiceItemParam& rhs);

    /*!
        @internal
        @brief      RMCリクエスト前にパラメタをチェックする関数です。
    */
    qBool RequestPreCheck() const
    {
        qBool success = true;
        if(GetItemCode().GetLength() == 0)
        {
            QLOG(EventLog::Error, NEX_T("Invalid itemCode. Please set itemCode with ServiceItemPurchaseServiceItemParam::SetItemCode()."));
            success = false;
        }
        if(GetPriceId().GetLength() == 0)
        {
            QLOG(EventLog::Error, NEX_T("Invalid priceId. Please set priceId with ServiceItemPurchaseServiceItemParam::SetPriceId()."));
            success = false;
        }
        if(GetReferenceId() == ServiceItemConstants::INVALID_REFERENCE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid referenceId. Please set referenceId with ServiceItemPurchaseServiceItemParam::SetReferenceId()."));
            success = false;
        }
        if(GetCurrentBalanceRawValue().GetLength() == 0)
        {
            QLOG(EventLog::Error, NEX_T("Invalid balance. Please set balance with ServiceItemPurchaseServiceItemParam::SetCurrentBalanceRawValue()."));
            success = false;
        }
        if(GetItemName().GetLength() == 0)
        {
            QLOG(EventLog::Error, NEX_T("Invalid itemName. Please set itemName with ServiceItemPurchaseServiceItemParam::SetItemName()."));
            success = false;
        }
        if(GetEcServiceToken().GetLength() == 0)
        {
            QLOG(EventLog::Error, NEX_T("Invalid ecServiceToken. Please set itemName with ServiceItemPurchaseServiceItemParam::SetEcServiceToken()."));
            success = false;
        }
        if(GetUniqueId() == ServiceItemConstants::INVALID_UNIQUE_ID)
        {
            QLOG(EventLog::Error, NEX_T("Invalid uniqueId. Please set uniqueId with ServiceItemPurchaseServiceItemParam::SetUniqueId()."));
            success = false;
        }
        if(GetLanguage() == ServiceItemConstants::INVALID_LANGUAGE)
        {
            QLOG(EventLog::Error, NEX_T("Invalid language. Please set language with ServiceItemPurchaseServiceItemParam::SetLanguage()."));
            success = false;
        }
        return success;
    }
};

inline StringStream& operator<<(StringStream& oStr, const ServiceItemPurchaseServiceItemParam& rhs)
{
    oStr << NEX_T("ServiceItemPurchaseServiceItemParam: (")
               << NEX_T("itemCode=") << rhs.itemCode
               << NEX_T(", priceId=") << rhs.priceId
               << NEX_T(", referenceId=") << rhs.referenceId
               << NEX_T(", balance=") << rhs.balance
               << NEX_T(", itemName=") << rhs.itemName
               << NEX_T(", language=") << rhs.language
               << NEX_T(", uniqueId=") << rhs.uniqueId
               << NEX_T(", periodId=") << rhs.periodId
               << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

