﻿/*---------------------------------------------------------------------------*
  Copyright (C)2011-2017 Nintendo Co., Ltd.  All rights reserved.
 
  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <RendezVous/Core/src/Client/ServiceClient.h>
#include <RendezVous/Services/Jugem/Subscriber/src/Common/SubscriberConstants.h>
#include <RendezVous/Services/Jugem/Subscriber/src/Common/SubscriberDDF.h>
#include <RendezVous/Services/Jugem/Subscriber/src/Common/SubscriberProtocolDDL.h>
#include <RendezVous/Services/Jugem/Subscriber/src/Common/SubscriberErrors.h>
#include <RendezVous/Services/Jugem/Subscriber/src/Common/SubscriberSuccesses.h>
#include <RendezVous/Services/Jugem/Subscriber/src/Client/SubscriberUtil.h>

BEGIN_NAMESPACE_NEX

/*!
    @brief Subscriber 機能を利用するためのクラス。
*/
class SubscriberClient : public ServiceClient
{
public:

    /*!
    @brief      コンストラクタです。

    コンストラクタです。
    */
    SubscriberClient();

    /*!
    @brief      デストラクタです。

    デストラクタです。
    */
    virtual ~SubscriberClient();

    /*!
        @brief    サービスを利用するための証明書を登録します。

        サービスを利用するための証明書をセットします。
        機能を利用する前にセットする必要があります。@ref NgsFacade::Login() の非同期処理に成功後、
        @ref NgsFacade::GetCredentials() で取得した証明書を引数に指定してください。
        @ref Bind() で証明書をセットした後もう一度 @ref Bind() を呼び出すと証明書を差し替える事ができます。

        @param[in]     pCredentials    サービスにアクセスするための証明書。

        @retval     true 成功。pCredentials が NULL でない場合は必ず成功します。
        @retval     false pCredentials に NULL が指定された。
    */
    virtual qBool Bind(Credentials* pCredentials);

    /*!
        @brief     証明書の登録を解除します。

        @ref Bind() でセットされた証明書を解除します。@ref Bind() が呼び出されていなくても本関数を呼び出すことができます。
        本オブジェクトを破棄する際に自動的に @ref Unbind() が呼び出されるため、明示的に呼び出す必要はありません。

        @retval      true 証明書の解除を行った。
        @retval      false Bind() が行われていないため何も行わなかった。
    */
    virtual qBool Unbind();

    virtual void UpdateProtocolsDefaultCredentials(Credentials* pCredentials);

    /*!
        @brief      コンテンツをトピックに投稿します。

        一つのコンテンツを一つもしくは複数のトピックに投稿します。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックを一つも指定していない。トピックのフォーマットが不正。
        @li QERROR(Subscriber, OverLimit) : 投稿するメッセージ長が SubscriberConstants::MAX_CONTENT_MESSAGE_LEN を超えている。
                                        投稿するバイナリサイズが SubscriberConstants::MAX_CONTENT_BINARY_SIZE を超えている。
                                        予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。
                                        投稿先のトピックの数が SubscriberConstants::MAX_POST_CONTENT_TOPIC_SIZE を超えている。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] contentId コンテンツに割り当てられた ID。この ID はゲームサーバー内で一意となり、投稿ごとに増加されます。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool PostContent(ProtocolCallContext* pContext, const SubscriberPostContentParam& param, qUnsignedInt64* contentId);

    /*!
        @refmanoverload{single}
        @brief      トピックに投稿されたコンテンツを取得します。

        トピックに投稿されたコンテンツをコンテンツ ID 順 (降順) に取得します。
        コンテンツ ID は投稿された順に発行され、増加していきますがコンテンツに含まれる投稿日時は数秒前後する可能性があります。
        厳密な投稿日時順ではないことにご注意ください。
        サーバーでの処理タイミングの差により、過去取得した最大のコンテンツ ID より小さいコンテンツ ID のコンテンツが後で挿入されることがあります。

        本関数は例外的にガイドラインに定められた範囲において定期的に呼び出すことが可能です。
        呼び出し頻度の制限についてはガイドラインを参照してください。
        ポーリングする際は SubscriberGetContentParam::SetMinimumContentId() を用いて前回取得したコンテンツを再度取得しないようにし、
        一つのトピックから一度に取得する件数を 20 件以内にしてください。
        取得した情報を利用しない場面に遷移した場合などはポーリングを停止し、できる限り不要なアクセスを避けるようにしてください。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックのフォーマットが不正。取得するサイズに 0 を指定した。
        @li QERROR(Subscriber, OverLimit) : 予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] contents 取得したコンテンツ。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetContent(ProtocolCallContext* pContext, const SubscriberGetContentParam& param, qVector<SubscriberContent>* contents);

    /*!
        @refmanoverload{multi}
        @brief      複数のトピックに投稿されたコンテンツを一度に取得します。

        トピックに投稿されたコンテンツをコンテンツ ID 順 (降順) に取得します。
        コンテンツ ID は投稿された順に発行され、増加していきますがコンテンツに含まれる投稿日時は数秒前後する可能性があります。
        厳密な投稿日時順ではないことにご注意ください。
        サーバーでの処理タイミングの差により、過去取得した最大のコンテンツ ID より小さいコンテンツ ID のコンテンツが後で挿入されることがあります。

        本関数は例外的にガイドラインに定められた範囲において定期的に呼び出すことが可能です。
        呼び出し頻度の制限についてはガイドラインを参照してください。
        ポーリングする際は SubscriberGetContentParam::SetMinimumContentId() を用いて前回取得したコンテンツを再度取得しないようにし、
        一つのトピックから一度に取得する件数を 20 件以内にしてください。
        取得した情報を利用しない場面に遷移した場合などはポーリングを停止し、できる限り不要なアクセスを避けるようにしてください。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックのフォーマットが不正。取得するサイズに 0 を指定した。
                                                  params が空。
        @li QERROR(Subscriber, OverLimit) : 予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。
                                            params の個数が SubscriberConstants::MAX_GET_CONTENT_PARAMS_SIZE を超えている。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  params 詳細情報。
        @param[out] contentsPerParam 取得したコンテンツのリスト。コンテンツのリストの順は指定したparamsの順に対応します。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetContent(ProtocolCallContext* pContext, const qVector<SubscriberGetContentParam>& params, qVector<qVector<SubscriberContent> >* contentsPerParam);

    /*!
        @internal
        @refmanoverload{multi}

        @brief      トピックを複数フォローします。

        自分自身が現在フォローしているトピックに加えて指定したトピックをフォローします。
        最大 SubscriberConstants::MAX_FOLLOWING_SIZE 個のトピックをフォローできます。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックを一つも指定していない。トピックのフォーマットが不正。
        @li QERROR(Subscriber, OverLimit) : フォロー数が SubscriberConstants::MAX_FOLLOWING_SIZE を超える。
                                            予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  topics フォローするトピック。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool Follow(ProtocolCallContext* pContext, const qVector<String>& topics);

    /*!
        @internal
        @refmanoverload{single}

        @brief      トピックを一つフォローします。

        自分自身が現在フォローしているトピックに加えて指定したトピックをフォローします。
        最大 SubscriberConstants::MAX_FOLLOWING_SIZE 個のトピックをフォローできます。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックのフォーマットが不正。
        @li QERROR(Subscriber, OverLimit) : フォロー数が SubscriberConstants::MAX_FOLLOWING_SIZE を超える。
                                            予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  topic フォローするトピック。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool Follow(ProtocolCallContext* pContext, const String& topic);

    /*!
        @internal
        @refmanoverload{multi}

        @brief      トピックを複数アンフォローします。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックを一つも指定していない。トピックのフォーマットが不正。
        @li QERROR(Subscriber, OverLimit) : 予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  topics アンフォローするトピック。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool Unfollow(ProtocolCallContext* pContext, const qVector<String>& topics);

    /*!
        @internal
        @refmanoverload{single}

        @brief      トピックを一つアンフォローします。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックのフォーマットが不正。
        @li QERROR(Subscriber, OverLimit) : 予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  topic アンフォローするトピック。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool Unfollow(ProtocolCallContext* pContext, const String& topic);

    /*!
        @internal
        @brief      現在フォローしているトピックを全てアンフォローします。

        @param[in,out]  pContext 呼び出し情報。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UnfollowAll(ProtocolCallContext* pContext);

    /*!
        @internal
        @brief      現在フォローしているトピックを全てアンフォローし、指定したトピック(複数)をフォローし直します。

        フォローするトピックを指定したトピックに全て置き換えたい場合に使用します。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックのフォーマットが不正。
        @li QERROR(Subscriber, OverLimit) : フォロー数が SubscriberConstants::MAX_FOLLOWING_SIZE を超える。
                                            予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  topics 新たにフォローするトピック。空を指定した場合は SubscriberClient::UnfollowAll() と同じ動作をします。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool UnfollowAllAndFollow(ProtocolCallContext* pContext, const qVector<String>& topics);

    /*!
        @internal
        @brief      指定したプリンシパル ID のユーザーがフォローしているトピックを取得します。

        @param[in,out]  pContext 呼び出し情報。
        @param[out] topics フォローしているトピック。順番はフォローした順等に関わりなく、不定です。
        @param[in]  pid 対象のプリンシパル ID。INVALID_PRINCIPALIDを指定すると自分自身の情報を取得します。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetFollowing(ProtocolCallContext* pContext, qVector<String>* topics, PrincipalID pid=INVALID_PRINCIPALID);

    /*!
        @internal
        @brief      指定したトピックをフォローしているユーザーのプリンシパル ID を取得します。

        フォロワーの人数も取得できます。フォロワーの人数だけを取得したい場合は SubscriberClient::GetNumFollowers() を使用してください。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックのフォーマットが不正。取得するサイズに 0 を指定した。
        @li QERROR(Subscriber, OverLimit) : 取得するサイズの指定が SubscriberConstants::MAX_GET_FOLLOWER_SIZE を超えている。
                                            予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] result 取得結果。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetFollower(ProtocolCallContext* pContext, const SubscriberGetFollowerParam& param, SubscriberGetFollowerResult* result);

    /*!
        @internal
        @brief      指定したトピックをフォローしているユーザーの人数を取得します。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックのフォーマットが不正。取得するサイズに 0 を指定した。
                                                  取得するサイズとオフセットの合計値が 0x80000000 を超えている。
        @li QERROR(Subscriber, OverLimit) : 予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。
                                            取得するサイズが SubscriberConstants::MAX_GET_FOLLOWER_SIZE を超えている。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  topic フォローしている人数を取得するトピック。
        @param[out] numFollowers フォローしている人数。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetNumFollowers(ProtocolCallContext* pContext, const String& topic, qUnsignedInt32* numFollowers);

    /*!
        @internal
        @brief      タイムラインを取得します。

        タイムラインをコンテンツ ID 順 (降順) に取得します。
        タイムラインとはそのユーザーがフォローしているトピックに投稿されたコンテンツを時系列に並べた物です。
        任意のユーザーのタイムラインを取得できます。
        フォローしているトピックが一つも無い場合は空のコンテンツが取得されます。
        コンテンツ ID は投稿された順に発行され、増加していきますがコンテンツに含まれる投稿日時は数秒前後する可能性があります。
        厳密な投稿日時順ではないことにご注意ください。
        サーバーでの処理タイミングの差により、過去取得した最大のコンテンツ ID より小さいコンテンツ ID のコンテンツが後で挿入されることがあります。

        本関数は例外的にガイドラインに定められた範囲において定期的に呼び出すことが可能です。
        呼び出し頻度の制限についてはガイドラインを参照してください。
        ただし、取得した情報を利用しない場面に遷移した場合などはポーリングを停止し、できる限り不要なアクセスを避けるようにしてください。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : 取得するサイズに 0 を指定した。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  param 詳細情報。
        @param[out] contents 取得したコンテンツ。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool GetTimeline(ProtocolCallContext* pContext, const SubscriberGetTimelineParam& param, qVector<SubscriberContent>* contents);

    /*!
        @brief      コンテンツを削除します。

        指定したトピックに存在するコンテンツを削除します。
        投稿時に複数のトピックを指定した場合、全てのトピックからコンテンツを削除するにはその全てのトピックを指定する必要があります。
        投稿したトピックの内、一部のトピックを指定しなかった場合はそのトピックのコンテンツは削除されません。
        自分自身が投稿したコンテンツのみ削除できます。
        指定したトピックにコンテンツが見つからない場合は無視され成功の結果になります。

        非同期処理の結果、以下の Result が返ることがあります。
        @errorapplet

        @li QERROR(Subscriber, InvalidArgument) : トピックを一つも指定していない。トピックのフォーマットが不正。
        @li QERROR(Subscriber, OverLimit) : 予約済みトピック番号が SubscriberConstants::NUM_RESERVED_TOPICS 以上。
        @li QERROR(Subscriber, PermissionDenied) : 自分が投稿したコンテンツではない。

        @param[in,out]  pContext 呼び出し情報。
        @param[in]  topics 削除元のトピック。
        @param[out] contentId 削除するコンテンツ ID。

        @retval     true 非同期処理の開始に成功。
        @retval     false 指定した pContext の状態が CallContext::CallInProgress となっており非同期処理を開始出来ない。
                          これは他の非同期処理で使用中であることを示します。
                          プログラミングエラーであるため、このケースはリリースビルド以外では内部で Assert により停止します。
    */
    qBool DeleteContent(ProtocolCallContext* pContext, const qVector<String>& topics, qUnsignedInt64 contentId);

private:
    SubscriberProtocolClient m_oProtocol;
};

END_NAMESPACE_NEX

