﻿/*---------------------------------------------------------------------------*
  (C) Copyright 1998-2017, Technologies Quazal Inc.  All rights reserved.
  Quazal Net-Z, Quazal Rendez-Vous and Quazal are trademarks of Quazal.

  Copyright (C)2011-2017 Nintendo  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo and/or its licensors and are
  protected by national and international copyright laws.
  They may not be disclosed to third parties or copied or duplicated
  in any form, in whole or in part, without the prior written consent
  of Nintendo.
  The content herein is highly confidential and should be handled
  accordingly.
 *---------------------------------------------------------------------------*/

#pragma once

#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include "SubscriberGetTimelineParamDDL.h"

BEGIN_NAMESPACE_NEX

/*!
    @internal
    @brief      タイムラインの取得に関するパラメータを表すクラスです。
*/
class SubscriberGetTimelineParam: public DDLCLASS(SubscriberGetTimelineParam)
{
public:
    /*!
        @brief      コンストラクタです。
    */
    SubscriberGetTimelineParam()
    {
        Reset();
    }

    /*!
        @brief      デストラクタです。
    */
    virtual ~SubscriberGetTimelineParam() {}


    /*!
    @brief      タイムラインを取得する対象のプリンシパル ID を設定します。

    INVALID_PRINCIPAL_ID を指定した場合は自分自身のタイムラインを取得します。初期値は INVALID_PRINCIPAL_ID です。

    @param[in] pid_ タイムラインを取得する対象のプリンシパル ID。
    */
    void SetTargetPrincipalId(PrincipalID pid_) { this->pid = pid_; }

    /*!
    @brief      タイムラインを取得する対象のプリンシパル ID を取得します。

    @return タイムラインを取得する対象のプリンシパル ID。
    */
    PrincipalID GetTargetPrincipalId() const { return pid; }

    /*!
    @brief      取得するコンテンツの最大数を設定します。

    初期値は 20 です。一つのタイムラインに保持されるコンテンツは SubscriberConstants::MAX_TIMELINE_CONTENT_SIZE 件です。
    そのため SubscriberConstants::MAX_TIMELINE_CONTENT_SIZE 以上のサイズを指定した場合は最大 SubscriberConstants::MAX_TIMELINE_CONTENT_SIZE 件取得されます。
    SubscriberClient::GetTimeline() でポーリングする際は 20 以下に設定してください。

    @param[in] size 取得するコンテンツの最大数。
    */
    void SetSize(qUnsignedInt32 size_) { this->size = size_; }

    /*!
    @brief      取得するコンテンツの最大数を取得します。

    @return 取得するコンテンツの最大数。
    */
    qUnsignedInt32 GetSize() const { return size; }

    /*!
    @brief      何件目から取得するかを表すオフセットを設定します。

    何件目から取得するかを表すオフセットを設定します。初期値は 0 です。

    @param[in] offset_ 何件目から取得するかを表すオフセット。
    */
    void SetOffset(qUnsignedInt32 offset_) { this->offset = offset_; }

    /*!
    @brief      何件目から取得するかを表すオフセットを取得します。

    @return 何件目から取得するかを表すオフセット。
    */
    qUnsignedInt32 GetOffset() const { return offset; }

    /*
    @brief      タイムラインを強制的に再生成するかどうかを設定します。利用できません。

    タイムラインを強制的に再生成するかどうかを設定します。初期値は false です。

    @param[in] forceCreate_ タイムラインを強制的に再生成するかどうか。
    */
    void SetForceCreate(qBool forceCreate_) { this->forceCreate = forceCreate_; }

    /*
    @brief      タイムラインを強制的に再生成するかどうかを取得します。

    @return タイムラインを強制的に再生成するかどうか。
    */
    qUnsignedInt32 GetForceCreate() const { return forceCreate; }

    /*!
    @brief      取得するコンテンツを限定するための最小のコンテンツ ID を設定します。

    取得するコンテンツを minimumContentId より大きい物のみに限定します。minimumContentId と同じコンテンツ ID のものは含まれません。
    SubscriberClient::GetTimeline() でポーリングする際は同じコンテンツを何度も取得しないように前回取得したコンテンツのうち最も大きいコンテンツ ID 記憶しておき、それを指定してください。
    初期値は 0 です。

    @param[in] minimumContentId_ 取得するコンテンツを限定するための最小のコンテンツ ID。
    */
    void SetMinimumContentId(qUnsignedInt64 minimumContentId_) { this->minimumContentId = minimumContentId_; }

    /*!
    @brief      取得するコンテンツを限定するための最小のコンテンツ ID を取得します。

    @return 取得するコンテンツを限定するための最小のコンテンツ ID。
    */
    qUnsignedInt64 GetMinimumContentId() const { return minimumContentId; }

    /*!
        @brief      オブジェクトをデフォルトコンストラクタで生成した状態に初期化します。
    */
    void Reset()
    {
        SetTargetPrincipalId(INVALID_PRINCIPALID);
        SetSize(20);
        SetOffset(0);
        SetForceCreate(false);
        SetMinimumContentId(0);
    }

    /*!
        @brief      SubscriberGetTimelineParam オブジェクトの内容をトレースします。

        @param[in]  uiTraceFlags TraceLog::SetFlag() の引数と同様です。
    */
    void Trace(qUnsignedInt64 uiTraceFlags=TRACE_ALWAYS) const
    {
        QTRACE(uiTraceFlags, *this);
    }

    /*!
        @internal
        @brief      Trace() で使用するためのフレンド宣言です。
    */
    inline friend StringStream& operator<<(StringStream& oStr, const SubscriberGetTimelineParam& rhs);
};

inline StringStream& operator<<(StringStream& oStr, const SubscriberGetTimelineParam& rhs)
{
    oStr << NEX_T("SubscriberGetTimelineParam: (")
               << NEX_T("pid=") << rhs.pid
               << NEX_T(", size=") << rhs.size
               << NEX_T(", offset=") << rhs.offset
               << NEX_T(", forceCreate=") << rhs.forceCreate
               << NEX_T(", minimumContentId=") << rhs.minimumContentId;
    oStr << NEX_T(")");
    return oStr;
}

END_NAMESPACE_NEX

